--actor.lua
local AnimationType = Enum:getTable('AnimationType');
local DirectionType = Enum:getTable('DirectionType');

--========================================================================
--角色类

local Actor = class("Actor");

function Actor:ctor(id, resID, pid)
	self.id					= id;
	self.resID				= resID;
	self.animationID		= AnimationType.none;
	self.faceDir			= DirectionType.right;	--朝向
	self.armature			= nil;
	self.pid				= pid;
	self.s_icon				= '';
end

--========================================================================
--属性
--id
function Actor:GetID()
	return self.id;
end

function Actor:GetPid()
	return self.pid
end

--resID
function Actor:GetResID()
	return self.resID;
end

function Actor:GetSIcon()
	return self.s_icon
end

function Actor:SetGodSIcon(img)
	self.s_icon = img
end

function Actor:setAvatar(avatar_no)
	self.avatar_no = avatar_no
end

function Actor:genArmature( dir )
	self.armatureNode = cc.Node:create();

	self.workNode = cc.Node:create()
		:setLocalZOrder(-100)
		:addTo(self.armatureNode)

	self.buffBackEffectNode = cc.Node:create()
		:setLocalZOrder(-50)
		:addTo(self.armatureNode)

	local res_id = ResTable:getValue('role', self.resID, 'resource_id')
	if self.avatar_no and self.avatar_no ~= 0 then
		res_id = ResTable:getValue('role_avatar', self.avatar_no, 'resource_id')
	elseif self.pid and self.pid > 0 then
		res_id = model.CardModel:getCardResIdByPid(self.pid);
	end
	self.s_icon = ResTable:getValue('role_resource', res_id, 'iconsmall')
	local modelPath = ResTable:getValue('role_resource', res_id, 'model')
	self.armature, self.db_id = DragonBonesManager:GetArmature(modelPath)
	self.armature:setLocalZOrder(0)
	if dir == DirectionType.left then
		self.armature:getArmature():setFlipX(true)
	end
	self.faceDir = dir
	self.armature:addTo(self.armatureNode)

	self.buffFrontEffectNode = cc.Node:create()
		:setLocalZOrder(50)
		:addTo(self.armatureNode)

	self.effectNode = cc.Node:create()
		:setName('effect')
		:setLocalZOrder(100)
		:addTo(self.armatureNode)

	self.labelNode = cc.Node:create()
		:setName('wenben')
		:setLocalZOrder(110)
		:addTo(self.armatureNode)

	self.buffWordNode = cc.Node:create()
		:setLocalZOrder(125)
		:addTo(self.armatureNode)

	self:SetScale(FIGHTER_ALLY_SCALE)
	return self.armatureNode;
end

function Actor:genMonArmature( dir )
	self.armatureNode = cc.Node:create()

	self.workNode = cc.Node:create()
		:setLocalZOrder(-100)
		:addTo(self.armatureNode)

	self.buffBackEffectNode = cc.Node:create()
		:setLocalZOrder(-50)
		:addTo(self.armatureNode)

	local monsterData = ResTable:getRowData('monster', self.resID)
	self.s_icon = monsterData['iconsmall']
	local db_str = monsterData['model']
	local dbScale = tonumber(monsterData['size'])
	if dbScale <= 0 then
		dbScale = 1.0
	end
	self.armature, self.db_id = DragonBonesManager:GetArmature(db_str)
	if dir == DirectionType.left then
		self.armature:getArmature():setFlipX(true)
	end
	self.faceDir = dir;
	self.armature:addTo(self.armatureNode)

	self.buffFrontEffectNode = cc.Node:create()
		:setLocalZOrder(50)
		:addTo(self.armatureNode)

	self.effectNode = cc.Node:create()
		:setName('effect')
		:setLocalZOrder(100)
		:addTo(self.armatureNode)

	self.labelNode = cc.Node:create()
		:setName('wenben')
		:setLocalZOrder(110)
		:addTo(self.armatureNode)

	self.buffWordNode = cc.Node:create()
		:setLocalZOrder(125)
		:addTo(self.armatureNode)

	self:SetScale(dbScale);
	return self.armatureNode;
end

function Actor:getArmature()
	return self.armatureNode;
end

function Actor:getArmatureDB()
	return self.armature
end

function Actor:getEffectNode()
	return self.effectNode;
end

function Actor:getLabelNode()
	return self.labelNode;
end

function Actor:getBuffBackEffectNode()
	return self.buffBackEffectNode
end

function Actor:getBuffFrontEffectNode()
	return self.buffFrontEffectNode
end

function Actor:getWorkNode()
	return self.workNode
end

function Actor:getBottomPos()
	local height = self.armature:getArmature():getArmatureHeight()
	local y = self.armature:getArmature():getArmatureY()
	return cc.p(0, -(height+y)*self.m_scale)
end

function Actor:getTopPos()
	local y = self.armature:getArmature():getArmatureY()
	return cc.p(0, -y*self.m_scale)
end

function Actor:getBuffWordNode()
	return self.buffWordNode
end

function Actor:clearEffect()
	if self.effectNode then
		self.effectNode:removeAllChildren();
	end
end

--设置朝向
function Actor:SetDirection( dir )
	if not self.armature then
		return;
	end
	if dir == DirectionType.right then
		self.armature:getArmature():setFlipX(false);	
	elseif dir == DirectionType.left then
		self.armature:getArmature():setFlipX(true);
	end
	self.faceDir = dir;
end

--获取朝向
function Actor:GetDirection()
	return self.faceDir;
end

--获取位置
function Actor:GetPosition()
	if self.armature == nil then
		return cc.p(0, 0);
	end
	return self.armatureNode:getPosition();
end

--设置位置
function Actor:SetPosition( x, y )
	if self.armature then
		self.armatureNode:setPosition( x, y );
	end
end

function Actor:addArmatureEventListener(callback)
	if self.armatureListener then
		self.armature:getEventDispatcher():removeEventListener(self.armatureListener)
		self.armatureListener = nil
	end

	self.armatureListener = cc.EventListenerCustom:create("frameEvent", function(event)
		if callback then
			callback()
		end
	end)
	self.armature:getEventDispatcher():setEnabled(true)
	self.armature:getEventDispatcher():addEventListenerWithFixedPriority(self.armatureListener, 1);
end

function Actor:setCompleteCallback()
	local completeListener = cc.EventListenerCustom:create("complete", function(event)
		if self.animationID ~= AnimationType.idle and self.animationID ~= AnimationType.die then
			self:SetAnimation(AnimationType.idle)
		end
	end)
	self.armature:getEventDispatcher():setEnabled(true)
	self.armature:getEventDispatcher():addEventListenerWithFixedPriority(completeListener, 1);
end

function Actor:SetLocalZOrder( zorder )
	if self.armature then
		self.armatureNode:setLocalZOrder(zorder);
		self.zorder = zorder;
	end
end

function Actor:GetLocalZOrder()
	if self.armature then
		return self.armatureNode:getLocalZOrder();
	else
		return 1;
	end
end

-- 设置被选中
function Actor:setBeSelected( flag )
end

--设置缩放
function Actor:SetScale( scale )
	self.m_scale = scale;
	self.armature:setScale(scale);
	self.effectScale = scale/FIGHTER_ALLY_SCALE
end

function Actor:GetScale()
	if self.m_scale then
		return self.m_scale;
	else
		return 1;
	end
end

function Actor:GetEffectScale()
	if self.effectScale then
		return self.effectScale
	else
		return 1
	end
end

--设置旋转
function Actor:SetRotation( angle )
	self.armature:setRotation( angle );
end

function Actor:AttachEffect( avatarPos, effect, effectHandler )
	if self.armature == nil then
		print("AttachEffect: Error! " .. self.id);
		return;
	end
	local effectNode = cc.Node:create();
	effectNode:setName(tostring(effectHandler:GetID()))
	effectNode:addChild(effect);
	effectNode:addTo(self.armatureNode);
end

function Actor:DetachEffect(effectHandler)
	if self.armature == nil then
		print("DetachEffect: Error! " .. self.id);
		return;
	end
	local effectName = tostring(effectHandler:GetID());
	if self.armatureNode:getChildByName(effectName) then
		self.armatureNode:removeChildByName(effectName);
	end
end

--卸载所有特效
function Actor:DetachAllEffect()
	if self.armature == nil then
		print("DetachAllEffect: Error! " .. self.id);
		return;
	end

	self.armature:DetachAllEffect();
end

--========================================================================
--动作

--加载Armature
--设置动作
function Actor:SetAnimation( id, playTimes, timeScale)
	playTimes = playTimes or -1;
	timeScale = timeScale or 1;
	if self.armature == nil then
		print("SetAnimation: Error! " .. self.id..'---resid---->'..tostring(self.resID));
		return;
	end

	self.animationID = id;

	if self.armature:getAnimation():hasAnimation(id) then
		self.armature:getAnimation():play(id, playTimes, timeScale);
	end
end

function Actor:StopAnimation( id )
	-- body
	if self.armature and self.armature:getAnimation():hasAnimation(id) then
		self.armature:getAnimation():gotoAndStopByFrame(id, 36);
	end
end

function Actor:ContinuePlayAnimation( id )
	-- body
	if self.armature and self.armature:getAnimation():hasAnimation(id) then
		self.armature:getAnimation():gotoAndPlayByFrame(id, 36, 1);
	end
end

-- 销毁db
function Actor:DestroyArmature()
	self.armatureNode:removeFromParent();
	DragonBonesManager:RemoveDisplay(self.db_id)
	self.armature = nil
end

function Actor:GetAnimation()
	return self.animationID;
end

return Actor;
