local DragonBonesManager = class("DragonBonesManager")

function DragonBonesManager:ctor()
	self.loadList = {};
	self.displayList = {}
	self.m_id = 0;
	self.factory = db.CCFactory:getFactory();
end

function DragonBonesManager:loadDragonBonesResources( name, relativePath, force )
	relativePath = relativePath or 'RoleModel/';
	if not self.loadList[name] or force then
		if cc.FileUtils:getInstance():isFileExist(relativePath..name.."/"..name.."_tex.json") then
			self.factory:loadDragonBonesData(relativePath..name.."/"..name.."_ske.json");
			self.factory:loadTextureAtlasData(relativePath..name.."/"..name.."_tex.json");
		else
			self.factory:loadDragonBonesData(relativePath..name.."/"..name.."_ske.json");
			local index = 0;
			local btrue = true 
			local fileName = nil ;
			while btrue do
				
				fileName = relativePath..name.."/"..name.."_tex_"..index..".json"
				if cc.FileUtils:getInstance():isFileExist(fileName) then
					self.factory:loadTextureAtlasData(fileName);
					index = index + 1;
				else
					btrue = false;
				end
			end
			if index == 0 then
				local errorMsg = 'error===== 找不到文件: ['..tostring(name)..'] in relativePath:['..tostring(relativePath)..']'
				print(errorMsg)
				GlobalFunctions:showErrorPromt(errorMsg);
				return -1;
			end
		end
	end
	self.loadList[name] = true;
	return 1;
end

function DragonBonesManager:GetArmature( name )
	local ret = self:loadDragonBonesResources(name,'RoleModel/')
	if ret ~= -1 then
		local display = self.factory:buildArmatureDisplay(name)
		local id = self:AddDisplay(display, name)
		return display, id;
	end
end

function DragonBonesManager:GetFightEffect( name )--暂时不用了
	-- local ret =	self:loadDragonBonesResources(name,'Effect/Fight/')
	-- if ret ~= -1 then
	-- 	local display = self.factory:buildArmatureDisplay(name)
	-- 	local id = self:AddDisplay(display, name)
	-- 	return display, id;
	-- end
end

function DragonBonesManager:GetUIEffect( name )
	local ret =  self:loadDragonBonesResources(name,'Effect/UI/')
	if ret ~= -1 then
		local display = self.factory:buildArmatureDisplay(name)
		local id = self:AddDisplay(display, name)
		return display, id;
	end
end

function DragonBonesManager:preLoadRoleImg( name, callback )
	if not self.loadList[name] then
		local fileName = "RoleModel/"..name.."/"..name.."_tex.png";
		if cc.FileUtils:getInstance():isFileExist(fileName) then
			cc.Director:getInstance():getTextureCache():addImageAsync(fileName, callback);
		else
			print(name .. ' load failed.')
			if callback then callback(); end
		end
	else
		if callback then callback(); end
	end
end

function DragonBonesManager:getFactory( ... )
	return self.factory;
end

function DragonBonesManager:AddDisplay(display, name)
	self.m_id = self.m_id + 1
	self.displayList[self.m_id] = {display = display, name = name, active = true}
	return self.m_id
end

function DragonBonesManager:RemoveDisplay(id)
	if self.displayList[id] then
		self.displayList[id].active = false
		self.displayList[id].display:dispose()
	end
end

function DragonBonesManager:DestroyAll()
	for k, v in pairs(self.displayList) do
		if v.active then
			v.display:dispose()
		end
	end
	self.displayList = {}
	self.loadList = {}
	self.m_id = 0
	db.CCFactory:getFactory():clear()
end

cc.exports.DragonBonesManager = DragonBonesManager:create()
