local AudioManager = class("AudioManager")
local fileUtils = cc.FileUtils:getInstance();
local scheduler = cc.Director:getInstance():getScheduler();

local cvTypeEnum = Enum:getTable("cvTypeEnum");
local cvTypeEnumStr = Enum:getTable("cvTypeEnumStr");

function AudioManager:ctor()
	self.audio = cc.SimpleAudioEngine:getInstance();
	self.musicPath = 'audio/music/';
	self.effectPath = 'audio/effect/';
	self.cvPath = 'audio/cv/';
	self.loveCvPath = 'audio/cv/dialog_love/'

	-- music sound type:
	-- win32:
	-- .mp3 .mid .wav
	-- android:
	-- .mp3 .mid .ogg
	-- ios:
	-- .aac .caf .mp3 .m4a .wav
	self.musicType = ".mp3";
	self.musicNameList = 
	{
		--战斗过程音频
		commonFight			=	'V_BGM_FC',
		bossFight			=	'V_BGM_FB',
		copyFight			=	'V_BGM_FE',
		--attributeFight		=	'',
		winAudio 			=	'V_BGM_win',
		failAudio			=	'V_BGM_fail',
	
		--界面BGM
		title				=	'V_BGM_title',
		main				=	'V_BGM_home',
		logo				=	'V_BGM_logo',
	}
	-- effect sound type:
	-- win32:
	-- .mid .wav
	-- android:
	-- .ogg
	-- ios:
	-- .caf .wav
	self.effectType = ".mp3";
	self.effectNameList = 
	{
		test					=	'test_hit',
		GameStart				=	'V_UI_Load_001',
		ChangeAccount			=	'V_UI_Load_002',
		ButtonClick				=	'V_UI_Common_001',
		ConfirmButtonClick		=	'V_UI_Common_002',
		CancelButtonClick		=	'V_UI_Common_003',
		CardHeroLevelUp			=	'V_UI_Common_LvUp',
		CommonControlFly		=	'V_UI_Common_ControlFly',
		CardProgressGrowth		=	'V_UI_Card_001',
		CardLevelUpStrenger		=	'V_UI_Card_002',
		CardHeroLevelUp			=	'V_UI_Card_003',
		CardOpen				=	'V_UI_Card_Open',
		CardGet					=	'V_UI_Card_Get',
		EquipmentChange_1		=	'V_UI_Equip_001',
		EquipmentChange_2		=	'V_UI_Equip_002',
		EquipmentDecompose		=	'V_UI_Equip_003',
		Equipment_1				=	'V_UI_Equip_003001',
		equipmentSell			=	'V_UI_Equip_004',
		equipmentStrengthen		=	'V_UI_Equip_005',
		equipmentBoxOpen		=	'V_UI_Equip_006',
		equipmentBoxOpening		=	'V_UI_Equip_007',
		GetCoinAndResource		=	'V_UI_Home_001',
		Skill_1					=	'V_SKILL_001',
		Skill_2					=	'V_SKILL_002',
		Skill_3					=	'V_SKILL_003',
		Skill_4					=	'V_SKILL_004',
		Skill_5					=	'V_SKILL_005',
		Skill_6					=	'V_SKILL_006',
		Skill_7					=	'V_SKILL_007',
		Skill_8					=	'V_SKILL_008',
		Skill_9					=	'V_SKILL_009',
		Skill_10				=	'V_SKILL_010',
		Skill_11				=	'V_SKILL_011',
	}

	self.cvType = '.mp3';
	
	self.cvSoundList = {};

	--对应cv
	-- cvName = cv_id(R001)+ '_' + cvKeyType[key] + '.mp3' 如：ROO1_gacha.mp3 
	-- 1. login
	-- 2. main
	-- 3. fight
	-- 4. plot
	self.state = 1;
	if self.m_scheduleHandler == nil then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.update), 0, false);
	end
	self.time = 0;
	self.speed = 1.0;

	self:initAudioVolume();
	self:initCVPath();
end

function AudioManager:initCVPath()
	local curLn = model.UserDefaultModel:getCVLanguageSet();
	if curLn == 'kr' then
		self.cvPath = 'audio/cv_kr/';
		self.loveCvPath = 'audio/cv_kr/dialog_love/'
	else
		self.cvPath = 'audio/cv/';
		self.loveCvPath = 'audio/cv/dialog_love/'
	end
	self.cvSoundList = {};
end

function AudioManager:getCVPath()
	return self.cvPath;
end

function AudioManager:changeSpeed(v)
	self.speed = v
end

function AudioManager:Destory()
	cc.SimpleAudioEngine:destroyInstance();
	self.audio = nil;
end

function AudioManager:initAudioVolume()
	local fileUtils = cc.FileUtils:getInstance();
	local fileName = device.writablePath..'/setting_config.json';

	local musicVolume = 1;
	local effectsVolume = 1;

	if fileUtils:isFileExist(fileName) then
		local settingConfig = {};
		local fileData = fileUtils:getStringFromFile(fileName);
		if fileData ~= '' then
			settingConfig = Cjson.decode(fileData);

			if settingConfig.musicVolume then
				musicVolume = settingConfig.musicVolume;
			end

			if settingConfig.effectsVolume then
				effectsVolume = settingConfig.effectsVolume
			end
		end
	end

	self:setMusicVolume(musicVolume);
	self:setEffectsVolume(effectsVolume);
end

function AudioManager:preLoad()
	local musicIndex = 0;
	local effectIndex = 0;
	for _, musicName in pairs(self.musicNameList) do
		musicIndex = musicIndex + 1;
		self:preloadMusic(musicName);
	end
	for _, effectName in pairs(self.effectNameList) do
		effectIndex = effectIndex + 1;		
		self:preloadEffect(effectName);
	end
end

--=============================Music=============================
-- proload Music
-- string fileName
-- return nil
function AudioManager:preloadMusic( fileName )
	local file = self.musicPath .. fileName .. self.musicType;
	if fileUtils:isFileExist( file ) then
		self.audio:preloadMusic( file );
	else
		print("Load music failed, no File named", file);
	end
end

-- play Music
-- string fileName
-- int state (see ctor)
-- bool isLoop ( default false )
-- return nil
function AudioManager:playMusic( fileName, state, isLoop )
	state = state or 2;
	if state ~= self.state then
		return;
	end
	isLoop = isLoop or false;
	local file = self.musicPath .. fileName .. self.musicType;
	if fileUtils:isFileExist(file) then
		if self.audio:isMusicPlaying() then
			self.audio:stopMusic()
		end
		self.audio:playMusic( file, isLoop );
	else
		print("Play music failed, no File named", file);
	end
end

-- stop Music
-- bool isRelease
-- return nil
function AudioManager:stopMusic( isRelease )
	isRelease = isRelease or false;
	self.audio:stopMusic( isRelease );
end

-- pause Music
-- return nil
function AudioManager:pauseMusic()
	self.audio:pauseMusic();
end

-- resume Music
-- return nil
function AudioManager:resumeMusic()
	self.audio:resumeMusic()
end

-- set Music Volume
-- float vol (range 0 - 1)
function AudioManager:setMusicVolume( vol )
	self.audio:setMusicVolume( vol );
end

-- get Music Volume
-- return vol (range 0 - 1)
function AudioManager:getMusicVolume()
	return self.audio:getMusicVolume();
end

-- check Music playing
-- return bool
function AudioManager:isMusicPlaying()
	return self.audio:isMusicPlaying();
end

--=============================Effect=============================
-- proload Effect
-- string fileName
-- return nil
function AudioManager:preloadEffect( fileName )
	local file = self.effectPath .. fileName .. self.effectType;
	if fileUtils:isFileExist( file ) then
		self.audio:preloadEffect( file );
	else
		print("Load effect failed, no File named", file);
	end
end

-- play Effect
-- string fileName
-- int state (see ctor)
-- bool isLoop ( default false ) 
-- return soundID
function AudioManager:playEffect( fileName, state, isLoop )
	--state = state or 2;
	--if state ~= self.state then
	--	return;
	--end
	isLoop = isLoop or false;
	local file = self.effectPath .. fileName .. self.effectType;
	if fileUtils:isFileExist( file ) then
		return self.audio:playEffect( file, isLoop, self.speed );
	else
		print("Play effect failed, no File named", file);
		return -1;
	end
end

-- stop Effect
-- int soundID
-- return nil
function AudioManager:stopEffect( soundID )
	self.audio:stopEffect( soundID );
end

-- stop all Effects
-- return nil
function AudioManager:stopAllEffects()
	self.audio:stopAllEffects();
end

-- pause Effect
-- float soundID
-- return nil
function AudioManager:pauseEffect( soundID )
	self.audio:pauseEffect( soundID );
end

-- pause all Effects
-- return nil
function AudioManager:pauseAllEffects()
	self.audio:pauseAllEffects();
end

-- resume Effect
-- int soundID
-- return nil
function AudioManager:resumeEffect( soundID )
	self.audio:resumeEffect( soundID );
end

-- resume all Effects
-- return nil
function AudioManager:resumeAllEffects()
	self.audio:resumeAllEffects();
end

-- set Effects Volume
-- float vol (range 0 - 1)
-- return nil
function AudioManager:setEffectsVolume( vol )
	self.audio:setEffectsVolume( vol );
end

-- get Effects Volume
-- return vol (range 0 - 1)
function AudioManager:getEffectsVolume()
	return self.audio:getEffectsVolume();
end
function AudioManager:isEffectPlaying(soundID)
	return self.audio:isEffectPlaying(soundID);
end

--==========================================================
function AudioManager:playCVEffect( fileName, state, isLoop )
	isLoop = isLoop or false;
	local file = self.cvPath .. fileName .. self.effectType;
	if fileUtils:isFileExist( file ) then
		return self.audio:playEffect( file, isLoop, self.speed );
	else
		print("Play effect failed, no File named", file);
		return -1;
	end
end

function AudioManager:preloadCVFile( roleID, cvEnumIndex )
	local ret = false;
	local ok = true;
	local i = 1;
	while ok do
		local cvFullFile = self:getCVFileName(roleID,cvEnumIndex,i);
		if fileUtils:isFileExist( cvFullFile ) then
			-- print('preloadCVFile ===============  roleID == ',roleID,'   cvType == ',tostring(cvEnumIndex), ' ,   cvFile == ' ,cvFullFile)
			--self.audio:preloadEffect( cvFullFile );
			if self.cvSoundList[tonumber(roleID)][cvEnumIndex] == nil then
				self.cvSoundList[tonumber(roleID)][cvEnumIndex] = {}
			end
			table.insert( self.cvSoundList[tonumber(roleID)][cvEnumIndex],cvFullFile )
			ret = true;
		else
			print("Load cv failed, no File named:", cvFullFile);
			ok = false;
		end
		i = i + 1;
	end


	return ret;
end

function AudioManager:downloadCV( roleID )

	local cvID = self:getCVIdByRoleId(roleID);
	if cvID == nil then
		return false;
	end
	local folderName = tostring(cvID);
	local cvFileName = tostring(cvID) ..'_gacha1';
	local cvForlderName = self.cvPath .. folderName --.. '/' .. cvFileName .. self.cvType;

	if fileUtils:isDirectoryExist( cvForlderName ) == false then
		DownloadControler:addDownLoadList(cvID..'cv', 'audio/cv/',  function()end)
	end

end

function AudioManager:preloadCV( roleID )

	local cvID = self:getCVIdByRoleId(roleID);
	if cvID == nil then
		return false;
	end
	self.cvSoundList[tonumber(roleID)] = {};
	for key, value in pairs(cvTypeEnum) do
		if self:preloadCVFile(roleID, value) == false then
			self:downloadCV( roleID )
		end
	end
	return true
end

function AudioManager:loadCV( roleID, soundType )
	if not self.cvSoundList[roleID] then
		self.cvSoundList[roleID] = {};
	end
	if not self.cvSoundList[roleID][soundType] then
		if self:preloadCVFile(roleID, soundType)==false then
			self:downloadCV( roleID )
		end
	end
end
-- R001
function AudioManager:getCVIdByRoleId( roleID )
	if roleID >= 20000 then
		local cvData = ResTable:getRowData('monster', roleID);
		if cvData == nil then
			return nil
		end
		local cvStr = tostring(cvData['CV_id']);
		return cvStr;
	else
		local cvData = ResTable:getRowData('role_resource', roleID);
		if cvData == nil then
			return nil
		end
		local cvStr = tostring(cvData['cv_id']);
		return cvStr;
	end
end

function AudioManager:getCVFileName( roleID,cvEnumIndex,cvIndex )
	local cvID = self:getCVIdByRoleId(roleID); 
	if cvID == nil then
		return '';
	end
	local cvFileName1 = tostring(cvID)
	-- if avatarId ~= nil then
	-- 	-- 主界面皮肤特殊命名
	-- 	local avatarCVId = self:getCVIdByRoleId(avatarId); 
	-- 	if avatarCVId == nil then
	-- 		return '';
	-- 	end
	-- 	cvFileName1 = tostring(avatarCVId)
	-- end
	local cvEnumStr = cvTypeEnumStr[cvEnumIndex];
	local cvFullFile = self.cvPath .. tostring(cvID) .. '/' .. cvFileName1 ..'_'.. tostring(cvEnumStr) .. tostring(cvIndex) .. self.cvType;
	-- print('getCVFileName ===============  cvType == ',tostring(cvEnumIndex), ' ,   cvFile == ' ,cvFullFile)
	return cvFullFile;

end

function AudioManager:getLoveCvFileName(roleId,loveLv,index)
	local cvName = string.format( 'R%.3d_dialog_love%d_%d',roleId,loveLv,index);
	local file = self.loveCvPath ..''.. cvName .. self.effectType;
	return file;
end

function AudioManager:preloadCVList( cvLoadList )
	if not self.loadCVHandler then
		self.preTimes = 0;
		self.loadCVHandler = scheduler:scheduleScriptFunc(handler(self, self.updatePreloadCVList), 0, false);
	end
	self.preloadCVFlag = true;
	self.cvLoadList = cvLoadList;

	self.coroutine = coroutine.create(function()
		for k, v in pairs(self.cvLoadList) do
			local role_id = ResTable:getValue('role', k, 'role_id');
			self:loadCV(role_id, cvTypeEnum.CV_GACHA);
			coroutine.yield();
		end
		self.preloadCVFlag = false;
		if self.loadCVHandler then 
			scheduler:unscheduleScriptEntry(self.loadCVHandler)
			self.loadCVHandler = nil 
		end
	end)
end

function AudioManager:updatePreloadCVList(dt)
	self.preTimes = self.preTimes + 1;
	if self.preTimes % 5 == 1 and self.preloadCVFlag then
		if self.coroutine then
			coroutine.resume(self.coroutine);
		end
	end
end

function AudioManager:playDialogLoveCV( roleId,loveLv, loveIndex )
	-- local cvName = 'dialog_love/love'..tostring(loveID);
	-- R028_dialog_love1_1.mp3
	--print(" roleId=>",roleId," loveLv=>",loveLv," cvType=>",loveIndex);
	if roleId == nil or loveLv == nil or loveIndex == nil then
		print(" role_id or love_lv or loveIndex is nil");
		return -1;
	end

	local file = self:getLoveCvFileName(roleId,loveLv, loveIndex);
	if fileUtils:isFileExist( file ) then
		self:stopAllEffects();
		return self.audio:playEffect( file, false, self.speed);
	else
		print(" play DialogLoveCV failed, no File named", file);
		return -1;
	end
end

function AudioManager:playLoveCVStopBefor(roleId,loveLv,cvType)
	--print(" roleId=>",loveLv," loveLv=>",loveLv," cvType=>",cvType);
	local cvIndex,soundType = model.UserModel:getLoveCvIndex(roleId,loveLv,cvType);
	return self:playDialogLoveCV( roleId,loveLv, cvIndex)
end

function AudioManager:playCV( roleID, soundType, state, force )

	state = state or 2;
	force = force or false;
	if state ~= self.state then
		return;
	end

	if not self.cvSoundList[roleID] then
		self.cvSoundList[roleID] = {};
	end
	if not self.cvSoundList[roleID][soundType] then
		if self:preloadCVFile(roleID, soundType)==false then
			self:downloadCV( roleID )
		end
	end
	if self.cvSoundList[roleID] and self.cvSoundList[roleID][soundType] then
		local file = self.cvSoundList[roleID][soundType][1]; -- 默认播放第一个cv
		if fileUtils:isFileExist( file ) then
			if force then
				self.playCVTime = self.time;
				self:stopAllEffects();
				return self.audio:playEffect( file, false, self.speed);
			else
				if (self.state == 3) then
					-- 战斗中，间隔1s才播放下一次cv
					if self.playCVTime then
						if self.time > (self.playCVTime + 1) then
							self.playCVTime = self.time;
							return self.audio:playEffect( file, false, self.speed);
						else
							return -1;
						end
					else
						self.playCVTime = self.time;
						return self.audio:playEffect( file, false, self.speed);
					end
				else
					return self.audio:playEffect( file, false, self.speed);
				end
			end
		else
			print("Play cv failed, no File named", file);
			self:downloadCV( roleID )
			return -1;
		end
	else
		print("Play cv failed, no soundType in the list", roleID, tostring(soundType));
		return -1;
	end
end

--==============================--
--desc:
--time:2020-05-14 04:21:21
--@roleID:角色id,默认看板娘
--@avatarID:皮肤id,默认nil
--@cvType:cv类型 enum表cvTypeEnum
--@notStop:是否不停止之前的音效，默认否
--@return 
--==============================--
function AudioManager:playCVStopBefor( roleID,avatarID, cvType,defaultIndex , notStop)
	local cvRoleID,cvIndex,soundType = model.UserModel:getCvTypeIndex(roleID,avatarID, cvType,defaultIndex);
	--print('cvRoleID == ',cvRoleID,'  soundType == ',soundType,'   cvIndex == ',cvIndex);
	cvIndex = cvIndex or 1;
	notStop = notStop or false;
	if not self.cvSoundList[cvRoleID] then
		if not self:preloadCV(cvRoleID) then
			return -1
		end
	end
	if self.cvSoundList[cvRoleID] and self.cvSoundList[cvRoleID][soundType] and self.cvSoundList[cvRoleID][soundType][cvIndex]  then
		local file = self.cvSoundList[cvRoleID][soundType][cvIndex] ;
		--print('==== playCVStopBefor ====     cvType == ',soundType,'   file == ',file);
		if fileUtils:isFileExist( file ) then
			if not notStop then
				self:stopAllEffects();
			end

			return self.audio:playEffect( file, false,1); -- , self.speed
		else
			print("Play cv failed, no File named", file);
			self:downloadCV( cvRoleID )
			return -1;
		end
	else
		print("Play cv failed, no soundType in the list", cvRoleID, tostring(soundType));
		return -1;
	end
end

function AudioManager:preloadFightCV()
end

--=======================================================
--play sound
function AudioManager:playSound(file, state)
	state = state or 2;
	if state ~= self.state then
		return;
	end
	if fileUtils:isFileExist( file ) then
		return self.audio:playEffect( file, false, self.speed );
	else
		print("Play sound failed, file: ", file);
		return -1;
	end
end

--=======================================================
function AudioManager:UnloadAll()
	self.audio:destroyInstance();
end

--=======================================================

function AudioManager:SetState(state)
	self.state = state
end

function AudioManager:preloadTeam(team)
	for i=1, 6 do
		if team[i] ~= -1 then
			local no = model.CardModel:GetCard(team[i]).no;
			self:preloadCV(no);
		end
	end
end

function AudioManager:update(dt)
	self.time = self.time + dt;
end

cc.exports.AudioManager = AudioManager:create();
