
local AsyncLoadManager = class('AsyncLoadManager')

function AsyncLoadManager:ctor(  )
    -- body
    self.count = 0;             -- 异步加载次数

    self.startIndex = 1;        -- 起始
    self.endIndex = -1;         -- 终止
    self.limit = 10;            -- 每次刷新数量

    self.eventTypeName = nil    -- 时间调度 

    self.asyncFinishCall = nil; -- 异步结束回调
    self.asyncUpdateFunc = nil; -- 异步加载事件 (nowIndex,count)=>{ return true}
end

--==============================--
--desc:
--time:2019-09-03 01:08:22
--@startIndex: 起始下标
--@endIndex: 终止下标
--@limit: 每次加载数量
--@asyncUpdateFunc: 异步加载item事件 
--@asyncFinishCall: 异步加载完成回调
--@return 
--==============================--
function AsyncLoadManager:initAsync( startIndex,endIndex,limit,asyncUpdateFunc,asyncFinishCall )
    
    self.startIndex = startIndex or 1
    self.endIndex = endIndex or -1
    self.limit = limit or 10;

    self.asyncUpdateFunc = asyncUpdateFunc
    self.asyncFinishCall = asyncFinishCall;
end

function AsyncLoadManager:startAsync( TimeMangerEventType )

    if  self.asyncUpdateFunc == nil then
        print('error ===========11111')
        return 
    end
    if self.eventTypeName == TimeMangerEventType then
        print('already start ===========2222')
        return 
    end
    self.eventTypeName = TimeMangerEventType;

    -- print('=====startAsync ===== self.startIndex  == '..tostring(self.startIndex))
    -- print('=====startAsync ===== self.endIndex  == '..tostring(self.endIndex))

    TimeManager:registUpdateEvent(self.eventTypeName,handler(self,self._asyncLoadList))
end

function AsyncLoadManager:stopAsync( ... )
    -- print('=====stopAsync ===== eventName == '..tostring(self.eventTypeName))

    if self.asyncFinishCall then
        self.asyncFinishCall(self.count)
    end

    self:_clear()

end




function AsyncLoadManager:_asyncLoadList( dt )
    -- print('=====asyncLoadList ===== self.startIndex  == '..tostring(self.startIndex))
    local asyncTrue = true ;
    local count = tonumber(self.limit);
    local bTrue = count > 0 and  self.startIndex <= self.endIndex
    while bTrue do
        if self.asyncUpdateFunc then
            asyncTrue = self.asyncUpdateFunc(self.startIndex,self.count) ;
        end
        self.startIndex = self.startIndex + 1;
        self.count = self.count + 1;

        count  = count - 1;

        bTrue = asyncTrue and count > 0 and self.startIndex <= self.endIndex

    end

    if not asyncTrue or  self.startIndex > self.endIndex then
        self:stopAsync();
        return 
    end


end

function AsyncLoadManager:_clear(  )
    TimeManager:registUpdateEvent(self.eventTypeName,nil)
    self.asyncUpdateFunc = nil;


    self.eventTypeName = nil;
    self.startIndex = 1;
    self.endIndex = -1;
    self.count = 0;
end


return AsyncLoadManager;