--TimeManager.lua

--========================================================================
--========================================================================

local TimeManager = class("TimeManager")
local RefreshShopTime = {1 , 21600 , 43200 , 64800};
function TimeManager:ctor()
	self:clearAll();
end

function TimeManager:clearAll(  )

	if self.m_scheduleHandler then 
		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
	self.skillLevelUpTime = 0;
	self.time = 0;
	self.timeTemp = 0;
	self.reqRefreshShopState = false;

	self.backgroundTime = 0 --累计进入后台的时间
	self.startIntoBackGroundTime = -1 --进入后台的当前时间

	self.eventState = false ; -- 启动事件刷新

	self.timeUpdateSecondEvent = {}
	self.timeUpdateFrameEvent = {}
end

function TimeManager:setReqTime()
	self.reqServerTime = socket.gettime()
end

function TimeManager:setOriTime(time, nextHeartBeatTime)
	self.oriServerTime = time
	self.nextHeartBeatTime = nextHeartBeatTime
end

function TimeManager:mainSceneLoadOver()
	local deltaTime = socket.gettime() - self.reqServerTime
	local serverTime = self.oriServerTime + deltaTime
	self.lastHeartBeatTime = serverTime
	self.heartBeatTimes = 0
	self:startEvent()
	self:initServerTime(serverTime)
end

function TimeManager:heartBeatUpdate(serverTime, nextHeartBeatTime)
	self.serverTime = serverTime or self.serverTime
	self.nextHeartBeatTime = nextHeartBeatTime or (self.serverTime + Conf.HEART_BEAT_TIME)
end

function TimeManager:initServerTime( serverTime )
	self.serverTime = serverTime;
end

function TimeManager:beginUpdate(time)
	self:clearAll()
	self.time = 0;
	self.serverTime = time;
	if self.m_scheduleHandler == nil then
		local scheduler = cc.Director:getInstance():getScheduler();
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self , self.update) , 0 , false);
	end
end

function TimeManager:setZeroTime(time)
	self.todayZeroTime = time;
	self.tomorrowZeroTime = time + 24*3600;
end

function TimeManager:getTime2Tomorrow()
	return math.floor(self.tomorrowZeroTime - self.serverTime)
end

function TimeManager:startEvent(serverTime)
	self.eventState = true 
end

function TimeManager:stopEvent(  )
	self.eventState = false
end

function TimeManager:getBackGroundTime()
	return self.backgroundTime
end

function TimeManager:timeBackGround()
	if self.clientLastHeartBeatTime==nil or (socket.gettime() - self.clientLastHeartBeatTime > Conf.HEART_BEAT_TIME) then
		self:reqHeartBeat(0)
	end
end

function TimeManager:reqHeartBeat(heartBeatTimes)
	local msg = {};
	msg.last_id = model.AnnouncementModel:getLastID()
	msg.gm_id = model.AnnouncementModel:getAnnouncementGM()
	msg.heartBeatTimes = heartBeatTimes
	--msg.gm_id = model.AnnHeartBeatModel:getAnnouncementGM();
	Network.NetworkManager:Send(Network.NetworkCmdType.req_heartbeat, msg , true);
end

function TimeManager:getHeatBeatTime(  )
	return self.heartBeatTime;
end
function TimeManager:update(dt)

	-- 客户端事件更新
	self:updateClientEvent(dt);

	-- 客户端时间更新
	self.serverTime = self.serverTime + dt;
	
end

function TimeManager:updateClientEvent( dt )
	if self.eventState ~= true   then
		return 
	end
	self.time = self.time + dt;
	self.timeTemp = self.time - 1;

	--每秒刷新事件
	if self.timeTemp >= 0 then
		self:updateSecondEventCall();
		--基地科技cd
		-- model.HomeModel:upDateSkillCd();
		-- model.TreatmentModel:updateTime();

		--buff 每秒刷新
		-- EventManager:FireEvent(EventType.FightUpdateBuffNode);
		--
		model.AutoFightModel:checkBuff()

		self.time = 0;
		if self.serverTime >= self.tomorrowZeroTime then
			self:setZeroTime(self.tomorrowZeroTime)

			model.GachaAdmobModel:reset();
		end
	end


	--每帧刷新事件

	--基地
	-- model.HomeModel:updateHomeTime(dt);

	--快速战斗奖励
	-- model.AutoFightModel:updateFastTime(dt);

	self:updateFrameEventCall();


	-- 心跳每20s请求一次
	if self.lastHeartBeatTime then
		local sendReq = false
		if self.serverTime >= self.nextHeartBeatTime then
			if self.heartBeatTimes == 0 then			
				sendReq = true
			elseif (self.serverTime - self.lastHeartBeatTime) > (Conf.HEART_BEAT_TIME - 2) then
				sendReq = true
			end
		end
		if sendReq then
			self:reqHeartBeat(self.heartBeatTimes)
			self.heartBeatTimes = self.heartBeatTimes + 1
			self.nextHeartBeatTime = self.nextHeartBeatTime + Conf.HEART_BEAT_TIME;
			self.lastHeartBeatTime = self.serverTime
			self.clientLastHeartBeatTime = socket.gettime()
		end
	end
end

---注册倒计时事件
---取消注册 eventCallback = nil
-- intervalIs1 时间间隔为1s 或者每帧
function TimeManager:registUpdateEvent( eventType,eventCallback,intervalIs1 )
	if eventType == nil then 
		return
	end

	if not intervalIs1 then 
		self.timeUpdateFrameEvent[eventType] = eventCallback;
	else
		self.timeUpdateSecondEvent[eventType] = eventCallback;
	end 
end
function TimeManager:updateSecondEventCall(  )
	
	for eventType,eventCallback in pairs(self.timeUpdateSecondEvent) do
		if eventCallback then
			eventCallback(self.serverTime);
		end
	end
end
function TimeManager:updateFrameEventCall( )

	for eventType,eventCallback in pairs(self.timeUpdateFrameEvent) do
		if eventCallback then
			eventCallback(self.serverTime);
		end
	end
end

function TimeManager:getCurTime()
	return self.serverTime;
end

function TimeManager:getCurHour(  )
	local curTime = self.serverTime or os.time();
	local hour = tonumber(os.date("%H",curTime));
	return hour;
end

function TimeManager:getZeroTime()
	local y = tonumber(os.date("%Y",self.severTime));
	local m = tonumber(os.date("%m",self.severTime));
	local d = tonumber(os.date("%d",self.severTime));
	return os.time({day=d, month=m, year=y, hour=0, min=0, sec=0});
end

function TimeManager:getNextDayTime()
	local y = tonumber(os.date("%Y",self.severTime));
	local m = tonumber(os.date("%m",self.severTime));
	local d = tonumber(os.date("%d",self.severTime));
	return os.time({day=(d+1), month=m, year=y, hour=0, min=0, sec=0});
end

function TimeManager:GetNextWeekTime()

	local d = os.date('*t',self.severTime)
	local dd = d.wday - 1;
	if dd == 0 then
		dd = 7;	
	end
	
	d.day = d.day - dd + 7;

	local tab = {year=d.year, month=d.month, day=d.day,hour=23,min=59,sec=59};

	return os.time(tab)
end

cc.exports.TimeManager = TimeManager:create();
