local DownloadControler = class("DownloadControler")

local scheduler = cc.Director:getInstance():getScheduler();
local fileUtils = cc.FileUtils:getInstance();

function DownloadControler:ctor()

	self:clearAll();

end

function DownloadControler:clearAll(  )

	if self.m_scheduleHandler then
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil
	end

	self:registEvent(false)

	self.downloadList = {};
	self.curLoadingList = {};	
	self.downloadErrorList = {};
	self.loadEndCallBackList = {}
	self.unzipPath = {}

	print('DownloadControler--init')

end

function DownloadControler:EnterGame()
	self:clearAll()
	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.Update), 1, false);

	self:registEvent()
end

function DownloadControler:registEvent( regist )
end

function DownloadControler:Update( dt )
	if #self.downloadList == 0 then --没有等待下载的任务了
		-- print('DownloadControler--downloadList is empty!')
		return
	end

	if #self.curLoadingList >= 5 then --当前正在下载的队列总数 不能超过上限
		print('DownloadControler--curLoadingList is full!')
		return
	end

	local url = table.remove(self.downloadList , 1)
	self:download( url )

end

function DownloadControler:overCurDownLoadTask( url )
	local idx = 0
	for i,v in ipairs(self.curLoadingList) do
		if v == url then
			idx = i
			break;
		end
	end
	if idx > 0 then
		local tmpUrl = table.remove(self.curLoadingList, idx)
		print('DownloadControler--overCurDownLoadTask-- ', tmpUrl)
	end
end


function DownloadControler:addDownLoadList( resName, path, callback )
	-- body
	local urlHead = 'http://cdn.kemvin.com/'
	local url = urlHead..resName..'.zip'

	if self.loadEndCallBackList[url] then --已经加入过了
		return;
	end
	if self.downloadErrorList[ url ] then --下载失败后，不删除记录 防止再次重复下载
		return
	end
	for i,v in ipairs(self.curLoadingList) do --已经加入过了
		if v == url then
			return;
		end
	end

	local tmpPath = device.writablePath.."/DynamicRes/"..path;
    if not fileUtils:isDirectoryExist(tmpPath) then
    	fileUtils:createDirectory(tmpPath);
    end

	self.unzipPath[ url ] = path

	self.loadEndCallBackList[ url ] = callback

	table.insert(self.downloadList, url)
	
end

function DownloadControler:download( url )
	-- body
	--local url = 'http://qin.saikyo.biz/upload/file-10011-1533720096.png';
	print('DownloadControler==download  ',url)
	local dlURL = url;
	dlURL = string.gsub(dlURL, "\\","/");
	
	local xhr = cc.XMLHttpRequest:new();
	xhr:open("GET", dlURL, true);
	xhr.responseType = cc.XMLHTTPREQUEST_RESPONSE_STRING;
	xhr:registerScriptHandler(function()
		print('DownloadControler---download---', url)
		local curTime = TimeManager:getCurTime();

		if  xhr.readyState == 4 and xhr.status == 200 then

			local fileData = xhr.response;
			local fullFileName = device.writablePath.."new_dres_"..curTime..".zip";
			print("　　└ >fileWrite："..fullFileName);
			
			local file = io.open(fullFileName,"wb");
			file:write(fileData);
			file:close();

			--成功 
			if self.downloadErrorList[ url ] then --删除错误记录
				self.downloadErrorList[ url ] = nil
			end

			local subpath = self.unzipPath[ url ]
			ego.EgoTools:uncompressDir( fullFileName , device.writablePath.."/DynamicRes/"..subpath, '')
			-- cc.ZipUtils:uncompressDir( fullFileName , device.writablePath.."/DynamicRes/"..subpath, '')

			if self.loadEndCallBackList[ url ] then
				self.loadEndCallBackList[ url ]()
				self.loadEndCallBackList[ url ] = nil
			end
			os.remove( fullFileName )

		else
			--失败
			if self.downloadErrorList[ url ] == nil then
				self.downloadErrorList[ url ] = 1
			else
				self.downloadErrorList[ url ] = self.downloadErrorList[ url ] + 1
			end
			if self.downloadErrorList[ url ] > 3 then --下载失败后，重复下载，如果超过重复下载次数，则不再下载
				print('DownloadControler--repeat download error: ', url)
				if self.loadEndCallBackList[ url ] then
					self.loadEndCallBackList[ url ]()
					self.loadEndCallBackList[ url ] = nil
				end
			else
				table.insert(self.downloadList, url)
			end

		end

		self:overCurDownLoadTask( url ) --结束当前下载任务
		
		
	end);

	table.insert(self.curLoadingList, url) --加入当前下载队列里
	
	xhr:send();
end

function DownloadControler:checkNeedDownloadSpineModel(modelName)
	if modelName==nil then
		return false
	end
	if not cc.FileUtils:getInstance():isFileExist('RoleModel/'..modelName.."/"..modelName.."_tex.json") then --判断是否需要加载
		self:addDownLoadList(modelName, 'RoleModel/', function()end)
		return false;
	end
	return true
end

function DownloadControler:checkNeedFightRes( fighterList )
	local ret = true
	for i,v in ipairs(fighterList) do
		local modelName = v
		if modelName then
			if not cc.FileUtils:getInstance():isFileExist('RoleModel/'..modelName.."/"..modelName.."_tex.json") then --判断是否需要加载
				self:addDownLoadList(modelName, 'RoleModel/', function()end)
				ret = false;
			end
		end
	end
	
	return ret
end

function DownloadControler:checkNeedMonsterRes(monsterIdList)
	local ret = true
	for i,v in ipairs(monsterIdList) do
		local monsterId = v
		if monsterId then
			local monsterData = ResTable:getRowData('monster', monsterId);
			if monsterData then
				local modelName = monsterData.model;
			    local naviName= monsterData.Navi;
				if not cc.FileUtils:getInstance():isFileExist('RoleModel/'..modelName.."/"..modelName.."_tex.json") then --判断是否需要加载
					self:addDownLoadList(modelName, 'RoleModel/', function()end)
					ret = false;
				end

				if naviName and not cc.FileUtils:getInstance():isFileExist('image/role/'..naviName..".png") then --判断是否需要加载
					self:addDownLoadList(naviName, 'image/role/', function()end)
					ret = false;
				end
			end
		end
	end
	
	return ret
end

function DownloadControler:isNeedDownload( name )
	-- body
end

function DownloadControler:testFB()
	-- body
	-- createDic(device.writablePath.."res/aaa/bbb/ccc")
	-- deletePath(device.writablePath.."res/aaa/bbb/ccc")
	UnzipFile(device.writablePath.."fbtest/test.zip", device.writablePath.."fbtest")
end

cc.exports.DownloadControler = DownloadControler:create();
