local LevelLockTools = class('LevelLockTools')
local fonts = Enum:getTable("SystemFont")

--[[
    api
    1 判断是否加锁 checkLockById(tag:start表对应ID)
    2 设置锁节点 setLockToNode(node:cc.Node,bAdd:bool,lockLv:number,lockType:number,lockPos:cc.p)

    -- demo 
    local locked,lockLv = tools.LevelLockTools:checkLockById(1);
    local lockType = 1;
    tools.LevelLockTools:setLockToNode(myNode,locked,lockLv,lockType,cc.p(0,0))

]]

function LevelLockTools:ctor(  )

end

-- tag = Enum.levelLockItemIds 
--ret bool,lockLv
function LevelLockTools:checkLockById( tag )

    if tag == 1000 then
        -- 测试 
        -- return true,10 ;
    end
    local levelLockItem = model.UserModel:getLevelLockItemById(tag);
    if levelLockItem == nil then
        return false;
    end

	local lv = model.UserModel:getUserLevel()
    return levelLockItem.lv > lv,levelLockItem.lv;

end


--==============================--
--desc: 给node增加锁 组件
--time:2019-09-10 02:45:58
--@node: 
--@bAdd: 加/删
--@lockType: 锁类型 1 主界面 商店 邮件  2, 主界面 商场 福利 不固定剪影 3 基地 子控件，4 战斗 快速战斗按钮,5 阵型解锁，圆形
--@lockPos: 锁坐标
--@return 
--==============================--
function LevelLockTools:setLockToNode( node,bAdd,lockLv,lockType,lockPos,lockImg )
    if node == nil then
        return 
    end
    local lockTag = Conf.TAG_LOCK_DEFAULT;
    local lockNode = node:getChildByTag(lockTag);

    if bAdd == false then
        if lockNode then
            lockNode:setVisible(false);
        end
        return 
    end

    if lockNode == nil then
        if lockPos == nil then
            lockPos = cc.p(0,0)
        end
        lockNode = self:_getLockNodeByType(lockType,lockLv,lockImg);
        lockNode:addTo(node)
        lockNode:setPosition(lockPos)
        lockNode:setTag(lockTag)
    end

    lockNode:setVisible(true)
end

-- 通知当前等级需要解锁的UI
function LevelLockTools:pushEventLevelUp( unlockId )
    print('=============================pushEventLevelUp========  ',unlockId)
    if unlockId ~= -1  then
        EventManager:FireEvent(EventType.UIEventUnlockChange,unlockId);
    end
end








--------------------------------private-------------------------------------

function LevelLockTools:_getLockNodeByType( lockType,lockLv,lockImg )
    
    local lockNode = nil;
    if lockType == 1 then
            -- 主界面 右侧 邮件聊天 方形
        lockNode = self:_getLockNodeTypeOne(lockLv);
    elseif lockType == 2 then 
            -- 主界面 商场 福利 不固定剪影
        lockNode = self:_getLockNodeTypeTwo(lockLv,lockImg);
    elseif lockType == 3 then 
            --基地 子按钮
        lockNode = self:_getLockNodeTypeThree(lockLv);
    elseif lockType == 4 then
        -- 战斗 快速战斗 菱形
        lockNode = self:_getLockNodeTypeFour(lockLv)
    elseif lockType == 5 then
        -- 队伍 阵型 圆形
        lockNode = self:_getLockNodeTypeFive(lockLv)
    elseif lockType == 6 then
        -- menu菜单
        lockNode = self:_getLockNodeTypeSix(lockLv)
    else
        lockNode = cc.Node:create();
        -- lockNode = self:getLockNodeTypeOne();
        print('error==============  lockNode需要提供类型！！！');
    end

    return lockNode;

end

function LevelLockTools:_getLockNodeTypeOne( lv )
    local node = cc.Node:create();

    --local bg = cc.Sprite:create('image/lock/item_bg5.png')
    --bg:addTo(node);

    local lock = cc.Sprite:create('image/lock/item_lock2.png')
    lock:addTo(node);
    lock:setPosition(cc.p(0,0))

    --local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_JP , 15);
	--lv:addTo(node);
    --lv:setPosition(0 , -27);
    --lv:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 1);

    return node;
end

function LevelLockTools:_getLockNodeTypeTwo( lv,lockImg )
    local node = cc.Node:create();

    if lockImg ~= nil then
        local bg = cc.Sprite:create(lockImg)
        bg:addTo(node);
    end

    local lock = cc.Sprite:create('image/lock/item_lock2.png')
    lock:addTo(node);
    lock:setPosition(cc.p(0,0))

    local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_JP , 15);
	lv:addTo(node);
    lv:setPosition(0 , -37);
	lv:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 1);


    return node;
end

function LevelLockTools:_getLockNodeTypeThree( lv )
    local node = cc.Node:create();

    local bg = cc.Sprite:create('image/lock/home_lock_bg.png')
    bg:addTo(node);

    local lock = cc.Sprite:create('image/lock/home_lock_.png')
    lock:addTo(node);
    lock:setPosition(cc.p(-37,0))

    local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_SIYUAN , 18);
	lv:addTo(node);
    lv:setPosition(-17, 0);
    lv:setAnchorPoint(0 , 0.5);
    lv:setTextColor(cc.c3b(255,255,0));
	lv:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);
    -- function node:setLockLv( lv )
    --     lv:setString(string.format( LANG.UI_ACTIVITY_25,lv ));
    -- end

    return node;
end

function LevelLockTools:_getLockNodeTypeFour( lv )
    local node = cc.Node:create();

    local bg = cc.Sprite:create('image/lock/item_bg4.png')
    bg:addTo(node);

    local titleBg = cc.Sprite:create('image/lock/item_title_bg4.png')
    titleBg:addTo(node)
    -- titleBg:setPosition(cc.p(0,-59))

    local lock = cc.Sprite:create('image/lock/item_lock1.png')
    lock:addTo(node);
    -- lock:setPosition(cc.p(-56,-58))

    -- local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_JP , 20);
	-- lv:addTo(node);
    -- lv:setPosition(19 , -56);
	-- lv:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 1);
    -- function node:setLockLv( lv )
    --     lv:setString(string.format( LANG.UI_ACTIVITY_25,lv ));
    -- end

    return node;
end

function LevelLockTools:_getLockNodeTypeFive( lv )
    local node = cc.Node:create();

    local bg = cc.Sprite:create('image/lock/item_bg6.png')
    bg:addTo(node);



    local lock = cc.Sprite:create('image/lock/item_lock2.png')
    lock:addTo(node);
    -- lock:setPosition(cc.p(-56,-58))

    local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_SIYUAN , 20);
	lv:addTo(node);
    lv:setPosition(0 , -37);
	lv:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);

    return node;
end

function LevelLockTools:_getLockNodeTypeSix( lv )
    local node = cc.Node:create();

    local bg = cc.Sprite:create('image/lock/lock_bg_1.png')
    bg:setPosition(82 , 0);
    bg:addTo(node);

    local lv =  cc.Label:createWithTTF(string.format( LANG.UI_ACTIVITY_25,lv ) ,  fonts.FONT_SIYUAN , 17);
	lv:addTo(node);
    lv:setAnchorPoint(0 , 0.5);
    lv:setPosition(41 , -3);

    return node;
end


return LevelLockTools