local SensitiveTools = class('SensitiveTools')

--[[ example 
    local strFrom = '   我是江泽 民000.2011 aa wyt.com,炸药fdsaf  '
    local strTo,bRet = tools.SensitiveTools:checkSensitive( strFrom )

    ---
    1 词库为json格式 ["","",""]
    2 词库加载时 词组默认转为小写，并去除空格

    3 检测时，默认去除空格，并且转为小写
]]

-- 敏感词过滤方法
--[[
    '敏感'
    '敏感词'
    --DFA 有限状态机
    hashRoot = {
        '敏' = {
            '感' = {
                '词' = {
                    'end' = 1
                },
                'end' = 1 --标记可终结
            }
        }
    }

]]

function SensitiveTools:ctor(  )
    self:clearAll()
end

function SensitiveTools:clearAll( ... )
    self.init = false;
    self.hashRoot = {}
end

function SensitiveTools:initSensitives(  )

    local lg = ''-- 屏蔽词库不做多语言切换 model.UserDefaultModel:getLanguageSet();
    -- if lg == 'cn' then
    --     lg = ''
    -- else
    --     lg = '_'..tostring(lg)
    -- end
    local index = 1
    local bTrue = true;
    while bTrue do
        bTrue = false
        local fileName = 'SensitiveWord'..tostring(lg)..index..'.json'
        local ret = self:loadFile(fileName)
        if ret == 0 then
            index = index + 1
            bTrue = true;
        end
    end
    

    self.init = true;
    -- self:printHash();
end

function SensitiveTools:checkSensitive( strFrom )

    if self.init ~= true then
        self:initSensitives()
    end

    -- local strFrom = '   我是江泽 民000.2011  wyt.com,大坏人sfds  '
    -- local strFrom = '   我是江泽1 民000.2011 aa wyt.com,大坏人sfds  '
   local strTo,bRet = self:transformString(strFrom)
   print('strFrom === ',strFrom)
   print('strTo   === ',strTo)
   return strTo,bRet
end


--==============================--
--desc:导入敏感词
--time:2019-12-01 02:07:03
--@sensitiveFile: 敏感词文件 （内容格式 固定为["","",""]）
--@return 
--==============================--
function SensitiveTools:loadFile( sensitiveFile )
    local fileData = nil;
    print('SensitiveTools =====  load file ===== ',sensitiveFile)
    local tablePath = 'res/table/'..tostring(sensitiveFile)
    local fullPath = device.writablePath..tablePath

    local fileUtils = cc.FileUtils:getInstance();
    if fileUtils:isFileExist(sensitiveFile) then
        fileData = fileUtils:getStringFromFile(sensitiveFile);
    elseif fileUtils:isFileExist(tablePath) then 
        fileData = fileUtils:getStringFromFile(tablePath);
    elseif fileUtils:isFileExist(fullPath) then
        fileData = fileUtils:getStringFromFile(fullPath);
    else
        print('=stop =======  =========no sensitiveFile name : '..tostring(sensitiveFile) )
        return -1
    end
    local jsonData =  Cjson.decode(fileData);
    local lowerWords = nil;
    for k,v in pairs(jsonData) do
        lowerWords  = string.lower( v )
        lowerWords = string.gsub(lowerWords, "%s", "") -- 去除所有空格
        self:addSensitiveKey(lowerWords)
    end

    return 0;

end

function SensitiveTools:addSensitiveKey( strKey )
    if strKey == nil or string.len( strKey ) == 0 then
        return
    end
    if self.hashRoot == nil then
        self.hashRoot = {}
    end
    local index = 1;
    local deep = 1;
    local hashNode = self.hashRoot;

    local nextCode,byteLen = self:findeNextChar(strKey,index)

    local hashFather,fatherCode = nil,nil;

    while nextCode  do
        -- print('index === ',index,'...nextCode = ',nextCode)
        if hashNode[nextCode] == nil then
            if deep == 1 then
                self:createNode(self.hashRoot,nextCode)
            else 
                self:createNode(hashFather,fatherCode,nextCode)
            end
        end

        if deep == 1 then
            hashFather = self.hashRoot;
        else
            hashFather = hashFather[fatherCode]
        end
        fatherCode = nextCode

        hashNode = hashNode[nextCode]

        deep = deep + 1;
        index = index + byteLen
        nextCode,byteLen = self:findeNextChar(strKey,index)
    end

    if deep == 1 then
        self.hashRoot[fatherCode]['end'] = 1
    else
       hashFather[fatherCode]['end'] = 1
    end

end
--
function SensitiveTools:createNode( t,k,k2 )
	if t == nil then 
		return
	end

	if t[k] == nil then 
		t[k] = {}
	end

	if k2 ~= nil and  t[k][k2] == nil then 
		t[k][k2] = {}
	end
end
--==============================--
--desc: 从idnexFrom检测是否有敏感词
--time:2019-12-01 02:48:13
--@strFrom:
--@indexFrom:起始位置
--@return 
--==============================--
function SensitiveTools:checkString( strFrom ,indexFrom)
    local bRet = false
    if self.hashRoot == nil then
        return -1,bRet
    end
    local index = indexFrom or 1;
    local nextCode,byteLen = self:findeNextChar(strFrom,index)
    if nextCode == nil then
        return -1,bRet;
    end

    local hashNode = self.hashRoot[nextCode]
    index = index + byteLen
    local trigerEndIndex = index;

    while hashNode ~= nil do
        -- print('===checkString  ...code = ',nextCode,'...index == ',index)
        if hashNode['end'] == 1 then
            bRet = true
            trigerEndIndex = index
        end
        nextCode,byteLen = self:findeNextChar(strFrom,index)
        hashNode = hashNode[nextCode]
        index = index + byteLen
    end

    -- print('=======bRet =  ',bRet,'..nextCode = ',nextCode,'... index === ',trigerEndIndex)
    return trigerEndIndex,bRet

end

function SensitiveTools:transformString( strFrom )
    local strTo = ''
    local hasSensitive = false
    local toIndexFrom = 1;

    strFrom = string.gsub(strFrom, "%s", "") -- 去除所有空格
    strFrom = string.lower( strFrom ) -- 转小写

    local len = string.len( strFrom )
    local index = 1;
    local nextIndex = 1;
    local bRet = false;
    while nextIndex <= len do
        nextIndex,bRet = self:checkString(strFrom,index)
        if bRet == true then
            -- 有敏感词
            -- print('index==== ',index,'...nextIndex == ',nextIndex)
            -- print('敏感词=== ',string.sub( strFrom,index,nextIndex-1 ))
        
            hasSensitive = true ;
            strTo = strTo..string.sub(strFrom,toIndexFrom,index-1)..string.rep( '*', nextIndex- index )
            toIndexFrom = nextIndex
        end
        if nextIndex >= len or nextIndex == -1 then
            -- 字符串到达末尾
            strTo = strTo..string.sub(strFrom,toIndexFrom,len)
            nextIndex = len+1
        end
        index =  nextIndex

    end
    return strTo,hasSensitive;
end

function SensitiveTools:getInstance(  )
    if SensitiveTools.instance == nil then
        SensitiveTools.instance =  SensitiveTools.new()

    end
end

-----------------------------tool----------------------------

-- 返回下一个字，以及所占字节长度
function SensitiveTools:findeNextChar( str,index )
    local curByte = string.byte(str, index)
    local byteCount = 1;
    if curByte == nil then
        byteCount = 0
        return nil,0
    elseif curByte > 0 and curByte <= 127 then
        byteCount = 1
    elseif curByte>=192 and curByte<=223 then
        byteCount = 2
    elseif curByte>=224 and curByte<=239 then
        byteCount = 3
    elseif curByte>=240 and curByte<=247 then
        byteCount = 4
    end

    return string.sub( str, index,index + byteCount-1 ) ,byteCount;
end


function SensitiveTools:printHash(  )
    local jsonStr = Cjson.encode(self.hashRoot)
    print('printRoot === ',jsonStr)
    -- self:saveJson(jsonStr,'hashRoot.json')
end

function SensitiveTools:saveJson( jsonStr,jsonFileName )
    
    local file = io.open(jsonFileName,'w')
    if file then
        print('==================savejson ======== ',jsonFileName)
        file:write(jsonStr)
        file:close()
    end
end

function SensitiveTools:saveSensitiveWordsToJson( configFile,saveName )
    local fileData = nil
    if debug then
        local file = io.open(configFile,'r');
        if file then
            fileData = file:read('*a')
            file:close()
        end
    else
        local fileUtils = cc.FileUtils:getInstance();
        fileData = fileUtils:getStringFromFile(device.writablePath.."/"..configFile);
    end

    if fileData then
        local jsonArr = {}
        local lineCount = 1
        local matchFunc = string.gmatch(fileData,"[^%,%s]+") -- 匹配非 空格和,
        local matchStr = matchFunc()
        while matchStr do 
            -- print('mathch =-= ',matchStr)
            jsonArr[lineCount] = matchStr
            lineCount = lineCount + 1;
            matchStr = matchFunc()
        end
        if next(jsonArr) then
            self:saveJson(Cjson.encode(jsonArr),saveName)
        end
    end
end


--- ceshi--------------


-- function SensitiveTools:new(...)
--     local instance = {}
--     setmetatable(instance, SensitiveTools)
--     instance:ctor(...)
--     return instance
-- end

--[[ 
    --测试 

function SensitiveTest(  )
    local sensitive = SensitiveTools.new()

    -- sensitive:saveSensitiveWordsToJson('./temp/sensitive1.txt','SensitiveWord1.json')
    -- sensitive:saveSensitiveWordsToJson('./temp/sensitive2.txt','SensitiveWord2.json')
    -- sensitive:saveSensitiveWordsToJson('./temp/sensitive3.txt','SensitiveWord3.json')
    -- sensitive:saveSensitiveWordsToJson('./temp/sensitive4.txt','SensitiveWord4.json')
    -- sensitive:saveSensitiveWordsToJson('./temp/sensitive5.txt','SensitiveWord5.json')
    
    sensitive:loadFile('SensitiveWord1.json')
    sensitive:loadFile('SensitiveWord2.json')
    sensitive:loadFile('SensitiveWord3.json')
    sensitive:loadFile('SensitiveWord4.json')
    sensitive:loadFile('SensitiveWord5.json')
    
    -- sensitive:printHash()
    
    print('========222222222====================')
    local strFrom = '   我是江泽 民000.2011  wyt.com,大坏人sfds  '
    local strFrom = '   我是江泽1 民000.2011 aa wyt.com,大坏人sfds  '
   local strTo = sensitive:transformString(strFrom)
   print('strFrom === ',strFrom)
   print('strTo   === ',strTo)
    
end
SensitiveTest()

]]
return SensitiveTools












