local scheduler = cc.Director:getInstance():getScheduler();

local SDKLoginType = Enum:getTable("SDKLoginType")
local fonts = Enum:getTable("SystemFont")
local MessageBoxType = Enum:getTable('MessageBoxType')
local cvTypeEnum = Enum:getTable("cvTypeEnum");

local TITLE_SCENE_STEP = {
	STEP_BEGIN 			= 1,	-- 开始
	STEP_CLAUSE			= 2,	-- 条款
	STEP_ANNOUNCEMENT	= 3,	-- 公告
	STEP_PLATFORM_SELECT= 4,	-- 选择登陆方式
	STEP_SERVER_LIST	= 5,	-- 请求服务器列表
	STEP_ENTER_GAME		= 6,	-- 进入下一场景
}

local TitleScene = class("TitleScene", function()
	local scene = nil;
	scene = display.newScene("TitleScene")

	return scene;
end)

function TitleScene:ctor()
	local function onNodeEvent(event)
		if event == "enter" then
			self:onEnter()
		elseif event == "exit" then
			self:onExit()
		end
	end
	self:registerScriptHandler(onNodeEvent);
	self.m_scheduleHandler = nil;

	local sp = cc.Sprite:create('image/commonImg/lbg.png');
	local texture = sp:getTexture();
	texture:setTexParameters(gl.LINEAR, gl.LINEAR, gl.REPEAT, gl.REPEAT)
	local test = cc.Sprite:createWithTexture(texture, cc.rect(0, 0, StageSize.width, StageSize.height))
	test:addTo(self)
	test:setPosition(StageSize.width/2, StageSize.height/2)
	test:setLocalZOrder(-1);

	self:initUI()
	self:initClauseNode()
	self:initAnnouncementNode()
	self:initLoginTypeSelect()
	self:initSelectServerNode()
	self:initDownJson()
	self:registEvent()

	self.step = TITLE_SCENE_STEP.STEP_BEGIN;
	MessageBox:init(self, self)
	
end

function TitleScene:initUI( ... )
	local nodePrompt = cc.Node:create();
	nodePrompt:setPosition(display.center);
	nodePrompt:setLocalZOrder(2000);
	nodePrompt:addTo(self);
	Prompt:Init(nodePrompt, self);

	-- bg
	self.bgNode = cc.Node:create();
	self.bgNode:setPosition(StageSize.width/2, StageSize.height/2);
	self.bgNode:addTo(self);

	local bg = cc.Sprite:create('image/title/bg.png');
	bg:setLocalZOrder(-100);
	bg:addTo(self.bgNode);

	local spine_res = 'fengmian_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(self.bgNode)
		spine:setAnimation(0, 'idle', true)
	end  

	local spine_res = 'logo_fengmian_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(self.bgNode)
		spine:setAnimation(0, GlobalFunctions:lAnimationName( 'idle' ), true)
		spine:setPosition(0, StageSize.height/2-147)
	end 

	--local logo = cc.Sprite:create('image/title/logo.png')
	--:setPosition(0, StageSize.height/2-147)
	--:addTo(self.bgNode)

	local tabToStart = cc.Sprite:create('image/title/tabToStart.png')
	:setPosition(0, -StageSize.height/2+129)
	:addTo(self.bgNode)

	local tabToStartTitle = cc.Sprite:create('image/title/tabToStartTitle.png')
	:setPosition(0, -StageSize.height/2+128)
	:addTo(self.bgNode)
	:setOpacity(0)
	:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.FadeIn:create(0.5),cc.FadeOut:create(0.5),cc.DelayTime:create(0.5))))

	self.versionLabel = cc.Label:createWithSystemFont('v1.0.2', 'arial', 24);
	self.versionLabel:setTextColor(cc.c3b(0, 0, 0))
	self.versionLabel:setPosition(StageSize.width/2-10, -StageSize.height/2+25);
	self.versionLabel:setAnchorPoint(cc.p(1, 1));
	self.versionLabel:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT);
	self.versionLabel:setLocalZOrder(100);
	self.versionLabel:addTo(self.bgNode);

	self.touchLayer = cc.Layer:create()
	self.touchLayer:setContentSize(cc.size(StageSize.width, StageSize.height));
	self.touchLayer:setTouchEnabled(true);
    self.touchLayer:registerScriptTouchHandler(handler(self, self.onTouch));
	self.touchLayer:addTo(self);

	local particle1 = cc.ParticleSystemQuad:create('Effect/particle/mainf1.plist')
		:setPosition(0, StageSize.height/2)
		:addTo(self.bgNode)

	local particle2 = cc.ParticleSystemQuad:create('Effect/particle/mainf2.plist')
		:setPosition(0, StageSize.height/2)
		:addTo(self.bgNode)

	local particle3 = cc.ParticleSystemQuad:create('Effect/particle/mainf3.plist')
		:setPosition(0, StageSize.height/2)
		:addTo(self.bgNode)

	local label = cc.Label:createWithTTF(LANG.UI_TITLE_28,fonts.FONT_SIYUAN,20);
	self.loginOutBtn = ccui.Button:create('image/title/title_btn.png')
		:setPosition(StageSize.width-85, StageSize.height-36)
		:setTitleLabel(label)
		:addTo(self)
	label:setPosition(self.loginOutBtn:getContentSize().width/2+15,self.loginOutBtn:getContentSize().height/2+2);

	self.loginOutBtn:addClickEventListener(handler(self, self.logout))

	self.serverNode = cc.Node:create()
	:addTo(self)
	:setVisible(false)
	:setPosition(StageSize.width/2, 230)
	local serverBtn = ccui.Button:create('image/title/server_btn.png' , 'image/title/server_btn.png');
	serverBtn:addTo(self.serverNode);
	serverBtn:addClickEventListener(handler(self, self.showSelServerNode))

	local rbg = cc.Sprite:create('image/commonImg/ric_27.png')
	:addTo(self.serverNode)
	:setPosition(-160,0)
	self.servreTipbg = cc.Sprite:create('image/commonImg/ric_green.png')
	:addTo(self.serverNode)
	:setPosition(-160,0)

	self.serverName = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 24);
	self.serverName:setPosition(-140, 0);
	self.serverName:setAnchorPoint(0,0.5)
	self.serverName:addTo(self.serverNode);

	self.serverTip = cc.Label:createWithTTF(LANG.UI_TITLE_SERVER_6, fonts.FONT_SIYUAN, 24);
	self.serverTip:setPosition(150, 0);
	self.serverTip:setAnchorPoint(1,0.5)
	self.serverTip:addTo(self.serverNode);
	self.serverID = -1;

	self.copyRightLabel = cc.Label:createWithTTF('Copyright 2020 ⓒ NaNi soft. All rights reserved.', fonts.FONT_SIYUAN, 18);
	self.copyRightLabel:setPosition(0, 15-StageSize.height/2);
	self.copyRightLabel:setDimensions(450, 0)
	self.copyRightLabel:addTo(self.bgNode)
	self.copyRightLabel:setTextColor(cc.c3b(136,136,136))
end

function TitleScene:initAnnouncementNode( ... )
	self.announcementNode = cc.Node:create();
	self.announcementNode:addTo(self);
	self.announcementNode:setPosition(StageSize.width/2, StageSize.height/2);
	self.announcementNode:setLocalZOrder(100);

	local bg = ccui.Layout:create()
		:setBackGroundImage('image/task/blackbg.png')
		:setBackGroundImageScale9Enabled(true)
		:setTouchEnabled(true)
		:setContentSize(StageSize)
		:setLocalZOrder(-1)
		:setAnchorPoint(cc.p(0.5,0.5))
		:addTo(self.announcementNode)

	local bg2 = ccui.Scale9Sprite:create("image/commonImg/bg_c9_1_2.png")
		:setCapInsets(CCRectMake(18,50,123,73)) -- (left,top,width,heigth)
		:setContentSize(cc.size(579 , 744))
		:setPosition(cc.p(0 , 0))
		:addTo(self.announcementNode)

	local bg3 = ccui.Scale9Sprite:create("image/commonImg/bg_c9_2_3.png")
		:setCapInsets(CCRectMake(10,10,117,94))
		:setContentSize(cc.size(546 , 595))
		:setPosition(cc.p(0 , 15))
		:addTo(self.announcementNode)

	self.announcementListView = ccui.ListView:create()
		:ignoreContentAdaptWithSize(false)
		:setAnchorPoint(cc.p(0.5,0.5))
		:setBounceEnabled(true)
		:setDirection(ccui.ScrollViewDir.vertical)
		:setContentSize(cc.size(540 , 580))
		:setScrollBarEnabled(false)
		:setItemsMargin(10)
		:setPosition(cc.p(0 , 17))
		:addTo(self.announcementNode)
	
	local title = cc.Label:createWithTTF(LANG.UI_GUILD_78, Enum:getTable("SystemFont").FONT_JP, 18)
		:setPosition(0, 350)
		:setTextColor(cc.c4b(200 , 200 , 200 , 255))
		:addTo(self.announcementNode)

	local btnLabel = cc.Label:createWithTTF(LANG.COMMON_OK , fonts.FONT_SIYUAN , 20);
		btnLabel:enableOutline(cc.c4b(55 , 85 , 155 , 255) , 2);
	self.ok = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png')
		:addTo(self.announcementNode)
		:setTitleLabel(btnLabel)
		:setPosition(0 , -326)
	self.ok:addClickEventListener(function() 
		self:closeAnnouncementNode();
	end);
	self.announcementNode:setVisible(false);
end

function TitleScene:initSelectServerNode(  )
	self.selectServerNode = require('app.layer.MainScene.SelectServerPanel'):create()
	if self.selectServerNode then
		self.selectServerNode:addTo(self)
			:setPosition(cc.p(StageSize.width/2, StageSize.height/2 ))
			:setVisible(false)
	end
end

function TitleScene:initClauseNode()
	local url1='https://ramengo.nanisoft.co.kr/policy/en/info.html'
	local url2='https://ramengo.nanisoft.co.kr/policy/en/privacy.html'
	if GlobalFunctions:isKorean() then
		url1='https://ramengo.nanisoft.co.kr/policy/info.html'
		url2='https://ramengo.nanisoft.co.kr/policy/privacy.html'
	end
	self.clauseList = { 	[1]={ check=false, must=true, msg=LANG.UI_TITLESCENE_MSG[1], url=url1}, 
							[2]={ check=false, must=true, msg=LANG.UI_TITLESCENE_MSG[2], url=url2},
							[3]={ check=false, must=false, msg=LANG.UI_TITLESCENE_MSG[3], url=''}, 
					}
	local clauseNode = cc.Node:create()
	:setLocalZOrder(201)
	:setPosition(StageSize.width/2, StageSize.height/2)
	:addTo(self)
	:setVisible(false)

	self.clauseNode = clauseNode;

	local accountLayout = ccui.Layout:create()
		:setBackGroundImage('image/task/blackbg.png')
		:setBackGroundImageScale9Enabled(true)
		:setTouchEnabled(true)
		:setContentSize(StageSize)
		:setLocalZOrder(-1)
		:setAnchorPoint(cc.p(0.5,0.5))
		:addTo(clauseNode)

	local bg1 = cc.Sprite:create('image/commonImg/common_tips_bg.png')
		:setPosition(0, 0)
		:addTo(clauseNode)

	local bg2 = cc.Scale9Sprite:create('image/commonImg/bg_3_200_42.png');
	bg2:addTo(clauseNode);
	bg2:setContentSize(cc.size(582 , 186));

	local titlelabel = cc.Label:createWithTTF(LANG.UI_TITLE_14 , fonts.FONT_SIYUAN , 20)
	:addTo(self.clauseNode)
	:setPosition(0, 135)

	for i,v in ipairs(self.clauseList) do
		print(i,v)
		local cell = cc.Node:create()
		:addTo(clauseNode)
		:setPosition(cc.p(-250, 100-50*i))

		local label = cc.Label:createWithTTF(v.msg , fonts.FONT_SIYUAN , 20);
		label:addTo(cell)
		label:setPosition(50, 0)
		label:setAnchorPoint(cc.p(0,0.5))
		label:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)

		local checked = v.check;
		local checkbox = ccui.CheckBox:create()
		checkbox:setSelected(checked)
		checkbox:loadTextures("image/setup/ui_audio_bg.png","image/setup/ui_audio_bg.png","image/setup/lang_s.png","image/setup/ui_audio_bg.png","image/setup/ui_audio_bg.png",ccui.TextureResType.localType)
		checkbox:setEnabled(true)
		checkbox:setTag(i)
		cell:addChild(checkbox)
		   
	    local function tick(Ref,type)
	       	if Ref == checkbox then
	       		local idx = Ref:getTag()
	       		if type == ccui.CheckBoxEventType.selected then
	       			self.clauseList[idx].check = true
	       		else
	       			self.clauseList[idx].check = false
	       		end
		    end
		    
		    self.agreeBtn:setEnabled(true)
		    for _,v in pairs(self.clauseList) do
		    	if v.must == true and v.check == false then
		    		self.agreeBtn:setEnabled(false)
		    		break;
		    	end
		    end
	    end
	    checkbox:addEventListener(tick)

	    if v.url ~= '' then
			local label = cc.Label:createWithTTF(LANG.UI_TITLE_16 , fonts.FONT_SIYUAN , 20);
			label:addTo(cell)
			label:setPosition(450, 0)
			label:setAnchorPoint(cc.p(1,0.5))
			label:setTextColor(cc.c3b(135, 155, 190))
			label:setColor(cc.c3b(135, 155, 190))
			label:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
			label:enableUnderline()

			local btn = ccui.Button:create('image/commonImg/top_1_bg_59_30.png');
			btn:addTo(cell);
			btn:setPosition(450, 0);
			btn:setAnchorPoint(1, 0.5)
			btn:setContentSize(cc.size(label:getContentSize().width , 30))
			btn:setScale9Enabled(true)
			btn:setTag(i)
			btn:addClickEventListener(handler(self, self.checkAllMsg));
			btn:setOpacity(0)
		end
	end

	local btnLabel = cc.Label:createWithTTF(LANG.UI_TITLE_22 , fonts.FONT_SIYUAN , 20);
	btnLabel:enableOutline(cc.c4b(50 , 85 , 150 , 255) , 2);
	self.agreeBtn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	self.agreeBtn:loadTextureDisabled( 'image/commonImg/btn_1_219_55_2.png' )
	self.agreeBtn:addTo(clauseNode);
	self.agreeBtn:setPosition(-150 , -130);
	self.agreeBtn:setEnabled(false);
	self.agreeBtn:setTitleLabel(btnLabel);
	self.agreeBtn:addClickEventListener(handler(self, self.agree));

	local btnLabel1 = cc.Label:createWithTTF(LANG.UI_TITLE_13 , fonts.FONT_SIYUAN , 20);
	btnLabel1:enableOutline(cc.c4b(50 , 85 , 150 , 255) , 2);
	self.agreeAllBtn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	self.agreeAllBtn:addTo(clauseNode);
	self.agreeAllBtn:setPosition(150 , -130);
	self.agreeAllBtn:setEnabled(true);
	self.agreeAllBtn:setTitleLabel(btnLabel1);
	self.agreeAllBtn:addClickEventListener(handler(self, self.agreeAll));

end

function TitleScene:agree()
	GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').term_agree);
	self.clauseNode:setVisible(false)
	model.UserDefaultModel:setClauseSetState(true)
	
	if self.clauseList[3].check == true then
		self:changePush(1)
	else
		self:changePush(2)
	end
	self:checkStep();
end

function TitleScene:agreeAll()
	GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').term_all);
	self.clauseNode:setVisible(false)
	model.UserDefaultModel:setClauseSetState(true)
	self:changePush(1)
	self:checkStep();
end

function TitleScene:changePush( state )
	local function callback2( newState )
		local pushState = model.UserDefaultModel:getPushEventState()
		if pushState ~= 0 then
			local curTime = os.date("(%Y-%m-%d)",os.time())
			MessageBox:create(LANG.UI_TITLE_15.."\n"..tostring(curTime),MessageBoxType.OK, function () end)
		end
	end
	model.UserModel:changePushState(state,callback2)
end

function TitleScene:checkAllMsg(sender , event)
	-- body
	local idx = sender:getTag()
	local url = self.clauseList[idx].url
	print('url:',url)

	cc.Application:getInstance():openURL(url)
end

function TitleScene:initLoginTypeSelect(  )
	self.loginSelectNode = cc.Node:create()
		:setLocalZOrder(200)
		:setPosition(StageSize.width/2, StageSize.height/2)
		:addTo(self)
		:setVisible(false)

	local selectLayout = ccui.Layout:create()
		:setBackGroundImage('image/task/blackbg.png')
		:setBackGroundImageScale9Enabled(true)
		:setTouchEnabled(true)
		:setContentSize(StageSize)
		:setLocalZOrder(-1)
		:setAnchorPoint(cc.p(0.5,0.5))
		:addTo(self.loginSelectNode)

	local title = cc.Label:createWithTTF(LANG.UI_TITLE_23, Enum:getTable("SystemFont").FONT_JP, 35)
		:setPosition(0, 210)
		:addTo(self.loginSelectNode);
	
	local title2 = cc.Label:createWithTTF(LANG.UI_TITLE_ACCOUNT, Enum:getTable("SystemFont").FONT_JP, 20)
		:setPosition(0, 170)
		:addTo(self.loginSelectNode);

	local loginType = {}

	local platform = cc.Application:getInstance():getTargetPlatform()
	if (cc.PLATFORM_OS_IPHONE == platform) or (cc.PLATFORM_OS_IPAD == platform) then --or (cc.PLATFORM_OS_MAC == platform) 
		loginType = {
			SDKLoginType.LOGIN_IOS,
			SDKLoginType.LOGIN_GOOGLE,
			SDKLoginType.LOGIN_FACEBOOK,
		};
	else
		loginType = {
			SDKLoginType.LOGIN_GOOGLE,
			SDKLoginType.LOGIN_FACEBOOK,
			SDKLoginType.LOGIN_GUEST,
		};

	end

	for i=1, #loginType do
		local btn = ccui.Button:create('image/login/login_type_'..string.format( "%.3d",loginType[i] )..'.png')
			:setPosition(0, 175-i*100)
			:addTo(self.loginSelectNode)
			:setTag(loginType[i])
			:addClickEventListener(handler(self, self.onClickLoginType))
	end
end

function TitleScene:registEvent(  )
	EventManager:RegisterEvent(EventType.UILoginAnnouncement, handler(self, self.showAnnouncementNode));
	EventManager:RegisterEvent(EventType.UILoginShowServerInfo, handler(self, self.showServerInfo));
	EventManager:RegisterEvent(EventType.UILoginWithUID, handler(self, self.retServerLoginUID));
	EventManager:RegisterEvent(EventType.LoginRevertUnreg, handler(self, self.retRevertUnregist));
	EventManager:RegisterEvent(EventType.UILoginShowUnregInfo, handler(self, self.showUnregistMessage));
	EventManager:RegisterEvent(EventType.UILoginRecoverAccount, handler(self, self.retRecoverAccount));
	
end

function TitleScene:onEnter()
	self.isSavingJson = false;
	self.showAnnouncement = false;
	self.receiveServerList = false;
	self.chooseServer = false;

	GlobalFunctions:recordTime('fb-log---TitleSceneEnter')
	GlobalFunctions:clearAllModel(  )

	-- music
	--AudioManager:setMusicVolume(0.2);
	AudioManager:SetState(Enum:getTable('GameState').Login);
	AudioManager:playMusic(Conf.title, 1, true);

	-- version
	local newVersionFile = define.writablePath .. "/new_version/project.manifest";
	local fileUtils = cc.FileUtils:getInstance();
	local file;
	if fileUtils:isFileExist(newVersionFile) then
		local fileData = fileUtils:getStringFromFile(newVersionFile);
		file = json.decode(fileData);
	else
		local fileData = fileUtils:getStringFromFile('version/project.manifest');
		file = json.decode(fileData);
	end
	self.versionLabel:setString(tostring(file['version']));
	model.AccountModel:setGameVersion(tostring(file['version']));

	-- role voice
	cc.SimpleAudioEngine:getInstance():playEffect(AudioManager:getCVPath()..'login_cv/login_cv.mp3', false, 1);
	
	-- check step
	self.step = TITLE_SCENE_STEP.STEP_CLAUSE;
	self:checkStep();
end

function TitleScene:onExit()
	--调度器销毁
	GlobalFunctions:recordTime('fb-log---TitleSceneExit')
	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
end

function TitleScene:checkStep()
	if TITLE_SCENE_STEP.STEP_CLAUSE == self.step then
		local clauseSet = model.UserDefaultModel:getClauseSetState()
		if clauseSet then
			self.step = TITLE_SCENE_STEP.STEP_ANNOUNCEMENT;
			self:checkStep()
		else
			self.clauseNode:setVisible(true);
			GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').term_on);
		end

	elseif TITLE_SCENE_STEP.STEP_ANNOUNCEMENT == self.step then
		if self.showAnnouncement then
			self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
			self:checkStep()
		else
			model.AccountModel:reqAnnouncement();
		end

	elseif TITLE_SCENE_STEP.STEP_PLATFORM_SELECT == self.step then
		self.platform_type = model.UserDefaultModel:getPlatformType();
		if self.platform_type == SDKLoginType.EMPTY then
			GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').login_on);
			self.loginSelectNode:setVisible(true);
			--[[
			local args = {
				eventName = Enum:getTable('TrackEventToken').TRACK_LOGIN_ON
			}
			sdkManager.SDKLuaHelper:trackEvent(args)
			]]
			GlobalFunctions:trackEvent('TRACK_LOGIN_ON')
			
		else
			GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').login_success);
			self:onSelectLoginType(self.platform_type)
		end
	
	elseif TITLE_SCENE_STEP.STEP_SERVER_LIST == self.step then
		if self.receiveServerList then
			self.step = TITLE_SCENE_STEP.STEP_ENTER_GAME
		else
			self.serverNode:setVisible(false)
			model.AccountModel:reqServerInfo()
		end 

	elseif TITLE_SCENE_STEP.STEP_ENTER_GAME == self.step then
		if self.chooseServer then
			self:enterNextScene()
		end
	end

end

function TitleScene:onClickLoginType( sender, event)
	local tag = sender:getTag();
	self:onSelectLoginType(tag)
end

function TitleScene:onSelectLoginType( tag )
	model.UserDefaultModel:setPlatformType(tag);
	self.platform_type = tag;

	self.loginSelectNode:setVisible(false)

	if tag == SDKLoginType.LOGIN_ACCOUNT then 

	elseif tag == SDKLoginType.LOGIN_GUEST then 
		self.step = TITLE_SCENE_STEP.STEP_SERVER_LIST
		self:checkStep()
	else
		sdkManager.SDKLuaHelper:signIn(self.platform_type,handler(self,self.retSDKLogin));
	end
end

function TitleScene:onTouch(eventType, x, y)
	if self.isSavingJson then
		return;
	end
	if not self.receiveServerList then
		return
	end
	if self.step == TITLE_SCENE_STEP.STEP_ENTER_GAME then 
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').start_push)
		if not model.AccountModel:reqEnterServer(self.serverID) then
			print('error, platform type')
		end
	end
end

function TitleScene:logout( sender,event )
	if self.step < TITLE_SCENE_STEP.STEP_SERVER_LIST then
		return
	end
	self.receiveServerList = false;
	if self.platform_type == SDKLoginType.LOGIN_ACCOUNT or self.platform_type == SDKLoginType.LOGIN_GUEST then
		model.UserDefaultModel:setPlatformType(SDKLoginType.EMPTY);
		self.platform_type = SDKLoginType.EMPTY;
		self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
		self:checkStep()
	elseif self.platform_type == SDKLoginType.EMPTY then
		self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
		self:checkStep()
	else
		model.AccountModel:sdkSignOut(self.platform_type, function()
			model.UserDefaultModel:setPlatformType(SDKLoginType.EMPTY);
			self.platform_type = SDKLoginType.EMPTY;
			self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
			self:checkStep()
		end)
	end
	self.serverNode:setVisible(false);
end

function TitleScene:retSDKLogin( strData )
	print("sdkLoginCallback === "..tostring(strData));
	local jsonData = Cjson.decode(strData)
	if jsonData == nil or jsonData.code ~= 0 then
		model.AccountModel:sdkSignOut(self.platform_type, function()
			model.UserDefaultModel:setPlatformType(SDKLoginType.EMPTY);
			self.platform_type = SDKLoginType.EMPTY;
			self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
			self:checkStep()
		end)
		return 
	end

	model.AccountModel:setSDKLoginInfo( jsonData )
	self.step = TITLE_SCENE_STEP.STEP_SERVER_LIST
	self:checkStep()
end

function TitleScene:showServerInfo(msg)
	self.serverNode:setVisible(true)

	local serverInfo = model.AccountModel:getServerInfo()
	-- set tip
	local defaultServer = model.UserDefaultModel:getDefaultServer()
	local aimKey = nil;
	for k, v in ipairs(serverInfo) do
		aimKey = k;
		if defaultServer > 0 and defaultServer == v['server'] then
			break;
		end
	end
	
	self.serverID = serverInfo[aimKey]['server'];
	self.serverName:setString(ResTable:getLangByNum( tonumber(serverInfo[aimKey]['name']) ))
	local tbgName = 'ric_green.png'
	local st = serverInfo[aimKey]['state']
	if st == 0 or st==3 then
		tbgName = 'ric_green.png'
	elseif st == 1 then
		tbgName = 'ric_red.png'
	else
		tbgName = 'ric_grey.png'
	end
	self.servreTipbg:setTexture('image/commonImg/'..tbgName);
	
	self.receiveServerList = true
	self:checkStep()
end

function TitleScene:retServerLoginUID(msg)
	self.chooseServer = true;
	self.account_long_time = msg.account_long_time;
	self.account = msg.account
	self.account_new = msg.account_new;
	self:checkStep()
end

function TitleScene:retRecoverAccount(msg)	
	MessageBox:create(LANG.ACCOUNT_12, MessageBoxType.OK, function()
		self.account_new = msg.account_new;
		self:checkStep()
	end, function()
		self.account_new = msg.account_new;
		self:checkStep()
	end)
end

function TitleScene:enterNextScene()
	if self.account_long_time then		
		local msgBox = MessageBox:create(LANG.ACCOUNT_10, MessageBoxType.CANCEL_OK, function()
			model.AccountModel:reqRecoverAccount(self.account);
		end, function()
			cc.Director:getInstance():endToLua();
		end, function()
			cc.Director:getInstance():endToLua();
		end);
		MessageBox:setOkBtnName(LANG.ACCOUNT_11);
	elseif self.account_new then
		App:run("NewPlayerScene");
	else
		AudioManager:playEffect(Conf.commonLogin , 1);
		App:run("LoginScene");
	end
end

function TitleScene:showSelServerNode(  )
	if self.selectServerNode then
		self.selectServerNode:onEnter();
	end
end

--==================================================================
-- Announcement
function TitleScene:showAnnouncementNode(msg)
	self.announcementListView:removeAllChildren();
	if msg and msg.announcement then
		local announcements = {};
		for _k , announcement in pairs(msg.announcement) do
			table.insert(announcements , announcement);
		end
		local num = #announcements;
		if num <= 0 then
			self:closeAnnouncementNode();
			return
		end
		if num > 1 then
			table.sort(announcements , function(first , second)
				return first.order < second.order;
			end);
		end
		local y = -10;
		local announcementNode = cc.Node:create();
		for _k , data in pairs(announcements) do
			local arrow = cc.Sprite:create('image/commonImg/arrow_5.png')
				:addTo(announcementNode)
				:setAnchorPoint(cc.p(0 , 1))
				:setPosition(0 , y)
			local title = cc.Label:createWithTTF(tostring(data.title) , Enum:getTable("SystemFont").FONT_JP , 20);
			title:setAnchorPoint(cc.p(0 , 1));
			title:setPosition(50 , y)
			title:setTextColor(cc.c4b(7 , 176 , 255 , 255))
			title:addTo(announcementNode);

			y = y - title:getContentSize().height - 6;


			local formatStr , colorPos = self:formatText(data.info);
			local info = cc.Label:createWithTTF(tostring(formatStr) , Enum:getTable("SystemFont").FONT_HUAKANG , 18);
			info:addTo(announcementNode);
			info:setAnchorPoint(cc.p(0 , 1));
			info:setLineHeight(24);
			info:setLineBreakWithoutSpace(true);
			info:setMaxLineWidth(535)
			info:setPosition(0 , y);
			y = y - info:getContentSize().height - 30;

			for _k , pos in pairs(colorPos) do
				for i = pos.sPos-1 , pos.ePos-1 do
					local label = info:getLetter(i);
					if label then
						label:setColor(pos.color);
					end
				end
			end
		end
		local layout = ccui.Layout:create();
		layout:setContentSize(cc.size(561 , -y));
		layout:addChild(announcementNode);
		announcementNode:setPosition(5 , -y);
		self.announcementListView:pushBackCustomItem(layout);

		self.announcementNode:setVisible(true);
	else
		self:closeAnnouncementNode();
	end

end

function TitleScene:closeAnnouncementNode()
	self.announcementNode:setVisible(false);
	self.showAnnouncement = true;
	self:checkStep();
end

function TitleScene:formatText(str)
	local pos = 1;
	local strPos = {};
	local cStr = '';
	local startPos = string.find(str , "<" , pos);
	if not startPos then
		cStr = str;
	else
		local endPos = 0;
		while (startPos) do
			if startPos ~= pos then
				cStr = cStr..string.sub(str , pos , startPos - 1);
			end
			endPos = string.find(str , ">" , startPos+1);
			local cStrStartPos = self:getStrLen(cStr);
			local strColorPos = {};
			local colorStartPos = string.find(str , "color:" , startPos+1);
			cStr = cStr..string.sub(str , startPos + 1 , colorStartPos - 1);

			local colorEnderPos = string.find(str , "," , colorStartPos + 6);
			local R = tonumber(string.sub(str , colorStartPos + 6 , colorEnderPos-1));

			colorStartPos = colorEnderPos;
			colorEnderPos = string.find(str , "," , colorStartPos + 1);
			local G = tonumber(string.sub(str , colorStartPos + 1 , colorEnderPos-1));

			local B = tonumber(string.sub(str , colorEnderPos + 1 , endPos-1));
			table.insert(strPos , {sPos = cStrStartPos + 1 ,  ePos = self:getStrLen(cStr), color = cc.c3b(R , G , B)});
			pos = endPos + 1;
			startPos = string.find(str , "<" , pos);
		end
		if endPos ~= #str then
			cStr = cStr..string.sub(str , pos , #str);
		end
	end
	return cStr , strPos;
end

function TitleScene:getStrLen(text)
	local nextUtf8strIndex = function(str , index)
		if string.byte(str , index) > 127 then
			index = index + 3;
		elseif string.byte(str , index) < 128 then
			index = index + 1;
		end
		return index;
	end
	local index = 1;
	local len = 0;
	while index < string.len(text) do
		local oldIndex = index;
		index = nextUtf8strIndex(text , index)
		len = len + 1;
	end
	return len;
end

--======================= unreg ========================================
function TitleScene:showUnregistMessage(msg)
	local msgBox = MessageBox:create(LANG.ACCOUNT_6, MessageBoxType.CANCEL_OK, function()
		model.AccountModel:reqUnregisterSDK(0);
	end, function()
		model.AccountModel:setSDKLoginInfo({})
		model.UserDefaultModel:setPlatformType( 0 ) 
		sdkManager.SDKLuaHelper:signOut(self.platform_type, function()
			self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
			self:checkStep();
		end)
	end, function()
		model.AccountModel:setSDKLoginInfo({})
		model.UserDefaultModel:setPlatformType( 0 ) 
		sdkManager.SDKLuaHelper:signOut(self.platform_type, function()
			self.step = TITLE_SCENE_STEP.STEP_PLATFORM_SELECT;
			self:checkStep();
		end)
	end);
	msgBox:setTitle(LANG.UI_TITLE_NOTICE);
	MessageBox:setOkBtnName(LANG.UI_TITLE_17);
end

function TitleScene:retRevertUnregist( msg )
	local msgBox = MessageBox:create(LANG.ACCOUNT_7, MessageBoxType.OK, function()
		self.serverNode:setVisible(false)
		model.AccountModel:reqEnterServer(self.serverID);
	end, function()
	end, function()
		self.serverNode:setVisible(false)
		model.AccountModel:reqEnterServer(self.serverID);
	end);
	msgBox:setTitle(LANG.UI_TITLE_NOTICE);
end

--======================= test code ====================================

function TitleScene:initDownJson(  )
	--debug_model 显示分支
	if define.debug_model == true then
		--重新更新json数据

		self.reloadJsonlabel = cc.Label:createWithTTF(LANG.UI_TITLE_UPDATE,fonts.FONT_SIYUAN,20);
		self.reloadJsonBtn = ccui.Button:create('image/title/title_btn.png')
			:setPosition(StageSize.width-85, StageSize.height-83)
			:setTitleLabel(self.reloadJsonlabel)
			:addTo(self)
		self.reloadJsonBtn:addClickEventListener(handler(self, self.SaveJson));
		--self.reloadJsonlabel:setPositionX(self.loginOutBtn:getContentSize().width/2+17);

		local branch = cc.Label:createWithTTF('channel - '..tostring(global_branch.branch), Enum:getTable("SystemFont").FONT_JP, 20)
		:setPosition(StageSize.width-14, StageSize.height-120)
		:setAnchorPoint(cc.p(1,0.5))
		:addTo(self)
	end

end

function TitleScene:SaveJson()
	self.isSavingJson = true;
	self.reloadJsonlabel:setString('updateing')

	if not self.m_scheduleHandler then
		self.time = 0;
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.update), 0, false);
	end
	local req = {};
	req.reqcode = 2004;
	getDBdata(self, define.server_url, req, function(response) 
		self.jsonLoadingFlag = true;
		self.coroutine = coroutine.create(function()
			for key,val in pairs(response.db) do
				if(key ~= 'config')then
					print('save', key)
					jsonDataSave(val, define.jsonDataPath..key..".json")
					coroutine.yield();
				end
			end
			print('save over')
			self.reloadJsonlabel:setString(LANG.UI_UPDATE_1)
			self.isSavingJson = false;
		end)
	end, 0)
end

function TitleScene:update(dt)
	self.time = self.time + 1;
	if self.time % 5 == 1 and self.isSavingJson then
		if self.coroutine then
			coroutine.resume(self.coroutine);
		end
	end
end

--=====================================================================

return TitleScene;
