local scheduler = cc.Director:getInstance():getScheduler();
local NetworkManager = Network.NetworkManager;
local NetworkCmdType = Network.NetworkCmdType;
local fonts = Enum:getTable("SystemFont");
local MessageBoxType = Enum:getTable('MessageBoxType')

local CHECK_STEP = {
	STEP_NULL 			= 0,

	STEP_SPLASH			= 1,	-- 闪屏
	STEP_AGE_LIMIT		= 2,	-- 年龄分级
	STEP_PERMISSION		= 3,	-- 权限
	
	STEP_CHECK_DOWN		= 4,	-- 检测更新

	STEP_UDW			= 5,	-- 根据包类型，连接UDW

}

local full_blank = function(str, length)
	if string.len(str) >= length then
		return str
	else
		local full_len = length - string.len(str)
		local str_blank = ''
		for i=1, full_len do
			str_blank = str_blank .. ' '
		end
		return str_blank .. str
	end
end

local VersionUpdateScene = class("VersionUpdateScene", function()
	local scene = nil;
	scene = display.newScene("VersionUpdateScene")

	return scene;
end)

-- step:
-- 1. play logo
-- 2. preload imgs
-- check version
function VersionUpdateScene:clearAll( ... )
	-- self.preloadFlag = true;
	self.checkVersionFlag = false;	
	self.updateError = false
	self.tipError = false;
	self.tipDelayTime = -1;
	self.showNewVersionMessageBox = 0
	self.showRetryMessageBox = 0
	self.percentList = {}
	self.curPercent = 0
	self.nextPercent = 0
	self.curDownload = 0
	self.nextDownload = 0
	self.maxDownload = 875
	self.is50 = false;
	self.showBox = false;
	self.stopUpdate = false;
	
	self.checkStep = CHECK_STEP.STEP_NULL
end
function VersionUpdateScene:ctor()
	--加载资源
	--设置ui根结点


	local function onNodeEvent(event)
		if event == "enter" then
			self:onEnter()
		elseif event == "exit" then
			self:onExit()
		end
	end
	self:registerScriptHandler(onNodeEvent)

	self:clearAll()
	self:startUpdate()

	local sp = cc.Sprite:create('image/commonImg/lbg.png');
	local texture = sp:getTexture();
	texture:setTexParameters(gl.LINEAR, gl.LINEAR, gl.REPEAT, gl.REPEAT)
	local test = cc.Sprite:createWithTexture(texture, cc.rect(0, 0, StageSize.width, StageSize.height))
	test:addTo(self)
	test:setPosition(StageSize.width/2, StageSize.height/2)
	test:setLocalZOrder(-1);

	local bg = cc.Sprite:create('image/loading/loading.png')
		:addTo(self)
		:setLocalZOrder(-1)
		:setPosition(StageSize.width/2, StageSize.height/2)

	self:initSplashNode()
	self:initAgeLimitNode()
	self:initPermissionNode()
	self:initPermissionExitNode()
	self:initDownLoadNode()

end

function VersionUpdateScene:initSplashNode( ... )
	-- logo
	self.splashNode = cc.Node:create()
		-- :setLocalZOrder(-10)
		:addTo(self)
		:setVisible(false)

	local bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 255),StageSize.width, StageSize.height)
	:setLocalZOrder(0)
	:addTo(self.splashNode)

	-- self.logo1Img = cc.Sprite:create('image/logo/krlogo1.png') 
	-- :addTo(self.splashNode)
	-- :setPosition(StageSize.width/2, StageSize.height/2)

	self.logo1Img = cc.Sprite:create('image/logo/NaNi-soft.png') 
	:addTo(self.splashNode)
	:setPosition(StageSize.width/2, StageSize.height/2)
end

function VersionUpdateScene:initAgeLimitNode( ... )
	-- logo
	self.ageLimitNode = cc.Node:create()
		:addTo(self)
		:setVisible(false)
	
	local bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 255),StageSize.width, StageSize.height)
		:setLocalZOrder(0)
		:addTo(self.ageLimitNode)

	local ageImg = cc.Sprite:create('image/title/age_limit1.png') 
		:addTo(self.ageLimitNode)
		:setPosition(StageSize.width/2, StageSize.height/2)
	local ageImg2 = cc.Sprite:create('image/title/age_limit2.png') 
		:addTo(self.ageLimitNode)
		:setPosition(StageSize.width - 230 , StageSize.height - 130 )
	local ageImg3 = cc.Sprite:create('image/title/age_limit3.png') 
		:addTo(self.ageLimitNode)
		:setPosition(StageSize.width - 90, StageSize.height - 130)
end

function VersionUpdateScene:initPermissionNode( ... )
	-- logo
	-- 权限检测时，如果拒绝，则直接退出游戏
	self.permissionNode = cc.Node:create()
		:addTo(self)
		:setVisible(false)
	
	local bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 255),StageSize.width, StageSize.height)
		:setLocalZOrder(0)
		:addTo(self.permissionNode)

	local bg = cc.Sprite:create('image/title/permission_grant.png') 
		:addTo(self.permissionNode)
		:setPosition(StageSize.width/2, StageSize.height/2+52)

	local permissionCallback = function ( strData )
		print("==============permission call back ==== "..tostring(strData))

		local jsonData = Cjson.decode(strData)
		if jsonData == nil  then
			-- do nothing 

		elseif jsonData.code == 0 then
			-- 同意权限
			self.permissionNode:setVisible(false)
			self.checkStep = CHECK_STEP.STEP_CHECK_DOWN
			self:checkStepUpdate() 
		else
			if  jsonData.show_info == false then
				-- 拒绝并不在提示，
				-- 显示 关闭游戏页面
				-- self:showPermissionExit();
				self.permissionExitNode:setVisible(true)
			else
				-- 拒绝
				-- do nothing 
			end
		end



	end
	local permissionBtnClick = function (  )
		sdkManager.SDKLuaHelper:showPermission(permissionCallback)
	end

	local btn = ccui.Button:create('image/commonImg/common_button_63.png')
		:setPosition(StageSize.width/2, StageSize.height/2 - 439)
		:setTitleText(LANG.COMMON_OK)
		:setTitleFontSize(24)
		:setTitleFontName(Enum:getTable("SystemFont").FONT_JP)
		:addTo(self.permissionNode)
	btn:addClickEventListener(permissionBtnClick)

end

function VersionUpdateScene:initDownLoadNode( ... )
	-- download progress
	self.downloadNode = cc.Node:create()
		:setPosition(StageSize.width/2, 123)
		:setLocalZOrder(10)
		:setVisible(false)
		:addTo(self)

	self.progressBg = cc.Sprite:create('image/loading/loading_progress_bg_1.png')
		:addTo(self.downloadNode)

	self.progressBg2 = cc.Sprite:create('image/loading/loading_progress_bg_2.png')
		:addTo(self.downloadNode)
		:setPosition(0,-49)

	self.progressBar = cc.Scale9Sprite:create('image/loading/loading_progress.png');
	self.progressBar:setCapInsets(CCRectMake(8,0,6,17));
	self.progressBar:addTo(self.downloadNode);
	self.progressBar:setAnchorPoint(0 , 0.5);
	self.progressBar:setPosition(-271 , -2.5);

	--self.progressBar = cc.ProgressTimer:create(cc.Sprite:create('image/loading/progress_fg.png'));
    --self.progressBar:setType(cc.PROGRESS_TIMER_TYPE_BAR);
    --self.progressBar:setMidpoint(cc.p(0, 0));
    --self.progressBar:setBarChangeRate(cc.p(1, 0));
	--self.progressBar:addTo(self.downloadNode);

	--self.light = cc.Sprite:create('image/loading/light.png')
	--	:setPosition(-221, 0)
	--	:addTo(self.downloadNode)
	self.barWidth = 541;

	self.progressLabel = cc.Label:createWithTTF("0",fonts.FONT_SIYUAN, 18)
		:enableOutline(cc.c3b(0,105,180))
		:addTo(self.downloadNode)

	self.loadingLabel = cc.Label:createWithTTF('Downloading    ', fonts.FONT_SIYUAN, 20)
		:setPosition(cc.p(-269, 27))
		:setAnchorPoint(cc.p(0, 0.5))
		:addTo(self.downloadNode)

	self.loadingLabel:runAction(cc.RepeatForever:create(cc.Sequence:create(
			cc.DelayTime:create(0.2),
			cc.CallFunc:create(function() self.loadingLabel:setString('Downloading .  ') end),
			cc.DelayTime:create(0.2),
			cc.CallFunc:create(function() self.loadingLabel:setString('Downloading .. ') end),
			cc.DelayTime:create(0.2),
			cc.CallFunc:create(function() self.loadingLabel:setString('Downloading ...') end),
			cc.DelayTime:create(0.2),
			cc.CallFunc:create(function() self.loadingLabel:setString('Downloading    ') end)
		)))
	
	self.word = cc.Label:createWithTTF(LANG.VERSION_UPDATE_8, fonts.FONT_SIYUAN, 16)
		:setPosition(0, -49)
		:setTextColor(cc.c3b(255 , 200, 0))
		:addTo(self.downloadNode)

end

function VersionUpdateScene:initPermissionExitNode( ... )
	self.permissionExitNode = cc.Node:create()
		:addTo(self)
		:setVisible(false)

	local bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 255),StageSize.width, StageSize.height)
		:setLocalZOrder(0)
		:addTo(self.permissionExitNode)

	local bg = cc.Sprite:create('image/title/permission_exit.png') 
		:addTo(self.permissionExitNode)
		:setPosition(StageSize.width/2, StageSize.height/2)


	local permissionBtnClick = function (  )
		cc.Director:getInstance():endToLua()
	end

	local btn = ccui.Button:create('image/commonImg/common_button_63.png')
		:setPosition(StageSize.width/2, StageSize.height/2 - 380)
		:setTitleText(LANG.COMMON_EXIT_GAME)
		:setTitleFontSize(24)
		:setTitleFontName(Enum:getTable("SystemFont").FONT_JP)
		:addTo(self.permissionExitNode)
	btn:addClickEventListener(permissionBtnClick)

end
function VersionUpdateScene:showPermissionExit(  )
	-- body
end
function VersionUpdateScene:onEnter()
	-- flag;
	self:clearAll()
	
	--设置界面对应事件回调
	NetworkManager:Init();
	--FrameEffectManager:preLoadPngs(handler(self, self.setPreloadFlag));

	-- AudioManager:preloadMusic( 'V_BGM_logo' )
	-- AudioManager:SetState(Enum:getTable('GameState').Logo);
	-- AudioManager:playMusic('V_BGM_logo', 0);

	-- message box
	MessageBox:init(self, self)
	
	-- UI开始展示闪屏
	self.checkStep = CHECK_STEP.STEP_SPLASH
	self:checkStepUpdate()
end

function VersionUpdateScene:onExit()
	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
	GlobalFunctions:recordTime('fb-log---VerSceneExit')
end

function VersionUpdateScene:getNextStepIsAgeLimit()
	if GlobalFunctions:isKorean() then
		return CHECK_STEP.STEP_AGE_LIMIT
	else
		return CHECK_STEP.STEP_CHECK_DOWN
	end
end

function VersionUpdateScene:checkStepUpdate(  )

	if self.checkStep == CHECK_STEP.STEP_SPLASH then
		-- 检测闪屏
		-- print("==============show STEP_SPLASH ==========")
		self.logoCount = 0;
		local actionCallFunc = cc.CallFunc:create( function (  )
			if self.logoCount >= 8 then
				self.splashNode:stopAllActions();
				self.splashNode:setVisible(false)
				if global_branch.package_enum == 0 then
					self.checkStep = self:getNextStepIsAgeLimit()
				else
					self.checkStep = CHECK_STEP.STEP_UDW
				end
				self:checkStepUpdate() 
				return 
			end
			-- local img = 'image/logo/krlogo2.png'
			-- if self.logoCount %2 == 0 then
			-- 	img = 'image/logo/krlogo1.png'
			-- end
			-- self.logo1Img:setTexture(img)
			self.logoCount = self.logoCount +1;
		end )
		self.splashNode:setVisible(true)
		self.splashNode:runAction( cc.Repeat:create(cc.Sequence:create(cc.DelayTime:create(0.5),actionCallFunc) ,10) )

	elseif self.checkStep == CHECK_STEP.STEP_UDW then
		local url = define.udw_url[global_branch.package_enum];
		if not url then
			self.checkStep = self:getNextStepIsAgeLimit()
			self:checkStepUpdate()
			return

		else
			local ssn = define.ssn
			local bundleid, market;
			local package_name = sdkManager.SDKLuaHelper:getPackageName()
			if package_name == 'com.nani.ego.kr.aos' then
				bundleid = package_name
				market = 12

			elseif package_name == 'com.nani.ego.kr.one' then
				bundleid = package_name
				market = 13

			else
				self.checkStep = self:getNextStepIsAgeLimit()
				self:checkStepUpdate()
				return
			end

			local newVersionFile = define.writablePath .. "/new_version/project.manifest";
			local fileUtils = cc.FileUtils:getInstance();
			local file;
			if fileUtils:isFileExist(newVersionFile) then
			local fileData = fileUtils:getStringFromFile(newVersionFile);
				file = json.decode(fileData);
			else
				local fileData = fileUtils:getStringFromFile('version/project.manifest');
				file = json.decode(fileData);
			end
			local version = tostring(file['version']);

			local getShowMsg = function(msgType, msg, globalMsg, lan)
				if globalMsg and globalMsg[msgType] and globalMsg[msgType].messages and globalMsg[msgType].messages[lan] then
					return globalMsg[msgType].messages[lan];
				else
					return msg;
				end
			end

			local xhr = cc.XMLHttpRequest:new()
			xhr.responseType = cc.XMLHTTPREQUEST_RESPONSE_STRING
			local udw_url = url..'?ssn='..ssn..'&bundleid='..bundleid..'&market='..market..'&version='..version
			xhr:open("GET", udw_url)
			xhr:registerScriptHandler(function()
				if  xhr.readyState == 4 and xhr.status == 200 then
					local r = Cjson.decode(xhr.response)
					local code = tonumber(r.Code)
					local msg = r.Msg
					local url = r.Url
					local ext = r.ext
					local data = r.data
					local glMsg = r.globalMsg
					local lan = cc.UserDefault:getInstance():getStringForKey('lg_set', 'default')
					if lan == 'default' then
						lan = device.language
					end
					if lan == 'kr' then
						lan = 'ko'
					elseif lan == 'jp' then
						lan = 'ja'
					elseif lan == 'sp' then
						lan = 'es'
					elseif lan == 'cn' then
						lan = 'zh-tw'
					elseif lan == 'tw' then
						lan = 'zh-tw'
					end
					if code == 0 then
						-- 正常
						define.server_url = url
						self.checkStep = self:getNextStepIsAgeLimit()
						self:checkStepUpdate()

					elseif code == 1 then
						-- 维护
						local time = '';
						if glMsg and glMsg.maintenance and glMsg.maintenance.utcStartTime and glMsg.maintenance.utcEndTime then
							local c = os.date('!*t',os.time())--中时区的时间
							local d = os.date('*t',os.time())
							local timeZone= (d.hour - c.hour) * 3600 + (d.min - c.min) * 60
							timeZone = timeZone/3600
							local a, b
							local st = {}
							local et = {}
							a, b, st.year, st.month, st.day, st.hour, st.min = string.find(glMsg.maintenance.utcStartTime, "(%d+)-(%d+)-(%d+)%s*(%d+):(%d+)");
							st.hour = st.hour + timeZone
							local st_c = os.date("%H:%M", os.time(st))
							a, b, et.year, et.month, et.day, et.hour, et.min = string.find(glMsg.maintenance.utcEndTime, "(%d+)-(%d+)-(%d+)%s*(%d+):(%d+)");
							et.hour = et.hour + timeZone
							local et_c = os.date("%H:%M", os.time(et))
							time = string.format('(%s~%s)', tostring(st_c), tostring(et_c))
						end
						local showMsg = getShowMsg('maintenance', msg, glMsg, lan)
						showMsg = showMsg .. time
						MessageBox:create(showMsg , MessageBoxType.OK, function()
							self:cancelUpdate();

						end, function()
						end, function()
						end);
						
					elseif code == 2 then
						-- 大版本更新
						local showMsg = getShowMsg('marketUpdate', msg, glMsg, lan)
						MessageBox:create(showMsg , MessageBoxType.OK, function()
							local url = ''
							if package_name == 'com.nani.ego.kr.aos' then
								url = 'https://play.google.com/store/apps/details?id=com.nani.ego.kr.aos'
							elseif package_name == 'com.nani.ego.kr.one' then
								url = 'https://onesto.re/0000751708'
							end
							if url ~= '' then
								cc.Application:getInstance():openURL(url)
							end
							self:cancelUpdate();

						end, function()
						end, function()
						end);
						
					elseif code == -2 then
						-- 错误
						print('UDW ERROR')
						print(xhr.response)

					else
						-- 错误
						print('UDW ERROR')
						print(xhr.response)
					end
				else
					print('UDW HTTP ERROR')
					print(xhr.readyState)
				end
			end)
			xhr:send()
		end
		
	elseif self.checkStep == CHECK_STEP.STEP_AGE_LIMIT then
		-- 检测年龄分级
		-- print("==============show age ==========")
		local actionCallFunc = cc.CallFunc:create( function (  )
			self.ageLimitNode:stopAllActions();
			self.ageLimitNode:setVisible(false)

			self.checkStep = CHECK_STEP.STEP_PERMISSION
			self:checkStepUpdate() 
		end )
		self.ageLimitNode:setVisible(true)
		self.ageLimitNode:runAction(cc.Sequence:create(cc.DelayTime:create(3.0),actionCallFunc))

	elseif self.checkStep == CHECK_STEP.STEP_PERMISSION then
		-- 检测权限
		-- print("==============show STEP_PERMISSION ==========")

		local  ret = sdkManager.SDKLuaHelper:checkPermission()
		print('ret ================================  ',ret);
		if ret == 0 or ret == 2 then
			self.checkStep = CHECK_STEP.STEP_CHECK_DOWN
			self:checkStepUpdate() 
		else
			GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').perm_on);
			self.permissionNode:setVisible(true)
		end

	elseif self.checkStep == CHECK_STEP.STEP_CHECK_DOWN then
		-- 检测热更新
		-- print("==============show STEP_CHECK_DOWN ==========")
		self.newVersionFlag = false;
		self.newVersionForceUpdate = false;
		self:checkVersionUpdate();
		self.updating = false;
	
	else
		print("==============checkStepUpdate error ==========")
	end

end

function VersionUpdateScene:startUpdate( ... )
	if self.m_scheduleHandler == nil then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc( handler(self, self.update), 0, false);
	end
end

-- function VersionUpdateScene:setPreloadFlag()
-- 	self.preloadFlag = true;
-- end

-- function VersionUpdateScene:playLogOver()
-- 	self.splashNode:stopAllActions()
-- 	self.splashNode:setVisible(false)
-- end

function VersionUpdateScene:update(dt)
	if not self.updating and self.showRetryMessageBox > 0 and self.updateError and not self.showBox and self.tipError==false then

		if self.showRetryMessageBox > 3 then
			MessageBox:create(LANG.VERSION_UPDATE_2 , MessageBoxType.CANCEL_OK, function()
				-- self:checkVersionUpdate()
				self.showBox = false;
				self.showRetryMessageBox = 1;
				self:downloadFailedAssets()
			end, function()
				self:cancelUpdate()
			end, function()
			end);
			self.showBox = true;
		else
			-- self:checkVersionUpdate()
			self:downloadFailedAssets()
		end

	elseif  self.newVersionFlag and self.showNewVersionMessageBox == 1 then 
		self.showNewVersionMessageBox = self.showNewVersionMessageBox + 1
		local str = string.format(LANG.VERSION_UPDATE_4, tostring(self.maxDownload));
		MessageBox:create(str , MessageBoxType.CANCEL_OK, function()

			GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').re_start);
			self.downloadNode:setVisible(true)
			self.assetsManagerEx:update()
		end, function()
			self:cancelUpdate()
		end, function()
		end);

	elseif  self.newVersionForceUpdate and self.showNewVersionMessageBox == 1 then 
		self.showNewVersionMessageBox = self.showNewVersionMessageBox + 1
		MessageBox:create(LANG.UI_L_VERSION , MessageBoxType.OK, function()
			local package_name = sdkManager.SDKLuaHelper:getPackageName()
			local url = ''
			if package_name == 'com.nani.ego.kr.aos' then
				url = 'https://play.google.com/store/apps/details?id=com.nani.ego.kr.aos'
			elseif package_name == 'com.nani.ego.kr.one' then
				url = 'https://onesto.re/0000751708'
			end
			if url ~= '' then
				cc.Application:getInstance():openURL(url)
			end
			self:cancelUpdate();

		end, function()
		end, function()
		end);

	elseif  self.checkVersionFlag then -- self.preloadFlag 
		if self.downloadNode:isVisible() then
			if self.curPercent>=100 then
				if self.m_scheduleHandler then
					scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
					self.m_scheduleHandler = nil
				end
				App:run("TitleScene");
				GlobalFunctions:recordTime('fb-log---runTitleSceneend')
			else
				self:updatePlayPercent()
			end
		else 
			if self.m_scheduleHandler then
				scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
				self.m_scheduleHandler = nil
			end
			App:run("TitleScene");
			GlobalFunctions:recordTime('fb-log---runTitleSceneend')
		end
	else 
		if self.downloadNode:isVisible() then
			if self.fileCount == 0 then
				local genLan = function (index)
					if index % 4 == 0 then
						return LANG.VERSION_UPDATE_8 .. ' '
					elseif index % 4 == 1 then
						return LANG.VERSION_UPDATE_8 .. ' .'
					elseif index % 4 == 2 then
						return LANG.VERSION_UPDATE_8 .. ' ..'
					elseif index % 4 == 3 then
						return LANG.VERSION_UPDATE_8 .. ' ...'
					end
				end
				if not self.beginTime then
					self.beginTime = dt
					self.beginIndex = 0
					self.word:setString(genLan(self.beginIndex))
					self:updatePorgressBar(self.beginIndex)
				else
					self.beginTime = self.beginTime + dt
				end
				if self.beginTime >= 1 then
					self.beginTime = self.beginTime - 1
					self.beginIndex = self.beginIndex + 1
					self.word:setString(genLan(self.beginIndex))
					self:updatePorgressBar(self.beginIndex*9)
				end
			else
				self:updatePlayPercent()
			end
		end
		
	end

	if self.tipDelayTime >= 0 then
		self.tipDelayTime = self.tipDelayTime + dt;
		if self.tipDelayTime >= 5 then
			self.tipError = false;
			self.tipDelayTime = -1;
		end
	end
end

function VersionUpdateScene:downloadFailedAssets()
	if self.assetsManagerEx then
		self.updating = true;
		self.updateError = false;
		self.assetsManagerEx:downloadFailedAssets();
	end
end

function VersionUpdateScene:checkVersionUpdate()
	-- 获取设备可写路径

	local manifestPath = "src/version/project.manifest";

	local storagePath = define.writablePath .. "/new_version/";

	-- hotfix manager 
	self.assetsManagerEx = cc.AssetsManagerEx:create(manifestPath, storagePath);
	self.assetsManagerEx:retain()

	self.updating = true;
	if not self.assetsManagerEx:getLocalManifest():isLoaded() then
		self.checkVersionFlag = true;
    else
		local handlerAssetsManagerEx = function(event) 
			local eventCode = event:getEventCode();
			print("=======event code:", eventCode)

			if (cc.EventAssetsManagerEx.EventCode.ERROR_NO_LOCAL_MANIFEST == eventCode) then
				-- 0
				MessageBox:create(LANG.VERSION_UPDATE_13 , MessageBoxType.OK);
				
			elseif (cc.EventAssetsManagerEx.EventCode.ERROR_DOWNLOAD_MANIFEST == eventCode) then
				-- 1
				print('error download');
				MessageBox:create(LANG.VERSION_UPDATE_14 , MessageBoxType.OK, function()
						self:cancelUpdate()
					end);

			elseif (cc.EventAssetsManagerEx.EventCode.ERROR_PARSE_MANIFEST == eventCode) then
				-- 2
				print('error poarse manifest');
				MessageBox:create(LANG.VERSION_UPDATE_15 , MessageBoxType.OK, function()
						self:cancelUpdate()
					end);

			elseif (cc.EventAssetsManagerEx.EventCode.NEW_VERSION_FOUND == eventCode) then
				-- 3
				--
				print("Asset ", event:getAssetId(), ", ", event:getMessage())
				local size = event:getAssetId()
				if tonumber(size) > 0 then
					local trueSize = tostring(tonumber(size)/1024/1024)
					self.maxDownload = trueSize - trueSize % 0.01
				end
				print('new version', os.time())
				GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').re_on);
				if self.updateError then
					print('retry!!!!!!')
					self.updateError = false
					self.assetsManagerEx:update()
				else
					print('new version flag')
					self.showNewVersionMessageBox = self.showNewVersionMessageBox + 1
					self.newVersionFlag = true;
					--[[
					local args = {
						eventName = Enum:getTable('TrackEventToken').TRACK_RE_START, -- 
					}
					sdkManager.SDKLuaHelper:trackEvent(args)
					]]
					GlobalFunctions:trackEvent('TRACK_RE_START')
				end
			elseif (cc.EventAssetsManagerEx.EventCode.UPDATE_FOREC == eventCode) then
				print('new version and must force update!')
				self.showNewVersionMessageBox = self.showNewVersionMessageBox + 1
				self.newVersionForceUpdate = true;

			elseif (cc.EventAssetsManagerEx.EventCode.ALREADY_UP_TO_DATE == eventCode) then
				-- 4
				print('no need to update');
				self.checkVersionFlag = true;
				self.updating = false;

			elseif (cc.EventAssetsManagerEx.EventCode.UPDATE_PROGRESSION == eventCode) then
				-- 5
				print('update progression', os.time())
				local assetId = event:getAssetId();
				local percent = event:getPercent();
				local strInfo = "";

				if assetId == cc.AssetsManagerExStatic.VERSION_ID then
					strInfo = string.format("Version file: %d%%", percent);
				elseif assetId == cc.AssetsManagerExStatic.MANIFEST_ID then
					strInfo = string.format("Manifest file: %d%%", percent);
				else
					local msg = event:getMessage()
					if string.len(msg) > 0 then
						local b, e = string.find(msg, "%d+ ")
						local num = tonumber(string.sub(msg, b, e))
						if not self.maxCount then
							self.maxCount = num
							self.fileCount = 0
						end
						if assetId == 'RESUME' then
							self.word:setString(string.format(LANG.VERSION_UPDATE_5, tostring(num)));
						elseif assetId == 'START' then
							self.word:setString(string.format(LANG.VERSION_UPDATE_6, tostring(num)));
						else
							--self.word:setString(msg)
						end
					else
						-- local percent = math.floor(tonumber(self.fileCount / self.maxCount * 100))
						--local percent = math.floor(tonumber(event:getPercentByFile()))
						-- self.progressLabel:setString(tostring(tonumber(percent) .. '%'))
						-- self.progressBar:setPercentage(percent)
						-- self.light:setPosition(-self.barWidth/2 + percent/100*self.barWidth, 0)
						print('add percent ' .. percent .. ' to list')

						table.insert(self.percentList, 1, percent)
					end
					print(event:getPercentByFile(), event:getPercent())
				end
				print(strInfo);
			elseif (cc.EventAssetsManagerEx.EventCode.ASSET_UPDATED == eventCode) then
				-- 6
				print('asset update', os.time())
				if self.fileCount == 0 then
					self:updatePorgressBar(100)
					--local str = '0MB/'..self.maxDownload..'MB'
					--str = full_blank(str, 20)
					--self.word:setString(str)
				end
				self.fileCount = self.fileCount + 1
				--self.word:setString(tostring(event:getAssetId()) .. '    ' .. self.fileCount .. '/' .. self.maxCount)

			elseif (cc.EventAssetsManagerEx.EventCode.ERROR_UPDATING == eventCode) then
				-- 7
				print("Asset ", event:getAssetId(), ", ", event:getMessage(), ',', event:getCURLECode(), ', ', event:getCURLMCode())
				local curlMcode = event:getCURLMCode();
				local errorMsg = event:getMessage();
				local bTimeout = false;
				if errorMsg and string.find(errorMsg, "Timeout", 1) ~= nil then
					bTimeout = true;
				end

				if self.tipError == false and bTimeout==false then
					self.tipError = true
					local tipMsg = ''..event:getAssetId();
					-- if curlMcode == 0 then
					-- 	tipMsg = tipMsg..' '..LANG.VERSION_UPDATE_16 --下载超时
					-- elseif curlMcode == 7 then
					-- 	tipMsg = tipMsg..' '..LANG.VERSION_UPDATE_17 --服务器连接失败
					-- else
					-- 	tipMsg = tipMsg..' '..event:getMessage()..'  mc:'..curlMcode..'  ec:'..event:getCURLECode()
					-- end
					tipMsg = tipMsg..' '..event:getMessage()..'  mc:'..curlMcode..'  ec:'..event:getCURLECode() .. ')'
					local is_release = sdkManager.SDKLuaHelper:retApkRelease();
					if is_release == 1 then
						tipMsg = '게임 접속 중 오류가 발생하였습니다. 게임을 재시작해주세요.재시작 후에도 문제가 지속된다면 해당 화면을 캡쳐하여 고객센터로 문의주세요. (EC : '..curlMcode..' )'
					end
					local msgBox = MessageBox:create(tipMsg , MessageBoxType.CANCEL_OK, function()
						self.tipDelayTime = 0;
					end, function()
						self:cancelUpdate();
					end, function()
						self.tipDelayTime = 0;
					end);
					msgBox:setOkBtnName(LANG.COMMON_GAME_CONT)
					msgBox:setCancelBtnName(LANG.COMMON_GAME_EXIT)

				end

			elseif (cc.EventAssetsManagerEx.EventCode.UPDATE_FINISHED == eventCode) then
				-- 8
				print('update finish');
				--self.checkVersionFlag = true;
				self:updateFinish();
				self.updating = false;

			elseif (cc.EventAssetsManagerEx.EventCode.UPDATE_FAILED == eventCode) then
				-- 9
				print('update failed');
				local msg = event:getMessage()
				print(msg)
				self.updateError = true;
				self.showRetryMessageBox = self.showRetryMessageBox + 1
				self.updating = false;

			elseif (cc.EventAssetsManagerEx.EventCode.ERROR_DECOMPRESS == eventCode) then
				-- 10
				self.word:setString(LANG.VERSION_UPDATE_7);

			end
		end

		local dispatcher = cc.Director:getInstance():getEventDispatcher();
		local eventListenerAssetsManagerEx = cc.EventListenerAssetsManagerEx:create(self.assetsManagerEx, handlerAssetsManagerEx);
		dispatcher:addEventListenerWithFixedPriority(eventListenerAssetsManagerEx, 1);

		self.assetsManagerEx:update();
	end
end

function VersionUpdateScene:playShowProgressAni()
	-- body
	if #self.percentList <= 0 then
		return;
	end

	self.nextPercent = table.remove( self.percentList )

end

function VersionUpdateScene:updatePlayPercent()
	-- body
	if self.stopUpdate then
		return
	end
	
	if self.curPercent >= self.nextPercent then
		self:playShowProgressAni()
		return;
	end
	self.curPercent = self.curPercent + 0.5;
	self.curDownload = (self.curPercent/100)*self.maxDownload--self.curDownload + self.maxDownload / 200;
	if self.curPercent >= 100 then
		self.curPercent = 100;
	end
	if self.curDownload >= self.maxDownload then
		self.curDownload = self.maxDownload;
	end

	if self.curPercent >= 50 and not self.is50 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').re_50);
		self.is50 = true
	end
	local str = self.curDownload .. 'MB/' .. self.maxDownload .. 'MB'
	str = full_blank(str, 20)
	self.word:setString(str)

	self:updatePorgressBar(self.curPercent)
end

function VersionUpdateScene:updatePorgressBar(percent100)
	if percent100 > 100 then
		percent100 = 100
	end
	if self.stopUpdate then
		return
	end
	local percent = percent100 / 100;
	local width = percent*self.barWidth;
	if width > 0 then
		width = width < 22 and 22 or width;
	end
	self.progressLabel:setString(tostring(tonumber(percent100) .. '%'))
	self.progressBar:setContentSize(cc.size(width , 17));
	print('bar percent:', percent100, os.time())
end

function VersionUpdateScene:updateFinish()
	--[[
	local args = {
		eventName = Enum:getTable('TrackEventToken').TRACK_RE_COM
	}
	sdkManager.SDKLuaHelper:trackEvent(args)
	]]
	GlobalFunctions:trackEvent('TRACK_RE_COM')
	GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').re_com);
	self:updatePorgressBar(100)
	local str = self.maxDownload .. 'MB/' .. self.maxDownload .. 'MB'
	str = full_blank(str, 20)
	self.word:setString(str)
	
	self.stopUpdate = true;

	MessageBox:create(LANG.VERSION_UPDATE_3 , MessageBoxType.OK, function()
		self:reloadAllPerload()
	end, function()
		self:reloadAllPerload()
	end, function()
		self:reloadAllPerload()
	end);
end

function VersionUpdateScene:cancelUpdate()
	cc.Director:getInstance():endToLua();
end

function VersionUpdateScene:restart()
	cc.Director:getInstance():restart();
end

function VersionUpdateScene:reloadAllPerload()
	cc.FileUtils:getInstance():purgeCachedEntries()
	local lg = model.UserDefaultModel:getLanguageSet()

	cc.FileUtils:getInstance():addSearchPath('res_'..lg..'/', true)
	local storagePath = define.writablePath .. "/new_version/hotfix/";
	local fullPath = define.writablePath .. "/new_version/"
	cc.FileUtils:getInstance():addSearchPath(fullPath..'res/', true)
	cc.FileUtils:getInstance():addSearchPath(storagePath..'res/', true)
	--增加多语言资源搜索路径 
	cc.FileUtils:getInstance():addSearchPath(fullPath..'res_'..lg..'/',true)
	cc.FileUtils:getInstance():addSearchPath(storagePath..'res_'..lg..'/', true)
	cc.FileUtils:getInstance():addSearchPath(storagePath..'src/', true)

	package.loaded["app.AllRequire"] = nil  
	
	local unAbleDel = {VersionUpdateScene=1, MyApp=1, opensslToLua=1, SDKLuaHelper=1, BuglyManager=1, AsyncLoadManager=1, queue=1, configure=1}
	for k,v in pairs(package.loaded) do
		if string.find(k, "app.", 1) == 1 then
			local b = true
			for checkKey,value in pairs(unAbleDel) do
				if string.find(k, checkKey, 1) then
					b = false
					break;
				end
			end
			if b == true then
				package.loaded[k] = nil
			end
		elseif string.find(k, "language.", 1) == 1 then
			package.loaded[k] = nil
			-- v = nil
		else
			-- print('fubibin-p--',k)
		end
	end

	MessageBox = nil
	Network.NetworkManager = nil

    require("app.AllRequire")

    Network.NetworkManager:Init();
	MessageBox:init(self, self)

	App:run("TitleScene")
end

return VersionUpdateScene
