local FightEffectManager = class("FightEffectManager")
local scheduler = cc.Director:getInstance():getScheduler();

function FightEffectManager:ctor()
	self.globalEffectID = 0;
	self.effectList = {};
	self.effectActorList = {};
end

function FightEffectManager:Initialize()
end

-- 更新
function FightEffectManager:Update( dt )
	for _, v in pairs(self.effectList) do
		if not v:Update(dt) then
			self.effectList[_] = nil;
		end
	end
end

--重置
function FightEffectManager:Reset()
	--销毁动画
	for _, v in pairs(self.effectList) do
		v:Destroy();
	end

	for _, v in pairs(self.effectActorList) do
		for k, v1 in pairs(v) do
			v1:Destroy()
		end
	end

	self.effectList = {}
	self.effectActorList = {}
	
	self.globalEffectID = 0;
end

--========================================================================
--分配id
function FightEffectManager:allocEffectID()
	self.globalEffectID = self.globalEffectID + 1;
	return self.globalEffectID;
end

--创建特效动作
function FightEffectManager:createFrameEffectAction( effectID )
	local data = ResTable:getRowData(TableName.SkillEffect, effectID);
	local animation = FrameEffectManager:createFrameAnimation(data.name, data.num, data.frame);
	local action = cc.Animate:create(animation);
	return action;
end

--暂停全部特效
function FightEffectManager:PauseAllEffect()
	for _, effect in pairs(self.effectList) do
		effect:Pause();
	end
end

--恢复全部特效更新
function FightEffectManager:ResumeAllEffect()
	for _, effect in pairs(self.effectList) do
		effect:Resume();
	end
end

--暂停特定战斗特效
function FightEffectManager:PauseEffect(id)
	self.effectList[id]:Pause();
end

--恢复特定战斗特效
function FightEffectManager:ResumeEffect(id)
	self.effectList[id]:Resume();
end

function FightEffectManager:GetEffect(id)
	return self.effectList[id]
end

--========================================================================
--特效管理

--创建角色挂点特效
function FightEffectManager:CreateFighterAvatarEffect(actorID, skill, pos, restraintType)
	local scriptData = ResTable:getValue(TableName.SkillScript, skill:GetScriptID(), 'event_1')
	local ScriptEvent = Enum:getTable('ScritEvent')
	local effectID
	for _, v in pairs(scriptData) do
		if v[2] == ScriptEvent.AnimationTypeAction then
			effectID = v[4]
			break
		end
	end
	if effectID then
		if not self.effectActorList[actorID] then
			self.effectActorList[actorID] = {}
		end
		self.effectActorList[actorID][pos] = require("app.effect.AvatarActorEffect"):create(id, skill:GetResID())
		self.effectActorList[actorID][pos]:Initialize(actorID, pos, 1, false, effectID, restraintType)
	end
end

--创建挂点特效
function FightEffectManager:CreateAvatarPosEffect( actorID, avatarPos, pos, effectScale, layer, effectID, skillID, restraintType)
	if pos.data == 1 then
		if self.effectActorList[actorID] and self.effectActorList[actorID][pos.pos] then
			self.effectActorList[actorID][pos.pos]:run()
			return -1
		end
	elseif pos.data == 2 then
		local id = self:allocEffectID();
		local effect = require("app.effect.AvatarEffect"):create(id, skillID);
		effect:Initialize(actorID, avatarPos, pos, effectScale, layer, effectID, restraintType);

		self.effectList[id] = effect;
		return id;
	end
end

--创建火箭炮特效
function FightEffectManager:CreateMissileEffect( attackerID, skillID )
	local id = self:allocEffectID();
	local action = self:createFrameEffectAction('s_atk4_B');
	local effect = require("app.effect.MissileEffect"):create(id, skillID);
	effect:Initialize(attackerID, action);

	self.effectList[id] = effect;
	return id;
end

--创建火箭炮攻击场地特效
function FightEffectManager:CreateMissileHitEffect( attackerID, effectName, skillID )
	local id = self:allocEffectID();
	local action = self:createFrameEffectAction(effectName);
	local effect = require("app.effect.MissileHitEffect"):create(id, skillID);
	if not effect:Initialize(attackerID, action) then
		return;
	end

	self.effectList[id] = effect;
	return id;
end

--创造场景特效
function FightEffectManager:CreateSceneEffect(targetID, posType, effectScale, effectID, skillID)
	local id = self:allocEffectID();
	local effect = require("app.effect.SceneEffect"):create(id, skillID);
	effect:Initialize(targetID, posType, effectScale, effectID);

	self.effectList[id] = effect;
	return id;
end

--创造追踪特效
function FightEffectManager:CreateTraceEffect(attackerID, startOffsetPos, flightPathType, traceTime, effectScale, targetID, effectType, effectID, skillID)
	local id = self:allocEffectID();
	local action;
	if effectType == 1 then
		action = 'image/fight/scene/bullet' .. effectID .. '.png';
	elseif effectType == 2 then
		action = self:createFrameEffectAction(effectID);
	end
	local effect = require("app.effect.TraceEffect"):create(id, skillID);
	effect:Initialize(attackerID, targetID, effectType, action, startOffsetPos, flightPathType, traceTime, effectScale);

	self.effectList[id] = effect;
	return id;
end

--销毁特效
function FightEffectManager:DestroyEffect( effectID )
	local effect = self.effectList[effectID];
	if effect then
		effect:Destroy();
		self.effectList[effectID] = nil;
	end	
end

function FightEffectManager:checkEffectRunning( skill )
	local findEffect = false;
	for _, effect in pairs(self.effectList) do
		if effect:GetSkillID() == skill:GetResID() then
			findEffect = true;
			break;
		end
	end
	return findEffect;
end

cc.exports.FightEffectManager = FightEffectManager:create();
