local TraceEffect = class("TraceEffect", require("app.effect.Effect"))

function TraceEffect:ctor(id, skillResid)
	TraceEffect.super.ctor(self, id, skillResid)
end

--追踪者id
function TraceEffect:Initialize( attackerID, targetID, effectType, action, startOffsetPos, flightPathType, traceTime, effectScale )
	self.attacker = FightManager:GetFighter(attackerID);
	self.target = FightManager:GetFighter(targetID);
	if type(targetID) == 'table' then
		for _, v in pairs(targetID) do
			self.target = FightManager:GetFighter(v);
			break;
		end
	end

	if not self.attacker or not self.target then
		return;
	end

	self.effectType = effectType;
	if effectType == 1 then
		self.effectSprite = cc.Sprite:create(action);
	elseif effectType == 2 then
		self.effectSprite = cc.Sprite:create();
		self.effectSprite:runAction(cc.RepeatForever:create(action));
	else
		return;
	end
	self.effectSprite:setScale(effectScale);
	self.effectSprite:setZOrder(self.attacker:GetLocalZOrder());

	local startX, startY = self.attacker:GetPosition();
	startX = startX+startOffsetPos.x;
	startY = startY+startOffsetPos.y;
	if self.attacker:IsEnemy() then
		self.effectSprite:setFlipX(true);
		startX = startX - startOffsetPos.x*2;
	end
	self.startPosition = {x=startX, y=startY};

	local endX, endY = self.target:GetPosition();
	
	self.speedX, self.speedY = 0, 0;
	if flightPathType == 1 then
		self.speedX = (endX - startX) / (traceTime/1000);
		self.speedY = (endY - startY) / (traceTime/1000);
	else
		return;
	end

	EventManager:FireEvent(EventType.FightAddEffect, self.effectSprite, self);

	self.flightPathType = flightPathType;					-- 飞行类型

	self.lastTime = 0;										--特效当前飞行时间

	self:Update(0);
end

--销毁
function TraceEffect:Destroy()
	--卸载特效
	EventManager:FireEvent(EventType.FightDelEffect, self);
	TraceEffect.super.Destroy(self);
end

--更新
function TraceEffect:Update( dt )
	if not self.runningState then
		return true;
	end
	local running = true;

	self.lastTime = self.lastTime + dt;

	--目标位置
	local nextPosition = nil;
	local angle = nil;

	nextPosition, angle = self:CaculateCurHorizontalPosition();
	--[[
	if self.flightPathType == FlightPathType.HorizontalTrajectory then
		--水平轨迹
		nextPosition, angle = self:CaculateCurHorizontalPosition();
	elseif self.flightPathType == FlightPathType.Parabola then
		--抛物线轨迹
		nextPosition, angle = self:CaculateCurParabolaPosition(targetPos);

	elseif self.flightPathType == FlightPathType.StraightLine then
		--直线
		nextPosition, angle = self:CaculateCurStraightPosition(targetPos);
	end
	--]]

	local endX, endY = self.target:GetPosition();
	if self:IsEnd(nextPosition, {x=endX, y=endY}) then
		self:Destroy();
		running = false;
	else
		--设置特效位置
		self.effectSprite:setPosition(nextPosition.x, nextPosition.y);
	end
	return running;
end

--计算抛物线当前位置
function TraceEffect:CaculateCurParabolaPosition(endPosition)
	local totalTime = CheckDiv(math.abs(endPosition.x - self.startPosition.x) / self.xSpeed);			--飞行总时间
	local yOffset = endPosition.y - self.startPosition.y;									--y方向偏移
	local yAddSpeed = CheckDiv(2 * (yOffset - self.ySpeed * totalTime) / (totalTime * totalTime));	--y方向加速度
	local ty = (self.ySpeed * self.lastTime + 0.5 * yAddSpeed * self.lastTime * self.lastTime) + self.startPosition.y;
	local tx = self.startPosition.x + self.xSpeed * self.lastTime * self.dir;
	local ySpeed = self.ySpeed + self.lastTime * yAddSpeed;

	return {x = tx, y = ty, z = self.startPosition.z}, Math.ATan(CheckDiv(-ySpeed / self.xSpeed));
end

--计算水平当前位置
function TraceEffect:CaculateCurHorizontalPosition()
	local tx = self.startPosition.x + self.speedX * self.lastTime;
	local ty = self.startPosition.y + self.speedY * self.lastTime;
	return {x = tx, y = ty}, 0;
end

--计算直线当前位置
function TraceEffect:CaculateCurStraightPosition(endPosition)
    local totalTime = CheckDiv(math.abs(endPosition.x - self.startPosition.x) / self.xSpeed);			--飞行总时间
    if totalTime == 0 then
        return self.startPosition, 0;
    end

    local yOffset = endPosition.y - self.startPosition.y;									--y方向偏移
    local ySpeed = CheckDiv(yOffset / totalTime);
    local ty = self.startPosition.y + ySpeed * self.lastTime;
    local tx = self.startPosition.x + self.xSpeed * self.lastTime * self.dir;

    return {x = tx, y = ty, z = self.startPosition.z}, Math.ATan(CheckDiv(-ySpeed / self.xSpeed));
end

--飞行是否结束
function TraceEffect:IsEnd(curPos, targetPos)
	if not self.attacker:IsEnemy() then
		if curPos.x >= targetPos.x then
			return true;
		end
	else
		if curPos.x <= targetPos.x then
			return true;
		end
	end
	if self.lastTime > 2.5 then
		return true;
	end
	return false;
end

return TraceEffect;
