local UIEffectManager = class("UIEffectManager")
local scheduler = cc.Director:getInstance():getScheduler();

function UIEffectManager:ctor()
	self.globalEffectID = 0;
	self.effectList = {};
end

function UIEffectManager:start()
	if not self.m_scheduleHandler then 
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.Update), 0, false);
	end
end

-- 更新
function UIEffectManager:Update( dt )
	for _, v in pairs(self.effectList) do
		if not v:Update(dt) then
			self.effectList[_] = nil;
		end
	end
end

--重置
function UIEffectManager:reset()
	--销毁动画
	for _, v in pairs(self.effectList) do
		v:SetDestroy();
	end
	
	self.globalEffectID = 0;
end

--========================================================================
--分配id
function UIEffectManager:allocEffectID()
	self.globalEffectID = self.globalEffectID + 1;
	return self.globalEffectID;
end

--创建特效数据
function UIEffectManager:createEffectData( effectName )
	--TODO not dispose!!!
	if true then
		return nil;
	end
	local effectData = DragonBonesManager:GetFightEffect( effectName )
	return effectData;	
end

--暂停全部特效
function UIEffectManager:PauseAllEffect()
	for _, effect in pairs(self.effectList) do
		effect:Pause();
	end
end

--恢复全部特效更新
function UIEffectManager:ResumeAllEffect()
	for _, effect in pairs(self.effectList) do
		effect:Resume();
	end
end

--暂停特定战斗特效
function UIEffectManager:PauseEffect(id)
	self.effectList[id]:Pause();
end

--恢复特定战斗特效
function UIEffectManager:ResumeEffect(id)
	self.effectList[id]:Resume();
end

--========================================================================
--特效管理

--[[
--创建挂点特效
function UIEffectManager:CreateAvatarPosEffect( actorID, avatarPos, pos, effectScale, layer, effectName, skillID )
	local id = self:allocEffectID();
	local effectData = self:createEffectData(effectName);
	local effect = require("app.effect.AvatarEffect"):create(id, skillID);
	effect:SetEffectData(effectData);
	effect:Initialize(actorID, avatarPos, pos, effectScale, layer)

	self.effectList[id] = effect;
	return id;
end

--创建追踪特效
function UIEffectManager:CreateTraceEffect( attackerID, startOffsetPos, flightPathType, flyXSpeed, flyYInitSpeed, effectScale, targetID, targetAvatarPos, targetOffsetPos, effectName, skillScript, callback, skillID )
	
	local id = self:allocEffectID();
	local effectData = self:createEffectData(effectName);
	local effect = require("app.effect.TraceEffect"):create(id, skillID);
	effect:SetEffectData(effectData);
	effect:Initialize(attackerID, startOffsetPos, flightPathType, flyXSpeed, flyYInitSpeed, effectScale, targetID, targetAvatarPos, targetOffsetPos, skillScript, callback);

	self.effectList[id] = effect;
	return id;
end

--创造场景特效
function UIEffectManager:CreateSceneEffect(attackerID, posType, posOffset, effectScale, effectName, skillID)
	local id = self:allocEffectID();
	local effectData = self:createEffectData(effectName);
	local effect = require("app.effect.SceneEffect"):create(id, skillID);
	effect:SetEffectData(effectData);
	effect:Initialize(attackerID, posType, posOffset, effectScale);

	self.effectList[id] = effect;
	return id;
end

--销毁特效
function UIEffectManager:DestroyEffect( effectID )
	local effect = self.effectList[effectID];
	if effect then
		effect:Destroy();
		self.effectList[effectID] = nil;
	end	
end

function UIEffectManager:checkEffectRunning( skill )
	local findEffect = false;
	for _, effect in pairs(self.effectList) do
		if effect:GetSkillID() == skill:GetResID() then
			findEffect = true;
			break;
		end
	end
	return findEffect;
end
--]]
cc.exports.UIEffectManager = UIEffectManager:create();
