local SkillScript = class("SkillScript")
local SkillType = Enum:getTable('SkillType');
local ScriptEvent = Enum:getTable('ScritEvent');

function SkillScript:ctor(id, nameID, t)
	self.m_id = id;
	self.m_nameID = nameID;

	self.m_isRun = false;
	self.m_time = 0;
	self.m_time2 = nil;
	self.m_eventList = {};

	self.forceEnd = false;
	self.argsList = {};
	self.m_haveSecEff = false;
	self.m_secEffTagList = {};
	self.m_onlyOnce = false;

	self:initEventList(t);
end

function SkillScript:initEventList(t)
	local scriptData1 = ResTable:getValue(TableName.SkillScript, self.m_nameID, 'event_1');
	local scriptData2 = ResTable:getValue(TableName.SkillScript, self.m_nameID, 'event_2');

	self.m_eventList = {};
	local maxTime = 0

	for _, v in pairs(scriptData1) do
		local event = {};
		event.time = v[1]/1000;
		event.args = {};
		event.timeLine = 1
		if maxTime < event.time then
			maxTime = event.time
		end

		for i=2, #v do
			event.args[i-1] = v[i];
		end

		table.insert(self.m_eventList, event);
	end

	for _, v in pairs(scriptData2) do
		local event = {};
		event.time = v[1]/1000;
		event.args = {};
		event.timeLine = 2
		if maxTime < event.time then
			maxTime = event.time
		end

		for i=2, #v do
			event.args[i-1] = v[i];
		end

		table.insert(self.m_eventList, event);
	end

	if t == 2 then --连击一次
		for _, v in pairs(scriptData1) do
			if v[2] ~= ScriptEvent.AnimationTypeAction then
				local event = {};
				event.time = v[1]/1000 + maxTime + 0.1;
				event.args = {};
				event.timeLine = 1

				for i=2, #v do
					event.args[i-1] = v[i];
				end

				table.insert(self.m_eventList, event);
			end
			
		end

		for _, v in pairs(scriptData2) do
			if v[2] ~= ScriptEvent.AnimationTypeAction then
				local event = {};
				event.time = v[1]/1000 + maxTime + 0.1;
				event.args = {};
				event.timeLine = 2

				for i=2, #v do
					event.args[i-1] = v[i];
				end

				table.insert(self.m_eventList, event);
			end
			
		end
	end
end

function SkillScript:setSkillType(skillType)
	self.skillType = skillType
end

function SkillScript:GetID()
	return self.m_id;
end

function SkillScript:TakeOff()
	self.m_haveSecEff, self.m_secEffTagList = EffectFunction:MakeSkillSecEff(self.argsList.Attacker, self.argsList.Target, self.argsList.Skill);--计算是否有衍生效果
	self.m_isRun = true;
end

function SkillScript:Update(dt)
	if not self.m_isRun then
		return true;
	end
	if self.forceEnd then
		self:EndScript()
		return false
	end
	self.m_time = self.m_time + dt;
	if self.m_time2 then
		self.m_time2 = self.m_time2 + dt
	end
	local eventNum = 0
	for k, v in pairs(self.m_eventList) do
		if v.timeLine == 1 then
			if self.m_time >= v.time then
				self:runEvent(v.args);
				self.m_eventList[k] = nil;
			else
				eventNum = eventNum + 1;
			end
		elseif v.timeLine == 2 then
			if self.m_time2 and self.m_time2 >= v.time then
				self:runEvent(v.args);
				self.m_eventList[k] = nil;
			else
				eventNum = eventNum + 1;
			end
		end
	end
	if eventNum == 0 then
		self:EndScript();
		return false;
	end
	return true;
end

function SkillScript:setTimeLine2()
	self.m_time2 = 0;
end

function SkillScript:runEvent(args)
	local eventType = args[1];
	local attacker = FightManager:GetFighter(self.argsList.Attacker);
	local restraintType = attacker.fightAttribute
	if ScriptEvent.AnimationTypeAction == eventType then
		if self.argsList.Attacker == -1 then --god
			--触发播放技能特效
			attacker:GodShowSkill()
			self:setTimeLine2()
			return
		end
		local AnimationType = Enum:getTable('AnimationType');
		--人物进行动作
		local actionType = args[2];
		local action = 'attack';
		local id
		if actionType == 1 then
			action = 'attack'
			id = AnimationType.attack
		elseif actionType == 2 then
			action = 'skill'
			id = AnimationType.skill
		end
		local effectID = args[3]
		
		EffectFunction:SetAnimation(self.argsList.Attacker, id);
		
		self.m_onlyOnce = false;
		local fighter = FightManager:GetFighter(self.argsList.Attacker);
		fighter:addArmatureEventListener(function()
			if self.m_onlyOnce == true then
				return;
			end
			self.m_onlyOnce = true;
			self:setTimeLine2()
			EffectFunction:AttachAvatarPosEffect(self.argsList.Attacker, 0, {data=1, pos=action}, scale, 100, effectID, self.argsList.Skill:GetResID(), restraintType);
		end)

	elseif ScriptEvent.AddDamage == eventType then
		-- 造成伤害
		local damageIndex = args[2];
		EffectFunction:AddDamage(self.argsList.Attacker, self.argsList.Target, self.argsList.Skill, damageIndex);

	elseif ScriptEvent.AddBuff == eventType then
		-- 添加buff
		EffectFunction:AddBuff(self.argsList.Attacker, self.argsList.Target, self.argsList.Skill);
		EffectFunction:AddBuff2(self.argsList.Attacker, self.argsList.SecTarget, self.argsList.Skill);

	elseif ScriptEvent.AddFollow == eventType then
		-- 添加follow
		if self.m_haveSecEff then --添加衍生效果
			EffectFunction:SetSkillSecEff(self.argsList.Attacker, self.m_secEffTagList, self.argsList.Skill);
		end
	
	elseif ScriptEvent.AddSound == eventType then
		-- 添加声音
		local soundIndex = args[2];
		EffectFunction:PlaySound(soundIndex);
	
	elseif ScriptEvent.AddAvatarEffect == eventType then
		-- 添加挂点特效
		-- 挂点目标1攻击者，2目标 ;特效id ; 挂点位置; 缩放值
		local target = args[2];
		local effectID = args[3];
		local pos = args[4];
		local scale = args[5];

		local id
		if target == 1 then
			if self.argsList.Attacker == -1 then --god
				return
			end
			id = EffectFunction:AttachAvatarPosEffect(self.argsList.Attacker, 0, {data=2, pos=pos}, scale, 100, effectID, self.argsList.Skill:GetResID(), restraintType);
		elseif target == 2 then
			id = EffectFunction:AttachAvatarPosEffect(self.argsList.Target, 0, {data=2, pos=pos}, scale, 100, effectID, self.argsList.Skill:GetResID(), restraintType);
		elseif target == 3 then
			id = EffectFunction:AttachAvatarPosEffect(self.argsList.SecTarget, 0, {data=2, pos=pos}, scale, 100, effectID, self.argsList.Skill:GetResID(), restraintType);
		end
		if id and self.skillType == SkillType.activeSkill then
			FightShowSkillManager:addSkillScriptCheckCallback(function()
				if FightEffectManager:GetEffect(id) then
					return false
				else
					return true
				end
			end)
		end

	elseif ScriptEvent.AddTraceEffect == eventType then
		if true then
			return
		end
		-- 添加追踪特效
		-- 追踪特效类型1图片 2特效, 素材id, 弹道类型, 击中时间, 初始挂点x, 初始挂点y, 缩放值
		local effectType = args[2];
		local effectID = args[3];
		local traceType = args[4];
		local traceTime = args[5];
		local x, y, scale = args[6], args[7], args[9];
		EffectFunction:AttachTraceEffect(self.argsList.Attacker, {x=x, y=y}, traceType, traceTime, scale, self.argsList.Target, effectType, effectID, self.argsList.Skill:GetResID());

	elseif ScriptEvent.AddSceneEffect == eventType then
		-- 添加场景特效
		local effectID = args[2];
		local pos, scale = args[3], args[4];
		local targetType = 1;
		if #args == 5 then
			targetType = args[3];
			pos, scale = args[4], args[5];
		end

		local id;
		if targetType == 2 then
			id = EffectFunction:AttachSceneEffect(self.argsList.SecTarget, pos, scale, effectID, self.argsList.Skill:GetResID());
		else
			id = EffectFunction:AttachSceneEffect(self.argsList.Target, pos, scale, effectID, self.argsList.Skill:GetResID());
		end
		
		if id and self.skillType == SkillType.activeSkill then
			FightShowSkillManager:addSkillScriptCheckCallback(function()
				if FightEffectManager:GetEffect(id) then
					return false
				else
					return true
				end
			end)
		end

	elseif ScriptEvent.AddSceneShake == eventType then
		-- 添加震屏特效
		local shakeType = args[2] or 0;
		EventManager:FireEvent(EventType.FightPanelShake, shakeType);
		if self.skillType == SkillType.activeSkill then
			FightShowSkillManager:addSkillScriptCheckCallback(function()
				if FightShowSkillManager:shakeIsEnd() then
					return true
				else
					return false
				end
			end)
		end

	end
end

function SkillScript:SetArgs(key, value)
	self.argsList[key] = value;
end

function SkillScript:Pause()
	self.m_isRun = false;
end

function SkillScript:Continue()
	self.m_isRun = true;
end

function SkillScript:EndScript()
	self.m_isRun = false;
	for _, fid in pairs(self.argsList.Target) do --清除演算的状态
		local fighter = FightManager:GetFighter(fid);
		fighter:ClearWillCalcu();
	end
	if self.skillType == SkillType.activeSkill then
		FightShowSkillManager:setSkillScriptRunOver()
	end
end

function SkillScript:GetScriptName()
	return self.m_nameID;
end

function SkillScript:setDestroy()
	self.forceEnd = true
end

return SkillScript;
