--fightAttackClass.lua
--=======================================================================
--单次攻击类
local FightAttack = class("FightAttack");

function FightAttack:ctor(id, attacker, targetList, secList, attackSkill, t)
	self.m_id					= id;
	self.m_scriptID				= -1;
	self.m_attacker				= attacker;

	self.m_targetList			= targetList; 	--目标
	self.m_secList				= secList; 		--第二类目标
	self.m_attackSkill			= attackSkill;  --角色技能

	self.m_hasTraceEffect		= false;		--此次攻击是否已经创建了追踪特效
	self.m_isShowSkill			= false;        --是否展示技能
	self.m_needDestroy			= false;        --技能脚本结束时是否需要销毁
	self.m_t                    = t or 0

	--创建技能脚本
	if attackSkill then
		self:CreateSkillScript();
	end
end

--创建技能脚本
function FightAttack:CreateSkillScript()
	local skillScript = SkillScriptManager:CreateSkillScript(self.m_attackSkill:GetScriptID(), self.m_t);  
	skillScript:SetArgs('Attacker', self.m_attacker:GetID());        --攻击者的ID
	local targetIDList = {};
	for _, v in pairs(self.m_targetList) do
		table.insert(targetIDList, v:GetID());
	end
	local secTagIDList = {};
	for _, v in pairs(self.m_secList) do
		table.insert(secTagIDList, v:GetID());
	end
	skillScript:SetArgs('Target', targetIDList);        --目标
	skillScript:SetArgs('SecTarget', secTagIDList);        --目标
	skillScript:SetArgs('Skill', self.m_attackSkill);		-- 技能
	skillScript:TakeOff();                    --执行

	self.m_scriptID = skillScript:GetID();

	skillScript:setSkillType(self.m_attackSkill.m_skillType)
end

--销毁
function FightAttack:Destroy()
	if (self.m_scriptID ~= -1) then
		SkillScriptManager:DestroySkillScript(self.m_scriptID);
	end  

	self.m_scriptID = -1;
end

--判断技能脚本是否已经被销毁
function FightAttack:IsSkillScriptDestroy()
	if SkillScriptManager:GetSkillScript(self.m_scriptID) then
		return false
	end
	return true
end

--========================================================================
--Get方法
--获取id
function FightAttack:GetID()
	return self.m_id;
end

--获取攻击者
function FightAttack:GetAttacker()
	return self.m_attacker;
end

--获取技能
function FightAttack:GetAttackerSkill()
	return self.m_attackerSkill;
end

--获取攻击脚本
function FightAttack:GetScriptID()
	return self.m_scriptID;
end

--获取是否在技能脚本结束时需要销毁
function FightAttack:NeedDestroy()
	return self.m_needDestroy;
end

--设置攻击目标
function FightAttack:SetDamageTargeterList(targeterList)
	self.m_targeterList.damageTargeterList = targeterList;  
end

--获取攻击的基准目标
function FightAttack:GetDamageBaseTargeter()
	if self.m_targeterList.damageTargeterList == nil then
		--如果释放buff技能时，此值有可能为空
		return nil;
	else
		return self.m_targeterList.damageTargeterList[1];
	end  
end

--获取被攻击者
function FightAttack:GetDamageTargeterList()
	return self.m_targeterList.damageTargeterList;
end

--获取友军目标
function FightAttack:GetAllyTargeterList()
	return self.m_targeterList.buffTargeterList;
end

--设置是否需要在技能脚本结束时销毁对象
function FightAttack:SetNeedDestroy(flag)
	local skillScript = SkillScriptManager:GetSkillScript(self.m_scriptID)
	if skillScript then
		skillScript:setDestroy()
	end
end

--获取是否已经创建过追踪特效
function FightAttack:HasTraceEffectCreated()
	return self.m_hasTraceEffect;
end  

--设置是否创建追踪特效
function FightAttack:SetTraceEffectCreated(flag)
	self.m_hasTraceEffect = flag;
end

return FightAttack;
