--fightBaseSkill.lua
--=================================================================
--角色技能基础类
--
local FightBaseSkill = class("FightBaseSkill");
local SkillType = Enum:getTable("SkillType");

-- 构造
function FightBaseSkill:ctor(fighter, resid)
	self.m_skillType				= 0;		-- 技能类型
	self.m_priority					= 1;		-- 技能优先级从高到低：主动技能，被动技能，普通
	self.m_releaseFlag				= false;	-- 释放标志位
	self.m_fighter					= fighter;	-- 角色对象

	self.m_totalTime				= 0;		-- 持续时间
	self.m_scriptID					= '';		-- 脚本名称

	self.m_fightAttackClass			= nil;		-- 攻击类

	self.isCasting					= false;	-- 技能是否释放
	self.m_resid					= resid;
	if self.m_resid == nil then
		dump(fighter,'FightBaseSkill === ctor == ');
		-- GlobalFunctions:showErrorPromt('error=====  m_resid == nil ');
	end

	self:InitDamage();
	self:InitBuff();
end

--更新
function FightBaseSkill:Update( dt )
	self.m_curDuration = os.time() - self.m_beginTime;
	local isEnd = self.m_curDuration >= self.m_totalTime;
end

function FightBaseSkill:GetResID()
	return self.m_resid;
end

function FightBaseSkill:GetSkillType()
	return self.m_skillType;
end

function FightBaseSkill:SetSkillType( t )
	self.m_skillType = t
end

--====================================================================
--寻找技能释放目标，如果找到，则释放技能；没有找到，则跳过
function FightBaseSkill:CanReleaseSkill(elapse)

	local targeterList, secTagList = self:FindSkillTarget();

	local targetNums = 0;
	for _, v in pairs(targeterList) do
		targetNums = targetNums + 1;
	end

	for _, v in pairs(secTagList) do
		targetNums = targetNums + 1;
	end

	if targetNums > 0 then
		self:RunSkill(targeterList, secTagList);
	else
		FightSkillManager:AddSkill(self.m_fighter:GetID(), self);
	end
end

--寻找技能释放目标
function FightBaseSkill:FindSkillTarget()
	local targetList = {};
	targetList = FightManager:GetTargeterList(self.m_fighter, self.m_fighter:IsEnemy(), self.tableData['target_area'], self.tableData['target_condition']);
	-- if #targetList == 0 then 
	-- 	print('no targert')
	-- 	return {};
	-- end;

	local secTagList = {};
	if #self.tableData['target_area_2'] >0 and #self.tableData['target_condition_2']>0 then
		secTagList = FightManager:GetTargeterList(self.m_fighter, self.m_fighter:IsEnemy(), self.tableData['target_area_2'], self.tableData['target_condition_2']);
	end

	return targetList, secTagList;
end

--释放技能
function FightBaseSkill:RunSkill()
	--设定当前释放状态
	self.isCasting = true;
end

--强制结束技能
function FightBaseSkill:EndSkill()
	--销毁攻击类
	if self.m_fightAttackClass ~= nil then
		FightAttackManager:DestroyAttack(self.m_fightAttackClass);
	end
	self.m_fightAttackClass = nil;
	self.isCasting = false;
end

--触发觉醒技能
function FightBaseSkill:RunJueXingSkill()
	-- 只有主动技能 和 普工 才能触发 觉醒技能；
	if self.m_skillType==SkillType.activeSkill or self.m_skillType==SkillType.normalAttack then
		local skill = self.m_fighter.juexingSkill;
		if skill then
			local bRun = false
			if skill:GetSkillType()==SkillType.passive4Skill then
				if self.m_skillType==SkillType.normalAttack then
					--只有普通技能触发的觉醒技能
					bRun = skill:TriggerNum();
				end
			elseif skill:GetSkillType()==SkillType.passive5Skill then
				--普通 和 主动 技能 都有概率触发
				if self.m_skillType==SkillType.normalAttack then
					bRun = skill:TriggerRandom(1);
				else
					bRun = skill:TriggerRandom(2);
				end
			end

			if bRun then
				skill:CanReleaseSkill()
			end

		end
	end
	
end

--是否正在释放
function FightBaseSkill:isSkillCasting()
	return self.isCasting;
end

--====================================================================
--获取属性值

--获取技能类型
function FightBaseSkill:GetSkillType()
	return self.m_skillType;
end

--获取技能释放标志位
function FightBaseSkill:GetReleaseFlag()
	return self.m_releaseFlag;
end

--获取脚本名称
function FightBaseSkill:GetScriptID()
	return self.m_scriptID;
end

--设置脚本名称
function FightBaseSkill:SetScriptID( id )
	self.m_scriptID = id;
end

--获取攻击类
function FightBaseSkill:GetAttackClass()
	return self.m_fightAttackClass;
end

--技能脚本是否结束
function FightBaseSkill:IsSkillScriptEnd()
	return self.m_IsSkillScriptEnd;
end

--======================================================================
--设置属性值
--设置技能释放标志位
function FightBaseSkill:SetReleaseFlag(flag)
	self.m_releaseFlag = flag;
end

--设置技能伤害因子
function FightBaseSkill:InitDamage()
	local skillData = ResTable:getRowData('skill', self.m_resid);
	self.damageList = {};

	for i=1, #skillData['skill_damage_value'] do
		local damageItem = {};
		if skillData['skill_damage_value'][i] then
			damageItem.baseDamage = skillData['skill_damage_value'][i][1];
			damageItem.factor = skillData['skill_damage_value'][i][2];
			table.insert(self.damageList, damageItem);
		end
	end
end

function FightBaseSkill:InitBuff()
	local skillData = ResTable:getRowData('skill', self.m_resid);
	self.buffList = {};
	if skillData == nil or type(skillData.buff) ~= 'table' then
		GlobalFunctions:showErrorPromt( 'table [skill] error : id = '..tostring(self.m_resid) )
	end
	for k, v in pairs(skillData.buff) do
		if v[1] and v[2] then
			table.insert(self.buffList, {rate = v[1], buff_id = v[2]})
		end
	end

	self.buff2List = {};
	if skillData == nil or type(skillData.buff_2) ~= 'table' then
		return;
	end
	for k, v in pairs(skillData.buff_2) do
		if v[1] and v[2] then
			table.insert(self.buff2List, {rate = v[1], buff_id = v[2]})
		end
	end
end

function FightBaseSkill:GetDamageItem(index)
	return self.damageList[index];
end

function FightBaseSkill:checkIsConAndLastHit(index)
	if self.damageList[index] then
		if #self.damageList == 1 then
			return false, true
		else
			if index == #self.damageList then
				return true, true
			else
				return true, false
			end
		end
	else
		return false, false
	end
end

function FightBaseSkill:checkLastHit(index)
	if self.damageList[index] then
		if index == #self.damageList then
			return true, true
		else
			return true, false
		end
	else
		return false, false
	end
end

function FightBaseSkill:GetDamageList()
	return self.damageList;
end

function FightBaseSkill:GetBuffList()
	return self.buffList;
end

function FightBaseSkill:GetBuff2List()
	return self.buff2List;
end

function FightBaseSkill:GetFollowList()
	return self.followList;
end

return FightBaseSkill;
