local Buff = class("Buff")
local BuffType = Enum:getTable('BuffType')
local Attribute = Enum:getTable('Attribute')
local CastType = Enum:getTable('CastType')
local RestrainType = Enum:getTable("FightRestraintType")

local strList =
{
	'up',
	'down',
	'reflect',
	'dot_per',
	'hot_per',
	'heal_per',
	'dazed',
	'shield',
	'purge',
	'dispel',
	'dot',
	'hot',
	'heal',
	'silent',
	'refresh',
	'imm_neg',
	'imm_dam',
	'reject',
	'combo',
	'reborn',
	'cd_lang',
	'reborn_other',
	'heal_return',
	'cleanup',
	'breakup',
	'imm_kill',
}

function Buff:ctor(id, resid, target, origin, skill)
	self.m_id = id;
	self.resid = resid;
	self.target = target;
	self.origin = origin; -- skill caster
	self.isGod = self.origin:isGod()
	self.skill = skill;
	self.duration = 0;
	local buff_data = ResTable:getRowData('buff', self.resid);
	self.buff_type = tonumber(buff_data['buff_type']);
	self.buff_value_c = buff_data['effect_value'][1];
	self.buff_value_p = buff_data['effect_value'][2];
	self.buff_attribute = tonumber(buff_data['effect_type'][1]);
	self.total_time = tonumber(buff_data['impact_end']);
	self.buff_interval = tonumber(buff_data['frequency']);
	self.buff_icon = buff_data['icon']
	self.clear_type = tonumber(buff_data['clear_type'])
	self.aim_value = 0;
	self.hasStart = false;

	self.active = false;
	self.activeDur = 0; --当前激活生效 时长 或者次数 （次数的功能暂时没做）
	self.activeTotalTime = 0;
	self.activeType = 0;
	self.effShow = false;
	if buff_data['impact'] and #buff_data['impact'] == 2 then
		self.activeType = buff_data['impact'][1] --1 激活生效 为生效时间类型， 2 激活生效 为生效次数类型
		self.activeTotalTime = buff_data['impact'][2] --激活生效时长 或者 总次数
	end
	print('buff ctor:', json.encode(buff_data))
	
	-- 1 属性增加 atk aspd crit crit_dam
	-- 2 属性减少 atk aspd crit crit_dam
	-- 3 反射状态 
	-- 4 持续百分比掉血 
	-- 5 持续百分比回血
	-- 6 立即百分比回血
	-- 7 眩晕 
	-- 8 盾 atk 
	-- 9 清除 buff 1,5,12 进攻驱散     净化 purge
	-- 10 清除 debuff 2,4,11 防御驱散  驱散 dispel
	-- 11 持续掉血 基于atk
	-- 12 持续回血 基于atk
	-- 13 立即回血 基于atk
	-- 14 沉默 (增加技能cd)
	-- 15 刷新 (减少技能cd)
	-- 16 免疫负面状态
	-- 17 免疫伤害
	-- 18 法术反噬
	-- 19 法术连击
	-- 20 春哥

	if (self.buff_type == BuffType.down or
		self.buff_type == BuffType.dot_per or 
		self.buff_type == BuffType.dazed or 
		self.buff_type == BuffType.purge or 
		self.buff_type == BuffType.dot or 
		self.buff_type == BuffType.silent or 
		self.buff_type == BuffType.reject) and 
		self.target:HasSpecialState('imm_neg') then
		-- do nothing
	else
		self:Start()
	end
end

function Buff:getOrigin()
	return self.origin;
end

function Buff:getSkill()
	return self.skill;
end

function Buff:activeBuff(v)
	if self.buff_attribute >= v then
		self.active = true;
	end
	return self.active;
end

function Buff:checkActiveBuff(v)
	if self.buff_attribute >= v then
		return true;
	end
	return false;
end

function Buff:IsActive()
	return self.active;
end

function Buff:doActiveEff(v)
	self.aim_value = HpPlus(HpMultiply(v, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
	--if self.buff_interval == 1 then --出发次数计数，暂时没做 替换字段为 self.activeDur 和 self.activeTotalTime  ，buff_interval不用了
		--self:Over();
	--end
	return self.aim_value, self.buff_interval;
end

-- 更新
function Buff:Update( dt )
	-- type 1, 2, 3, 7, 8, do nothing util stop
	-- type 4, 5, 11, 12 update ever <interval> second
	if self.buff_type == BuffType.heal_return then
		if self.active==false then
			self.duration = self.duration + dt;
			if self.duration >= self.total_time then
				self:Over();
				return false;
			else
				return true;
			end
		end
		if self.effShow == false then --激活后显示buff效果
			self.effShow = true;
			self.target:AddSpecialState('heal_return', self.m_id, self.resid, self.origin.fightAttribute, false)--'heal'
		end
		self.activeDur = self.activeDur + dt;
		if self.activeDur >= self.activeTotalTime then
			self:Over();
			return false;
		end
		--如果没到有效时长，则进入下面逻辑 判断总生命时长 是否有效
	end

	self.duration = self.duration + dt;
	if self.buff_type == BuffType.dot or 
		self.buff_type == BuffType.hot or 
		self.buff_type == BuffType.dot_per or 
		self.buff_type == BuffType.hot_per then
		if self.duration >= self.nextEffectTime then
			self:doUpdateEffect();
			self.nextEffectTime = self.nextEffectTime + self.buff_interval;
		end
	end

	if self.duration >= self.total_time then
		self:Over();
		return false;
	else
		return true;
	end
end

function Buff:doUpdateEffect()
	if self.buff_type == BuffType.dot then
		if self.origin.fightAttribute == RestrainType.GREEN_BASE then
			self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);
		else
			local callback = function()
				self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);
			end
			self.target:AddSpecialState('dot', self.m_id, self.resid, self.origin.fightAttribute, false, callback)
		end
	elseif self.buff_type == BuffType.dot_per then
		local callback = function()
			self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);
		end
		self.target:AddSpecialState('dot_per', self.m_id, self.resid, self.origin.fightAttribute, false, callback)
	elseif self.buff_type == BuffType.hot or 
		self.buff_type == BuffType.hot_per then
		self.target:AddHeal(self.aim_value, self.origin, self.skill);
		self.target:AddSpecialState('hot', self.m_id, self.resid, self.origin.fightAttribute, false)
	end
	print("buffId:", self.m_id, 'ori:',self.origin:getFormationIndex(),  'oriEnemy:',self.origin:IsEnemy(), 'tag:',self.target:getFormationIndex(),  'tagEnemy:',self.target:IsEnemy() )
	print("buff vale: " , self.buff_value_p , self.buff_value_c , " aim value: " , self.aim_value)
end

-- 开始
function Buff:Start()
	self.hasStart = true
	local str_b = strList[self.buff_type] or 'nil';
	print("-----------------Buff log start-----------------id:",self.m_id)
	print("--add " .. str_b .. "--")

	if self.buff_type == BuffType.up then
		-- 1 属性增加 atk aspd crit crit_dam
		if self.buff_attribute == 2 then
			local atk_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
		elseif self.buff_attribute == 4 or
			self.buff_attribute == 5 or 
			self.buff_attribute == 6 then
			local attribute_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = attribute_cur * self.buff_value_p + self.buff_value_c;
		elseif self.buff_attribute == 7 or
			self.buff_attribute == 8 or 
			self.buff_attribute == 9 then
			self.aim_value = self.buff_value_c+self.buff_value_p;
		end
		self.target.m_propertyData:IncreasePropertyData(Attribute[self.buff_attribute], self.aim_value);

		if self.buff_attribute == 2 then
			self.target:AddSpecialState('atk_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 4 then
			self.target:AddSpecialState('haste', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 5 then
			self.target:AddSpecialState('crit_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 6 then
			self.target:AddSpecialState('critDam_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 7 then
			self.target:AddSpecialState('damage_enlarge_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 8 then
			self.target:AddSpecialState('damage_reduce_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 9 then
			self.target:AddSpecialState('heal_effect_up', self.m_id, self.resid, self.origin.fightAttribute, true)
		end

	elseif self.buff_type == BuffType.down then
		-- 2 属性减少 atk aspd crit crit_dam
		if self.buff_attribute == 2 then
			local atk_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
		elseif self.buff_attribute == 4 then
			local aspd_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = aspd_cur * self.buff_value_p + self.buff_value_c;
			if aspd_cur - self.aim_value < 0.01 then
				self.aim_value = aspd_cur - 0.01
			end
		elseif self.buff_attribute == 5 then
			local attribute_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = attribute_cur * self.buff_value_p + self.buff_value_c;
		elseif self.buff_attribute == 6 then
			local attribute_cur = self.target.m_propertyData:GetCurrentPropertyData(Attribute[self.buff_attribute]);
			self.aim_value = attribute_cur * self.buff_value_p + self.buff_value_c;
			if attribute_cur - self.aim_value < 0 then
				self.aim_value = attribute_cur;
			end
		elseif self.buff_attribute == 7 or
			self.buff_attribute == 8 or 
			self.buff_attribute == 9 then
			self.aim_value = self.buff_value_c+self.buff_value_p;
		end
		self.target.m_propertyData:DecreasePropertyData(Attribute[self.buff_attribute], self.aim_value);

		if self.buff_attribute == 2 then
			self.target:AddSpecialState('atk_down', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 4 then
			self.target:AddSpecialState('slow', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 5 then
			self.target:AddSpecialState('crit_down', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 6 then
			self.target:AddSpecialState('critDam_down',	self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 7 then
			self.target:AddSpecialState('damage_enlarge_down', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 8 then
			self.target:AddSpecialState('damage_reduce_down', self.m_id, self.resid, self.origin.fightAttribute, true)
		elseif self.buff_attribute == 9 then
			self.target:AddSpecialState('heal_effect_down', self.m_id, self.resid, self.origin.fightAttribute, true)
		end
	
	elseif self.buff_type == BuffType.reflect then
		-- 3 反射状态 
		self.target:AddSpecialState('reflect', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.dot_per then
		-- 4 持续百分比掉血 
		local max_hp = self.target.m_propertyData:GetCurrentPropertyData('maxHp');
		self.aim_value = HpPlus(HpMultiply(max_hp, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
		self.buff_attribute = self.buff_attribute or 1
		local limitVal = HpMultiply(self.origin.m_propertyData:GetCurrentPropertyData('atk'), self.buff_attribute)
		if HpCompare(self.aim_value, '>', limitVal) and self.isGod==false then
			self.aim_value = limitVal;
		elseif HpCompare(max_hp, '<=', '5') and self.isGod==false then --没有血量上限
			self.aim_value = limitVal;
		elseif HpCompare(self.aim_value, '<=', '0') then
			self.aim_value = '1';
		end

		local callback = function()
			self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);
		end

		self.target:AddSpecialState('dot_per', self.m_id, self.resid, self.origin.fightAttribute, true, callback)

	elseif self.buff_type == BuffType.hot_per then
		-- 5 持续百分比回血
		local max_hp = self.target.m_propertyData:GetCurrentPropertyData('maxHp');
		self.aim_value = HpPlus(HpMultiply(max_hp, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));

		self.target:AddHeal(self.aim_value, self.origin, self.skill);

		self.target:AddSpecialState('hot_per', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.heal_per then
		-- 6 立即百分比回血
		local max_hp = self.target.m_propertyData:GetCurrentPropertyData('maxHp');
		self.aim_value = HpPlus(HpMultiply(max_hp, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));

		local callback = function()
			self.target:AddHeal(self.aim_value, self.origin, self.skill);
		end

		self.target:AddSpecialState('heal_per', self.m_id, self.resid, self.origin.fightAttribute, false, callback)

	elseif self.buff_type == BuffType.dazed then
		-- 7 眩晕 
		-- self.target:addSkillCastCD(self.total_time, 0);
		FightSkillManager:StopFighterSkill( self.target:GetID() , self.total_time );

		self.target:AddSpecialState('dazed', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.shield then
		-- 8 盾
		local atk_cur = self.origin.m_propertyData:GetCurrentPropertyData('atk');
		self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
		self.remain_value = self.aim_value;

		self.target:AddSpecialState('shield', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.purge then
		-- 9 清除 buff 1,5,12 进攻驱散
		BuffManager:Purge(self.target);
		self.target:AddSpecialState('purge', self.m_id, self.resid, self.origin.fightAttribute, false)
	
	elseif self.buff_type == BuffType.dispel then
		-- 10 清除 debuff 2,4,11 防御驱散
		BuffManager:Dispel(self.target);
		self.target:AddSpecialState('dispel', self.m_id, self.resid, self.origin.fightAttribute, false)

	elseif self.buff_type == BuffType.dot then
		-- 11 持续掉血 基于atk
		local atk_cur = self.origin.m_propertyData:GetCurrentPropertyData('atk');
		self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));

		if HpCompare(self.aim_value, '>', atk_cur) and self.isGod==false then
			self.aim_value = atk_cur;
		elseif HpCompare(self.aim_value, '<=', '0') then
			self.aim_value = '1';
		end

		local callback = function()
			self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);		
		end

		self.target:AddSpecialState('dot', self.m_id, self.resid, self.origin.fightAttribute, true, callback)
		
	elseif self.buff_type == BuffType.hot then
		-- 12 持续回血 基于atk
		local atk_cur = self.origin.m_propertyData:GetCurrentPropertyData('atk');
		self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));

		self.target:AddHeal(self.aim_value, self.origin, self.skill);

		self.target:AddSpecialState('hot', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.heal then
		-- 13 立即回血 基于atk
		local atk_cur = self.origin.m_propertyData:GetCurrentPropertyData('atk');
		self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));

		local callback = function()
			self.target:AddHeal(self.aim_value, self.origin, self.skill);
		end

		self.target:AddSpecialState('heal', self.m_id, self.resid, self.origin.fightAttribute, false, callback)

	elseif self.buff_type == BuffType.silent then
		-- 14 沉默
		FightSkillManager:JumpFighterSkill( self.target:GetID() , self.total_time );
		self.target:AddSpecialState('silent', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.refresh then
		-- 15 刷新
		self.aim_value = self.buff_value_c
		self.target:reduceSkillCastCD(self.aim_value)
		self.target:AddSpecialState('refresh', self.m_id, self.resid, self.origin.fightAttribute, false)

	elseif self.buff_type == BuffType.imm_neg then
		-- 16 免疫负面状态
		self.target:AddSpecialState('imm_neg', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.imm_dam then
		-- 17 免疫伤害
		self.target:AddSpecialState('imm_dam', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.reject then
		-- 18 法术反噬
		local atk_cur = self.origin.m_propertyData:GetCurrentPropertyData('atk')
		self.aim_value = HpPlus(HpMultiply(atk_cur, tostring(self.buff_value_p)), ConvertDigitUnitToStr(tostring(self.buff_value_c)));
		self.target:AddSpecialState('reject', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.combo then
		-- 19 法术连击
		self.target:AddSpecialState('combo', self.m_id, self.resid, self.origin.fightAttribute, true)
	
	elseif self.buff_type == BuffType.reborn then
		-- 20 重生
		local hp_max = self.target.m_propertyData:GetMaxHP()
		self.aim_value = HpPlus(ConvertDigitUnitToStr(tostring(self.buff_value_c)), HpMultiply(hp_max, tostring(self.buff_value_p)))
		self.target:AddSpecialState('reborn', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.cd_lang then
		-- 21 cd时间加长 
		self.target:addSkillCastCD(self.buff_value_c, self.buff_value_p);
		self.target:AddSpecialState('cd_lang', self.m_id, self.resid, self.origin.fightAttribute, true)

	elseif self.buff_type == BuffType.reborn_other then
		-- 22 重生另外一个死去的人 
		FightManager:reviveFighter( self.target:IsEnemy() );
		self.target:AddSpecialState('reborn_other', self.m_id, self.resid, self.origin.fightAttribute, false)

	elseif self.buff_type == BuffType.heal_return then
		-- 23 回光返照 加血 激活后才显示效果
		--self.target:AddSpecialState('heal_return', self.m_id, self.resid, self.origin.fightAttribute, false)--'heal'

	elseif self.buff_type == BuffType.cleanup then
		-- 24 大净化 debuff 2,4,11, 7 14 18 驱散
		BuffManager:ExtDispel(self.target);
		self.target:AddSpecialState('cleanup', self.m_id, self.resid, self.origin.fightAttribute, false)--dispel

	elseif self.buff_type == BuffType.breakup then
		-- 25 大驱散 buff 1,5,12, 8,16,17,19 进攻驱散
		BuffManager:ExtPurge(self.target);
		self.target:AddSpecialState('breakup', self.m_id, self.resid, self.origin.fightAttribute, false)--purge

	elseif self.buff_type == BuffType.imm_kill then
		self.target:AddSpecialState('imm_kill', self.m_id, self.resid, self.origin.fightAttribute, false)--purge

	end
	self.nextEffectTime = self.buff_interval;
	print("buffId:", self.m_id, 'ori:',self.origin:getFormationIndex(),  'oriEnemy:',self.origin:IsEnemy(), 'tag:',self.target:getFormationIndex(),  'tagEnemy:',self.target:IsEnemy() )
	print("buff vale: " , self.buff_value_p , self.buff_value_c , " aim value: " , self.aim_value)

	local enemy = self.target:IsEnemy()
	EventManager:FireEvent(EventType.FightFooterBuff, { isAdd=true, pos=self.target:GetPositionIndex(), isEnemy = enemy, bid=self.m_id, btype=self.buff_type, icon=self.buff_icon });
	
end

-- 结束
function Buff:Over()

	-- log
	local str_b = strList[self.buff_type] or 'nil';
	print("-----------------Buff log over-----------------id:",self.m_id)
	print("--del " .. str_b .. "--")

	if self.buff_type == BuffType.up then
		self.target.m_propertyData:DecreasePropertyData(Attribute[self.buff_attribute], self.aim_value);
		if self.buff_attribute == 2 then
			self.target:RemoveSpecialState('atk_up', self.m_id)
		elseif self.buff_attribute == 4 then
			self.target:RemoveSpecialState('haste', self.m_id)
		elseif self.buff_attribute == 5 then
			self.target:RemoveSpecialState('crit_up', self.m_id)
		elseif self.buff_attribute == 6 then
			self.target:RemoveSpecialState('critDam_up', self.m_id)
		end
	elseif self.buff_type == BuffType.down then
		self.target.m_propertyData:IncreasePropertyData(Attribute[self.buff_attribute], self.aim_value);
		if self.buff_attribute == 2 then
			self.target:RemoveSpecialState('atk_down', self.m_id)
		elseif self.buff_attribute == 4 then
			self.target:RemoveSpecialState('slow', self.m_id)
		elseif self.buff_attribute == 5 then
			self.target:RemoveSpecialState('crit_down', self.m_id)
		elseif self.buff_attribute == 6 then
			self.target:RemoveSpecialState('critDam_down',	self.m_id)
		end
	elseif self.buff_type == BuffType.dot_per then
		self.target:RemoveSpecialState('dot_per', self.m_id, self.origin.fightAttribute);
	elseif self.buff_type == BuffType.dot then
		self.target:RemoveSpecialState('dot', self.m_id, self.origin.fightAttribute);
	elseif self.buff_type == BuffType.hot_per or 
		self.buff_type == BuffType.hot then
		self.target:RemoveSpecialState('hot', self.m_id);
	elseif self.buff_type == BuffType.shield then
		self.target:RemoveSpecialState('shield', self.m_id);
	elseif self.buff_type == BuffType.reflect then
		self.target:RemoveSpecialState('reflect', self.m_id);
	elseif self.buff_type == BuffType.dazed then
		self.target:RemoveSpecialState('dazed', self.m_id, self.origin.fightAttribute);
	elseif self.buff_type == BuffType.silent then
		self.target:RemoveSpecialState('silent', self.m_id)
	elseif self.buff_type == BuffType.imm_neg then
		self.target:RemoveSpecialState('imm_neg', self.m_id)
	elseif self.buff_type == BuffType.imm_dam then
		self.target:RemoveSpecialState('imm_dam', self.m_id)
	elseif self.buff_type == BuffType.reject then
		self.target:RemoveSpecialState('reject', self.m_id)
	elseif self.buff_type == BuffType.reborn then
		self.target:RemoveSpecialState('reborn', self.m_id)
	elseif self.buff_type == BuffType.cd_lang then
		self.target:RemoveSpecialState('cd_lang', self.m_id)
	elseif self.buff_type == BuffType.reborn_other then
		self.target:RemoveSpecialState('reborn_other', self.m_id)
	elseif self.buff_type == BuffType.heal_return then
		self.target:RemoveSpecialState('heal_return', self.m_id)
	elseif self.buff_type == BuffType.cleanup then
		self.target:RemoveSpecialState('cleanup', self.m_id)
	elseif self.buff_type == BuffType.breakup then
		self.target:RemoveSpecialState('breakup', self.m_id)
	elseif self.buff_type == BuffType.combo then
		self.target:RemoveSpecialState('combo', self.m_id)
	elseif self.buff_type == BuffType.imm_kill then
		self.target:RemoveSpecialState('imm_kill', self.m_id)
	end

	local enemy = self.target:IsEnemy()
	EventManager:FireEvent(EventType.FightFooterBuff, { isAdd=false, pos=self.target:GetPositionIndex(), isEnemy = enemy, bid=self.m_id, btype=self.buff_type });

end

function Buff:isStart()
	return self.hasStart
end

function Buff:makeEffect()
	local v = false;
	if self.buff_type == BuffType.reborn then
		self.target:AddHeal(self.aim_value, self.origin, self.skill);  --治疗来源 是buff释放者？		
		self.target:AddSpecialStateEffect('reborn')
		v = true;

	elseif self.buff_type == BuffType.combo and math.random()<=self.buff_value_p then
		self.target:AddSpecialStateEffect('combo')
		v = true;

	elseif self.buff_type == BuffType.reject then
		self.target:AddDamage(self.aim_value, CastType.normal, self.origin, self.skill, nil, 3);		
		self.target:AddSpecialStateEffect('reject')
		v = true;
	end
	return v;
end

function Buff:isPositive()
	if self.clear_type == 3 then
		return true;
	end
	return false;
	-- if self.buff_type == BuffType.up or 
	-- 	self.buff_type == BuffType.hot_per or
	-- 	self.buff_type == BuffType.hot then
	-- 	return true;
	-- else
	-- 	return false;
	-- end
end

function Buff:isNegative()
	if self.clear_type == 1 then
		return true;
	end
	return false;
	-- if self.buff_type == BuffType.down or 
	-- 	self.buff_type == BuffType.dot_per or
	-- 	self.buff_type == BuffType.dot then
	-- 	return true;
	-- else
	-- 	return false;
	-- end
end

function Buff:isExtPositive()
	if self.clear_type ==3 or self.clear_type == 4 then
		return true;
	end
	return false;
	-- if self.buff_type == BuffType.up or 
	-- 	self.buff_type == BuffType.hot_per or
	-- 	self.buff_type == BuffType.hot or
	-- 	self.buff_type == BuffType.shield or
	-- 	self.buff_type == BuffType.imm_neg or
	-- 	self.buff_type == BuffType.imm_dam or
	-- 	self.buff_type == BuffType.combo then
	-- 	return true;
	-- else
	-- 	return false;
	-- end
end

function Buff:isExtNegative()
	if self.clear_type ==1 or self.clear_type == 2 then
		return true;
	end
	return false;
	-- if self.buff_type == BuffType.down or 
	-- 	self.buff_type == BuffType.dot_per or
	-- 	self.buff_type == BuffType.dot or
	-- 	self.buff_type == BuffType.dazed or
	-- 	self.buff_type == BuffType.silent or
	-- 	self.buff_type == BuffType.reject then
	-- 	return true;
	-- else
	-- 	return false;
	-- end
end

function Buff:getType()
	return self.buff_type;
end

function Buff:getAttribute()
	return self.buff_attribute;
end

function Buff:getReflctValue()
	if self.buff_type == BuffType.reflect then
		return self.buff_value_c, self.buff_value_p;
	else
		return '0', 0;
	end
end

function Buff:getShieldValue()
	if self.buff_type == BuffType.shield then
		return self.remain_value;
	else
		return '0';
	end
end

function Buff:setShieldValue(value)
	if self.buff_type == BuffType.shield then
		self.remain_value = value;
	end
end

return Buff;
