local BuffManager = class("BuffManager")
local BuffType = Enum:getTable('BuffType')

function BuffManager:ctor()
	self:Initialize();
end

function BuffManager:Initialize()
	self.buffList = {};
	self.fighterList = {};
	self.buff_id = 0;
end

function BuffManager:Reset()
	self.buffList = {};
	self.buff_id = 0;
end

function BuffManager:Update( dt )
	for fighter_id, buffList in pairs(self.buffList) do
		local count = 0;
		for buff_type, buffs in pairs(buffList) do
			for _, buff in pairs(buffs) do
				if not buff:Update(dt) then
					if buff:getType() == BuffType.up or buff:getType() == BuffType.down then
						count = count + 1;
					end
					buffs[_] = nil;
				end
			end
		end
	end
end

function BuffManager:allocID()
	self.buff_id = self.buff_id + 1;
	return self.buff_id;
end

function BuffManager:AddBuff(target, origin, buff_resid, skill)
	if not self.buffList[target.id] then
		self.buffList[target.id] = {};
		self.fighterList[target.id] = target;
	end

	local buffItem = require("app.fight.fightBuff"):create(self:allocID(), buff_resid, target, origin, skill);
	local buff_type = buffItem:getType()
	if buffItem:isStart() then
		if not self.buffList[target.id][buff_type] then
			self.buffList[target.id][buff_type] = {}
		end
		self.buffList[target.id][buff_type][buffItem.m_id] = buffItem
	end
end

function BuffManager:DelDeadFighterBuff(fighter_id)
	local count = 0;
	if self.buffList[fighter_id] then
		for buff_type, buffs in pairs(self.buffList[fighter_id]) do
			for _, buff in pairs(buffs) do
				if buff:getType() == BuffType.up or buff:getType() == BuffType.down then
					count = count + 1;
				end
				buff:Over();
			end
		end
	end
	self.buffList[fighter_id] = {};
end

function BuffManager:getReflectValue(fighter_id)
	if self.buffList[fighter_id] and
		self.buffList[fighter_id][BuffType.reflect] then
		local sum_c, sum_p = '0', 0;
		for _, buff in pairs(self.buffList[fighter_id][BuffType.reflect]) do
			local c, p = buff:getReflctValue();
			sum_c = HpPlus(sum_c, c);
			sum_p = sum_p + p;
		end
		return sum_c, sum_p;
	else
		return '0', 0;
	end
end

function BuffManager:causeBuffEffect(fighter_id, buff_type)
	local ret = false;
	if self.buffList[fighter_id] and
		self.buffList[fighter_id][buff_type] then
		if buff_type == BuffType.reborn then
			local key = nil
			for _, buff in pairs(self.buffList[fighter_id][BuffType.reborn]) do
				buff:makeEffect()
				key = _;
				break
			end
			if key then
				self.buffList[fighter_id][BuffType.reborn][key]:Over()
				self.buffList[fighter_id][BuffType.reborn][key] = nil
			end
		else
			for _, buff in pairs(self.buffList[fighter_id][buff_type]) do
				local v = buff:makeEffect()
				if v==true then
					ret = v
				end
			end
		end
	end
	return ret;
end

function BuffManager:getShieldValue(fighter_id)
	local shieldList = {};
	if self.buffList[fighter_id] and 
		self.buffList[fighter_id][BuffType.shield] then
		for _, buff in pairs(self.buffList[fighter_id][BuffType.shield]) do
			local id, value = buff.m_id, buff:getShieldValue();
			table.insert(shieldList, {id=id, value=value});
		end
	end
	return shieldList;
end

function BuffManager:consumeShield(fighter_id, consumeList)
	if #consumeList > 0 and 
		self.buffList[fighter_id] and 
		self.buffList[fighter_id][BuffType.shield] then
		for k, v in pairs(consumeList) do
			self.buffList[fighter_id][BuffType.shield][v]:Over();
			self.buffList[fighter_id][BuffType.shield][v] = nil;
		end
	end
end

function BuffManager:setShield(fighter_id, buff_id, setValue)
	if self.buffList[fighter_id] and 
		self.buffList[fighter_id][BuffType.shield] and
		self.buffList[fighter_id][BuffType.shield][buff_id] then
		self.buffList[fighter_id][BuffType.shield][buff_id]:setShieldValue(setValue);
	end
end

function BuffManager:checkPositiveBuff(fighter_id)
	if self.buffList[fighter_id] then
		for buff_type, buffs in pairs(self.buffList[fighter_id]) do
			for k, v in pairs(buffs) do
				if v:isPositive() then
					return true;
				end
			end
		end
		return false;
	else
		return false;
	end
end

function BuffManager:checkNegativeBuff(fighter_id)
	if self.buffList[fighter_id] then
		for buff_type, buffs in pairs(self.buffList[fighter_id]) do
			for k, v in pairs(buffs) do
				if v:isNegative() then
					return true;
				end
			end
		end
		return false;
	else
		return false;
	end
end

function BuffManager:Purge(fighter)
	if self.buffList[fighter.id] then
		for buff_type, buffs in pairs(self.buffList[fighter.id]) do
			for _, buff in pairs(buffs) do
				if buff:isPositive() then
					buff:Over();
					buffs[_] = nil;
				end
			end
		end
	end
end

function BuffManager:ExtPurge(fighter)
	if self.buffList[fighter.id] then
		for buff_type, buffs in pairs(self.buffList[fighter.id]) do
			for _, buff in pairs(buffs) do
				if buff:isExtPositive() then
					buff:Over();
					buffs[_] = nil;
				end
			end
		end
	end
end

function BuffManager:Dispel(fighter)
	if self.buffList[fighter.id] then
		for buff_type, buffs in pairs(self.buffList[fighter.id]) do
			for _, buff in pairs(buffs) do
				if buff:isNegative() then
					buff:Over();
					buffs[_] = nil;
				end
			end
		end
	end
end

function BuffManager:ExtDispel(fighter)
	if self.buffList[fighter.id] then
		FightSkillManager:JingHuaBuff( fighter.id )
		for buff_type, buffs in pairs(self.buffList[fighter.id]) do
			for _, buff in pairs(buffs) do
				if buff:isExtNegative() then
					buff:Over();
					buffs[_] = nil;
				end
			end
		end
	end
end

function BuffManager:isHaveBuff(fighter_id, buff_type, active)
	local act = active or false;
	if self.buffList[fighter_id] and self.buffList[fighter_id][buff_type] then
		for k, v in pairs(self.buffList[fighter_id][buff_type]) do
			if v and v:IsActive()==act then
				return true, k;
			end
		end
	end
	return false, 0;
end

function BuffManager:activeBuff(fighter_id, buff_type, av)
	if self.buffList[fighter_id] and self.buffList[fighter_id][buff_type] then
		for k, v in pairs(self.buffList[fighter_id][buff_type]) do
			if v and v:IsActive()==false and v:activeBuff(av)==true then
				return true;
			end
		end
	end
	return false;
end

function BuffManager:checkActiveBuff(fighter_id, buff_type, av)
	if self.buffList[fighter_id] and self.buffList[fighter_id][buff_type] then
		for k, v in pairs(self.buffList[fighter_id][buff_type]) do
			if v and v:IsActive()==false and v:checkActiveBuff(av)==true then
				return true;
			end
		end
	end
	return false;
end

function BuffManager:getActiveValue(fighter_id, buff_type, buff_id, av)
	local value = 0;
	local onlyOnce = 0;
	local origin = nil;
	local skill = nil;
	if self.buffList[fighter_id] and 
		self.buffList[fighter_id][buff_type] and
		self.buffList[fighter_id][buff_type][buff_id] then
		value, onlyOnce = self.buffList[fighter_id][buff_type][buff_id]:doActiveEff( av );
		origin = self.buffList[fighter_id][buff_type][buff_id]:getOrigin();
		skill = self.buffList[fighter_id][buff_type][buff_id]:getSkill();
	end
	if onlyOnce==1 then
		self.buffList[fighter_id][buff_type][buff_id] = nil;
	end
	return value, origin, skill;
end

cc.exports.BuffManager = BuffManager:create();
