local FightManager = class("FightManager")

local FightState = Enum:getTable('FightState');
local FightType = Enum:getTable('FightType');
local FighterState = Enum:getTable('FighterState');
local scheduler = cc.Director:getInstance():getScheduler();
local Attribute = Enum:getTable('Attribute');
local SkillType = Enum:getTable('SkillType');
local TeamType = Enum:getTable('TeamType')

local leftFormationPos = {
	{x = -90, y = -140, z = 0},
	{x = -90, y = -230, z = 10},
	{x = -90, y = -320, z = 20},

	{x = -180,y = -140, z = 0},
	{x = -180,y = -230, z = 10},
	{x = -180,y = -320, z = 20},

	{x = -270,y = -140, z = 0},
	{x = -270,y = -230, z = 10},
	{x = -270,y = -320, z = 20},
}

local rightFormationPos = {
	{x = 90, y = -140, z = 0},
	{x = 90, y = -230, z = 10},
	{x = 90, y = -320, z = 20},

	{x = 180,y = -140, z = 0},
	{x = 180,y = -230, z = 10},
	{x = 180,y = -320, z = 20},

	{x = 270,y = -140, z = 0},
	{x = 270,y = -230, z = 10},
	{x = 270,y = -320, z = 20},
}


function FightManager:ctor()

	self:clearAll();

end

function FightManager:clearAll(  )
	self.speedup = nil;

	if self.m_scheduleHandler then
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil
	end
	
	SkillScriptManager:clearAll( )
	FightShowSkillManager:clearAll();

	FightSkillManager:Reset();
	SkillScriptManager:Reset();
	FightEffectManager:Reset();
	BuffManager:Reset();

	self:registEvent(false)


	self.needLeftActorList = {};
	self.needRightActorList = {};
	self.leftActorList = {};
	self.rightActorList = {};
	self.deadActorList = {};

	self.leftResultList = {}
	self.rightResultList = {}	
	self.leftAllResult = {['Atk']=0, ['Dam']=0, ['Help']=0}
	self.rightAllResult = {['Atk']=0, ['Dam']=0, ['Help']=0}
	self.leftSkillTime = {}
	self.rightSkillTime = {}

	--star
	self.deathRoleNum = 0;

	-- flags
	self.OverFlag = false;
	self.FightEndFlag = false;

	self.Foreground = false;

	self.isPause = false;
	self.isSkillPause = false
	self.state = FightState.None;

	self.delayTime = 0;
	self.delayDie = false;

	self.gotoView = nil;
	self.isScaling = false;
	self.bPlayDrawing = nil;

	self.isPlayRoleBookAni = false

	self.guildBossTotalDamage = 0

	self.downLoadRes = false;
	self.fightType = -1;

	self.Godfighter = nil;

	self.enemyFirstDieTime = 0;
	self.cheatingWarning = {};

	self.delayCheckOver = -1;
	self.deathIdx = 0;
end

function FightManager:getFightType()
	return self.fightType;
end

function FightManager:setIsScaling( v )
	self.isScaling = v;
end

function FightManager:setSpeedUp( v )
	self.speedup = v;
	model.UserDefaultModel:setDataByKey('speedup', self.speedup);
end

function FightManager:SpeedUp()
	if self.fightType == FightType.MultArena then
		return 0.5
	end
	if self.speedup==nil then
		self.speedup = model.UserDefaultModel:getDataByKey('speedup');
		if self.speedup==nil then
			self.speedup = 1
		end
	end
	return self.speedup
end

function FightManager:EnterGame()
	self:clearAll()
	self.state = FightState.None;
	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.Update), 0, false);

	SkillScriptManager:Init( )
	FightShowSkillManager:Init();
	--[[
	local tutorialFlag = false;
	self.Foreground = false;
	if tutorialFlag then
	else
		UIManager:preInit('FightPanel');		
	--]]
	self:registEvent()
end

function FightManager:registEvent( regist )
	if regist == false  then
		EventManager:UnregisterEvent(EventType.FightNext)
		EventManager:UnregisterEvent(EventType.Die)
		EventManager:UnregisterEvent(EventType.PlotFinish)
		return 
	end
	--	EventManager:RegisterEvent(EventType.FightChangeTotalHp, handler(self, self.ChangeTotalHp));
	EventManager:RegisterEvent(EventType.FightNext, handler(self, self.onFightNext));
	EventManager:RegisterEvent(EventType.Die, handler(self, self.onFighterDie));
	EventManager:RegisterEvent(EventType.PlotFinish, handler(self, self.finishPlot));
end

function FightManager:Update( dt )
	if self.downLoadRes == true then
		if DownloadControler:checkNeedFightRes( self.arenaModelList ) then
			self.downLoadRes = false
			self.needRightActorList = self:GetArenaMonsterList();
			self:switchState(FightState.Prepare);
		end
	end

	if self.isPause then
		return;
	end

	local timeScale = 1/self:SpeedUp()
	dt = dt * timeScale;

	if self.delayCheckOver>0 then
		self.delayCheckOver = self.delayCheckOver - dt;
		if self.delayCheckOver<=0 then
			self:checkIsOver()
			self.delayCheckOver=-1
		end
	end

	if FightState.None == self.state or FightState.Will == self.state then
	elseif FightState.Prepare == self.state then
		--加载战斗角色
		if not self.loadFinish then
			self:loadFighter(dt);
		else
			if self.doorCloseFlag then
				self:switchState(FightState.PlotBegin);
			end
		end
	elseif FightState.PlotBegin == self.state then
		if self.plotFinishFlag and self.doorOpenFlag then
			self:switchState(FightState.Progress);
		end
	elseif FightState.Progress == self.state then
		if self.isPlayRoleBookAni then
			return
		end
		if self.OverFlag then
			FightSkillManager:Reset();
			SkillScriptManager:Reset();
			FightEffectManager:Reset();
			BuffManager:Reset();

			self:switchState(FightState.PlotEnd);
			return;
		end
		if self.isSkillPause then
			SkillScriptManager:Update( dt );
			FightEffectManager:Update( dt );
			FightShowSkillManager:checkShowOver()
		else
			for k, v in pairs(self.leftActorList) do
				if not v:Update( dt ) then
					return;
				end
			end

			if self.isSkillPause == false and self.Godfighter then
				self.Godfighter:GodSkillCanRelease(dt)
			end

			if self.isSkillPause == false then --阻止本帧内 两次播放特效
				for k, v in pairs(self.rightActorList) do
					if not v:Update( dt ) then
						return;
					end
				end
			end
			
			SkillScriptManager:Update( dt );
			FightEffectManager:Update( dt );
			if self.isSkillPause == false then
				FightSkillManager:Update( dt );
				BuffManager:Update( dt );
				self:updateFightTime( dt );
				self:checkTimeOver();
			end
			
		end
	elseif FightState.PlotEnd == self.state then
		self.delayTime = self.delayTime + dt;
		if self.delayTime>2 and self.plotFinishFlag then
			self:switchState(FightState.End);
		end
	elseif FightState.End == self.state then
		if self.FightEndFlag then
			self:switchState(FightState.None);
		--	cc.Director:getInstance():getTextureCache():removeUnusedTextures()
		end
	end
end

function FightManager:switchState(state)
	if self.state == state then
		-- FTcheck
		print('Fault-Tolerant:')
		print(debug.traceback())
		return;
	end

	if FightState.None == state then
	elseif FightState.Prepare == state then
		FightSkillManager:Initialize();
		SkillScriptManager:Initialize();
		FightEffectManager:Initialize();
		BuffManager:Initialize();
		--self:preLoadEffect();
		self.loadFinish = false;
		self.loadKey = 1;
		EventManager:FireEvent(EventType.FightPreloadSkillEffect, self.needLeftActorList)
		EventManager:FireEvent(EventType.FightPreloadSkillEffect, self.needRightActorList)
	elseif FightState.PlotBegin == state then
		self.plotFinishFlag = false;
		self.plotFinishFlag = true;
		EventManager:FireEvent(EventType.FightDoorOpen, function()
			self:setDoorOpenFlag(true)
		end);
	elseif FightState.Progress == state then		
		self.fightTime = 0;
		self.fightSec = 0;
		EventManager:FireEvent(EventType.FightUpdateTime, self.fightMaxTime - self.fightSec);
		AudioManager:playMusic(Conf.bossFight, 3, true);

		if self.roundInfo.fightType == FightType.Boss and self.roundInfo.roundIndex %5 == 0 then
			EventManager:FireEvent(EventType.FightBossWarning);
		end
	elseif FightState.PlotEnd == state then
		--self.plotFinishFlag = false;
		self.delayTime = 0;
		self.plotFinishFlag = true; -- 孙杨说完全不管战后剧情界面
	elseif FightState.End == state then
		self.FightEndFlag = false;
		self:onFightExit();
	else
		-- FTcheck
		print('Fault-Tolerant:')
		print(debug.traceback())
		return;
	end
	print('self.state switch to ', state)

	self.state = state;
end

function FightManager:enterFight(fightType, option)
	if self.state ~= FightState.None then
		Prompt:Show(LANG.UI_FIGHT_52);
		return false;
	end
	-- option :
	-- boss: round_id,
	-- random boss: boss_id, 
	if not fightType then
		print(fightType)
		-- FTcheck
		print('Fault-Tolerant:')
		print(debug.traceback())
		return false;
	end
	self.doorOpenFlag = false;
	self.doorCloseFlag = false;
	local msg = {};
	if fightType == FightType.Boss then
		msg.resid = option.round_id;
		msg.flag = fightType;
		msg.team = self:getCurTeam();
		self:initTeam(msg.team);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	elseif fightType == FightType.RandomBoss then
		msg.resid = option.pid;
		msg.flag = fightType;
		msg.team = self:getCurTeam();
		self:initTeam(msg.team);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	elseif fightType == FightType.GuildBoss then
		self.guildBossTotalDamage = autolua.opensslToLua:aes_ecb_encrypt('0')
		self.showDamage = 0
		msg.resid = 1;
		msg.flag = fightType;
		msg.raid_no = model.GuildModel:getRaidNo()
		msg.team = self:getCurTeam(FightType.GuildBoss);
		self:initTeam(msg.team);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	elseif fightType == FightType.TrialTower then 

		msg.resid = option.trial_no;
		msg.auto = option.auto;
		msg.flag = fightType;
		msg.type = option.trial_type;
		local tid = model.TrialModel:getTrialTeamIdByType(option.trial_type)
		msg.team = self:getCurTeam(FightType.TrialTower, tid);
		if option.auto == 0 then
			self:initTeam(msg.team);
			Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
		else
			--扫荡直接调用onFightExit
			msg.win = 1;
			Network.NetworkManager:Send(Network.NetworkCmdType.req_exit_fight, msg);
			return true;
		end
	elseif fightType == FightType.CopyRound then 

		msg.resid = option.round_id;
		msg.flag = fightType;
		msg.team = self:getCurTeam();
		self:initTeam(msg.team);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);

	elseif fightType == FightType.Story then
		msg.resid = option.resid;
		msg.flag = fightType;
		msg.storyid = option.storyid;
		msg.team = self:getCurTeam(FightType.Story);
		self:initTeam(msg.team, FightType.Story);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
		
	elseif fightType == FightType.DailyRound then
		msg.resid = option.id;
		msg.flag = fightType;
		msg.team = self:getCurTeam()
		if option.auto then
			msg.win = 1;
			Network.NetworkManager:Send(Network.NetworkCmdType.req_exit_fight, msg);
			return true;
		else
			self:initTeam(msg.team);
			Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
		end
	elseif fightType == FightType.Event then
		if not model.TeamModel:canEventFight() then
			Prompt:Show(LANG.UI_FIGHT_77);
			return false;
		end
		msg.resid = option.id;
		msg.flag = fightType;
		msg.team = self:getCurTeam(FightType.Event)
		msg.event_id = model.EventModel:getEventId()
		self:initTeam(msg.team, FightType.Event);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	elseif fightType == FightType.Arena then
		msg.season = model.ArenaModel:getSeason();
		msg.resid = option.id;
		msg.flag = fightType;
		msg.team = self:getCurTeam(FightType.Arena)
		self:initTeam(msg.team, FightType.Arena);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	elseif fightType == FightType.MultArena then
		msg.challenge_no = option.subIndex;
		msg.resid = option.id;
		msg.flag = fightType;
		msg.season = model.ArenaModel:getMultArenaSeason();
		msg.team = self:getCurTeam(FightType.MultArena, option.selfTeamId)
		self:initTeam(msg.team, FightType.MultArena);
		Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_fight, msg);
	end
	return true;
end

function FightManager:BeginFight(msg)
	self.leftResultList = {}
	self.rightResultList = {}
	self.leftAllResult = {['Atk']=0, ['Dam']=0, ['Help']=0}
	self.rightAllResult = {['Atk']=0, ['Dam']=0, ['Help']=0}
	self.leftSkillTime = {}
	self.rightSkillTime = {}

	self.leftActorList = {};
	self.rightActorList = {};
	self.deadActorList = {};	
	self.OverFlag = false;

	self.Godfighter = nil;
	self.errorMsg = nil;
	self.deathIdx = 0;
	self.cheatingWarning = {};

	self.roundInfo = self:GetRoundInfo(msg);
	EventManager:FireEvent(EventType.FightPanelInitRoundInfo, self.roundInfo);
	
	self.forceEnd = false;

	-- 载入友方信息
	self.fightType = msg.flag;
	if msg.flag == FightType.Event then
		model.EventModel:addRobotToCardModel()
		self.needLeftActorList = self:GetEventAllyRoleList();
		self:initGodFighter();
	elseif msg.flag == FightType.Arena then
		self.needLeftActorList = self:GetAllyRoleList(6);
	elseif msg.flag == FightType.GuildBoss then
		self.needLeftActorList = self:GetAllyRoleList(2);
	elseif msg.flag == FightType.TrialTower then
		local tid = model.TrialModel:getTrialTeamIdByType(msg.type)
		self.needLeftActorList = self:GetAllyRoleList(tid);
	elseif msg.flag == FightType.MultArena then
		local tid = model.ArenaModel:getMultArenaTeamIdByIdx( msg.challenge_no );
		self.needLeftActorList = self:GetAllyRoleList(tid);
	elseif msg.flag == FightType.Story then
		local tid = model.TeamModel:getTeamIdByFightType(msg.flag)
		self.needLeftActorList = self:GetStoryAllyRoleList(tid);
	else
		self.needLeftActorList = self:GetAllyRoleList(1);
	end

	--载入敌方信息
	if msg.flag == FightType.GuildBoss then
		self.needRightActorList = self:GetGuildBoss();
	elseif msg.flag == FightType.Event then
		self.needRightActorList = self:GetEventMonsterList();
	elseif msg.flag == FightType.Arena then
		self:makeArenaMonsterRes()
		if DownloadControler:checkNeedFightRes( self.arenaModelList ) == false then
			self.downLoadRes = true
			return
		end
		self.needRightActorList = self:GetArenaMonsterList();
	elseif msg.flag == FightType.MultArena then
		local totalFp = 0;
		self.needRightActorList, totalFp = self:GetArenaMonsterList();
		model.ArenaModel:setMultArenaPower(totalFp)
	elseif msg.flag == FightType.Story then
		self.needRightActorList = self:GetEventMonsterList();
	else
		self.needRightActorList = self:GetMonsterList();
	end

	self:switchState(FightState.Prepare);
end

function FightManager:makeArenaMonsterRes()
	-- body
	self.arenaModelList = {}
	local rankListData = self.roundInfo.monsterList;
	for i=1, 9 do
		local data = rankListData.arena_team[i];
		if data and data.no ~= -1 then --玩家队伍
			if rankListData.isRobot == 0 then
				local res_id = ResTable:getValue('role', data.no, 'resource_id')
				local modelPath = ResTable:getValue('role_resource', res_id, 'model')
				table.insert( self.arenaModelList, modelPath )
			else
				local modelPath = ResTable:getValue('monster', data.no, 'model')
				table.insert( self.arenaModelList, modelPath )
			end
		end
	end
end

function FightManager:finishPlot()
	self.plotFinishFlag = true;
	UIManager:switch('FightPanel');
end

function FightManager:setDoorCloseFlag(flag)
	self.doorCloseFlag = flag;
end

function FightManager:setDoorOpenFlag(flag)
	self.doorOpenFlag = flag;
end

function FightManager:onFightNext()
	self:Destroy();
end

function FightManager:getCurTeam(fightType, teamType)
	fightType = fightType or FightType.Boss
	local teamPid = {};
	local roleList = nil;
	if fightType == FightType.TrialTower then
		roleList = model.TeamModel:getTeamById(teamType);

	elseif fightType == FightType.MultArena then
		roleList = model.TeamModel:getTeamById(teamType);

	else
		local tid = model.TeamModel:getTeamIdByFightType( fightType )
		roleList = model.TeamModel:getTeamById(tid);
		
	end
	for i=1, TEAM_MAX_MEMBER do
		teamPid[i] = roleList[i];
	end
	return teamPid;
end

function FightManager:initTeam(team, fightType)
	fightType = fightType or FightType.Boss
	self.team = {}
	for i=1, TEAM_MAX_MEMBER do
		local data = {};
		data.pid = team[i];
		data.hp = 0;
		data.cd = 0;
		if data.pid ~= -1 then
			-- 移除血量判断
			-- if model.TreatmentModel:isInTreatment(data.pid) then
			-- 	data.pid = -1;
			-- elseif model.CardModel:GetCard(data.pid).hp <= 0 then
			-- 	data.pid = -1;
			-- else
			-- 	data.hp = model.CardModel:GetCard(data.pid).hp;
			-- end
			if fightType == FightType.Event then
				data.hp = model.CardModel:getEventHp(data.pid)
				data.cd = model.CardModel:getEventCD(data.pid)
			elseif fightType == FightType.Story then
				data.hp, data.cd = model.StoryModel:getCardHpAndCd(data.pid)
			else
				data.hp = 10000;
			end
		end
		self.team[i] = data;
	end
end

function FightManager:ExitFight(msg)
	FightShowSkillManager:ForceOver()
	self.FightEndFlag = true;
	EventManager:FireEvent(EventType.FightOver, msg);
	EventManager:FireEvent(EventType.FightEnableChallenge);
end
-- local fightOverData = {
-- 	isLevelUp = self.isLevelUp,
-- 	levelUpData = self.levelUpData,
-- 	roundBreak = roundBreak,
-- 	round = self.resid, -- 上一关
-- 	newRoundBoss = result.newRoundBoss
--  step = 2 -- 初始2 
-- }
-- 战斗结束，顺序播放 1.碎片掉落 — > 2.角色升级  —>3.5关奖励 —> 4.小关推进/章节切换 —>5.挂机收益+ 随机BOSS   
function FightManager:showFightOverStep( callbackParams )
	if self.fightOverData == nil then
		return
	end
	local step = self.fightOverData.step 
	-- print('=========showFightOverStep === step == ',step)
	-- dump(callbackParams,'callbackParams === ')
	if step == 1 then
		self.fightOverData.step = 2
		self:showFightOverStep()
	elseif step == 2 then
		self.fightOverData.step  = 3
		if self.fightOverData.isLevelUp then
			local option =  {
				callback = handler(self,self.showFightOverStep),
			};
			self.fightOverData.showLevelUpGuide = true;
			UIManager:push('LevelUpPanel',self.fightOverData.levelUpData,option)
		else
			self:showFightOverStep()
		end
	elseif step == 3 then 
		self.fightOverData.step  = 4
		if self.fightOverData.roundBreak then
			local option =  {
				round = self.fightOverData.round,
				type = 1,
				callback = handler(self,self.showFightOverStep),
			};
			UIManager:push('FightRoundBreakPanel',option)

			callbackParams = nil;
		else
			self:showFightOverStep()
		end
	elseif step == 4 then 
		self.fightOverData.step = 5
		if self.fightOverData.bWin == true  then
			EventManager:FireEvent(EventType.FightMapSetNewRound) -- 大小关切换
		else
			self:showFightOverStep()
		end
	elseif step == 5 then 
		self.fightOverData.step = 6
		if self.fightOverData.roundBreak then
			local option =  {
				round = self.fightOverData.round,
				type = 2,
			};
			UIManager:push('FightRoundBreakPanel',option)
		end
		if self.fightOverData.newRoundBoss then
			EventManager:FireEvent(EventType.RoundBossUpdate,self.fightOverData.newRoundBoss);

			-- Prompt:Show('获得随机boss');

		end
		self:showFightOverStep()
	elseif step == 6 then 
		if self.fightOverData.showLevelUpGuide == true or self.fightOverData.round ~= nil  then
			dump(self.fightOverData,"self.fightOverData == ")
			model.GuideModel:resumeGuide();

			if self.fightOverData.showLevelUpGuide == true then
				--当当前引导id为0时 才起作用
				model.GuideModel:checkLevelUpGuide( model.UserModel:getUserLevel() )
				model.GuideModel:startTriggerGuide();
			end

			if self.fightOverData.round == 10104 then
				-- 关卡4结束，则新手引导结束，无需胜利
				--[[
				local args = {
					eventName = Enum:getTable('TrackEventToken').TRACK_TU_FINISH
				}
				sdkManager.SDKLuaHelper:trackEvent(args)
				]]
				GlobalFunctions:trackEvent('TRACK_TU_FINISH')
			end

			if self.fightOverData.round ~= nil and self.fightOverData.bWin==true then
				--当当前引导id为0时 才起作用
				model.GuideModel:checkRoundGuide( self.fightOverData.round )
				model.GuideModel:startTriggerGuide();
			end
			
		end
		if self.fightOverData.fightType == FightType.Event then
			model.EventModel:checkShowPanel();
		end
		if self.fightOverData.fightType == FightType.Story then
			model.StoryModel:checkShowPanel();
		end
		if not tools.ConfigStateTools:checkLockById(2601) and self.fightOverData.trial_round == 150 and self.fightOverData.trial_type == 1 and self.fightOverData.bWin==true then
			model.GuideModel:resumeGuide();
			model.GuideModel:triggerGuide( 86 )
			model.GuideModel:startTriggerGuide();
		end
		self.fightOverData = nil;
		
	end

end

function FightManager:showFightOverAni( fightOverData )
	self.fightOverData = fightOverData;
	self:showFightOverStep()
end

function FightManager:Destroy()
	for _, v in pairs(self.leftActorList) do
		v:DestroyArmature();
	end
	for _, v in pairs(self.rightActorList) do
		v:DestroyArmature();
	end
end

function FightManager:setAutoRelease()
	-- body
	if self.isScaling then
		return;
	end
	if self.gotoView then
		self.gotoView = false;
	else 
		self.gotoView = true;
		EventManager:FireEvent(EventType.FightShowSkillAuto)
	end
	model.UserDefaultModel:setDataByKey('gotoView', self.gotoView);
end

function FightManager:getAutoRelease()
	if self.fightType == FightType.MultArena then
		return true
	end
	if self.gotoView==nil then
		self.gotoView = model.UserDefaultModel:getDataByKey('gotoView');
		if self.gotoView==nil then
			self.gotoView = false
		end
	end
	return self.gotoView;
end

function FightManager:isPlayDrawing()
	if self.fightType == FightType.MultArena then
		return false
	end
	if self.bPlayDrawing==nil then
		self.bPlayDrawing = model.UserDefaultModel:getDataByKey('playDrawing');
		if self.bPlayDrawing==nil then
			self.bPlayDrawing = true;
		end
	end
	return self.bPlayDrawing;
end

function FightManager:setPlayDrawing()
	-- body
	if self.isScaling then
		return;
	end
	if self.bPlayDrawing then
		self.bPlayDrawing = false;
	else 
		self.bPlayDrawing = true;
	end
	model.UserDefaultModel:setDataByKey('playDrawing', self.bPlayDrawing);
end


function FightManager:Pause()
	self.isPause = true;
	EventManager:FireEvent(EventType.FightPanelPause, true);
end

function FightManager:Continue()
	self.isPause = false;
	EventManager:FireEvent(EventType.FightPanelPause, false);
end 

function FightManager:setDelayDie()
	self.delayDie = true;
end
function FightManager:isDelayDie()
	return self.delayDie;
end

function FightManager:SkillPause()
	self.isSkillPause = true
end

function FightManager:SkillContinue()
	self.isSkillPause = false;
	self.delayDie = false;
	self:CheckDelayDie();
end

function FightManager:GetSkillPause()
	return self.isSkillPause;
end

function FightManager:SetPlayRoleBookState( v )
	self.isPlayRoleBookAni = v
end

--=======================================================================================
--预加载
--声音资源
--动画效果

--=======================================================================================
-- 获取关卡信息
function FightManager:GetRoundInfo(msg)
	local resid = msg.resid;
	local flag = msg.flag;
	local info = {};
	info.fightType = flag;
	if flag == FightType.Boss then
		info.roundIndex = resid;
		info.chapter = tonumber(string.sub(tostring(resid), -4, -3));
		local rowData = ResTable:getRowData('round', info.roundIndex);
		local str = (flag == 2) and 'boss' or 'monster';
		info.monsterList = rowData[str];
		info.formation = rowData.formation
		self.fightMaxTime = rowData['boss_timelimit'];
	elseif flag == FightType.RandomBoss then
		info.roundIndex = resid;
		local boss_id = model.RoundBossModel:getBossDataByPid(resid).round_bossinfo_id;
		local rowData = ResTable:getRowData('round_bossinfo', boss_id);
		info.monsterList = {{rowData.monster_id, rowData.attribute_id}}
		info.formation = {rowData.formation or Conf.FORMATION_DEFAULT_INDEX}

		info.bg = rowData.bg;
		info.scene = rowData.bg_id;
		self.fightMaxTime = MAX_CHALLENGE_TIME;
	elseif flag == FightType.GuildBoss then
		local raidMonsterData = ResTable:getRowData('guild_raid_order',model.GuildModel:getOrderNo());
		if msg.armor_hp > 0 then
			info.boss_no = raidMonsterData.monster_armor
		else 
			info.boss_no = raidMonsterData.monster_body
		end
		-- info.boss_no = msg.boss_no;
		info.boss_id = 0;--msg.boss_id;
		info.boss_hp = msg.armor_hp;--msg.boss_hp;
		self.fightMaxTime = 30--MAX_CHALLENGE_TIME;
		info.bg = Conf.arena_fight_map;
		info.scene = Conf.arena_fight_scene;
	elseif flag == FightType.TrialTower then
		info.roundIndex = resid;
		local rowData = model.TrialModel:getItemDataById(resid, msg.type)
		info.monsterList = rowData.monster;
		info.formation = rowData.formation;
		info.auto = 1;
		info.trial_type = msg.type;
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = rowData.bg;
		info.scene = rowData.bg_id;

	elseif flag == FightType.CopyRound then
		info.roundIndex = resid;
		local rowData = ResTable:getRowData('adventure_round', resid);
		info.monsterList = rowData.monster;
		info.formation = rowData.formation;
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = rowData.bg;
		info.scene = rowData.bg_id;

	elseif flag == FightType.Event then
		info.roundIndex = resid
		local fightInfo = model.EventModel:getMapMonsterInfo(resid)
		info.monsterList = {}
		for k, v in pairs(fightInfo.monster) do
			table.insert(info.monsterList,{[1] = v, [2] = fightInfo.att, 
				[3] = fightInfo.monster_hp and fightInfo.monster_hp[k] or 10000, 
				[4] = fightInfo.monster_cd and fightInfo.monster_cd[k] or 0}
			)
		end
		info.formation = ResTable:getValue('formation', fightInfo.formation, 'position')
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = Conf.event_fight_map;
		info.scene = Conf.event_fight_scene;

	elseif flag == FightType.Story then
		info.roundIndex = resid
		local hpcdList = model.StoryModel:getCurMonsterHpList();
		local itemData = ResTable:getRowData('dungeon_round', resid);
		info.monsterList = {}
		local tmpIdx = 1;
		for k, v in pairs(itemData.monster) do
			if hpcdList[tmpIdx] then
				table.insert(info.monsterList,{[1] = v[1], [2] = v[2], [3] = hpcdList[tmpIdx].hp, [4] = hpcdList[tmpIdx].cd })
			else
				table.insert(info.monsterList,{[1] = v[1], [2] = v[2], [3] = 10000, [4] = 10000 })
			end
			tmpIdx = tmpIdx+1;
		end
		info.formation = itemData.position;
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = Conf.event_fight_map;
		info.scene = Conf.event_fight_scene;

	elseif flag == FightType.Arena then
		info.roundIndex = resid
		local rankListData = model.ArenaModel:getRankList(resid);
		info.monsterList = rankListData;
		info.formation = ResTable:getValue('formation', rankListData.formation, 'position')
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = Conf.arena_fight_map;
		info.scene = Conf.arena_fight_scene;

	elseif flag == FightType.MultArena then
		info.roundIndex = resid
		info.subIndex = msg.challenge_no;
		local rankListData = model.ArenaModel:getMultArenaEnemysTeam(info.roundIndex, info.subIndex);
		info.monsterList = rankListData;
		info.formation = ResTable:getValue('formation', rankListData.formation, 'position')
		self.fightMaxTime = MAX_CHALLENGE_TIME;
		info.bg = Conf.arena_fight_map;
		info.scene = Conf.arena_fight_scene;
	else
		return {}
	end
	return info;
end

function FightManager:getRoundType()
	return self.roundInfo.fightType;
end

function FightManager:getCurrentRoundInfo()
	return self.roundInfo;
end

function FightManager:addTotalDamage( v )
	local d = autolua.opensslToLua:aes_ecb_decrypt(self.guildBossTotalDamage)
	d = tonumber(d) + v
	self.guildBossTotalDamage = autolua.opensslToLua:aes_ecb_encrypt(tostring(d))
	self.showDamage = d
end

function FightManager:getTotalDamage()
	return autolua.opensslToLua:aes_ecb_decrypt(self.guildBossTotalDamage)
end

function FightManager:getShowDamage()
	return self.showDamage
end

--角色入场
function FightManager:loadFighter()
	if self.loadKey < 7 then
		self:loadAllyPlayer(self.loadKey);
		self.loadKey = self.loadKey + 1;
	else
		self:loadMonster();
		self.loadFinish = true;
	end
end

--左方角色
function FightManager:loadAllyPlayer(pos)
	local fighter;
	for _, enterFighter in pairs(self.needLeftActorList) do
		if enterFighter:GetPositionIndex() == pos then
			fighter = enterFighter;
			fighter:SetFighterState(FighterState.idle);
			table.insert(self.leftActorList, fighter);
			--[[
			if fighter:getHpPercent() == 0 then
				fighter:SetFightBeginDie();
			end
			--]]
			break;
		end
	end
	EventManager:FireEvent(EventType.FightRoleEnter, fighter, pos);
	if fighter then
		fighter:InitSkillEffect()
	end
end

--右方角色
function FightManager:loadMonster()
	for _, enterFighter in pairs(self.needRightActorList) do
		EventManager:FireEvent(EventType.FightRoleEnter, enterFighter, enterFighter:GetPositionIndex());
		enterFighter:SetFighterState(FighterState.idle);
		table.insert(self.rightActorList, enterFighter);
		enterFighter:InitSkillEffect()
	end
end

function FightManager:GetFighter(fighter_id)
	for k, v in pairs(self.leftActorList) do
		if v:GetID() == fighter_id then
			return v;
		end
	end
	for k, v in pairs(self.rightActorList) do
		if v:GetID() == fighter_id then
			return v;
		end
	end
	if fighter_id == -1 then
		return self.Godfighter;
	end
	return nil;
end

-- 初始化本方应上场角色
function FightManager:GetAllyRoleList(formation)
	self.leftTotalHp = 0;
	local leftFighterList = {};
	local curFormation = model.TeamModel:getCurrentTeamFormation(formation)
	local teamPos = model.TeamModel:getTeamFormationPosById(curFormation);
	local teamType = model.TeamModel:getTeamTypeById(formation)

	-- get actor from team
	for i=1, TEAM_MAX_MEMBER do
		-- get role pid from team
		local data = self.team[i];
		local pid = tonumber(data.pid);
		local hp = data.hp
		
		if tonumber(pid) ~= -1 then
			local card = model.CardModel:GetCard(pid);
			local cardData = ResTable:getRowData('role', card.no);
			-- init fighter data from role data;
			local fighter = ActorManager:CreateFighter(card.no, pid);
			fighter:SetIsEnemy(false);
			fighter:SetPositionIndex(i);
			fighter:setFomationIndex(teamPos[i]);
			fighter:InitSkill(pid);
			fighter.pid = pid;
			local pro = {};
			pro.pro = CardProManager:GenProInTeam(pid, nil, nil, teamType);
			fighter:InitFightData(pro);
			fighter:setCurHp(hp/10000);
			leftFighterList[i] = fighter;
		end
	end
--	EventManager:FireEvent(EventType.RecountInit, self.team);
	return leftFighterList;
end

function FightManager:initGodFighter()
	local godskill = model.EventModel:getGodSkillList();
	if next(godskill) then
		local fighter = require("app.fight.fighter"):create(-1, 0, 0);
		fighter:initGodSkill();
		fighter:SetIsEnemy(false);
		fighter:setFomationIndex(5);
		fighter:SetGodSIcon('')
		local pro = {};
		pro.hp			= 1
		pro.atk			= 1
		pro.def			= 1
		pro.aspd		= 1;
		pro.crit		= 1;
		pro.crit_dam	= 1;
		pro.fp			= '0';
		pro.lv			= 1;
		fighter:InitFightData({pro=pro});

		self.Godfighter = fighter;
		print('dhx--god--FightManager:initGodFighter()')
	end

end

function FightManager:GetEventAllyRoleList()
	self.leftTotalHp = 0;
	local leftFighterList = {};
	local curFormation = model.TeamModel:getCurrentTeamFormation(5)
	local teamPos = model.TeamModel:getTeamFormationPosById(curFormation);

	for i=1, TEAM_MAX_MEMBER do
		-- get role pid from team
		local data = self.team[i];
		local pid = tonumber(data.pid);
		local hp = data.hp
		if hp > 0 then
			if tonumber(pid) ~= -1 then
				local card = model.CardModel:GetCard(pid);
				local cardData = ResTable:getRowData('role', card.no);
				-- init fighter data from role data;
				local fighter = ActorManager:CreateFighter(card.no, pid);
				fighter:SetIsEnemy(false);
				fighter:SetPositionIndex(i);
				fighter:setFomationIndex(teamPos[i]);
				fighter:InitSkill(pid);
				fighter.pid = pid;
				local pro = {};
				pro.pro = CardProManager:GenProInTeam(pid, nil, nil, TeamType.EVENT);
				fighter:InitFightData(pro);
				-- for k, v in pairs(buffList[fighter.fightAttribute]) do
				-- 	fighter:setPropertyCoefficient(Attribute[v.attribute], v.value);
				-- end
				-- for k, v in pairs(buffList[4]) do
				-- 	fighter:setPropertyCoefficient(Attribute[v.attribute], v.value);
				-- end
				fighter:setCurHp(hp/10000);
				leftFighterList[i] = fighter;
				fighter:initActiveSkillCD( data.cd )
			end
		end
	end
--	EventManager:FireEvent(EventType.RecountInit, self.team);
	return leftFighterList;
end

function FightManager:GetStoryAllyRoleList(tid)
	self.leftTotalHp = 0;
	local leftFighterList = {};
	local curFormation = model.TeamModel:getCurrentTeamFormation(tid)
	local teamPos = model.TeamModel:getTeamFormationPosById(curFormation);

	for i=1, TEAM_MAX_MEMBER do
		-- get role pid from team
		local data = self.team[i];
		local pid = tonumber(data.pid);
		local hp = data.hp
		if hp > 0 then
			if tonumber(pid) ~= -1 then
				local card = model.CardModel:GetCard(pid);
				local cardData = ResTable:getRowData('role', card.no);
				local fighter = ActorManager:CreateFighter(card.no, pid);
				fighter:SetIsEnemy(false);
				fighter:SetPositionIndex(i);
				fighter:setFomationIndex(teamPos[i]);
				fighter:InitSkill(pid);
				fighter.pid = pid;
				local pro = {};
				pro.pro = CardProManager:GenProInTeam(pid, nil, nil, TeamType.STORY);
				fighter:InitFightData(pro);
				-- for k, v in pairs(buffList[fighter.fightAttribute]) do
				-- 	fighter:setPropertyCoefficient(Attribute[v.attribute], v.value);
				-- end
				-- for k, v in pairs(buffList[4]) do
				-- 	fighter:setPropertyCoefficient(Attribute[v.attribute], v.value);
				-- end
				fighter:setCurHp(hp/10000);
				leftFighterList[i] = fighter;
				fighter:initActiveSkillCD( data.cd )
			end
		end
	end
	return leftFighterList;
end

function FightManager:winResult( v )
	self.isWin = v
end

function FightManager:getWin()
	return self.isWin
end

-- 初始化敌方应上场角色
function FightManager:GetMonsterList()
	self.rightTotalHp = '0';
	self.rightCurHp = '0';
	local rightFighterList = {};

	-- init fighter data from monster data;
	local restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(self.roundInfo.monsterList);
	self.roundInfo.enemyRestraintString = enemyRestraintString;
	for i=1, 9 do 
		local data = self.roundInfo.monsterList[i];
		if data and data[1] then
			-- local posData = self.roundInfo.monsterCoordination[i];
			local monsterData = ResTable:getRowData('monster', data[1]);
			local formation = self.roundInfo.formation 
			local fighter = ActorManager:CreateFighter(data[1], 0);
			fighter:SetIsEnemy(true);
			fighter:SetIsBoss(monsterData.ifboss == 1)
			fighter:SetPositionIndex(i);
			fighter:setFomationIndex(formation[i]);
			fighter:InitMonsterSkill();
			local pro = {};
			pro.pro = CardProManager:GenMonsterPro(data[1], data[2],restraintFactor);
			fighter:InitFightData(pro);
			rightFighterList[i] = fighter;
			-- fighter.coordination = {x = posData[1], y = posData[2]};

			self.rightCurHp = HpPlus(self.rightCurHp, fighter.m_propertyData:GetCurrentHP());
			self.rightTotalHp = HpPlus(self.rightTotalHp, fighter.m_propertyData:GetMaxHP());
		end
	end
	self.damageMake = '0';
	local bossPer = HpDivide(self.rightCurHp, self.rightTotalHp)
	bossPer = math.floor(bossPer*100);
	EventManager:FireEvent(EventType.BossDamageDown, '0', 0, bossPer)
	return rightFighterList;
end

function FightManager:GetGuildBoss()
	self.rightTotalHp = '0';
	self.rightCurHp = '0';
	local rightFighterList = {};

	self.roundInfo.enemyRestraintString='000';
	local data = self.roundInfo;
	-- local posData = {110, -300};
	local monsterData = ResTable:getRowData('monster', data.boss_no);
	local fighter = ActorManager:CreateFighter(data.boss_no, 0);
	fighter:SetIsEnemy(true);
	fighter:SetIsBoss(monsterData.ifboss == 1)
	fighter:SetPositionIndex(1);
	fighter:setFomationIndex(Conf.FORMATION_DEFAULT_INDEX);
	fighter:InitMonsterSkill();
	local pro = {};
	pro.pro = CardProManager:GenGuildBossPro(data.boss_id, data.boss_no);
	fighter:InitFightData(pro);
	fighter:setCurHpNum(tostring(data.boss_hp));
	rightFighterList[1] = fighter;
	-- fighter.coordination = {x = posData[1], y = posData[2]};

	self.rightCurHp = HpPlus(self.rightCurHp, fighter.m_propertyData:GetCurrentHP());
	self.rightTotalHp = HpPlus(self.rightTotalHp, fighter.m_propertyData:GetMaxHP());

	self.damageMake = '0';
	local bossPer = HpDivide(self.rightCurHp, self.rightTotalHp)
	bossPer = math.floor(bossPer*100);
	EventManager:FireEvent(EventType.BossDamageDown, '0', 0, bossPer)
	return rightFighterList;
end

function FightManager:GetEventMonsterList()
	self.rightTotalHp = '0';
	self.rightCurHp = '0';
	local rightFighterList = {};

	-- init fighter data from monster data;
	local restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(self.roundInfo.monsterList);
	self.roundInfo.enemyRestraintString = enemyRestraintString;
	for i=1, 9 do 
		local data = self.roundInfo.monsterList[i];
		if data and data[1] and data[3] > 0 then
			-- local posData = self.roundInfo.monsterCoordination[i];
			local monsterData = ResTable:getRowData('monster', data[1]);
			local formation = self.roundInfo.formation 
			local fighter = ActorManager:CreateFighter(data[1], 0);
			fighter:SetIsEnemy(true);
			fighter:SetIsBoss(monsterData.ifboss == 1)
			fighter:SetPositionIndex(i);
			fighter:setFomationIndex(formation[i]);
			fighter:InitMonsterSkill();
			local pro = {};
			pro.pro = CardProManager:GenMonsterPro(data[1], data[2],restraintFactor);
			fighter:InitFightData(pro);
			fighter:setCurHp(data[3]/10000);
			fighter:initActiveSkillCD( data[4] )
			rightFighterList[i] = fighter;
			-- fighter.coordination = {x = posData[1], y = posData[2]};

			self.rightCurHp = HpPlus(self.rightCurHp, fighter.m_propertyData:GetCurrentHP());
			self.rightTotalHp = HpPlus(self.rightTotalHp, fighter.m_propertyData:GetMaxHP());
		end
	end
	self.damageMake = '0';
	local bossPer = HpDivide(self.rightCurHp, self.rightTotalHp)
	bossPer = math.floor(bossPer*100);
	EventManager:FireEvent(EventType.BossDamageDown, '0', 0, bossPer)
	return rightFighterList;
end

function FightManager:GetArenaMonsterList()
	self.rightTotalHp = '0';
	self.rightCurHp = '0';
	local totalFp = '0';
	local rightFighterList = {};
	-- init fighter data from monster data;
	local rankListData = self.roundInfo.monsterList;

	local enemyRestraintString = nil;
	local restraintFactor = nil;
	if rankListData.isRobot == 0 then
		self.roundInfo.enemyRestraintString = '000';
	else
		local monsterList = {};
		for _k , team in pairs(self.roundInfo.monsterList.arena_team) do
			table.insert(monsterList , {team.no});
		end
		restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(monsterList);
		self.roundInfo.enemyRestraintString = enemyRestraintString;
	end

	print_ro(rankListData)
	for i=1, 9 do
		local data = rankListData.arena_team[i];
		if data and data.no ~= -1 then --玩家队伍
			local fighter = ActorManager:CreateFighter(data.no, 0);
			fighter:setAvatar(data.avatar_no);
			local formation = self.roundInfo.formation;
			local pro = {};
			if rankListData.isRobot == 0 then
				fighter:SetIsEnemy(true);
				fighter:SetArenaRoleMonsterType(Enum:getTable('ArenaRoleMonsterType').ROLE);
				--fighter:SetIsBoss(true);
				fighter:SetPositionIndex(i);
				fighter:setFomationIndex(formation[i]);
				fighter:InitSkillByBreakLevel(rankListData.pro[i].break_lv);
				pro.pro = rankListData.pro[i];
				print_ro(pro.pro)
				fighter:InitFightData(pro);
				fighter:setAutoMonster(false)
				rightFighterList[i] = fighter;
				totalFp = totalFp + pro.pro.fp;
			else --机器人
				local monsterData = ResTable:getRowData('monster', data.no);
				fighter:SetIsEnemy(true);
				fighter:SetArenaRoleMonsterType(Enum:getTable('ArenaRoleMonsterType').MONSTER);
				fighter:SetIsBoss(monsterData.ifboss == 1)
				fighter:SetPositionIndex(i);
				fighter:setFomationIndex(formation[i]);
				fighter:InitMonsterSkill();
				pro.pro = CardProManager:GenMonsterPro(data.no, data.att,restraintFactor);
				fighter:InitFightData(pro);
				fighter:setAutoMonster(true)
				rightFighterList[i] = fighter;
				totalFp = HpPlus(totalFp, ConvertDigitUnitToStr(pro.pro.fp));
			end
			fighter.uid = rankListData.uid;

			self.rightCurHp = HpPlus(self.rightCurHp, fighter.m_propertyData:GetCurrentHP());
			self.rightTotalHp = HpPlus(self.rightTotalHp, fighter.m_propertyData:GetMaxHP());
		end
	end
	self.damageMake = '0';
	local bossPer = HpDivide(self.rightCurHp, self.rightTotalHp)
	bossPer = math.floor(bossPer*100);
	EventManager:FireEvent(EventType.BossDamageDown, '0', 0, bossPer)
	return rightFighterList, totalFp;
end

function FightManager:ChangeTotalHp(args)
	local percent = 0;
	local changePercent = false;
	local curHp, maxHp = nil,nil;
	if args.isEnemy then
		self.rightCurHp = self.rightCurHp + args.hp;
		percent = self.rightCurHp / self.rightTotalHp;
		changePercent = true;
		curHp = self.rightCurHp;
		maxHp = self.rightTotalHp;
	else
		if args.pos >= 1 and args.pos <= 3 then
			self.leftCurHp = self.leftCurHp + args.hp;
			percent = self.leftCurHp / self.leftTotalHp;
			changePercent = true;
			curHp = self.leftCurHp;
			maxHp = self.leftTotalHp;
		end
	end
	if percent > 1 then
		percent = 1;
	elseif percent < 0 then
		percent = 0
	end
	if changePercent then
		EventManager:FireEvent(EventType.FightChangeShow, {percent = percent, isEnemy = args.isEnemy, cur = curHp, max = maxHp});
	end
end

function FightManager:AddAllDamageDown(damage)
	self.damageMake = HpPlus(self.damageMake, damage)
	self.rightCurHp = HpMinus(self.rightCurHp, damage)
	local per = HpDivide(self.damageMake, self.rightTotalHp)
	per = math.floor(per*10000)/100;

	local bossPer = HpDivide(self.rightCurHp, self.rightTotalHp)
	bossPer = math.floor(bossPer*100);

	EventManager:FireEvent(EventType.BossDamageDown, self.damageMake, per, bossPer)
end

function FightManager:getEnemyPosByFormaitionIndex( index )
	return rightFormationPos[index]

end

function FightManager:getRolePosByFormationIndex( index )
	return leftFormationPos[index]
end

-- 判断是否是挂机状态
function FightManager:getIsNotFight( ... )
	return self.state == FightState.None;
end

function FightManager:setWillFightSt()
	self.state = FightState.Will
end

--=======================================================================================
--战斗
function FightManager:checkTimeOver()
	local flag = self.roundInfo.fightType;
	if self.fightTime > self.fightMaxTime then
		for _, role in pairs(self.leftActorList) do
			role:SetDie();
		end
	end
end

function FightManager:ForceExitFight()
	self.fightTime = self.fightMaxTime
	self.forceEnd = true;
end

function FightManager:updateFightTime( dt )
	self.fightTime = self.fightTime + dt;
	local sec = math.floor(self.fightTime)
	if sec > self.fightSec then
		self.fightSec = sec;
		EventManager:FireEvent(EventType.FightUpdateTime, self.fightMaxTime - self.fightSec);
	end
end

function FightManager:addDeathIdx()
	self.deathIdx = self.deathIdx+1
	return self.deathIdx
end

function FightManager:checkIsOver()
	if self.OverFlag then
		return;
	end

	--战斗结束判断
	local flag = self.roundInfo.fightType;
	local leftWin = true;
	local rightWin = true;
	for _, role in pairs(self.rightActorList) do
		local pos = role:GetPositionIndex();
		if role:isAlive() then
			leftWin = false;
			break;
		end
	end
	for _, role in pairs(self.leftActorList) do
		local pos = role:GetPositionIndex();
		if role:isAlive() then
			rightWin = false;
			break;
		end
	end

	if self.setDieFlag then
		EventManager:FireEvent(EventType.FightStopActiveSkill);
		self.OverFlag = true;
		return;
	end

	if leftWin then
		for _, fighter in pairs(self.leftActorList) do
			local pos = fighter:GetPositionIndex();
			self.team[pos].hp = math.floor(fighter:getHpPercent()*10000);
			self.team[pos].cd = math.floor(model.CardModel:getEventCD(self.team[pos].pid));
			if self.team[pos].hp > 10000 then
				self.team[pos].hp = 10000;
			elseif self.team[pos].hp < 0 then
				self.team[pos].hp = 0;
			end

			if self.team[pos].cd > 10000 then
				self.team[pos].cd = 10000;
			elseif self.team[pos].cd < 0 then
				self.team[pos].cd = 0;
			end
		end
		self.OverFlag = true;
		self.result = 1;
		self.roundInfo.boss_rest_hp = "0";				

	elseif rightWin then
		for _, fighter in pairs(self.leftActorList) do
			local pos = fighter:GetPositionIndex();
			self.team[pos].hp = math.floor(fighter:getHpPercent()*10000);
			self.team[pos].cd = math.floor(model.CardModel:getEventCD(self.team[pos].pid));
			if self.team[pos].hp > 10000 then
				self.team[pos].hp = 10000;
			elseif self.team[pos].hp < 0 then
				self.team[pos].hp = 0;
			end

			if self.team[pos].cd > 10000 then
				self.team[pos].cd = 10000;
			elseif self.team[pos].cd < 0 then
				self.team[pos].cd = 0;
			end
		end
		if flag == FightType.Event or flag == FightType.Story then
			self.enemyTeam = {[1]={hp=0,cd=0}, [2]={hp=0,cd=0}, [3]={hp=0,cd=0}, [4]={hp=0,cd=0}}
			local totalHp = 0;
			local lastPos = 1;
			for _, fighter in pairs(self.rightActorList) do
				local pos = fighter:GetPositionIndex();
				local hp = math.floor(fighter:getHpPercent()*10000);
				if fighter:isAlive() and hp<=0 then
					hp = 10
				end
				totalHp = totalHp+hp
				self.enemyTeam[pos] = {hp=hp, cd=math.floor(fighter:getSaveSkillCD()*10000)}
				if fighter:getDeathIdx() == self.deathIdx then
					lastPos = pos;
				end
			end
			if totalHp==0 then
				self.enemyTeam[lastPos] = 10;
			end
		end
		if flag == FightType.GuildBoss then
			local hp = '0';
			for _, fighter in pairs(self.rightActorList) do
				hp = fighter:getHpNum();
				break;
			end
			self.roundInfo.boss_rest_hp = hp;				
		end
		self.OverFlag = true;
		self.result = 0;
	end

	if self.OverFlag then
		if (self.result == 0) then
			--战斗失败
			for _,actor in pairs(self.rightActorList) do
				actor:SetWin();
			end
		else
			--战斗胜利
			self.deathRoleNum = 0;
			for _,actor in pairs(self.leftActorList) do
				actor:SetWin();
				if not actor:isAlive() then
					self.deathRoleNum = self.deathRoleNum + 1;
				end
			end

		end
		EventManager:FireEvent(EventType.FightStopActiveSkill);
	end
end

function FightManager:setErrorMsg(pid, damage, atk, skillId, fromeType)
	self.errorMsg = {pid=pid, damage=damage, atk=atk, skillId=skillId, fromeType=fromeType}
end

function FightManager:onFightExit()

	local dieTime = self.fightTime-self.enemyFirstDieTime
	if self.fightTime <= 7 and dieTime<=1 then
		self:addFightWarning(12, dieTime)
	end
	self.enemyFirstDieTime = 0;

	AudioManager:changeSpeed(1)--战斗结束音效加速恢复
	EventManager:FireEvent(EventType.UIGuidePanel);
	local msg = {};
	local flag = self.roundInfo.fightType;
	msg.forceEnd = self.forceEnd and 1 or 0;
	if flag == FightType.Boss then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;
	elseif flag == FightType.RandomBoss then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;
		msg.operate_id = model.RoundBossModel:getDBActId();
	elseif flag == FightType.GuildBoss then
		msg.resid = 1;
		msg.flag = self.roundInfo.fightType;
		msg.boss_id = self.roundInfo.boss_id;
		msg.win = 1;--self.result;
		msg.team = self.team;
		msg.boss_hp = self.roundInfo.boss_hp;
		msg.boss_rest_hp = self.roundInfo.boss_rest_hp or self.roundInfo.boss_hp;

		msg.damage = tonumber(autolua.opensslToLua:aes_ecb_decrypt(self.guildBossTotalDamage))
		msg.raid_no = model.GuildModel:getRaidNo()
		self.guildBossTotalDamage = nil

	elseif flag == FightType.TrialTower then 
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.type = self.roundInfo.trial_type;
		msg.win = self.result;
		msg.team = self.team;
		msg.auto = 0;

	elseif flag == FightType.CopyRound then 
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;

	elseif flag == FightType.DailyRound then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;
	elseif flag == FightType.Event then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;
		-- msg.isKey = model.EventModel:getMapMonsterInfo(msg.resid).isKey
		msg.monsterTeam = self.enemyTeam
	elseif flag == FightType.Arena then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.win = self.result;
		msg.team = self.team;
		msg.season = model.ArenaModel:getSeason();
		local rankData = model.ArenaModel:getRankList(self.roundInfo.roundIndex);
		msg.fight_score = rankData.fight_score;
	elseif flag == FightType.MultArena then 
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.season = model.ArenaModel:getMultArenaSeason();
		msg.challenge_no = self.roundInfo.subIndex;
		msg.win = self.result;
		msg.team = self.team;

	elseif flag == FightType.Story then
		msg.resid = self.roundInfo.roundIndex;
		msg.flag = self.roundInfo.fightType;
		msg.storyid = model.StoryModel:getCurStoryId();
		msg.win = self.result;
		msg.team = self.team;
		msg.monsterTeam = self.enemyTeam

	end

	if self.errorMsg then
		msg.errorMsg = self.errorMsg
	end

	msg.cheatingWarning = self.cheatingWarning
	Network.NetworkManager:Send(Network.NetworkCmdType.req_exit_fight, msg);
	msg = nil
	
end

function FightManager:addFightWarning(wType, p1, p2)
	table.insert(self.cheatingWarning, {cheatType=wType, parma1=p1, parma2=p2} )
end

function FightManager:reviveFighter( isEnemy )
	-- body
	for k,v in pairs(self.deadActorList) do
		if v:IsEnemy() == isEnemy then
			v:Revive();
			table.remove(self.deadActorList, k);
			break;
		end
	end
end

function FightManager:onFighterDie(fighter)
	--del fighter
	if fighter:IsEnemy() then
		if self.enemyFirstDieTime==0 then
			self.enemyFirstDieTime = self.fightTime;
		end

		if self.fightType == FightType.Event then
			local skillId = model.EventModel:getSpBuffByType(7)
			if skillId and self.Godfighter then
				self.Godfighter:runSkillById(2, skillId)
				table.insert(self.deadActorList, fighter);
				self.delayCheckOver = 2;
				return;
			end
		end
	end
	self:InfoFighterDie(fighter);
	-- drop item if monster
end

function FightManager:InfoFighterDie(fighter)
	table.insert(self.deadActorList, fighter);
	--[[
	local actorList = {};
	if fighter:IsEnemy() then
		actorList = self.rightActorList;
	else
		actorList = self.leftActorList;
	end

	local index = nil;
	for i,role in pairs(actorList) do
		if fighter:GetID() == role:GetID() then
			index = i;
			break;
		end
	end
	if index then table.remove(actorList, index); end
	--]]

	--判断战斗是否结束
	self:checkIsOver();
end

function FightManager:IsOver()
	return self.OverFlag;
end
--=======================================================================================
--属性设置和获取
--获取战斗持续时间
function FightManager:GetTime()
	return self.time;
end

function FightManager:CheckDelayDie()
	-- body
	for k, v in pairs(self.leftActorList) do
		v:DelayDie();
	end
	for k, v in pairs(self.rightActorList) do
		v:DelayDie();
	end
end

function FightManager:AddBuffToALL( attacker, rate, buffId, skill, benemy )
	-- body
	local isEnemy = false
	local orift = attacker
	if attacker then
		isEnemy = attacker:IsEnemy();
	else
		isEnemy = benemy
		orift = self.Godfighter;
	end
	if isEnemy then
		for k, v in pairs(self.rightActorList) do
			if math.random() <= rate then
				BuffManager:AddBuff(v, orift, buffId, skill);
			end
		end
	else
		for k, v in pairs(self.leftActorList) do
			if math.random() <= rate then
				BuffManager:AddBuff(v, orift, buffId, skill);
			end
		end
	end
	
end

function FightManager:recordActiveSkillTime( fid, isEnemy, skillId)
	if isEnemy then
		if not self.rightSkillTime[ fid ] then
			self.rightSkillTime[ fid ] = {}
		end
		if self.rightSkillTime[ fid ][skillId] == nil then
			self.rightSkillTime[ fid ][skillId] = 0
		end
		self.rightSkillTime[ fid ][skillId] = self.rightSkillTime[ fid ][skillId]+1

	else
		if not self.leftSkillTime[ fid ] then
			self.leftSkillTime[ fid ] = {}
		end
		if self.leftSkillTime[ fid ][skillId] == nil then
			self.leftSkillTime[ fid ][skillId] = 0
		end
		self.leftSkillTime[ fid ][skillId] = self.leftSkillTime[ fid ][skillId]+1

	end
end

function FightManager:getSkillDataByFid( fid, isEnemy )
	-- body
	if isEnemy and self.rightResultList[ fid ] then
		return self.rightResultList[ fid ]['Skill'], self.rightSkillTime[ fid ] or {}
	elseif self.leftResultList[ fid ] then
		return self.leftResultList[ fid ]['Skill'], self.leftSkillTime[ fid ] or {}
	end
end

function FightManager:recordFightMsg( fid, isEnemy, attName, v, skill )
	-- skill 为nil，算为普工，
	if fid == -1 then --god
		return
	end
	if isEnemy then
		if not self.rightResultList[ fid ] then
			self.rightResultList[ fid ] = { ['Atk']=0, ['Dam']=0, ['Help']=0, ['Skill']={} }
		end
		self.rightResultList[ fid ][attName] = self.rightResultList[ fid ][attName] + v
		self.rightAllResult[attName] = self.rightAllResult[attName] + v

		if attName == 'Atk' or attName == 'Help' then
			local skillId = 0
			if skill and skill:GetSkillType() == SkillType.activeSkill  then
				skillId = skill:GetResID()
			end
			if self.rightResultList[ fid ]['Skill'][skillId] == nil then
				self.rightResultList[ fid ]['Skill'][skillId] = {['Atk']=0, ['Help']=0}
			end
			if attName == 'Atk' then
				self.rightResultList[ fid ]['Skill'][skillId]['Atk'] = self.rightResultList[ fid ]['Skill'][skillId]['Atk']+v
			else
				self.rightResultList[ fid ]['Skill'][skillId]['Help'] = self.rightResultList[ fid ]['Skill'][skillId]['Help']+v
			end

		end

	else
		if not self.leftResultList[ fid ] then
			self.leftResultList[ fid ] = { ['Atk']=0, ['Dam']=0, ['Help']=0, ['Skill']={} }
		end
		self.leftResultList[ fid ][attName] = self.leftResultList[ fid ][attName] + v
		self.leftAllResult[attName] = self.leftAllResult[attName] + v

		if attName == 'Atk' or attName == 'Help' then
			local skillId = 0
			if skill and skill:GetSkillType() == SkillType.activeSkill  then
				skillId = skill:GetResID()
			end
			if self.leftResultList[ fid ]['Skill'][skillId] == nil then
				self.leftResultList[ fid ]['Skill'][skillId] = {['Atk']=0, ['Help']=0}
			end
			if attName == 'Atk' then
				self.leftResultList[ fid ]['Skill'][skillId]['Atk'] = self.leftResultList[ fid ]['Skill'][skillId]['Atk']+v
			else
				self.leftResultList[ fid ]['Skill'][skillId]['Help'] = self.leftResultList[ fid ]['Skill'][skillId]['Help']+v
			end

		end

	end

end

function FightManager:getRecordFightMsg()
	return self.leftResultList, self.rightResultList, self.leftAllResult, self.rightAllResult
end

function FightManager:GetLoseReasonList( team )
	-- body
	local teamNum = 0
	local RCardNum = 0
	local SSRAllCardNum = true
	local LvUpPid = -1
	local BreakUpPid = -1
	local StarUpPid = -1
	local ret = {[1]=0,[2]=0,[3]=0,[4]=0,[5]=0,[6]=0,[7]=0}

	for i=1, TEAM_MAX_MEMBER do

		local pid = team[i]
		if pid == -1 or (pid>=5000 and pid<6000) then
		else
			teamNum = teamNum+1
			local cardData = model.CardModel:GetCard(pid);
			-- local roleInfoData = ResTable:getRowData('role_info', cardData.no)
			if cardData.quality <= 3 then
				RCardNum = RCardNum + 1
			end

			if cardData.quality <= 4 then --低于SSR品质时
				SSRAllCardNum = false
			end

			local retType = model.CardModel:getCanLevelUpAndSoOn(pid);
			if LvUpPid == -1 and retType == 1 then
				LvUpPid = pid
			end

			if LvUpPid == -1 and BreakUpPid == -1 and retType == 2 then
				BreakUpPid = pid
			end

			if LvUpPid == -1 and BreakUpPid == -1 and StarUpPid == -1 and retType == 3 then
				StarUpPid = pid
			end

		end

	end

	local isEnough = 0
	if teamNum < 4 then --请上满4名角色再进行挑战吧
		ret[1] = 1
		isEnough = isEnough + 1
	else
		local rightPer = HpDivide(self.rightCurHp, self.rightTotalHp)
		rightPer = math.floor(rightPer*100);
		if rightPer < 3 then --敌方剩余血量不足3%
			ret[2] = 1
			isEnough = isEnough + 1
		end
	end

	if RCardNum >= 2 then --两个R卡及以上
		ret[3] = 1
		isEnough = isEnough + 1
	end

	if LvUpPid > -1 then
		ret[4] = LvUpPid
		isEnough = isEnough + 1
	end

	if BreakUpPid > -1 and LvUpPid==-1 then
		ret[5] = BreakUpPid
		isEnough = isEnough + 1
	end

	if StarUpPid > -1 and LvUpPid==-1 and BreakUpPid==-1 then
		ret[6] = StarUpPid
		isEnough = isEnough + 1
	end

	if RCardNum < 2 and SSRAllCardNum==false then --不全是ssr
		ret[7] = 1
		isEnough = isEnough + 1
	end

	if isEnough<3 then
		ret[2] = 1
		ret[7] = 1
	-- elseif isEnough==1 then
	-- elseif isEnough==2 then
	end

	return ret

end

--=======================================================================================
--====================================================================
--目标选取
function FightManager:GetTargeterList(fighter, isEnemy, target_area, target_cond)
	local row_1 = {1, 2, 3};
	local row_2 = {4, 5, 6};
	local row_3 = {7, 8, 9};
	local col_1 = {1, 4, 7};
	local col_2 = {2, 5, 8};
	local col_3 = {3, 6, 9};
	local rev_col_1 = {	[1] = true, [4] = true, [7] = true }
	local rev_col_2 = {	[2] = true, [5] = true, [8] = true }
	local rev_col_3 = {	[3] = true, [6] = true, [9] = true }

	local aimActorList = {};
	target_area = target_area or {1, 1, 1};
	target_cond = target_cond or {1, 0, 0};

	if type(target_cond) ~= 'table' then
		target_cond = {1, 0, 0}
	end

	if target_area[1] == -1 then
		table.remove(target_area, 1);
	end
	if target_cond[1] == -1 then
		table.remove(target_cond, 1);
	end

	if isEnemy then
		if target_area[1] == 1 then
			aimActorList = self.leftActorList;
		elseif target_area[1] == 0 then
			aimActorList = self.rightActorList;
		end
	else
		if target_area[1] == 1 then
			aimActorList = self.rightActorList;
		elseif target_area[1] == 0 then
			aimActorList = self.leftActorList;
		end
	end
	local actorList = {};
	for k, v in pairs(aimActorList) do
		if v.m_fighterstate ~= FighterState.death or v.m_isDelayDie==true then
			actorList[v:getFormationIndex()] = v;
		end
	end

	if actorList == {} then
		return actorList;
	end

	local cond_select = false;
	local targetList = {};
	if target_area[1] == 0 then
		if target_area[3] == 0 then
			if target_area[2] == 0 then
				-- 0 0 0
				cond_select = true;
				for k, v in pairs(actorList) do
					table.insert(targetList, v);
				end

			elseif target_area[2] == 1 then
				-- 0 1 0 
				cond_select = true;
				local hasSelect = false
				for i=1, #row_1 do
					if actorList[row_1[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_1[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_2 do
						if actorList[row_2[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_2[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			elseif target_area[2] == 3 then
				-- 0 3 0 
				cond_select = true;
				local hasSelect = false
				for i=1, #row_2 do
					if actorList[row_2[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_2[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_1 do
						if actorList[row_1[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_1[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			elseif target_area[2] == 2 then
				-- 0 2 0 
				cond_select = true;
				local hasSelect = false
				for i=1, #row_3 do
					if actorList[row_3[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_3[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_1 do
						if actorList[row_1[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_1[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_2 do
						if actorList[row_2[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_2[i]]);
						end
					end
				end

			elseif target_area[2] == 4 then
				-- 0 4 0 
				cond_select = true;
				local hasSelect = false
				for i=1, #row_1 do
					if actorList[row_1[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_1[i]]);
					end
				end
				for i=1, #row_2 do
					if actorList[row_2[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_2[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end

		elseif target_area[3] == 1 then
			if target_area[2] == 0 then
				-- 0 0 1
				for k, v in pairs(actorList) do
					if v:getFormationIndex() == fighter:getFormationIndex() then
						table.insert(targetList, v);
						break;
					end
				end

			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end

		elseif target_area[3] == 2 then
			if target_area[2] == 0 then
				-- 0 0 2
				cond_select = true;
				for k, v in pairs(actorList) do
					if v:getFormationIndex() ~= fighter:getFormationIndex() then
						table.insert(targetList, v);
					end
				end

				if #targetList == 0 then
					return {};
				end

			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end

		else
			print('error!', json.encode(target_area))
			print(debug.traceback())
			return {};
		end

	elseif target_area[1] == 1 then
		if target_area[3] == 0 then
			if target_area[2] == 0 then
				-- 1 0 0
				cond_select = true;
				for k, v in pairs(actorList) do
					table.insert(targetList, v);
				end
				
			elseif target_area[2] == 1 then
				-- 1 1 0
				cond_select = true;
				local hasSelect = false
				for i=1, #row_1 do
					if actorList[row_1[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_1[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_2 do
						if actorList[row_2[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_2[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			elseif target_area[2] == 3 then
				-- 1 3 0 
				cond_select = true;
				local hasSelect = false
				for i=1, #row_2 do
					if actorList[row_2[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_2[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_1 do
						if actorList[row_1[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_1[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			elseif target_area[2] == 2 then
				-- 1 2 0
				cond_select = true;
				local hasSelect = false
				for i=1, #row_3 do
					if actorList[row_3[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_3[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_1 do
						if actorList[row_1[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_1[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #row_2 do
						if actorList[row_2[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_2[i]]);
						end
					end
				end

			elseif target_area[2] == 4 then
				-- 1 4 0
				cond_select = true;
				local hasSelect = false
				for i=1, #row_1 do
					if actorList[row_1[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_1[i]]);
					end
				end
				for i=1, #row_2 do
					if actorList[row_2[i]] then
						hasSelect = true
						table.insert(targetList, actorList[row_2[i]]);
					end
				end
				if not hasSelect then
					for i=1, #row_3 do
						if actorList[row_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[row_3[i]]);
						end
					end
				end

			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end

		elseif target_area[3] == 1 then
			-- 1 对位
			-- 2 前排对位
			-- 3 1 2 3
			-- 4 中排对位
			-- 5 4 5 6
			-- 6 后排对位
			-- 7 7 8 9
			local pos = fighter:getFormationIndex();
			local col
			if pos == 1 or pos == 4 or pos == 7 then
				col = col_1
			elseif pos == 2 or pos == 5 or pos == 8 then
				col = col_2
			elseif pos == 3 or pos == 6 or pos == 9 then
				col = col_3
			else
				print('error!', json.encode(target_area), pos)
				print(debug.traceback())
				return {};
			end

			local hasSelect = false
			if target_area[2] == 1 then
				-- 1 1 1
				if actorList[col[1]] then
					table.insert(targetList, actorList[col[1]]);
					hasSelect = true
				end
			elseif target_area[2] == 2 then
				-- 1 2 1
				if actorList[col[3]] then
					table.insert(targetList, actorList[col[3]]);
					hasSelect = true
				end
			elseif target_area[2] == 3 then
				-- 1 3 1
				if actorList[col[2]] then
					table.insert(targetList, actorList[col[2]]);
					hasSelect = true
				end
			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end

			if actorList[col[1]] and not hasSelect then
				table.insert(targetList, actorList[col[1]]);
				hasSelect = true
			end
			if not hasSelect then
				for i=1, 3 do
					if actorList[row_1[i]] then
						table.insert(targetList, actorList[row_1[i]]);
						hasSelect = true
						break
					end
				end
			end
			if actorList[col[2]] and not hasSelect then
				table.insert(targetList, actorList[col[2]]);
				hasSelect = true
			end
			if not hasSelect then
				for i=1, 3 do
					if actorList[row_2[i]] then
						table.insert(targetList, actorList[row_2[i]]);
						hasSelect = true
						break
					end
				end
			end
			if actorList[col[3]] and not hasSelect then
				table.insert(targetList, actorList[col[3]]);
				hasSelect = true
			end
			if not hasSelect then
				for i=1, 3 do
					if actorList[row_3[i]] then
						table.insert(targetList, actorList[row_3[i]]);
						hasSelect = true
						break
					end
				end
			end

		elseif target_area[3] == 2 then
			if target_area[2] == 0 then
				-- 1 0 2
				cond_select = true;
				local hasSelect = false
				local pos = fighter:getFormationIndex();
				local col
				if pos == 1 or pos == 4 or pos == 7 then
					col = col_1
				elseif pos == 2 or pos == 5 or pos == 8 then
					col = col_2
				elseif pos == 3 or pos == 6 or pos == 9 then
					col = col_3
				else
					print('error!', json.encode(target_area), pos)
					print(debug.traceback())
					return {};
				end

				for i=1, #col do
					if actorList[col[i]] then
						hasSelect = true
						table.insert(targetList, actorList[col[i]]);
					end
				end
				if not hasSelect then
					for i=1, #col_1 do
						if actorList[col_1[i]] then
							hasSelect = true
							table.insert(targetList, actorList[col_1[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #col_2 do
						if actorList[col_2[i]] then
							hasSelect = true
							table.insert(targetList, actorList[col_2[i]]);
						end
					end
				end
				if not hasSelect then
					for i=1, #col_3 do
						if actorList[col_3[i]] then
							hasSelect = true
							table.insert(targetList, actorList[col_3[i]]);
						end
					end
				end

			else
				print('error!', json.encode(target_area))
				print(debug.traceback())
				return {};
			end
		else
			print('error!', json.encode(target_area))
			print(debug.traceback())
			return {};
		end
	else
		print('error!', json.encode(target_area))
		print(debug.traceback())
		return {};
	end

	-- 0 random
	-- 1 highest hp value
	-- 2 lowest hp value
	-- 3 highest hp percent
	-- 4 lowest hp percent
	-- 5 have positive buff
	-- 6 have negative buff
	local sum = 0;
	for k, v in pairs(targetList) do
		sum = sum + 1;
	end
	if cond_select and target_cond[2] and sum > target_cond[2] and target_cond[2] < TEAM_MAX_MEMBER then
		local number = target_cond[2];
		if target_cond[1] == 0 then
			-- 0 random
			local chooseList = {};
			for i=1, number do
				local key = math.random(1, sum);
				local count = 0;
				for k, v in pairs(targetList) do
					count = count + 1;
					if count == key then
						sum = sum - 1;
						table.insert(chooseList, v);
						targetList[k] = nil;
						break;
					end
				end
			end
			targetList = chooseList;
		elseif target_cond[1] == 1 then
			-- 1 highest hp value
			local chooseList = {};
			for i=1, number do
				local mark = 0;
				local ContrastValue = nil;
				for k, v in pairs(targetList) do
					if not ContrastValue then
						ContrastValue = v.m_propertyData:GetCurrentHP();
						mark = k;
					elseif HpCompare(v.m_propertyData:GetCurrentHP(), '>', tostring(ContrastValue)) then
						ContrastValue = v.m_propertyData:GetCurrentHP();
						mark = k;
					end
				end
				table.insert(chooseList, targetList[mark]);
				targetList[mark] = nil;
			end
			targetList = chooseList;
		elseif target_cond[1] == 2 then
			-- 2 lowest hp value
			local chooseList = {};
			for i=1, number do
				local mark = 0;
				local ContrastValue = nil;
				for k, v in pairs(targetList) do
					if not ContrastValue then
						ContrastValue = v.m_propertyData:GetCurrentHP();
						mark = k;
					elseif HpCompare(v.m_propertyData:GetCurrentHP(), '<', tostring(ContrastValue)) then
						ContrastValue = v.m_propertyData:GetCurrentHP();
						mark = k;
					end
				end
				table.insert(chooseList, targetList[mark]);
				targetList[mark] = nil;
			end
			targetList = chooseList;
		elseif target_cond[1] == 3 then
			-- 3 highest hp percent
			local chooseList = {};
			for i=1, number do
				local mark = 0;
				local ContrastValue = nil;
				for k, v in pairs(targetList) do
					if not ContrastValue then
						ContrastValue = v:getHpPercent();
						mark = k;
					elseif v:getHpPercent() > ContrastValue then
						ContrastValue = v:getHpPercent();
						mark = k;
					end
				end
				table.insert(chooseList, targetList[mark]);
				targetList[mark] = nil;
			end
			targetList = chooseList;
		elseif target_cond[1] == 4 then
			-- 4 lowest hp percent
			local chooseList = {};
			for i=1, number do
				local mark = 0;
				local ContrastValue = nil;
				for k, v in pairs(targetList) do
					if not ContrastValue then
						ContrastValue = v:getHpPercent();
						mark = k;
					elseif v:getHpPercent() < ContrastValue then
						ContrastValue = v:getHpPercent();
						mark = k;
					end
				end
				table.insert(chooseList, targetList[mark]);
				targetList[mark] = nil;
			end
			targetList = chooseList;
		elseif target_cond[1] == 5 then
			-- 5 have positive buff
			local chooseList = {};
			local count = 0;
			for k, v in pairs(targetList) do
				if BuffManager:checkPositiveBuff(v.id) then
					table.insert(chooseList, v);
					count = count + 1;
				end
				if count >= number then
					break;
				end
			end
			targetList = chooseList;
		elseif target_cond[1] == 6 then
			-- 6 have negative buff
			local chooseList = {};
			local count = 0;
			for k, v in pairs(targetList) do
				if BuffManager:checkNegativeBuff(v.id) then
					table.insert(chooseList, v);
					count = count + 1;
				end
				if count >= number then
					break;
				end
			end
			targetList = chooseList;
		elseif target_cond[1] == 8 then
			-- 8 select atk top 
			local chooseList = {};
			for i=1, number do
				local mark = 0;
				local ContrastValue = nil;
				for k, v in pairs(targetList) do
					if not ContrastValue then
						ContrastValue = v.m_propertyData:GetCurrentPropertyData('atk');
						mark = k;
					elseif HpCompare(v.m_propertyData:GetCurrentPropertyData('atk'), '>', tostring(ContrastValue)) then
						ContrastValue = v.m_propertyData:GetCurrentPropertyData('atk');
						mark = k;
					end
				end
				table.insert(chooseList, targetList[mark]);
				targetList[mark] = nil;
			end
			targetList = chooseList;
		end
	end

	if target_cond[1] == 7 then
		-- 7 red green blue
		local chooseList = {};
		local count = 0;
		for k, v in pairs(targetList) do
			if v.fightAttribute == target_cond[3] then
				table.insert(chooseList, v);
				count = count + 1;
			end
			if count >= target_cond[2] then
				break;
			end
		end
		targetList = chooseList;
	end

	return targetList;
end

cc.exports.FightManager = FightManager:create();
