local FightSkillManager = class("FightSkillManager")
local SkillType = Enum:getTable("SkillType");
local scheduler = cc.Director:getInstance():getScheduler();

function FightSkillManager:ctor()
	self.skillList = {};
	self.fighterState = {};--记录fighter 自己的时间轴，cd跳过的 状态值，暂停的状态值
	self.allySkillWiteList = {}
	self.enemySkillWiteList = {}
end

function FightSkillManager:Initialize()
	self.time = 0;
	self.isPause = false;
	self.nextActiveSkillCastTime = 0
	self.nextEnemyActiveSkillCastTime = 0
end

function FightSkillManager:Reset()
	self.skillList = {};
	self.fighterState = {};
	self.allySkillWiteList = {}
	self.enemySkillWiteList = {}
end

function FightSkillManager:ResetFighter(fighter_id)
	if self.skillList[fighter_id] then
		self.skillList[fighter_id] = nil;
		self.fighterState[fighter_id] = nil;
	end
end

function FightSkillManager:Update( dt )
	if self.isPause then
		return;
	end
	self.time = self.time + dt;
	for k, v in pairs(self.skillList) do
		local fighter = FightManager:GetFighter(k);
		if fighter and fighter:GetID()~=-1 then--and not fighter:IsEnemy()
			if self.time < self.fighterState[k].stop then --眩晕，进度条停止，主时间继续累加，则技能上次释放时间也继续累加 保证同步
				for k1, v1 in pairs(v.skillLastTime) do
					v.skillLastTime[k1] = v1 + dt;
				end
			else
				for k1, v1 in pairs(v.skillLastTime) do
					if k1 == fighter:getActiveSkill():GetResID() then
						local cdPercent = 0;
						local lastTime = self.time - v1;
						lastTime = lastTime > 0 and lastTime or 0;
						local cd = fighter:getActiveSkill():GetCastCD();
						if cd > 0 then
							cdPercent = lastTime / cd;
						end
						if self.time < self.fighterState[k].jump and cdPercent>=1 then --沉默 cd到了 但不释放技能，重新设置cd
							self.skillList[k].skillLastTime[k1] = self.time;
							cdPercent = 0;
						end
						fighter:saveSkillCD(cdPercent)
						EventManager:FireEvent(EventType.FightUpdateSkillIcon, fighter, cdPercent)
						break;
					end
				end
			end

			
		end
	end
end

function FightSkillManager:Pause()
	self.isPause = true;
end

function FightSkillManager:Continue()
	self.isPause = false;
end

function FightSkillManager:AddSkill(fighter_id, skill)
	if skill:GetSkillType()==SkillType.passive4Skill or skill:GetSkillType()==SkillType.passive5Skill then
		--觉醒技能 4 和 5类型的 不加入队列
		return
	end
	if skill.m_skillType == SkillType.activeSkill then
		local fighter = FightManager:GetFighter(fighter_id);
		if fighter:IsEnemy() then
			self.nextEnemyActiveSkillCastTime = self.time + 0.03;
		else
			self.nextActiveSkillCastTime = self.time + 0.03;
			if (self.time - self.skillList[fighter_id].skillLastTime[skill:GetResID()]) <=1 then
				FightManager:addFightWarning(19, skill:GetResID())
			end
		end
		self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.time;

		fighter:saveSkillCD(0)
		EventManager:FireEvent(EventType.FightUpdateSkillIcon, fighter, 0)
	else
		self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.time;
	end
	
end

function FightSkillManager:reduceSkillCastCD(fighter_id, skill, cd)
	self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.skillList[fighter_id].skillLastTime[skill:GetResID()] - cd;
	self:immRefreshCD(fighter_id, self.skillList[fighter_id].skillLastTime[skill:GetResID()])
end

function FightSkillManager:immRefreshCD(fighter_id, curCd)
	local fighter = FightManager:GetFighter(fighter_id);
	if fighter then--and not fighter:IsEnemy()
		local cdPercent = 0;
		local lastTime = self.time - curCd;
		lastTime = lastTime > 0 and lastTime or 0;
		local cd = fighter:getActiveSkill():GetCastCD();
		if cd > 0 then
			cdPercent = lastTime / cd;
		end
		EventManager:FireEvent(EventType.FightUpdateSkillIcon, fighter, cdPercent)		
	end
	-- body
end

function FightSkillManager:CanSkillCast(fighter_id, skill)

	local canCast = false;

	local lastTime = self.time - self.skillList[fighter_id].skillLastTime[skill:GetResID()];
	local cd = skill:GetCastCD();
	canCast = (lastTime > cd);

	if skill.m_skillType == SkillType.activeSkill and canCast then

		if self.time < self.fighterState[fighter_id].stop then
			return false;
		end
		if self.time < self.fighterState[fighter_id].jump then --沉默 cd到了 但不释放技能，重新设置cd
			self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.time;
			self:immRefreshCD(fighter_id, self.time)
			return false
		end

		--检查是否已经再等待队列里了
		local bInWite = 0
		local nKey = fighter_id..'_'..skill:GetResID()
		local fighter = FightManager:GetFighter(fighter_id);
		if skill.m_skillType == SkillType.activeSkill then
			if fighter:IsEnemy() then
				for i,v in ipairs(self.enemySkillWiteList) do
					if v == nKey then
						bInWite = i
						break
					end
				end
			else
				for i,v in ipairs(self.allySkillWiteList) do
					if v == nKey then
						bInWite = i
						break
					end
				end
			end
		end
		--检查是否已经再等待队列里了

		if fighter:IsEnemy() then
			if self.time < self.nextEnemyActiveSkillCastTime then
				if canCast and bInWite == 0 then
					table.insert(self.enemySkillWiteList, nKey)
				end
				return false
			end
		else
			if self.time < self.nextActiveSkillCastTime then
				if canCast and bInWite == 0 then
					table.insert(self.allySkillWiteList, nKey)
				end
				return false
			end
		end

	--已经渡过公共cd了
		if bInWite == 0 then
			--不存在于等待队列里，但等待队列里 有其他 技能等待释放，则 进入等待队列
			if canCast then 
				if fighter:IsEnemy() then
					if #self.enemySkillWiteList > 0 then
						table.insert(self.enemySkillWiteList, nKey)
						return false
					end
				else
					if #self.allySkillWiteList > 0 then
						table.insert(self.allySkillWiteList, nKey)
						return false
					end
				end
			end
		else
			--存在于等待队列里，且已经渡过 公共cd时间
			if canCast then --从等待队列里删除
				if fighter:IsEnemy() then
					table.remove( self.enemySkillWiteList , bInWite )
				else
					table.remove( self.allySkillWiteList , bInWite )
				end
			end
		end
	end

	return canCast;
end

function FightSkillManager:InitSkill(fighter_id, skill)
	if not self.skillList[fighter_id] then
		self.skillList[fighter_id] = {};
		self.skillList[fighter_id].skillLastTime = {};

		self.fighterState[fighter_id] = {};
		-- self.fighterState[fighter_id].time = self.time;
		self.fighterState[fighter_id].jump = 0;
		self.fighterState[fighter_id].stop = 0;

	end
	if not self.skillList[fighter_id].skillLastTime[skill:GetResID()] then
		self.skillList[fighter_id].skillLastTime[skill:GetResID()] = 0;
	end
end

function FightSkillManager:initActiveSkillCD(fighter, skill_id, cd)
	-- body
	if self.skillList[fighter:GetID()].skillLastTime[skill_id] then
		local cdTime = fighter:getActiveSkill():GetCastCD();
		self.skillList[fighter:GetID()].skillLastTime[skill_id] = -(cdTime*(cd/10000))
	end
end

function FightSkillManager:StopFighterSkill( fighter_id, cd )
	self.fighterState[fighter_id].stop = self.time + cd;
end

function FightSkillManager:JumpFighterSkill( fighter_id, cd )
	self.fighterState[fighter_id].jump = self.time + cd;
end

function FightSkillManager:addSkillCastCD( fighter_id, skill, cd, pro )
	local proTime = pro > 0 and (skill:GetCastCD()*pro) or 0;
	self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.skillList[fighter_id].skillLastTime[skill:GetResID()] + cd + proTime;
	local lastTime = self.time - self.skillList[fighter_id].skillLastTime[skill:GetResID()];
	if lastTime < 0 then --如果加cd的时间 上次释放时间 晚于 当前时间，则设置为当前时间
		self.skillList[fighter_id].skillLastTime[skill:GetResID()] = self.time
	end
	self:immRefreshCD(fighter_id, self.skillList[fighter_id].skillLastTime[skill:GetResID()])
end

function FightSkillManager:JingHuaBuff( fighter_id )
	self.fighterState[fighter_id].jump = 0;
	self.fighterState[fighter_id].stop = 0;
end

function FightSkillManager:reviveFighterSkill( fighter_id )
	-- body
	for k,v in pairs( self.skillList[fighter_id].skillLastTime ) do
		v = self.time;
	end
	self.fighterState[fighter_id].jump = 0;
	self.fighterState[fighter_id].stop = 0;
end

cc.exports.FightSkillManager = FightSkillManager:create();
