--fighter.lua
--============================================================================
local Fighter = class("Fighter", require("app.actor.Actor"))
local FighterState = Enum:getTable('FighterState');
local DirectionType = Enum:getTable("DirectionType");
local AnimationType = Enum:getTable('AnimationType');
local SkillType = Enum:getTable('SkillType');
local AttackState = Enum:getTable('AttackState');
local CastType = Enum:getTable('CastType');
-- local Adapter = Enum:getTable('Adapter');
local RestrainType = Enum:getTable("FightRestraintType")
local FighterSpecialStateType = Enum:getTable('FighterSpecialStateType');
local BuffType = Enum:getTable('BuffType')
local ArenaRoleMonsterType = Enum:getTable('ArenaRoleMonsterType');
local FightType = Enum:getTable('FightType');
local cvTypeEnum = Enum:getTable("cvTypeEnum");

function Fighter:ctor(id, resid, pid)
	Fighter.super.ctor(self, id, resid, pid)
	self.m_propertyData			= require("app.fight.fighterPropertyData"):create(self); -- 角色基本属性类

	-- 其他战斗相关属性
	-- 类别相关
	self.m_actorType         = 0;                             -- 角色类型(伙伴、怪物、boss)
	self.m_isEnemy           = false;                         -- 是否敌方角色

	-- 站位相关
	self.m_standArea          = 1;                             -- 站立区

	-- 战斗相关
	self.m_teamIndex         = 0;                             -- 角色的出场顺序
	self.m_curAttackLastTime = 0;                             -- 该次攻击的持续时间
	self.m_attackCount       = 0;                             -- 本次战斗中的攻击次数
	self.m_goodsIDList       = {};                            -- 角色死亡时掉落物品的id列表

	-- 技能
	self.m_skillList         = {};                            -- 角色技能列表
	self.m_curSkill          = nil;                           -- 当前释放的技能

	-- 技能展示
	self.m_RunningState      = false;                         -- 无视战斗运行状态，如果为true更新，false则根据战斗运行状态
	self.m_ZOrder            = 0;
	self.m_HaveBringToFront  = false;
	self.m_IsShowSkill       = false;                         -- 是否处于技能展示过程

	--颜色
	self.fightAttribute = 0; --角色的颜色
	--职业
	self.job = 0; --角色职业

	-- buff
	self.m_buffItemList      = {};                            -- 角色身上的buff

	--个人光环效果
	self.m_haloBuffIDList = {};                               -- 自带的被动效果，上场触发不可驱散

	-- 特殊状态效果
	self.m_specialStateList = {};

	-- 特效结束回调
	self.m_effectCallbackList = {};

	-- buff list
	self.buffList = {};
	--
	self.m_isEnemy = false;

	self.m_isBoss = false;

	self.m_isDelayDie = false;

	self.m_willCalcu = false;
	self.m_willCalcuStList = {};
	self.m_willCalcuDam = 0;
	self.m_willCalcuDamEff = 0;
	self.releaseSkill = nil;
	self.juexingSkill = nil;

	--arena人怪
	self.m_arenaRoleMonsterType = ArenaRoleMonsterType.NONE; --0无状态，1人，2怪
	-- 等级
	self.m_lv = 0;

	self.m_isRoleBookMonster = 0;
	self.activeRelease = false

	self.isAutoMonster = true
	self.saveCD = 0;
	self.deathIdx = 0;
end

function Fighter:setAutoMonster(v)
	self.isAutoMonster = v
end

function Fighter:getIsAutoMonster()
	return self.isAutoMonster;
end

function Fighter:setFightLv( lv )
	self.m_lv = lv or 0;
end

function Fighter:getFightLv( lv )
	return self.m_lv ;
end

function Fighter:SetIsEnemy(flag)
	self.m_isEnemy = flag;
end

function Fighter:SetArenaRoleMonsterType(t)
	self.m_arenaRoleMonsterType = t;
end

function Fighter:GetArenaRoleMonsterType()
	return self.m_arenaRoleMonsterType
end

function Fighter:GetJob()
	return self.job;
end

function Fighter:GetColor()
	return self.fightAttribute;
end

function Fighter:SetIsBoss(flag)
	self.m_isBoss = flag;
end

function Fighter:SetPositionIndex( pos )
	self.positionIndex = pos;
end

function Fighter:GetPositionIndex( pos )
	return self.positionIndex;
end

function Fighter:setFomationIndex( pos )
	self.formationIndex = pos;
	if pos == 1 or pos == 2 or pos == 3 then
		self.martixIdx = 1;
	elseif pos == 4 or pos == 5 or pos == 6 then
		self.martixIdx = 2;
	else
		self.martixIdx = 3;
	end
end

function Fighter:getMartixIdx()
	return self.martixIdx;
end

function Fighter:getFormationIndex(  )
	return self.formationIndex;
end

--初始化角色形象（要在战斗预加载结束之后才能执行）
function Fighter:genFightArmature(facedir)
	if self.m_arenaRoleMonsterType == ArenaRoleMonsterType.ROLE then
		self:genArmature(DirectionType.left)
	elseif self.m_isEnemy then
		self:genMonArmature(DirectionType.left);
	else
		self:genArmature(DirectionType.right)
	end
	self:SetAnimation(AnimationType.idle);
	self:setCompleteCallback()
	self.armatureNode.id = self.id;
	return self.armatureNode;
end

function Fighter:InitFightData(data)
	self.m_propertyData:InitPlayerFightData(data);
	if data and data.pro and data.pro.lv then
		self:setFightLv(data.pro.lv);
	end
end

function Fighter:setPropertyCoefficient(property, coefficient)
	self.m_propertyData:setPropertyCoefficient(property, coefficient);
end

function Fighter:setCurHp(percent)
	self.m_propertyData:setCurHp(percent);
end

function Fighter:setCurHpNum(num)
	self.m_propertyData:setCurHpNum(num);
end

function Fighter:getHpNum()
	return self.m_propertyData:getHpNum();
end

function Fighter:getHpPercent()
	return self.m_propertyData:getHpPercent();
end

function Fighter:getWillHpPercent(missVal)
	return self.m_propertyData:getWillHpPercent(missVal);
end

function Fighter:setEventPower()
	self.m_propertyData:setEventPower();
end

function Fighter:InitSkill(pid)
	local cardData = model.CardModel:GetCard(pid);
	self:InitSkillByBreakLevel(cardData.break_lv);
end

function Fighter:InitSkillByBreakLevel(break_lv)
	local roleData = ResTable:getRowData('role', self.resID)
	local skillItemId = 0;
	if self.avatar_no and self.avatar_no ~= 0 then
		skillItemId = ResTable:getValue('role_avatar', self.avatar_no, 'skill_id')
	elseif self.pid and self.pid > 0 then
		skillItemId = model.CardModel:getCardSkillIdByPid(self.pid);
	end
	if skillItemId == 0 then
		skillItemId = roleData.skill_id;
	end
	local data = ResTable:getRowData('role_skill', skillItemId)

	local skillTable = {'skill1', 'skill2'};
	--local break_lv = cardData.break_lv;
	for k, v in pairs(ResTable:getTable('role_grade').contain) do
		if break_lv >= v.grade and v.skill_open then
			table.insert(skillTable, v.skill_open)
		end
	end

	for k, v in pairs(skillTable) do
		local skill_id = data[v .. '_no'];
		if skill_id then
			local skillID = skill_id;
			local skillData = ResTable:getRowData('skill', skillID);
			if skillData then
				local skillType = skillData['skill_type'];
				local skillItem = nil;
				if skillType == 0 then
					if FightManager:getFightType() == FightType.Event then
						skillID = model.EventModel:getReplaceSkill(self.resID ,0) or skillID;
					end
					skillItem = require("app.fight.fightNormalSkill"):create(self, skillID);
					skillItem:SetScriptID(data[v .. '_scr']);
				elseif skillType == 1 then
					if FightManager:getFightType() == FightType.Event then
						skillID = model.EventModel:getReplaceSkill(self.resID ,1) or skillID;
					end
					skillItem = require("app.fight.fightActiveSkill"):create(self, skillID);
					skillItem:SetScriptID(data[v .. '_scr']);
					self.activeSkill = skillItem;
				elseif skillType == 2 then
					if FightManager:getFightType() == FightType.Event then
						skillID = model.EventModel:getReplaceSkill(self.resID ,2) or skillID;
					end
					--local inTeam, pos = model.TeamModel:isRoleInTeam(pid);
					for k, v in pairs(skillData['seat']) do
						if self.formationIndex == v then
							skillItem = require("app.fight.fightPassiveSkill"):create(self, skillID);
							break;
						end
					end
				elseif skillType==4 or skillType==5 then
					if FightManager:getFightType() == FightType.Event then
						skillID = model.EventModel:getReplaceSkill(self.resID ,4) or skillID;
					end
					self:initJuexingSkill(skillID)
				end
				if skillItem then
					if not self.m_skillList[skillType] then
						self.m_skillList[skillType] = {};
					end
					if skillItem then
						table.insert(self.m_skillList[skillType], skillItem);
						FightSkillManager:InitSkill(self:GetID(), skillItem);
					end
				end
				
			end

		end
	end

	-- set fight attribute
	self.fightAttribute = ResTable:getValue('role_pre', roleData.role_id, 'def_type');
	self.job = ResTable:getValue('role_pre', roleData.role_id, 'job_type');
end

function Fighter:initGodSkill()
	self.fightAttribute =1
	local godSkillList = model.EventModel:getGodSkillList();
	for k,v in pairs(godSkillList) do
		local skillID = v[1];
		print('dhx--god--skill:', v[1], v[2])
		local skillData = ResTable:getRowData('skill', skillID);
		if skillData then
			local skillType = skillData['skill_type'];
			local skillItem = nil;
			if skillType == 1 then
				skillItem = require("app.fight.fightActiveSkill"):create(self, skillID);
				skillItem:SetScriptID(v[2]);
				self.activeSkill = skillItem;
			elseif skillType == 2 then
				skillItem = require("app.fight.fightPassiveSkill"):create(self, skillID);
			end
			if skillItem then
				if not self.m_skillList[skillType] then
					self.m_skillList[skillType] = {};
				end
				if skillItem then
					table.insert(self.m_skillList[skillType], skillItem);
					FightSkillManager:InitSkill(self:GetID(), skillItem);
				end
			end
		end
	end
end
function Fighter:isGod()
	return self:GetID() == -1
end
function Fighter:runSkillById(stype, sid)
	if self.m_skillList[stype] then
		for k,v in pairs(self.m_skillList[stype]) do
			if v:GetResID() == sid then
				v:CanReleaseSkill(0);
			end
		end
	end
end

function Fighter:initActiveSkillCD( cd )
	FightSkillManager:initActiveSkillCD(self, self:getActiveSkill():GetResID(), cd);
end

function Fighter:saveSkillCD(cd)
	self.saveCD = cd;
end
function Fighter:getSaveSkillCD()
	return self.saveCD;
end

function Fighter:initJuexingSkill( skillID )
	local skillData = ResTable:getRowData('skill', skillID);
	if skillData then
		local skillType = skillData['skill_type'];
		if skillType == 4 or skillType == 5 then
			self.juexingSkill = require("app.fight.fightPassiveSkill"):create(self, skillID);
			self.juexingSkill:SetSkillType(skillType)
		end 
	end
end

function Fighter:InitSkillEffect()
	if self.m_skillList[0] and self.m_skillList[0][1] then
		self:InitNormalSkillEffect(self.m_skillList[0][1])
	end
	if self.m_skillList[1] and self.m_skillList[1][1] then
		self:InitActiveSkillEffect(self.m_skillList[1][1])
	end
end

function Fighter:InitNormalSkillEffect(skill)
	FightEffectManager:CreateFighterAvatarEffect(self.id, skill, 'attack', self.fightAttribute)
end

function Fighter:InitActiveSkillEffect(skill)
	FightEffectManager:CreateFighterAvatarEffect(self.id, skill, 'skill', self.fightAttribute)
end

function Fighter:getActiveSkill()
	return self.activeSkill;
end

function Fighter:InitMonsterSkill()
	self.m_skillList = {};
	local allSkillData = ResTable:getRowData('monster', self.resID);
	for i=1, 4 do
		if allSkillData['skill_'..i] and (allSkillData['skill_'..i] > 0) then
			local skillID = allSkillData['skill_'..i];
			local skillScript = allSkillData['script_skill_'..i];
			local skillData = ResTable:getRowData('skill', skillID);
			if skillData then
				local skillType = skillData['skill_type'];
				local skillItem = nil;
				if skillType == 0 then
					skillItem = require("app.fight.fightNormalSkill"):create(self, skillID);
					skillItem:SetScriptID(allSkillData['script_skill_1']);
				elseif skillType == 1 then
					skillItem = require("app.fight.fightActiveSkill"):create(self, skillID);
					skillItem:SetScriptID(allSkillData['script_skill_2']);
					self.activeSkill = skillItem;
				elseif skillType == 2 then
					-- skillItem = require("app.fight.fightPassiveSkill"):create(self, skillID);
					for k, v in pairs(skillData['seat']) do
						if self.formationIndex == v then
							skillItem = require("app.fight.fightPassiveSkill"):create(self, skillID);
							break;
						end
					end
				elseif skillType==4 or skillType==5 then
					self:initJuexingSkill(skillID)
				end
				if skillItem then
					if not self.m_skillList[skillType] then
						self.m_skillList[skillType] = {};
					end
					if skillItem then
						table.insert(self.m_skillList[skillType], skillItem);
						FightSkillManager:InitSkill(self:GetID(), skillItem);
					end
				end
			end
		end
	end

	self.m_isRoleBookMonster = allSkillData.show;

	-- set fight attribute
	self.fightAttribute = ResTable:getValue('monster', self.resID, 'def_type');
	local job = ResTable:getValue('monster', self.resID, 'job');
	if job then
		self.job = tonumber(job);
	end
end

function Fighter:isShowRoleBook()
	return self.m_isRoleBookMonster;
end

function Fighter:GetActiveSkill()
	if self.m_skillList[1] and self.m_skillList[1][1] then
		return self.m_skillList[1][1];
	end
	return nil;
end

--更新
function Fighter:Update(elapse)
	if not self.time then
		self.time = elapse;
		self.lastEffectTime = self.time;
	else
		self.time = self.time + elapse;
	end

	if self.m_fighterstate == FighterState.death then
		return true;        --如果处于死亡状态，返回
	end

	if self:HasSpecialState('dazed') then
		return true
	end

	--更新角色的状态
	return self:updateFighterState(elapse);
end

--更新角色状态
function Fighter:updateFighterState(elapse)
	if FighterState.idle == self.m_fighterstate then          --待机状态更新
		return self:updateStateIdle(elapse);

	elseif FighterState.attack == self.m_fighterstate then      --攻击状态更新
		return self:updateStateAttack(elapse);

	elseif FighterState.showSkill == self.m_fighterstate then	--展示技能状态更新
		return self:updateStateShowSkill(elapse);

	else     --其它状态
		return true;
	end
end

--设置状态
function Fighter:SetFighterState(state)
	if self.m_fighterstate == FighterState.death then
		return false;
	end
	if (self.m_fighterstate == state) then
		return false;
	else
		self.m_fighterstate = state;
		return true;
	end
end

--=============================================================================================
--技能释放执行顺序
function Fighter:getSkillPriority()
	local turn = {};
	turn[1] = 1; -- active skill
	turn[2] = 2; -- passive skill
	turn[3] = 0; -- normal skill
	return turn;
end

function Fighter:SetActiveRelease()
	if self.activeRelease then
		return false
	end
	self.activeRelease = true
	return true
end

function Fighter:GodSkillCanRelease( elapse )
	if FightManager:GetSkillPause() then
		return true;
	end
	local skillTurn = self:getSkillPriority();
	local canRelease = false;

	for key, turn in ipairs(skillTurn) do
		if self.m_skillList[turn] then
			for _, skillItem in pairs(self.m_skillList[turn]) do
				canRelease = FightSkillManager:CanSkillCast(self.id, skillItem);
				if canRelease then
					print('dhx--god--skill--release')
					if turn == SkillType.activeSkill then --在触发死亡动画之前 播放技能特效，如果有死亡，则死亡延迟播放
						FightManager:SkillPause();--暂停播放
						FightManager:setDelayDie();
						self.releaseSkill = skillItem;
					end
					skillItem:CanReleaseSkill(elapse);--这里会触发fighter死亡动画
					
					break;
				end
			end
			if canRelease then
				break;
			end
		end
	end
end

--遍历角色技能，寻找技能释放目标。如果找到，则释放技能
function Fighter:CanReleaseSkill(elapse)
	if FightManager:GetSkillPause() then
		return true;
	end
	local activeFlag = false;
	local skillTurn = self:getSkillPriority();
	local canRelease = false;

	local banList = {
		[1] = false,
		[2] = false,
		[3] = false,
	}
	local speedup = FightManager:SpeedUp()
	if self:HasSpecialState('silent') then
		banList[1] = true;
	end
	local releaseSkillFlag = -1;
	local releaseSkill
	for key, turn in ipairs(skillTurn) do
		local st = true
		if turn==SkillType.activeSkill and self:IsEnemy()==false and FightManager:getAutoRelease()==false then --如果自动释放开关是 关闭的
			st = self.activeRelease;
		end
		if st and self.m_skillList[turn] and not banList[turn] then
			for _, skillItem in pairs(self.m_skillList[turn]) do
				canRelease = FightSkillManager:CanSkillCast(self.id, skillItem);
				if canRelease then
					if turn == SkillType.activeSkill then --在触发死亡动画之前 播放技能特效，如果有死亡，则死亡延迟播放
						FightManager:SkillPause();--暂停播放
						FightManager:setDelayDie();
						self.releaseSkill = skillItem;
					end
					skillItem:CanReleaseSkill(elapse);--这里会触发fighter死亡动画
					releaseSkillFlag = turn
					releaseSkill = skillItem
					
					break;
				end
			end
			if canRelease then
				break;
			end
		end
	end
	if releaseSkillFlag == SkillType.activeSkill then
		FightManager:recordActiveSkillTime(self:GetID(), self:IsEnemy(), self.releaseSkill:GetResID())
		self:SetFighterState(FighterState.showSkill)
	elseif releaseSkillFlag == SkillType.normalAttack then
		FightManager:recordActiveSkillTime(self:GetID(), self:IsEnemy(), 0)
		self:SetFighterState(FighterState.attack)
		self.attackFinishFlag = false
		local tmpNode = cc.Node:create()
			:addTo(self:getWorkNode())
		tmpNode:runAction(cc.Sequence:create(
			cc.DelayTime:create(0.05*speedup),
			cc.CallFunc:create(function()
				self.attackFinishFlag = true
			end)))
	elseif releaseSkillFlag == SkillType.passiveSkill then
		-- self:SetFighterState(FighterState.attack)
		-- self.attackFinishFlag = false
		-- local tmpNode = cc.Node:create()
		-- 	:addTo(self:getWorkNode())
		-- tmpNode:runAction(cc.Sequence:create(
		-- 	cc.DelayTime:create(0.05*speedup),
		-- 	cc.CallFunc:create(function()
		-- 		self.attackFinishFlag = true
		-- 	end)))
	end
	return (not activeFlag);
end

function Fighter:reduceSkillCastCD(cd)
	for _, skillItem in pairs(self.m_skillList[1]) do
		FightSkillManager:reduceSkillCastCD(self:GetID(), skillItem, cd)
	end
end

function Fighter:addSkillCastCD(cd, pro)
	for _, skillItem in pairs(self.m_skillList[1]) do
		FightSkillManager:addSkillCastCD(self:GetID(), skillItem, cd, pro);
	end
end

--更新待机状态的处理函数
function Fighter:updateStateIdle(elapse)
	--判断技能释放条件，如果成立，则释放技能
	return self:CanReleaseSkill(elapse);
end

--攻击攻击状态结束后回复空闲
function Fighter:updateStateAttack(elapse)
	if self.attackFinishFlag and (self.animationID == AnimationType.idle) then
		self:SetFighterState(FighterState.idle)
	end
	return true;
end

--被击动作结束后恢复待机
function Fighter:updateStateShowSkill(elapse)
	return true;
end

--=========================================================================================
--演算技能效果影响
function Fighter:WillSkillEffect( attacker, skill)
	-- body
	self.m_willCalcu = true;
	self.m_willCalcuStList = {};
	self.m_willCalcuDam = 0;
	self.m_willCalcuDamEff = 0;
	for index, _ in pairs(skill:GetDamageList()) do
		self:SkillEffect(attacker, skill, index);
	end

	local totalShield = 0
	local shieldList = BuffManager:getShieldValue(self.id);
	if #shieldList > 0 then
		for k, v in pairs(shieldList) do
			totalShield = totalShield + v.value
		end
	end

	self.m_willCalcuDam = self.m_willCalcuDam-totalShield

	local isDie = false;
	if self:IsBoss() and self:IsEnemy() and FightManager:getRoundType()==FightType.GuildBoss then
		isDie = false --工会boss不会死
	else
		if HpCompare(self:getHpNum(), '<=', self.m_willCalcuDam) then
			isDie = true;
		else
			local per = self:getWillHpPercent( tostring(self.m_willCalcuDam) );
			local wAct = BuffManager:checkActiveBuff(self:GetID(), BuffType.imm_kill, per);
			if wAct then
				isDie = true;
			end
		end
	end
	
	self.m_willCalcu = false;

	return isDie, HpDivide(self.m_willCalcuDam, self.m_propertyData:GetMaxHP());

end

function Fighter:SetSecDamEff( v )
	self.m_willCalcuDamEff = v;
end

function Fighter:ClearWillCalcu()
	self.m_willCalcuDamEff = 0;
	self.m_willCalcuStList = {};
end

--=========================================================================================
--受到技能效果影响
function Fighter:SkillEffect(attacker, skill, damageNum)
	if (self.m_fighterstate == FighterState.death) then
		--如果该角色已经死亡
		return;
	end

	-- add damage
	-- check and add targeter damage
	local damageItem = skill:GetDamageItem(damageNum);
	if not damageItem then
		return;
	end
	--计算攻击状态（暴击、命中、闪避）和实际伤害数值
	local state, realDamage, cast_type = self:GetActualDamage(attacker, skill, damageItem, damageNum);
	realDamage = self:calcuDamageDeep(realDamage, attacker)
	local args = {attacker = attacker, attackState = state, value = realDamage, pos = self:GetPositionIndex(), cast_type = cast_type, damageNum = damageNum};
	self:checkStateXorAddDamage(attacker, skill, args);	
end

function Fighter:GetActualDamage(attacker, skill, damageItem, damageIdx)
	local attackState = AttackState.normal;--self:RandomAttackState(attacker, skill);    --攻击状态（未命中 正常 暴击）
	if self.m_willCalcu then
		attackState = self:RandomAttackState(attacker, skill);    --攻击状态（未命中 正常 暴击）
		-- table.insert(self.m_willCalcuStList, attackState);
		self.m_willCalcuStList[damageIdx] = attackState;
	else
		if self.m_willCalcuStList[damageIdx] then
			attackState = self.m_willCalcuStList[damageIdx];
		else
			attackState = self:RandomAttackState(attacker, skill);
		end
	end

	local isCrit = false;
	if (attackState == AttackState.miss) then
		return attackState, 0;                  --如果是miss状态，返回攻击状态miss和实际伤害0
	elseif (attackState == AttackState.crital) then
		isCrit = true;
	end

	local realDamage = 0;
	local cast_type;
	realDamage, cast_type = FightFunction:CaculateAttackDamage(attacker, self, damageItem, isCrit);
	return attackState, realDamage, cast_type;
	
--	return attackState, realDamage > 0 and realDamage or 0, cast_type;
end

function Fighter:calcuDamageDeep(damage, attacker)
	local coefficient = 1 - self.m_propertyData:GetCurrentPropertyData('damage_reduce') + attacker.m_propertyData:GetCurrentPropertyData('damage_enlarge')
	coefficient = coefficient + self.m_willCalcuDamEff;
	if coefficient < 0.0001 then
		return '1';
	end
	damage = HpMultiply(damage, coefficient)
	if HpCompare(damage, '<', '1') then
		return '1'
	else
		return damage
	end
end

function Fighter:RandomAttackState(attacker, skill)
	if skill:GetSkillType() == SkillType.normalAttack then
		--local attackHit = attacker.m_propertyData:GetCurrentPropertyData('hit');
		--local defenderDodge = self.m_propertyData:GetCurrentPropertyData('dodge');
		--local hitRate = 9000 + attackHit*30 - defenderDodge*30;
		--if math.random(1, 10000) > hitRate then
		--	return AttackState.miss;
		--end
		local attackCrit = attacker.m_propertyData:GetCurrentPropertyData('crit');
		if math.random(1, 10000) <= attackCrit then
			return AttackState.crital;
		else
			return AttackState.normal;
		end
	else
		return AttackState.normal;
	end
end

function Fighter:checkStateXorAddDamage(attacker, attackerSkill, args)
	if self:HasSpecialState('imm_dam') then
		return
	end
	if args.attackState ~= AttackState.miss then
		--self:PlayHitSound();
		if attackerSkill then
			if HpCompare(args.value, '>=', '100000') and attackerSkill:GetSkillType() == SkillType.normalAttack then
				FightManager:addFightWarning(4, attackerSkill:GetResID(), args.value)
			end
			if HpCompare(args.value, '>=', '1000000') and attackerSkill:GetSkillType() == SkillType.activeSkill then
				FightManager:addFightWarning(3, attackerSkill:GetResID(), args.value)
			end
		end
		self:AddDamage(args.value, args.cast_type, attacker, attackerSkill, args.damageNum, 2);

		if self.m_willCalcu == false then
			-- check reflect
			local reflect_c, reflect_p = BuffManager:getReflectValue(self.id);
			if HpCompare(reflect_c, '>', '0') or reflect_p > 0 then
				local damage = HpPlus(HpMultiply(args.value, reflect_p), reflect_c);
				attacker:AddDamage(damage, CastType.reflect, self, nil, nil, 1);
			end

		end

	elseif args.attackState == AttackState.miss then
	end
end

function Fighter:damageValIsError(value)
	if tonumber(value) > (tonumber(self.m_propertyData:GetOriPropertyData('atk'))*30) then
		return true, self.m_propertyData:GetOriPropertyData('atk');
	end
	return false,0;
end

function Fighter:AddDamage(value, cast_type, attacker, skill, damageNum, fromeType)--fromeType 1 反伤，2 技能，3 buff
	if self:GetID()==-1 then --god 是受击方（来自反伤，则返回）
		return
	end
	if self:HasSpecialState('imm_dam') then
		return
	end
	if attacker and not attacker:IsEnemy() and FightManager:getFightType() ~= FightType.Event then
		local st, oriV = attacker:damageValIsError(value);
		if st==true then
			local skillId = 0;
			if skill then
				skillId = skill:GetResID()
			end
			FightManager:setErrorMsg(attacker:GetPid(), value, oriV, skillId, fromeType)
			FightManager:ForceExitFight() --伤害值过高 结束战斗
			return;
		end
	end

	-- 如果回光返照buff激活了
	local isHave, bId = BuffManager:isHaveBuff(self:GetID(), BuffType.heal_return, true);
	if isHave==true then
		if self.m_willCalcu==true then --预计算 如果触发 则无伤害 返回，预计算 不做真实数据修改
			return
		end
		-- 	加血
		local bvalue, origin, skill = BuffManager:getActiveValue(self:GetID(), BuffType.heal_return, bId, value);
		self:AddHeal(bvalue, origin, skill);
		return;
	end

	if self.m_willCalcu then
		self.m_willCalcuDam = self.m_willCalcuDam + value;
		return;
	end
	
	local shieldList = BuffManager:getShieldValue(self.id);

	local trueDamageDown = value;
	local absorbValue = 0
	if #shieldList > 0 then
		local comsumeList = {};
		for k, v in pairs(shieldList) do
			if HpCompare(trueDamageDown, '>', v.value) then
				if self.m_willCalcu == false then
					table.insert(comsumeList, v.id);
				end
				trueDamageDown = HpMinus(trueDamageDown, v.value);

			else
				local newShieldValue = HpMinus(v.value, trueDamageDown);
				trueDamageDown = '0';
				if self.m_willCalcu == false then
					BuffManager:setShield(self.id, v.id, newShieldValue);
				end
				break;

			end
		end
		absorbValue = HpMinus(value, trueDamageDown);
		if self.m_willCalcu == false then
			BuffManager:consumeShield(self.id, comsumeList);
		end

	end

	local conType = 0
	if fromeType == 2 then
		conType = self:checkDamageContinueType(skill, damageNum);
	end
	local skillID = 0;
	if conType > 0 and skill then
		skillID = skill:GetResID(); 
	end

	absorbValue = tonumber(absorbValue)
	if absorbValue > 0 then
		if self:IsEnemy() then
			EventManager:FireEvent(EventType.FightAddLabel, {icon=attacker:GetSIcon(), pos = self:GetPositionIndex(), value = absorbValue, cast_type = CastType.absorb, isEnemy = true, hitType = conType, fid=attacker:GetID(), skillId = skillID});
		else
			EventManager:FireEvent(EventType.FightAddLabel, {icon=attacker:GetSIcon(), pos = self:GetPositionIndex(), value = absorbValue, cast_type = CastType.absorb, isEnemy = false, hitType = conType, fid=attacker:GetID(), skillId = skillID});
		end
	end
	local isDie, realDamage = false, '0';
	-- 减少自身实际血量
	if HpCompare(trueDamageDown, '>', '0') then
		isDie, realDamage = self:DecreaseHp(trueDamageDown);
	else
		return;
	end			
	print('realDamage', realDamage)

	-- 	激活回光返照buff，如果有
	if isDie==false then
		local per = self:getHpPercent();
		local isAct = BuffManager:activeBuff(self:GetID(), BuffType.imm_kill, per);
		if isAct then --斩杀
			local callback = function()
			end
			self:AddSpecialStateEffect('imm_kill', callback)
			local t1, t2 = self:DecreaseHp(0, true);
			isDie = t1;
			realDamage = realDamage + t2;
			
		else
			BuffManager:activeBuff(self:GetID(), BuffType.heal_return, per);
		end
	end

	--EventManager:FireEvent(EventType.RecountAdd, {pos = attacker:GetPositionIndex(), damage = value});
	
	local timeScale = 1/FightManager:SpeedUp()
	if fromeType == 2 and skill and skill:GetSkillType() == SkillType.activeSkill and damageNum then
		local legal, isLast = skill:checkLastHit(damageNum)
		if legal then
			if isLast then
				if isDie and FightManager:isDelayDie() then
					self:SetAnimation(AnimationType.keep, 1, timeScale);
				else
					self:SetAnimation(AnimationType.hit, 1, timeScale);
				end
				
			else
				self:SetAnimation(AnimationType.keep, 1, timeScale);	
			end
		end
	end

	--[[
	if (not isDie) and value > (self.m_propertyData:GetCurrentPropertyData('maxHp') * 0.05) then
		self:SetHit();
	end
	--]]

	-- 添加label
	if self:IsEnemy() then
		EventManager:FireEvent(EventType.FightAddLabel, {icon=attacker:GetSIcon(), pos = self:GetPositionIndex(), value = value-absorbValue, cast_type = cast_type, curHp = self.m_propertyData:GetCurrentHP(), isEnemy = true, hitType = conType, fid=attacker:GetID(), skillId = skillID});
	else
		EventManager:FireEvent(EventType.FightAddLabel, {icon=attacker:GetSIcon(), pos = self:GetPositionIndex(), value = value-absorbValue, cast_type = cast_type, curHp = self.m_propertyData:GetCurrentHP(), isEnemy = false, hitType = conType, fid=attacker:GetID(), skillId = skillID});
	end
	
	-- 更新逻辑总血量
	-- EventManager:FireEvent(EventType.FightChangeTotalHp, {hp = -value, isEnemy = self:IsEnemy(), pos = self:GetPositionIndex()});
	FightManager:recordFightMsg( attacker:GetID(), attacker:IsEnemy(), 'Atk', realDamage, skill )
	FightManager:recordFightMsg( self:GetID(), self:IsEnemy(), 'Dam', realDamage )
	if not self:IsEnemy() then
		-- 更新屏幕血条
		--EventManager:FireEvent(EventType.FightHpChange, {percent = self.m_propertyData:GetCurrentPropertyData('hp')/self.m_propertyData:GetCurrentPropertyData('maxHp'), pos = self:GetPositionIndex()});
	else
		FightManager:AddAllDamageDown(realDamage)
	end
end

function Fighter:checkDamageContinueType( skill, damageNum )
	local isLianji, isLastHit = false, false;
	local ret = 0;
	if skill and skill:GetSkillType() == SkillType.activeSkill and damageNum then
		isLianji, isLastHit = skill:checkIsConAndLastHit(damageNum);
		ret = -1;
	end
	if isLianji and isLastHit then
		return 2;
	elseif isLianji and isLastHit==false then
		return 1;
	else
		return ret;
	end
end

function Fighter:AddHeal(value, healer, skill)
	-- 增加自身实际血量
	local realVal = self:calcuHealDeep(value)
	local isHeal, realHeal = self:IncreaseHp(realVal);
	if not isHeal then
		return;
	end

	if healer then
		FightManager:recordFightMsg( healer:GetID(), healer:IsEnemy(), 'Help', realHeal, skill )
	end

	--EventManager:FireEvent(EventType.RecountAdd, {pos = healer:GetPositionIndex(), heal = realHeal});
	-- 添加label
	if self:IsEnemy() then
		EventManager:FireEvent(EventType.FightAddLabel, {icon=healer:GetSIcon(), pos = self:GetPositionIndex(), value = realHeal, cast_type = CastType.heal, curHp = self.m_propertyData:GetCurrentHP(), isEnemy = true, hitType = 0});
	else
		EventManager:FireEvent(EventType.FightAddLabel, {icon=healer:GetSIcon(), pos = self:GetPositionIndex(), value = realHeal, cast_type = CastType.heal, curHp = self.m_propertyData:GetCurrentHP(), isEnemy = false, hitType = 0});
	end
	-- 更新逻辑总血量
	-- EventManager:FireEvent(EventType.FightChangeTotalHp, {hp = realHeal, isEnemy = self:IsEnemy(), pos = self:GetPositionIndex()});
	if not self:IsEnemy() then
		-- 更新屏幕血条
		--EventManager:FireEvent(EventType.FightHpChange, {percent = self.m_propertyData:GetCurrentPropertyData('hp')/self.m_propertyData:GetCurrentPropertyData('maxHp'), pos = self:GetPositionIndex()});
	end
end

function Fighter:calcuHealDeep(heal)
	local coefficient = 1 + self.m_propertyData:GetCurrentPropertyData('heal_effect')
	if coefficient <= 0 then
		return '1'
	end
	heal = HpMultiply(heal, coefficient)
	if HpCompare(heal, '<', '1') then
		return '1'
	else
		return heal
	end
end

--添加buff
function Fighter:AddBuff(attacker, skill)
	if (self.m_fighterstate == FighterState.death) then
		--如果该角色已经死亡
		return;
	end

	for _, v in pairs(skill:GetBuffList()) do
		if math.random() <= v.rate then
			BuffManager:AddBuff(self, attacker, v.buff_id, skill);
		end
	end
end

--添加buff 第二类目标
function Fighter:AddBuff2(attacker, skill)
	if (self.m_fighterstate == FighterState.death) then
		--如果该角色已经死亡
		return;
	end

	for _, v in pairs(skill:GetBuff2List()) do
		if math.random() <= v.rate then
			BuffManager:AddBuff(self, attacker, v.buff_id, skill);
		end
	end
end

--执行后续技能
function Fighter:CastFollowSkill(Skill)
	if (self.m_fighterstate == FighterState.death) then
		--如果该角色已经死亡
		return;
	end

	for _, v in pairs(Skill:GetFollowList()) do
		if math.random() <= v.rate then
			local skillItem = require("app.fight.fightFollowSkill"):create(self, v.skill_id, 1);
			skillItem:CanReleaseSkill();
		end
	end
end

function Fighter:castNoScriptSkill(skill, targetList)
	if (self.m_fighterstate == FighterState.death) then
		--如果该角色已经死亡
		return;
	end
	for _, target in pairs(targetList) do
		-- add damage
		for index, _ in pairs(skill:GetDamageList()) do
			target:SkillEffect(self, skill, index);
		end
		
		-- add buff
		target:AddBuff(self, skill);

		-- cast follow
		--self:CastFollowSkill(skill);
	end
end

--=========================================================================================
-- 特殊状态相关
-- 用于判断角色身上是否有特殊状态
-- 用于特效是否触发的判断依据
-- 不用于数值计算
function Fighter:AddSpecialState(state, buff_id, buff_resId, attribute, isContinued, callback)
	isContinued = isContinued or false;
	local timeScale = 1/FightManager:SpeedUp()

	local word = state;
	local addWord = true;
	if word == 'dazed' then
		if attribute == RestrainType.RED_BASE then -- Adapter.red 
			word = word .. '_red';
		elseif attribute == RestrainType.GREEN_BASE then --Adapter.green 
			word = word .. '_green';
		elseif attribute == RestrainType.BLUE_BASE then --Adapter.blue
			word = word .. '_blue';
		end

	elseif word == 'dot' then
		if attribute == RestrainType.RED_BASE then
			word = word .. '_red';
		elseif attribute == RestrainType.GREEN_BASE then
			word = word .. '_green';
		elseif attribute == RestrainType.BLUE_BASE then
			word = word .. '_blue';
		end
		if not isContinued then
			addWord = false;
		end

	elseif word == 'hot' or word == 'hot_per' then
		word = 'hot';
		if not isContinued then
			addWord = false;
		end

	elseif word == 'heal_per' then
		word = 'heal'
	elseif word == 'reject' or word == 'combo' or word == 'reborn' then
		addWord = false
	end

	if addWord then
		local wordEffect = cc.Sprite:create('image/buff/'..word..'.png')
		if wordEffect then
			wordEffect:setScale(0.6)
			wordEffect:addTo(self:getBuffWordNode())
			self:addWordEffect(wordEffect);
		end
	end

	local buffEffData = ResTable:getValue('buff', buff_resId, 'show_effect')
	if buffEffData==nil or #buffEffData<3 then
		return
	end
	local buff_resName = ResTable:getValue('skill_effect', buffEffData[1], 'name')
	local effPos = buffEffData[2]
	local effZoder = buffEffData[3]

	local effect, db_id;

	local addEffect = true
	local repeatEffect = true
	local pos_str = state

	DragonBonesManager:loadDragonBonesResources(buff_resName, 'Effect/db/')
	effect = DragonBonesManager:getFactory():buildArmatureDisplay(buff_resName)
	db_id = DragonBonesManager:AddDisplay(effect, buff_resName)

	if state == 'reflect' then
		effect:getAnimation():play(buff_resName, -1, timeScale)	

	elseif state == 'dot_per' then
		-- if isContinued then
		-- 	addEffect = false

		-- else
			effect:getAnimation():play(buff_resName, 1, timeScale)
			local listener = cc.EventListenerCustom:create("complete", function(event)
				effect:removeFromParent()
				DragonBonesManager:RemoveDisplay(db_id)
			end)
			local listener2 = cc.EventListenerCustom:create("frameEvent", function(event)
				if callback then
					callback()
				end
			end)
			effect:getEventDispatcher():setEnabled(true)
			effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
			effect:getEventDispatcher():addEventListenerWithFixedPriority(listener2, 1)
			repeatEffect = false
		-- end

	elseif state == 'dot' then
		if attribute == RestrainType.BLUE_BASE then
			-- blue
			-- if isContinued then
			-- 	addEffect = false

			-- else
				effect:getAnimation():play(buff_resName, 1, timeScale)

				local listener = cc.EventListenerCustom:create("complete", function(event)
					effect:removeFromParent()
					DragonBonesManager:RemoveDisplay(db_id)
				end)
				local listener2 = cc.EventListenerCustom:create("frameEvent", function(event)
					if callback then
						callback()
					end
				end)
				effect:getEventDispatcher():setEnabled(true)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener2, 1)

				pos_str = 'dot_blue'
				repeatEffect = false
			-- end

		elseif attribute == RestrainType.RED_BASE then
			-- red
			-- if isContinued then
			-- 	addEffect = false

			-- else
				effect:getAnimation():play(buff_resName, 1, timeScale)
				local listener = cc.EventListenerCustom:create("complete", function(event)
					effect:removeFromParent()
					DragonBonesManager:RemoveDisplay(db_id)
				end)
				local listener2 = cc.EventListenerCustom:create("frameEvent", function(event)
					if callback then
						callback()
					end
				end)
				effect:getEventDispatcher():setEnabled(true)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener2, 1)

				repeatEffect = false
			-- end

		elseif attribute == RestrainType.GREEN_BASE then
			-- green
			if isContinued then
				effect:getAnimation():play(buff_resName, 1, timeScale)
				local listener = cc.EventListenerCustom:create("complete", function(event)
					effect:getAnimation():play(buff_resName, 1, timeScale)
				end)
				effect:getEventDispatcher():setEnabled(true)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
			else
				addEffect = false
			end
		end

	elseif state == 'heal' or state == 'heal_per' then
		effect:getAnimation():play(buff_resName, 1, timeScale)
		local listener = cc.EventListenerCustom:create("complete", function(event)
			effect:removeFromParent()
			DragonBonesManager:RemoveDisplay(db_id)
		end)
		local listener2 = cc.EventListenerCustom:create("frameEvent", function(event)
			if callback then
				callback()
			end
		end)
		effect:getEventDispatcher():setEnabled(true)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener2, 1)
		
		repeatEffect = false

		return

	elseif state == 'dazed' then
		if attribute == RestrainType.BLUE_BASE then
			effect:getAnimation():play(buff_resName..'_1', 1, timeScale)
			local listener = cc.EventListenerCustom:create("complete", function(event)
				effect:getAnimation():play(buff_resName..'_2', 1, timeScale)
			end)
			effect:getEventDispatcher():setEnabled(true)
			effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
			pos_str = 'dazed_blue'

		elseif attribute == RestrainType.RED_BASE then
			effect:getAnimation():play(buff_resName, -1, timeScale)

		elseif attribute == RestrainType.GREEN_BASE then
			effect:getAnimation():play(buff_resName..'_1', 1, timeScale)
			local listener = cc.EventListenerCustom:create("complete", function(event)
				effect:getAnimation():play(buff_resName..'_2', 1, timeScale)
			end)
			effect:getEventDispatcher():setEnabled(true)
			effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)

		end

	elseif state == 'shield' or state == 'haste' or state == 'slow' 
		or state == 'silent' or state == 'imm_neg' or state == 'imm_dam' 
		or state == 'combo' or state == 'cd_lang' 
		or state == 'reborn_other' or state == 'heal_return' then
		effect:getAnimation():play(buff_resName, -1, timeScale)
	elseif state == 'reject' or state == 'imm_kill' then
		effect:getAnimation():play(buff_resName..'_1', -1, timeScale)

	elseif state == 'cleanup' or state == 'breakup' or state == 'refresh' or state == 'dispel' or state == 'purge' then

		effect:getAnimation():play(buff_resName, 1, timeScale)

		local listener = cc.EventListenerCustom:create("complete", function(event)
			effect:removeFromParent()
			DragonBonesManager:RemoveDisplay(db_id)
		end)
		effect:getEventDispatcher():setEnabled(true)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)

		repeatEffect = false

	else

		addEffect = false
	end

	if addEffect then
		if effPos == 2 then
			effect:setPosition(cc.p(0, 0))
		elseif effPos == 1 then
			effect:setPosition(self:getTopPos())
		elseif effPos == 3 then
			effect:setPosition(self:getBottomPos())
		end
		if effZoder == 1 then
			effect:addTo(self:getBuffFrontEffectNode())
		elseif effZoder == 0 then
			effect:addTo(self:getBuffBackEffectNode())
		end
		if self:IsEnemy() then
			effect:getArmature():setFlipX(true)
		end
		effect:setScale(self:GetEffectScale())

		if repeatEffect then
			self.m_effectCallbackList[buff_id] = {}
			self.m_effectCallbackList[buff_id].effect = effect
			self.m_effectCallbackList[buff_id].db_id = db_id
			self.m_effectCallbackList[buff_id].fun = function()
				self.m_effectCallbackList[buff_id].effect:removeFromParent()
				DragonBonesManager:RemoveDisplay(self.m_effectCallbackList[buff_id].db_id)
			end
		end
	end

	if not isContinued then
		return;
	end

	if not self.m_specialStateList[state] then
		self.m_specialStateList[state] = {};
	end
	table.insert(self.m_specialStateList[state], buff_id);
end

function Fighter:RemoveSpecialState(state, buff_id, attribute)
	if self.m_specialStateList[state] then
		for k, v in pairs(self.m_specialStateList[state]) do
			if v == buff_id then
				table.remove(self.m_specialStateList[state], k)
				break;
			end
		end
	end
	if self.m_effectCallbackList[buff_id] then
		self.m_effectCallbackList[buff_id].fun();
		self.m_effectCallbackList[buff_id] = nil;
	end
end

function Fighter:HasSpecialState(state)
	local hasState = false
	if self.m_specialStateList[state] then
		for k, v in pairs(self.m_specialStateList[state]) do
			hasState = true
			break
		end
	end
	return hasState
end

function Fighter:AddSpecialStateEffect(state, callback)

	-- effect root
	local rootNode = self:getEffectNode();
	if not rootNode then
		return;
	end
	local timeScale = 1/FightManager:SpeedUp()

	if state ~= 'imm_kill' then
		local word = state;
		local wordEffect = cc.Sprite:create('image/buff/'..word..'.png')
			:setScale(0.6)
			:addTo(rootNode)
		self:addWordEffect(wordEffect);
	end

	local effect
	local pos_str = state
	local db_id
	if state == 'reject' then
		DragonBonesManager:loadDragonBonesResources('buff_20', 'Effect/db/')
		effect = DragonBonesManager:getFactory():buildArmatureDisplay('buff_20')
		db_id = DragonBonesManager:AddDisplay(effect, 'buff_20')

		effect:getAnimation():play('buff_20_2', 1, timeScale)

		local listener = cc.EventListenerCustom:create("complete", function(event)
			effect:removeFromParent()
			DragonBonesManager:RemoveDisplay(db_id)
		end)
		effect:getEventDispatcher():setEnabled(true)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)

		pos_str = 'reject_2'

	elseif state == 'combo' then

	elseif state == 'reborn' then
		DragonBonesManager:loadDragonBonesResources('buff_22', 'Effect/db/')
		effect = DragonBonesManager:getFactory():buildArmatureDisplay('buff_22')
		db_id = DragonBonesManager:AddDisplay(effect, 'buff_22')

		effect:getAnimation():play('buff_22', 1, timeScale)

		local listener = cc.EventListenerCustom:create("complete", function(event)
			effect:removeFromParent()
			DragonBonesManager:RemoveDisplay(db_id)
		end)
		effect:getEventDispatcher():setEnabled(true)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)

	elseif state == 'imm_kill' then
		DragonBonesManager:loadDragonBonesResources('buff_28', 'Effect/db/')
		effect = DragonBonesManager:getFactory():buildArmatureDisplay('buff_28')
		db_id = DragonBonesManager:AddDisplay(effect, 'buff_28')

		effect:getAnimation():play('buff_28_2', 1, timeScale)
		pos_str = 'imm_kill_2'

		local listener = cc.EventListenerCustom:create("complete", function(event)
			effect:removeFromParent()
			DragonBonesManager:RemoveDisplay(db_id)
			if callback then
				callback()
			end
		end)
		local listener2 = cc.EventListenerCustom:create("frameEvent", function(event)
			EventManager:FireEvent(EventType.FightPanelShake, 0);
		end)
		effect:getEventDispatcher():setEnabled(true)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
		effect:getEventDispatcher():addEventListenerWithFixedPriority(listener2, 1)
			
	end

	if effect then
		if BUFF_EFFECT_POSITION[pos_str].pos == 'mid' then
			effect:setPosition(cc.p(0, 0))
		elseif BUFF_EFFECT_POSITION[pos_str].pos == 'top' then
			effect:setPosition(self:getTopPos())
		elseif BUFF_EFFECT_POSITION[pos_str].pos == 'bot' then
			effect:setPosition(self:getBottomPos())
		end
		if BUFF_EFFECT_POSITION[pos_str].zorder == 'front' then
			effect:addTo(self:getBuffFrontEffectNode())
		elseif BUFF_EFFECT_POSITION[pos_str].zorder == 'back' then
			effect:addTo(self:getBuffBackEffectNode())
		end
	end
end

function Fighter:addWordEffect(effect)
	local effectTime = 1;
	if not self.lastEffectTime then
		self.lastEffectTime = self.time - effectTime;
	end
	local speedup = FightManager:SpeedUp()

	effect:setOpacity(0)
	local action1 = cc.Sequence:create(
		cc.FadeIn:create(0.1*speedup),
		cc.DelayTime:create((effectTime-0.2)*speedup),
		cc.FadeOut:create(0.1*speedup)
		)
	local action2 = cc.Sequence:create(
		cc.ScaleTo:create(0.1*speedup, 0.8),
		cc.DelayTime:create((effectTime-0.2)*speedup),
		cc.ScaleTo:create(0.1*speedup, 0.6)
		)
	if self.time - self.lastEffectTime < effectTime then
		local delayTime = effectTime-(self.time-self.lastEffectTime);
		self.lastEffectTime = self.lastEffectTime + effectTime;
		effect:runAction(cc.Sequence:create(
			cc.DelayTime:create(delayTime*speedup),
			cc.Spawn:create(
				cc.MoveBy:create(effectTime*speedup, cc.p(0, 40)),
				action1,
				action2),
			cc.CallFunc:create(function()
				effect:removeFromParent();
			end)
		))
	else
		self.lastEffectTime = self.time;
		effect:runAction(cc.Sequence:create(
			cc.Spawn:create(
				cc.MoveBy:create(effectTime*speedup, cc.p(0, 40)),
				action1,
				action2),
			cc.CallFunc:create(function()
				effect:removeFromParent();
			end)
		))
	end
end

--=========================================================================================
--复活
function Fighter:Revive()
	FightSkillManager:reviveFighterSkill( self:GetID() );
	self:setCurHpNum( self.m_propertyData:GetMaxHP() );
	--更改状态
	self.m_fighterstate = FighterState.idle;
	--播放待机动作
	local timeScale = 1/FightManager:SpeedUp()
	Fighter.super.SetAnimation(self, AnimationType.idle, -1, timeScale);

	EventManager:FireEvent(EventType.Revive, self);

end

--设置成死亡
function Fighter:DelayDie()
	-- body
	if self.m_isDelayDie then
		self.m_isDelayDie = false;
		self:SetDie();
	end
end

function Fighter:SetDie()
	if FightManager:isDelayDie() == true then
		self.m_isDelayDie = true;
		return;
	end
	if self.m_fighterstate == FighterState.death then
		return;
	end

	--[[
	-- play dir voice
	local soundName = resTableManager:GetValue(ResTable.actor,tostring(self:GetResID()),'death_voice')
	if soundName ~= nil then
	SoundManager:PlayVoice( tostring(soundName))
	end
	--]]
	-- 删除buff
	BuffManager:DelDeadFighterBuff(self.id);

	-- 停止技能
	for _, skills in pairs(self.m_skillList) do
		for k, v in pairs(skills) do
			v:EndSkill()
		end
	end

	--更改状态
	self:SetFighterState(FighterState.death);
	--播放死亡动作
	self:SetAnimation(AnimationType.die);
	--触发角色死亡的事件
	if self:IsEnemy() then
		self.deathIdx = FightManager:addDeathIdx();
	end
	EventManager:FireEvent(EventType.Die, self);

end

function Fighter:getDeathIdx()
	return self.deathIdx;
end

function Fighter:SetFightBeginDie()
	if self.m_fighterstate == FighterState.death then
		return;
	end

	--更改状态
	self:SetFighterState(FighterState.death);
	--播放死亡动作
	self:SetAnimation(AnimationType.die);
end

--设置胜利
function Fighter:SetWin()
	if self.m_fighterstate == FighterState.death then
		return;
	end
	--更改状态
	--self:SetFighterState(FighterState.win);
	--播放胜利动作
	self:SetAnimation(AnimationType.win);	
end

function Fighter:isAlive()
	return self.m_fighterstate ~= FighterState.death;
end

--设置最大血量
function Fighter:SetMaxHp(maxHP)
	self.m_propertyData:SetMaxHp(maxHP);
end

--调整血量
function Fighter:AdjustHP(hpValue)
	self.m_propertyData:AdjustHP(hpValue);
end

--扣除血量
function Fighter:DecreaseHp(value, forceKill)
	self.lastHp = self.m_propertyData:GetCurrentHP();
	if self:IsBoss() and self:IsEnemy() and FightManager:getRoundType()==FightType.GuildBoss then
		FightManager:addTotalDamage( value )
		if model.GuildModel:IsFightBossBody() then
			return false, value;
		end
	end

	if forceKill==true then
		value = self.lastHp;
	end
	
	if self.m_propertyData:DecreaseHp(value) then
	--print("===Damage===   before: " .. self.lastHp .. " after: " .. self.m_propertyData:GetCurrentHP())
		--角色死亡
		if self:HasSpecialState('reborn') then
			BuffManager:causeBuffEffect(self.id, BuffType.reborn)
			return false, self.lastHp
		else
			self:SetDie();
			return true, self.lastHp;
		end
	end
	--print("===Damage===   before: " .. self.lastHp .. " after: " .. self.m_propertyData:GetCurrentHP())
	return false, value;
end

--增加血量
function Fighter:IncreaseHp(value)
	self.lastHp = self.m_propertyData:GetCurrentHP();
	self.m_propertyData:RecoverHP(value);
	--print("===heal===   before: " .. self.lastHp .. " after: " .. self.m_propertyData:GetCurrentHP())
	--local isHeal = self.lastHp < self.m_propertyData:GetCurrentHP();
	--local realHeal = self.m_propertyData:GetCurrentHP() - self.lastHp;
--	return isHeal, realHeal;
	return true, value;
end

--设置体型放大倍数
function Fighter:SetShapeScale(scale)
	self.m_propertyData:SetShapeScale(scale);
end

--设置图层
function Fighter:SetZOrder(zOrder)
	if (not self.m_isEnemy) then
		--本方
		self:GetCppObject().ZOrder = zOrder;
	else
		--敌方
		self:GetCppObject().ZOrder = zOrder - 1;
	end
end

--=========================================================================================
--Get函数
--是否敌方角色
function Fighter:IsEnemy()
	return self.m_isEnemy;
end

function Fighter:IsBoss()
	return self.m_isBoss
end

--检查动作完成度 并在攻击/技能/被击动作完成时 切换为待机动作
function Fighter:checkAnimation()
	local completed = false
	if (self.animationID == AnimationType.attack) or (self.animationID == AnimationType.hit) or (self.animationID == AnimationType.skill) then
		if self.armature:getAnimation():isCompleted() then
			completed = true;
		end
	end
	return completed;
end

--天神释放技能表现
function Fighter:GodShowSkill()
	if self.releaseSkill then
		print('dhx--god--GodShowSkill')
		EventManager:FireEvent(EventType.FightPanelScale, self, self.releaseSkill); --放大施法对象

		FightManager:SkillPause();--暂停战斗
		SkillScriptManager:Pause();--技能脚本暂停
		AudioManager:playEffect(Conf.skill , 1);

		FightShowSkillManager:showSkill(self, self.releaseSkill);
		self.releaseSkill = nil;
	end
end

--设置动作 并切换状态
function Fighter:SetAnimation(id)
	--死亡动作不能被别的动作替换
	if (self.animationID == AnimationType.die) then
		return;
	end

	-- if self.animationID == AnimationType.hit and self.m_isDelayDie then
	-- 	Fighter.super.SetAnimation(self, AnimationType.keep, 1);
	-- end
	if self.animationID == AnimationType.keep and self.m_isDelayDie then
		return;
	end

	local timeScale = 1/FightManager:SpeedUp()

	self:PlayFightSound(id);
	if (id == AnimationType.attack) or (id == AnimationType.hit) or (id == AnimationType.die) or (id == AnimationType.skill) then
		-- Fighter.super.SetAnimation(self, id, 1, timeScale);
		if id == AnimationType.skill then
			self.activeRelease = false;
			EventManager:FireEvent(EventType.FightPanelScale, self, self.releaseSkill); --高亮
			EventManager:FireEvent(EventType.FightShowSkillExt, self:GetID(), 1); --footer动画

			FightManager:SkillPause();--暂停战斗
			
			AudioManager:playEffect(Conf.skill , 1); --爆气+立绘音效
			Fighter.super.SetAnimation(self, id, 1, timeScale); --技能动作
			if self.releaseSkill then
				FightShowSkillManager:showSkill(self, self.releaseSkill); --爆气动画
			end

			local node = self:getArmature();
			node:runAction(cc.Sequence:create(

				cc.DelayTime:create(0.33*FightManager:SpeedUp()), --0.33 2/3 0.5
				cc.CallFunc:create(function()
					SkillScriptManager:Pause();--技能脚本暂停
					Fighter.super.StopAnimation(self, id);

					-- if self.releaseSkill then
					-- 	FightShowSkillManager:showSkill(self, self.releaseSkill);
					-- end
					self.releaseSkill = nil;
				end)

				))
		else
			Fighter.super.SetAnimation(self, id, 1, timeScale);

		end
	else
		Fighter.super.SetAnimation(self, id, -1, timeScale);
	end
end

function Fighter:continuePlay()
	Fighter.super.ContinuePlayAnimation(self, AnimationType.skill);
end

function Fighter:PlayHitSound()
	if self:IsEnemy() then
		return;
	end
	if not self.lastPlayHitTime then
		self.hitSecond = 30;
		-- AudioManager:playCVStopBefor(self.resID ,cvTypeEnum.CV_ATTACK );

		self.lastPlayHitTime = self.time;
	else
		if self.time > self.lastPlayHitTime + (self.hitSecond/10) then
			if self.hitSecond > 5 then
				self.hitSecond = math.floor(self.hitSecond * 0.66);
			end
			-- AudioManager:playCVStopBefor(self.resID ,cvTypeEnum.CV_ATTACK );
			self.lastPlayHitTime = self.time;
		end
	end
end

function Fighter:PlayFightSound(id)
	-- if self:IsEnemy() then
	-- 	return;
	-- end
	if id == AnimationType.hit then
		AudioManager:playCVStopBefor(self.resID,nil,cvTypeEnum.CV_HIT, nil, true );
	elseif id == AnimationType.die then
		AudioManager:playCVStopBefor(self.resID ,nil ,cvTypeEnum.CV_UNABLE, nil, true );
	end
end

return Fighter;
