--fighterPropertyData.lua
--============================================================================
--战斗者的属性数据
local FighterPropertyData = class("FighterPropertyData");
local Attribute = Enum:getTable('Attribute');

function FighterPropertyData:ctor(actor)
	self.fighter                   = actor;      --角色

	self.Property = {}
	
	--战斗属性
	for k, v in pairs(Attribute) do
		self.Property[v] = {};
		if v == 'hp' then
			self.Property[v]['ori']		= '0';
			self.Property[v]['cur']		= '0';
			self.Property[v]['max']		= '0';
			self.Property[v]['extra']	= '0';
		elseif v == 'atk' then
			self.Property[v]['ori']		= '0';
			self.Property[v]['cur']		= '0';
			self.Property[v]['extra']	= '0';
		else
			self.Property[v]['ori']		= 0;
			self.Property[v]['cur']		= 0;
			self.Property[v]['extra']	= 0;
		end
	end
end

--初始化玩家基本数据
function FighterPropertyData:InitPlayerBaseData()
end

function FighterPropertyData:SetExtra(data)
	for k, v in pairs(data) do
		if self.Property[k]['extra'] then
			self.Property[k]['extra'] = v;
		end
	end
end

--初始化玩家数据
function FighterPropertyData:InitPlayerFightData(data)
	data = data.pro

	for k, v in pairs(Attribute) do
		if data[v] and self.Property[v] then
			if v == 'hp' then
				self.Property[v]['ori'] = ConvertDigitUnitToStr(data[v])
				self.Property[v]['cur'] = self.Property[v]['ori']
				self.Property[v]['max'] = self.Property[v]['ori']
			elseif v == 'atk' then
				self.Property[v]['ori'] = ConvertDigitUnitToStr(data[v])
				self.Property[v]['cur'] = self.Property[v]['ori']
			else
				self.Property[v]['ori']	= data[v];
				self.Property[v]['cur']	= data[v];
			end
		end
	end

	if data.def_type then
		self.Property['def']['type'] = data.def_type;
	end
end

function FighterPropertyData:setPropertyCoefficient(v, coefficient)
	if v == 'hp' then
		self.Property[v]['ori'] = HpMultiply(self.Property[v]['ori'], (1+coefficient))
		self.Property[v]['cur'] = self.Property[v]['ori']
		self.Property[v]['max'] = self.Property[v]['ori']
	elseif v == 'atk' then
		self.Property[v]['ori'] = HpMultiply(self.Property[v]['ori'], (1+coefficient))
		self.Property[v]['cur'] = self.Property[v]['ori']
	else
		self.Property[v]['ori'] = self.Property[v]['ori'] * (1+coefficient)
		self.Property[v]['cur'] = self.Property[v]['ori']
	end
end

--初始化怪物数据
function FighterPropertyData:InitMonsterData(level, roundType)
	local monsterData = {};
	monsterData = resTableManager:GetRowValue(ResTable.monster, self.fighter:GetResID());
	self:initMonsterBaseProperty(monsterData);          --初始化基本属性
	self:initNormalMonsterFightProperty(level, roundType);    --初始化战斗属性
end

--初始化Boss数据
function FighterPropertyData:InitBossData(level, roundType)
	local bossData = {};
	bossData = resTableManager:GetRowValue(ResTable.monster, self.fighter:GetResID());
	self:initMonsterBaseProperty(bossData);            --初始化基本属性
end

--初始化Boss战斗属性
function FighterPropertyData:InitBossFightData(bossData)
end

--初始化Boss的战斗属性
function FighterPropertyData:initBossFightData(level, roundType)
end

--初始化敌方的基本属性
function FighterPropertyData:initMonsterBaseData(monsterData)
end

--初始化敌方的战斗属性
function FighterPropertyData:initMonsterFightData(level, roundType)
end

--=====================================================================================
--Set方法

--设置怪物的最大生命值
function FighterPropertyData:SetMaxHp(maxHP)
	self.Property['hp']['max'] = maxHp;
end

--调整血量
function FighterPropertyData:AdjustHP()
	if HpCompare(self.Property['hp']['cur'], '>', self.Property['hp']['max']) then
		self.Property['hp']['cur'] = self.Property['hp']['max'];
	end
	if HpCompare(self.Property['hp']['cur'], '<=', 0) then
		self.Property['hp']['cur'] = '0';
	end
end

function FighterPropertyData:SetCurrentHP(hp)
	self.Property['hp']['cur'] = hp;
	self:AdjustHp();
end

--=====================================================================================
--Get方法
--获取当前生命值
function FighterPropertyData:GetCurrentHP()
	return self.Property['hp']['cur'];
end

--获取最大生命值
function FighterPropertyData:GetMaxHP()
	return self.Property['hp']['max'];
end

-- 获取当前属性值
function FighterPropertyData:GetCurrentPropertyData(propertyType)
	if self.Property[propertyType] then
		return self.Property[propertyType]['cur'];
	end
	if propertyType == 'maxHp' then
		return self.Property['hp']['max'];
	end
	if propertyType == 'def_type' then
		return self.Property['def']['type'];
	end
	return 0
end

function FighterPropertyData:GetOriPropertyData(propertyType)
	if self.Property[propertyType] then
		return self.Property[propertyType]['ori'];
	end
	return 0
end

--atk aspd crit crit_dam
--改变buff相关的属性值
function FighterPropertyData:IncreasePropertyData(propertyType, value)
	print("=== Increase " .. propertyType .. " ===" )
	print("before: " .. self.Property[propertyType]['cur']..'  value:', value)
	if propertyType == 'atk' then
		self.Property[propertyType]['cur'] = HpPlus(self.Property[propertyType]['cur'], value);
	elseif propertyType == 'aspd' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] + value;
	elseif propertyType == 'crit' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] + value;
	elseif propertyType == 'crit_dam' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] + value;
	else
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] + value;
	end
	if propertyType ~= 'atk' and propertyType ~= 'hp' then
		if math.abs(self.Property[propertyType]['cur'])<0.0001 then
			self.Property[propertyType]['cur'] = 0
		end
	end
	print("after: " .. self.Property[propertyType]['cur'])
end

function FighterPropertyData:DecreasePropertyData(propertyType, value)
	print("=== Decrease " .. propertyType .. " ===" )
	print("before: " .. self.Property[propertyType]['cur']..'  value:', value)
	if propertyType == 'atk' then
		self.Property[propertyType]['cur'] = HpMinus(self.Property[propertyType]['cur'], value);
	elseif propertyType == 'aspd' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] - value;
	elseif propertyType == 'crit' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] - value;
	elseif propertyType == 'crit_dam' then
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] - value;
	else
		self.Property[propertyType]['cur'] = self.Property[propertyType]['cur'] - value;
	end
	if propertyType ~= 'atk' and propertyType ~= 'hp' then
		if math.abs(self.Property[propertyType]['cur'])<0.0001 then
			self.Property[propertyType]['cur'] = 0
		end
	end
	print("after: " .. self.Property[propertyType]['cur'])
end

--==========================================================================
--其他函数
--添加受到的伤害， 如果角色死亡返回true，角色没有死亡返回false
function FighterPropertyData:DecreaseHp(hpValue)
	self.beforeHp = self.Property['hp']['cur'];
	self.Property['hp']['cur'] = HpMinus(self.Property['hp']['cur'], hpValue);
	self:AdjustHP();
	self.afterHp = self.Property['hp']['cur'];
	return HpCompare(self.afterHp, '<=', 0);
end

--恢复血量
function FighterPropertyData:RecoverHP(hpValue)
	self.Property['hp']['cur'] = HpPlus(self.Property['hp']['cur'], hpValue);
	self:AdjustHP();
end

function FighterPropertyData:getBeforeAfterHp()
end

function FighterPropertyData:setCurHp(percent)
	self.Property['hp']['cur'] = HpMultiply(self.Property['hp']['max'], percent);
end

function FighterPropertyData:getHpPercent()
	if self.Property['hp']['max'] ~= 0 then
		return HpDivide(self.Property['hp']['cur'], self.Property['hp']['max']);
	else
		return 1;
	end
end

function FighterPropertyData:getWillHpPercent(missVal)
	if self.Property['hp']['max'] ~= 0 then
		local lastVal = HpMinus(self.Property['hp']['cur'], missVal);
		return HpDivide(lastVal, self.Property['hp']['max']);
	else
		return 1;
	end
end

function FighterPropertyData:setCurHpNum(num)
	self.Property['hp']['cur'] = num;
end

function FighterPropertyData:getHpNum()
	return self.Property['hp']['cur'];
end

function FighterPropertyData:getCurHp()
end

function FighterPropertyData:setEventPower()
	self.Property['cAtt']['ori']		= self.Property['cAtt']['ori']*3;
	self.Property['cAtt']['cur']		= self.Property['cAtt']['cur']*3;

	self.Property['atAtt']['ori']		= self.Property['atAtt']['ori']*3;
	self.Property['atAtt']['cur']		= self.Property['atAtt']['cur']*3;

	self.Property['gAtt']['ori']		= self.Property['gAtt']['ori']*3;
	self.Property['gAtt']['cur']		= self.Property['gAtt']['cur']*3
end
return FighterPropertyData;
