local BaseLayer = class("BaseLayer")

local creatorFileNameList =	{
	MainScene = 'creator/Scene/main_scene.ccreator';
}

function BaseLayer:ctor(scene, sceneName, name)
	self.scene = scene;
	self.sceneName = sceneName;
	self.name = name;

	self.haveInitialize = false;
	self.useCCreator = false;
	self.haveSwallowLayer = true;
	self.loadAnim = false;
	self.isPopup = false;
	self.hasBase = true;
	self.args = {}

	self.swallowCount = 0;
end

function BaseLayer:Initialize(initializeCallBack)
	self.haveInitialize = true;
	local root;
	local posX, posY = self.posX or 0, self.posY or 0;

	if self.useCCreator then
		print('==================error :use creator ,need remove ===== name = ',self.name);
	else
		root = cc.Node:create();
		root:setCascadeOpacityEnabled(true);
		root:setName(self.name);
	end
	self.posX = posX
	self.posY = posY
	
	root:setPosition(StageSize.width/2 + posX, StageSize.height/2 + posY);	
	root:addTo(self.scene);
	self.root = root;
	UIManager:InitPanelZorder(self.name, root);
	if self.preLoad then
		self.checkLoaded = function()
			return true;
		end
		self.initializeCallBack = initializeCallBack;
		UIManager:push('PreparePanel');
		self:preLoad();
	else
		self:Init(root);
		if initializeCallBack then
			initializeCallBack();
		end
	end
end

function BaseLayer:checkLoadFinish()
	if self.checkLoaded() then
		UIManager:pop('PreparePanel');
		self:Init(self.root);
		if self.initializeCallBack then
			self.initializeCallBack();
			self.initializeCallBack = nil;
		end
	end
end

function BaseLayer:isDataInit()
	return true
end

function BaseLayer:reqData(callback)
end

function BaseLayer:onBaseEnter( ... )
	if not self.isPopup then
		if self.showHeaderType then
			UIManager:push("HeaderPanel" , self.name , self);
			--EventManager:FireEvent(EventType.UIHeaderPanelBackEvent, handler(self, self.onClickBack));
		else
			UIManager:pop("HeaderPanel");
		end
		if self.showFooter then
			UIManager:push("FooterPanel" , self.name);
		else
			UIManager:pop("FooterPanel");
		end
	else
		if self.name ~= 'FooterPanel' and self.root:isVisible() == true then
			if self.name == 'HeaderPanel' then
				EventManager:FireEvent(EventType.UIHeadeUpdateHeadType,...);
			end
			--if self.name == 'FooterPanel' then
			--	EventManager:FireEvent(EventType.UIFooterUpdateZOrder,...);
			--end
			if self.name == 'GuidePanel' then
				self:addSwallowLayer();
				self:onEnter(...);
			end
			return;
		end
	end
	if self.haveSwallowLayer then
		self:addSwallowLayer();
	end	
	local args = { ... };
	if #args > 0 then
		self.args = args
	end
	if not self:isDataInit() then
		self:reqData(function()
			self:onEnter( unpack(self.args) )
		end)
	else
		self:onEnter( unpack(self.args) );
	end
end

function BaseLayer:onUpdateArgs(idx, val)
	self.args[idx] = val;
end

function BaseLayer:onBaseResume(...)
	if not self.isPopup then
		if self.showHeaderType then
			UIManager:push("HeaderPanel" , self.name , self);
		else
			UIManager:pop("HeaderPanel");
		end
		if self.showFooter then
			UIManager:push("FooterPanel" , self.name);
		else
			UIManager:pop("FooterPanel");
		end
	else
		if self.root:isVisible() == true then
			if self.name == 'HeaderPanel' then
				EventManager:FireEvent(EventType.UIHeadeUpdateHeadType,...);
			end
			if self.name == 'FooterPanel' then
				EventManager:FireEvent(EventType.UIFooterUpdateZOrder,...);
			end
			return;
		end
	end
	
	if self.haveSwallowLayer then
		self:addSwallowLayer();
	end	

	self:onResume(...);
end

function BaseLayer:onBaseLeave()
	if self.haveSwallowLayer then
		self:removeSwallowLayer();
	end
	self:onLeave();
end

function BaseLayer:addSwallowLayer()
	if self.swallowCount > 0 then
		return;
	end
	self.swallowCount = self.swallowCount + 1;
	-- layer    
	self.swallowLayer = cc.LayerColor:create(cc.c4b(0, 0, 0, (self.isPopup and 210 or 0)));
	self.swallowLayer:setContentSize( cc.size(StageSize.width, StageSize.height) );
	self.swallowLayer:setPosition(-StageSize.width/2, -StageSize.height/2);
	self.swallowLayer:setLocalZOrder(-100);
	self.swallowLayer:addTo(self.root);

	-- listener
	self.swallowListener = cc.EventListenerTouchOneByOne:create();
	local onTouchBegan = function(tap, event)
		return true;
	end
	local onTouchEnd = function()
		if self.swallowCallback then
			if self.isPopup then
				AudioManager:playEffect(Conf.cancleButton);
			end
			self.swallowCallback();
		end
	end
	self.swallowListener:setSwallowTouches(true);
	self.swallowListener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN);
	self.swallowListener:registerScriptHandler(onTouchEnd, cc.Handler.EVENT_TOUCH_ENDED);
	local eventDispatcher = self.swallowLayer:getEventDispatcher();
	eventDispatcher:addEventListenerWithSceneGraphPriority(self.swallowListener, self.swallowLayer);
end

function BaseLayer:removeSwallowLayer()
	if self.swallowCount <= 0 then
		return;
	end
	self.swallowCount = self.swallowCount - 1;
	self.swallowLayer:getEventDispatcher():removeEventListener(self.swallowListener);
	self.swallowListener = nil;
	self.swallowCallback = nil;

	self.swallowLayer:removeFromParent();
end

function BaseLayer:addSwallowCallback(callback)
	self.swallowCallback = callback;
end

function BaseLayer:isVisible( )
	return self.root:isVisible();
end

return BaseLayer;
