local ArenaMultRewardPanel = class('ArenaMultRewardPanel' , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont")
local imagePath = 'image/arena/arenaReward/';

function ArenaMultRewardPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'ArenaMultRewardPanel');
    self.isPopup = true;
end

function ArenaMultRewardPanel:Init(root)
    self.root = root;
    self.root:setVisible(false);
    self:initPanel()
end

function ArenaMultRewardPanel:initPanel(  )

    local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(606,629));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , -16);
    self.bg = bg;

	local titleNode =  GlobalFunctions:createTitle(LANG.UI_L_REWARD_1 , 606 , 63 , LANG.UI_L_REWARD_2 , true);
	titleNode:addTo(self.root);
    titleNode:setPosition(0 , 329);
    
    local bg = cc.Scale9Sprite:create('image/commonImg/bg_3_11_4_33_20.png');
	bg:addTo(self.root);
	bg:setPosition(0 , 10);
    bg:setCapInsets(cc.rect(11,4,33,20));
	bg:setContentSize(cc.size(580 , 580));

    self.qualityBg = ccui.ImageView:create('image/commonImg/item_quality_2.png') --cc.p(103,103)
        :addTo(self.root)
        :setPosition(-200, 220)
    self.playerIcon = cc.Sprite:create('image/icon/R012_Icon.png')
    self.playerIcon:addTo(self.root)
    self.playerIcon:setPosition(-200, 220)
    self.playerIcon:setScale(0.8)

    local btnLabel = cc.Label:createWithTTF('奖励预览', fonts.FONT_SIYUAN, 20)
        :enableOutline(cc.c3b(55 , 85 , 155) , 2);
    local chatBtn = ccui.Button:create('image/commonImg/btn_11.png')
        :addTo(self.root)
        :setPosition(cc.p(200,200))
        :setTitleLabel(btnLabel)
        :addClickEventListener(handler(self , self.btnClick));  




    local listNode = cc.Node:create()
    listNode:addTo(self.root)
    listNode:setPosition(0, 80)

	local listBg = cc.Sprite:create('image/arena/arenaMult/am_rwd_bg.png');
	listBg:addTo(listNode);

    local arrowIcon = cc.Sprite:create('image/commonImg/arrow_bg.png')
    arrowIcon:addTo(listNode)
    arrowIcon:setPosition(-240, 60)

    local t1Label = cc.Label:createWithTTF('赛季奖励' ,  fonts.FONT_SIYUAN , 22)
    t1Label:addTo(listNode);
    t1Label:setAnchorPoint(0 , 0.5);
    t1Label:setPosition(-220 , 60);

    local t2Label = cc.Label:createWithTTF('赛季奖励可领取:' ,  fonts.FONT_SIYUAN , 20)
    t2Label:addTo(listNode);
    t2Label:setAnchorPoint(0 , 0.5);
    t2Label:setPosition(-220 , -40);
    -- t1Label:setTextColor(cc.c3b(255,255,0));

    self.rankLabel = cc.Label:createWithTTF('第123名' ,  fonts.FONT_SIYUAN , 22)
    self.rankLabel:addTo(listNode);
    self.rankLabel:setAnchorPoint(1 , 0.5);
    self.rankLabel:setPosition(250 , 60);
    -- self.rankLabel:setTextColor(cc.c3b(255,255,0));

    self.rwdNode = cc.Node:create()
    self.rwdNode:addTo(listNode)
    self.rwdNode:setPosition(220, -20)

    self.itemList = {}
    for j = 1 , 2 do
        local itemCell = template.ItemTemplate:create(2);
        itemCell:addTo(self.rwdNode);
        itemCell:setPosition(0 - 100 * (j - 1) , 0);
        itemCell:setScale(0.7)
        self.itemList[j] = itemCell;
    end

    local initData = {
		width = 554,
		height = 270,
		cellWidth = 160 ,
		cellHeight = 270,
		direction = 0,
		cellReloadFunc = handler(self,self.reloadCell),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

	}
	self.rankRewardListView = template.TableViewTemplate:create()
	self.rankRewardListView:initWithSizeAndItem(initData);
	self.rankRewardListView:getView():setPosition(-277, -270);
	self.rankRewardListView:getView():addTo(self.root)


    self.curRank = 0;

end

function ArenaMultRewardPanel:onEnter()
    self:addSwallowCallback(handler(self , self.onClickBack))
    self:updateData();
    self.root:setVisible(true);
end

function ArenaMultRewardPanel:updateData()
    local arenaData = model.ArenaModel:getMultArenaInfo();

    local iconNo = model.UserModel:getHeadIcon() or 1;
    local headIconFile = GlobalFunctions:getHeadIconFile(iconNo)
    self.playerIcon:setTexture(headIconFile);

    if arenaData.rank<=0 then
        self.rankLabel:setString('未上榜')
    else
        self.rankLabel:setString('第'..arenaData.rank..'名')
    end

    local rwdItem = nil;
    local maxRank = 0;
    local itemRwdDatas = ResTable:getTable('multiple_arena_reward').contain;
    for k,v in pairs(itemRwdDatas) do
        if v.rank[2] > maxRank then
            rwdItem = v;
            maxRank = v.rank[2];
        end
        if v.rank[1]<=arenaData.rank and arenaData.rank<=v.rank[2] then
            rwdItem = v;
            maxRank = v.rank[2];
            break;
        end
    end

    if rwdItem then
        for k,v in pairs(rwdItem.reward_season) do
            if self.itemList[k] then
                self.itemList[k]:Init(v[1], v[2])
            end
        end
    end


    local rewardDatas = ResTable:getTable('multiple_arena_rank_reward').contain;
    self.tabData = {}
    for k,v in pairs(rewardDatas) do
        table.insert(self.tabData, v)
    end
    table.sort(self.tabData , function(first , second)
        return first.reward > second.reward;
    end);
    local jumpToIndex = 1;
    self.rankRewardListView:reloadData(self.tabData);

    --if self.rewardType == 1 and jumpToRewardIndex > 0 then
    --   jumpToIndex = jumpToRewardIndex;
    --end
    -- self.rankRewardListView:scrollToCellIndex(jumpToIndex - 1);

end

function ArenaMultRewardPanel:reloadCell(cell , index)
    local cellNode  = cell:getChildByTag(999);
	if nil == cellNode then
	 	cellNode = template.ArenaMultRwdItem:create(cell);
		cellNode:setPosition(80, 135);
		cellNode:setTag(999);
    end
    local data = self.tabData[index + 1];
    cellNode:initData(data);

end

function ArenaMultRewardPanel:cellItemTouch(table,cell)
end

function ArenaMultRewardPanel:btnClick()
    UIManager:pop('ArenaMultRewardPanel');
    UIManager:push('ArenaMultRwdListPanel')
end

function ArenaMultRewardPanel:onLeave()
    self.root:setVisible(false);
end
function ArenaMultRewardPanel:onClickBack()
    UIManager:pop('ArenaMultRewardPanel');
end

return ArenaMultRewardPanel