local BareCollectSelectPanel = class("BareCollectSelectPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

function BareCollectSelectPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'BareCollectSelectPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function BareCollectSelectPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function BareCollectSelectPanel:initPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_5.png');
	self.bg:addTo(self.root);

	local viewBg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	viewBg:addTo(self.root);
	viewBg:setPosition(0 , 2);
	viewBg:setCapInsets(cc.rect(11,4,18,21));
	viewBg:setContentSize(cc.size(604 , 884));

	local line = cc.Sprite:create('image/commonImg/line.png');
	line:addTo(self.root);
	line:setContentSize(cc.size(604,1))
	line:setPosition(cc.p(0,-407));

	local initData = {
        width = 618,
        height = 842,
        cellWidth = 604 ,
        cellHeight = 425,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData) ;
    self.cardListView:getView():setPosition(cc.p(-302,-404));
	self.cardListView:getView():addTo(self.root)
	
	self:registEvent();
end

function BareCollectSelectPanel:registEvent()
    EventManager:RegisterEvent(EventType.NetworkRetBareCollectWear,handler(self,self.retBareCollectWear));
end

function BareCollectSelectPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 3 + 1;
	local curMaxNum = (idx+1) * 3;
	curMaxNum = curMaxNum > self.bareMaxNum and self.bareMaxNum or curMaxNum;
	local children = cell:getChildren();
	if #children <= 0 then
		for i = 1 , 3 do
			local x = 101 + 201*(i-1);
			local y = 212.5;
			local bigCard = template.CardTemplate:create(cell);
			bigCard:setPosition(x,y);
			bigCard:setTouchEnabled(false);
			bigCard:addCardClickEventListener(handler(self, self.bareCollectClick));
			if startNum <= curMaxNum then
				local bare = self.bareCollects[startNum];
				bigCard:InitBareCollect(bare);
				bigCard:setCardTag(bare.bare_id);
				startNum = startNum + 1;
			else
				bigCard:setVisible(false);
			end
		end
	else
		for i = 1 , 3 do
			local bigCard = children[i];
			if startNum <= curMaxNum then
				local bare = self.bareCollects[startNum];
				bigCard:InitBareCollect(bare);
				bigCard:setCardTag(bare.bare_id);
				bigCard:setVisible(true);
				startNum = startNum + 1;
			else
				bigCard:setVisible(false);
			end
		end
	end
	return cell;
end

function BareCollectSelectPanel:cellItemTouch(table)
	if self.curBareId then
		local msg = {};
    	msg.bare_id = self.curBareId;
    	msg.pid  = self.curPid;
		Network.NetworkManager:Send(Network.NetworkCmdType.req_bare_collect_wear, msg);
		self.curBareId = nil;
	end
end

function BareCollectSelectPanel:bareCollectClick(sender , event)
	self.curBareId = sender:getTag();
end

function BareCollectSelectPanel:onEnter(pid)
	local card = model.CardModel:GetCard(pid);

	if not card then
		UIManager:pop('BareCollectSelectPanel');
		return
	end

	local roleId = ResTable:getValue('role' , card.no , 'role_id');

	self.bareCollects = model.BareCollectModel:getBareDataByRoleId(roleId);

	if #self.bareCollects > 1 then
		table.sort(self.bareCollects, function(first, second)
			return first.bare_id < second.bare_id;
		end);
	end

	self.bareMaxNum = #self.bareCollects;
	local tableViewCellMaxNum = math.ceil(self.bareMaxNum/3);

	self.cardListView:setDataCount(tableViewCellMaxNum);
	self.cardListView:reloadData();

	self.curBareId = nil;
	self.curPid = pid;

	self.root:setVisible(true);
end

function BareCollectSelectPanel:retBareCollectWear()
	self:onClickBack();
end

function BareCollectSelectPanel:onClickBack()
	self.curBareId = nil;
	UIManager:pop('BareCollectSelectPanel');
end

function BareCollectSelectPanel:onLeave()
	self.root:setVisible(false);
end

return BareCollectSelectPanel;
