local BareCollectViewPanel = class("BareCollectViewPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/bareCollect/'
local cvTypeEnum = Enum:getTable("cvTypeEnum");

function BareCollectViewPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'BareCollectViewPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = false;
end

function BareCollectViewPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function BareCollectViewPanel:initPanels()

	local bg = cc.Sprite:create('image/bg/Bg_6.png');
	bg:addTo(self.root);

	local bg = cc.Sprite:create(imagePath..'bare_collect_1.png');
	bg:addTo(self.root);
	bg:setPosition(0 , 14);

	self.cardPanelNavi = cc.Sprite:create();
	self.cardPanelNavi:addTo(self.root);
	self.cardPanelNavi:setPosition(0, 26);

	self.skeletonNode = cc.Node:create();
	self.skeletonNode:addTo(self.root)
	self.skeletonNode:setPosition(0, 26)

	local roleIconBg = cc.Sprite:create(imagePath..'bare_collect_6.png');
	roleIconBg:addTo(self.root);
	roleIconBg:setPosition(0,-385);

	self.roleIcon = cc.Sprite:create();
	self.roleIcon:addTo(self.root);
	self.roleIcon:setPosition(-2 , -388);

	self.roleName = cc.Label:createWithTTF('0' , fonts.FONT_SIYUAN , 25);
	self.roleName:setPosition(0 , -490);
	self.roleName:addTo(self.root);

	local bg = cc.Sprite:create(imagePath..'bare_collect_4.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -511);

	self.infoLabel = cc.Label:createWithTTF('0' , fonts.FONT_SIYUAN , 16);
	self.infoLabel:setPosition(0 , -528);
	self.infoLabel:addTo(self.root);

	self.infoIcon = cc.Sprite:create('image/commonImg/def_type_1.png')
	self.infoIcon:addTo(self.root)
	self.infoIcon:setPosition(0, -528)
	self.infoIcon:setScale(0.5)
	self.infoIcon:setVisible(false)
	
	self.swtichBtn = ccui.Button:create('image/card/live_switch_off.png' ,'image/card/live_switch_off.png' , 'image/card/live_switch_on.png');
	self.swtichBtn:addTo(self.root);
	self.swtichBtn:setPosition(227 ,StageSize.height/2 - 88);
	self.swtichBtn:setTag(1);
	self.swtichBtn:addClickEventListener(handler(self , self.swtichBtnClick));
end

function BareCollectViewPanel:onEnter(bareId)

	local bareData = model.BareCollectModel:getBareData(bareId);
	if not bareData then
		UIManager:pop('BareCollectViewPanel');
		return;
	end
	self.role_id = bareData.role_id
	local resData = ResTable:getRowData('role_resource',bareData.resource_id);
	local imgPath = resData.card
	self.spine_res = resData.spine;
	AudioManager:preloadCVFile(self.role_id)
	self.cardPanelNavi:setTexture('image/role/'..imgPath..'.png');
	self.roleIcon:setTexture('image/icon/'..resData.icon..'.png');

	local roleInfo = ResTable:getRowData('role_info' , self.role_id);
	if roleInfo then
		self.roleName:setString(tostring(roleInfo.name));

		local str = '';
		local att = bareData.attribute_effect;
		if att[2] > 0 then
			str = string.format(LANG.UI_CARD_LOVE_ATT_NAME[att[1]],tostring(att[2]));
		end
		if att[3] > 0 then
			local str1 = string.format(LANG.UI_CARD_LOVE_ATT_NAME[att[1]],(tostring(att[3] * 100)..'%'));
			if att[2] > 0 then
				str = str..', '..str1;
			else
				str = str1;
			end
		end
		
		if att[2]>0 or att[3] > 0 then
			str = string.format(LANG.UI_BARE_COLLECT_16, str);
			self.infoIcon:setVisible(false)
		else
			att = bareData.attribute_effect2;
			local colorName = LANG.UI_GROUP_NAME[bareData.attribute_color]
			if att[2] > 0 then
				str = string.format(LANG.UI_CARD_BARE_COLOR_ATT_NAME[att[1]],tostring(att[2]));
			end
			if att[3] > 0 then
				local str1 = string.format(LANG.UI_CARD_BARE_COLOR_ATT_NAME[att[1]],(tostring(att[3] * 100)..'%'));
				if att[2] > 0 then
					str = str..', '..str1;
				else
					str = str1;
				end
			end

			str = string.format(LANG.UI_BARE_COLLECT_17, colorName, str);

			self.infoIcon:setTexture('image/commonImg/def_type_'..bareData.attribute_color..'.png')
			self.infoIcon:setVisible(true)
		end

		self.infoLabel:setString(str);
		self.infoIcon:setPosition(-20-self.infoLabel:getContentSize().width/2,-528)
	end

	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end

	if self.spine_res ~= nil then
		if cc.FileUtils:getInstance():isFileExist('spine/'..self.spine_res.."/"..self.spine_res..".json") then
			self:addSpineAnimation()
			-- self:switchState(1)
		else
			DownloadControler:addDownLoadList(self.spine_res, 'spine/',  function()
					self:addSpineAnimation()
					self:switchState(1)
					tools.RedDotTools:setRedDotToNode(self.swtichBtn,false)
				end)
			-- self:switchState(2)
		end
		-- self.swtichBtn:setEnabled(true)

	else
		-- self.swtichBtn:setEnabled(false)
		-- self:switchState(2)
	end
	self.root:setVisible(true);

	self:addSwallowCallback(handler(self, self.onTouchEvent));
	self.canTouch = true;
	self:switchState(2)

end

function BareCollectViewPanel:onTouchEvent( ... )
	if self.canTouch==true then
		self.cvSoundId =  AudioManager:playCVStopBefor(self.role_id ,nil,cvTypeEnum.CV_PURIFIED );
		self.canTouch = false;
		self.root:runAction(cc.Sequence:create(cc.DelayTime:create(0.5),cc.CallFunc:create( function()
			self.canTouch = true;
		end )))
	end
end

function BareCollectViewPanel:addSpineAnimation()
	-- body
	if self.spine_res == nil or self.spine_res == '' then
		return ;
	end
	local spine_res = self.spine_res;

	if spine_res~=nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then

		if self.skeletonNode then
			self.skeletonNode:removeAllChildren();
		end

		local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		skAnimation:addTo(self.skeletonNode)
		skAnimation:setAnimation(0, "idle", true)
		-- registerEvent
		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_START)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_END)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_COMPLETE)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_EVENT)

		-- self:switchState(1);
	end
end
function BareCollectViewPanel:swtichBtnClick(sender , event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	self:switchState(tag)
end

function BareCollectViewPanel:switchState(state)

	if state == 1 and self.skeletonNode:getChildrenCount() == 0 then
		Prompt:Show(LANG.UI_CARD_2);
		return
	end

	self.swtichBtn:setTag(state == 1 and 2 or 1);
	self.swtichBtn:setBright(state == 2)

	self.cardPanelNavi:setVisible(state == 2);

	if self.skeletonNode:getChildrenCount() > 0 then
		self.skeletonNode:setVisible(state == 1);
	end

	-- self.swtichBtn:setBrightStyle(state == 1 and ccui.BrightStyle.highlight or ccui.BrightStyle.normal)
end

function BareCollectViewPanel:onLeave()
	if self.cvSoundId ~= nil and self.cvSoundId ~= -1 then
		AudioManager:stopEffect( self.cvSoundId )
		self.cvSoundId = -1;
	end

	self.isShowAnimation = false;
	self.root:stopAllActions();
	self.root:setVisible(false);
end


return BareCollectViewPanel;
