local BgSelectPanel = class("BgSelectPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont")

function BgSelectPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'BgSelectPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
	
end

function BgSelectPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function BgSelectPanel:initPanels()
	self.bg = cc.Sprite:create();
	self.bg:addTo(self.root);

	self.listNode = ccui.Layout:create();
	self.listNode:setContentSize(cc.size(640, 890));
	self.listNode:setTouchEnabled(false);
	self.listNode:setClippingEnabled(true);
	self.listNode:setPosition(-320 , -408);
	self.listNode:addTo(self.root);

	EventManager:RegisterEvent(EventType.UIBgSelectUpdateBg , handler(self, self.updateBg));
end

function BgSelectPanel:registerEventListener()
	self.click = true
	self:unRegisterEventListener();

	self.eventTouchListener = cc.EventListenerTouchOneByOne:create();
	local onTouchBegan = function(touch, event)
		local pos = touch:getLocation()
		local nodePos = self.listNode:getParent():convertToNodeSpace(pos)
		local rect = cc.rectContainsPoint(self.listNode:getBoundingBox(),nodePos);
		if cc.rectContainsPoint(self.listNode:getBoundingBox(),nodePos) then
			self.click = true
			self:setStopAutoScroll();
			self.touchMovePreviousTimestamp = math.floor(socket.gettime() * 1000) / 1000;
			return true;
		end

		return false;
	end
	local onTouchMove = function(touch, event)
		local delta = touch:getDelta();
		self:updateBgList(delta.y);
		self:gatherTouchMove(delta);
		
		if delta.y > 1.5 then
			self.click = false;
		end
	end

	local onTouchEnd = function(touch, event)
		local delta = touch:getDelta();
		self:gatherTouchMove(delta);
		self:update();
		self:checkAutoScroll()
	end

	local onTouchCancell = function(touch, event)
		self.click = false;
	end
	self.eventTouchListener:setSwallowTouches(true);
	self.eventTouchListener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN);
	self.eventTouchListener:registerScriptHandler(onTouchMove, cc.Handler.EVENT_TOUCH_MOVED);
	self.eventTouchListener:registerScriptHandler(onTouchEnd, cc.Handler.EVENT_TOUCH_ENDED);
	self.eventTouchListener:registerScriptHandler(onTouchCancell, cc.Handler.EVENT_TOUCH_CANCELLED);
	local eventDispatcher = cc.Director:getInstance():getEventDispatcher();
	eventDispatcher:addEventListenerWithSceneGraphPriority(self.eventTouchListener, self.listNode);

	local scheduler = cc.Director:getInstance():getScheduler();
	self.touchScheduleHandler = scheduler:scheduleScriptFunc(handler(self , self.update) , 0 , false);
	
	local scheduler = cc.Director:getInstance():getScheduler();
	self.timeScheduleHandler = scheduler:scheduleScriptFunc(handler(self , self.updateTime) , 1 , false);

end

function BgSelectPanel:unRegisterEventListener()

	if self.eventTouchListener then
		local eventDispatcher = cc.Director:getInstance():getEventDispatcher();
		eventDispatcher:removeEventListener(self.eventTouchListener);
		self.eventTouchListener = nil;
	end

	if self.touchScheduleHandler then
		local scheduler = cc.Director:getInstance():getScheduler();
		scheduler:unscheduleScriptEntry(self.touchScheduleHandler)
		self.touchScheduleHandler = nil
	end

	if self.timeScheduleHandler then
		local scheduler = cc.Director:getInstance():getScheduler();
		scheduler:unscheduleScriptEntry(self.timeScheduleHandler)
		self.timeScheduleHandler = nil
	end
end

function BgSelectPanel:setStopAutoScroll()
	self.touchMoveDisplacements={};
	self.touchMoveTimeDeltas={};
	self.autoScroll = false;
	self.autoScrollAccumulatedTime = 0;
	self.touchMovePreviousTimestamp = 0;
	self.scrollTotalTime = 0;
	self.targetDelta = {};
	self.subHeight = {};
	self.bgListStartPos = {};
end

function BgSelectPanel:gatherTouchMove(delta)
	if #self.touchMoveDisplacements >= 5 then
		table.remove(self.touchMoveDisplacements , 1);
	end
	
	if #self.touchMoveTimeDeltas >= 5 then
		table.remove(self.touchMoveTimeDeltas , 1);
	end

	table.insert(self.touchMoveDisplacements,delta)
	local curTime = math.floor(socket.gettime() * 1000) / 1000;
	local subTime = curTime - self.touchMovePreviousTimestamp;
	table.insert(self.touchMoveTimeDeltas,subTime);
	self.touchMovePreviousTimestamp = curTime;
end

function BgSelectPanel:calculateTouchMoveVelocity()

	local totalTime = 0;

	for _k , moveTime in pairs(self.touchMoveTimeDeltas) do
		totalTime = totalTime + moveTime;
	end

	if totalTime == 0 or totalTime > 0.5 then
		return {x = 0 , y = 0};
	end

	local totalMove = {x = 0 , y = 0};
	for _k , delta in pairs(self.touchMoveDisplacements) do
		totalMove.x = totalMove.x + delta.x;
		totalMove.y = totalMove.y + delta.y;
	end
	return {x = totalMove.x / totalTime , y = totalMove.y / totalTime};
end

function BgSelectPanel:checkAutoScroll()
	local touchMoveVelocity = self:calculateTouchMoveVelocity();

	if touchMoveVelocity.x == 0 and touchMoveVelocity.y == 0 then
		return
	end
	self.targetDelta = {}
	self.targetDelta.x = touchMoveVelocity.x * 0.7;
	self.targetDelta.y = touchMoveVelocity.y * 0.7;

	local length = math.sqrt(touchMoveVelocity.x * touchMoveVelocity.x + touchMoveVelocity.y * touchMoveVelocity.y);
	self.scrollTotalTime = math.sqrt(math.sqrt(length/5));

	self.bgListStartPos = {};
	for _k , bg in ipairs(self.bgList) do
		self.bgListStartPos[_k] = bg:getPositionY();
		self.subHeight[_k] = 0;
	end

	self.autoScroll = true;
	self.autoScrollAccumulatedTime = 0;
end

function BgSelectPanel:updateBgList(deltaY)
	for i , bg in pairs(self.bgList) do
		local curPosY = bg:getPositionY();
		local newPosY = curPosY + deltaY;

		if deltaY > 0 then
			if curPosY < 445 and newPosY > 445 then
				AudioManager:playEffect(Conf.buttonClick);
			end
		else
			if curPosY > 445 and newPosY < 445 then
				AudioManager:playEffect(Conf.buttonClick);
			end
		end

		if newPosY > self.maxY then
			newPosY = self.minY + newPosY - self.maxY;
		elseif newPosY < self.minY then
			newPosY = self.maxY - (self.minY - newPosY);
		end

		bg:setPositionY(newPosY);
		self:updateBgItemScale(bg)
	end
end

function BgSelectPanel:updateBgItemScale(bg)           
	local posY = bg:getPositionY();
	local scale = 1;
	if posY >= 0 and posY <= 890 then
		if posY > 445 then
			scale = 0.8 + (890 - posY) / 445 * 0.2
		elseif posY < 445 then
			scale = 0.8 + posY / 445 * 0.2
		end
	else
		scale = 0.8;
	end
	bg:setScale(scale);
end

function BgSelectPanel:update(dt)
	
	if self.autoScroll then
		self.autoScrollAccumulatedTime = self.autoScrollAccumulatedTime+dt;

		local pecent = math.min(1 , self.autoScrollAccumulatedTime/self.scrollTotalTime);
		pecent = pecent - 1;
		pecent = (pecent * pecent * pecent * pecent * pecent + 1);
		local stopAutoScroll = pecent >= 1;
		if stopAutoScroll then
			for _k , startPosY in ipairs(self.bgListStartPos) do
				local moveDeltaY = self.targetDelta.y;
				local newPosY = startPosY + moveDeltaY;

				if self.targetDelta.y > 0 then
					if newPosY > self.maxY then
						local lastDeltaY = moveDeltaY - (self.maxY - startPosY);
						local height = self.maxY - self.minY;
						local heightNum = math.floor(lastDeltaY / height);
						lastDeltaY = lastDeltaY -  height * heightNum;
						self.bgList[_k]:setPositionY(self.minY + lastDeltaY);
					else
						self.bgList[_k]:setPositionY(newPosY);
					end
					self:updateBgItemScale(self.bgList[_k])
				else
					if newPosY < self.minY then
						local lastDeltaY = moveDeltaY + (startPosY - self.minY);
						local height = self.maxY - self.minY;
						local heightNum = math.floor(math.abs(lastDeltaY) / height);
						lastDeltaY = lastDeltaY +  height * heightNum;
						self.bgList[_k]:setPositionY(self.maxY + lastDeltaY);
					else
						self.bgList[_k]:setPositionY(newPosY);
					end
					self:updateBgItemScale(self.bgList[_k])
				end
			end
			self:setStopAutoScroll();
		else

			for _k , startPosY in ipairs(self.bgListStartPos) do
				local moveDeltaY = (self.targetDelta.y * pecent);
				if self.targetDelta.y >= 0 then
					moveDeltaY = moveDeltaY - self.subHeight[_k]

					local newPosY = 0;
					if self.subHeight[_k] == 0 then
						newPosY = startPosY + moveDeltaY;
					else
						newPosY = self.minY + moveDeltaY;
					end

					local curPosY = self.bgList[_k]:getPositionY();

					if newPosY > self.maxY then
						if self.subHeight[_k] == 0 then
							self.subHeight[_k] = self.subHeight[_k] + self.maxY - startPosY;
						else
							self.subHeight[_k] = self.subHeight[_k] + self.maxY - self.minY;
						end
						local addPosY = newPosY - self.maxY;
						self.bgList[_k]:setPositionY(self.minY + addPosY);
					else
						self.bgList[_k]:setPositionY(newPosY);
					end
					if curPosY < 445 and newPosY > 445 then
						AudioManager:playEffect(Conf.buttonClick);
					end

					self:updateBgItemScale(self.bgList[_k])
				else
					moveDeltaY = moveDeltaY + self.subHeight[_k];

					local newPosY = 0;

					if self.subHeight[_k] == 0 then
						newPosY = startPosY + moveDeltaY;
					else
						newPosY = self.maxY + moveDeltaY;
					end

					local curPosY = self.bgList[_k]:getPositionY();

					if newPosY < self.minY then
						if self.subHeight[_k] == 0 then
							self.subHeight[_k] = self.subHeight[_k] + startPosY - self.minY;
						else
							self.subHeight[_k] = self.subHeight[_k] + self.maxY - self.minY;
						end
						local addPosY = self.minY - newPosY;
						self.bgList[_k]:setPositionY(self.maxY - addPosY);
					else
						self.bgList[_k]:setPositionY(newPosY);
					end

					if curPosY > 445 and newPosY < 445 then
						AudioManager:playEffect(Conf.buttonClick);
					end
					self:updateBgItemScale(self.bgList[_k])
				end
			end
		end
	end
end

function BgSelectPanel:onEnter()

	local bgData = model.UserModel:getMainBgList();
	local curBgId = model.UserModel:getMainBg()
	local bgDatas = {};
	local curBgData = {};
	if bgData then
		for id , data in pairs(bgData) do
			table.insert(bgDatas , data);

			if id == curBgId then
				curBgData = data;
			end
		end
	end

	self.bg:setTexture('image/background/' .. curBgData.res .. '.png');
	GlobalFunctions:setGaussianGary(self.bg)

	local tNum = #bgDatas;
	self.totalNum = tNum < 6 and 6 or tNum;

	self.maxY = 180 + 809;
	self.minY = 809 - 180 * (self.totalNum-1);

	if self.totalNum > 1 then
		table.sort(bgDatas , function(first , second)
			return first.id < second.id
		end);
	end

	self.listNode:removeAllChildren();

	self.bgList= {};
	for i = 1 ,self.totalNum do
		local y = 809 - 180*(i-1);
		local template = template.MainBgSelectTemplate:create();
		if bgDatas[i] then
			template:Init(bgDatas[i]);
			template:setBgTag(bgDatas[i].id)
		else
			template:Init(nil);
			template:setBgTag(-1)
		end
		template:setPosition(320 , y);
		template:addTo(self.listNode);
		template:addBgClickEventListener(handler(self , self.bgSelectClick))
		self.bgList[i] = template;
		self:updateBgItemScale(template)
	end

	self:registerEventListener();
	self.root:setVisible(true);
end

function BgSelectPanel:updateBg()
	local curBgId = model.UserModel:getMainBg()
	local bgData =  model.UserModel:getMainBgList(curBgId);
	self.bg:setTexture('image/background/' .. bgData.res .. '.png');
	GlobalFunctions:setGaussianGary(self.bg)
end

function BgSelectPanel:updateTime()
	for _k , bg in pairs(self.bgList) do
		local bgId = bg:getBgTag();
		if bgId ~= -1 then
			local bgData = model.UserModel:getMainBgList(bgId);
			if bgData.activateTime > 0 then
				local lastTime = bgData.activateTime - TimeManager:getCurTime()
				if lastTime <= 0 then
					model.UserModel:setBgTimeOut(bgId);
					bg:updateState(model.UserModel:getMainBgList(bgId));
					EventManager:FireEvent(EventType.UIBgSelectUpdateBg);
				end
				bg:updateTime(lastTime);
			end
		end
	end
end

function BgSelectPanel:bgSelectClick(sender , event)

	if self.click then
		local bgId = sender:getTag();
		if bgId == -1 then
			return;
		end

		local bgData = model.UserModel:getMainBgList(bgId);
		if bgData.activateTime >= 0 then
			model.UserModel:setMainBg(bgId)
			self.bg:setTexture('image/background/' .. bgData.res .. '.png');
			GlobalFunctions:setGaussianGary(self.bg)
			Prompt:Show(LANG.UI_USER_2..tostring(bgData.bg_name));
		end

		for _k , template in pairs(self.bgList) do
			local curBgId = template:getBgTag();
			if curBgId ~= -1 then
				template:updateUseState(curBgId);
			end
		end

		AudioManager:playEffect(Conf.confirmButton);
		EventManager:FireEvent(EventType.UIMainPanelMainBg);
	end
end
function BgSelectPanel:onClickBack()
	UIManager:pop('BgSelectPanel');
end
function BgSelectPanel:onLeave()
	self:unRegisterEventListener();
	self.root:setVisible(false);
end
return BgSelectPanel;
