local CardBookPanel = class("CardBookPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');
local imagePath = 'image/cardBook/';
local ALL_DEF_TYPE = 99;
function CardBookPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardBookPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function CardBookPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function CardBookPanel:initPanels()
	self.bg = cc.Sprite:create('image/bg/Bg_5.png');
	self.bg:addTo(self.root);

	local topHight = NOTCH_HEIGHT + 102
	local menuHeight = 46;
	local menuPosY = StageSize.height/2-topHight;
	local menuNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0,menuPosY)

	self.cardDefTypeMune = {};
    for i = 1 , 4 do

		local texture = 'image/commonImg/menu_left_71_46.png';
		local unTexture = 'image/commonImg/menu_left_71_46_u.png';

		if i == 2 or i == 3 then
			texture = 'image/commonImg/menu_mid_71_46.png';
			unTexture = 'image/commonImg/menu_mid_71_46_u.png';
		end
		
		self.cardDefTypeMune[i] = {};

		local node = cc.Node:create();
		node:addTo(menuNode);
		node:setPosition(-226.5 + 151*(i-1) , 0);

		local btn = ccui.Button:create(texture , texture , unTexture);
		btn:addTo(node);
		btn:setTag(i == 1 and ALL_DEF_TYPE or (i - 1));
		btn:addClickEventListener(handler(self , self.switchDefType));
		btn:setScale9Enabled(true);
		btn:setCapInsets(cc.rect(10 , 0 , 39 , 46));
		btn:setContentSize(cc.size(151 , 46));
		self.cardDefTypeMune[i].btn = btn;

		local light = cc.Sprite:create('image/commonImg/menu_light_1.png');
		light:addTo(node);
		light:setPosition(0 , btn:getContentSize().height/2);
		self.cardDefTypeMune[i].light = light;

		if i == 1 then
			local label = cc.Label:createWithTTF(LANG.UI_CARD_44 , fonts.FONT_SIYUAN , 20);
			label:setTextColor(cc.c3b(180 , 180 , 180));
			btn:setTitleLabel(label);
			self.cardDefTypeMune[i].label = label;
		else
			local defType = cc.Sprite:create('image/commonImg/def_type_'..(i-1)..'.png');
			defType:addTo(node);
			defType:setScale(0.5);
		end

		self.cardDefTypeMune[i].redDotNode = node;
	end

	local viewBgHeight = StageSize.height-topHight-menuHeight/2-FOOTER_HEIGHT;
	local viewPosY = menuPosY-menuHeight/2-viewBgHeight/2;
	local viewBg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	viewBg:addTo(self.root);
	viewBg:setCapInsets(cc.rect(11,4,18,21));
	viewBg:setContentSize(cc.size(604 , viewBgHeight));
	viewBg:setPosition(0 , viewPosY);

	local initData = {
        width = 604,
        height = viewBgHeight-38,
        cellWidth = 604 ,
        cellHeight = 425,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData);
    self.cardListView:getView():setPosition(cc.p(-302,viewPosY-viewBgHeight/2+36));
	self.cardListView:getView():addTo(self.root)
	
	local bottomNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0,viewPosY-viewBgHeight/2);

	local line = cc.Sprite:create('image/commonImg/line.png');
	line:addTo(bottomNode);
	line:setContentSize(cc.size(604,1))
	line:setPosition(cc.p(0,33))

	local cardNumBg = cc.Sprite:create('image/commonImg/num_bg.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(211 , 23);

	cardNumBg = cc.Sprite:create('image/commonImg/num_icon.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(209 , 16);
	
	self.curCardNum = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.curCardNum:addTo(bottomNode);
	self.curCardNum:setAnchorPoint(1 , 0.5);
	self.curCardNum:setPosition(289 , 16);


	EventManager:RegisterEvent(EventType.UIUpdateCardBookData, handler(self, self.updateCardBookData));
end

function CardBookPanel:onEnter()
	self:onShow(ALL_DEF_TYPE);
end

function CardBookPanel:onResume()
	self.cardListView:jumpToCurPos(function() 
		self:onShow(self.selectDefType);
	end)
end

function CardBookPanel:onShow(defType)
	self.cards = {};
	self.cards[ALL_DEF_TYPE] = {};
	self.hasOpenNums = {}
	self.hasOpenNums[ALL_DEF_TYPE] = 0
	self.totalNum = 0;
	local redDotState = {false,false,false,false};

	local rewardState = model.CardModel:getAllGetCardRewardState()
	local getCard = model.CardModel:getAllGetCard();


	local roles = ResTable:getTable('role_book').contain;
	for _k , role in pairs(roles) do
		local data = {};
		data.no = role.role_id;
		data.rewardState = rewardState[role.role_id] or 0;
		data.getCard = getCard[role.role_id] or 0;
		table.insert(self.cards[ALL_DEF_TYPE] , data);

		self.totalNum = self.totalNum + 1;

		local rolePre = ResTable:getRowData('role_pre', role.role_id);
		if not self.cards[rolePre.def_type] then
			self.cards[rolePre.def_type] = {};
		end

		table.insert(self.cards[rolePre.def_type] , data);

		if data.getCard == 1 and data.rewardState == 0 then

			if not redDotState[ALL_DEF_TYPE] then
				redDotState[1] = true;
			end

			if not redDotState[rolePre.def_type+1] then
				redDotState[rolePre.def_type+1] = true;
			end
		end

		if data.getCard ~= 0 then
			self.hasOpenNums[ALL_DEF_TYPE] = self.hasOpenNums[ALL_DEF_TYPE]+1
			if not self.hasOpenNums[rolePre.def_type] then
				self.hasOpenNums[rolePre.def_type] = 0;
			end
			self.hasOpenNums[rolePre.def_type] = self.hasOpenNums[rolePre.def_type] + 1
		end
	end
	
	for _k , state in ipairs(redDotState) do
		tools.RedDotTools:setRedDotToNode(self.cardDefTypeMune[_k].redDotNode,state,nil,cc.p(65,16));
	end

	for _k , cards in pairs(self.cards) do
		if(#cards > 1) then
			table.sort(cards , function(first , second)
				return first.no > second.no;
			end);
		end
	end

	self:setData(defType);

	self.root:setVisible(true);
end

function CardBookPanel:switchDefType(sender , event)
	local defType = sender:getTag();

	if self.selectDefType == defType then
		return;
	end

	self:setData(defType);
end

function CardBookPanel:setData(selectDefType)

	if not self.cards[selectDefType] then
		return;
	end

	self.selectDefType = selectDefType;

	for _defType , menu in pairs(self.cardDefTypeMune) do
		_defType = _defType == 1 and ALL_DEF_TYPE or _defType - 1;

		if _defType == ALL_DEF_TYPE then
			if selectDefType == ALL_DEF_TYPE then
				menu.label:setTextColor(cc.c3b(255 , 255 , 255));
			else
				menu.label:setTextColor(cc.c3b(180 , 180 , 180));
			end
		end
		
		menu.btn:setBright(_defType == selectDefType);
		menu.light:setVisible(_defType == selectDefType);
	end

	self.cardMaxNum = #self.cards[selectDefType];
	local tableViewCellMaxNum = math.ceil(self.cardMaxNum/3);

	local hasOpenN = self.hasOpenNums[selectDefType]
	self.curCardNum:setString(tostring(hasOpenN)..'/'..tostring(self.totalNum));

	self.cardList = {};
	self.cardListView:setDataCount(tableViewCellMaxNum);
	self.cardListView:reloadData();
end

function CardBookPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 3 + 1;
	local curMaxNum = (idx+1) * 3;
	curMaxNum = curMaxNum > self.cardMaxNum and self.cardMaxNum or curMaxNum;
	local children = cell:getChildren();
	if #children <= 0 then
		for i = 1 , 3 do
			local x = 101 + 201*(i-1);
			local y = 212.5;
			local bigCard = template.CardTemplate:create(cell);
			bigCard:setTouchEnabled(false);
			if startNum <= curMaxNum then
				local card = self.cards[self.selectDefType][startNum];
				bigCard:InitByCardBook(card);
				bigCard:setPosition(x,y);
				bigCard:setCardTag(card.no);
				bigCard:checkFlag(card.no);
				bigCard:addCardClickEventListener(handler(self , self.cardClick));

				startNum = startNum + 1;
			else
				bigCard:setPosition(x, y);
				bigCard:setVisible(false);
			end
		end
	else
		for i = 1 , 3 do
			local bigCard = children[i];
			if startNum <= curMaxNum then
				local card = self.cards[self.selectDefType][startNum];
				bigCard:InitByCardBook(card);
				bigCard:setCardTag(card.no);
				bigCard:checkFlag(card.no);
				bigCard:setVisible(true);
				bigCard:addCardClickEventListener(handler(self , self.cardClick));

				startNum = startNum + 1;
			else
				bigCard:setVisible(false);
			end
		end
	end
	return cell;
end

function CardBookPanel:cellItemTouch(table)
	if self.selectNo then
		local getCards = model.CardModel:getAllGetCard();
		if getCards[self.selectNo] then
			UIManager:push('CardBookViewPanel' , self.selectNo, 0);
		else
			Prompt:Show(LANG.UI_CARD_60);
		end
		self.selectNo = nil;
		AudioManager:playEffect(Conf.buttonClick);
	end
end

function CardBookPanel:cardClick(sender , event)
	local no = sender:getTag();
	self.selectNo = no;
end

function CardBookPanel:updateCardBookData(no)
	if self.cardList[no] then

		local rewardState = model.CardModel:getAllGetCardRewardState()
		local getCard = model.CardModel:getAllGetCard();

		local data = {
			no = no,
			getCard = getCard[no] or 0,
			rewardState = rewardState[no] or 0,
		};
		self.cardList[no]:InitByCardBook(data);
	end
end

function CardBookPanel:onClickBack()
	UIManager:pop('CardBookPanel');
end

function CardBookPanel:onLeave()
	self.root:setVisible(false);
end

return CardBookPanel;
