local CardBookViewPanel = class("CardBookViewPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');
local imagePath = 'image/cardBook/'
local restraintType = {
	[1] = 2, 
	[2] = 3, 
	[3] = 1, 
	[4] = 3, 
	[5] = 2, 
	[6] = 1, 
}

function CardBookViewPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardBookViewPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE_NO_MONEY;
end

function CardBookViewPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.curPid = nil;

	self:initPanels()
end

function CardBookViewPanel:initPanels()

	self.qualityBg = cc.Sprite:create();
	self.qualityBg:addTo(self.root);

	--self.cardQualityBg = cc.Sprite:create();
	--self.cardQualityBg:addTo(self.root);
	--self.cardQualityBg:setPosition(0 , -57);

	self.cardPanelNavi = cc.Sprite:create();
	self.cardPanelNavi:addTo(self.root);
	self.cardPanelNavi:setPosition(0, 55);

	self.skeletonNode = cc.Node:create();
	self.skeletonNode:addTo(self.root)
	self.skeletonNode:setPosition(0, 55)

	local infoNode = cc.Node:create();
	infoNode:addTo(self.root);
	infoNode:setLocalZOrder(100);
	infoNode:setPosition(-181 , StageSize.height/2 - 136);
	self.infoNode = infoNode

	local bg = cc.Sprite:create('image/card/card_live_name_bg.png');
	bg:addTo(infoNode);

	self.quality = cc.Sprite:create();
	self.quality:addTo(infoNode);
	self.quality:setAnchorPoint(0 , 0.5);
	self.quality:setPosition(-114 , 45);

	self.starNode = cc.Node:create();
	self.starNode:addTo(infoNode);
	self.starNode:setPosition(-7 , 43);
	self.starList = {};
	for i = 1 , 5 do
		bg = cc.Sprite:create('image/card/star_level_bg.png');
		bg:addTo(self.starNode);
		bg:setPosition(26 * (i - 1) , 0);

		self.starList[i] = cc.Sprite:create();
		self.starList[i]:addTo(self.starNode);
		self.starList[i]:setPosition(26 * (i - 1) , 0);
	end
	
	self.defType = cc.Sprite:create();
	self.defType:addTo(infoNode);
	self.defType:setPosition(-99 , 0);
	self.defType:setScale(0.85);

	self.infoName = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 18);
	self.infoName:addTo(infoNode);
	self.infoName:setAnchorPoint(0 , 0.5);
	self.infoName:setPosition(-66 , 1);

	self.jopTypeNode = cc.Sprite:create('image/card/card_job_bg_1.png');
	self.jopTypeNode:addTo(infoNode);
	self.jopTypeNode:setPosition(-70 , -46);

	--local size = self.jopTypeNode:getContentSize();
	--self.jopTypeImg = cc.Sprite:create('image/card/card_job_bg.png');
	--self.jopTypeImg:addTo(self.jopTypeNode);
	--self.jopTypeImg:setPosition(size.width/2-21 , size.height/2);
--
	--self.jopType = cc.Label:createWithTTF('0' , fonts.FONT_SIYUAN , 20);
	--self.jopType:addTo(self.jopTypeNode);
	--self.jopType:enableOutline(cc.c3b(90 , 90 , 90),2);
	--self.jopType:setPosition(size.width/2+17 , size.height/2);

	self.boxBtn = ccui.Button:create('image/onlineReward/online_nbtn.png' , 'image/onlineReward/online_nbtn.png');
	self.boxBtn:addTo(self.root);
	self.boxBtn:setPosition(227 , -StageSize.height/2 + 103);
	self.boxBtn:addClickEventListener(handler(self , self.getBoxClick));

	local boxImg = cc.Sprite:create('image/cardBook/card_book_1.png');
	boxImg:addTo(self.boxBtn);
	boxImg:setPosition(36.5,36.5);

	local label = cc.Label:createWithTTF(LANG.UI_LIBRARY_9 , fonts.FONT_SIYUAN , 20);
	label:addTo(self.boxBtn);
	label:setPosition(36.5 , 0);
	label:enableOutline(cc.c4b(100 , 100 , 100 , 255) , 2);

	EventManager:RegisterEvent(EventType.NetworkCardBookGetReward, handler(self, self.retGetReward));

	self.swtichBtn = ccui.Button:create('image/card/live_switch_off.png' ,'image/card/live_switch_off.png' , 'image/card/live_switch_on.png');
	self.swtichBtn:addTo(self.root);
	self.swtichBtn:setPosition(227 ,StageSize.height/2 - 88);
	self.swtichBtn:setTag(1);
	self.swtichBtn:addClickEventListener(handler(self , self.swtichBtnClick));

	self:initAvatarFlag();

end

function CardBookViewPanel:onEnter(no, resourceId , enterType)

	if not no and not resourceId then
		UIManager:pop('CardBookViewPanel');
		return;
	end

	self.roleId = no;

	local spine_res = '';
	local imgPath = '';

	if enterType == Enum:getTable('LiveEnterType').BARE_COLLECT then
		local bareId = no;
		local bareData = model.BareCollectModel:getBareData(bareId);
		if not bareData then
			UIManager:pop('CardBookViewPanel');
			return;
		end
		imgPath = bareData.res_name;
		local role = ResTable:getRowData('role' , bareData.role_id);
		if role then
			self.qualityBg:setTexture('image/card/cardQualityBg/card_bg_quality_'..role.quality..'.png');
		end
	else
		local resId = resourceId;
		if resId == 0 then
			resId = model.CardModel:getCardResIdByRoldId( no );
		end

		local roleResource = ResTable:getRowData('role_resource', resId);
		if not roleResource then
			UIManager:pop('CardBookViewPanel');
			return;
		end

		self.resId = resId;

		spine_res = roleResource.spine;
		imgPath = roleResource.card;
	end

	self.cardPanelNavi:setTexture('image/role/'..imgPath..'.png');

	if spine_res ~= '' and spine_res~=nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then

		if self.skeletonNode then
			self.skeletonNode:removeAllChildren();
		end

		local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		skAnimation:addTo(self.skeletonNode)
		skAnimation:setAnimation(0, "idle", true)
		-- registerEvent
		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_START)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_END)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_COMPLETE)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_EVENT)
		
	end

	if enterType == Enum:getTable('LiveEnterType').BARE_COLLECT then
		self.infoNode:setVisible(false);
		self.boxBtn:setVisible(false);
	else
		local role = ResTable:getRowData('role' , no);
		if role then
			--self.cardQualityBg:setTexture(imagePath..'library_map_quality_bg_'..role.quality..'.png');
			self.quality:setTexture('image/card/card_quality_'..role.quality..'.png');
			self.qualityBg:setTexture('image/card/cardQualityBg/card_bg_quality_'..role.quality..'.png');
		end
		local roleInfo = ResTable:getRowData('role_info' , no);
		if roleInfo then
			self.infoName:setString(tostring(roleInfo.name));
		end

		local rolePre = ResTable:getRowData('role_pre' , no);
		if rolePre then
			self.defType:setTexture('image/commonImg/def_type_'..rolePre.def_type..'.png');

			local color = math.ceil(rolePre.initial_star / 5);
			local num = rolePre.initial_star % 5;
			num = num == 0 and 5 or num;
			for i = 1 , 5 do
				if i <= num then
					self.starList[i]:setTexture('image/card/star_level_'..color..'.png');
				end
				self.starList[i]:setVisible(i <= num);
			end
			self.starNode:setPositionX(self.quality:getPositionX() + self.quality:getContentSize().width+20);
			--self.jopType:setString(tostring(LANG.UI_CARD_JOB_TYPE[rolePre.job_type]));
			self.jopTypeNode:setTexture('image/card/card_job_bg_'..rolePre.job_type..'.png');
		end

		local rewardState = model.CardModel:getAllGetCardRewardState()
		local getCard = model.CardModel:getAllGetCard();
		self.boxBtn:setVisible(rewardState[no] == nil and getCard[no] ~= nil);

		self.no = no;
	end

	self:switchState(2);

	local showAvatar = false;
	if self.roleId ~= 0 then
		self.idx = 1;
		self.resIdList = model.CardModel:getAvatarResList(self.roleId,true);
		if #self.resIdList > 1 then
			for i,v in ipairs(self.resIdList) do
				if v == self.resId then
					self.idx = i;
				end
			end
			showAvatar = true;
			self:setAvatarState()
		end
	end
	self.avatarNode:setVisible(showAvatar);

	self.root:setVisible(true);
end

--[[
function CardBookViewPanel:initAvatarData()
	-- body
	self.idx = 1;
	self.resIdList = model.CardModel:getAvatarResList(self.roleId);
	--print_ro(self.resIdList);
	for i,v in ipairs(self.resIdList) do
		if v == self.resId then
			self.idx = i;
		end
	end

end
]]
function CardBookViewPanel:updateInfo( idx )
	-- body
	if idx < 1 or idx > #self.resIdList then
		return;
	end

	local resId = self.resIdList[idx];

	if resId >= 1000 then 
		local isHave = model.CardModel:onIsHaveAvatarByAvatarItemId(resId);
		if not isHave then
			return;
		end
	end

	self.idx = idx;
	local roleResource = ResTable:getRowData('role_resource', resId);
	if not roleResource then
		return;
	end

	local spine_res = roleResource.spine;
	local imagePath = roleResource.card;

	self.cardPanelNavi:setTexture('image/role/'..imagePath..'.png');
	self.resId = resId;

	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end
	
	if spine_res~=nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then

		local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		skAnimation:addTo(self.skeletonNode)
		skAnimation:setAnimation(0, "idle", true)
		-- registerEvent
		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_START)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_END)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_COMPLETE)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_EVENT)
	end

	self:setAvatarState();
	self:switchState(2);
end

function CardBookViewPanel:initAvatarFlag()
	-- body
	self.avatarNode = cc.Node:create();
	self.avatarNode:addTo(self.root);

	local avatarFlagNode = cc.Node:create();
	avatarFlagNode:addTo(self.avatarNode);
	-- avatarFlagNode:setPosition(-StageSize.width/2 + 30 , 400);
	avatarFlagNode:setPosition(-StageSize.width/2 + 10 , StageSize.height/2 - 162 - 80);

	local avatarFlag = cc.Sprite:create('image/card/avatar_flag.png');
	avatarFlag:addTo(avatarFlagNode);
	avatarFlag:setAnchorPoint(cc.p(0, 0.5));
	avatarFlag:setPosition(0 , 0);

	self.leftBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	self.leftBtn:addTo(self.avatarNode);
	self.leftBtn:setAnchorPoint(cc.p(0, 0.5));
	self.leftBtn:setPosition(-StageSize.width/2 + 30 , 0);
	self.leftBtn:addClickEventListener(handler(self , self.leftBtnClick));

	self.rightBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	self.rightBtn:addTo(self.avatarNode);
	self.rightBtn:setAnchorPoint(cc.p(0, 0.5));
	self.rightBtn:setRotation(180);
	self.rightBtn:setPosition(StageSize.width/2 - 30 , 0);
	self.rightBtn:addClickEventListener(handler(self , self.rightBtnClick));

	self.selAvatarList = {};

	for i=1,10 do

		local idxNode = cc.Node:create();
		idxNode:addTo(avatarFlagNode);
		idxNode:setPosition(i*35+70, 0);
		idxNode:setVisible(false);

		local bg1 = cc.Sprite:create('image/card/avatar_s2.png');
		bg1:addTo(idxNode);
		local bg2 = cc.Sprite:create('image/card/avatar_s1.png');
		bg2:addTo(idxNode);
		bg2:setTag(0);

		local idxLabel = cc.Label:createWithTTF(''..i , fonts.FONT_SIYUAN , 20);
		idxLabel:addTo(idxNode);
		idxLabel:setTextColor(cc.c3b(255 , 255 , 255));
		idxLabel:setTag(1);

		self.selAvatarList[ i ] = idxNode;

	end

end

function CardBookViewPanel:setAvatarState()
	-- body
	for k,v in pairs(self.selAvatarList) do

		if k <= #self.resIdList then
			v:setVisible(true);
			if k == self.idx then
				v:getChildByTag(0):setVisible(true);
				v:getChildByTag(1):setTextColor(cc.c3b(255 , 255 , 255));
				v:getChildByTag(1):enableOutline(cc.c4b(100 , 60 , 150 , 255) , 2);
			else 
				v:getChildByTag(0):setVisible(false);
				v:getChildByTag(1):setTextColor(cc.c3b(100 , 60 , 150));
				v:getChildByTag(1):disableEffect();
			end
		else
			v:setVisible(false);
		end
		
	end

	if self.idx == 1 then
		self.leftBtn:setVisible(false);
	else
		self.leftBtn:setVisible(true);
	end
	if self.idx == #self.resIdList then
		self.rightBtn:setVisible(false);
	else
		self.rightBtn:setVisible(true);
	end

end

function CardBookViewPanel:leftBtnClick(sender , event)
	self:updateInfo( self.idx - 1 );
end

function CardBookViewPanel:rightBtnClick(sender , event)
	self:updateInfo( self.idx + 1 );
end

function CardBookViewPanel:swtichBtnClick(sender , event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	self:switchState(tag)
end

function CardBookViewPanel:switchState(state)

	if state == 1 and self.skeletonNode:getChildrenCount() == 0 then
		Prompt:Show(LANG.UI_CARD_2);
		return
	end

	self.swtichBtn:setTag(state == 1 and 2 or 1);

	self.cardPanelNavi:setVisible(state == 2);

	if self.skeletonNode:getChildrenCount() > 0 then
		self.skeletonNode:setVisible(state == 1);
	end

	self.swtichBtn:setBright(state == 2)
end

function CardBookViewPanel:getBoxClick()
	AudioManager:playEffect(Conf.confirmButton);
	Network.NetworkManager:Send(Network.NetworkCmdType.req_card_book_get_reward, {no = self.no});
end

function CardBookViewPanel:retGetReward(msg)
	self.boxBtn:setVisible(false);

	UIManager:push('RewardPanel',msg);
end

-- function CardBookViewPanel:onClickBack()
-- 	--EventManager:FireEvent(EventType.UIUpdateCardBookData , self.no);
-- 	EventManager:FireEvent(EventType.UIHeadeUpdateMoneyNodeVisible,{[1] = true , [2] = true});
-- 	UIManager:pop('CardBookViewPanel');
-- end

function CardBookViewPanel:onLeave()
	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end
	self.root:setVisible(false);
end

return CardBookViewPanel;
