local CardBreakOpenSkillPanel = class("CardBreakOpenSkillPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/card/cardBreak/'
local MessageBoxType = Enum:getTable('MessageBoxType')


function CardBreakOpenSkillPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardBreakOpenSkillPanel');

	self.isPopup = true

	self.useCCreator = false;
end

function CardBreakOpenSkillPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function CardBreakOpenSkillPanel:initPanels()

	local spine_res = 'RewardPanelEffect'
	self.effect = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
	self.effect:addTo(self.root);
	self.effect:setPositionY(92);

	self.infoNode = cc.Node:create();
	self.infoNode:addTo(self.root);

	local skillBg = cc.Sprite:create(imagePath..'card_break_6.png');
	skillBg:addTo(self.infoNode);
	skillBg:setPosition(0 , 167);

	skillBg = cc.Sprite:create(imagePath..'card_break_5.png');
	skillBg:addTo(self.infoNode);
	skillBg:setPosition(0 , 75);

	self.skillName = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.skillName:addTo(self.infoNode);
	self.skillName:setPosition(0 , 75);
	self.skillName:enableOutline(cc.c4b(0 , 85 , 100 , 255));

	self.skillBg = cc.Sprite:create();
	self.skillBg:addTo(self.infoNode);
	self.skillBg:setPosition(0 , 167);

	self.skillIcon = cc.Sprite:create();
	self.skillIcon:addTo(self.infoNode);
	self.skillIcon:setPosition(0 , 167);

	self.skillInfo = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 18);
	self.skillInfo:addTo(self.infoNode);
	self.skillInfo:setPosition(0 , 16);

	self.skillAnimation = cc.Sprite:create();
	self.skillAnimation:addTo(self.infoNode);
	self.skillAnimation:setPosition(0 , 170);
	self.skillAnimation:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));
end

function CardBreakOpenSkillPanel:onEnter(skillResid)

	self:addSwallowCallback(handler(self , self.onClickBack));

	if not skillResid then 
		return;
	end

	self.effect:clearTracks();
	self.effect:addAnimation(0, GlobalFunctions:lAnimationName('OpenFeaturesEffect') , false)
	-- self.effect:getAnimation():play("OpenFeaturesEffect", 1);

	local skillData = ResTable:getRowData('skill' , skillResid);
	self.skillBg:setTexture('image/card/cardSkill/card_skill_type_'..skillData.skill_type..'.png');
	self.skillIcon:setTexture('image/skillicon/'..skillData.icon..'.png');
	self.skillName:setString(tostring(skillData.name));

	local formatStr , colorPos = formatColorText(skillData.info);

	self.skillInfo:setString(tostring(formatStr));
	local strLen = getStrLen(formatStr);
	for i = 0 , strLen-1 do
		local label = self.skillInfo:getLetter(i);
		if label then
			local changeColor = false
			for _k , pos in pairs(colorPos) do
				if i >= pos.sPos-1 and i <= pos.ePos-1 then
					label:setColor(cc.c3b(pos.color[1] , pos.color[2] , pos.color[3]));
					changeColor = true;
					break;
				end
			end
			if not changeColor then
				label:setColor(cc.c3b(255 , 255 , 255));
			end
		end
	end

	self.skillAnimation:setVisible(false);
	self.infoNode:setOpacity(0);
	self.infoNode:runAction(cc.Sequence:create(cc.FadeIn:create(0.3) , cc.CallFunc:create(function() 
		self.skillAnimation:setVisible(true);
		self.skillAnimation:stopAllActions();
		local skillAnimation = FrameEffectManager:createFrameAnimation('OpenFeaturesEffect', 16, 15)
		self.skillAnimation:runAction(cc.Sequence:create(cc.Animate:create(skillAnimation) , cc.CallFunc:create(function() 
			self.skillAnimation:setVisible(false);
		end)));
	end)));

	self.root:setVisible(true);

end

function CardBreakOpenSkillPanel:onLeave()
	self.root:setVisible(false);
end

function CardBreakOpenSkillPanel:onClickBack()
	UIManager:pop('CardBreakOpenSkillPanel');
end

return CardBreakOpenSkillPanel;
