local CardBreakSuccessPanel = class("CardBreakSuccessPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/card/cardBreak/'
local MessageBoxType = Enum:getTable('MessageBoxType')


function CardBreakSuccessPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardBreakSuccessPanel');

	self.useCCreator = false;
end

function CardBreakSuccessPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.isPopup = true;

	self:initPanels()
end

function CardBreakSuccessPanel:initPanels()

	--self.particleWin = cc.ParticleSystemQuad:create("Effect/particle/winp.plist")
	--self.particleWin:addTo(self.root);
	--self.particleWin:setPosition(cc.p(0,245))
	--self.particleWin:stopSystem();

	local spine_res = 'RewardPanelEffect'
	self.effect = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
	self.effect:setPositionY(30);
	self.effect:addTo(self.root);

	self.infoNode = cc.Node:create();
	self.infoNode:addTo(self.root);
	self.infoNode:setCascadeOpacityEnabled(true);

	self.cardIcon = template.IconTemplate:create(self.infoNode);
	self.cardIcon:setPosition(0 , 168);

	local breakLevelBg = cc.Sprite:create(imagePath..'card_break_1.png');
	breakLevelBg:addTo(self.infoNode);
	breakLevelBg:setPosition(0 , 75);

	self.breakLevelList = {};
	for i = 1 , 6 do
		local bg = cc.Sprite:create(imagePath..'card_break_2.png');
		bg:addTo(self.infoNode);
		bg:setPosition(-80 + 31 * (i - 1) , 75);

		self.breakLevelList[i] = cc.Sprite:create(imagePath..'card_break_4.png');
		self.breakLevelList[i]:addTo(self.infoNode);
		self.breakLevelList[i]:setPosition(-80 + 31 * (i - 1) , 75);
	end

	self.breakAnimation = cc.Sprite:create();
	self.breakAnimation:addTo(self.infoNode);
	self.breakAnimation:setPosition(0 , 75);
	self.breakAnimation:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.attList = {};
	for i = 1 , 5 do
		self.attList[i] = {};
		self.attList[i].node = cc.Node:create();
		self.attList[i].node:addTo(self.infoNode);
		self.attList[i].node:setPosition(0 , 17 - 59 * (i - 1));

		self.attList[i].bg = cc.Sprite:create(imagePath..'card_break_7.png');
		self.attList[i].bg:addTo(self.attList[i].node);

		self.attList[i].attImg = cc.Sprite:create();
		self.attList[i].attImg:addTo(self.attList[i].node);
		self.attList[i].attImg:setPosition(-230 , 2);

		self.attList[i].attName = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
		self.attList[i].attName:addTo(self.attList[i].node);
		self.attList[i].attName:setAnchorPoint(0 , 0.5);
		self.attList[i].attName:setPosition(-207 , 2);

		self.attList[i].att = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
		self.attList[i].att:addTo(self.attList[i].node);
		self.attList[i].att:setPosition(-11 , 2);

		self.attList[i].nextImg = cc.Sprite:create(imagePath..'card_break_3.png');
		self.attList[i].nextImg:addTo(self.attList[i].node);
		self.attList[i].nextImg:setPosition(80 , 2);

		self.attList[i].nextAtt = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
		self.attList[i].nextAtt:addTo(self.attList[i].node);
		self.attList[i].nextAtt:setTextColor(cc.c3b(255 , 228 , 0));
		self.attList[i].nextAtt:setPosition(179 , 2);
	end
end

function CardBreakSuccessPanel:onEnter(oldPro , pid)

	self:addSwallowCallback(handler(self , self.onClickBack));

	self.data = model.CardModel:GetCard(pid);
	if not self.data then
		return;
	end

	local pro = CardProManager:GetCardPro(pid);
	if not pro then
		return;
	end

	self.oldPro = oldPro;
	self.pid = pid;

	--self.particleWin:resetSystem();
	self.effect:clearTracks();
	self.effect:addAnimation(0, GlobalFunctions:lAnimationName('BreakthroughSuccessEffect') , false)
	-- self.effect:getAnimation():play("BreakthroughSuccessEffect", 1);

	self.cardIcon:Init(ROLE_TYPE , self.data.no, 0,0, pid);

	for i = 1 , 6 do
		self.breakLevelList[i]:setVisible(i <= self.data.break_lv);
	end
	
	for i = 1 , 5 do
		self.attList[i].node:setVisible(false);
	end

	local index = 1;
	for _k , attName in pairs({'fp' , 'hp' , 'atk' , 'def'}) do
		if oldPro[attName] then
			self.attList[index].node:setVisible(true);
			self.attList[index].attImg:setTexture(imagePath..'att_bg_'.._k..'.png');
			self.attList[index].attName:setString(LANG.UI_CARAD_BREAK_SUCCESS_ATT_TYPE[_k]);
			if attName == 'crit' then
				self.attList[index].att:setString(tostring(math.floor(oldPro[attName] / 10000 *100))..'%');
			elseif attName == 'crit_dam' then
				self.attList[index].att:setString(tostring(oldPro[attName] + 200)..'%');
			else
				self.attList[index].att:setString(tostring(oldPro[attName]));
			end

			if pro[attName] then
				if attName == 'crit' then
					self.attList[index].nextAtt:setString(tostring(math.floor(pro[attName] / 10000 *100))..'%');
				elseif attName == 'crit_dam' then
					self.attList[index].nextAtt:setString(tostring(pro[attName] + 200)..'%');
				else
					self.attList[index].nextAtt:setString(tostring(pro[attName]));
				end
			end
			index = index + 1;
		end
	end

	self.breakAnimation:setVisible(false);

	self.infoNode:setOpacity(0);
	self.infoNode:runAction(cc.Sequence:create(cc.FadeIn:create(0.3) , cc.CallFunc:create(function() 
		self.breakAnimation:setPositionX(-80 + 31 * (self.data.break_lv - 1));
		self.breakAnimation:setVisible(true);
		self.breakAnimation:stopAllActions();
		local breakAnimation = FrameEffectManager:createFrameAnimation('BreakthroughSuccessEffect', 7, 15)
		self.breakAnimation:runAction(cc.Sequence:create(cc.Animate:create(breakAnimation) , cc.CallFunc:create(function() 
			self.breakAnimation:setVisible(false);
		end)));
	end)));

	self.root:setVisible(true);

end

function CardBreakSuccessPanel:onLeave()
	local breakData = model.CardModel:getCardBreakTableData(self.data.break_lv);
	if breakData and breakData.skill_open and breakData.skill_open ~= '' then
		local skillId = model.CardModel:getCardSkillIdByPid(self.pid);
		local roleSkillData = ResTable:getRowData('role_skill' , skillId);
		if roleSkillData[tostring(breakData.skill_open)..'_no'] then
			UIManager:push('CardBreakOpenSkillPanel' , roleSkillData[tostring(breakData.skill_open)..'_no']);
		end
	end

	local newFp = CardProManager:GetCardPro(self.pid).fp;
	newFp = ConvertDigitUnitToStr(tostring(newFp));
	local oldFp = ConvertDigitUnitToStr(tostring(self.oldPro.fp));
	
	if true or HpCompare(newFp , '>' , oldFp) then
		FpUpPanel:Show(oldFp , newFp);
	end

	self.root:setVisible(false);
end

function CardBreakSuccessPanel:onClickBack()
	--self.particleWin:stopSystem();
	UIManager:pop('CardBreakSuccessPanel');

	--local breakData = model.CardModel:getCardBreakTableData(self.data.break_lv);
	--if breakData and breakData.skill_open and breakData.skill_open ~= '' then
	--	local skillId = model.CardModel:getCardSkillIdByPid(self.pid);
	--	local roleSkillData = ResTable:getRowData('role_skill' , skillId);
	--	if roleSkillData[tostring(breakData.skill_open)..'_no'] then
	--		UIManager:push('CardBreakOpenSkillPanel' , roleSkillData[tostring(breakData.skill_open)..'_no']);
	--	end
	--end
--
	--local newFp = CardProManager:GetCardPro(self.pid).fp;
	--newFp = ConvertDigitUnitToStr(tostring(newFp));
	--local oldFp = ConvertDigitUnitToStr(tostring(self.oldPro.fp));
	--
	--if HpCompare(newFp , '>' , oldFp) then
	--	FpUpPanel:Show(oldFp , newFp);
	--end
end

return CardBreakSuccessPanel;
