local CardLevelUpStarCardListPanel = class("CardLevelUpStarCardListPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

local ALL_DEF_TYPE = 99;

function CardLevelUpStarCardListPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardLevelUpStarCardListPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function CardLevelUpStarCardListPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self.selectCards = {};
	self:initPanels()
end

function CardLevelUpStarCardListPanel:initPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_5.png');
	self.bg:addTo(self.root);

	local topHight = NOTCH_HEIGHT + 102
	local menuHeight = 46;
	local menuPosY = StageSize.height/2-topHight;
	local menuNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0,menuPosY)

	self.cardDefTypeMune = {};
	for i = 1 , 4 do

		self.cardDefTypeMune[i] = {};
		local texture = 'image/commonImg/menu_left_71_46.png';
		local unTexture = 'image/commonImg/menu_left_71_46_u.png';

		if i == 2 or i == 3 then
			texture = 'image/commonImg/menu_mid_71_46.png';
			unTexture = 'image/commonImg/menu_mid_71_46_u.png';
		end

		local node = cc.Node:create();
		node:addTo(menuNode);
		node:setPosition(-226.5 + 151*(i-1) , 0);

		local btn = ccui.Button:create(texture,texture,unTexture);
		btn:addTo(node);
		btn:setTag(i == 1 and ALL_DEF_TYPE or (i - 1));
		btn:setScale9Enabled(true);
		btn:setScaleX(i == 4 and -1 or 1);
		btn:setCapInsets(cc.rect(10 , 0 , 51 , 46));
		btn:setContentSize(cc.size(151 , 46));
		btn:addClickEventListener(handler(self , self.switchDefType));
		self.cardDefTypeMune[i].btn = btn;

		local light = cc.Sprite:create('image/commonImg/menu_light_1.png');
		light:addTo(node);
		light:setPosition(0 , btn:getContentSize().height/2);
		self.cardDefTypeMune[i].light = light;

		if i == 1 then
			local label = cc.Label:createWithTTF(LANG.UI_CARD_44 , fonts.FONT_SIYUAN , 20);
			btn:setTitleLabel(label)
		else
			local defType = cc.Sprite:create('image/commonImg/def_type_'..(i-1)..'.png');
			defType:addTo(node);
			defType:setScale(0.5);
		end
	end

	local viewBgHeight = StageSize.height-topHight-menuHeight/2-FOOTER_HEIGHT;
	local viewPosY = menuPosY-menuHeight/2-viewBgHeight/2;
	local viewBg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	viewBg:addTo(self.root);
	viewBg:setCapInsets(cc.rect(11,4,18,21));
	viewBg:setContentSize(cc.size(604 , viewBgHeight));
	viewBg:setPosition(0 , viewPosY);

	local initData = {
        width = 604,
        height = viewBgHeight-38,
        cellWidth = 604 ,
        cellHeight = 425,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData) ;
    self.cardListView:getView():setPosition(cc.p(-302,viewPosY-viewBgHeight/2+36));
	self.cardListView:getView():addTo(self.root)

	local bottomNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0,viewPosY-viewBgHeight/2);

	local line = cc.Sprite:create('image/commonImg/line.png');
	line:addTo(bottomNode);
	line:setContentSize(cc.size(604,1))
	line:setPosition(cc.p(0,33))

	local cardNumBg = cc.Sprite:create('image/commonImg/num_bg.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(211 , 23);

	cardNumBg = cc.Sprite:create('image/commonImg/num_icon.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(209 , 16);
	
	self.curCardNum = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.curCardNum:addTo(bottomNode);
	self.curCardNum:setAnchorPoint(1 , 0.5);
	self.curCardNum:setPosition(289 , 16);

end

function CardLevelUpStarCardListPanel:onEnter(enterFrom)
	if enterFrom == 'HomeBasePanel' then
		self.selectDefType = ALL_DEF_TYPE;
	end
	self:initData();
	self.root:setVisible(true);
end

function CardLevelUpStarCardListPanel:onResume()
	self:initData();
	self.root:setVisible(true);
end

function CardLevelUpStarCardListPanel:initData()
	self.cards = {};
	self.cards[ALL_DEF_TYPE] = {};

	local curDefTypeCards = {};
	for k, v in pairs(model.CardModel:GetAllCard()) do
		local quality = v.quality;
		if v.star_lv > 3 and ((quality < 5 and v.star_lv < CARD_STAR_SSR_MAX_LEVEL) or (quality >= 5 and v.star_lv < CARD_STAR_MAX_LEVEL)) then
			local rolePre = ResTable:getRowData('role_pre', v.no);
			if not curDefTypeCards[rolePre.def_type] then
				curDefTypeCards[rolePre.def_type] = {};
				self.cards[rolePre.def_type] = {};
			end

			if not curDefTypeCards[rolePre.def_type][v.no] then
				curDefTypeCards[rolePre.def_type][v.no] = {};
			end

			if not curDefTypeCards[rolePre.def_type][v.no][v.star_lv] then
				curDefTypeCards[rolePre.def_type][v.no][v.star_lv] = v;
			else 
				local card = curDefTypeCards[rolePre.def_type][v.no][v.star_lv];
				local cardInTeam = model.TeamModel:isRoleInTeam(card.pid);
				local vInTeam = model.TeamModel:isRoleInTeam(v.pid);

				if vInTeam and not cardInTeam then
					curDefTypeCards[rolePre.def_type][v.no][v.star_lv] = v;
				elseif (vInTeam and cardInTeam) or (not vInTeam and not cardInTeam) then
					if v.level > card.level then
						curDefTypeCards[rolePre.def_type][v.no][v.star_lv] = v;
					end
				end
			end
		end
	end

	for _defType , noCards in pairs(curDefTypeCards) do
		for _no , starCards in pairs(noCards) do
			for _star , v in pairs(starCards) do
				local isEnough , mainNum , secondNum , totalNum = model.CardModel:getCardStarActiveState(v.pid);
				local data = {
					card = v , 
					data = {
						isEnough = isEnough, 
						mainNum = mainNum , 
						secondNum = secondNum , 
						totalNum = totalNum
					}
				}
				table.insert(self.cards[_defType] , data);
				table.insert(self.cards[ALL_DEF_TYPE] , data);
			end
		end
	end
	
	for _defType , cards in pairs(self.cards) do
		if #cards > 1 then
			table.sort(cards , function(first , second) 
				local firstIsEnough = first.data.isEnough;
				local secondIsEnough = second.data.isEnough;

				if firstIsEnough and not secondIsEnough then
					return true;
				elseif (firstIsEnough and secondIsEnough) or (not firstIsEnough and not secondIsEnough) then
					local firstQuality = first.card.quality;
					local secondQuality = second.card.quality;

					if firstQuality > secondQuality then
						return true;
					elseif firstQuality == secondQuality then
						if first.card.star_lv > second.card.star_lv then
							return true;
						elseif first.card.star_lv == second.card.star_lv then
							return first.card.no > second.card.no;
						end
					end
				end

				return false;
			end);
		end
	end

	self:setData();
end

function CardLevelUpStarCardListPanel:switchDefType(sender , event)
	local defType = sender:getTag();

	if defType == self.selectDefType then
		return;
	end

	self.selectDefType = defType;

	self:setData();
end

function CardLevelUpStarCardListPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 3 + 1;
	local curMaxNum = (idx+1) * 3;
	curMaxNum = curMaxNum > self.cardMaxNum and self.cardMaxNum or curMaxNum;
	local children = cell:getChildren();
	if #children <= 0 then
		for i = 1 , 3 do
			local x = 101 + 201*(i-1);
			local y = 212.5;
			local bigCard = template.CardTemplate:create(cell);
			bigCard:setTouchEnabled(false);
			bigCard:setPosition(x,y);
			bigCard:addCardClickEventListener(handler(self, self.cardClick));
		
			if startNum <= curMaxNum then
				local card = self.cards[self.selectDefType][startNum];
				local pid = card.card.pid;
				bigCard:InitUpStar(pid , card.data);
				bigCard:setCardTag(pid);

				startNum = startNum + 1;
			else
				bigCard:setPosition(x, y);
				bigCard:setVisible(false);
			end
		end
	else
		for i = 1 , 3 do
			local bigCard = children[i];
			if startNum <= curMaxNum then
				local card = self.cards[self.selectDefType][startNum];
				local pid = card.card.pid;
				bigCard:InitUpStar(pid , card.data);
				bigCard:setCardTag(pid);
				bigCard:setVisible(true);

				startNum = startNum + 1;
			else
				bigCard:setVisible(false);
			end
		end
	end
	return cell;
end

function CardLevelUpStarCardListPanel:setData()
	self.cardList = {};
	self.selectPid = nil;

	for _defType , menu in pairs(self.cardDefTypeMune) do
		local defType = _defType == 1 and ALL_DEF_TYPE or _defType - 1;
		if defType == self.selectDefType then
			if defType == ALL_DEF_TYPE then
				local label = menu.btn:getTitleLabel();
				label:setTextColor(cc.c3b(255,255,255));
			end
			menu.btn:setBright(true)
			menu.light:setVisible(true);
		else
			if defType == ALL_DEF_TYPE then
				local label = menu.btn:getTitleLabel();
				label:setTextColor(cc.c3b(180,180,180));
			end
			menu.btn:setBright(false)
			menu.light:setVisible(false);
		end
	end

	if not self.cards[self.selectDefType] then
		return;
	end

	self.cardMaxNum = #self.cards[self.selectDefType];
	local tableViewCellMaxNum = math.ceil(self.cardMaxNum/3);

	self.cardListView:setDataCount(tableViewCellMaxNum);
	self.cardListView:reloadData();

	--if self.cardMaxNum > 0 then
	--	local cards = self.cards[self.selectDefType];
	--	local rowNum = math.floor(self.cardMaxNum/3);
	--	if self.cardMaxNum % 3 ~= 0 then
	--		rowNum = rowNum + 1;
	--	end
	--	local bigListMaxHeight = 424*rowNum + 50;
	--	if bigListMaxHeight < 842 then
	--		bigListMaxHeight = 842;
	--	end
	--	local maxWidth = 618;
	--	self.cardListView:setInnerContainerSize(cc.size(maxWidth, bigListMaxHeight));
	--	local row, col = 1, 1;
	--	for i=1, self.cardMaxNum do
--
	--		local pid = cards[i].card.pid;
--
	--		local x = 104 + 205*(col-1);
	--		local y = bigListMaxHeight - 205.5 - 424 * (row - 1);
	--		local bigCard = template.CardTemplate:create(self.cardListView);
	--		bigCard:setPosition(x, y);
	--		bigCard:addCardClickEventListener(handler(self, self.cardClick));
	--		bigCard:InitUpStar(pid , cards[i].data);
	--		bigCard:setCardTag(pid);
--
	--		self.cardList[tostring(pid)] = bigCard;
--
	--		col = col + 1;
	--		if col > 3 then
	--			col = 1;
	--			row = row + 1;
	--		end
	--	end
	--end

	self.curCardNum:setString(tostring(self.cardMaxNum)..'/'..tostring(CARD_MAX_NUM));

end

function CardLevelUpStarCardListPanel:cellItemTouch()
	if self.selectPid then
		UIManager:push('CardLevelUpStarPanel', self.selectPid);
		self.selectPid = nil
	end
end

function CardLevelUpStarCardListPanel:cardClick(sender , event)
	self.selectPid = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
end


function CardLevelUpStarCardListPanel:onClickBack()
	self.selectPid = nil;
	UIManager:pop('CardLevelUpStarCardListPanel');
	tools.RedDotTools:pushHomeBaseRedDot(2,model.HomeModel.HomeMenuTag.TAG_CARD);
end

function CardLevelUpStarCardListPanel:onLeave()
	self.root:setVisible(false);
end

return CardLevelUpStarCardListPanel;
