local CardLevelUpStarSuccessPanel = class("CardLevelUpStarSuccessPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/card/cardLevelUpStar/'
local MessageBoxType = Enum:getTable('MessageBoxType')


function CardLevelUpStarSuccessPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardLevelUpStarSuccessPanel');

	self.useCCreator = false;
end

function CardLevelUpStarSuccessPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function CardLevelUpStarSuccessPanel:initPanels()

	self.starUpSpineNode1 = cc.Node:create();
	self.starUpSpineNode1:addTo(self.root);

	self.cardIcon = template.ItemTemplate:create();
	self.cardIcon:addTo(self.root);
	self.cardIcon:setPosition(0 , 253);

	self.starUpSpineNode2 = cc.Node:create();
	self.starUpSpineNode2:addTo(self.root);

	self.starUpEffectNode = cc.Node:create();
	self.starUpEffectNode:addTo(self.root);

	self.btnSkip = ccui.Button:create('image/chouka/result/btnSkip.png');
	self.btnSkip:setPosition(StageSize.width/2-90, -StageSize.height/2+90);
	self.btnSkip:setZoomScale(0);
	self.btnSkip:addClickEventListener(function() 
		self:onClickBack();
	end);
	self.btnSkip:setLocalZOrder(42)
	self.btnSkip:addTo(self.root)
end

function CardLevelUpStarSuccessPanel:onEnter()

	local tempMsg = model.CardModel:getStarBreakTempData();
	local card = model.CardModel:GetCard(tempMsg.pid);

	if not card then
		return;
	end

	self.starUpEffectNode:removeAllChildren();
	self.starUpSpineNode1:removeAllChildren();
	self.starUpSpineNode2:removeAllChildren();

	self.cardIcon:setVisible(false);
	self.cardIcon:stopAllActions();
	self.cardIcon:setOpacity(0);
	self.cardIcon:setPosition( 0 , 253);
	self.cardIcon:Init(ROLE_TYPE , card.no ,card.star_lv-1,card.level, tempMsg.pid);	

	self:runSpineAction();

	self.root:setVisible(true);
end

function CardLevelUpStarSuccessPanel:runSpineAction()

	AudioManager:playEffect(Conf.cardHeroStarLevelUp);

	local action1 = cc.Sequence:create(cc.DelayTime:create(10 / 30) , cc.MoveTo:create(10 / 30 , cc.p(0,-15)),cc.MoveTo:create(10 / 30 , cc.p(0,0)));
	local action2 = cc.Sequence:create(cc.DelayTime:create(10 / 30) , cc.FadeIn:create(10/30));
	self.cardIcon:setVisible(true);
	self.cardIcon:runAction(cc.Spawn:create(action1,action2));

	if cc.FileUtils:getInstance():isFileExist('spine/shengxing_spine/shengxing_spine1.json') then
		local starUpSpine = sp.SkeletonAnimation:create('spine/shengxing_spine/shengxing_spine1.json', 'spine/shengxing_spine/shengxing_spine1.atlas', 1)

		starUpSpine:addTo(self.starUpSpineNode1)
		starUpSpine:setAnimation(0, "idle", false)

		starUpSpine:registerSpineEventHandler(function (event)
			if event.eventData.name == 'shengxing_spine2' then	
				for i = 1 , 5 do
					local effect = cc.ParticleSystemQuad:create('Effect/particle/shengxing'..i..'.plist');
					effect:addTo(self.starUpEffectNode);
					effect:resetSystem();
				end

				if cc.FileUtils:getInstance():isFileExist('spine/shengxing_spine/shengxing_spine2.json') then
					local sUpSpine = sp.SkeletonAnimation:create('spine/shengxing_spine/shengxing_spine2.json', 'spine/shengxing_spine/shengxing_spine2.atlas', 1)
					sUpSpine:addTo(self.starUpSpineNode2);
					sUpSpine:setAnimation(0, "idle", false);

					sUpSpine:registerSpineEventHandler(function (event)
						if event.eventData.name == 'remove' then
							self.cardIcon:stopAllActions();
							self.cardIcon:setVisible(false);
						end
					end, sp.EventType.ANIMATION_EVENT)

					sUpSpine:registerSpineEventHandler(function (event)

						self.starUpEffectNode:removeAllChildren();

						self:onClickBack();
					end, sp.EventType.ANIMATION_COMPLETE)
				end
			end		
		end, sp.EventType.ANIMATION_EVENT)
	end
end

function CardLevelUpStarSuccessPanel:onLeave()
	self.starUpEffectNode:removeAllChildren();
	self.root:runAction(cc.Sequence:create(cc.DelayTime:create(0.1),cc.CallFunc:create(function() 
		self.starUpSpineNode2:removeAllChildren();
		self.starUpSpineNode1:removeAllChildren();
	end)))
	self.cardIcon:stopAllActions();
	self.root:setVisible(false);
end

function CardLevelUpStarSuccessPanel:onClickBack()
	EventManager:FireEvent(EventType.NetworkRetStarLevelUp);
	UIManager:pop('CardLevelUpStarSuccessPanel');
end

return CardLevelUpStarSuccessPanel;
