local CardLivePanel = class("CardLivePanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');

local restraintType = {
	[1] = 2, 
	[2] = 3, 
	[3] = 1, 
	[4] = 3, 
	[5] = 2, 
	[6] = 1, 
}

function CardLivePanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'CardLivePanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE_NO_MONEY;
end

function CardLivePanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.curPid = nil;

	self:initPanels()
end

function CardLivePanel:initPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_3.png');
	self.bg:addTo(self.root);

	self.cardPanelNavi = cc.Sprite:create();
	self.cardPanelNavi:addTo(self.root);
	self.cardPanelNavi:setPosition(0, 55);

	self.skeletonNode = cc.Node:create();
	self.skeletonNode:addTo(self.root)
	self.skeletonNode:setPosition(0, 55)

	self:initAvatarFlag();

	self.swtichBtn = ccui.Button:create('image/card/live_switch_off.png' ,'image/card/live_switch_off.png' , 'image/card/live_switch_on.png');
	--self.swtichBtn:loadTextureDisabled( 'image/card/live_switch_off.png' )
	self.swtichBtn:addTo(self.root);
	self.swtichBtn:setPosition(227 , StageSize.height/2 - 88);
	self.swtichBtn:setTag(1);
	self.swtichBtn:addClickEventListener(handler(self , self.swtichBtnClick));
	
	EventManager:RegisterEvent(EventType.UICardLiveUpdate , handler(self, self.onShow));
end

function CardLivePanel:onEnter(pid, resourceId)

	self:onShow(pid , resourceId);

	self.root:setVisible(true);
	
end

function CardLivePanel:onShow(pid , resourceId , quality)
	if not pid and not resourceId then
		UIManager:pop('CardLivePanel');
		return;
	end

	local card = model.CardModel:GetCard(pid);
	local roleId = 0;
	local roleQuality = quality;
	if card then
		roleId = card.no;
		roleQuality = ResTable:getValue('role' , roleId , 'quality');
	end

	self.roleId = roleId;

	local resId = resourceId;
	if resId == 0 then
		resId = model.CardModel:getCardResIdByPid( pid );
	end

	local roleResource = ResTable:getRowData('role_resource', resId);
	if not roleResource then
		UIManager:pop('CardLivePanel');
		return;
	end
	self.resId = resId;

	local spine_res = roleResource.spine;
	local imagePath = roleResource.card;
	self.spine_res = spine_res;

	self.bg:setTexture('image/card/cardQualityBg/card_bg_quality_'..roleQuality..'.png');
	self.cardPanelNavi:setTexture('image/role/'..imagePath..'.png');

	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end

	local show = false
	if self.spine_res ~= nil then
		if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
			self:addSpineAnimation()
		else
			DownloadControler:addDownLoadList(spine_res, 'spine/',  function()
					self:addSpineAnimation()
					tools.RedDotTools:setRedDotToNode(self.swtichBtn,false)
				end)
			show = true
		end
		--self.swtichBtn:setEnabled(true)
	else
		--self.swtichBtn:setEnabled(false)
	end
	
	AudioManager:downloadCV(self.resId)

	-- EventManager:FireEvent(EventType.UIHeadeUpdateMoneyNodeVisible,{[1] = false , [2] = false});

	self:switchState(2);
	
	local showAvatar = false;
	if self.roleId ~= 0 then
		self.idx = 1;
		self.resIdList = model.CardModel:getAvatarResList(self.roleId,true);
		if #self.resIdList > 1 then
			for i,v in ipairs(self.resIdList) do
				if v == self.resId then
					self.idx = i;
				end
			end
			showAvatar = true;
			self:setAvatarState()
		end
	end
	self.avatarNode:setVisible(showAvatar);

	tools.RedDotTools:setRedDotToNode(self.swtichBtn,show,nil,cc.p(130,35))
end
--[[
function CardLivePanel:initAvatarData()
	-- body
	self.idx = 1;
	self.resIdList = model.CardModel:getAvatarResList(self.roleId);
	for i,v in ipairs(self.resIdList) do
		if v == self.resId then
			self.idx = i;
		end
	end

end
]]
function CardLivePanel:updateInfo( idx )
	-- body
	if idx < 1 or idx > #self.resIdList then
		return;
	end

	local resId = self.resIdList[idx];

	if resId >= 1000 then 
		local isHave = model.CardModel:onIsHaveAvatarByAvatarItemId(resId);
		if not isHave then
			return;
		end
	end

	self.idx = idx;
	local roleResource = ResTable:getRowData('role_resource', resId);
	if not roleResource then
		return;
	end

	local spine_res = roleResource.spine;
	local imagePath = roleResource.card;
	self.spine_res = spine_res;

	self.cardPanelNavi:setTexture('image/role/'..imagePath..'.png');
	self.resId = resId;

	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end

	if spine_res ~= nil then
		if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
			self:addSpineAnimation()
		else
			DownloadControler:addDownLoadList(spine_res, 'spine/',  function()
					self:addSpineAnimation()
				end)
		end
		--self.swtichBtn:setEnabled(true)
	else
		--self.swtichBtn:setEnabled(false)
	end
	
	AudioManager:downloadCV(self.resId)

	self:setAvatarState();
	self:switchState(2);
end

function CardLivePanel:addSpineAnimation()
	-- body
	local roleResource = ResTable:getRowData('role_resource', self.resId);
	if not roleResource then
		return;
	end
	local spine_res = roleResource.spine;
	self.spine_res = spine_res;

	if spine_res~=nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then

		if self.skeletonNode then
			self.skeletonNode:removeAllChildren();
		end

		local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		skAnimation:addTo(self.skeletonNode)
		skAnimation:setAnimation(0, "idle", true)
		-- registerEvent
		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_START)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_END)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_COMPLETE)

		skAnimation:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_EVENT)
	end
end

function CardLivePanel:initAvatarFlag()
	-- body
	self.avatarNode = cc.Node:create();
	self.avatarNode:addTo(self.root);

	local avatarFlagNode = cc.Node:create();
	avatarFlagNode:addTo(self.avatarNode);
	avatarFlagNode:setPosition(-StageSize.width/2 + 30 , StageSize.height/2-168);

	local avatarFlag = cc.Sprite:create('image/card/avatar_flag.png');
	avatarFlag:addTo(avatarFlagNode);
	avatarFlag:setAnchorPoint(cc.p(0, 0.5));
	avatarFlag:setPosition(0 , 0);

	self.leftBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	self.leftBtn:addTo(self.avatarNode);
	self.leftBtn:setAnchorPoint(cc.p(0, 0.5));
	self.leftBtn:setPosition(-StageSize.width/2 + 30 , 0);
	self.leftBtn:addClickEventListener(handler(self , self.leftBtnClick));

	self.rightBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	self.rightBtn:addTo(self.avatarNode);
	self.rightBtn:setAnchorPoint(cc.p(0, 0.5));
	self.rightBtn:setRotation(180);
	self.rightBtn:setPosition(StageSize.width/2 - 30 , 0);
	self.rightBtn:addClickEventListener(handler(self , self.rightBtnClick));

	self.selAvatarList = {};

	for i=1,10 do

		local idxNode = cc.Node:create();
		idxNode:addTo(avatarFlagNode);
		idxNode:setPosition(i*35+70, 0);
		idxNode:setVisible(false);

		local bg1 = cc.Sprite:create('image/card/avatar_s2.png');
		bg1:addTo(idxNode);
		local bg2 = cc.Sprite:create('image/card/avatar_s1.png');
		bg2:addTo(idxNode);
		bg2:setTag(0);

		local idxLabel = cc.Label:createWithTTF(''..i , fonts.FONT_SIYUAN , 20);
		idxLabel:addTo(idxNode);
		idxLabel:setTextColor(cc.c3b(255 , 255 , 255));
		idxLabel:setTag(1);

		self.selAvatarList[ i ] = idxNode;

	end

end

function CardLivePanel:setAvatarState()
	-- body
	for k,v in pairs(self.selAvatarList) do

		if k <= #self.resIdList then
			v:setVisible(true);
			if k == self.idx then
				v:getChildByTag(0):setVisible(true);
				v:getChildByTag(1):setTextColor(cc.c3b(255 , 255 , 255));
				v:getChildByTag(1):enableOutline(cc.c4b(100 , 60 , 150 , 255) , 2);
			else 
				v:getChildByTag(0):setVisible(false);
				v:getChildByTag(1):setTextColor(cc.c3b(100 , 60 , 150));
				v:getChildByTag(1):disableEffect(); 
			end
		else
			v:setVisible(false);
		end
		
	end

	if self.idx == 1 then
		self.leftBtn:setVisible(false);
	else
		self.leftBtn:setVisible(true);
	end
	if self.idx == #self.resIdList then
		self.rightBtn:setVisible(false);
	else
		self.rightBtn:setVisible(true);
	end

end

function CardLivePanel:leftBtnClick(sender , event)
	self:updateInfo( self.idx - 1 );
end

function CardLivePanel:rightBtnClick(sender , event)
	self:updateInfo( self.idx + 1 );
end

function CardLivePanel:swtichBtnClick(sender , event)
	local tag = sender:getTag();
	self:switchState(tag)
end

function CardLivePanel:switchState(state)


	if state == 1 and self.skeletonNode:getChildrenCount() == 0 then
		--if self.spine_res and self.spine_res ~= '' then
		--	Prompt:Show(LANG.UI_UPDATE_2);
		--else
			Prompt:Show(LANG.UI_CARD_2);
		--end
		return
	end

	self.swtichBtn:setTag(state == 1 and 2 or 1);
	self.swtichBtn:setBright(state == 2)

	self.cardPanelNavi:setVisible(state == 2);

	if self.skeletonNode:getChildrenCount() > 0 then
		self.skeletonNode:setVisible(state == 1);
	end
	--self.swtichLabel:setString(tostring(LANG.UI_CARD_LIST_SWITCH_BTN_LABEL[state]));
end

-- function CardLivePanel:onClickBack()
-- 	EventManager:FireEvent(EventType.UIHeadeUpdateMoneyNodeVisible,{[1] = true , [2] = true});
-- 	UIManager:pop('CardLivePanel');
-- end

function CardLivePanel:onLeave()
	-- if self.skeletonNode then
	-- 	self.skeletonNode:removeFromParent()
	-- 	self.skeletonNode = nil
	-- end
	if self.skeletonNode then
		self.skeletonNode:removeAllChildren();
	end
	self.root:setVisible(false);
end

return CardLivePanel;
