local CardResolvePanel = class("CardResolvePanel", require('app.layer.BaseLayer'))

local fonts = Enum:getTable('SystemFont');
local MessageBoxType = Enum:getTable('MessageBoxType')

local imagePath = 'image/cardResolve/';

local limitNum = 15;

local ResolveType = {
	RESOLVE = 1,
	RESET = 2,
}

function CardResolvePanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'CardResolvePanel');
	
	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;

	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function CardResolvePanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels();
	self:registEvent();
	
end

function CardResolvePanel:initPanels()

	local bg = cc.Sprite:create('image/bg/Bg_5.png');
	bg:addTo(self.root);

	local bg = cc.Sprite:create(imagePath..'resolve_bg_1.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -10);

	self.menuBtnList = {};
	for i = 1 , 2 do
		self.menuBtnList[i] = {};

		local node = cc.Node:create();
		node:addTo(self.root);
		node:setPosition(i == 1 and -151 or 151  , 430);
	    self.menuBtnList[i].unselectNode = node;
	
	    local unTexture = imagePath..'resolve_btn_3.png';
	    local texture = imagePath..'resolve_btn_4.png';

		local bgBtn = ccui.Button:create(unTexture , unTexture);
		bgBtn:addTo(node);
		bgBtn:setTag(i);
		bgBtn:addClickEventListener(handler(self , self.menuClick));

		local label = cc.Label:createWithTTF(LANG.UI_CARD_RESOLVE_MENU_BTN[i] , fonts.FONT_SIYUAN , 20);
		label:addTo(node);
		label:setTextColor(cc.c3b(180 , 180 , 180));

		node = cc.Node:create();
		node:addTo(self.root);
		node:setPosition(i == 1  and -151 or 151  , 430);
		self.menuBtnList[i].selectNode = node;

		bgBtn = ccui.Button:create(texture , texture);
		bgBtn:addTo(node);

		label = cc.Label:createWithTTF(LANG.UI_CARD_RESOLVE_MENU_BTN[i] , fonts.FONT_SIYUAN , 20);
		label:addTo(node);
	end
	
	self.topBg = cc.Sprite:create(imagePath..'resolve_bg_3.png');
	self.topBg:addTo(self.root);
	self.topBg:setPosition(0 , 197);

	self.selectCardNode = cc.Node:create();
	self.selectCardNode:addTo(self.root);
	self.selectCardNode:setPosition(0 , 197);

	self.resetCardSpineNode = cc.Node:create();
	self.resetCardSpineNode:addTo(self.root);
	self.resetCardSpineNode:setPosition(0 , 197);

	self.resolveMidNode = cc.Node:create();
	self.resolveMidNode:addTo(self.root);
	self.resolveMidNode:setPosition(-87 , -30); -- -96,-44

	local icon = ResTable:getValue('item' , COLOR_DIA , 'icon');
	self.resloveMidIcon = cc.Sprite:create("image/icon/I"..icon..".png");
	self.resloveMidIcon:addTo(self.resolveMidNode);
	self.resloveMidIcon:setPosition(-165 , 0);
	self.resloveMidIcon:setScale(0.45);

	self.resloveMidLabel = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.resloveMidLabel:addTo(self.resolveMidNode);
	self.resloveMidLabel:setPosition(-133 , 0); -- -125
	self.resloveMidLabel:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);

	self.resolveMidNum = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.resolveMidNum:addTo(self.resolveMidNode);
	self.resolveMidNum:enableOutline(cc.c4b(45 , 115 , 30 , 255) , 2);
	self.resolveMidNum:setAnchorPoint(1 , 0.5);
	self.resolveMidNum:setPosition(193 , 0); --170
	
	self.resolveBtn = ccui.Button:create(imagePath..'resolve_btn_1.png' , imagePath..'resolve_btn_1.png');
	self.resolveBtn:addTo(self.root);
	self.resolveBtn:setPosition(250 , -30);
	self.resolveBtn:addClickEventListener(handler(self , self.resolveBtnClick));

	self.resolveTipNode = cc.Node:create();
	self.resolveTipNode:addTo(self.root);
	self.resolveTipNode:setPosition(-270 , -30); -- 0 , -44

	self.resolveTipImg = cc.Sprite:create('image/commonImg/tip_2.png');
	self.resolveTipImg:addTo(self.resolveTipNode);

	self.resolveTip = cc.Label:createWithTTF('',fonts.FONT_SIYUAN , 20);
	self.resolveTip:addTo(self.resolveTipNode);
	self.resolveTip:setAnchorPoint(0 , 0.5);
	self.resolveTip:setPosition(20 , 0)
	self.resolveTip:setTextColor(cc.c3b(255 , 200 , 0));

	local initData = {
		width = 580,
		height = 378,
		cellWidth = 150,
		cellHeight = 378,
		direction = 0,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

	}
	self.cardView = template.TableViewTemplate:create()
	self.cardView:initWithSizeAndItem(initData) ;
	self.cardView:getView():setPosition(cc.p(-290, -430));
	self.cardView:getView():addTo(self.root);

	local label = cc.Label:createWithTTF(LANG.UI_CARD_RESOLVE_1 , fonts.FONT_SIYUAN , 20);
	label:enableOutline(cc.c4b(125 , 125 , 125, 255) , 2);

	local resolveShopBtn = ccui.Button:create('image/commonImg/btn_5_132_45.png' , 'image/commonImg/btn_5_132_45.png');
	resolveShopBtn:addTo(self.root);
	resolveShopBtn:setPosition(244 , StageSize.height/2 - 92 - NOTCH_HEIGHT);
	resolveShopBtn:addClickEventListener(handler(self , self.resolveShopBtnClick));
	resolveShopBtn:setTitleLabel(label);
end

function CardResolvePanel:registEvent(  )
	EventManager:RegisterEvent(EventType.NetworkRetCardResolve , handler(self , self.retCardResolve));
	EventManager:RegisterEvent(EventType.NetworkRetCardReset , 	handler(self , self.retCardReset));
end

function CardResolvePanel:onEnter()

	self.resolveType = ResolveType.RESOLVE;
	self:updateMenuState()
	self:reset();

	self.showSpine = false;
	self.resetCardSpineNode:removeAllChildren();

	self.root:setVisible(true);
end

function CardResolvePanel:menuClick(sender , event)
    local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	if self.showSpine then
		return;
	end

    if tag == self.resolveType then
        return;
    end

    self.resolveType = tag
	self:updateMenuState();
	self:reset();
	self.resetCardSpineNode:removeAllChildren();

end

function CardResolvePanel:updateMenuState()
	for i = 1 , 2 do
        self.menuBtnList[i].unselectNode:setVisible(i ~= self.resolveType);
        self.menuBtnList[i].selectNode:setVisible(i == self.resolveType);
	end
	
	if self.resolveType == ResolveType.RESOLVE then
		self.topBg:setTexture(imagePath..'resolve_bg_3.png');
	else
		self.topBg:setTexture(imagePath..'resolve_bg_5.png');
	end
end

function CardResolvePanel:reset()

	self:updateCard();

	self:updateResolveMidState(0);

	self.selectCardNode:removeAllChildren();

	self.selectCardIndex = {};
	self.selectCardTemplateList = {};

	self.cardView:reloadData(self.cards);
end

function CardResolvePanel:updateCard()
	local allCard = model.CardModel:GetAllCard();
	self.cards = {};
	if self.resolveType == ResolveType.RESOLVE then
		for i , card in pairs(allCard) do
			if card.quality <= 4 and card.star_lv <= 4 then
				table.insert(self.cards , card);
			end
		end

		if #self.cards > 1 then
			table.sort(self.cards , function(first , second)

				local firstInModel = model.CardModel:isInOtherModule(first.pid);
				local secondInModel = model.CardModel:isInOtherModule(second.pid);

				if not firstInModel and secondInModel then
					return false
				elseif firstInModel and not secondInModel then
					return true;
				end

				local ret = model.CardModel:sortPid(first.pid, second.pid, -1)
				return ret;

				-- if first.level ~= second.level then
				-- 	return first.level < second.level
				-- end
				-- if first.star_lv ~= second.star_lv then
				-- 	return first.star_lv < second.star_lv;
				-- end
				
				-- return first.no < second.no; 
			end);
		end

	else
		for i , card in pairs(allCard) do
			if card.oriLv then
				if card.oriLv > 1 then
					table.insert(self.cards , card);
				end
			elseif card.level > 1 then
				table.insert(self.cards , card);
			end
		end

		if #self.cards > 1 then
			model.CardModel:sortFuc2(self.cards)
			-- table.sort(self.cards , function(first , second) 
			-- 	if first.level ~= second.level then
			-- 		return first.level < second.level
			-- 	end
			-- 	if first.star_lv ~= second.star_lv then
			-- 		return first.star_lv < second.star_lv;
			-- 	end
				
			-- 	return first.no < second.no;
			-- end);
		end
	end
	local temCards = {}
	local idx = #self.cards
	for i=1,#self.cards do
		table.insert( temCards, self.cards[idx])
		idx = idx - 1;
	end
	self.cards = temCards;
end

function CardResolvePanel:reloadCellPanel( cell,index )
	local cellNode  = cell:getChildByTag(999);
	if nil == cellNode then
		cellNode = template.CardTemplate:create(cell);
		cellNode:setTouchEnabled(false);
		cellNode:setPosition(79.5, 189);
		cellNode:setScale(0.75);
		cellNode:setTag(999);
	end

	local cardData = self.cards[index+1];

	if self.resolveType == ResolveType.RESOLVE then
		cellNode:Init(cardData.pid);
		local isInModel , states = model.CardModel:isInOtherModule(cardData.pid);
		cellNode:initCardInTeamState(states,true);
	else
		cellNode:InitCardReset(cardData.pid);
		cellNode:initCardInTeamState({});
	end

	if self.selectCardIndex[index+1] then
		cellNode:setState('select',2);
	else
		cellNode:setState('nona');
	end
end

function CardResolvePanel:resolveBtnClick(sender , event)
	local btnType = sender:getTag();
	AudioManager:playEffect(Conf.confirmButton);
	if self.showSpine then
		return;
	end

	if btnType == 1 then --快速添加
		local selectNum = 0;
		local resolve_coin = 0;
		self.selectCardIndex = {};
		self.selectCardTemplateList = {};
		self.selectCardNode:removeAllChildren();
		for index , card in ipairs(self.cards) do
			local isIn = model.CardModel:isInOtherModule(card.pid);
			if not isIn then
				local cell = self.cardView:cellAtIndex(index - 1);
				local cellNode = nil;
				if cell then
					cellNode = cell:getChildByTag(999);
				end

				if card.level <= 1 and card.star_lv < 4 then
					selectNum = selectNum + 1;
					resolve_coin = resolve_coin + 1;
					self.selectCardIndex[index] = card.pid;
					table.insert(self.selectCardTemplateList , {idx = index})
					if cellNode then
						cellNode:setState('select',2);
					end
				else
					if cellNode then
						cellNode:setState('nona');
					end
				end
			end

			if selectNum >= limitNum then
				break;
			end
		end

		if selectNum > 0 then
			local cellWidth = 442 / selectNum;
			local posX = cellWidth / 2 - 442 / 2;
			for index , data in pairs(self.selectCardTemplateList) do
				local posX = posX + cellWidth * (index - 1);
				local bigCard = template.CardTemplate:create(self.selectCardNode);
				bigCard:Init(self.cards[data.idx].pid);
				bigCard:setPosition(posX, 0);
				bigCard:setScale(0.8);
				bigCard:addCardClickEventListener(handler(self, self.selectCardClick));
				bigCard:setCardTag(index);
				bigCard:setCardSwallowTouches(true);
				bigCard:setLocalZOrder(index * 10);
				self.selectCardTemplateList[index].bigCard = bigCard;
			end
			
			self:updateResolveMidState(resolve_coin);
		end
	elseif btnType == 2 then--分解
		local selectCardNum = #self.selectCardTemplateList;
		if selectCardNum <= 0 then
			return;
		elseif selectCardNum > limitNum then
			Prompt:Show(string.format( LANG.UI_CARD_RESOLVE_17, tostring(limitNum)) );
			return;
		end

		local msg = {};
		msg.cardList = {};

		for _k , cardData in pairs(self.selectCardTemplateList) do
			if self.cards[cardData.idx] and self.cards[cardData.idx].pid then
				table.insert(msg.cardList , self.cards[cardData.idx].pid);
			end
		end

		for _ , pid in pairs(msg.cardList) do
			if model.CardModel:isFutureCard(pid) then
				Prompt:Show(LANG.UI_L_11706);
				return;
			end
		end
		AudioManager:playEffect('V_UI_Resolve', 1)

		Network.NetworkManager:Send(Network.NetworkCmdType.req_card_resolve , msg);
	elseif btnType == 3 then --重置
		if #self.selectCardTemplateList <= 0 then
			return;
		end

		MessageBox:create(LANG.UI_CARD_RESOLVE_18 , MessageBoxType.CANCEL_OK,function()
			local msg = {};
			msg.cardList = {};

			for _k , cardData in pairs(self.selectCardTemplateList) do
				if self.cards[cardData.idx] and self.cards[cardData.idx].pid then
					table.insert(msg.cardList , self.cards[cardData.idx].pid);
				end
			end

			Network.NetworkManager:Send(Network.NetworkCmdType.req_card_reset , msg);
		end);
	end
end

function CardResolvePanel:updateSelectCard()
	local selectNum = #self.selectCardTemplateList;
	local cellWidth = 442 / selectNum;
	local posX = cellWidth / 2 - 442 / 2;
	local totalCost = 0;
	local resolve_coin = 0;
	for index = 1 , selectNum do
		self.selectCardTemplateList[index].bigCard:setPosition(posX + cellWidth * (index - 1), 0);
		self.selectCardTemplateList[index].bigCard:setCardTag(index);
		self.selectCardTemplateList[index].bigCard:setLocalZOrder(index * 10);

		if self.resolveType == ResolveType.RESOLVE then
			local pid = self.selectCardTemplateList[index].bigCard.pid;
			local card = model.CardModel:GetCard(pid);
			if card then
				if card.quality >= 4 then
					resolve_coin = resolve_coin + 5;
				else
					resolve_coin = resolve_coin + 1;
				end
			else
				resolve_coin = resolve_coin + 1;
			end

		else
			local idx = self.selectCardTemplateList[index].idx;
			local costs = ResTable:getMultiValue('role_reset' , self.cards[idx].star_lv);
			for _k , cost in pairs(costs['cost']) do
				totalCost = totalCost + cost[2];
			end
			self.resolveMidNum:setString(tostring(totalCost));
		end
	end

	if self.resolveType == ResolveType.RESOLVE then
		self:updateResolveMidState(resolve_coin);
	else
		self:updateResolveMidState(selectNum);
	end
end

function CardResolvePanel:updateResolveMidState(selectNum)

	if self.resolveType == ResolveType.RESOLVE then
		--self.resolveTipNode:setPosition(-58 , -30);
		self.resolveMidNode:setPosition(-87 , -30);
		self.resolveMidNum:setPosition(193 , 0);
		self.resloveMidLabel:setPosition(-133 , 0);
		self.resloveMidIcon:setVisible(false);
		self.resolveBtn:setVisible(true);
		self.resloveMidLabel:setString(tostring(LANG.UI_CARD_RESOLVE_2));
		if selectNum > 0 then
			self.resolveTipNode:setVisible(false);
			self.resolveMidNode:setVisible(true);
			self.resolveMidNum:setString(tostring(selectNum));
			self.resolveBtn:setTag(2);
			self.resolveBtn:loadTextures(imagePath..'resolve_btn_1.png',imagePath..'resolve_btn_1.png')
		else
			self.resolveTipNode:setVisible(true);
			self.resolveMidNode:setVisible(false);
			self.resolveBtn:setTag(1);
			self.resolveTip:setString(tostring(LANG.UI_CARD_RESOLVE_5));
			self.resolveBtn:loadTextures(imagePath..'resolve_btn_6.png',imagePath..'resolve_btn_6.png')

			--local width = self.resolveTip:getContentSize().width;
			--local posX = -width / 2;
			--self.resolveTipImg:setPositionX(posX);
			--self.resolveTip:setPositionX(posX + self.resolveTipImg:getContentSize().width / 2 + width / 2);
		end
	else
		--self.resolveTipNode:setPosition(0 , -30);
		self.resolveMidNode:setPosition(-96 , -30);
		self.resolveMidNum:setPosition(170 , 0);
		self.resloveMidLabel:setPosition(-125 , 0);
		self.resloveMidIcon:setVisible(true);
		self.resloveMidLabel:setString(tostring(LANG.UI_CARD_RESOLVE_6));
		self.resolveBtn:loadTextures(imagePath..'resolve_btn_5.png',imagePath..'resolve_btn_5.png')

		if selectNum > 0 then
			self.resolveBtn:setVisible(true);
			self.resolveTipNode:setVisible(false);
			self.resolveMidNode:setVisible(true);
			self.resolveBtn:setTag(3);
		else
			self.resolveTipNode:setVisible(true);
			self.resolveMidNode:setVisible(false);
			self.resolveBtn:setVisible(false);

			self.resolveTip:setString(tostring(LANG.UI_CARD_RESOLVE_8));

			--local width = self.resolveTip:getContentSize().width;
			--local posX = -width / 2;
			--self.resolveTipImg:setPositionX(posX);
			--self.resolveTip:setPositionX(posX + self.resolveTipImg:getContentSize().width / 2 + width / 2);
		end
	end
end

function CardResolvePanel:selectCardClick(sender , event)
	local index = sender:getTag();
	self:selectCancel(index)
end

function CardResolvePanel:selectCancel(index)
	if self.showSpine then
		return;
	end

	if self.selectCardTemplateList[index] then
		local data = self.selectCardTemplateList[index];
		local cell = self.cardView:cellAtIndex(data.idx - 1);
		if cell then
			local cellNode = cell:getChildByTag(999);
			if cellNode then
				cellNode:setState('nona');
			end
		end

		self.selectCardTemplateList[index].bigCard:removeFromParent();

		self.selectCardIndex[data.idx] = nil;

		table.remove(self.selectCardTemplateList , index);

		self:updateSelectCard();
	end
end

function CardResolvePanel:cellItemTouch( table,cell )
	local idx = cell:getIdx();
	AudioManager:playEffect(Conf.buttonClick);
	if self.showSpine then
		return;
	end

	if self.selectCardIndex[idx+1] then
		--移出
		local fidIdx = 0;
		for k,v in pairs(self.selectCardTemplateList) do
			if v.idx == (idx+1) then
				fidIdx = k;
				break;
			end
		end
		self:selectCancel(fidIdx)
		return;
	end

	local index = #self.selectCardTemplateList + 1;
	if index > limitNum then
		Prompt:Show(string.format( LANG.UI_CARD_RESOLVE_17, tostring(limitNum)) );
		return;
	end

	local cardData = self.cards[idx + 1];
	local pid = cardData.pid;

	if self.resolveType == ResolveType.RESOLVE then
		if model.CardModel:isInOtherModule(pid) then
			Prompt:Show(LANG.UI_CARD_RESOLVE_16);
			return;
		end

		if model.CardModel:isFutureCard(pid) then
			Prompt:Show(LANG.UI_L_11706);
			return;
		end
	else
		local pidInEventTeam = model.TeamModel:isRoleInEventTeam(pid)
		if pidInEventTeam==true then
			Prompt:Show(LANG.UI_L_11996);
			return;
		end
	end

	local cellNode = cell:getChildByTag(999);
	if cellNode then
		cellNode:setState('select',2);
		self.selectCardIndex[idx+1] = pid;
	end

	local bigCard = template.CardTemplate:create(self.selectCardNode);
	if self.resolveType == ResolveType.RESOLVE then
		bigCard:Init(pid)
	else
		bigCard:InitCardReset(pid)
	end
	bigCard:setScale(0.8);
	bigCard:addCardClickEventListener(handler(self, self.selectCardClick));
	bigCard:setCardSwallowTouches(true);
	
	self.selectCardTemplateList[index] = {idx = idx + 1 , bigCard = bigCard};

	self:updateSelectCard();
end

function CardResolvePanel:retCardResolve(msg)
	self.cardResolveMsg = msg;
	local selectNum = #self.selectCardTemplateList;
	local cellWidth = 442 / selectNum;
	local posX = cellWidth / 2 - 442 / 2;
	self.showSpine = true;
	self.resetCardSpineNode:removeAllChildren();
	self:showCardResolveSpine(posX , cellWidth , selectNum);
end

function CardResolvePanel:showCardResolveSpine(posX , cellWidth , index)

	if index <= 0 then
		self:reset();
		self.showSpine = false;
		UIManager:push('CardResolveRewardPanel' ,self.cardResolveMsg,1);
		return;
	end
	if cc.FileUtils:getInstance():isFileExist('spine/fenjie_spine/fenjie_spine.json') then
		local cardResolveSpine = sp.SkeletonAnimation:create('spine/fenjie_spine/fenjie_spine.json', 'spine/fenjie_spine/fenjie_spine.atlas', 1)

		cardResolveSpine:addTo(self.resetCardSpineNode)
		cardResolveSpine:setPosition(posX + cellWidth * (index - 1) , 0);
		cardResolveSpine:setAnimation(0, "fenjie_spine", false);
		cardResolveSpine:setTimeScale(2);
		cardResolveSpine:registerSpineEventHandler(function (event)
			if event.eventData.name == 'remove' then
				self.selectCardTemplateList[index].bigCard:removeFromParent();
				self.selectCardTemplateList[index] = nil;
				self:showCardResolveSpine(posX , cellWidth , (index-1));
			end
		end, sp.EventType.ANIMATION_EVENT);
		cardResolveSpine:registerSpineEventHandler(function (event)
			cardResolveSpine:runAction(cc.Sequence:create(cc.DelayTime:create(0.01), cc.CallFunc:create(function() 
				cardResolveSpine:removeFromParent();
			end)));
		end, sp.EventType.ANIMATION_COMPLETE)
		
	end
end

function CardResolvePanel:retCardReset(msg)
	self.resetCardSpineNode:removeAllChildren();
	if cc.FileUtils:getInstance():isFileExist('spine/chongzhi_spine/chongzhi_spine.json') then
		local cardResetSpine = sp.SkeletonAnimation:create('spine/chongzhi_spine/chongzhi_spine.json', 'spine/chongzhi_spine/chongzhi_spine.atlas', 1)

		cardResetSpine:addTo(self.resetCardSpineNode)
		cardResetSpine:setAnimation(0, "chongzhi_spine", false);
		cardResetSpine:registerSpineEventHandler(function (event)
			if event.eventData.name == 'remove' then
				self:reset();
				UIManager:push('CardResolveRewardPanel' ,msg,2);
				self.showSpine = false;
			end
		end, sp.EventType.ANIMATION_EVENT);

		self.showSpine = true;
	end
end

function CardResolvePanel:resolveShopBtnClick()
	AudioManager:playEffect(Conf.buttonClick);
	--Prompt:Show(LANG.COMMON_CLOSE);
	UIManager:push('StorePanel' , Enum:getTable('CommonShopType').RESOLVE);
end

function CardResolvePanel:onClickBack()
	if self.showSpine then
		return;
	end
	UIManager:pop('CardResolvePanel');
end

function CardResolvePanel:onLeave()
	self.showSpine = false;
	self.resetCardSpineNode:removeAllChildren();
	self.root:setVisible(false);
end

return CardResolvePanel;
