local ChatPanel = class('ChatPanel', require('app.layer.BaseLayer'))

local imgPath = 'image/chat/';
local fonts = Enum:getTable("SystemFont");

function ChatPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'ChatPanel');

	self.useCCreator = false;
	self.isPopup = true;
	self.haveSwallowLayer = true;

end

function ChatPanel:Init()
	self:initPanels();
	self.root:setVisible(false);

	self.channel = 1;
	self.chatTime1 = 0;
	self.chatTime3 = 0;
	self.editInputStr = ''

	EventManager:RegisterEvent(EventType.ChatChangeRoom, handler(self, self.setRoom));
	EventManager:RegisterEvent(EventType.ChatGenMsg, handler(self, self.getMessage));
	--EventManager:RegisterEvent(EventType.AddNewAnnouncement, handler(self, self.pushAnnouncement))
end

function ChatPanel:initPanels()
	self.touchLayer = ccui.Layout:create();
	self.touchLayer:setPosition(-StageSize.width/2, StageSize.height/2-755);
	self.touchLayer:setLocalZOrder(0);
	self.touchLayer:setContentSize(cc.size(StageSize.width, 755));
	self.touchLayer:setTouchEnabled(true);
	self.touchLayer:setSwallowTouches(true);
	self.touchLayer:addClickEventListener(handler(self, self.hideNumBoard));
	self.touchLayer:setVisible(false)
	self.touchLayer:addTo(self.root);

	self.bg = cc.Scale9Sprite:create(imgPath .. 'board2.png')
		:setCapInsets(CCRectMake(15, 5, 19, 26))
		:setContentSize(cc.size(604, 608))
		:setAnchorPoint(cc.p(0.5, 1))
		:setPosition(0, StageSize.height/2-10)
		:addTo(self.root)

	self.bg2 = cc.Scale9Sprite:create(imgPath .. 'bg.png')
		:setCapInsets(CCRectMake(11, 11, 18, 12))
		:setContentSize(cc.size(582, 526))
		:setAnchorPoint(cc.p(0.5, 1))
		:setPosition(0, StageSize.height/2-40)
		:addTo(self.root)

	-- word view
	self.viewList = {};
	for i=1, 3 do
		local listView = ccui.ListView:create();
		listView:setTouchEnabled(true)
		listView:setBounceEnabled(true)
		listView:setDirection(ccui.ScrollViewDir.vertical)
		listView:setContentSize(cc.size(580, 505));
		listView:setAnchorPoint(cc.p(0.5, 1))
		listView:setPosition(0, StageSize.height/2-55)
		--view:setScrollBarEnabled(false)
		listView:setItemsMargin(5);
		listView:addTo(self.root)
		--listView:ignoreContentAdaptWithSize(false);
		self.viewList[i] = listView;
	end

	-- channel
	self.channelList = {}
	local channelPos = {
		[1] = cc.p(-194, StageSize.height/2-35),
		[2] = cc.p(0, StageSize.height/2-35),
		[3] = cc.p(194, StageSize.height/2-35),
	}
	local curRoom = 1
	for i=1, 3 do
		local channel = {}
		self.channelList[i] = channel;

		local btn = ccui.Button:create(imgPath .. 'channel_btn.png')
			:setScale9Enabled(true)
			:setCapInsets(CCRectMake(8, 8, 55, 30))
			:setContentSize(cc.size(194, 30))
			:setPosition(channelPos[i])
			:setTag(i)
			:addTo(self.root)
		btn:addClickEventListener(handler(self, self.clickChannelBtn))
		channel.btn = btn

		local channelName = LANG.CHAT_CHANNEL[i]
		--[[
		if i == 1 then
			channelName = LANG.CHAT_CHANNEL[i] .. curRoom;
			local arrow = cc.Sprite:create(imgPath .. 'arrow.png');
			arrow:setPosition(230, 16);
			arrow:addTo(btn);
		end
		local label = cc.Label:createWithTTF(channelName, fonts.FONT_SIYUAN, 20)
			:setTextColor(cc.c3b(50, 50, 50))
			:enableOutline(cc.c3b(255, 255, 255), 2)
			--]]
		if i ~= 1 then
			local label = cc.Label:createWithTTF(channelName, fonts.FONT_SIYUAN, 20)
				:setTextColor(cc.c3b(158, 158, 158))
--				:enableOutline(cc.c3b(255, 255, 255), 2)

			btn:setTitleLabel(label);
		end
		
		local node = cc.Node:create()
			:setPosition(channelPos[i])
			:addTo(self.root)

		local sp = cc.Scale9Sprite:create(imgPath .. 'channel_sp1.png')
			:setCapInsets(CCRectMake(8, 8, 55, 30))
			:setContentSize(cc.size(194, 30))
			:addTo(node)
		local sp2 = cc.Sprite:create(imgPath .. 'channel_sp2.png')
			:addTo(node)
			:setPosition(10, 14)
		channel.sp = node

		local word = cc.Label:createWithTTF(channelName, fonts.FONT_SIYUAN, 20)
			:setTextColor(cc.c3b(255, 255, 255))
			:enableOutline(cc.c3b(60, 60, 60), 1)
			:setPosition(channelPos[i])
			:addTo(self.root)
		channel.word = word

		if i == 1 then
			local layout = ccui.Layout:create()
			layout:setContentSize(cc.size(194, 30))
			layout:setPosition(channelPos[i].x-101, channelPos[i].y-16)
			layout:addTo(self.root)
			layout:setTouchEnabled(true)
			layout:addClickEventListener(handler(self, self.showNumBoard))

			channel.layout = layout
		end
	end

	-- bottom
	self.bottomNode = cc.Node:create();
	self.bottomNode:setPosition(0, StageSize.height/2-590);
	self.bottomNode:addTo(self.root);

	local textBoxBg = cc.Scale9Sprite:create(imgPath .. 'board1.png')
		:setCapInsets(CCRectMake(11, 11, 34, 28))

	self.textBox = cc.EditBox:create(cc.size(415, 42), textBoxBg);
	self.textBox:setFontColor(cc.c3b(255, 255, 255))
	self.textBox:setPlaceHolder(LANG.CHAT_1);
	self.textBox:setPlaceholderFontColor(cc.c3b(100, 100, 100));
	self.textBox:setPlaceholderFont(fonts.FONT_SIYUAN,15)
	self.textBox:setMaxLength(42);
	self.textBox:setFont(fonts.FONT_SIYUAN, 15);
	self.textBox:registerScriptEditBoxHandler(handler(self, self.handlerInput));
	self.textBox:setPosition(-35, 0);
	self.textBox:addTo(self.bottomNode);

	self.sendBtn = ccui.Button:create(imgPath .. 'btn_blue.png');
	self.sendBtn:setPosition(240, 0);
	self.sendBtn:setContentSize(cc.size(140, 50))
	self.sendBtn:addClickEventListener(handler(self, self.clickSendMsg));
	self.sendBtn:addTo(self.bottomNode);
	local label = cc.Label:createWithTTF(LANG.CHAT_2, fonts.FONT_SIYUAN, 17)
		:setTextColor(cc.c3b(255, 255, 255))
		:enableOutline(cc.c3b(55, 85, 155), 2)
	self.sendBtn:setTitleLabel(label)

	-- num board
	self.numBoardNode = cc.Node:create()
		:setPosition(0, StageSize.height/2-160)
		:setVisible(false)
		:addTo(self.root)

	local numBg = cc.Sprite:create(imgPath .. 'numboard_bg.png')
		:addTo(self.numBoardNode)

	local boardBg1 = cc.Scale9Sprite:create(imgPath .. 'board1.png')
		:setCapInsets(CCRectMake(11, 11, 34, 28))
		:setPosition(-55, 40)
		:setContentSize(cc.size(454, 42))
		:addTo(self.numBoardNode)

	local boardBg2 = cc.Scale9Sprite:create(imgPath .. 'board1.png')
		:setCapInsets(CCRectMake(11, 11, 34, 28))
		:setPosition(-55, -30)
		:setContentSize(cc.size(454, 87))
		:addTo(self.numBoardNode)

	self.labelNum = cc.Label:createWithTTF(LANG.CHAT_4, fonts.FONT_SIYUAN, 20)
		:setPosition(-260, 40)
		:setTextColor(cc.c3b(68, 68, 68))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setAnchorPoint(cc.p(0, 0.5))
		:addTo(self.numBoardNode)

	for i=0, 9 do
		local x, y = 0;
		local btn = ccui.Button:create(imgPath .. 'num.png');
		btn:setTag(i);
		btn:addClickEventListener(handler(self, self.clickNum));
		btn:setZoomScale(0);
		if i < 5 then
			y = -10
		else
			y = -50
		end
		if i < 5 then
			x = (i-3)*90+35;
		else
			x = (i-8)*90+35;
		end
		btn:setPosition(x, y);
		btn:addTo(self.numBoardNode);

		local btnLabel = cc.Label:createWithTTF(tostring(i), fonts.FONT_SIYUAN, 17)
			:setTextColor(cc.c3b(255, 255, 255))
		btn:setTitleLabel(btnLabel)
	end

	local label = cc.Label:createWithTTF(LANG.UI_MAIL_9, fonts.FONT_SIYUAN, 17)
		:setTextColor(cc.c3b(255, 255, 255))
		:enableOutline(cc.c4b(140, 50, 50, 255), 2)
	local btnExit = ccui.Button:create(imgPath .. 'btn_red.png')
	btnExit:setTitleLabel(label)
	btnExit:setPosition(230, 40)
	btnExit:addClickEventListener(handler(self, self.hideNumBoard))
	btnExit:addTo(self.numBoardNode)

	local label = cc.Label:createWithTTF(LANG.CHAT_10, fonts.FONT_SIYUAN, 17)
		:setTextColor(cc.c3b(60, 60, 60))
		:enableOutline(cc.c4b(255, 255, 255, 255) , 2)
	local btnCancel = ccui.Button:create(imgPath .. 'btn_white.png')
	btnCancel:setTitleLabel(label)
	btnCancel:setPosition(230, -7)
	btnCancel:setTag(11)
	btnCancel:addClickEventListener(handler(self, self.clickNum))
	btnCancel:addTo(self.numBoardNode)

	local label = cc.Label:createWithTTF(LANG.COMMON_OK, fonts.FONT_SIYUAN, 17)
		:setTextColor(cc.c3b(255, 255, 255))
		:enableOutline(cc.c4b(55, 85, 155, 255), 2)
	local btnOk = ccui.Button:create(imgPath .. 'btn_blue.png')
	btnOk:setTitleLabel(label)
	btnOk:setPosition(230, -54)
	btnOk:setTag(12)
	btnOk:addClickEventListener(handler(self, self.clickNum))
	btnOk:addTo(self.numBoardNode)

	self.textNum = 0;
end

function ChatPanel:onEnter()
	self:chooseChannel(1);
	self.root:setVisible(true);

	self:addSwallowCallback(function()
		UIManager:pop('ChatPanel');
	end)
	self.editInputStr = ''
	self.textBox:setText(self.editInputStr);
	model.ChatModel:setPanelShow(true)
	if not model.ChatModel:isRunning() then
		model.ChatModel:open()
		self:firstEnter();
	end
end

function ChatPanel:onLeave()
	model.ChatModel:setPanelShow(false)
	self.root:setVisible(false);
end

function ChatPanel:firstEnter()
	-- set room 
	--self:setRoom()

	-- set messages
	for channel=1, 3 do
		local msgs = model.ChatModel:getMsgs(channel)
		for i=1, #msgs do
			self:genMsg(channel, msgs[i])
		end
		if #msgs > 0 then
			self.viewList[channel]:scrollToBottom(0.5, true);
		end
	end
end

function ChatPanel:setRoom()
	local curRoom = model.ChatModel:getRoom();
	local channelName = LANG.CHAT_CHANNEL[1] .. curRoom;
	local label = cc.Label:createWithTTF(channelName, fonts.FONT_SIYUAN, 20)
		:setTextColor(cc.c3b(255, 255, 255))
		:enableOutline(cc.c3b(60, 60, 60), 1)
	self.channelList[1].btn:setTitleLabel(label);
	self.channelList[1].word:setString(LANG.CHAT_CHANNEL[1] .. curRoom);
end

function ChatPanel:chooseChannel(id)
	if id == 2 then
		local guild_id = model.GuildModel:getGuildInfo().gid
		if guild_id == 0 then
			Prompt:Show(LANG.CHAT_6);
			return;
		end
	end

	self.channel = id;
	for i=1, 3 do
		self.channelList[i].btn:setVisible(i ~= id)
		self.channelList[i].sp:setVisible(i == id)
		self.channelList[i].word:setVisible(i == id)
		self.viewList[i]:setVisible(i == id)
	end

	self.channelList[1].layout:setVisible(1 == id)

	if id == 3 then
		self.textBox:setTouchEnabled(false);
		self.sendBtn:setTouchEnabled(false);
		self.textBox:setPlaceHolder(LANG.CHAT_7);
		self.textBox:setPlaceholderFontColor(cc.c3b(255, 255, 255))
	else
		self.textBox:setTouchEnabled(true);
		self.sendBtn:setTouchEnabled(true);
		self.textBox:setPlaceHolder(LANG.CHAT_1);
		self.textBox:setPlaceholderFontColor(cc.c3b(68, 68, 68));
	end
	self.editInputStr = ''

end

function ChatPanel:handlerInput(eventName, pSender)
	-- if eventName == "began" then
	-- 	self.textBox:setText("");
	-- end
	
	-- if event == "changed" then
	-- 	local str =  self.noticeEditBox:getText();
	-- 	-- self.notice:setString(str);
	-- 	self.changeNotice = true ;
	-- end
	
	if eventName == "return" then
		local str =  self.textBox:getText()
		self.editInputStr = str;
		local str2,bRet = tools.SensitiveTools:checkSensitive( str )
		self.textBox:setText(str2)
		if bRet == true then
			Prompt:Show(LANG.UI_SENSITIVE);
			return 
		end
	end

end

function ChatPanel:clickChannelBtn(sender, event)
	local tag = sender:getTag();
	self:chooseChannel(tag);
end

function ChatPanel:clickSendMsg()
	local status, statusTime = model.UserModel:getUserStatus()
	local UserStatus = Enum:getTable('UserStatus');
	if status == UserStatus.MUTED then
		local MessageBoxType = Enum:getTable('MessageBoxType')
		local mutedTime = statusTime
		local t = os.date('*t', mutedTime)
		local timeStr = string.format(LANG.TIME_FORMAT, t.year, t.month, t.day, t.hour, t.min)
		local str = string.format(LANG.USER_STATUS_MUTED, tostring(timeStr))
		MessageBox:create(str, MessageBoxType.OK, function()
		end, function()
		end, function()
		end);
		return
	end
	local text = self.editInputStr --self.logEditInput:getText();
	if text == '' then
		return 
	end
	local _,bRet = tools.SensitiveTools:checkSensitive( text )
	if bRet == true then
		Prompt:Show(LANG.UI_SENSITIVE);
		return 
	end
	if self.channel == 1 then
		if (os.time() - self.chatTime1) < 10 then
			Prompt:Show(LANG.CHAT_5);
			return;
		end
		self.chatTime1 = os.time();
		model.ChatModel:sendMessage(self.textBox:getText(), self.channel);
		self.textBox:setText('');
	elseif self.channel == 2 then
		if (os.time() - self.chatTime3) < 10 then
			Prompt:Show(LANG.CHAT_5);
			return;
		end
		self.chatTime3 = os.time();
		model.ChatModel:sendMessage(self.textBox:getText(), self.channel);
		self.textBox:setText('');
	end
end

function ChatPanel:showNumBoard()
	self.numBoardNode:setVisible(true)
	self.touchLayer:setVisible(true)
end

function ChatPanel:hideNumBoard()
	self.numBoardNode:setVisible(false)
	self.touchLayer:setVisible(false)
end

function ChatPanel:clickNum(sender, event)
	local tag = sender:getTag();
	if tag ~= 11 and string.len(tostring(self.textNum)) > 5 then
		return;
	end

	if (tag >= 0 and tag <= 9) then
		self.textNum = self.textNum*10 + tag;
		self.labelNum:setString(tostring(self.textNum));
	elseif tag == 11 then
		self.textNum = 0;
		self.labelNum:setString(tostring(self.textNum));
	elseif tag == 12 then
		if (self.textNum >= 1 and self.textNum <= 20) then
			model.ChatModel:changeRoom(self.textNum);
			self.textNum = 0;
			self.labelNum:setString(tostring(LANG.CHAT_4));
			self.numBoardNode:setVisible(false);
		end
	end
end

function ChatPanel:getMessage(channel, msg)
	self:genMsg(channel, msg);
	if model.ChatModel:getMsgNum(channel) > 1 then
		self.viewList[channel]:scrollToBottom(0.5, true);
	end
end

function ChatPanel:pushAnnouncement(announcement)
	if not self.root:isVisible() then
		return
	end

	for _, word in pairs(announcement) do
		local msg = {};
		msg.uid = -1;
		msg.head = 45;
		msg.data = tostring(word)
		msg.name = 'System'
		self:genMsg(1, msg)
	end
	self.viewList[1]:scrollToBottom(0.5, true);
end

function ChatPanel:genMsg(channel, msg)
	local layoutBaseWidth = 580
	local layoutBaseHight = 90
	if channel == 1 or channel == 2 then
		local layout = ccui.Layout:create();

		local isSelf = (msg.uid == model.UserModel:getUserUid());

		local headSpaceW, headSpaceH = 5, 0
		local headAp = cc.p(0, 1)
		local wordSpaceW, wordSpaceH = 100, 32
		local wordAp = cc.p(0, 1)
		local nameSpaceW, nameSpaceH = 85, 3
		local nameAp = cc.p(0, 1)
		local bgSpaceW, bgSpaceH = 80, 25
		local bgAp = cc.p(0, 1)
		local nameOutlineColor = cc.c3b(60, 60, 60)
		local wordToLabelH = wordSpaceH - bgSpaceH + 5
		if isSelf then
			headAp = cc.p(1, 1)
			wordAp = cc.p(1, 1)
			nameAp = cc.p(1, 1)
			bgAp = cc.p(1, 1)
			nameOutlineColor = cc.c3b(60, 60, 60)
		end
		
		local headbg = cc.Sprite:create('image/head/head_bg1.png')
			:addTo(layout)
			:setScale(0.6)
			:setAnchorPoint(headAp)

		local fileName = GlobalFunctions:getHeadIconFile(msg.head)
		local headIcon = cc.Sprite:create();
		if cc.FileUtils:getInstance():isFileExist(fileName) then
			headIcon:setTexture(fileName);
		end
		headIcon:addTo(layout)
			:setScale(0.6)
			:setAnchorPoint(headAp)

		local bg
		if isSelf then
			bg = cc.Scale9Sprite:create(imgPath .. 'word_bg1.png');
			bg:setFlippedX(true)
			bg:setCapInsets(CCRectMake(15, 20, 30, 8));
		else
			bg = cc.Scale9Sprite:create(imgPath .. 'word_bg2.png');
			bg:setCapInsets(CCRectMake(15, 20, 30, 8));
		end
		bg:setAnchorPoint(bgAp);
		bg:addTo(layout);

		local wordLabel = cc.Label:createWithTTF(tostring(msg.data), fonts.FONT_SIYUAN, 17)
			:setTextColor(cc.c3b(60, 60, 60))
			:setAnchorPoint(wordAp)
			--:setDimensions(360, 0)
			:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
			:addTo(layout)


		local nameLabel = cc.Label:createWithTTF(tostring(msg.name), fonts.FONT_SIYUAN, 15)
			:enableOutline(nameOutlineColor, 2)
			:setTextColor(cc.c3b(255, 255, 255))
			:setAnchorPoint(nameAp)
			:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
			:addTo(layout);

		local labelHeight = wordLabel:getContentSize().height;
		local labelWidth = wordLabel:getContentSize().width;

		if labelWidth > 360 then
			wordLabel:setDimensions(360, 0)
		end

		labelHeight = wordLabel:getContentSize().height;
		labelWidth = wordLabel:getContentSize().width;
		local layoutHeight = wordSpaceH + labelHeight;

		if layoutHeight < layoutBaseHight then
			layoutHeight = layoutBaseHight
		end

		bg:setContentSize(cc.size(labelWidth+50, labelHeight+wordToLabelH))
		if isSelf then
			headIcon:setPosition(layoutBaseWidth-headSpaceW, layoutHeight-headSpaceH)
			headbg:setPosition(layoutBaseWidth-headSpaceW, layoutHeight-headSpaceH)
			bg:setPosition(layoutBaseWidth-bgSpaceW, layoutHeight-bgSpaceH)
			wordLabel:setPosition(layoutBaseWidth-wordSpaceW, layoutHeight-wordSpaceH)
			nameLabel:setPosition(layoutBaseWidth-nameSpaceW, layoutHeight-nameSpaceH)
		else
			headIcon:setPosition(headSpaceW, layoutHeight-headSpaceH)
			headbg:setPosition(headSpaceW, layoutHeight-headSpaceH)
			bg:setPosition(bgSpaceW, layoutHeight-bgSpaceH)
			wordLabel:setPosition(wordSpaceW, layoutHeight-wordSpaceH)
			nameLabel:setPosition(nameSpaceW, layoutHeight-nameSpaceH)
		end

		layout:setContentSize(cc.size(layoutBaseWidth, layoutHeight));
		self.viewList[channel]:pushBackCustomItem(layout);
	end
end

return ChatPanel;
