local DailyRoundPanel = class("DailyRoundPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/dailyRound/'

local fonts = Enum:getTable("SystemFont")
local FightType = Enum:getTable('FightType')

function DailyRoundPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'DailyRoundPanel');

	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;

	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function DailyRoundPanel:Init(root)
	self.root:setVisible(false);

	self:initPanels();
	self:registEvent();
	
	self.chooseIndex = 1;
end

function DailyRoundPanel:initPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_3.png')
		:addTo(self.root)

	self.bg2 = cc.Sprite:create('image/common/common_bg4.png')
		:setPosition(0, 30)
		:addTo(self.root)
	
	self.titleList = {}
	for i=1, 3 do
		local title = {}
		title.btn = ccui.Button:create(imgPath .. 'btn_bg1.png')
		title.btn:setPosition(-207+(i-1)*207, 455)
		title.btn:addTo(self.root)
		local label = cc.Label:createWithTTF(LANG.UI_Daily_Round_TITLE[i], fonts.FONT_SIYUAN, 20)
				:setTextColor(cc.c3b(60, 60, 60))
				:enableOutline(cc.c3b(255, 255, 255), 2)
		title.btn:setTitleLabel(label)
		title.btn:setTag(i)
		title.btn:addClickEventListener(handler(self, self.clickTitle))

		title.sp = cc.Sprite:create(imgPath .. 'btn_bg2.png')
			:setPosition(-207+(i-1)*207, 455)
			:addTo(self.root)

		title.word = cc.Label:createWithTTF(LANG.UI_Daily_Round_TITLE[i], fonts.FONT_SIYUAN, 20)
			:setPosition(-207+(i-1)*207, 455)
			:setTextColor(cc.c3b(255, 255, 255))
			:enableOutline(cc.c3b(60, 60, 60), 2)
			:addTo(self.root)

		title.lock = cc.Sprite:create('image/lock/item_lock2.png')
			:setScale(0.5)
			:setPosition(-207+(i-1)*207-65, 455)
			:addTo(self.root)

		self.titleList[i] = title
	end

	-- times
	self.timesBg = cc.Sprite:create(imgPath .. 'word_bg.png')
		:setPosition(160, 395)
		:addTo(self.root)

	self.timesWord = cc.Label:createWithTTF(LANG.UI_DAILY_ROUND_1, fonts.FONT_SIYUAN, 20)
		:setPosition(120, 395)
		:addTo(self.root)

	self.timesLabel = cc.Label:createWithTTF('2/3', fonts.FONT_SIYUAN, 20)
		:setTextColor(cc.c3b(60, 60, 60))
		:setPosition(240, 395)
		:addTo(self.root)
	
	self.timesList = {}
	for i=1, 3 do
		self.timesList[i] = 0;
	end

	self.viewList = {}
	for i=1, 3 do
		local view = ccui.ListView:create()
		view:setTouchEnabled(true)
		view:setBounceEnabled(true)
		view:setDirection(ccui.ScrollViewDir.vertical)
		view:setContentSize(cc.size(595, 745))
		view:setAnchorPoint(cc.p(0.5, 0.5))
		view:setPosition(0, -15)
		view:setScrollBarEnabled(false)
		view:setItemsMargin(16)
		view:addTo(self.root)
		self.viewList[i] = view;
	end
	
	local data = ResTable:getTable('daily_round')
	local info = {}
	for k, v in pairs(data.contain) do
		if not info[v['type']] then
			info[v['type']] = {}
		end
		table.insert(info[v['type']], {id = v.no, fp = v.fp, name = v.info, quality=v.quality, reward=v.drop})
	end

	self.roundIdList = {}
	self.roundList = {}
	for i=1, 3 do
		self.roundList[i] = {};
		table.sort(info[i], function(first, second)
			return first.id < second.id
		end)
		for k, v in ipairs(info[i]) do
			local roundLayout = ccui.Layout:create();
			roundLayout:setContentSize(cc.size(595, 150));
			roundLayout:setTag(v.id);
			roundLayout:setTouchEnabled(true);
			roundLayout:addClickEventListener(handler(self, self.clickRound));
			self.viewList[i]:pushBackCustomItem(roundLayout);

			local round = template.DailyRoundTemplate:create(roundLayout)
			round:Init(v)
			round:setPosition(595/2, 150/2)

			table.insert(self.roundList[i], {id = v.id, round = round})
			self.roundIdList[v.id] = round
		end
	end
end

function DailyRoundPanel:registEvent()
	EventManager:RegisterEvent(EventType.DailyRoundInfo, handler(self, self.chooseTitle))
	
end

function DailyRoundPanel:isDataInit()
	return model.DailyRoundModel:isInit()
end

function DailyRoundPanel:reqData(callback)
	local msg = {}
	Network.NetworkManager:Send(Network.NetworkCmdType.req_daily_round_info, msg, false, callback);
end

function DailyRoundPanel:onEnter()
	self:onShow()
end

function DailyRoundPanel:onLeave()
	self.root:setVisible(false)
end

function DailyRoundPanel:onShow(msg)
	self.root:setVisible(true)

	local topfp = ConvertDigitUnitToStr(model.UserModel:getUserTopPower())
	for i=1, 3 do
		self:updateLock(i);

		local roundPass = model.DailyRoundModel:getRoundPass(i)
		for k, v in ipairs(self.roundList[i]) do
			v.round:setFp(topfp)
			v.round:setCanPass(v.id <= roundPass)
		end
	end

	local index = self.chooseIndex or 1;
	if self:checkClickLock(index) then
		return ;
	end
	self.chooseIndex = index
	self:chooseTitle()


end

function DailyRoundPanel:chooseTitle()

	local times = model.DailyRoundModel:getTimes(self.chooseIndex)
	for i=1, 3 do
		self.viewList[i]:setVisible(i==self.chooseIndex)
		self.titleList[i].btn:setVisible(i~=self.chooseIndex)
		self.titleList[i].sp:setVisible(i==self.chooseIndex)
	end
	local roundPass = model.DailyRoundModel:getRoundPass(self.chooseIndex)
	local index = 0;
	for k, v in ipairs(self.roundList[self.chooseIndex]) do
		if v.id == roundPass then
			break
		end
		index = index + 1
	end
	self.viewList[self.chooseIndex]:jumpToItem(index, cc.p(0.5, 1), cc.p(0.5, 1));
	self.timesLabel:setString(times .. '/' .. Conf.daily_round_max_times)
end

function DailyRoundPanel:clickTitle(sender, event)
	local tag = sender:getTag()

	if self:checkClickLock(tag) then
		return ;
	end
	self.chooseIndex = tag

	self:chooseTitle()
end

function DailyRoundPanel:checkClickLock( index,notShowPrompt )
	local locked,lockLv = model.DailyRoundModel:getLockStateByTag(index);

	if locked and  notShowPrompt ~= true then
		Prompt:Show(string.format(LANG.UI_DAILY_ROUND_5 , lockLv));
	end

	return locked
end

function DailyRoundPanel:updateLock( tag )

	local locked,lockLv = self:checkClickLock(tag,true);
	if self.titleList[tag] then
		self.titleList[tag].lock:setVisible(locked)
	end

	-- tools.LevelLockTools:setLockToNode(self.homeBtnList[baseType],locked,lockLv,3,cc.p(0,0))

end
function DailyRoundPanel:clickRound(sender, event)
	local tag = sender:getTag()
	if (model.DailyRoundModel:getRoundPass(self.chooseIndex) > 0) then
		if (tag > model.DailyRoundModel:getRoundPass(self.chooseIndex)+1) then
			local name = ResTable:getValue('daily_round', model.DailyRoundModel:getRoundPass(self.chooseIndex)+1, 'info')
			Prompt:Show(string.format(LANG.UI_DAILY_ROUND_3, name));
			return
		end
	else
		if tag % 10 ~= 1 then
			local id = tag - tag % 10 + 1;
			local name = ResTable:getValue('daily_round', id, 'info')
			Prompt:Show(string.format(LANG.UI_DAILY_ROUND_3, name));
			return
		end
	end
	if model.DailyRoundModel:getTimes(self.chooseIndex) <= 0 then
		Prompt:Show(LANG.UI_DAILY_ROUND_4)
		return
	end
	if self.roundIdList[tag] and self.roundIdList[tag]:getCanFight() then
		if self.roundIdList[tag]:getCanPass() then
			if FightManager:enterFight(FightType.DailyRound, {id = tag, auto = true}) then
				model.DailyRoundModel:setCurrentFightInfo(self.chooseIndex, true, tag);
			end
		else
			UIManager:push('RoundInfoPanel', {fightType = FightType.DailyRound, fromPanel = 'DailyRoundPanel', select_key = tag})
			model.DailyRoundModel:setCurrentFightInfo(self.chooseIndex, false, tag);
		end
	end
end

function DailyRoundPanel:onClickBack()
	UIManager:pop('DailyRoundPanel')
	tools.RedDotTools:pushHomeBaseRedDot(2, model.HomeModel.HomeMenuTag.TAG_DAILYROUND)
end

function DailyRoundPanel:guideItemClick(  )
	local node = self.viewList[1]:getItem(0);
	if node then
		self:clickRound(node);
	end
end
-- 引导相关
function DailyRoundPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	if guideNo == 62 or guideNo == 64 then
		local btn = self.viewList[1]:getItem(0);
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	end
	return pos;
end
return DailyRoundPanel;
