local EquipPanel = class("EquipPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local homeImagePath = 'image/equip/'
local TeamType = Enum:getTable('TeamType');

function EquipPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'EquipPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function EquipPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function EquipPanel:initPanels()

	local bgSprite1 = cc.Sprite:create('image/bg/Bg_5.png');
	bgSprite1:addTo(self.root);

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -3.5);
    bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(604 , 903));

	local initData = {
        width = 604,
        height = 858,
        cellWidth = 604 ,
        cellHeight = 120,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.equipListView = template.TableViewTemplate:create()
    self.equipListView:initWithSizeAndItem(initData) ;
    self.equipListView:getView():setPosition(cc.p(-302,-418));
	self.equipListView:getView():addTo(self.root)

	self.swtichBtnList = {};
	for i = 1 , 2 do
		self.swtichBtnList[i] = ccui.Button:create('image/commonImg/btn_5_132_45.png' , 'image/commonImg/btn_5_132_45.png');
		self.swtichBtnList[i]:addTo(self.root);
		self.swtichBtnList[i]:setPosition(i == 1 and 94 or 244 , StageSize.height/2 - 92 - NOTCH_HEIGHT);
		self.swtichBtnList[i]:setTag(i);
		self.swtichBtnList[i]:addClickEventListener(handler(self , self.swtichBtnClick));

		local label = cc.Label:createWithTTF(LANG.UI_EQUIP_SWTICH_BTN[i] , fonts.FONT_SIYUAN , 20);
		label:enableOutline(cc.c4b(125 , 125 , 125, 255) , 2);
		self.swtichBtnList[i]:setTitleLabel(label);
	end

	EventManager:RegisterEvent(EventType.UIEquipComposeUpdateList, handler(self, self.updateList));

end

function EquipPanel:onEnter(teamType)
	self.teamType = teamType or TeamType.COMMON;
	self.root:setVisible(true);
	self:initEquipData();
	self:updateBtnRedDot();
end

function EquipPanel:updateBtnRedDot()
	local state = model.EquipModel:getTeamSlotEquipActiveState();
	tools.RedDotTools:setRedDotToNode(self.swtichBtnList[1],state,nil,cc.p(120 , 33));
end

function EquipPanel:swtichBtnClick(sender , event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	if tag == 1 then
		UIManager:push('EquipWearPanel', self.teamType);
	else
		UIManager:push('EquipComposePanel', self.teamType);
	end
end

function EquipPanel:initEquipData()
	local allWearEquips = model.EquipModel:getAllEquipWear();
	local allEquips = model.EquipModel:getAllEquip();
	self.equips = {};

	for team_slot, data in pairs(allWearEquips) do
		for slot, no in pairs(data) do
			if no > 0 then
				table.insert(self.equips, {
					no 		= no,
					num		= 1,
					isWear	= true,
				});
			end
		end
	end

	for no, num in pairs(allEquips) do
		if num > 0 then
			table.insert(self.equips, {
				no		= no,
				num		= num,
				isWear	= false,
			});
		end
	end
	
	if #self.equips > 1 then
		table.sort(self.equips , function(first , second)

			if first.isWear and not second.isWear then
				return true;
			elseif not first.isWear and second.isWear then
				return false;
			end

			local firstQuailty = ResTable:getValue('item' , first.no , 'quality');
			local secondQuailty = ResTable:getValue('item' , second.no , 'quality');

			if firstQuailty > secondQuailty then
				return true;
			elseif firstQuailty < secondQuailty then
				return false;
			end

			local firstType = ResTable:getValue('equip' , first.no , 'type');
			local secondType = ResTable:getValue('equip' , second.no , 'type');

			if firstType > secondType then
				return true;
			else
				return false;
			end
		end);
	end

	self.equipMaxNum = #self.equips;
	local groupNum = math.ceil(self.equipMaxNum / 5);
	self.equipListView:setDataCount(groupNum);
	self.equipListView:reloadData();
end

function EquipPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 5 + 1;
	local curMaxNum = (idx+1) * 5;
	curMaxNum = math.min(curMaxNum , self.equipMaxNum);

	local children = cell:getChildren();

	if not children or #children <= 0 then
		for i = 1 , 5 do
			local x = 65 + 118.5 * (i - 1);
			local y = 62;
			local equipTemplate = template.ItemTemplate:create();
			equipTemplate:setPosition(cc.p(x , y));
			equipTemplate:addTo(cell);
			equipTemplate:addItemClickEventListener(handler(self , self.equipClick));
			if startNum <= curMaxNum then
				equipTemplate:Init(self.equips[startNum].no, self.equips[startNum].num);
				equipTemplate:setItemTag(self.equips[startNum].no);
				equipTemplate:setEquipInSlot(self.equips[startNum].isWear);
				startNum = startNum + 1;
			else
				equipTemplate:setVisible(false);
			end
		end
	else
		for i = 1 , 5 do
			if startNum <= curMaxNum then
				children[i]:setVisible(true);
				children[i]:Init(self.equips[startNum].no, self.equips[startNum].num);
				children[i]:setItemTag(self.equips[startNum].no);
				children[i]:setEquipInSlot(self.equips[startNum].isWear);

				startNum = startNum + 1;
			else
				children[i]:setVisible(false);
			end
		end
	end

	return cell;
end

function EquipPanel:cellItemTouch(table)
	AudioManager:playEffect(Conf.confirmButton);
	if self.curSelectNo then
		local data = {
			no = self.curSelectNo, 
			enterType = 3,
		}
		UIManager:push('TipsEquipPanel' , data);
	end
end

function EquipPanel:equipClick(sender , event)
	self.curSelectNo = sender:getTag();
end

function EquipPanel:footerButtonClick(sender , event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.confirmButton);
	if tag == 1 then
		UIManager:push('EquipWearPanel',self.teamType);
	else
		UIManager:push('EquipComposePanel',self.teamType);
	end
end

function EquipPanel:updateList()
	self:initEquipData();
	self:showEquipListPanel();
end

function EquipPanel:onClickBack()
	EventManager:FireEvent(EventType.UIHeaderPanelUpdateFP); -- 战斗力刷新
	UIManager:pop('EquipPanel');
end

function EquipPanel:onLeave()
	self.root:setVisible(false);
end

return EquipPanel;
