local EquipWearPanel = class("EquipWearPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local equipImagePath = 'image/equip/equip_wear/'
local Attribute = Enum:getTable('Attribute');
local MessageBoxType = Enum:getTable('MessageBoxType')
local TeamType = Enum:getTable('TeamType')

local rolePos = {
	[1] = {x = 172, y = 213, zorder = 80},
	[2] = {x = 0, y = 302, zorder = 80},
	[3] = {x = 0, y = 141, zorder = 100},
	[4] = {x = -159, y = 212, zorder = 50},
}
local equipSlotPos = {
	[1] = cc.p(-231 , -249),
	[2] = cc.p(231 , -249),
	[3] = cc.p(-231 , -411),
	[4] = cc.p(231 , -411),
}

local equipSlotActionPos = {
	[1] = {cc.p(80 , -14) , cc.p(140 , -38)},
	[2] = {cc.p(-80 , -14) , cc.p(-140 , -38)},
	[3] = {cc.p(80 , 11) , cc.p(140 , 33)},
	[4] = {cc.p(-80 , 11) , cc.p(-140 , 33)},
}

local teamSlotActivePos = {
	[1] = cc.p(-1 , -2),
	[2] = cc.p(0 , -3),
	[3] = cc.p(0 , -4),
	[4] = cc.p(0 , -3),
}

local outlineColor = {
	[1] = cc.c3b(30,130,35),
	[2] = cc.c3b(210,80,10),
}

function EquipWearPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'EquipWearPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
	self.roleRedDot = {};
end

function EquipWearPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function EquipWearPanel:initPanels()

	local bgSprite1 = cc.Sprite:create('image/bg/Bg_5.png');
	bgSprite1:addTo(self.root);

	local bg = cc.Sprite:create('image/commonImg/bg_5_602_496.png');
	bg:setPosition(cc.p(0,207));
	bg:addTo(self.root);
	
	local height = StageSize.height/2-71
	height = height < 502 and 502 or height;
    local bg = cc.Scale9Sprite:create(equipImagePath..'equip_wear_2.png')	
	bg:setPosition(0 , -71)
	bg:addTo(self.root)
	bg:setAnchorPoint(0.5 , 1)
	bg:setCapInsets(cc.rect(0,360,640,462))
	bg:setContentSize(cc.size(640 , height))

	self.showRoleList = {};
	for i=TEAM_MAX_MEMBER, 1, -1 do
		self.showRoleList[i] = {};

		local node = cc.Node:create();
		node:addTo(self.root);
		node:setPosition(rolePos[i].x, rolePos[i].y);
		node:setLocalZOrder(rolePos[i].zorder);
		self.showRoleList[i].node = node;

		local template = require('app.template.TeamRoleTemplate'):create(node);
		template:Init(i);
		template:setClickEvent(handler(self, self.selectPos));
		self.showRoleList[i].role = template;
	end

	self.infoNode = cc.Node:create();
	self.infoNode:setPosition(0, 0);
	self.infoNode:addTo(self.root);

	self.infoBg = cc.Sprite:create(equipImagePath..'equip_wear_3.png')
		:setPosition(0, 20)
		:addTo(self.infoNode)

	self.fpBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(-272, 44.5)
		:addTo(self.infoNode)

	self.hpBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(-272 , -5)
		:addTo(self.infoNode)

	self.atkBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(22, -5)
		:addTo(self.infoNode)

	self.fpWord = cc.Label:createWithTTF(LANG.UI_TEAM_21, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setAnchorPoint(0,0.5)
		:setPosition(-254, 45.5)
		:addTo(self.infoNode)

	self.fpLabel = cc.Label:createWithTTF('123.23MK', fonts.FONT_IMPACT, 20)
		:setPosition(276, 44.5)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)

	self.atkWord = cc.Label:createWithTTF(LANG.UI_TEAM_13, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setPosition(-231, -5)
		:addTo(self.infoNode)

	self.atkLabel = cc.Label:createWithTTF('233.43AJ', fonts.FONT_IMPACT, 20)
		:setPosition(-18, -5)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)

	self.hpWord = cc.Label:createWithTTF(LANG.UI_TEAM_14, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setPosition(62, -5)
		:addTo(self.infoNode)

	self.hpLabel = cc.Label:createWithTTF('123.45K', fonts.FONT_IMPACT, 20)
		:setPosition(276, -5)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)

	self.equipSlot = {};
	for i = 1 , 4 do
		self.equipSlot[i] = {};
		self.equipSlot[i].node = cc.Node:create();
		self.equipSlot[i].node:setPosition((i == 1 or i == 4) and -212 or 209 , (i == 1 or i == 2) and -155 or -314);
		self.equipSlot[i].node:addTo(self.root);

		self.equipSlot[i].btn = ccui.Button:create(equipImagePath..'equip_wear_1.png' , equipImagePath..'equip_wear_1.png');
		self.equipSlot[i].btn:addTo(self.equipSlot[i].node);
		self.equipSlot[i].btn:setTag(i);
		self.equipSlot[i].btn:addClickEventListener(handler(self , self.equipSlotClick));

		local emptyImg = cc.Sprite:create(equipImagePath..'equip_wear_5.png');
		emptyImg:addTo(self.equipSlot[i].node);

		self.equipSlot[i].equip = template.ItemTemplate:create();
		self.equipSlot[i].equip:addTo(self.equipSlot[i].node);
	end

	self.bareLoveList = {};
	for i = 1 , 3 do
		self.bareLoveList[i] = {};
		local node = cc.Node:create()
			:addTo(self.root)
			:setPosition( (i-1)*90-90 , -250)

		self.bareLoveList[i].node = node;

		local bareLoveBtn = ccui.Button:create(equipImagePath..'equip_wear_1.png' , equipImagePath..'equip_wear_1.png');
		bareLoveBtn:addTo(node);
		bareLoveBtn:setScale(0.5);
		bareLoveBtn:setTag(i);
		bareLoveBtn:addClickEventListener(handler(self , self.bareLoveBtnClick));
	
		local bareCollectEmpty = cc.Sprite:create(equipImagePath..'equip_wear_5.png')
			:addTo(bareLoveBtn)
			:setScale(0.55)
			:setPosition(bareLoveBtn:getContentSize().width/2, bareLoveBtn:getContentSize().height/2)
		
		self.bareLoveList[i].bareLoveIconBg = cc.Sprite:create('image/commonImg/item_quality_2.png')
			:addTo(node)
			:setScale(0.55)

		self.bareLoveList[i].bareLoveIcon = cc.Sprite:create()
			:addTo(node)
			:setScale(0.55)

		local bg = cc.Sprite:create(equipImagePath..'equip_wear_9.png')
			:addTo(node)
			:setPosition(26,-27)
		
		local iconPath = equipImagePath..'equip_wear_icon_'..i..'.png';
		if i==3 then
			iconPath = equipImagePath..'equip_wear_icon_2.png';
		end
		local bg = cc.Sprite:create(iconPath)
			:addTo(node)
			:setPosition(26,-27)
		
		local lockNode = cc.Node:create()
			:addTo(node)
		self.bareLoveList[i].lockNode = lockNode;

		local bg = cc.Sprite:create(equipImagePath..'equip_wear_8.png')
			:addTo(lockNode)
			:setPosition(4,-3)
		local bg = cc.Sprite:create('image/lock/lock_bg_2.png')
			:addTo(lockNode)

		local bareLoveAttNode = cc.Node:create()
			:addTo(self.root)
			:setPosition(0, -305-(i-1)*33)
			:setCascadeOpacityEnabled(true)
		self.bareLoveList[i].bareLoveAttNode = bareLoveAttNode;
		
		local bg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png')
			:addTo(bareLoveAttNode)
			:setCapInsets(cc.rect(11,11,34,28))
			:setContentSize(cc.size(266 , 31))

		--local bg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		--	:addTo(bareLoveAttNode)
		--	:setPosition(-112,0)
		
		local label = cc.Label:createWithTTF(LANG.UI_EQUIP_WEAR_NAME[i],fonts.FONT_SIYUAN,15)
			:addTo(bareLoveAttNode)
			:setPosition(-125,0)
			:setAnchorPoint(0,0.5)
			:setTextColor(cc.c3b(185,200,255))
		
		local attNode = cc.Node:create()
			:addTo(bareLoveAttNode)
		self.bareLoveList[i].attNode = attNode;

		local attName1 = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,15)
			:addTo(attNode)
			:setPosition(-33,0)
		self.bareLoveList[i].attName1 = attName1;

		local att1 = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,15)
			:addTo(attNode)
			:setPosition(-12,0)
			:setAnchorPoint(0,0.5)
		self.bareLoveList[i].att1 = att1;

		local attName2 = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,15)
			:addTo(attNode)
			:setPosition(56,0)
		self.bareLoveList[i].attName2 = attName2;

		local att2 = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,15)
			:addTo(attNode)
			:setPosition(75,0)
			:setAnchorPoint(0,0.5)
		self.bareLoveList[i].att2 = att2;

		local infoIcon = cc.Sprite:create('image/commonImg/def_type_1.png')
			:addTo(attNode)
			:setPosition(100, 0)
			:setScale(0.5)
			:setVisible(false)
		self.bareLoveList[i].infoIcon = infoIcon;

		local attTip = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,15)
			:addTo(bareLoveAttNode)
			:setPosition(120,0)
			:setAnchorPoint(1,0.5)
		self.bareLoveList[i].attTip = attTip;
	end

	self.takeOffAllBtn = ccui.Button:create('image/commonImg/btn_8_219_55.png' , 'image/commonImg/btn_8_219_55.png');
	self.takeOffAllBtn:addTo(self.root);
	self.takeOffAllBtn:setPosition(-138 , -426);
	self.takeOffAllBtn:addClickEventListener(handler(self , self.takeOffAllBtnClick));

	local btnLabel = cc.Label:createWithTTF(LANG.UI_EQUIP_32 , fonts.FONT_SIYUAN , 20);
	btnLabel:enableOutline(cc.c4b(140 , 50 , 50 , 255) , 2);
	self.takeOffAllBtn:setTitleLabel(btnLabel);

	self.oneKeyWearBtn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	self.oneKeyWearBtn:addTo(self.root);
	self.oneKeyWearBtn:setPosition(133 , -426);
	self.oneKeyWearBtn:addClickEventListener(handler(self , self.oneKeyWearBtnClick));

	local btnLabel = cc.Label:createWithTTF(LANG.UI_EQUIP_33 , fonts.FONT_SIYUAN , 20);
	btnLabel:enableOutline(cc.c4b(50 , 85 , 150 , 255) , 2);
	self.oneKeyWearBtn:setTitleLabel(btnLabel);

	local bg = cc.Sprite:create(equipImagePath..'equip_wear_4.png');
	bg:addTo(self.root);
	bg:setPosition(-3 , -146);

	self.roleIcon = template.ItemTemplate:create();
	self.roleIcon:addTo(self.root);
	self.roleIcon:setPosition(-3 , -146);

	self.GNLabel = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 25);
	self.GNLabel:addTo(self.root);
	self.GNLabel:setPosition(-3 , -194);
	self.GNLabel:enableOutline(cc.c3b(60 , 60 , 60) , 2);

	self.swtichBtnList = {};
	for i = 1 , 2 do
		self.swtichBtnList[i] = ccui.Button:create('image/commonImg/btn_5_132_45.png' , 'image/commonImg/btn_5_132_45.png');
		self.swtichBtnList[i]:addTo(self.root);
		self.swtichBtnList[i]:setPosition(i == 1 and 94 or 244 , StageSize.height/2 - 92 - NOTCH_HEIGHT);
		self.swtichBtnList[i]:setTag(i);
		self.swtichBtnList[i]:addClickEventListener(handler(self , self.swtichBtnClick));

		local label = cc.Label:createWithTTF(LANG.UI_EQUIP_WEAR_SWTICH_BTN[i] , fonts.FONT_SIYUAN , 20);
		label:enableOutline(cc.c4b(125 , 125 , 125, 255) , 2);
		self.swtichBtnList[i]:setTitleLabel(label);
	end

	EventManager:RegisterEvent(EventType.NetworkRetEquipWear , handler(self, self.retEquipWear));
	EventManager:RegisterEvent(EventType.NetworkRetEquipOneKey , handler(self, self.retEquipOneKey));
	EventManager:RegisterEvent(EventType.NetworkRetEquipTakeOff , handler(self, self.retEquipTakeOff));
	EventManager:RegisterEvent(EventType.NetworkRetEquipTakeOffAll , handler(self, self.retEquipTakeOffAll));
	EventManager:RegisterEvent(EventType.NetworkRetUpdateTeamSlot , handler(self, self.updateTeamSlot));
	EventManager:RegisterEvent(EventType.NetworkRetBareCollectChange,handler(self,self.retBareCollectChange));
	EventManager:RegisterEvent(EventType.NetworkRetLoveChange , handler(self, self.retLoveChange));
end

function EquipWearPanel:swtichBtnClick(sender , event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	if tag == 1 then
		UIManager:push('EquipComposePanel', self.teamType);
	else
		UIManager:push('EquipPanel', self.teamType);
	end
end


function EquipWearPanel:onEnter(teamType)
	self.teamType = teamType or TeamType.COMMON;
	if self.teamType == TeamType.EVENT then
		model.EventModel:addRobotToCardModel()
	end
	
	self.roleRedDot = {};
	for i = 1 , TEAM_MAX_MEMBER do
		self.roleRedDot[i] = {
			equip = false,
			bare  = false,
			love  = false,
		}
	end
	self.curTeamFp = 0;
	self.curTeamSlot = 1;
	self:onShow();
end

function EquipWearPanel:onResume()
	if self.teamType == TeamType.EVENT then
		model.EventModel:addRobotToCardModel()
	end
	self:onShow();
end

function EquipWearPanel:onShow()
	self.root:setVisible(true);
	self.swtichBtnList[1]:setVisible(true);
	self.swtichBtnList[2]:setVisible(true);
	if self.teamType == TeamType.EVENT then
		self.swtichBtnList[1]:setVisible(false);
		self.swtichBtnList[2]:setVisible(false);
	end
	local tid = model.TeamModel:getTeamIdByTeamType( self.teamType )
	self.team = model.TeamModel:getTeamById(tid);

	self.teamSlotData = {};

	self:initSlotRole(true);
	self:updateTeamSlot();
	self:updateAvtiveState(0);

end

function EquipWearPanel:initSlotRole(initRole)
	self.teamSlotAttData = {};
	for i=1, TEAM_MAX_MEMBER do
		if initRole then
			self.showRoleList[i].role:setInfo(self.team[i]);
		end
		self.teamSlotAttData[i] = self:getTeamSlotAttData(i);
	end
end

function EquipWearPanel:updateTeamSlot()

	self.curTeamFp = model.TeamModel:getCurrentTeamFp(model.TeamModel:getTeamIdByTeamType(self.teamType));

	for i = 1 , TEAM_MAX_MEMBER do
		self.showRoleList[i].role:setSelected(self.curTeamSlot == i);
	end

	local equipSlot = model.EquipModel:getEquipSlotByIndex(self.curTeamSlot);
	
	for i = 1 , 4 do
		if equipSlot[i] ~= 0 then
			self.equipSlot[i].equip:setVisible(true);

			self.equipSlot[i].equip:Init(equipSlot[i]);
		else
			self.equipSlot[i].equip:setVisible(false);
		end
	end

	local slotAttData = self.teamSlotAttData[self.curTeamSlot];

	self.fpWord:setString(string.format(LANG.UI_TEAM_30 , self.curTeamSlot));

	self.fpLabel:setString(tostring(slotAttData.fp or 0));
	self.atkLabel:setString(tostring(slotAttData.atk or 0));
	self.hpLabel:setString(tostring(slotAttData.hp or 0));
	--self.defLabel:setString(tostring(slotAttData.def or 0));

	self.GNLabel:setString('GN-0'..tostring(self.curTeamSlot));

	if self.team[self.curTeamSlot] ~= -1 then
		local pid = self.team[self.curTeamSlot];
		local data = model.CardModel:GetCard(pid);
		self.roleIcon:Init(ROLE_TYPE , data.no , data.star_lv, 0, pid);
		self.roleIcon:setVisible(true);
	else
		self.roleIcon:setVisible(false);
	end

	local qualityEquips = model.EquipModel:getActiveState(self.curTeamSlot);
	local isHave = false
	for slot, haveBetter in pairs(qualityEquips) do
		if haveBetter then
			if not isHave then
				isHave = true;
			end
		end
		tools.RedDotTools:setRedDotToNode(self.equipSlot[slot].node,haveBetter,nil,cc.p(66 , 68));
	end
	tools.RedDotTools:setRedDotToNode(self.oneKeyWearBtn,isHave,nil,cc.p(200 , 40));
	
	self:updateBareCollect();
	self:updateBareCollect2();
	self:updateLove()
end

function EquipWearPanel:updateBareCollect2()

	local isWearBare = false
	local isHaveAllActiveBare = false;
	local isHaveCanWear = false;
	local pid = self.team[self.curTeamSlot];
	local data = nil;
	if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
		isHaveAllActiveBare,isHaveCanWear = model.BareCollectModel:isHaveAllActiveBare();
		data = model.CardModel:GetCard(pid);
		if data and data.bare_id_2 and data.bare_id_2 > 0 then
			local bareData = model.BareCollectModel:getBareData(data.bare_id_2);
			if bareData then
				isWearBare = true;
				local attData = nil;
				if bareData.attribute_effect[2]>0 or bareData.attribute_effect[3] > 0 then
					attData = {bareData.attribute_effect};
					self.bareLoveList[3].infoIcon:setVisible(false)
				else
					attData = {bareData.attribute_effect2};
					self.bareLoveList[3].infoIcon:setTexture('image/commonImg/def_type_'..bareData.attribute_color..'.png')
					self.bareLoveList[3].infoIcon:setVisible(true)
				end

				for i = 1 , 2 do
					local att = attData[i];
					if att then
						self.bareLoveList[3]['attName'..i]:setVisible(true);
						self.bareLoveList[3]['att'..i]:setVisible(true);
						self.bareLoveList[3]['attName'..i]:setString(tostring(LANG.UI_CARD_ATT_NAME[att[1]]));
						self.bareLoveList[3]['attName'..i]:enableOutline(outlineColor[att[1]],2);
						if att[2] > 0 then
							self.bareLoveList[3]['att'..i]:setString('+'..tostring(att[2]));
						elseif att[3] > 0 then
							self.bareLoveList[3]['att'..i]:setString('+'..tostring(att[3] * 100)..'%');
						end
					else
						self.bareLoveList[3]['attName'..i]:setVisible(false);
						self.bareLoveList[3]['att'..i]:setVisible(false);
					end
				end
				local bareIcon = ResTable:getValue('role_resource',bareData.resource_id,'icon');
				self.bareLoveList[3].bareLoveIcon:setTexture('image/icon/'..bareIcon..'.png');
			end
		end
	end

	if isHaveAllActiveBare then
		self.bareLoveList[3].bareLoveAttNode:setOpacity(255);
		self.bareLoveList[3].attTip:setString(LANG.UI_EQUIP_55);
	else
		self.bareLoveList[3].bareLoveAttNode:setOpacity(127.5);
		self.bareLoveList[3].attTip:setString(LANG.UI_EQUIP_54);
	end

	self.bareLoveList[3].bareLoveIconBg:setVisible(isWearBare);
	self.bareLoveList[3].bareLoveIcon:setVisible(isWearBare);
	self.bareLoveList[3].attNode:setVisible(isWearBare);
	self.bareLoveList[3].attTip:setVisible(not isWearBare);
	self.bareLoveList[3].lockNode:setVisible(not isHaveAllActiveBare);
	if data and data.star_lv>=10 then
		local state = isHaveCanWear and not isWearBare;
		tools.RedDotTools:setRedDotToNode(self.bareLoveList[3].node,state,nil,cc.p(28 , 28));
	else
		self.bareLoveList[3].lockNode:setVisible(true);
		tools.RedDotTools:setRedDotToNode(self.bareLoveList[3].node,false,nil,cc.p(28 , 28));
	end
	
end

function EquipWearPanel:updateBareCollect()

	local isWearBare = false
	local isHaveAllActiveBare = false;
	local isHaveCanWear = false;
	local pid = self.team[self.curTeamSlot];
	if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
		isHaveAllActiveBare,isHaveCanWear = model.BareCollectModel:isHaveAllActiveBare();
		local data = model.CardModel:GetCard(pid);
		if data and data.bare_id and data.bare_id > 0 then
			local bareData = model.BareCollectModel:getBareData(data.bare_id);
			if bareData then
				isWearBare = true;
				local attData = nil;
				if bareData.attribute_effect[2]>0 or bareData.attribute_effect[3] > 0 then
					attData = {bareData.attribute_effect};
					self.bareLoveList[2].infoIcon:setVisible(false)
				else
					attData = {bareData.attribute_effect2};
					self.bareLoveList[2].infoIcon:setTexture('image/commonImg/def_type_'..bareData.attribute_color..'.png')
					self.bareLoveList[2].infoIcon:setVisible(true)
				end

				for i = 1 , 2 do
					local att = attData[i];
					if att then
						self.bareLoveList[2]['attName'..i]:setVisible(true);
						self.bareLoveList[2]['att'..i]:setVisible(true);
						self.bareLoveList[2]['attName'..i]:setString(tostring(LANG.UI_CARD_ATT_NAME[att[1]]));
						self.bareLoveList[2]['attName'..i]:enableOutline(outlineColor[att[1]],2);
						if att[2] > 0 then
							self.bareLoveList[2]['att'..i]:setString('+'..tostring(att[2]));
						elseif att[3] > 0 then
							self.bareLoveList[2]['att'..i]:setString('+'..tostring(att[3] * 100)..'%');
						end
					else
						self.bareLoveList[2]['attName'..i]:setVisible(false);
						self.bareLoveList[2]['att'..i]:setVisible(false);
					end
				end
				local bareIcon = ResTable:getValue('role_resource',bareData.resource_id,'icon');
				self.bareLoveList[2].bareLoveIcon:setTexture('image/icon/'..bareIcon..'.png');
			end
		end
	end

	if isHaveAllActiveBare then
		self.bareLoveList[2].bareLoveAttNode:setOpacity(255);
		self.bareLoveList[2].attTip:setString(LANG.UI_EQUIP_55);
	else
		self.bareLoveList[2].bareLoveAttNode:setOpacity(127.5);
		self.bareLoveList[2].attTip:setString(LANG.UI_EQUIP_54);
	end

	self.bareLoveList[2].bareLoveIconBg:setVisible(isWearBare);
	self.bareLoveList[2].bareLoveIcon:setVisible(isWearBare);
	self.bareLoveList[2].attNode:setVisible(isWearBare);
	self.bareLoveList[2].attTip:setVisible(not isWearBare);
	self.bareLoveList[2].lockNode:setVisible(not isHaveAllActiveBare);
	
	local state = isHaveCanWear and not isWearBare;
	tools.RedDotTools:setRedDotToNode(self.bareLoveList[2].node,state,nil,cc.p(28 , 28));
end

function EquipWearPanel:updateLove()

	local isWearLove = false
	local isHaveAllActiveLove = false;
	local pid = self.team[self.curTeamSlot];
	if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
		local data = model.CardModel:GetCard(pid);
		if data then
			if data.love_id and data.love_id > 0 then
				local loveData = model.CardModel:getLoveData(data.love_id);
				if loveData then
					isWearLove = true;
					local attData = loveData.loveData[5].effect;
					for i = 1 , 2 do
						local att = attData[i];
						if att then
							self.bareLoveList[1]['attName'..i]:setVisible(true);
							self.bareLoveList[1]['att'..i]:setVisible(true);
							self.bareLoveList[1]['attName'..i]:setString(tostring(LANG.UI_CARD_ATT_NAME[att[1]]));
							self.bareLoveList[1]['attName'..i]:enableOutline(outlineColor[att[1]],2);
							if att[2] > 0 then
								self.bareLoveList[1]['att'..i]:setString('+'..tostring(att[2]));
							elseif att[3] > 0 then
								self.bareLoveList[1]['att'..i]:setString('+'..tostring(att[3] * 100)..'%');
							end
						else
							self.bareLoveList[1]['attName'..i]:setVisible(false);
							self.bareLoveList[1]['att'..i]:setVisible(false);
						end
					end
					self.bareLoveList[1].bareLoveIcon:setTexture('image/icon/'..loveData.loveData[5].show_icon..'.png');
				end
			end

			if not isWearLove then
				local loveDatas = model.CardModel:getLoveDataByRoleId(data.no);
				if loveDatas then
					for _k , data in pairs(loveDatas) do
						if data.loveLevel >= 5 then
							isHaveAllActiveLove = true;
							break;
						end
					end
				end
			end
		end
	end

	self.bareLoveList[1].bareLoveIconBg:setVisible(isWearLove);
	self.bareLoveList[1].bareLoveIcon:setVisible(isWearLove);
	self.bareLoveList[1].attNode:setVisible(isWearLove);
	self.bareLoveList[1].attTip:setVisible(not isWearLove);

	if isWearLove then
		self.bareLoveList[1].bareLoveAttNode:setOpacity(255);
		self.bareLoveList[1].lockNode:setVisible(false);
	elseif isHaveAllActiveLove then
		self.bareLoveList[1].bareLoveAttNode:setOpacity(255);
		self.bareLoveList[1].attTip:setString(LANG.UI_EQUIP_55);
		self.bareLoveList[1].lockNode:setVisible(false);
	else
		self.bareLoveList[1].bareLoveAttNode:setOpacity(127.5);
		self.bareLoveList[1].attTip:setString(LANG.UI_EQUIP_53);
		self.bareLoveList[1].lockNode:setVisible(true);
	end

	local state = isHaveAllActiveLove and not isWearLove;
	tools.RedDotTools:setRedDotToNode(self.bareLoveList[1].node,state,nil,cc.p(28 , 28));
end

function EquipWearPanel:updateEquipActiveState(teamSlot)
	local starIndex = teamSlot or 1;
	local endIndex = teamSlot or TEAM_MAX_MEMBER;
	for i = starIndex , endIndex do
		local qualityEquips = model.EquipModel:getActiveState(i);
		local isHave = false
		for slot, haveBetter in pairs(qualityEquips) do
			if haveBetter then
				if not isHave then
					isHave = true;
				end
			end
			if teamSlot or self.curTeamSlot == i then
				tools.RedDotTools:setRedDotToNode(self.equipSlot[slot].node,haveBetter,nil,cc.p(66 , 68));
			end
		end
		if teamSlot or self.curTeamSlot == i then
			tools.RedDotTools:setRedDotToNode(self.oneKeyWearBtn,isHave,nil,cc.p(200 , 40));
		end
		self.roleRedDot[i].role = isHave;
		tools.RedDotTools:setRedDotToNode(self.showRoleList[i].node,isHave,nil,cc.p(70 , 8));
	end
end

function EquipWearPanel:updateAvtiveState(stateType)
	for i = 1 , TEAM_MAX_MEMBER do
		if stateType == 1 or stateType == 0 then
			self.roleRedDot[i].equip = false;
			local qualityEquips = model.EquipModel:getActiveState(i);
			for slot, haveBetter in pairs(qualityEquips) do
				if haveBetter then
					self.roleRedDot[i].equip = true;
					break;
				end
			end
		end
		local pid = self.team[i];
		if stateType == 2 or stateType == 0 then --大破
			self.roleRedDot[i].bare = false;
			if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
				local data = model.CardModel:GetCard(pid);
				if data and ((data.bare_id and data.bare_id <= 0) or (data.bare_id_2 and data.bare_id_2 <= 0 and data.star_lv>=10)) then
					local isHaveAllActiveBare,isHaveCanWear = model.BareCollectModel:isHaveAllActiveBare()
					self.roleRedDot[i].bare = isHaveAllActiveBare and isHaveCanWear;
				end
			end
		end
		if stateType == 3 or stateType == 0 then --爱恋
			self.roleRedDot[i].love = false;
			if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
				local data = model.CardModel:GetCard(pid);
				if data and data.love_id and data.love_id <= 0 then
					local loveDatas = model.CardModel:getLoveDataByRoleId(data.no);
					if loveDatas then
						for _k , data in pairs(loveDatas) do
							if data.loveLevel >= 5 then
								self.roleRedDot[i].love = true;
								break;
							end
						end
					end
				end
			end
		end

		local state = self.roleRedDot[i].equip or self.roleRedDot[i].bare or self.roleRedDot[i].love 
		tools.RedDotTools:setRedDotToNode(self.showRoleList[i].node,state,nil,cc.p(70 , 8));
	end
end

function EquipWearPanel:getTeamSlotAttData(teamSlot)
	local slotAttData = {};
	slotAttData.active = 0;
	slotAttData.activeNumber = 0;
	slotAttData.activeType = 0;

	--local equipEffectNum = {};
	local equipSlot = model.EquipModel:getEquipSlotByIndex(teamSlot);
	for i = 1 , 4 do
		if equipSlot[i] ~= 0 then
			local equipData = ResTable:getRowData('equip' , equipSlot[i]);
			if equipData then
				for i = 1 , 3 do
					local att = GetMathsValue(string.format(tostring(equipData['value'..i]) , 1));
					if math.ceil(att) > 0 then
						if equipData['attribute'..i] == 1 or equipData['attribute'..i] == 2 then
							if not slotAttData[Attribute[equipData['attribute'..i]]] then
								slotAttData[Attribute[equipData['attribute'..i]]] = '0';
							end
							slotAttData[Attribute[equipData['attribute'..i]]] = HpPlus(ConvertDigitUnitToStr(slotAttData[Attribute[equipData['attribute'..i]]]) , ConvertDigitUnitToStr(tostring(att)));
						elseif equipData['attribute'..i] > 2 and equipData['attribute'..i] <= 6 then
							if not slotAttData[Attribute[equipData['attribute'..i]]] then
								slotAttData[Attribute[equipData['attribute'..i]]] = 0;
							end
							slotAttData[Attribute[equipData['attribute'..i]]] = slotAttData[Attribute[equipData['attribute'..i]]] + tonumber(att);
						end
					end
				end
			end
		end
	end

	if self.team[teamSlot] ~= -1 then
		local pro = CardProManager:GenProInTeam(self.team[teamSlot], nil, nil, self.teamType);
		if pro then
			slotAttData = pro;
			slotAttData.active = 0;
			slotAttData.activeNumber = 0;
			slotAttData.activeType = 0;
		end
	end

	return slotAttData;
end

function EquipWearPanel:selectPos(slot)
	AudioManager:playEffect(Conf.confirmButton);
	if slot == self.curTeamSlot then
		return;
	end
	self.curTeamSlot = slot;
	self:updateTeamSlot();
	--self:updateEquipActiveState(slot);
end

function EquipWearPanel:equipSlotClick(sender , event)
	AudioManager:playEffect(Conf.confirmButton);
	local tag = sender:getTag();
	local equipSlot = model.EquipModel:getEquipSlotByIndex(self.curTeamSlot);
	if not equipSlot or equipSlot[tag] == 0 then
		UIManager:push('EquipWearSelectPanel' , self.curTeamSlot , tag);
	else
		local data = {
			no = equipSlot[tag], 
			teamSlot = self.curTeamSlot,
			slot = tag,
			enterType = 2,
		}
		UIManager:push('TipsEquipPanel' , data);
	end
end

function EquipWearPanel:takeOffAllBtnClick()
	AudioManager:playEffect(Conf.cancleButton);
	local msg = {};
	msg.teamSlot = self.curTeamSlot;
	--NetworkBeforeValue.OldFp = model.TeamModel:getCurrentTeamFp();
	Network.NetworkManager:Send(Network.NetworkCmdType.req_equip_takeoff_all , msg);
end

function EquipWearPanel:oneKeyWearBtnClick()
	AudioManager:playEffect(Conf.buttonClick);
	local equipWear = model.EquipModel:getEquipSlotByIndex(self.curTeamSlot);
	local equipWillWear = {};
	local equips = {}
	for no, num in pairs(model.EquipModel:getAllEquip()) do
		if num > 0 then
			local t = ResTable:getValue('equip', no, 'type');
			if not equips[t] then
				equips[t] = {};
			end
			equips[t][no] = num;
		end
	end
	for i=1, 4 do
		local no = equipWear[i];
		local qu = 0;
		if no ~= 0 then
			qu = ResTable:getValue('item', no, 'quality');
		end
		if equips[i] then
			for noComp, num in pairs(equips[i]) do
				if num > 0 and noComp ~= no then
					local quComp = ResTable:getValue('item', noComp, 'quality');
					if quComp > qu then
						no = noComp;
						qu = quComp;
					end
				end
			end
			if no > 0 and equips[i][no] and no ~= equipWear[i] then
				equips[i][no] = equips[i][no] - 1;
			end
		end
		equipWillWear[i] = no
	end

	local isChange = false;
	for i=1, 4 do
		if equipWear[i] ~= equipWillWear[i] then
			isChange = true;
			break;
		end
	end

	if isChange then
		local msg = {};
		msg.teamSlot = self.curTeamSlot;
		msg.equipList = {};
		for i=1, 4 do
			table.insert(msg.equipList,{no = equipWillWear[i], slot = i})
		end
		Network.NetworkManager:Send(Network.NetworkCmdType.req_equip_one_key , msg);
	else
		Prompt:Show(LANG.UI_EQUIP_34);
	end
end

function EquipWearPanel:retEquipWear(msg)
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(1);
end
function EquipWearPanel:retEquipOneKey()
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(1);
end
function EquipWearPanel:retEquipTakeOff(msg)
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(1);
end
function EquipWearPanel:retEquipTakeOffAll()
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(1);
end

function EquipWearPanel:retBareCollectChange()
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(2);
end

function EquipWearPanel:retLoveChange()
	model.TeamModel:checkTeamFpUp(self.curTeamFp,model.TeamModel:getTeamIdByTeamType(self.teamType))
	self:initSlotRole()
	self:updateTeamSlot();
	self:updateAvtiveState(3);
end

function EquipWearPanel:bareLoveBtnClick(sender , event)

	local tag = sender:getTag();
	local pid = self.team[self.curTeamSlot];
	if pid == -1 then
		return;
	end

	local data = model.CardModel:GetCard(pid);
	if not data then
		return
	end
	if tag == 3 and data.star_lv < 10 then
		Prompt:Show(LANG.UI_EQUIP_62);
		return
	end

	if model.CardModel:isEventRobotCard(pid) then
		Prompt:Show(LANG.UI_MAZE_51);
		return
	end

	if tag == 1 then
		local loveState = 1;
		local loveDatas = model.CardModel:getLoveDataByRoleId(data.no);
		if loveDatas then
			if #loveDatas > 0 then
				loveState = 2;
				for _k , data in pairs(loveDatas) do
					if data.loveLevel >= 5 then
						loveState = 0;
						break;
					end
				end
			end
		end

		if loveState == 1 then
			Prompt:Show(LANG.UI_EQUIP_57);
			return
		elseif loveState == 2 then
			Prompt:Show(LANG.UI_EQUIP_58);
			return
		end
	else
		local isHaveAllActiveBare = model.BareCollectModel:isHaveAllActiveBare();
		if not isHaveAllActiveBare then
			Prompt:Show(LANG.UI_EQUIP_56);
			return;
		end
	end
	
	UIManager:push('EquipWearBareLoveSelectPanel' , pid,tag,self.teamType);

end

function EquipWearPanel:onClickBack()
	UIManager:pop('EquipWearPanel');
end

function EquipWearPanel:onLeave()
	if self.teamType == TeamType.EVENT then
		model.EventModel:removeRobotFromCardModel()
	end
	self.root:setVisible(false);
end

function EquipWearPanel:guideOneKeyWearBtnClick( ... )
	-- model.GuideModel:stopGuide()
	self:oneKeyWearBtnClick()
	-- self.root:runAction(cc.Sequence:create(
	-- 	cc.DelayTime:create(0.5),
	-- 	cc.CallFunc:create(function (  )
	-- 		-- UIManager:switch('MainPanel');
	-- 		model.GuideModel:resumeGuide()
	-- 	end )
	-- ))
end

function EquipWearPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	if guideNo == 22 then
		local btn = self.oneKeyWearBtn;
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	end
	return pos;
end
return EquipWearPanel;
