local EquipWearSelectPanel = class("EquipWearSelectPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local homeImagePath = 'image/home/'

function EquipWearSelectPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'EquipWearSelectPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function EquipWearSelectPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function EquipWearSelectPanel:initPanels()

	local bgSprite1 = cc.Sprite:create('image/bg/Bg_5.png');
	bgSprite1:addTo(self.root);

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -3.5);
    bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(604 , 903));

	local initData = {
        width = 604,
        height = 858,
        cellWidth = 604 ,
        cellHeight = 120,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.equipListView = template.TableViewTemplate:create()
    self.equipListView:initWithSizeAndItem(initData) ;
    self.equipListView:getView():setPosition(cc.p(-302,-418));
	self.equipListView:getView():addTo(self.root)
end

function EquipWearSelectPanel:onEnter(teamSlot , slot)

	if not teamSlot or not slot then
		return;
	end

	self.teamSlot = teamSlot;
	self.slot = slot;

	self.root:setVisible(true);
	self:onShow();
end

function EquipWearSelectPanel:onShow()

	local allEquips = model.EquipModel:getAllEquip();
	self.equips = {};

	for no, num in pairs(allEquips) do
		local t = ResTable:getValue('equip' , no , 'type');
		if t == self.slot then
			if num > 0 then
				table.insert(self.equips, {
					no = no,
					num = num,
				});
			end
		end
	end
	
	if #self.equips > 1 then
		table.sort(self.equips , function(first , second)
			local firstQuailty = ResTable:getValue('item' , first.no , 'quality');
			local secondQuailty = ResTable:getValue('item' , second.no , 'quality');

			return firstQuailty > secondQuailty
		end);
	end

	self.curSelectNo = nil;

	self.equipMaxNum = #self.equips;
	local groupNum = math.ceil(self.equipMaxNum / 5);
	self.equipListView:setDataCount(groupNum);
	self.equipListView:reloadData();

end

function EquipWearSelectPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 5 + 1;
	local curMaxNum = (idx+1) * 5;
	curMaxNum = math.min(curMaxNum , self.equipMaxNum);

	local children = cell:getChildren();

	if not children or #children <= 0 then
		for i = 1 , 5 do
			local x = 65 + 118.5 * (i - 1);
			local y = 62;
			local equipTemplate = template.ItemTemplate:create();
			equipTemplate:setPosition(cc.p(x , y));
			equipTemplate:addTo(cell);
			equipTemplate:addItemClickEventListener(handler(self , self.equipClick));
			if startNum <= curMaxNum then
				equipTemplate:Init(self.equips[startNum].no, self.equips[startNum].num);
				equipTemplate:setItemTag(self.equips[startNum].no);
				equipTemplate:setEquipInSlot(false);
				startNum = startNum + 1;
			else
				equipTemplate:setVisible(false);
			end
		end
	else
		for i = 1 , 5 do
			if startNum <= curMaxNum then
				children[i]:setVisible(true);
				children[i]:Init(self.equips[startNum].no, self.equips[startNum].num);
				children[i]:setItemTag(self.equips[startNum].no);
				children[i]:setEquipInSlot(false);

				startNum = startNum + 1;
			else
				children[i]:setVisible(false);
			end
		end
	end

	return cell;
end

function EquipWearSelectPanel:cellItemTouch(table)
	if self.curSelectNo then
		AudioManager:playEffect(Conf.confirmButton);
		local no = self.curSelectNo;
		local data = {
			no = no, 
			teamSlot = self.teamSlot,
			slot = self.slot,
			enterType = 1,
		}

		UIManager:push('TipsEquipPanel' , data);
		self.curSelectNo = nil;
	end
end

function EquipWearSelectPanel:equipClick(sender , event)
	self.curSelectNo = sender:getTag();
end

function EquipWearSelectPanel:onClickBack()
	UIManager:pop('EquipWearSelectPanel');
end

function EquipWearSelectPanel:onLeave()
	self.curSelectNo = nil;
	self.root:setVisible(false);
end

return EquipWearSelectPanel;
