local EventMainPanel = class("EventMainPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/event/'
local fonts = Enum:getTable('SystemFont')

local Str2Color = function(color)
	if string.len(color) < 6 or string.len(color) > 6 then
		return cc.c3b(255, 255, 255)
	else
		return cc.c3b(tonumber(string.sub(color, 1, 2), 16), 
						tonumber(string.sub(color, 3, 4), 16),
						tonumber(string.sub(color, 5, 6), 16))
	end
end

function EventMainPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'EventMainPanel');

	self.scene = scene;
	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE_EVENT;
	self.haveSwallowLayer = true;
	self.openState = -1;
end

function EventMainPanel:Init(root)
	self.root = root;
	self.root:setVisible(false)

	self:InitPanel()
	self:registEvent()
end

function EventMainPanel:InitPanel()
	
	local eventResData = ResTable:getRowData('maze_resource', model.EventModel:getEventId())
	local AlignmentMode = {
		[1] = cc.p(0, StageSize.height/2),
		[2] = cc.p(0, 0),
		[3] = cc.p(0, -StageSize.height/2),
	}

	-- bg
	self.bg = cc.Sprite:create(imgPath .. eventResData.bg .. '.png')
		:setLocalZOrder(-100)
		:addTo(self.root)

	-- role spine
	self.roleSpineNode = cc.Node:create()
		:setLocalZOrder(-50)
		:addTo(self.root)
	self.roleSpineRes = eventResData.role

	-- other effect
	self.effectList = {}
	for k, v in pairs(eventResData.effect) do
		local effect = {}
		if v[1] == 1 then
			-- spine
			effect.e_type = 'spine'
			effect.name = v[2]
			effect.show_action = v[3]
			effect.continue_action = v[4]
			effect.p_type = v[5]
			effect.x = v[6]
			effect.y = v[7]
			effect.z = v[8]
			effect.effect = nil

		elseif v[1] == 2 then
			-- particle 
			effect.e_type = 'particle'
			effect.name = v[2]
			effect.show_action = (v[3] == 0 and true or false)
			effect.continue_action = (v[4] == 0 and true or false)
			effect.p_type = v[5]
			effect.x = v[6]
			effect.y = v[7]
			effect.z = v[8]

			effect.effect = cc.ParticleSystemQuad:create('Effect/particle/'.. effect.name .. '.plist')
			local mode = AlignmentMode[effect.p_type] or cc.p(0, 0)
			effect.effect:setPosition(mode.x+effect.x, mode.y+effect.y)
			effect.effect:setLocalZOrder(effect.z)
			effect.effect:addTo(self.root)
			effect.effect:stopSystem()

		elseif v[1] == 3 then
			-- image
			effect.e_type = 'image'
			effect.name = v[2]
			effect.show_action = (v[3] == 0 and true or false)
			effect.continue_action = (v[4] == 0 and true or false)
			effect.p_type = v[5]
			effect.x = v[6]
			effect.y = v[7]
			effect.z = v[8]

			effect.effect = cc.Sprite:create(imgPath .. effect.name .. '.png')
			local mode = AlignmentMode[effect.p_type] or cc.p(0, 0)
			effect.effect:setPosition(mode.x+effect.x, mode.y+effect.y)
			effect.effect:setLocalZOrder(effect.z)
			effect.effect:addTo(self.root)
		end		
		table.insert(self.effectList, effect)
	end

	local mode
	-- time
	mode = AlignmentMode[eventResData.time[2]] or cc.p(0, 0)
	-- self.timeImg = cc.Sprite:create(imgPath .. eventResData.time[1] .. '.png')
	-- 	:setPosition(mode.x+eventResData.time[3], mode.y+eventResData.time[4])
	-- 	:setLocalZOrder(eventResData.time[5])
	-- 	:addTo(self.root)
	local timeNode = cc.Node:create()
	timeNode:addTo(self.root)
	timeNode:setLocalZOrder(eventResData.time[5])
	timeNode:setPosition(mode.x+eventResData.time[3], mode.y+eventResData.time[4]-30)
	timeNode:setVisible(false)
	local timeBg = cc.Scale9Sprite:create('image/commonImg/edit_bg_28_0_177_49.png');
	timeBg:addTo(timeNode);
	timeBg:setContentSize(cc.size(220 , 49));
	self.timeLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 24)
	:addTo(timeNode)
	
	-- btns
	mode = AlignmentMode[eventResData.btn_gacha[2]] or cc.p(0, 0)
	self.gachaBtn = ccui.Button:create(imgPath .. eventResData.btn_gacha[1] .. '.png')
		:setPosition(mode.x+eventResData.btn_gacha[3], mode.y+eventResData.btn_gacha[4])
		:setLocalZOrder(eventResData.btn_gacha[5])
		:addTo(self.root)
		:setZoomScale(0)
	self.gachaBtn:addClickEventListener(function()
		UIManager:push('GachaListPanel', 4)
	end)

	mode = AlignmentMode[eventResData.btn_shop[2]] or cc.p(0, 0)
	self.shopBtn = ccui.Button:create(imgPath .. eventResData.btn_shop[1] .. '.png')
		:setPosition(mode.x+eventResData.btn_shop[3], mode.y+eventResData.btn_shop[4])
		:setLocalZOrder(eventResData.btn_shop[5])
		:addTo(self.root)
		:setZoomScale(0)
	self.shopBtn:addClickEventListener(function()
		-- UIManager:push('EventShopPanel')
		UIManager:push('StorePanel', Enum:getTable('CommonShopType').MAZE)
	end)

	mode = AlignmentMode[eventResData.btn_buff[2]] or cc.p(0, 0)
	self.buffBtn = ccui.Button:create(imgPath .. eventResData.btn_buff[1] .. '.png')
		:setPosition(mode.x+eventResData.btn_buff[3], mode.y+eventResData.btn_buff[4])
		:setLocalZOrder(eventResData.btn_buff[5])
		:addTo(self.root)
		:setZoomScale(0)
	self.buffBtn:addClickEventListener(function()
		UIManager:push('EventBuffPanel',1)
	end)

	-- main btn
	-- mode = AlignmentMode[eventResData.btn_style[1][2]] or cc.p(0, 0)
	self.challengeBtnNode = cc.Node:create()
		-- :setPosition(mode.x+eventResData.btn_style[1][3], mode.y+eventResData.btn_style[1][4])
		:setPosition(0, 80-StageSize.height/2)
		:addTo(self.root)

	self.challengeBtn = ccui.Button:create(imgPath ..'main_btn_bg.png')
		:setZoomScale(0)
		:addTo(self.challengeBtnNode)
	self.challengeBtn:addClickEventListener(handler(self,self.onChallengeCall))

	self.challengeSpr = cc.Sprite:create(imgPath ..'word_active.png')
	:addTo(self.challengeBtnNode)
	:setPosition(0, 15)
	local challLabel1 = cc.Label:createWithTTF(LANG.UI_MAZE_2, fonts.FONT_SIYUAN, 18)
	:addTo(self.challengeBtnNode)
	:setPosition(-40, -30)
	self.challTimeLabel = cc.Label:createWithTTF('48:39:52', fonts.FONT_SIYUAN, 18)
	:addTo(self.challengeBtnNode)
	:setPosition(40, -30)

	-- self.challengeCoinNode = cc.Node:create()
	-- 	:setPosition(eventResData.btn_style[5][2], eventResData.btn_style[5][3])
	-- 	:setLocalZOrder(eventResData.btn_style[5][4])
	-- 	:addTo(self.challengeBtnNode)

	-- self.challengeCoinBg1 = cc.Sprite:create(imgPath .. eventResData.btn_style[5][1] .. '.png')
	-- 	:addTo(self.challengeCoinNode)

	-- local iconPath = ResTable:getValue('item', EVENT_PT, 'icon')
	-- self.challengeCoin = cc.Sprite:create('image/icon/I'..iconPath..'.png')
	-- 	:setPosition(eventResData.btn_style[5][5], eventResData.btn_style[5][6])
	-- 	:setScale(0.4)
	-- 	:addTo(self.challengeCoinNode)
	
	-- self.coinNum = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, eventResData.btn_style[5][9])
	-- 	:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
	-- 	:setTextColor(Str2Color(eventResData.btn_style[5][10]))
	-- 	:setAnchorPoint(cc.p(1, 0.5))
	-- 	:setPosition(eventResData.btn_style[5][7], eventResData.btn_style[5][8])
	-- 	:addTo(self.challengeCoinNode)
	-- if eventResData.btn_style[5][12] > 0 then
	-- 	self.coinNum:enableOutline(Str2Color(eventResData.btn_style[5][11]), eventResData.btn_style[5][12])
	-- end

	-- self.challengeProgressNode = cc.Node:create()
	-- 	:setPosition(eventResData.btn_style[4][6], eventResData.btn_style[4][7])
	-- 	:setLocalZOrder(eventResData.btn_style[4][8])
	-- 	:addTo(self.challengeBtnNode)

	-- self.challengeProgressBg = cc.Sprite:create(imgPath .. eventResData.btn_style[4][1] .. '.png')
	-- 	:addTo(self.challengeProgressNode)

	-- self.challengeProgressLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, eventResData.btn_style[4][2])
	-- 	:setTextColor(Str2Color(eventResData.btn_style[4][3]))
	-- 	:addTo(self.challengeProgressNode)
	-- if eventResData.btn_style[4][5] > 0 then
	-- 	self.challengeProgressLabel:enableOutline(Str2Color(eventResData.btn_style[4][4]), eventResData.btn_style[4][5])
	-- end

	-- self.goonImg = cc.Sprite:create(imgPath .. eventResData.btn_style[2][1] .. '.png')
	-- 	:setPosition(eventResData.btn_style[2][2], eventResData.btn_style[2][3])
	-- 	:setLocalZOrder(eventResData.btn_style[2][4])
	-- 	:addTo(self.challengeBtnNode)

	-- self.activeImg = cc.Sprite:create(imgPath .. eventResData.btn_style[3][1] .. '.png')
	-- 	:setPosition(eventResData.btn_style[3][2], eventResData.btn_style[3][3])
	-- 	:setLocalZOrder(eventResData.btn_style[3][4])
	-- 	:addTo(self.challengeBtnNode)

	-- self.btnDifficultBg = cc.Sprite:create(imgPath .. eventResData.btn_style[6][1] .. '.png')
	-- 	:setPosition(eventResData.btn_style[6][2], eventResData.btn_style[6][3])
	-- 	:setLocalZOrder(eventResData.btn_style[6][4])
	-- 	:addTo(self.challengeBtnNode)

	-- self.difficultIcon = cc.Sprite:create()
	-- 	:setPosition(eventResData.btn_style[7][1], eventResData.btn_style[7][2])
	-- 	:setLocalZOrder(eventResData.btn_style[7][3])
	-- 	:addTo(self.challengeBtnNode)

	-- self.difficultWord = cc.Label:createWithTTF('12112', fonts.FONT_SIYUAN, eventResData.btn_style[8][1])
	-- 	:setTextColor(Str2Color(eventResData.btn_style[8][2]))
	-- 	:setPosition(eventResData.btn_style[8][5], eventResData.btn_style[8][6])
	-- 	:setLocalZOrder(eventResData.btn_style[8][7])
	-- 	:addTo(self.challengeBtnNode)
	-- if eventResData.btn_style[8][4] > 0 then
	-- 	self.difficultWord:enableOutline(Str2Color(eventResData.btn_style[8][3]), eventResData.btn_style[8][4])
	-- end

	-- -- reward
	-- self.rewardNode = cc.Node:create()
	-- 	:setPosition(0, -StageSize.height/2)
	-- 	:setLocalZOrder(10)
	-- 	:addTo(self.root)

	-- self.rewardBg = cc.Sprite:create(imgPath .. 'reward_bg.png')
	-- 	:setPosition(0, 90)
	-- 	:addTo(self.rewardNode)	

	-- self.allNode = cc.Node:create()
	-- 	:addTo(self.rewardNode)

	-- self.eventMark = cc.Sprite:create(imgPath .. 'event_bg_15.png')
	-- 	:setPosition(-140, 90)
	-- 	:addTo(self.allNode)

	-- self.allLabel = cc.Label:createWithTTF(LANG.UI_EVENT_15, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(255, 255, 255))
	-- 	:enableOutline(cc.c3b(65, 185, 60), 2)
	-- 	:setPosition(-20, 90)
	-- 	:addTo(self.allNode)

	-- self.rewardWord1 = cc.Label:createWithTTF(LANG.UI_EVENT_8, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(60, 60, 60))
	-- 	:enableOutline(cc.c3b(255, 255, 255), 2)
	-- 	:setPosition(-175, 150)
	-- 	:addTo(self.rewardNode)
	
	-- -- btn
	-- local label = cc.Label:createWithTTF(LANG.UI_EVENT_9, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(60, 60, 60))
	-- 	:enableOutline(cc.c3b(255, 255, 255), 2)
	-- self.btnReward = ccui.Button:create(imgPath .. 'btn_reward2.png')
	-- 	:setPosition(200, 80)
	-- 	:setTitleLabel(label)
	-- 	:addTo(self.rewardNode)
	-- self.btnReward:addClickEventListener(function()
	-- 		UIManager:push('RoundTargetRewardPanel',Enum:getTable("RoundTargetRewardType").EVENT)
	-- 	end)
	-- label = cc.Label:createWithTTF(LANG.UI_EVENT_24, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(255, 255, 255))
	-- 	:enableOutline(cc.c3b(38, 131, 232), 2)
	-- self.btnReward2 = ccui.Button:create(imgPath .. 'btn_reward.png')
	-- 	:setPosition(200, 80)
	-- 	:addTo(self.rewardNode)
	-- 	:setTitleLabel(label)
	-- self.btnReward2:addClickEventListener(function()
	-- 		UIManager:push('RoundTargetRewardPanel',Enum:getTable("RoundTargetRewardType").EVENT)
	-- 	end)
	-- --	
	-- self.targetNode = cc.Node:create()
	-- 	:addTo(self.rewardNode)

	-- self.moneyLabel1 = cc.Label:createWithTTF(LANG.UI_EVENT_10, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(252, 0, 97))
	-- 	:setPosition(-30, 105)
	-- 	:addTo(self.targetNode)

	-- iconPath = ResTable:getValue('item', EVENT_MONEY, 'icon')
	-- self.moneyIcon1 = cc.Sprite:create('image/icon/I'..iconPath..'.png')
	-- 	:setPosition(35, 105)
	-- 	:setScale(0.4)
	-- 	:addTo(self.targetNode)

	-- self.moneyNumLabel1 = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(252, 0, 97))
	-- 	:setPosition(90, 105)
	-- 	:addTo(self.targetNode)

	-- self.moneySp = cc.Sprite:create(imgPath .. 'event_bg_16.png')
	-- 	:setPosition(30, 80)
	-- 	:addTo(self.targetNode)
		
	-- self.moneyLabel2 = cc.Label:createWithTTF(LANG.UI_EVENT_23, fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(50, 50, 50))
	-- 	:setPosition(-20, 55)
	-- 	:addTo(self.targetNode)

	-- iconPath = ResTable:getValue('item', EVENT_MONEY, 'icon')
	-- self.moneyIcon2 = cc.Sprite:create('image/icon/I'..iconPath..'.png')
	-- 	:setPosition(25, 55)
	-- 	:setScale(0.4)
	-- 	:addTo(self.targetNode)

	-- self.moneyNumLabel2 = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
	-- 	:setTextColor(cc.c3b(50, 50, 50))
	-- 	:setPosition(70, 55)
	-- 	:addTo(self.targetNode)
		
	-- self.itemBg = cc.Sprite:create(imgPath .. 'item_bg.png')
	-- 	:setPosition(-175, 80)
	-- 	:addTo(self.targetNode)

	-- self.reward1 = require('app.template.IconTemplate'):create(self.targetNode)
	-- self.reward1:setPosition(-220, 80)
	-- self.reward1:setScale(0.7)
	-- self.reward1:Init(MONEY_ID, '100')

	-- self.reward2 = require('app.template.IconTemplate'):create(self.targetNode)
	-- self.reward2:setPosition(-135, 80)
	-- self.reward2:setScale(0.7)
	-- self.reward2:Init(MONEY_ID, '100')

	self.activeSpineNode = cc.Node:create()
		:setLocalZOrder(15)
		:addTo(self.root)
		:setVisible(false);

	self.enterEffect = eventResData.btn_effect

	local layout = ccui.Layout:create();
	layout:setContentSize(cc.size(StageSize.width, StageSize.height));
	layout:setPosition(cc.p(-StageSize.width/2, -StageSize.height/2))
	layout:addTo(self.activeSpineNode);
	layout:setTouchEnabled(true);

	self.activeSpineBtn = ccui.Button:create('image/commonImg/btn_bg1.png' , 'image/commonImg/btn_bg1.png');
	self.activeSpineBtn:addTo(self.activeSpineNode);
	self.activeSpineBtn:setPosition(0 , -203);
	self.activeSpineBtn:addClickEventListener(handler(self , self.activeSpineBtnClick));
	self.activeSpineBtn:setLocalZOrder(10)

	local label = cc.Label:createWithTTF(LANG.COMMON_OK , fonts.FONT_SIYUAN , 20);
	label:addTo(self.activeSpineBtn);
	label:setPosition(self.activeSpineBtn:getContentSize().width/2 , self.activeSpineBtn:getContentSize().height/2);
	label:setTextColor(cc.c3b(255 , 255 , 255));
	label:enableOutline(cc.c4b(19 , 93 , 188 , 255) , 2);

	self.activeParticle = cc.ParticleSystemQuad:create("Effect/particle/pipei.plist")
	self.activeParticle:addTo(self.activeSpineNode);
	self.activeParticle:stopSystem();

	local bpNode = cc.Node:create()
	bpNode:addTo(self.root)
	bpNode:setPosition(StageSize.width/2-100, StageSize.height/2-300 - NOTCH_HEIGHT)
	local bpBg = cc.Sprite:create('image/trial/n_r_btn.png')
	bpBg:addTo(bpNode)
	local bpBtn = ccui.Button:create('image/event/bp_enter.png')
	bpBtn:addTo(bpNode)
	bpBtn:addClickEventListener(handler(self , self.openMazeBattlePass))
	local spine_res = 'migongpass_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local bpSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		bpSpine:addTo(bpNode)
		bpSpine:setAnimation(0, "idle", true)
	end
	local bpLabel = cc.Label:createWithTTF(LANG.UI_MAZE_58, fonts.FONT_SIYUAN, 16)
	bpLabel:addTo(bpNode)
	bpLabel:setPosition(0, -38)
	bpLabel:enableOutline(cc.c3b(60, 60, 60), 2)
	self.bpNode = bpNode;
end

function EventMainPanel:openMazeBattlePass()
	local endTime = model.EventModel:getBPData().show_time;
	local lastTime = endTime - TimeManager:getCurTime()
	if lastTime > 0 then
		UIManager:push('EventBattlePassPanel')
	else
		Prompt:Show(LANG.UI_L_OVER)
	end
end

function EventMainPanel:registEvent()
	EventManager:RegisterEvent(EventType.NetworkEventInitInfo, handler(self, self.onShow))
	EventManager:RegisterEvent(EventType.NetworkGetEventInfo , handler(self, self.showActiveSpine));
	-- EventManager:RegisterEvent(EventType.UIEventRewardUpdate , handler(self, self.updateReward));
	EventManager:RegisterEvent(EventType.UIEventChallengeUpdate , handler(self, self.updateChallengeBtn));

	self.resetTime = -1

end

function EventMainPanel:updateTime()

	-- body
	if self.resetTime<=0 then
		return
	end
	local lastTime = math.floor(self.resetTime - TimeManager:getCurTime())
	if lastTime > 0 then
		self.challTimeLabel:setString( Sec2Time(lastTime) )
	else
		self.challTimeLabel:setString( '' )
	end
	
end

function EventMainPanel:onEnter()

	self.resetTime = model.EventModel:getResetTime();
	TimeManager:registUpdateEvent('EventMainPanel',handler(self,self.updateTime),true);
	-- if not model.EventModel:isInit() then
	-- 	model.EventModel:reqEventInfo();
	-- else
	-- 	self:onShow()
	-- end
	self:onShow()
end

function EventMainPanel:onLeave()
	self:removeSpineRole()
	self:removeSpineOther()
	self.root:setVisible(false)
	TimeManager:registUpdateEvent('EventMainPanel',nil,true);

	self.resetTime = -1
end

function EventMainPanel:onShow()
	-- if not self.haveInitialize then
	-- 	return
	-- end
	-- local event_id = model.EventModel:getEventId()
	-- if event_id <= 0 then
	-- 	return
	-- end
	self.root:setVisible(true)
	self:updateChallengeBtn()
	-- self:updateReward()

	self:addSpineRole()
	self:addSpineOther()
	self:playEnterEffect()
	self:playContinueEffect()

	local startTime = model.EventModel:getEventStartTime()
    local endTime = model.EventModel:getEventCloseTime()
    if startTime ~= 0 then
        local startTable = os.date('*t', startTime)
        local endTable = os.date('*t', endtime)
        local msg = string.format( LANG.UI_EVENT_29,startTable.month,startTable.day ).."~"..string.format( LANG.UI_EVENT_29,endTable.month,endTable.day )
        self.timeLabel:setString(msg);
    end
    local curTime = TimeManager:getCurTime()
    if curTime<startTime then
    	self.openState = 1
    elseif curTime>endTime then
    	self.openState = 2
    else
    	self.openState = 0
    end

    self.bpNode:setVisible( model.EventModel:isShowEventBp() );

end

function EventMainPanel:onChallengeCall( )
	if self.openState == 0 then
		UIManager:push('EventMapPanel')
	else
		Prompt:Show(LANG.UI_MAZE_18)
	end

end

function EventMainPanel:showActiveSpine()
	--pipei_spine 这个动画没用了
	self.activeSpineBtn:setVisible(false);
	self.activeSpineNode:setVisible(true);
	local spine_res = 'pipei_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.skeletonNodeActive = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)

		self.skeletonNodeActive:addTo(self.activeSpineNode)
		self.skeletonNodeActive:setAnimation(0, "start", false)

		self.skeletonNodeActive:registerSpineEventHandler(function (event)
			self.skeletonNodeActive:setAnimation(0, "idle", true)
		end, sp.EventType.ANIMATION_COMPLETE)

		self.skeletonNodeActive:registerSpineEventHandler(function (event)
			if event.eventData.name == 'nandu_spine' then
				spine_res = 'nandu_spine';
				if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
					-- local fpNo = model.EventModel:getFpNo();
					self.sktNode = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
					self.sktNode:addTo(self.activeSpineNode);
					-- self.sktNode:setAnimation(0, string.format("nandu%d" , fpNo), false);

					self.sktNode:registerSpineEventHandler(function (event)
						self.activeSpineBtn:setVisible(true);
					end, sp.EventType.ANIMATION_COMPLETE)
				end
			end		
		end, sp.EventType.ANIMATION_EVENT)
	end

	self.activeParticle:resetSystem()
end

function EventMainPanel:activeSpineBtnClick()
	self.activeSpineNode:setVisible(false);

	if self.skeletonNodeActive then
		self.skeletonNodeActive:removeFromParent();
		self.skeletonNodeActive = nil;
	end

	if self.sktNode then
		self.sktNode:removeFromParent();
	end

	self.activeParticle:stopSystem();

	UIManager:push('EventMapPanel');
end

function EventMainPanel:onClickBack()	
	UIManager:pop('EventMainPanel')
end

function EventMainPanel:updateChallengeBtn()
	if model.EventModel:getActiveFlag() ~= 0 then
		-- self.challengeCoinNode:setVisible(true)
		
		-- local data = ResTable:getMultiValue('event_hard', model.EventModel:getFpNo(), model.EventModel:getHard());
		-- if data then
		-- 	self.coinNum:setString(tostring(data.cost))
		-- end

		-- self.challengeProgressNode:setVisible(false)

		-- self.goonImg:setVisible(false)
		-- self.activeImg:setVisible(true)

		-- self.btnDifficultBg:setVisible(false)
		-- self.difficultIcon:setVisible(false)
		-- self.difficultWord:setVisible(false)

	else
		-- self.challengeCoinNode:setVisible(false)

		-- self.challengeProgressNode:setVisible(true)
		-- self.challengeProgressLabel:setString(LANG.UI_EVENT_11 .. model.EventModel:getTier()..'/5')

		-- self.goonImg:setVisible(true)
		-- self.activeImg:setVisible(false)

		-- self.btnDifficultBg:setVisible(true)
		-- self.difficultIcon:setVisible(true)
		-- local difficult = model.EventModel:getFpNo()
		-- self.difficultIcon:setTexture(imgPath .. 'difficult_' .. difficult .. '.png')
		-- self.difficultWord:setVisible(true)
		-- local name = ResTable:getValue('event_fp', difficult, 'fp_name')
		-- local hard = model.EventModel:getHard()
		-- self.difficultWord:setString(name..hard)--.. '-'
	end
end

-- function EventMainPanel:updateReward()
-- 	local curMoney = model.EventModel:getEventTotalMoney()
-- 	local event_id = model.EventModel:getEventId()
-- 	local minNo, data
-- 	local last_point = 0
-- 	local hasReward2Get = false

-- 	for k, v in pairs(ResTable:getTable('event_reward').contain) do
-- 		if v.event_id == event_id then
-- 			if v.total_point > curMoney then
-- 				if minNo then
-- 					if v.no < minNo then
-- 						minNo = v.no
-- 						data = v
-- 					end
-- 				else
-- 					minNo = v.no
-- 					data = v
-- 				end
-- 			elseif v.total_point <= curMoney then
-- 				if v.total_point > last_point then
-- 					last_point = v.total_point
-- 				end
-- 				if not model.EventModel:getEventRewardState(v.id) then
-- 					hasReward2Get = true
-- 				end
-- 			end
-- 		end
-- 	end
-- 	if data then
-- 		self.targetNode:setVisible(true)
-- 		self.allNode:setVisible(false)

-- 		self.moneyNumLabel1:setString(': ' .. tostring(curMoney))
-- 		self.moneyNumLabel2:setString(': ' .. tostring(data.total_point - curMoney))
-- 		if data.reward[1] then
-- 			self.reward1:Init(data.reward[1][1], data.reward[1][2])
-- 			self.reward1:setVisible(true)
-- 		else
-- 			self.reward1:setVisible(false)
-- 		end
-- 		if data.reward[2] then
-- 			self.reward2:Init(data.reward[2][1], data.reward[2][2])
-- 			self.reward2:setVisible(true)
-- 		else
-- 			self.reward2:setVisible(false)
-- 		end
-- 	else
-- 		self.targetNode:setVisible(false)
-- 		self.allNode:setVisible(true)
-- 	end
-- 	-- btn
-- 	if not hasReward2Get then
-- 		self.btnReward:setVisible(true)
-- 		self.btnReward2:setVisible(false)
-- 	else
-- 		self.btnReward:setVisible(false)
-- 		self.btnReward2:setVisible(true)
-- 	end
-- end

function EventMainPanel:addSpineRole()
	if self.skeletonNode ~= nil then
		return 
	end
	if cc.FileUtils:getInstance():isFileExist('spine/'..self.roleSpineRes..'/'..self.roleSpineRes..'.json') then
		self.skeletonNode = sp.SkeletonAnimation:create('spine/'..self.roleSpineRes..'/'..self.roleSpineRes..'.json', 'spine/'..self.roleSpineRes..'/'..self.roleSpineRes..'.atlas', 1)

		self.skeletonNode:addTo(self.roleSpineNode)
		self.skeletonNode:setLocalZOrder(5)
		self.skeletonNode:setAnimation(0, "idle", true)
	end
end

function EventMainPanel:removeSpineRole()
	if self.skeletonNode then
		self.skeletonNode:removeFromParent()
		self.skeletonNode = nil
	end
end

function EventMainPanel:addSpineOther()
	local AlignmentMode = {
		[1] = cc.p(0, StageSize.height/2),
		[2] = cc.p(0, 0),
		[3] = cc.p(0, -StageSize.height/2),
	}
	for k, v in pairs(self.effectList) do
		if v.e_type == 'spine' then
			if v.effect ~= nil then
				return;
			end
			local res = v.name
			if cc.FileUtils:getInstance():isFileExist('spine/'..res..'/'..res..'.json') then
				v.effect = sp.SkeletonAnimation:create('spine/'..res..'/'..res..'.json', 'spine/'..res..'/'..res..'.atlas', 1)

				local mode = AlignmentMode[v.p_type] or cc.p(0, 0)
				v.effect:addTo(self.root)
				v.effect:setPosition(mode.x+v.x, mode.y+v.y)
				v.effect:setLocalZOrder(v.z)
				v.effect:setAnimation(0, GlobalFunctions:lAnimationName(v.show_action), false)

				-- registerEvent
				v.effect:registerSpineEventHandler(function (event)
					v.effect:setAnimation(0, GlobalFunctions:lAnimationName(v.continue_action), false)
				end, sp.EventType.ANIMATION_COMPLETE)
			end
		end		
	end
end

function EventMainPanel:removeSpineOther()
	for k, v in pairs(self.effectList) do
		if v.e_type == 'spine' then
			if v.effect then
				v.effect:removeFromParent()
				v.effect = nil
			end
		end
	end
end

function EventMainPanel:playEnterEffect()
	if self.enterEffect[1] == 1 then
		local x, y = self.challengeBtnNode:getPosition()
		self.challengeBtnNode:setPosition(x-self.enterEffect[3], y-self.enterEffect[4])
		self.challengeBtnNode:runAction(cc.MoveBy:create(self.enterEffect[2], cc.p(self.enterEffect[3], self.enterEffect[4])))
		
		-- x, y = self.rewardNode:getPosition()
		-- self.rewardNode:setPosition(x-self.enterEffect[3], y-self.enterEffect[4])
		-- self.rewardNode:runAction(cc.MoveBy:create(self.enterEffect[2], cc.p(self.enterEffect[3], self.enterEffect[4])))

	elseif self.enterEffect[1] == 2 then
		self.challengeBtnNode:setOpacity(0)
		self.challengeBtnNode:runAction(cc.FadeIn:create(self.enterEffect[2]))

		-- self.rewardNode:setOpacity(0)
		-- self.rewardNode:runAction(cc.FadeIn:create(self.enterEffect[2]))
	end

	for k, v in pairs(self.effectList) do
		if v.e_type == 'particle' then
			if v.show_action then
				v.effect:resetSystem()
			else
				v.effect:stopSystem()
			end
		end
	end
end

function EventMainPanel:playContinueEffect()
	local node = cc.Node:create()
		:addTo(self.root)
	node:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.5),
		cc.CallFunc:create(function()
			for k, v in pairs(self.effectList) do
				if v.e_type == 'particle' then
					if v.continue_action then
						v.effect:resetSystem()
					else
						v.effect:stopSystem()
					end
				end
			end
			node:removeFromParent()
		end)
	))
end

return EventMainPanel
