local EventMapPanel = class("EventMapPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');
local imagePath = 'image/event/map/';
local MessageBoxType = Enum:getTable('MessageBoxType')
local AnimationType = Enum:getTable('AnimationType');

local MAP_ITEM_PT_COST = 5; -- 点击地板消耗

function EventMapPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'EventMapPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE_EVENT;
	
	self.spineCallback = nil;
end

function EventMapPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self.tier = 0
	self.mapCaoDiList = {}

	self:initPanels()
	self:initWinPanel()
	self:registEvent();
end

function EventMapPanel:initPanels()

	local bg = ccui.ImageView:create(imagePath..'map_bg.png');
	bg:addTo(self.root);
	bg:setTouchEnabled(true);
	bg:setPosition(0 , 0);-- -58

	local spine_res = 'bg_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local bgspine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		bgspine:addTo(self.root)
		bgspine:setAnimation(0,'bg_spine',true)
    end

	self.mapItemsNode = cc.Node:create();
	self.mapItemsNode:addTo(self.root);

	self.roleNode = cc.Node:create()
	:addTo(self.mapItemsNode)
	:retain()

	self.caoSpineList = {}
	for i=1,3 do
		local spine_res = 'cao_spine'
		if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
			local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
			spine:addTo(self.root)
			self.caoSpineList[i] = spine
	    end
	end

	local mapBar = cc.Sprite:create(imagePath..'map_bg_bar.png')
	mapBar:addTo(self.root)
	mapBar:setPosition(0, -385)

 	local bottomNode = cc.Node:create()
 	:addTo(self.root)
 	:setPosition(0, -StageSize.height/2)

 	local btnsBg = cc.Sprite:create('image/main/event_btns_bg.png')
 	btnsBg:addTo(bottomNode)
 	btnsBg:setPosition(140, 80)

	local buffBtn = ccui.Button:create('image/main/main_btn_bg.png')
	buffBtn:addTo(bottomNode)
	buffBtn:setPosition(240,80)
	buffBtn:setOpacity(0)
	buffBtn:addClickEventListener(handler(self , self.onShowEventBuff))
	local buffSp = cc.Sprite:create('image/main/evt_buff_icon.png')
	buffSp:addTo(bottomNode)
	buffSp:setPosition(cc.p(240,80));
	local btnName = cc.Label:createWithTTF( LANG.UI_MAZE_41, fonts.FONT_SIYUAN, 20)
	:enableOutline(cc.c3b(60, 60, 60), 2)
	:setPosition(240 , 52)
	:addTo(bottomNode)

	local spine_res = 'fly_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(bottomNode)
		spine:setPosition(240,80)
		self.buffFlySp = spine
	end

	self.btnTeamRole = ccui.Button:create('image/main/main_btn_bg.png')
	self.btnTeamRole:addTo(bottomNode)
	self.btnTeamRole:setPosition(140,80)
	self.btnTeamRole:setOpacity(0)
	self.btnTeamRole:addClickEventListener(handler(self , self.onShowEventTeamInfo))
	local roleSp = cc.Sprite:create('image/main/evt_hero_icon.png')
	roleSp:addTo(bottomNode)
	roleSp:setPosition(140,80);
	local btnName = cc.Label:createWithTTF( LANG.UI_MAZE_40, fonts.FONT_SIYUAN, 20)
	:enableOutline(cc.c3b(60, 60, 60), 2)
	:setPosition(140 , 52)
	:addTo(bottomNode)

	local itemBtn = ccui.Button:create('image/main/main_btn_bg.png')
	itemBtn:addTo(bottomNode)
	itemBtn:setPosition(40,80)
	itemBtn:setOpacity(0)
	itemBtn:addClickEventListener(handler(self , self.onShowItemPanel))
	local itemSp = cc.Sprite:create('image/main/evt_item_icon.png')
	itemSp:addTo(bottomNode)
	itemSp:setPosition(cc.p(40,80));
	local btnName = cc.Label:createWithTTF( LANG.UI_MAZE_39, fonts.FONT_SIYUAN, 20)
	:enableOutline(cc.c3b(60, 60, 60), 2)
	:setPosition(40 , 52)
	:addTo(bottomNode)

	self:initExitNode();
	self:initTopNode()

	local bgParticle = cc.ParticleSystemQuad:create("Effect/particle/events_bg_p.plist")
	bgParticle:addTo(self.root);
	bgParticle:setPosition(0, StageSize.height/2+20)

	self.buffParticle = cc.ParticleSystemQuad:create("Effect/particle/fly_p.plist")
	self.buffParticle:addTo(self.root);
	self.buffParticle:setVisible(false);

	local spine_res = 'yun_spine'
	self.skYun = nil;
	self.closeYun = nil;
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.skYun = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.skYun:addTo(self.root)
		self.skYun:setVisible(false)

		self.closeYun = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.closeYun:addTo(self.root)
		self.closeYun:setVisible(false)
	end

end

function EventMapPanel:initExitNode()
	-- body
	self.exitNode = cc.Node:create()
	self.exitNode:addTo(self.root)
	self.exitNode:setPosition(0, 480)

	local spine_res = 'chuansongmen_spine'
	self.exitAni = nil
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.exitAni = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.exitAni:addTo(self.exitNode)
		self.exitAni:registerSpineEventHandler(function (event)
			if event.animation == 'open1' then
				self.exitAni:setAnimation(0, "open2", true)
			end	
		end, sp.EventType.ANIMATION_COMPLETE)
		self.exitAni:registerSpineEventHandler(function (event)
			if event.eventData.name == 'shake' then
				self:shakeRoot();
			end
		end, sp.EventType.ANIMATION_EVENT);
    end

	local exitBtn = ccui.Button:create('image/event/map/item_door_1.png')
	exitBtn:addTo(self.exitNode)
	exitBtn:setOpacity(0)
	exitBtn:addClickEventListener(handler(self , self.gotoNextTire))


end

function EventMapPanel:initTopNode()
	-- body
	local topNode = cc.Node:create()
	topNode:addTo(self.root)
	topNode:setPosition(-StageSize.width/2+110, 460)

	local bg = cc.Sprite:create('image/event/event_tip_bg.png')
	:addTo(topNode)

	-- local ds1 = cc.Sprite:create('image/commonImg/com_des_icon.png')
	-- ds1:setPosition(-100, 7)
	-- ds1:addTo(topNode)

	-- local ds2 = cc.Sprite:create('image/commonImg/com_des_icon.png')
	-- ds2:setPosition(100, 7)
	-- ds2:setFlipX(true)
	-- ds2:addTo(topNode)

	self.title1Label = cc.Label:createWithTTF(LANG.UI_MAZE_3, fonts.FONT_SIYUAN, 22)
	self.title1Label:addTo(topNode)
	self.title1Label:setPosition(0, 18)
	self.title1Label:enableOutline(cc.c3b(0, 0, 0), 2)

	self.title2Label = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
	self.title2Label:addTo(topNode)
	self.title2Label:setPosition(0 , -23)
	self.title2Label:enableOutline(cc.c3b(60, 60, 60), 2)

	-- local bpNode = cc.Node:create()
	-- bpNode:addTo(topNode)
	-- bpNode:setPosition(420, 50)
	-- local bpBg = cc.Sprite:create('image/trial/n_r_btn.png')
	-- bpBg:addTo(bpNode)
	-- local bpBtn = ccui.Button:create('image/event/bp_enter.png')
	-- bpBtn:addTo(bpNode)
	-- bpBtn:addClickEventListener(handler(self , self.openMazeBattlePass))
	-- local bpLabel = cc.Label:createWithTTF(LANG.UI_MAZE_58, fonts.FONT_SIYUAN, 20)
	-- bpLabel:addTo(bpNode)
	-- bpLabel:setPosition(0, -40)
	-- bpLabel:enableOutline(cc.c3b(60, 60, 60), 2)



	local btmNode = cc.Node:create()
	btmNode:addTo(self.root)
	btmNode:setPosition(-StageSize.width/2+140, -StageSize.height/2+80)

	local timeNode = cc.Node:create()
	timeNode:addTo(btmNode)
	timeNode:setPosition(0, -20)
	local bg = cc.Scale9Sprite:create('image/commonImg/com_bg.png')
	:addTo(timeNode)
	:setContentSize(cc.size(200 , 30));
	-- :setAnchorPoint(0.5,0.5)
	local timeIcon = cc.Sprite:create('image/event/event_time_icon.png')
	timeIcon:addTo(timeNode)
	timeIcon:setPosition(-80, 0)
	self.timeLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 16)
	self.timeLabel:addTo(timeNode)
	self.timeLabel:setPosition(-60, 0)
	self.timeLabel:setAnchorPoint(0, 0.5)


	local movePtNode = cc.Node:create()
	movePtNode:addTo(btmNode)
	movePtNode:setPosition(0, 20)
	local bg = cc.Scale9Sprite:create('image/commonImg/com_bg.png')
	:addTo(movePtNode)
	:setContentSize(cc.size(200 , 30));
	-- :setAnchorPoint(1,0.5)
	local mvIcon = cc.Sprite:create('image/event/event_move_icon.png')
	mvIcon:addTo(movePtNode)
	mvIcon:setPosition(-80, 0)
	self.ptLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 16)
	self.ptLabel:addTo(movePtNode)
	self.ptLabel:setPosition(-60, 0)
	self.ptLabel:setAnchorPoint(0, 0.5)

end

function EventMapPanel:initWinPanel( ... )
	self.winPanel = cc.Node:create();
	self.winPanel:addTo(self.root);

	local blackBg = ccui.Layout:create()
		:setBackGroundImage('image/task/blackbg.png')
		:setBackGroundImageScale9Enabled(true)
		:setTouchEnabled(true)
		:setContentSize(cc.size(StageSize.width , StageSize.height))
		-- blackBg:setLocalZOrder(-2);
		:setAnchorPoint(cc.p(0.5,0.5))
		:setPosition(0, 0)
		:addTo(self.winPanel)
	blackBg:addTouchEventListener(function ( sender,eventType )
		if eventType == ccui.TouchEventType.ended then
			self:onChangeToEventMain();
		end
	end);

	self.particleWin = cc.ParticleSystemQuad:create("Effect/particle/winp.plist")
	self.particleWin:addTo(self.winPanel);
	-- self.particleWin:setPosition(cc.p(0,0))
	self.particleWin:stopSystem();

	local winSp = cc.Sprite:create('image/commonImg/common_win.png')
		:addTo(self.winPanel);
		-- :setPosition()

	
	self.winPanel:setVisible(false);

end

function EventMapPanel:registEvent(  )
	EventManager:RegisterEvent(EventType.NetworkEventGetMapInfo , handler(self, self.retEventMapGetInfo));

	EventManager:RegisterEvent(EventType.NetworkEventMapItemFight , handler(self, self.retEventMapItemFight));
	EventManager:RegisterEvent(EventType.NetworkEventReset , handler(self , self.retReset));
	EventManager:RegisterEvent(EventType.NetworkEventSelectBuff , handler(self, self.retSelectBuff));



	-- EventManager:RegisterEvent(EventType.NetworkGetEventInfo , handler(self, self.retEventMapGetInfo));

	EventManager:RegisterEvent(EventType.UIUpdateEventPos , handler(self, self.updateRolePos));
	EventManager:RegisterEvent('showChangeTierAct', handler(self, self.showChangeTierAct));

	self.resetTime = -1
	
end

function EventMapPanel:updateTime()
	if self.resetTime<=0 then
		return
	end
	local lastTime = math.floor(self.resetTime - TimeManager:getCurTime())
	if lastTime > 0 then
		self.timeLabel:setString( LANG.UI_MAZE_2..Sec2Time(lastTime) )
	else
		self.timeLabel:setString( LANG.UI_MAZE_19 )
	end

	if #self.mapCaoDiList > 0 then
		if self.caodiTime>=5 then
			local tmpPos = {}
			for i=1,3 do
				local randIdx = math.random(1, #self.mapCaoDiList)
				local pos = self.mapCaoDiList[randIdx]
				if pos ~= tmpPos[1] or pos ~= tmpPos[2] then
					if self.mapItemLists[pos] and self.mapItemLists[pos]:isCaoDi() then
						self.caoSpineList[i]:setPosition( self.mapItemLists[pos]:getPosition() )
						self.caoSpineList[i]:setAnimation(0,'cao_spine',false)
						self.caodiTime = 0

						tmpPos[i] = pos
					else
						table.remove(self.mapCaoDiList, randIdx)
					end
				end
			end
		else
			self.caodiTime = self.caodiTime + 1
		end
	end
	
end

function EventMapPanel:retEventMapGetInfo()
	self:onShow();
end

function EventMapPanel:updateRolePos( pos )
	-- body
	local rt = false
	if pos and self.mapItemLists[pos] then
		rt = self.mapItemLists[pos]:updateState();
	end

	local curPos = model.EventModel:getCurPos()
	if self.mapItemLists[curPos] then

		if self.lastPos ~= curPos then
			self.lastPos = curPos
			if rt == false then
				self.roleNode:setPosition( self.mapItemLists[curPos]:getPosition() )
				self.roleNode:setLocalZOrder(50-curPos)
			else
				self.roleNode:runAction( cc.Sequence:create( cc.DelayTime:create(0.6) ,cc.CallFunc:create(function()
			        self.roleNode:setPosition( self.mapItemLists[curPos]:getPosition() )
			        self.roleNode:setLocalZOrder(50-curPos)
			    end)) )
			end

			if model.EventModel:isEnd() then
				self.exitAni:setAnimation(0, "open1", false)
				local tier = model.EventModel:getTier()
				if tier ~= 1 and tier~=2 then
					UIManager:push('EventResultPanel');
				end
			end

			if pos then
				local ptValue = model.EventModel:getMovePoints()
				if ptValue == 15 or ptValue == 10 or ptValue == 5 then
					Prompt:Show(string.format( LANG.UI_MAZE_44, ptValue ))
				end
			end
		end
	end

	self.ptLabel:setString(LANG.UI_MAZE_7..model.EventModel:getMovePoints())
end


function EventMapPanel:retReset( msg )
	self:resetEvent(msg);
end

function EventMapPanel:retEventMapItemFight( msg )
end

function EventMapPanel:showKeyAct(  )
	local posIndex = 1--model.EventModel:getSelectPos();
	local value = model.EventModel.EVENT_MAP_ITEM_TYPES.EVENT_MAP_ITEM_TYPE_2;
	-- local itemValue = model.EventModel:getSelectValue() ;
	-- Prompt:Show('======= 触发开门,播放开门动画.. posIndex = '..tostring(posIndex)..'...itemValue = '..tostring(itemValue));

	self:showEventItemAct(posIndex,value,nil);

end

function EventMapPanel:showThingAct( posIndex,value,aimPid )
	-- Prompt:Show('======= 触发thing,执行thing事件播放thing动画.. pos = '..tostring(posIndex));
	local itemValue = 1--model.EventModel:getSelectValue() ;
	self:showEventItemAct(posIndex,value,itemValue,aimPid);

end

function EventMapPanel:resetEvent( msg )
	local newFlag = msg.newFlag;
	UIManager:push('RewardPanel' , msg);

	if  newFlag == 1  then
		-- 通关当前层 
		-- Prompt:Show(LANG.UI_EVENT_18);
		self:onShow(true);
	elseif newFlag == 2 then 
		-- 重置当前关卡
		self:onChangeToEventMain();
	elseif newFlag == 3 then 
		-- 通关当前难度 
		-- 返回 event主界面
		self:showWinPanel();

	end
end

function EventMapPanel:playBuffParticle(idx)

	-- local wpt = cc.p(-200 + 200 * (idx-1) + StageSize.width/2  , -3+StageSize.height/2)

	local wpt = self.mapItemLists[idx]:getParent():convertToWorldSpace( cc.p(self.mapItemLists[idx]:getPosition()) ) 

	local startPos = self.root:convertToNodeSpace(wpt);
	local endPos = self.root:convertToNodeSpace( cc.p(StageSize.width/2+240, 80) );

	self.buffParticle:setPosition(startPos)
	self.buffParticle:setVisible(true)

	local x,y = startPos.x, startPos.y;
	local aimX, aimY = endPos.x, endPos.y;
	local bezier2 ={
		cc.p((aimX-x)/3+x-100,	(aimY-y)/3+y),
		cc.p((aimX-x)/3*2+x-150,(aimY-y)/3*2+y),
	    cc.p(aimX,	aimY)
	}

	self.buffParticle:runAction(cc.Sequence:create(cc.BezierTo:create(2, bezier2), cc.CallFunc:create(function()
		self.buffParticle:setVisible(false)
		self.buffFlySp:setAnimation(0, "fly_spine", false)
	end)));

end


function EventMapPanel:showWinPanel(  )
	self.winPanel:setVisible(true);

	self.particleWin:resetSystem()

end

function EventMapPanel:showMapInfo(  )

end

function EventMapPanel:showMapItemLists(  )
	if self.tier == model.EventModel:getTier() then
		return
	end
	self.tier = model.EventModel:getTier()
	self.mapItemLists = {};
	self.mapCaoDiList = {}
	self.mapItemsNode:removeAllChildren();

	self.roleNode:addTo(self.mapItemsNode);

	local mapInfo = model.EventModel:getMapInfo()

	local tmpTier = self.tier < 3 and self.tier or 3;
	local tmpMsg = ''
	if self.tier == 3 then
		tmpMsg = LANG.UI_MAZE_5
	elseif self.tier == 4 then
		tmpMsg = LANG.UI_MAZE_6
	end
	self.title2Label:setString(string.format(LANG.UI_MAZE_4, tostring(tmpTier))..tmpMsg);

	local row = model.EventModel.EVENT_MAP_ROW;
	local col = model.EventModel.EVENT_MAP_COL;
	local delay = 0;
	for posI=1,row do
		for posJ=1,col do
			local pos = (posI-1)*col + posJ;
			local item
			local tileType = mapInfo[pos].type
			if tileType == 1 then
				item = require("app.template.EventExitMapItem"):create(pos);
			elseif tileType == 2 then
				item = require("app.template.EventExitMapItem"):create(pos);
			elseif tileType == 3 then--复活
				item = require("app.template.EventRebornMapItem"):create(pos);
			elseif tileType == 4 then--泉水
				item = require("app.template.EventHealthMapItem"):create(pos);
			elseif tileType == 5 then--炸弹
				item = require("app.template.EventBombMapItem"):create(pos);
			elseif tileType == 6 then--雇佣
				item = require("app.template.EventNpcMapItem"):create(pos);
			elseif tileType == 7 then--商人
				item = require("app.template.EventShopMapItem"):create(pos);
			elseif tileType == 8 then--宝箱
				item = require("app.template.EventBoxMapItem"):create(pos);
			elseif tileType == 9 then--宝藏
				item = require("app.template.EventMonster9MapItem"):create(pos);
			elseif tileType == 10 then--boss
				item = require("app.template.EventMonster10MapItem"):create(pos);
			elseif tileType == 11 then--怪
				item = require("app.template.EventMonster11MapItem"):create(pos);
			else
				item = require("app.template.EventBaseMapItem"):create(pos);
			end
			item:addTo(self.mapItemsNode);
			item:setLocalZOrder(50-pos)
			item:setPosition(cc.p(-220 +110*(posJ-1),-396+60 + 110*(posI - 1)));
			-- item:updateItemValue(pos,1);
			-- delay = (15 + math.abs(3-posJ)*3 +  (posI-1)*2)/15
			-- item:runChangeTierAct(delay,bShowAction);
			self.mapItemLists[pos] = item;
			if item:isCaoDi() then
				table.insert(self.mapCaoDiList, pos)
			end
		end
	end

end


function EventMapPanel:onEnter()

	if model.EventModel:getTier() == 0 or model.EventModel:isNeedRest() then
		self.closeYun:setVisible(true)
		self.closeYun:setAnimation(0, "stop", false)
		self:showChangeTierAct(1, 1)
		-- model.EventModel:reqEventMapInfo(1);
	else
		self:onShow();
	end

	model.EventModel:addRobotToCardModel()
	model.TeamModel:updateEventRestraint()
	self.root:setVisible(true);

	
	self.roleNode:removeAllChildren()
	local roleId,avatarId = model.UserModel:getKanbanRoleAndAvatar()
	local res_id = ResTable:getValue('role', roleId, 'resource_id')
	local modelPath = ResTable:getValue('role_resource', res_id, 'model')
	local armature, db_id = DragonBonesManager:GetArmature(modelPath)
	armature:setScale(0.35)
	armature:addTo(self.roleNode)
	armature:getAnimation():play(AnimationType.idle, -1);

	self.caodiTime = 0
	self.resetTime = model.EventModel:getResetTime();
	TimeManager:registUpdateEvent('EventMapPanel',handler(self,self.updateTime),true);

end

function EventMapPanel:onShow()

	self:showMapInfo();
	self:showMapItemLists()
	self:updateRolePos();

	if model.EventModel:isEnd() then
		self.exitAni:setAnimation(0, "open2", true)
	else
		self.exitAni:setAnimation(0, "close", true)
	end
	
end

function EventMapPanel:retSelectBuff(msg)
	self:playBuffParticle(msg)
	UIManager:pop('EventBuffSelectPanel');
end

function EventMapPanel:onChangeToEventMain(  )
	self.winPanel:setVisible(false);
	self.particleWin:stopSystem();

	Prompt:Show(LANG.UI_EVENT_19);
	EventManager:FireEvent(EventType.UIEventRewardUpdate)
	EventManager:FireEvent(EventType.UIEventChallengeUpdate)
	UIManager:pop('EventMapPanel');
	EventManager:FireEvent(EventType.NetworkEventInitInfo);
end

function EventMapPanel:onResetEventHard()
	MessageBox:create(LANG.UI_EVENT_20 , MessageBoxType.CANCEL_OK,function()
		model.EventModel:reqEventReset();
	end);
end
function EventMapPanel:onShowEventTeamInfo()
	UIManager:push('EventRoleStatePanel');
end
function EventMapPanel:onShowEventBuff()
	UIManager:push('EventBuffPanel');
end
function EventMapPanel:onShowItemPanel()
	-- UIManager:push('EventItemPanel');
	UIManager:push('EventItemUsePanel', model.EventModel.EVENT_ITEMID_1)
end
function EventMapPanel:openMazeBattlePass()
end

function EventMapPanel:onClickBack()
	EventManager:FireEvent(EventType.UIEventRewardUpdate)
	EventManager:FireEvent(EventType.UIEventChallengeUpdate)
	UIManager:pop('EventMapPanel');
end

function EventMapPanel:onLeave()
	model.EventModel:removeRobotFromCardModel()
	self.root:setVisible(false);
	TimeManager:registUpdateEvent('EventMapPanel',nil,true);
end

function EventMapPanel:gotoNextTire()
	if model.EventModel:isEnd() then
		local tier = model.EventModel:getTier()
		if tier == 1 then
			self:showChangeTierAct(0, tier+1)
			-- model.EventModel:reqEventMapInfo( tier+1 )
		elseif tier == 2 then
			if model.EventModel:getHard() < 4 then
				self:showChangeTierAct(0, tier+1)
				-- model.EventModel:reqEventMapInfo( tier+1 )
			else
				UIManager:push('EventTireSelPanel');
			end
		else
			-- Prompt:Show(LANG.UI_MAZE_17)
			UIManager:push('EventResultPanel');
		end
	else
		Prompt:Show(LANG.UI_MAZE_35)
	end
end

function EventMapPanel:showChangeTierAct( delay, tier )
	self.skYun:setVisible(true);
	self.skYun:setAnimation(0, "close", false);

	self.root:runAction( cc.Sequence:create( 
		cc.DelayTime:create(1) ,
		cc.CallFunc:create(function()
	        model.EventModel:reqEventMapInfo( tier )
	    end),
		cc.DelayTime:create(0.5+delay) ,
		cc.CallFunc:create(function()
	        self.skYun:setAnimation(0, "open", false);
	        self.closeYun:setVisible(false)
	    end)) 
    )

end

function EventMapPanel:shakeRoot()
	local ox,oy = self.root:getPosition()
    local act = cc.Sequence:create( cc.DelayTime:create(0.03) ,cc.CallFunc:create(function()
        self.root:setPosition( ox+math.random()*20-10, oy+math.random()*20-10)
    end))
    local repeqt = cc.Repeat:create(act ,10)

    self.root:runAction( cc.Sequence:create( repeqt ,cc.CallFunc:create(function()
        self.root:setPosition(ox, oy)
    end)) )
end














---- --------------------run act -------------------------

function EventMapPanel:showPtCostAct( pos   )
	-- local myIndex = pos or 1;
	-- local myRow = math.floor((myIndex-1)/model.EventModel.EVENT_MAP_COL );
	-- local myCol = (myIndex-1)%model.EventModel.EVENT_MAP_COL + 1;
	-- local beginPos = cc.p(-201 +100*(myCol-1),254 - 100*myRow)

	-- self.ptCostNode:setVisible(true);
	-- self.ptCostNode:setOpacity(255);
	-- self.ptCostNode:setPosition(beginPos);
	-- self.ptCostNode:runAction(cc.Spawn:create(cc.MoveBy:create(1,cc.p(0,80)),cc.FadeOut:create(1) ) )
end

function EventMapPanel:showEventItemAct( pos,value,itemKeyNo,aimPid )

	local myIndex = pos or 1;
	local myRow = math.floor((myIndex-1)/model.EventModel.EVENT_MAP_COL );
	local myCol = (myIndex-1)%model.EventModel.EVENT_MAP_COL + 1;
	local beginPos = cc.p(-201 +100*(myCol-1),254 - 100*myRow)

	-- print('myIndex = ',myIndex,'..row =-= ',myRow,',col = ',myCol, 'beginPos ===== ',beginPos.x,',',beginPos.y)
	-- print('pos = ',pos,'..value =-= ',value)

	if value == model.EventModel.EVENT_MAP_ITEM_TYPES.EVENT_MAP_ITEM_TYPE_2 then 
		local doorOpen = model.EventModel:getEventMapDoorState();
		-- print('doorOpen - ',doorOpen);
		if doorOpen == 2 then
			-- check key
			self:runKeyAct(beginPos);
			if self.mapItemLists[pos] then
				self.mapItemLists[pos]:runItemAct();
			end
		end

	elseif value == model.EventModel.EVENT_MAP_ITEM_TYPES.EVENT_MAP_ITEM_TYPE_4 then 
		-- show event thing 1 
		self:runThing2Act(beginPos,pos,itemKeyNo);
		if self.mapItemLists[pos] then
			self.mapItemLists[pos]:runItemAct();
		end
	elseif value == model.EventModel.EVENT_MAP_ITEM_TYPES.EVENT_MAP_ITEM_TYPE_5 then 
		-- do thing 
		self:runThing1Act(beginPos,pos,itemKeyNo);
		if self.mapItemLists[pos] then
			self.mapItemLists[pos]:runItemAct();
		end
	elseif value == model.EventModel.EVENT_MAP_ITEM_TYPES.EVENT_MAP_ITEM_TYPE_9 then 
		self:runThing3Act(beginPos,aimPid);
		if self.mapItemLists[pos] then
			self.mapItemLists[pos]:runItemAct();
		end
	end


end

function EventMapPanel:showKeyPaticle( beginPos )

	if self.particleKey == nil then
		self.particleKey = cc.ParticleSystemQuad:create("Effect/particle/yaoshi1.plist")
		self.particleKey:addTo(self.root);
	end
	self.particleKey:setPosition(beginPos)
	self.particleKey:resetSystem();
end

function EventMapPanel:runKeyAct( beginPos)
	self:showKeyPaticle(beginPos);
	if self.particleKey == nil then
		GlobalFunctions:showErrorPromt('error : no particle named [yaoshi1]  found ');
		return 
	end
	-- print('============runKeyAct  11111 =============')
	local doorIndex = model.EventModel:getEventMapDoorPos()
	local row = math.floor((doorIndex-1)/model.EventModel.EVENT_MAP_COL );
	local col = (doorIndex-1)%model.EventModel.EVENT_MAP_COL + 1;
	local endPos = cc.p(-201 +100*(col-1),254 - 100*row)

	-- print('endPos === ',row,'..',col,'.... pos = ',endPos.x,'..',endPos.y);
	 --贝塞尔曲线：控制点位置p1,p2 
	local bezierP1,bezierP2 = cc.p(0,0),cc.p(0,0);
	if endPos.x > beginPos.x  then
		bezierP1.x = endPos.x + 100;
		bezierP2.x = endPos.x + 50;
	else
		bezierP1.x = endPos.x - 100;
		bezierP2.x = endPos.x - 50;
	end
	bezierP1.y = beginPos.y + (endPos.y - beginPos.y)/4
	bezierP2.y = beginPos.y + (endPos.y - beginPos.y)/2
	local bezierAct = cc.BezierTo:create(24/24 ,{bezierP1,bezierP2,endPos});

	local callfunc2 = cc.CallFunc:create(function (  )
		self:runKeyAct2(endPos);
		if self.mapItemLists[doorIndex] then
			self.mapItemLists[doorIndex]:runItemAct();
		end
	end)
	self.particleKey:runAction(cc.Sequence:create(bezierAct,callfunc2));
end

function EventMapPanel:runKeyAct2( endPos )
	self.particleKey:stopSystem();
	-- print('============runKeyAct  2222 =============')

	local particle2 = cc.ParticleSystemQuad:create("Effect/particle/yaoshi2.plist")
		:addTo(self.root)
		:setPosition(endPos)
		:setAutoRemoveOnFinish(true)

end

function EventMapPanel:showRoleDamageAct( thingType, itemValue )

	local thingData = ResTable:getRowData('event_thing',itemValue);
	if thingData == nil then
		print('error============= thing data == null ,posIndex = ',itemValue);
		return ;
		-- thingData = { value = 0};
	end
	local showNum = thingData.value *100;
	local showMsg = '';
	local fnt = 'font/num_add.fnt';
	if thingType == 1 then
		showMsg = string.format( LANG.UI_EVENT_26,tostring(showNum) );
		showNum = '-' .. showNum..'%';
		fnt = 'font/num_damage.fnt';
	else
		showMsg = string.format( LANG.UI_EVENT_27,tostring(showNum) );
		showNum = '+' .. showNum..'%';
		fnt = 'font/num_add.fnt';
	end
	local thingActNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(cc.p(105,-477))
		-- :setScale(2.5)
	thingActNode:setCascadeOpacityEnabled(true);


	local numLabel = cc.Label:createWithBMFont(fnt, showNum);
	numLabel:addTo(thingActNode);

	thingActNode:runAction(cc.Sequence:create(
		cc.Spawn:create(
			cc.MoveBy:create(0.1, cc.p(0, 50)),
			cc.ScaleTo:create(0.1, 1.5)
			),
		cc.DelayTime:create(0.3),
		cc.FadeOut:create(0.3),
		cc.CallFunc:create(function()
			thingActNode:removeFromParent();
			Prompt:Show(showMsg)

		end)
		))


end

function EventMapPanel:runThingRoleAct( state )
	-- body
	-- print(' thing role act ===== 4444 ',state);

	local act1 = cc.RotateTo:create(0.05,30);
	local act2 = cc.RotateTo:create(0.1,-30);
	local act3 = cc.RotateTo:create(0.05,0);
	local act = cc.Sequence:create(act1,act2,act3);
	if state == true  then
		self.btnTeamRole:runAction(act);
	else
		self.btnTeamRole:stopAllActions();
		self.btnTeamRole:setRotation(0);
	end
end

function EventMapPanel:runThing1Act( beginPos,index,itemValue)
	-- 触发 thing1 事件
	local endPos = cc.p(self.btnTeamRole:getPosition());

	local effectFunc3 = function (  )
		self.particleKey:stopSystem();
		self:showThing1Spine( endPos, 'end',function (  )
			self:runThingRoleAct(false);
		end);
		self:showRoleDamageAct(1,itemValue);
		self:runThingRoleAct(true);
	end

	-- local effect2Callfunc = cc.CallFunc:create()

	local effectFunc2 = function (  )
		self:showKeyPaticle(beginPos);
		local bezierP1,bezierP2 = cc.p(0,0),cc.p(0,0);
		if endPos.x > beginPos.x  then
			bezierP1.x = endPos.x + 100;
			bezierP2.x = endPos.x + 50;
		else
			bezierP1.x = endPos.x - 100;
			bezierP2.x = endPos.x - 50;
		end
		bezierP1.y = beginPos.y + (endPos.y - beginPos.y)/4
		bezierP2.y = beginPos.y + (endPos.y - beginPos.y)/2
		local bezierAct = cc.BezierTo:create(24/24 ,{bezierP1,bezierP2,endPos});
		self.particleKey:runAction(cc.Sequence:create(bezierAct,cc.CallFunc:create(effectFunc3)));
	end
	self:showThing1Spine( beginPos, 'start',effectFunc2);
end

function EventMapPanel:showThing1Spine( beginPos, aniName,callback )

	if self.thing1Spine == nil then
		local spine_res = 'thing_1'
		self.thing1Spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.thing1Spine:addTo(self.root)
		self.thing1Spine:registerSpineEventHandler(function (event)
			if self.spineCallback then
				self.spineCallback();
				self.spineCallback = nil;
			end
		end, sp.EventType.ANIMATION_COMPLETE)
	end
	self.spineCallback = callback;
	self.thing1Spine:setPosition(beginPos);
	self.thing1Spine:setAnimation(0,aniName,false);
end


function EventMapPanel:runThing2Act( beginPos,index,itemValue)
	-- 触发 thing1 事件
	print('===========trgger thing1 event !!!');
	local endPos = cc.p(self.btnTeamRole:getPosition());

	local effectFunc3 = function (  )
		self.particleKey:stopSystem();
		self:showThing2Spine( endPos, 'end',function (  )
			self:runThingRoleAct(false);
		end);
		self:showRoleDamageAct(2,itemValue);
		self:runThingRoleAct(true);
	end

	local effectFunc2 = function (  )
		-- print('========effectFunc2 ==============');
		self:showKeyPaticle(beginPos);
		local bezierP1,bezierP2 = cc.p(0,0),cc.p(0,0);
		if endPos.x > beginPos.x  then
			bezierP1.x = endPos.x + 100;
			bezierP2.x = endPos.x + 50;
		else
			bezierP1.x = endPos.x - 100;
			bezierP2.x = endPos.x - 50;
		end
		bezierP1.y = beginPos.y + (endPos.y - beginPos.y)/4
		bezierP2.y = beginPos.y + (endPos.y - beginPos.y)/2
		local bezierAct = cc.BezierTo:create(24/24 ,{bezierP1,bezierP2,endPos});
		self.particleKey:runAction(cc.Sequence:create(bezierAct,cc.CallFunc:create(effectFunc3)));
	end
	self:showThing2Spine( beginPos, 'start',effectFunc2);
end

function EventMapPanel:showThing2Spine( beginPos, aniName,callback )

	if self.thing2Spine == nil then
		local spine_res = 'thing_2'
		self.thing2Spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.thing2Spine:addTo(self.root)
		self.thing2Spine:registerSpineEventHandler(function (event)
			if self.spineCallback then
				self.spineCallback();
				self.spineCallback = nil;
			end
		end, sp.EventType.ANIMATION_COMPLETE)

	end
	self.spineCallback = callback;
	self.thing2Spine:setPosition(beginPos);
	self.thing2Spine:setAnimation(0,aniName,false);
end

function EventMapPanel:runThing3Act( beginPos, resurgencePid)

	UIManager:pop('HeaderPanel');

	if self.resurgenceNode == nil   then
		self.resurgenceNode = cc.Node:create()
		:addTo(self.root)

		local bg = ccui.Layout:create();
		bg:setTouchEnabled(true);
		bg:setContentSize(StageSize);
		bg:setLocalZOrder(-10);
		bg:setAnchorPoint(cc.p(0.5,0.5));
		bg:setPosition(0, 0);
		bg:addTo(self.resurgenceNode);

		local blackBg = cc.LayerColor:create(cc.c4b(0, 0, 0, 160), StageSize.width, StageSize.height);
		blackBg:setPosition(-StageSize.width/2, -StageSize.height/2);
		blackBg:addTo(self.resurgenceNode);
	end
	self.resurgenceNode:setVisible(true)

	local card = model.CardModel:GetCard(resurgencePid)
	
	if self.resurgenceCard == nil then
		self.resurgenceCard = template.ItemTemplate:create()
		self.resurgenceCard:addTo(self.resurgenceNode);
	end
	self.resurgenceCard:Init(ROLE_TYPE , card.no , card.star_lv, 0, resurgencePid);
	self.resurgenceCard:setVisible(false);

	if self.particleResurgence == nil then
		self.particleResurgence = cc.ParticleSystemQuad:create("Effect/particle/shengxing1.plist")
		self.particleResurgence:addTo(self.resurgenceNode);
	end
	self.particleResurgence:resetSystem()

	local spine_res = 'thing3'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		if self.resurgenceSpine == nil  then
			self.resurgenceSpine = spine_res;
			self.resurgenceSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
			self.resurgenceSpine:addTo(self.resurgenceNode)
			self.resurgenceSpine:registerSpineEventHandler(function ( event )
				local eventName = event.eventData.name;
				if eventName == 'appear' then
					self.particleResurgence:stopSystem()
					self.particleResurgence:removeFromParent();
					self.particleResurgence = nil ;

					self.resurgenceCard:setVisible(true);
				end
			end,sp.EventType.ANIMATION_EVENT)
			self.resurgenceSpine:registerSpineEventHandler(function (event)
				self.resurgenceCard:runAction(cc.Sequence:create(
					cc.DelayTime:create(1),
					cc.FadeOut:create(0.5),
					cc.CallFunc:create(function (  )
						self.resurgenceCard:removeFromParent()
						self.resurgenceCard = nil 
	
						self.resurgenceNode:setVisible(false)
						UIManager:push('HeaderPanel','EventMapPanel',UIManager:getPanelHandler('EventMapPanel'));


					end)
				))
			end,sp.EventType.ANIMATION_COMPLETE)
		end
		self.resurgenceSpine:setAnimation(0, 'revive', false)

	end


end

return EventMapPanel;
