local EventNpcSelPanel = class("EventNpcSelPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/setup/'
local MessageBoxType = Enum:getTable('MessageBoxType')

function EventNpcSelPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'SettingPanel');

	self.useCCreator = false;
	self.isPopup = true;
end


function EventNpcSelPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()

end

function EventNpcSelPanel:initPanels()

	local itemNode = cc.Node:create();
	itemNode:addTo(self.root)

	self.selSpr = cc.Scale9Sprite:create('image/commonImg/select_1_20_20_93_117.png');
	self.selSpr:addTo(itemNode);
	self.selSpr:setContentSize(cc.size(164 , 348));
	self.selSpr:setVisible(false)

	self.itemList = {}
	for i=1,4 do
		local cellNode = cc.Node:create()
		cellNode:addTo(itemNode)
		cellNode:setPosition(-77+(i-2)*154, 0)

		local cardNode = cc.Node:create()
		:addTo(cellNode)
		:setScale(0.8)
		local cardItem = template.CardTemplate:create(cardNode);
		-- cardItem:InitByNo(32+i)

		local costNode = cc.Node:create()
		:addTo(cellNode)
		:setPosition(0, -148)

		local costIcon = cc.Sprite:create('image/head/header_bg_9.png')
		:addTo(costNode)
		:setPosition(-40, 0)

		local costLabel = cc.Label:createWithTTF('99999' , fonts.FONT_SIYUAN , 18)
		:addTo(costNode)
		:enableOutline(cc.c3b(120 , 120 , 120) , 1)
		-- :setAnchorPoint(cc.p(0,0.5))
		:setPosition(20,0)

		local costBtn = ccui.Button:create('image/commonImg/com_bg.png');
		costBtn:addTo(cellNode);
		costBtn:setPosition(0, -30);
		costBtn:setScale9Enabled(true)
		costBtn:setContentSize(cc.size(160 , 400))
		costBtn:setOpacity(0)
		costBtn:setTag(i)
		costBtn:addClickEventListener(handler(self, self.cellItemTouch));

		table.insert( self.itemList, {cellNode=cellNode, cardItem=cardItem, costLabel=costLabel, costBtn=costBtn} )

	end

	local okLabel = cc.Label:createWithTTF(LANG.COMMON_OK , fonts.FONT_SIYUAN , 20);
	okLabel:enableOutline(cc.c3b(145 , 40 , 40) , 2);

	self.okBtn = ccui.Button:create('image/event/eventBuff/buff_btn.png' , 'image/event/eventBuff/buff_btn.png');
	self.okBtn:addTo(self.root);
	self.okBtn:setPosition(118, -260);
	self.okBtn:addClickEventListener(handler(self, self.clickOkBtn));
	self.okBtn:setTitleLabel(okLabel);


	local titleBg = cc.Sprite:create('image/event/eventBuff/title_bg.png')
	titleBg:addTo(self.root)
	titleBg:setPosition(0, 208)

	local tlabel = cc.Label:createWithTTF(LANG.UI_MAZE_29 ,  fonts.FONT_SIYUAN , 20)
    tlabel:enableOutline(cc.c4b(10 , 60 , 150 , 255),2);
    tlabel:addTo(self.root)
    tlabel:setPosition(0, 208)
	
end

function EventNpcSelPanel:onEnter(tid, bclick)

	self:addSwallowCallback(handler(self, self.onClickBack));
	self.tid = tid;
	self.bclick = bclick;
	EventManager:RegisterEvent(EventType.UIUpdateEventRobot , handler(self, self.updateEventRobot));
	for i = 1 , 4 do
		self.itemList[i].cellNode:setVisible(false)
	end

	self.robots = model.EventModel:getFightRobotsById(self.tid);
	if self.robots == nil then
	else
		self:updateEventRobot();
	end

	self.selectIndex = -1;
	self.selSpr:setVisible(false)
	self.okBtn:setVisible(false)

	self.root:setVisible(true);
end

function EventNpcSelPanel:updateEventRobot()
	if self.robots == nil then
		self.robots = model.EventModel:getFightRobotsById(self.tid);
	end

	for i = 1 , 4 do
		self.itemList[i].cellNode:setVisible(true)
		if self.robots[i] then
			-- self.itemList[i].cardItem:InitByNo( self.robots[i][1] );
			local quality = 3
			if self.robots[i][6]==nil or self.robots[i][6] == 0 then
				quality = ResTable:getValue('role', self.robots[i][1],'quality');
				quality = quality or 3;
			else
				quality = self.robots[i][6]
			end
			self.itemList[i].cardItem:setData(self.robots[i][1] , self.robots[i][4] , quality,self.robots[i][2])
			self.itemList[i].cardItem:setNameInfo('')

			local data = {no=self.robots[i][1]}
			local pro = CardProManager:genPro(data , self.robots[i][4] , self.robots[i][3] , quality,self.robots[i][2])

			self.itemList[i].costLabel:setString( tostring(pro.fp) )
		end
	end

end


function EventNpcSelPanel:onClickBack()
	UIManager:pop('EventNpcSelPanel');
end

function EventNpcSelPanel:onLeave()
	EventManager:UnregisterEvent(EventType.UIUpdateEventRobot)
	self.root:setVisible(false);
end

function EventNpcSelPanel:cellItemTouch(sender , event)
	local index = sender:getTag();
	self.selectIndex = index;

	for k,v in pairs(self.itemList) do
		if self.selectIndex == k then
			local x,y = v.cellNode:getPosition();
			self.selSpr:setPosition( x,y-0 )
			self.selSpr:setVisible(true)

			self.okBtn:setPosition( x,y-220 )
			self.okBtn:setVisible(true)
		end
	end

end

function EventNpcSelPanel:clickOkBtn()
	if self.bclick==false then
		Prompt:Show(LANG.UI_EVENT_CLICK);
		return;
	end
	if self.selectIndex ~= -1 then
		UIManager:pop('EventNpcSelPanel');

		model.EventModel:reqEventMapEmployRobot(self.robots[self.selectIndex][1], self.tid);
		
		-- self.itemList[self.selectIndex].costBtn:setEnabled(false)
		-- self.itemList[self.selectIndex].costBtn:setOpacity(255);
		self.selectIndex = -1;
	else
		UIManager:pop('EventNpcSelPanel');
	end
end

return EventNpcSelPanel;
