--[[
local EventRewardPanel = class("EventRewardPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local scheduler = cc.Director:getInstance():getScheduler();
local MessageBoxType = Enum:getTable('MessageBoxType')


function EventRewardPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'EventRewardPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function EventRewardPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function EventRewardPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:setBackGroundImage('image/common/reward_bg1.png');
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(636 , 1013));
	bg:setAnchorPoint(cc.p(0.5,0.5));
	bg:setPosition(1, -4);
	bg:addTo(self.root);

	bg = cc.Scale9Sprite:create('image/common/common_bg2.png'); --587 , 143
    bg:setCapInsets(CCRectMake(100,40,300,80));
	bg:setContentSize(cc.size(619, 861));
	bg:setPosition(cc.p(-1,24));
	bg:addTo(self.root);
	
	local titleLabel = cc.Label:createWithTTF(LANG.UI_EVENT_2 , fonts.FONT_SIYUAN , 20);
	titleLabel:addTo(self.root);
	titleLabel:setPosition(0 , 488);
	titleLabel:setTextColor(cc.c3b(255 , 255 , 255));

	titleLabel = cc.Label:createWithTTF('- PRESENT BOX -' , fonts.FONT_SIYUAN , 9);
	titleLabel:addTo(self.root);
	titleLabel:setPosition(0 , 469);
	titleLabel:setTextColor(cc.c3b(0 , 142 , 219));

	self.rewardView = ccui.ScrollView:create();
	self.rewardView:setTouchEnabled(true);
	self.rewardView:setBounceEnabled(true);
	self.rewardView:setDirection(ccui.ScrollViewDir.vertical);
	self.rewardView:setContentSize(cc.size(619, 841));
	self.rewardView:setAnchorPoint(cc.p(0.5, 0.5));
	self.rewardView:setPosition(-1, 24);
	self.rewardView:setScrollBarEnabled(false);
	self.rewardView:addTo(self.root);

	self.cancelBtn = ccui.Button:create('image/commonImg/btn_bg2.png' , 'image/commonImg/btn_bg2.png');
	self.cancelBtn:addTo(self.root);
	self.cancelBtn:setPosition(-117 , -451);
	self.cancelBtn:addClickEventListener(handler(self , self.onClickBack));

	local cancelLabel = cc.Label:createWithTTF(LANG.UI_ROUND_14 , fonts.FONT_SIYUAN , 20);
	cancelLabel:addTo(self.cancelBtn);
	cancelLabel:setPosition(self.cancelBtn:getContentSize().width/2 , self.cancelBtn:getContentSize().height/2);
	cancelLabel:setTextColor(cc.c3b(60 , 60 , 60));
	cancelLabel:enableOutline(cc.c4b(255 , 255 , 255 , 255) , 2);

	self.oneKeyBtn = ccui.Button:create('image/commonImg/btn_bg1.png' , 'image/commonImg/btn_bg1.png');
	self.oneKeyBtn:addTo(self.root);
	self.oneKeyBtn:setPosition(117 , -451);
	self.oneKeyBtn:addClickEventListener(handler(self , self.oneKeyBtnClick));

	local oneKeyLabel = cc.Label:createWithTTF(LANG.UI_ROUND_15 , fonts.FONT_SIYUAN , 20);
	oneKeyLabel:addTo(self.oneKeyBtn);
	oneKeyLabel:setPosition(self.oneKeyBtn:getContentSize().width/2 , self.oneKeyBtn:getContentSize().height/2);
	oneKeyLabel:setTextColor(cc.c3b(255 , 255 , 255));
	oneKeyLabel:enableOutline(cc.c4b(19 , 93 , 188 , 255) , 2);

	EventManager:RegisterEvent(EventType.NetwordEventGetReward, handler(self, self.retReward));
end

function EventRewardPanel:onEnter()
	self:updateData();
	self:addSwallowCallback(handler(self, self.onClickBack));
	self.root:setVisible(true);
end

function EventRewardPanel:updateData()
	self.rewardNum = 0;
	self.rewards = {};
	for _k , reward in pairs(ResTable:getTable('event_reward').contain) do
		table.insert(self.rewards , reward);
		self.rewardNum = self.rewardNum + 1;
	end
	
	if self.rewardNum > 1 then
		table.sort(self.rewards , function(first , second)
			local firstGetReward = model.EventModel:getEventRewardState(first.id);
			local secondGetReward = model.EventModel:getEventRewardState(second.id);
			-- print(fristGetReward , secondGetReward);
			if firstGetReward == nil and secondGetReward ~= nil then
				return true;
			end
			if firstGetReward ~= nil and secondGetReward == nil then
				return false;
			end
			return first.id < second.id;
		end);
	end

	self.totalLoadedNum = 0;
	self.rewardView:removeAllChildren();
	self.rewardView:jumpToTop();

	self.maxHeight = self.rewardNum * 190;
	self.maxHeight = self.maxHeight < 841 and 841 or self.maxHeight;

	self.rewardView:setInnerContainerSize(cc.size(619, self.maxHeight));

	self.isLoadData = false;
	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.loadData), 0, false);
end

function EventRewardPanel:loadData()
	self.isLoadData = true;
	local startIndex = self.totalLoadedNum + 1;

	self.totalLoadedNum = self.totalLoadedNum + 5;
	self.totalLoadedNum = self.totalLoadedNum > self.rewardNum and self.rewardNum or self.totalLoadedNum;

	if startIndex > self.totalLoadedNum then
		if self.m_scheduleHandler then 
			scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
			self.m_scheduleHandler = nil 
		end
		self.isLoadData = false;
		return;
	end

	for i = startIndex , self.totalLoadedNum do
		local reward = self.rewards[i];
		local roundRewardTemplate = template.RoundTargetRewardTemplate:create(self.rewardView);
		roundRewardTemplate:InitEventReward(reward);
		roundRewardTemplate:setTag(reward.id);
		roundRewardTemplate:setPosition(309.5 , self.maxHeight - 86.5 - 190 * (i - 1));
		roundRewardTemplate:addClickEventListener(handler(self , self.rewardClick));
	end
end

function EventRewardPanel:oneKeyBtnClick()
	self:rewardClick(0 , 1);
end

function EventRewardPanel:rewardClick(id , flag)

	if self.isLoadData then
		return;
	end

	if flag == 1 then
		local msg = {};
		msg.id = id;
		msg.event_id = model.EventModel:getEventId()
		Network.NetworkManager:Send(Network.NetworkCmdType.req_event_get_reward , msg);
	else
		Prompt:Show(LANG.UI_ROUND_16);
		self:onClickBack();
	end
end

function EventRewardPanel:retReward(msg)

	UIManager:push('RewardPanel' , msg);

	if msg.id and #msg.id > 0 then
		self:updateData();
	else
		Prompt:Show(LANG.UI_ROUND_17);
	end
end

function EventRewardPanel:onClickBack()
	EventManager:FireEvent(EventType.UIEventRewardUpdate)
	UIManager:pop('EventRewardPanel');
end

function EventRewardPanel:onLeave()
	if self.m_scheduleHandler then
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end

	self.isLoadData = false;

	self.root:setVisible(false);
end

return EventRewardPanel;
]]