local EventRoundInfoPanel = class("EventRoundInfoPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/fight/roundInfo/';
local fonts = Enum:getTable("SystemFont")
local FightType = Enum:getTable('FightType');
local TeamType = Enum:getTable('TeamType');
local restrainType = Enum:getTable("FightRestraintType")
local BP_PointItem = 10028;

local bossPos = {
	[1] = cc.p(-196, 127),
	[2] = cc.p(-65, 127),
	[3] = cc.p(65, 127),
	[4] = cc.p(196, 127),
}

local scheduler = cc.Director:getInstance():getScheduler()

function EventRoundInfoPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'EventRoundInfoPanel');

	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;
	self.isPopup = true;
end

function EventRoundInfoPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.fightType = -1;

	self.bossHpList = {}
	self:initPanels();
end

function EventRoundInfoPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(604,790));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , 0);

	local titleNode = cc.Node:create()
	titleNode:addTo(self.root)
	titleNode:setPosition( 0, 395 )

	local titleBg = cc.Sprite:create('image/commonImg/com_title_bg.png')
	titleBg:addTo(titleNode)

	self.title = cc.Label:createWithTTF(LANG.UI_ROUND_6 ,  fonts.FONT_SIYUAN , 24)
    self.title:enableOutline(cc.c4b(60 , 60 , 60 , 255),2);
    self.title:addTo(titleNode)

    self.tIcon = cc.Sprite:create('image/event/e_shop_icon.png')
    self.tIcon:setPosition(-180, 0)
    self.tIcon:addTo(titleNode)

	local infoBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	infoBg:addTo(self.root);
	infoBg:setPosition(0 , 100);
    infoBg:setCapInsets(cc.rect(11,11,34,28));
	infoBg:setContentSize(cc.size(582 , 430));

	-- role list
	local ownNode = cc.Node:create()
	ownNode:addTo(self.root)
	ownNode:setPosition(0, 202)
	local roleBg = cc.Sprite:create('image/event/self_bg.png')
	roleBg:addTo(ownNode)
	self.roleNode = cc.Node:create()
		:setPosition(0, -20)
		:addTo(ownNode)
	local label = cc.Label:createWithTTF(LANG.UI_FIGHT_102, fonts.FONT_SIYUAN, 20)
	label:setPosition(-200,75)
	label:addTo(ownNode)
	label:setAnchorPoint(0, 0.5)
	label:enableOutline(cc.c3b(35,95,25), 2)
	local ownSp1 = cc.Sprite:create('image/head/header_bg_9.png')
	ownSp1:addTo(ownNode)
	ownSp1:setPosition(110, 75)
	self.ownFpNum = cc.Label:createWithTTF('9999', fonts.FONT_SIYUAN, 20)
		:setAnchorPoint(1 , 0.5)
		:setPosition(250, 75)
		:addTo(ownNode)

	-- enemy list
	local enemyNode = cc.Node:create()
	enemyNode:addTo(self.root)
	enemyNode:setPosition(0, -5)
	local roleBg = cc.Sprite:create('image/event/enemy_bg.png')
	roleBg:addTo(enemyNode)
	self.eNode = cc.Node:create()
		:setPosition(0, -20)
		:addTo(enemyNode)
	local label = cc.Label:createWithTTF(LANG.UI_FIGHT_34, fonts.FONT_SIYUAN, 20)
	label:setPosition(-200,69)
	label:addTo(enemyNode)
	label:setAnchorPoint(0, 0.5)
	label:enableOutline(cc.c3b(135,0,0), 2)
	local enemySp1 = cc.Sprite:create('image/head/header_bg_9.png')
	enemySp1:addTo(enemyNode)
	enemySp1:setPosition(110, 69)
	self.fpNum = cc.Label:createWithTTF('9999', fonts.FONT_SIYUAN, 20)
		:setAnchorPoint(1 , 0.5)
		:setPosition(250, 69)
		:addTo(enemyNode)


	-- boss
	self.bossList = {}
	for i=1, 4 do

		local bossInfo = {}
		self.bossList[i] = bossInfo

		local bg = cc.Sprite:create('image/commonImg/icon_enemy_empty.png')
			:addTo(self.eNode)
			:setPosition((i-1)*126-189, 5)

		bossInfo.node = cc.Node:create()
			:setPosition((i-1)*126-189, 5)
			:addTo(self.eNode)

		bossInfo.bg = cc.Sprite:create('image/commonImg/icon_enemy.png')
			:addTo(bossInfo.node)

		bossInfo.bareFlag = cc.Scale9Sprite:create('image/commonImg/common_item_select.png');
		bossInfo.bareFlag:addTo(bossInfo.node);
		bossInfo.bareFlag:setContentSize(cc.size(112 , 112));
		bossInfo.bareFlag:setVisible(false)

		bossInfo.icon = cc.Sprite:create()
		local stencil = cc.Sprite:create('image/commonImg/icon_clipping_bg.png');
		stencil:setScale(1.05);
		local clippingNode = cc.ClippingNode:create();
		clippingNode:setStencil(stencil);
		clippingNode:addChild(bossInfo.icon);
		clippingNode:setInverted(false);
		clippingNode:setAlphaThreshold(0);
		clippingNode:addTo(bossInfo.node);

		bossInfo.restraint = cc.Sprite:create('image/commonImg/def_type_1.png')
			:setPosition(-31, 32)
			:setScale(0.55)
			:addTo(bossInfo.node)

		bossInfo.starNode = cc.Node:create()
			:setPosition(-26, -37)
			:addTo(bossInfo.node)

		bossInfo.level = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 16)
			:setAnchorPoint(1, 0.5)
			:setPosition(44, 37)
			:enableOutline(cc.c3b(60,60,60),2)
			:addTo(bossInfo.node)
	end
	
	local label = cc.Label:createWithTTF(LANG.UI_L_RESTRAINT_TITLE , fonts.FONT_SIYUAN , 17);
	label:enableOutline(cc.c3b(55 , 85 , 155) , 2);
	local restraintBtn = ccui.Button:create('image/commonImg/btn_2_103_36.png','image/commonImg/btn_2_103_36.png');
	restraintBtn:addTo(self.root);
	restraintBtn:setPosition(230 , 355);
	restraintBtn:setTitleLabel(label);
	restraintBtn:addClickEventListener(function()
		EventManager:FireEvent(EventType.UITipsMessage);
		UIManager:push('TeamRestraintPanel',self.fightType , self.enemyRestraintString);
	end);	

	-- drop
	local dropNode = cc.Node:create()
	dropNode:addTo(self.root)
	dropNode:setPosition(0, -215)

	local dropBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	dropBg:addTo(dropNode);
    dropBg:setCapInsets(cc.rect(11,11,34,28));
	dropBg:setContentSize(cc.size(582 , 172));

	local dropSp1 = cc.Sprite:create(imgPath .. 'round_info_2.png')
	dropSp1:addTo(dropNode)
	dropSp1:setPosition(0, 55)

	local label = cc.Label:createWithTTF(LANG.UI_FIGHT_35, fonts.FONT_SIYUAN, 20)
	label:setPosition(0,55)
	label:addTo(dropNode)
	

	self.dropListView = ccui.ListView:create();
	self.dropListView:setTouchEnabled(true); 
	self.dropListView:setBounceEnabled(true);
	self.dropListView:setDirection(ccui.ScrollViewDir.horizontal);
	self.dropListView:setContentSize(cc.size(480, 110));
	self.dropListView:setAnchorPoint(0.5 , 0.5);
	self.dropListView:setPosition(-2, -22);
	self.dropListView:setScrollBarEnabled(false);
	self.dropListView:addTo(dropNode);
	self.dropListView:setItemsMargin(22);
	self.dropListView:setCascadeOpacityEnabled(true);

	--btn
	self.btnTeam = ccui.Button:create('image/commonImg/btn_1_219_55_2.png','image/commonImg/btn_1_219_55_2.png');
	self.btnTeam:setPosition(-120, -346);
	self.btnTeam:addClickEventListener(handler(self,self.onTeamClick))
	self.btnTeam:addTo(self.root);
	local labelTeam = cc.Label:createWithTTF(LANG.UI_FIGHT_36, fonts.FONT_SIYUAN, 20)
		:enableOutline(cc.c4b(120 , 120 , 120 , 255) , 2)
	self.btnTeam:setTitleLabel(labelTeam)

	self.btnChallenge = ccui.Button:create('image/commonImg/btn_1_219_55.png','image/commonImg/btn_1_219_55.png');
	self.btnChallenge:setPosition(120, -346);
	self.btnChallenge:addClickEventListener(handler(self, self.challengeBoss));
	self.btnChallenge:addTo(self.root);
	local labelChallenge = cc.Label:createWithTTF(LANG.UI_FIGHT_37, fonts.FONT_SIYUAN, 20)
		:enableOutline(cc.c4b(50 , 85 , 150 , 255) , 2)
	self.btnChallenge:setTitleLabel(labelChallenge)

	EventManager:RegisterEvent(EventType.FightEnableChallenge, handler(self, self.enableBtnChallenge));

end

function EventRoundInfoPanel:showBossList( enemyInfo, hp )
	for k, v in pairs(self.bossHpList) do
		v:removeFromParent()
	end
	enemyInfo = clone(enemyInfo);  -- 防止修改ResTable原始数据顺序
	self.bossHpList = {}

	self.resEnd = true

	self.enemyRestraintString = '000';

	local index = 0
	local bareIdx = -1;
	for i=1,#enemyInfo do
		local bossInfo = enemyInfo[i];
		if bossInfo and bossInfo[1] then
			index = index + 1
			local bossId = bossInfo[1];
			local data = ResTable:getRowData('monster', bossId);
			--restraint[data['def_type']] = restraint[data['def_type']] + 1;
			self.bossList[i].bareFlag:setVisible(false)
			if data then
				self.bossList[i].icon:setTexture('image/icon/'..data.icon..'.png')
				self.bossList[i].restraint:setTexture('image/commonImg/def_type_'..data['def_type']..'.png');

				if data.show > 0 then
					bareIdx = i;
					self.bossList[i].bareFlag:setVisible(true)
				end
			end
			if hp and hp[i] then
				-- self.bossHpList[i] = require('app.template.BloodVolume'):create(self.bossList[i].node, 2)
				-- self.bossHpList[i]:setPercent(hp[i]/100)
				-- self.bossHpList[i]:setPosition(15, -50)

				local flagNode = cc.Node:create()
				flagNode:addTo(self.bossList[i].node)
				flagNode:setPosition(-51, -65)
				self.bossHpList[i] = flagNode;

				local hpBarBg = ccui.Scale9Sprite:create("image/blood/e_hp_bg.png");
				hpBarBg:setContentSize(cc.size(86 , 13));
				hpBarBg:setPosition(cc.p(7 , 0));
				hpBarBg:setAnchorPoint(cc.p(0,0.5))
				hpBarBg:addTo(flagNode);

				local hpGreenProgress = ccui.Scale9Sprite:create("image/blood/e_hp_bar.png");
				hpGreenProgress:setContentSize(cc.size(80 , 7));
				hpGreenProgress:setPosition(cc.p(10 , 0));
				hpGreenProgress:setAnchorPoint(cc.p(0,0.5))
				hpGreenProgress:addTo(flagNode);
				hpGreenProgress:setScaleX(hp[i]/10000);
			end

			self.bossList[i].node:setVisible(true)

			local attributeData = ResTable:getRowData('monster_attribute', bossInfo[2]);
			self.bossList[i].level:setString('Lv.'..tostring(attributeData.lv));

			self.bossList[i].starNode:removeAllChildren();
			local color = math.ceil(attributeData.star / 5);
			local num = attributeData.star % 5;
			num = num == 0 and 5 or num;
			for j = 1 , 5 do
				if j <= num then
					local star = cc.Sprite:create('image/card/star_level_'..color..'.png');
					star:addTo(self.bossList[i].starNode);
					star:setPosition(14*(j-1),0)
					star:setScale(0.55);
				end
			end

			if index == 4 then
				break;
			end
		end
	end

	--self.enemyRestraintString = tostring(restraint[restrainType.BLUE_BASE])..tostring(restraint[restrainType.RED_BASE])..tostring(restraint[restrainType.GREEN_BASE]);

	local totalFp = '0';
	local restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(enemyInfo);
	for _k , data in pairs(enemyInfo) do
		local pro = CardProManager:GenMonsterPro(data[1], data[2],restraintFactor);
		if pro then
			totalFp = HpPlus(totalFp, ConvertDigitUnitToStr(pro.fp));
		end
	end
	self.fpNum:setString(tostring(totalFp));
	
	self.enemyRestraintString = enemyRestraintString;

	for i=index+1, 4 do
		self.bossList[i].node:setVisible(false)
	end

end

function EventRoundInfoPanel:showDropList( rewardList )
	self.dropListView:removeAllChildren()
	local bpData = model.EventModel:getBPData();
	local bpPointItem = false
	if bpData and bpData.close_time then
		local lastTime = bpData.close_time - TimeManager:getCurTime()
		if lastTime > 0 then
			bpPointItem = true;
		end
	end

	local dropSpace = 103;
	local index = 1;
	for i=1, #rewardList do
		local dropInfo = rewardList[i];
		if dropInfo and dropInfo[1] then
			local bShow = true;
			if dropInfo[1] == BP_PointItem then
				bShow = bpPointItem
			end
			if bShow then
				local layout = ccui.Layout:create();
				layout:setContentSize(cc.size(103,103))
				-- layout:setPosition()
				local icon = template.ItemTemplate:create(2);
				icon:addTo(layout);
				icon:Init(dropInfo[1], dropInfo[2]);
				icon:setPosition(51.5, 51.5)
				self.dropListView:pushBackCustomItem(layout)
			end
		end
	end	
end

function EventRoundInfoPanel:showEventInfo()
	local fightInfo = model.EventModel:getMapMonsterInfo(self.select_key)
	local titleName = ResTable:getValue('maze_thing', fightInfo.thingId, 'name')
	local titleIcon = ResTable:getValue('maze_thing', fightInfo.thingId, 'icon')

	self.title:setString(titleName);
	self.tIcon:setTexture('image/event/'..titleIcon..'.png')
	local monster = {}
	for k, v in pairs(fightInfo.monster) do
		table.insert(monster, {[1] = v, [2] = fightInfo.att})
	end
	self:showBossList(monster, fightInfo.monster_hp) --fightInfo.monster_hp 血量百分比数组
	local data = ResTable:getRowData('maze_box', fightInfo.rwd);
	self:showDropList(data.box_drop)
	-- self:showDropList({{[1]=EVENT_BUFF}})

	self.roleNode:removeAllChildren()
	local team = model.TeamModel:getEventTeam()
	for i=1, TEAM_MAX_MEMBER do
		if team[i] ~= -1 then
			local card = model.CardModel:GetCard(team[i])
			local icon = require('app.template.ItemTemplate'):create();
			icon:addTo(self.roleNode);
			icon:Init(ROLE_TYPE, card.no, card.star_lv, card.level, team[i])
			icon:updateAltarLevelColor(card.oriLv)
			icon:setPosition((i-1)*126-189, 15)

			local hp = model.CardModel:getEventHp(team[i])
			local cd = model.CardModel:getEventCD(team[i])

			local flagNode = cc.Node:create()
			flagNode:addTo(self.roleNode)
			flagNode:setPosition((i-1)*126-240, -50)

			local hpBarBg = ccui.Scale9Sprite:create("image/blood/e_hp_bg.png");
			hpBarBg:setContentSize(cc.size(86 , 13));
			hpBarBg:setPosition(cc.p(7 , 0));
			hpBarBg:setAnchorPoint(cc.p(0,0.5))
			hpBarBg:addTo(flagNode);

			local hpGreenProgress = ccui.Scale9Sprite:create("image/blood/e_hp_bar.png");
			hpGreenProgress:setContentSize(cc.size(80 , 7));
			hpGreenProgress:setPosition(cc.p(10 , 0));
			hpGreenProgress:setAnchorPoint(cc.p(0,0.5))
			hpGreenProgress:addTo(flagNode);
			hpGreenProgress:setScaleX(hp/10000);

			local skillBarBg = ccui.Scale9Sprite:create("image/blood/e_skill_bg.png");
			skillBarBg:setContentSize(cc.size(86 , 10));
			skillBarBg:setPosition(cc.p(7 , -15));
			skillBarBg:setAnchorPoint(cc.p(0,0.5))
			skillBarBg:addTo(flagNode);

			local skillProgress = ccui.Scale9Sprite:create("image/blood/e_skill_bar.png");
			skillProgress:setContentSize(cc.size(80 , 4));
			skillProgress:setPosition(cc.p(10 , -15));
			skillProgress:setAnchorPoint(cc.p(0,0.5))
			skillProgress:addTo(flagNode);
			skillProgress:setScaleX(cd/10000);

		else
			local sp = cc.Sprite:create('image/event/event_buff_team_empty.png')
				:setPosition((i-1)*126-189, 15)
				:addTo(self.roleNode)
		end
	end

	local totalFp = model.TeamModel:getCurrentTeamFp(5);
	self.ownFpNum:setString(tostring(totalFp));
end

function EventRoundInfoPanel:updateRedDotShow()
	local show = model.TeamModel:checkTeamFormationRedDotState();
	tools.RedDotTools:setRedDotToNode(self.btnTeam,show,nil,nil)
end

function EventRoundInfoPanel:onEnter( option )
	self.root:setVisible(true);
	self:addSwallowCallback(handler(self, self.onClickBack));

	self:updateRedDotShow()
	self.blook = false;
	if option then
		self.fightType = option.fightType;
		self.select_key = option.select_key;
		if option.blook then
			self.blook = option.blook
		end
	end

	self.enemyRestraintString = '000';

	self:showEventInfo()

end

function EventRoundInfoPanel:onLeave()
	self.root:setVisible(false);
	EventManager:FireEvent(EventType.UITipsMessage);
	if self.m_scheduleHandler then 
        scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
        self.m_scheduleHandler = nil 
    end
end

function EventRoundInfoPanel:clearData(  )
	self.fightType = -1;
	self.onFightCallback = nil;
	self.onTeamCallback = nil;
end

function EventRoundInfoPanel:onExitPanel(  )
	self:clearData()
	UIManager:pop('EventRoundInfoPanel');
end

function EventRoundInfoPanel:onClickBack()
	self:onExitPanel()
end

function EventRoundInfoPanel:onTeamClick(sender,eventType)
	
	AudioManager:playEffect(Conf.confirmButton)
	if self.blook then
		Prompt:Show(LANG.UI_EVENT_CLICK);
		return;
	end
	
	if self.fightType == FightType.Event then
		UIManager:push('TeamPanel', TeamType.EVENT)
	else
		UIManager:push('TeamPanel')
	end
	EventManager:FireEvent(EventType.UITeamUpdateRoundInfoOption , {fightType = self.fightType , select_key = self.select_key});
end

function EventRoundInfoPanel:challengeBoss()
	AudioManager:playEffect(Conf.buttonClick);
	if self.blook then
		Prompt:Show(LANG.UI_EVENT_CLICK);
		return;
	end

	if self.resEnd == false then
		Prompt:Show('resource is loading....');
		return
	end
	-- AudioManager:playCVStopBefor(model.UserModel:getKanban(), 'Joy');

	local option = {}
	if self.fightType == FightType.Event then
		option.id = self.select_key
		if not model.TeamModel:checkEventTeam() then
			Prompt:Show(LANG.UI_TEAM_28);
			return;
		end
	end
		

	if FightManager:enterFight(self.fightType, option) then 
		UIManager:pop('EventRoundInfoPanel')
		self.btnChallenge:setTouchEnabled(false);
		self.btnChallenge:runAction(cc.Sequence:create(
			cc.DelayTime:create(5),
			cc.CallFunc:create(function()
				self.btnChallenge:setTouchEnabled(true)
			end)))
	end
end

function EventRoundInfoPanel:enableBtnChallenge()
	self.btnChallenge:setTouchEnabled(true);
end

return EventRoundInfoPanel;
