local FightMapPanel = class("FightMapPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/fight/map/'

local mapBoundary = {
	left = -240,
	right = 240,
	top = 430,
	bottom = 0,
}
local maxRoundNum = 30;

local imgPath = 'image/fight/map/';

function FightMapPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'FightMapPanel');

	self.useCCreator = false;
	self.isPopup = true;
	self.haveSwallowLayer = false;
end

function FightMapPanel:Init(root)
	self.chapterMapNode = cc.Node:create();
	self.chapterMapNode:addTo(self.root);

	self.map = cc.Sprite:create('image/round/BG_chapter_1.png')
		:setPosition(0, 0)
		:addTo(self.chapterMapNode)

	self.mapPath = cc.Sprite:create('image/round/path_1.png')
		:setPosition(0, 0)
		:addTo(self.chapterMapNode)

	self.mapCover = cc.Sprite:create('image/round/BG_chapter_1.png')
		:setLocalZOrder(100)
		:setVisible(false)
		:setPosition(0, 0)
		:addTo(self.chapterMapNode)
	
	self.roundList = {};
	for i=1, maxRoundNum do
		local round = require('app.template.RoundIconTemplate'):create(self.chapterMapNode);
		self.roundList[i] = round;
	end
	self.endRoundIcon = require('app.template.RoundIconTemplate'):create(self.chapterMapNode);
	self.endRoundIcon:setVisible(false)

	self.root:setVisible(false);

	EventManager:RegisterEvent(EventType.FightMapSetNewRound, handler(self, self.setNewRound));
	EventManager:RegisterEvent(EventType.FightPanelShake, handler(self, self.ShowShakeEff));
end

function FightMapPanel:onEnter()
	self:initChapterMap();
	self:setCurRound();

	self.root:setVisible(true);
end

function FightMapPanel:onLeave()
	self.root:setVisible(false);
	if self.skeletonNodeMap then
		self.skeletonNodeMap:removeFromParent();
		self.skeletonNodeMap = nil;
	end
end

function FightMapPanel:initChapterMap(callback)
	self.chapter = model.RoundModel:getChapter();
	local chapterData = ResTable:getRowData('chapter', self.chapter);
	local pathData = ResTable:getRowData('round_path', chapterData.path)

	AudioManager:SetState(Enum:getTable('GameState').Fight);
	AudioManager:playMusic( chapterData.bgm , 3, true);

	self.map:setTexture('image/round/'..chapterData['bg']..'.png');
	self.mapPath:setTexture('image/round/path_'..pathData.img..'.png')
	self.mapPath:setPosition(pathData['img_coor'][1], pathData['img_coor'][2])
	local roundData = ResTable:getTable('round');
	local roundList = {};
	for _, v in pairs(roundData.contain) do
		if v['chapter'] == self.chapter then
			local round = {}
			local index = v.no % 100
			round.no = v.no
			round.x = pathData['coordination'][index][1]
			round.y = pathData['coordination'][index][2]	
			table.insert(roundList, round);
		end
	end

	if self.skeletonNodeMap then
		self.skeletonNodeMap:removeFromParent();
		self.skeletonNodeMap = nil;
	end
	local spine_res = chapterData['bg'];
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.skeletonNodeMap = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)

		self.skeletonNodeMap:addTo(self.chapterMapNode)
		self.skeletonNodeMap:setAnimation(0, spine_res, true)

		self.skeletonNodeMap:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_COMPLETE)

		self.skeletonNodeMap:registerSpineEventHandler(function (event)
		end, sp.EventType.ANIMATION_EVENT)
	end

	table.sort(roundList, function(first, second)
		return first.no < second.no;
	end);

	if model.RoundModel:IsAllEnd() then
		self:showAllEndRoundIcon();
	else
		self.endRoundIcon:setVisible(false);
		for i=1, maxRoundNum do
			if roundList[i] then
				self.roundList[i]:Init(roundList[i].no, false);
				self.roundList[i]:setVisible(true);
				self.roundList[i]:setPosition(roundList[i].x, roundList[i].y);
			else
				self.roundList[i]:setVisible(false);
			end
		end
	end

	if callback then
		callback();
	end
end

function FightMapPanel:showAllEndRoundIcon()
	local chapterData = ResTable:getRowData('chapter', self.chapter);
	local pathData = ResTable:getRowData('round_path', chapterData.path)

	for i=1, maxRoundNum do
		self.roundList[i]:setVisible(false);
	end

	local len = #pathData['coordination'];
	local x = pathData['coordination'][len][1]
	local y = pathData['coordination'][len][2]
	self.endRoundIcon:setVisible(true);
	self.endRoundIcon:setPosition(x,y);
	local tmpNo = model.RoundModel:getRoundIndex();
	self.endRoundIcon:Init(tmpNo, false);
	self.endRoundIcon:setCur(tmpNo);
end

function FightMapPanel:setCurRound()
	self.round = model.RoundModel:getRoundIndex();
	for i=1, maxRoundNum do
		self.roundList[i]:setCur(self.round);
	end

	FightManager:showFightOverStep()

	if model.RoundModel:IsAllEnd() then
		self:showAllEndRoundIcon();
	end
end

function FightMapPanel:setNewRound()
	self.round = model.RoundModel:getRoundIndex();
	for i=1, maxRoundNum do
		self.roundList[i]:setCur(-1);
	end
	local chapterID = model.RoundModel:getChapter();
	if self.root:isVisible() then
		if self.chapter ~= chapterID then
			UIManager:push('ScreenEffectPanel', 'newChapter', {callback = function()
				self:initChapterMap()
				self:setCurRound()
			end})
		else
			UIManager:push('ScreenEffectPanel', 'newRound', {callback = handler(self, self.setCurRound)});
		end
		local bgType = ResTable:getValue('round', self.round, 'bg_id');
		EventManager:FireEvent(EventType.FightPanelUpdateScene, bgType)
	else
		if self.chapter ~= chapterID then
			self:initChapterMap()
		else
			self:setCurRound()
		end
	end
end

function FightMapPanel:showCoverMap(img)
	self.mapCover:setTexture('image/round/'..img..'.png')
	self.mapCover:setVisible(true)
	if img == 'BG_chapter_0' then
		self.mapCover:setPosition(0,150)
	else
		self.mapCover:setPosition(0,0)
	end
end

function FightMapPanel:hideCoverMap()
	self.mapCover:setVisible(false)
end

function FightMapPanel:ShowShakeEff()
	-- body
	self.root:stopAllActions()
	self.root:setPosition(StageSize.width/2,StageSize.height/2)
	local seq = cc.Sequence:create( cc.MoveBy:create(1/30, cc.p(0,6)), cc.MoveBy:create(1/30, cc.p(0,-6)) )
	self.root:runAction( seq )
end

return FightMapPanel;
