local FightOverPanel = class("FightOverPanel", require('app.layer.BaseLayer'))

local FightType = Enum:getTable('FightType');
local TeamType = Enum:getTable('TeamType')
local imgPath = 'image/fight/result/';
local fonts = Enum:getTable("SystemFont");
local scheduler = cc.Director:getInstance():getScheduler();
local cvTypeEnum = Enum:getTable("cvTypeEnum");

local rolePos = {
	[1] = {x = 175, y = 248, zorder = 80},
	[2] = {x = 12, y = 152, zorder = 100},
	[3] = {x = 9, y = 325, zorder = 80},
	[4] = {x = -165, y = 248, zorder = 50},
}

function FightOverPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'FightOverPanel');
	
	self.scene = scene;
	self.useCCreator = false;
	--self.isPopup = true;
	self.haveSwallowLayer = true;
	self.result = nil;
	self.gotoState = false;
end

function FightOverPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self.root:setCascadeOpacityEnabled(true);

	local bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 210));
	bg:setContentSize( cc.size(StageSize.width, StageSize.height) );
	bg:setPosition(-StageSize.width/2, -StageSize.height/2);
	bg:setLocalZOrder(-100);
	bg:addTo(self.root);

	self.titleNode = cc.Node:create();
	self.titleNode:addTo(self.root)

	local spine_res = 'shine_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.bgAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.bgAnimation:addTo(self.titleNode)
		self.bgAnimation:setPosition(cc.p(0,190))
	else
        print('error : 缺少spine 资源')
    end 

	-- title
	self.titleEffect = DragonBonesManager:GetUIEffect('jiesuan');
	self.titleEffect:setPosition(0, 366);
	self.titleEffect:addTo(self.titleNode);

	self.stage = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 35);
	self.stage:addTo(self.titleNode);
	self.stage:setPosition(0,243);
	self.stage:enableOutline(cc.c3b(50,50,50),2);

	self.winNode = cc.Node:create()
		:addTo(self.root)

	-- role
	self.showRoleNode = cc.Node:create();
	self.showRoleNode:setPosition(0, 75);
	self.showRoleNode:addTo(self.winNode);

	local spine_res = 'win_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.roleSpine1 = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.roleSpine1:addTo(self.showRoleNode)
		self.roleSpine1:setPosition(cc.p(0,-172))
        self.roleSpine1:registerSpineEventHandler(handler(self,self.spineAnimationEventCall),sp.EventType.ANIMATION_EVENT)
		self.roleSpine1:registerSpineEventHandler(function (event)
					-- self.canPop = true 
					if self.particleKey then
						self.particleKey:resetSystem()
					end
                end, sp.EventType.ANIMATION_COMPLETE)
	else
        print('error : 缺少spine 资源')
    end  

	self.showRoleList = {};
	for i=1, TEAM_MAX_MEMBER do
		self.showRoleList[i] = {}

		self.showRoleList[i].node = cc.Node:create()
			:addTo(self.showRoleNode)
			:setPosition(cc.p(-222 +(i-1)*148,i%2 == 0 and 15 or -15))
			:setScale(0.8)

		self.showRoleList[i].bg = cc.Sprite:create(imgPath..'quality_bg_null.png')
			:addTo(self.showRoleList[i].node)

		self.showRoleList[i].roleBg = cc.Sprite:create('image/iconL/R003_IconL.png')
			:addTo(self.showRoleList[i].node)
			:setVisible(false)
			:setTextureRect(LRect)

		self.showRoleList[i].empty =  cc.Sprite:create(imgPath..'role_pos_'..i..'_2.png')
			:addTo(self.showRoleList[i].node)
			:setPosition(cc.p(33,-155))

		self.showRoleList[i].posSp =  cc.Sprite:create(imgPath..'role_pos_'..i..'.png')
			:addTo(self.showRoleList[i].node)
			:setPosition(cc.p(47,-168))
			:setVisible(false)

		local border =  cc.Sprite:create(imgPath..'quality_bg_border.png')
			:addTo(self.showRoleList[i].node)
	end 
	spine_res = 'win_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.roleSpine2 = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.roleSpine2:addTo(self.showRoleNode)
		self.roleSpine2:setPosition(cc.p(0,-172))
	else
        print('error : 缺少spine 资源')
	end  
	self.particleKey = cc.ParticleSystemQuad:create("Effect/particle/wshine.plist")
	self.particleKey:addTo(self.showRoleNode);
	self.particleKey:stopSystem()

	self.roleBookNode = cc.Node:create();
	self.roleBookNode:addTo(self.root);
	self.roleBookNode:setCascadeOpacityEnabled(true);
	
	-- lv and exp
	self.lvExpNode = cc.Node:create()
		:addTo(self.winNode)
		:setLocalZOrder(10)
	self.lvExpNode:setCascadeOpacityEnabled(true);

	self.userIcon = template.ItemTemplate:create();
	self.userIcon:addTo(self.lvExpNode);
	self.userIcon:setPosition(-227,-229);

	local expLable1 = cc.Label:createWithTTF(LANG.UI_FIGHT_82, fonts.FONT_SIYUAN, 20)
		:setPosition(-160, -197)
		:setAnchorPoint(0,0.5)
		:addTo(self.lvExpNode)
	self.userLv = cc.Label:createWithTTF('LV 4', fonts.FONT_SIYUAN, 20)
		:setPosition(-55, -197)
		:addTo(self.lvExpNode)
		:setAnchorPoint(cc.p(0, 0.5))
	self.userLvNext = cc.Label:createWithTTF('LV 5', fonts.FONT_SIYUAN, 20)
		:setPosition(74, -197)
		:addTo(self.lvExpNode)
		:setAnchorPoint(cc.p(0, 0.5))

	self.userLvArror = cc.Sprite:create("image/commonImg/arrow_6.png")
		:addTo(self.lvExpNode)
		:setPosition(34,-197)

	local expProgressBg = cc.Scale9Sprite:create("image/commonImg/pgs_bg_1_223_12.png")
		:addTo(self.lvExpNode)
		:setPosition(59,-228)
		:setCapInsets(cc.rect(10,0,203,12))
		:setContentSize(cc.size(439 , 15))

	local progress = cc.Scale9Sprite:create("image/commonImg/pgs_1_445_21.png")

	self.expProgress = cc.ProgressTimer:create(progress);
	self.expProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR);
	self.expProgress:setMidpoint(cc.p(0, 0));
	self.expProgress:setBarChangeRate(cc.p(1, 0));
	self.expProgress:setPosition(expProgressBg:getContentSize().width/2,expProgressBg:getContentSize().height/2);
	self.expProgress:setPercentage(10);
	self.expProgress:addTo(expProgressBg);

	--[[
	local expBg = cc.Sprite:create(imgPath.."exp_bg.png")
		:addTo(self.lvExpNode)
		:setPosition(-2,-106)
	
	local iconExp = cc.Sprite:create('image/icon/I10004.png')
		:setScale(0.4)
		:setPosition(-154,-106)
		:addTo(self.lvExpNode)

	self.expLabel = cc.Label:createWithTTF('+ 888', fonts.FONT_SIYUAN, 20)
		:setPosition(168, -108)
		:addTo(self.lvExpNode)
		:setAnchorPoint(cc.p(1, 0.5))
		:setTextColor(cc.c3b(30, 190, 140))
	]]
	-- 任务达成信息
	self.missionNode = cc.Node:create()
		:addTo(self.winNode)
		:setVisible(false)
		:setLocalZOrder(10)
	self.missionNode:setCascadeOpacityEnabled(true);

	local missionBg = cc.Sprite:create('image/commonImg/tip_bg_1_309_28.png');
	missionBg:addTo(self.missionNode);
	missionBg:setPosition(-3,-261);

	self.missionInfo = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
		:setPosition(-158, -261)
		:addTo(self.missionNode)
		:setAnchorPoint(cc.p(0, 0.5))

	self.missionProgress = cc.Label:createWithTTF('(2/5)', fonts.FONT_SIYUAN, 18)
		:setPosition(-14, -261)
		:addTo(self.missionNode)
		:setAnchorPoint(cc.p(0, 0.5))
		:setTextColor(cc.c3b(255, 255, 0))

	--竞技场
	self.arenaNode = cc.Node:create();
	self.arenaNode:addTo(self.winNode);
	self.arenaNode:setVisible(false);
	self.arenaNode:setLocalZOrder(10);

	self.curRankBg = cc.Sprite:create();
	self.curRankBg:addTo(self.arenaNode);
	self.curRankBg:setPosition(-192,-208);
	self.curRankBg:setScale(0.25);

	self.curRank = cc.Sprite:create();
	self.curRank:addTo(self.arenaNode);
	self.curRank:setPosition(-192,-208);
	self.curRank:setScale(0.25);

	local bg = cc.Sprite:create(imgPath..'result_win_bg_2.png');
	bg:addTo(self.arenaNode);
	bg:setPosition(-192 , -260);

	self.curRankName = cc.Label:createWithTTF('0', fonts.FONT_SIYUAN, 20);
	self.curRankName:addTo(self.arenaNode);
	self.curRankName:setPosition(-192 , -260);

	local bg = cc.Sprite:create(imgPath..'result_win_pgs_bg_1.png');
	bg:addTo(self.arenaNode);
	bg:setPosition(56 , -223);

	self.arenaRankProgress = cc.Sprite:create(imgPath..'result_win_pgs_1.png');
	self.arenaRankProgress:addTo(self.arenaNode);
	self.arenaRankProgress:setAnchorPoint(0 , 0.5);
	self.arenaRankProgress:setPosition(-119 , -223);

	self.arenaRankValue = cc.Label:createWithTTF('0', fonts.FONT_SIYUAN, 16);
	self.arenaRankValue:addTo(self.arenaNode);
	self.arenaRankValue:setPosition(56 , -229.5);
	self.arenaRankValue:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);

	-- bottom
	self.bottomNode = cc.Node:create()
		:addTo(self.winNode)

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.bottomNode);
	bg:setPosition(0 , -330);
	bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(604 , 274));
	self.btmBg = bg;

	local titleNode = GlobalFunctions:createTitle(nil,604,56);
	titleNode:addTo(self.bottomNode);
	titleNode:setPosition(0,-165);
	self.btmTitleNode = titleNode;

	local rewardBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	rewardBg:addTo(self.bottomNode);
	rewardBg:setPosition(0 , -391);
	rewardBg:setCapInsets(cc.rect(11,11,34,28));
	rewardBg:setContentSize(cc.size(551 , 134));

	for i = 1 , 2 do
		local bg = cc.Sprite:create(imgPath..'result_win_bg_1.png');
		bg:addTo(self.bottomNode);
		bg:setPosition(i == 1 and -130 or 130 , -301);
		bg:setScaleX(i==1 and 1 or -1);
	end

    local tmpLabel = cc.Label:createWithTTF(LANG.UI_FIGHT_22 ,  fonts.FONT_SIYUAN , 20);
    tmpLabel:addTo(self.bottomNode);
    tmpLabel:setPosition(0, -301);
    --tmpLabel:enableGlow( cc.c4b(50, 141, 174 , 255) );
    -- tmpLabel:enableOutline(cc.c4b(0, 0, 0 , 255) , 2);
    --tmpLabel:enableBold();

	self.rewardListView = ccui.ScrollView:create();
	self.rewardListView:setTouchEnabled(true)
		:setBounceEnabled(true)
		:setDirection(ccui.ScrollViewDir.horizontal)
		:setContentSize(cc.size(551 , 134))
		:setAnchorPoint(cc.p(0.5,0.5))
		:setScrollBarEnabled(false)
		:addTo(self.bottomNode)
		:setPosition(cc.p(0,-391))

	self.rewardList = {};

	self.guildBossNode = cc.Node:create()
	:addTo(self.winNode)
	:setPosition(0, -223)
	:setVisible(false)

	local spine_res = 'zongshanghai_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.guildBossSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.guildBossSpine:addTo(self.guildBossNode)
		self.guildBossSpine:setVisible(false)
	end 
	self.guildBossDamageLabel = cc.Label:createWithTTF('99999999999', fonts.FONT_SIYUAN, 30)
	:addTo(self.guildBossNode)
	
	self.stbtn = ccui.Button:create(imgPath .. 'reslut_statistics_btn.png');
	self.stbtn:setPosition(263, -173)
	self.stbtn:addTo(self.root);
	--self.stbtn:setLocalZOrder(10);
	self.stbtn:addClickEventListener(handler(self, self.gotoStatistics));

	self.bookRolePath = ''
	self.m_result = nil

	self:initLoseNode()

end

function FightOverPanel:initLoseNode()
	self.loseNode = cc.Node:create()
	:addTo(self.root)
	:setVisible(false)

	local topLabel = cc.Label:createWithTTF(LANG.UI_FIGHT_98, fonts.FONT_SIYUAN, 25)--'想要击败敌人'
		:setPosition(0, 145)
		:addTo(self.loseNode)

	local topLabel = cc.Label:createWithTTF(LANG.UI_FIGHT_87, fonts.FONT_SIYUAN, 20)--'你可以通过以下方式增强实力'
		:setPosition(0, 113)
		:addTo(self.loseNode)

	local bottomSpr = cc.Sprite:create(imgPath .. 'result_lose_bg_2.png')
	:addTo(self.loseNode)
	:setPosition(0, -355)
	local bottomLabel = cc.Label:createWithTTF(LANG.UI_FIGHT_88, fonts.FONT_SIYUAN, 18)--'点击屏幕继续'
		:setPosition(0, -355)
		:addTo(self.loseNode)

	self.btnList = {}
	self.btns = {}

	for i=1, 3 do

		local btn = {}
		local tag = i;
		self.btns[tag] = btn;

		btn.node = cc.Node:create()
			:setPosition(0, 19-129*(i-1))
			:addTo(self.loseNode)

		btn.btn = ccui.Button:create(imgPath .. 'result_lose_bg_1.png');
		btn.btn:setTag(tag);
		btn.btn:addTo(btn.node);
		btn.btn:addClickEventListener(handler(self, self.btnClick));

		btn.icon = cc.Sprite:create()
		btn.icon:addTo(btn.node);
		btn.icon:setPosition(-211, 1)

		btn.btnMsg = cc.Label:createWithTTF( '', fonts.FONT_SIYUAN, 18)
			:setPosition(-159 , 1)
			:addTo(btn.node)
			:setAnchorPoint(cc.p(0, 0.5))
			:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
			:setDimensions(280, 0)

		local gotoImg = cc.Sprite:create(imgPath .. 'result_lose_btn_1.png')
		gotoImg:addTo(btn.node);
		gotoImg:setPosition(198,1);

		self.btnList[tag] = btn.node
	end
end

function FightOverPanel:gotoStatistics()
	self.gotoState = true;
	UIManager:push('FightStatisticsPanel');
end

function FightOverPanel:showDamageNum( )
	-- body
	self.guildBossSpine:setVisible(true)
	self.guildBossSpine:setAnimation(0, GlobalFunctions:lAnimationName('zongshanghai_spine'), false)
	local labelFuc = cc.CallFunc:create(function()
		self.guildBossDamageLabel:setString('0')
		self.guildBossDamageLabel:setVisible(true)
		self.guildBossDamageLabel:stopAllActions()
		local totalCurValue = 0
		local tmpTimes = 50
		local totalDamageValue = FightManager:getShowDamage()
		local argDamageValue = math.floor(totalDamageValue/tmpTimes)
		local argDamageOffValue = math.floor(argDamageValue*0.2)
		local idx = 1;
		local act = cc.Sequence:create( cc.DelayTime:create(0.05), cc.CallFunc:create(function()
			totalCurValue = totalCurValue + argDamageValue + math.random(-argDamageOffValue, argDamageOffValue)
			idx = idx + 1;
			if totalCurValue > totalDamageValue or idx==tmpTimes then
				totalCurValue = totalDamageValue;
			end

			local showNum = tostring(totalCurValue);
			self.guildBossDamageLabel:setString( showNum );


			end))

		self.guildBossDamageLabel:runAction( cc.Repeat:create(act ,tmpTimes) );
	end)

	self.guildBossDamageLabel:setVisible(false)
	self.guildBossNode:runAction( cc.Sequence:create( cc.DelayTime:create(0.56), labelFuc ) )
end

function FightOverPanel:onEnter(result)
	self.gotoState = false;
	self.isLevelUp = false;
	self.levelUpData = nil;
	self.bookRolePath = ''
	self.time = 0
	self.m_result = result
	self.bookRoleFadeOut = false;
	if result.win == 1 or result.flag == 4  then
		self.canLeave = false;
	else
		self.canLeave = true;
	end

	AudioManager:stopMusic();
	--AudioManager:playMusic(result.win == 1 and Conf.winAudio or Conf.failAudio, 3, false);
	self:addSwallowCallback(handler(self,self.onClickBack));

	self.titleNode:setVisible(true);
	self.root:setVisible(true);
	self.root:setOpacity(255);
	self.index = 1;

	self.bgAnimation:setAnimation(0,'shine_spine',true);

	if result.flag == 2 then
		self.stage:setVisible(true);
		local roundData = ResTable:getRowData('round', result.resid);
		self.stage:setString('STAGE.'..tostring(roundData.name));
	else
		self.stage:setVisible(false);
	end

	FightManager:winResult( result.win )
	if result.win == 1 or result.flag == 4 then
		self:showWin(result)
		self.stage:setPosition(0,283);
		self.titleEffect:setPosition(0, 356);
		self.loseNode:setVisible(false)
		self.stbtn:setPosition(263, -173)
	else
		self.titleEffect:getAnimation():play(GlobalFunctions:lAnimationName("lose"), 1);
		self.winNode:setVisible(false)
		self.roleBookNode:setVisible(false)
		AudioManager:playEffect(Conf.failEffect);

		self.stbtn:setPosition(258, 121)
		self.loseNode:setVisible(true)
		self.stage:setPosition(0,243);
		self.titleEffect:setPosition(0, 316);
		self:showLose(result)
	end

	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.onEnterFrame), 1, false);

	self:updateBtnVisible();
end

function FightOverPanel:showSpecialOffer(isWin)
	if tools.ConfigStateTools:checkVisibleByName('SpecialOfferPanel') and
	  model.RechargeModel:getSpecialOfferFightShow() then
		self.gotoState = true
		UIManager:push('SpecialOfferPanel',isWin);
	end
end

function FightOverPanel:onClickBack()
	if self.bookRolePath ~= '' then
		return;
	end
	if self.roleBookNode:isVisible() then
		if self.bookRoleFadeOut then
			return;
		end
		self.bookRoleFadeOut = true
		local act = cc.Sequence:create( cc.FadeOut:create(0.5), cc.CallFunc:create(function()
			self.bookRoleFadeOut = false
			if not self.canLeave then
				self:stopAllActions(self.m_result);
				self.canLeave = true;
				return;
			end

			if tools.ConfigStateTools:checkVisibleByName('SpecialOfferPanel') and
	  			model.RechargeModel:getSpecialOfferFightShow() then
				UIManager:push('SpecialOfferPanel',true ,{
					callback = function() self:clickBackground(self.m_result) end;
					delayTime = 0;
				});
			else
				self:clickBackground(self.m_result);
			end
		end))
		self.roleBookNode:runAction( act )
	else
		if not self.canLeave then
			self:stopAllActions(self.m_result);
			self.canLeave = true;
			return;
		end
		self:clickBackground(self.m_result);
	end
end

function FightOverPanel:updateBtnVisible()
	local visible = tools.ConfigStateTools:checkVisibleByName('FightStatisticsPanel');
	self.stbtn:setVisible(visible);
end

function FightOverPanel:onShowGuildBossResult()
	-- body
	self.bottomNode:setVisible(false)
	self.guildBossNode:setVisible(true)
	self.lvExpNode:setVisible(false)
	self.missionNode:setVisible(false)
	self:showDamageNum()

end

function FightOverPanel:onEnterFrame()
	-- body
	if self.canLeave and self.bookRolePath == '' then
		self.time = self.time + 1
		if self.time >= 3 then
			if self.gotoState then
				return
			end
			self:clickBackground(self.m_result);
		end
	end
end

function FightOverPanel:showLose( result )
	-- body
	self.teamType = TeamType.COMMON
	local team = nil;
	if result.flag == FightType.Event then
		self.teamType = TeamType.EVENT
	elseif result.flag == FightType.Arena then
		self.teamType = TeamType.ARENA;
	elseif result.flag == FightType.GuildBoss then
		self.teamType = TeamType.GUILD_BOSS;
	elseif result.flag == FightType.TrialTower then
		-- local tid = model.TrialModel:getTrialTeamIdByType(result.type)
		-- team = model.TeamModel:getTeamById(tid);
		-- self.teamType = tid;
	elseif result.flag == FightType.Story then
		self.teamType = TeamType.STORY
	end

	if result.flag == FightType.TrialTower then
		local tid = model.TrialModel:getTrialTeamIdByType(result.type)
		team = model.TeamModel:getTeamById(tid);
		self.teamType = tid;
	else
		local tid = model.TeamModel:getTeamIdByFightType(result.flag)
		team = model.TeamModel:getTeamById(tid);
	end

	self.gotoList = FightManager:GetLoseReasonList( team )
	local initNum = 0
	for i,v in ipairs(self.gotoList) do
		if initNum >= 3 then
			break;
		end
		local msg = ""
		local iconV = 2
		if i==1 and v==1 then
			msg = LANG.UI_FIGHT_89--"请上满4名角色再进行挑战吧"
			iconV = 1

		elseif i==2 and v==1 then
			msg = LANG.UI_FIGHT_90--"好可惜，调整阵型，站位或组合再试试吧"
			iconV = 1

		elseif i==3 and v==1 then
			msg = LANG.UI_FIGHT_91--"角色强度急需提升"
			iconV = 3

		elseif i==4 and v>0 then
			msg = LANG.UI_FIGHT_92--"角色等级需要提升"

		elseif i==5 and v>0 then
			msg = LANG.UI_FIGHT_93--"角色需要进行突破"

		elseif i==6 and v>0 then
			msg = LANG.UI_FIGHT_94--"角色需要进行升星"

		elseif i==7 and v==1 then
			msg = LANG.UI_FIGHT_95--"角色强度有提升空间"
			iconV = 3
		end

		if msg ~= '' then
			initNum = initNum+1
			self.btns[initNum].btnMsg:setString( msg )
			self.btns[initNum].icon:setTexture(imgPath..'result_lose_icon_'..iconV..'.png')
			self.btns[initNum].btn:setTag(i)
		end
		
	end

	self:showSpecialOffer(false);
end

function FightOverPanel:showWin( result )
	local titlePlayTime = 0.5;
	self.titleEffect:getAnimation():play(GlobalFunctions:lAnimationName("win"), 1);
	AudioManager:playEffect(Conf.winEffect);
	self.arenaNode:setVisible(false);
	self.arenaNode:stopAllActions();
	self.winNode:setVisible(true)

	self.roleBookNode:setVisible(false)
	self.showRoleNode:setVisible(true)
	self.guildBossNode:setVisible(false)

	-- role
	self.teamType = TeamType.COMMON
	local team = nil;
	if result.flag == FightType.Event then
		self.teamType = TeamType.EVENT
	elseif result.flag == FightType.Arena then
		self.teamType = TeamType.ARENA;
	elseif result.flag == FightType.GuildBoss then
		self.teamType = TeamType.GUILD_BOSS;
	elseif result.flag == FightType.TrialTower then
		-- local tid = model.TrialModel:getTrialTeamIdByType(result.type)
		-- team = model.TeamModel:getTeamById(tid);
		-- self.teamType = tid;
	elseif result.flag == FightType.Story then
		self.teamType = TeamType.STORY
	end

	if result.flag == FightType.TrialTower then
		local tid = model.TrialModel:getTrialTeamIdByType(result.type)
		team = model.TeamModel:getTeamById(tid);
		self.teamType = tid;
	else
		local tid = model.TeamModel:getTeamIdByFightType(result.flag)
		team = model.TeamModel:getTeamById(tid);
	end

	for i=1, TEAM_MAX_MEMBER do

		local pid = team[i]
		if pid == -1  then
			self.showRoleList[i].bg:setTexture(imgPath..'quality_bg_null.png')
			self.showRoleList[i].roleBg:setVisible(false)
			self.showRoleList[i].empty:setVisible(true)
			self.showRoleList[i].posSp:setVisible(false)
		else
			local cardData = model.CardModel:GetCard(pid);
			--local roleData = ResTable:getRowData('role', cardData.no);
			local resId = model.CardModel:getCardResIdByPid(pid);
			local roleResource = ResTable:getRowData('role_resource', resId);

			self.showRoleList[i].bg:setTexture('image/card/card_quality_bg_'..cardData.quality..'.png')
			self.showRoleList[i].bg:setTextureRect(cc.rect(0 , 18 , 175 , 369));
			self.showRoleList[i].roleBg:setTexture('image/iconL/'..roleResource.iconlarge..'.png');
			self.showRoleList[i].roleBg:setTextureRect(LRect);

			self.showRoleList[i].roleBg:setVisible(true)
			self.showRoleList[i].empty:setVisible(false)
			self.showRoleList[i].posSp:setVisible(true)
		end
		self.showRoleList[i].node:setVisible(false)
	end
	self.roleSpine1:setAnimation(0,'win_spine2',false);
	self.roleSpine2:setAnimation(0,'win_spine1',false);

	local roleBeginTime = titlePlayTime;
	local rolePlayTime = 0.2
	-- self.showRoleNode:setPosition(-StageSize.width, 172);
	-- self.showRoleNode:runAction(cc.Sequence:create(cc.DelayTime:create(roleBeginTime), cc.MoveTo:create(rolePlayTime, cc.p(0, 172))));

	-- bottom
	local bottomBeginTime = roleBeginTime + rolePlayTime;
	local bottomPlayTime = 0.2;
	self.bottomNode:setVisible(true);
	self.bottomNode:setPosition(0, -450);
	self.bottomNode:runAction(cc.Sequence:create(cc.DelayTime:create(bottomBeginTime), cc.MoveTo:create(bottomPlayTime, cc.p(0, 0))));

	local expBeginTime = bottomBeginTime + bottomPlayTime;
	local expPlayTime = 0;
	if result.flag == 2 then
		-- 普通关卡
		self.expProgress:setVisible(false)
		self.lvExpNode:setVisible(true)
		self.lvExpNode:setOpacity(0)
		self.lvExpNode:runAction(cc.Sequence:create(
			cc.DelayTime:create(expBeginTime),
			cc.FadeIn:create(0.2),
			cc.CallFunc:create(function (  )
				self.expProgress:setVisible(true)
			end )
		));
		expBeginTime = expBeginTime + 0.2
		-- user exp
		--self.expLabel:setString('+'..result.getExp);
		expPlayTime = 0.3;
		local baseExp = 0;
		local maxExp = ResTable:getValue('user_lv', result.old_level, 'exp');
		if result.old_level > 1 then
			baseExp = ResTable:getValue('user_lv', result.old_level-1, 'exp');
		end
		local exp = result.old_exp;
		self.expProgress:setPercentage(math.floor((exp-baseExp)/(maxExp-baseExp)*100));
		local action
		if result.old_level < result.userExp.level then
			self.isLevelUp = true
			self.levelUpData = result.userExp.lv_up
			maxExp = ResTable:getValue('user_lv', result.userExp.level, 'exp');
			baseExp = ResTable:getValue('user_lv', result.userExp.level-1, 'exp');
			exp = tonumber(result.userExp.exp);
			local newPercent = math.floor((exp-baseExp)/(maxExp-baseExp)*100);

			action = cc.Sequence:create(
				cc.DelayTime:create(expBeginTime), 
				cc.ProgressTo:create(expPlayTime/2, 100),
				cc.CallFunc:create(function() self.expProgress:setPercentage(0) end), 
				cc.ProgressTo:create(expPlayTime/2, newPercent)
			);
			self.expProgressValue = newPercent;
		else
			exp = tonumber(result.userExp.exp);
			action = cc.Sequence:create(
				cc.DelayTime:create(expBeginTime), 
				cc.ProgressTo:create(expPlayTime, math.floor((exp-baseExp)/(maxExp-baseExp)*100))
			);
			self.expProgressValue = math.floor((exp-baseExp)/(maxExp-baseExp)*100);
		end
		self.expProgress:runAction(action);

		-- lv
		local no = model.UserModel:getHeadIcon() or 1;
		local iconData = GlobalFunctions:getHeadIconData(no)
		if iconData.quality then
			self.userIcon:Init(iconData.roleType, iconData.no,iconData.starLv,nil,nil,iconData.quality);
		else
			self.userIcon:Init(iconData.roleType, iconData.no);
		end

		if result.old_level ~= result.userExp.level then
			self.userLvArror:setVisible(true);
			self.userLvNext:setVisible(true);
			self.userLv:setString('Lv '..tostring(result.old_level));
			self.userLvNext:setString('Lv '..tostring(result.userExp.level));
		else
			self.userLvArror:setVisible(false);
			self.userLvNext:setVisible(false);
			self.userLv:setString('Lv '..tostring(result.userExp.level));
		end
	
	elseif result.flag == 8 then
		--竞技场
		local curRankData = model.ArenaModel:getCurArenaRankData();
    	local nextRankData = model.ArenaModel:getNextArenaRankData();
		local curScore = model.ArenaModel:getScore();
		
		local nextScore = 0;
		if nextRankData then
			nextScore = nextRankData.point;
		else 
			nextScore = curRankData.point;
		end
		local percent = curScore/nextScore;
		percent = percent > 1 and 1 or percent;
		local width = 349*percent;
		self.arenaRankProgress:setTextureRect(cc.rect(0 , 0 , width , 19));
		self.arenaRankValue:setString(tostring(curScore)..'/'..tostring(nextScore));
		self.curRankBg:setTexture('image/arena/arena_rank_bg_'..curRankData.rank..'.png');
		self.curRank:setTexture('image/arena/arena_rank_star_'..curRankData.order..'.png');
		self.curRankName:setString(tostring(curRankData.name));

		self.lvExpNode:setVisible(false)
		self.arenaNode:runAction(cc.Sequence:create(cc.DelayTime:create(bottomBeginTime+bottomPlayTime+0.1),cc.CallFunc:create(function() 
			self.arenaNode:setVisible(true);
		end)))
	elseif result.flag == 4 then
		self:onShowGuildBossResult()
	else
		self.lvExpNode:setVisible(false)
		self.missionNode:setVisible(false)
	end

	if result.flag==FightType.RandomBoss or result.flag==FightType.TrialTower or result.flag==FightType.CopyRound then
		self.btmTitleNode:setPosition(0,-235);
		self.btmBg:setContentSize(cc.size(604 , 204));
		self.btmBg:setPosition(0 , -365);
	elseif result.flag==FightType.Event then
		self.btmTitleNode:setPosition(0,-275);
		self.btmBg:setContentSize(cc.size(604 , 164));
		self.btmBg:setPosition(0 , -385);
	else
		self.btmTitleNode:setPosition(0,-165);
		self.btmBg:setContentSize(cc.size(604 , 274));
		self.btmBg:setPosition(0 , -330);
	end

	-- battlePass
	-- lifeMission 生涯任务
	local career = result.lifeMission
	if career ~= nil and result.flag ~= 8 then
		local _k,careerItem = next(career);
		if careerItem then
			local id = careerItem[1]
			local data = model.MissionModel:getLifeMissionData(id);
			if data then
				local nextNum = 0;
				if data.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
					local curNum = ConvertDigitUnitToStr(tostring(data.num));
					nextNum = '';
					if tostring(data.pre_times[1]) ~= '0' then
						for _k , num in ipairs(data.pre_times) do
							local pNum = ConvertDigitUnitToStr(tostring(num));
							if HpCompare(curNum,'<',pNum) then
								nextNum = pNum;
								break;
							end
						end

						if nextNum == '' then
							local count = #data.pre_times;
							local minusNum = HpMinus(curNum,tostring(data.pre_times[count]));
							local times = math.floor(tonumber(minusNum)/tonumber(data.times))+1;
							nextNum = HpPlus(tostring(data.times * times),tostring(data.pre_times[count]));
						end
					else
						local times = math.floor(tonumber(curNum)/tonumber(data.times))+1;
						nextNum = tostring(data.times * times);
					end
					self.missionProgress:setString(string.format("%s/%s",tostring(data.num),tostring(nextNum)));
				else
					local curNum = data.num;
					if data.pre_times[1] ~= 0 then
						for _k,num in ipairs(data.pre_times) do
							if curNum < num then
								nextNum = num;
								break;
							end
							if nextNum == 0 then
								local count = #data.pre_times;
								local times = math.floor((curNum - data.pre_times[count])/data.times) + 1;
								nextNum = data.times * times + data.pre_times[count];
							end
						end
					else
						nextNum = (math.floor(curNum/data.times)+1) * data.times;
					end

					self.missionProgress:setString(string.format("%d/%d",data.num,nextNum));
				end
				self.missionInfo:setString( string.format( data.name,nextNum ) )--tostring(data.name))

				local width = self.missionInfo:getContentSize().width;
				self.missionProgress:setPositionX(self.missionInfo:getPositionX()+width+20);
				

				self.missionNode:setVisible(true)
				self.missionNode:setOpacity(0)
				self.missionNode:runAction(cc.Sequence:create(
					cc.DelayTime:create(expBeginTime),
					cc.FadeIn:create(0.2)
				))
			end
		end
	else
		self.missionNode:setVisible(false);
	end


	local rewardBeginTime =  expBeginTime + expPlayTime;
	-- reward
	local rewardItem = result.getItems;
	local rewardItem = {};
	for k, v in pairs(result.getItems) do
		table.insert(rewardItem, {no = tonumber(k), num = v});
	end

	if result.flag == FightType.Boss then

		local rewardAdd = ResTable:getValue('round', result.resid , 'drop_boss');
		local indexList = {};
		indexList[10001] = 1;
		print('reward:    === ', json.encode(rewardAdd))
		for index, v in ipairs(rewardAdd) do
			indexList[tonumber(v[1])] = index + 1;
		end
		table.sort(rewardItem, function(first, second)
			return indexList[first.no] < indexList[second.no];
		end)
	end

    self.rewardListView:removeAllChildren();
    self.rewardList = {}
    local len = #rewardItem;
    local maxWidth = len *113
    if maxWidth < 551 then
        maxWidth = 551;
    end
	self.rewardListView:setInnerContainerSize(cc.size(maxWidth,134))

	for i=1,len do
		
		local item = template.ItemTemplate:create(2);
		item:addTo(self.rewardListView);
		item:Init(rewardItem[i].no, rewardItem[i].num);
		item:setPosition(56.6+(i-1)*113,67);
		item:setCanShowInfo(true,true);
		self.rewardList[i] = item;
		
		item:setOpacity(0);
		item:setScale(0);	
		if i == len then
			item:runAction(cc.Sequence:create(
				cc.DelayTime:create(rewardBeginTime+(i-1)*0.3), 
				cc.CallFunc:create(function() 
					AudioManager:playEffect(Conf.winReward);
				end),
				cc.Spawn:create(cc.FadeIn:create(0.2), 
				cc.ScaleTo:create(0.2, 1.1)), 
				cc.ScaleTo:create(0.1, 1), 
				cc.CallFunc:create(function()
					self.canLeave = true;
				end))
			);
		else
			item:runAction(cc.Sequence:create(
				cc.DelayTime:create(rewardBeginTime+(i-1)*0.3), 
				cc.CallFunc:create(function() 
					AudioManager:playEffect(Conf.winReward);
				end),
				cc.Spawn:create(cc.FadeIn:create(0.2), 
				cc.ScaleTo:create(0.2, 1.1)), 
				cc.ScaleTo:create(0.1, 1))
			);
		end
    end

    self.bookRolePath = ''
    local roleid = 0;
    if result.bareCollect and #result.bareCollect then
    	local bookId = result.bareCollect[1];
    	local bookInfo = ResTable:getRowData('bare_collect' , bookId);
		if bookInfo then
			self.bookRolePath = ResTable:getValue('role_resource', bookInfo.resource_id,'card');
			roleid = bookInfo.role_id;
		end
    end 

    if self.bookRolePath ~= '' then
    	self.gotoState = true;
    	self.roleBookNode:removeAllChildren();
	    self.roleBookNode:setVisible(true);
	    self.roleBookNode:setOpacity(255);
	    local bookAction = cc.Sequence:create(
					cc.DelayTime:create(3), 
					cc.CallFunc:create(function()
						self:showRoleBookSpine();
					end),
					cc.DelayTime:create(0.4),
					cc.CallFunc:create(function()
						self.roleBookSpr:runAction( cc.FadeIn:create(0.3) )
						self.canLeave = true;
						AudioManager:playCVStopBefor(roleid,nil ,cvTypeEnum.CV_BROKEN_FIGHT_OVER);
					end),
					cc.DelayTime:create(4),
					cc.CallFunc:create(function()
						self.bookRolePath = ''
					end)
				)
		self.roleBookNode:runAction( bookAction )
	else
		self:showSpecialOffer(true);
    end
    
end

function FightOverPanel:spineAnimationEventCall (event)
	local eventName = event.eventData.name;
	-- print('=========eventName === ',eventName)
	local _,index = string.find( eventName,'card' )
	if index ~= -1 then
		local cardIndex = string.sub( eventName, index+1,index+1 )
		if self.showRoleList[tonumber(cardIndex)] then
			self.showRoleList[tonumber(cardIndex)].node:setVisible(true)
		end
	end
	
end

function FightOverPanel:onLeave()
	if self.teamType == TeamType.EVENT then
		model.EventModel:removeRobotFromCardModel()
	end
	self.root:setVisible(false);

	self.particleKey:stopSystem();
	self.rewardListView:removeAllChildren()
	self.rewardList = {};

	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
end

function FightOverPanel:clickBackground(result)
	AudioManager:playMusic(Conf.commonFight, 3, true);
	UIManager:pop('CommonInfoPanel');
	UIManager:pop('FightOverPanel');
	local retTable = {}
	local levelUpOption = nil 
	local roundBreak = false;
	if result.flag == FightType.Boss then
		EventManager:FireEvent(EventType.FightNext);
		if result.win == 1 then
			-- EventManager:FireEvent(EventType.FightMapSetNewRound)
			self.resid = result.resid 
			if  self.resid%5 == 0 then
				roundBreak = true;
			end
		end
	elseif result.flag == FightType.RandomBoss then
		EventManager:FireEvent(EventType.FightNext);
	elseif result.flag == FightType.TrialTower then
		EventManager:FireEvent(EventType.FightNext);
		if result.type == 1 and result.resid<=200 then
			UIManager:set('TrialPanel', {'HomeBasePanel', 'TrialPanel'})
			retTable = {'HomeBasePanel', 'TrialPanel'}
		else
			UIManager:set('TrialPanel', {'HomeBasePanel', 'TrialSelPanel', 'TrialPanel'})
			retTable = {'HomeBasePanel', 'TrialSelPanel', 'TrialPanel'}
		end
	elseif result.flag == FightType.CopyRound then
		EventManager:FireEvent(EventType.FightNext);
		UIManager:set('CopyRoundPanel', {'MainPanel', 'CopyRoundPanel'})
		retTable = {'MainPanel', 'CopyRoundPanel'}
	elseif result.flag == FightType.DailyRound then
		if not model.DailyRoundModel:isCurrentFightPass() then
			EventManager:FireEvent(EventType.FightNext);
			UIManager:set('DailyRoundPanel', {'HomeBasePanel', 'DailyRoundPanel'})
			retTable = {'HomeBasePanel', 'DailyRoundPanel'}
		end

		EventManager:FireEvent(EventType.DailyRoundInfo);
	elseif result.flag == FightType.Event then
		EventManager:FireEvent(EventType.FightNext);
		UIManager:set('EventMapPanel', {'MainPanel', 'EventMainPanel', 'EventMapPanel'})
		retTable = {'MainPanel', 'EventMainPanel', 'EventMapPanel'}
	elseif result.flag == FightType.Arena then
		EventManager:FireEvent(EventType.FightNext);
		UIManager:set('ArenaPanel', {'HomeBasePanel', 'ArenaPanel'})
		retTable = {'HomeBasePanel', 'ArenaPanel'}
	elseif result.flag == FightType.GuildBoss then
		EventManager:FireEvent(EventType.FightNext);
		UIManager:set('GuildBossPanel', {'GuildMainPanel', 'GuildBossPanel'})
		retTable = {'GuildMainPanel', 'GuildBossPanel'}
	elseif result.flag == FightType.Story then
		EventManager:FireEvent(EventType.FightNext);
		UIManager:set('StoryRoundPanel', {'StoryPanel', 'StoryRoundPanel'})
		retTable = {'StoryPanel', 'StoryRoundPanel'}
	end

	local fightOverData = {
		isLevelUp = self.isLevelUp,
		levelUpData = self.levelUpData,
		roundBreak = roundBreak,
		round = self.resid,
		newRoundBoss = result.newRoundBoss,
		achivementPiece = result.achivementPiece,
		bWin = result.win == 1,
		fightType = result.flag,
		step = 1
	}
	if result.flag == FightType.TrialTower then
		fightOverData.trial_round = result.resid
		fightOverData.trial_type = result.type
	end
	FightManager:showFightOverAni(fightOverData)

	return retTable;
end

function FightOverPanel:btnClick(sender, event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);

	local retTable = self:clickBackground(self.m_result);

	if #retTable == 0 then
		if tag == 1 then
			UIManager:push('TeamPanel', self.teamType)
		elseif tag == 2 then
			UIManager:push('TeamPanel', self.teamType)
		elseif tag == 3 then
			UIManager:push('GachaListPanel');
		elseif tag == 4 then
			UIManager:push('CardPanel' , self.gotoList[tag], false);
		elseif tag == 5 then
			UIManager:push('CardPanel' , self.gotoList[tag], false);
		elseif tag == 6 then
			UIManager:push('CardLevelUpStarPanel' , self.gotoList[tag], false);
		elseif tag == 7 then
			UIManager:push('GachaListPanel');
		end
	else
		if tag == 1 then
			table.insert(retTable, 'TeamPanel')
			UIManager:set('TeamPanel', retTable, self.teamType)
		elseif tag == 2 then
			table.insert(retTable, 'TeamPanel')
			UIManager:set('TeamPanel', retTable, self.teamType)
		elseif tag == 3 then
			table.insert(retTable, 'GachaListPanel')
			UIManager:set('GachaListPanel', retTable)
		elseif tag == 4 then
			table.insert(retTable, 'CardPanel')
			UIManager:set('CardPanel', retTable, self.gotoList[tag], false)
		elseif tag == 5 then
			table.insert(retTable, 'CardPanel')
			UIManager:set('CardPanel', retTable, self.gotoList[tag], false)
		elseif tag == 6 then
			table.insert(retTable, 'CardLevelUpStarPanel')
			UIManager:set('CardLevelUpStarPanel', retTable, self.gotoList[tag], false)
		elseif tag == 7 then
			table.insert(retTable, 'GachaListPanel')
			UIManager:set('GachaListPanel', retTable)
		end
	end

	
end

function FightOverPanel:calExpPercent(lv, exp)
	local maxExp = ResTable:getValue('user_lv', lv, 'exp');
	local percent = 0;
	if lv == 1 then
		percent = math.floor(exp / maxExp * 100);
	else
		local prevExp = ResTable:getValue('user_lv', lv-1, 'exp');
		percent = math.floor((exp-prevExp) / (maxExp-prevExp) * 100);
	end
	return percent;
end

function FightOverPanel:stopAllActions(result)
	-- self.showRoleNode:stopAllActions();
	-- self.showRoleNode:setPosition(0, 172);

	self.bottomNode:stopAllActions();
	self.bottomNode:setPosition(0, 0);

	if self.lvExpNode:isVisible() then
		self.lvExpNode:stopAllActions()
		self.lvExpNode:setOpacity(255)
		self.expProgress:setVisible(true)
		self.expProgress:stopAllActions();
		self.expProgress:setPercentage(self.expProgressValue);
	end

	if self.missionNode:isVisible() then
		self.missionNode:stopAllActions()
		self.missionNode:setOpacity(255)
	end

	if self.arenaNode:isVisible() then
		self.arenaNode:setVisible(false);
	end

	for i=1, #self.rewardList do
		self.rewardList[i]:stopAllActions();
		self.rewardList[i]:setOpacity(255);
		self.rewardList[i]:setScale(1);
	end
end

function FightOverPanel:showRoleBookSpine()
	-- body
	self.showRoleNode:setVisible(false)
	self.winNode:setVisible(false)
	self.titleNode:setVisible(false);
	self.stbtn:setVisible(false);

	local sprPath = 'image/role/'..self.bookRolePath..'.png';
	self.roleBookSpr = cc.Sprite:create(sprPath)
	:setOpacity(255)
	:addTo(self.roleBookNode)
	GlobalFunctions:setSpriteGrey(self.roleBookSpr);

	-- local rolebookblack = cc.Sprite:create('image/role/'..self.bookRolePath..'_Black.png');
	-- rolebookblack:addTo(self.roleBookNode)
	
	-- local spine_res = 'lianzi_spine'
	-- if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
	-- 	local lianziSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
	-- 	lianziSpine:addTo(self.roleBookNode)
	-- 	lianziSpine:setAnimation(0,'appear',false);

	-- 	self.lianzi = false;
 --        lianziSpine:registerSpineEventHandler(function (event)
 --        	if self.lianzi == true then
 --        		return;
 --        	end
 --        	self.lianzi = true;
 --        	lianziSpine:setAnimation(0,'idle',true);
 --        end, sp.EventType.ANIMATION_COMPLETE)
	-- else
	-- 	print('error : 缺少spine 资源')
 --    end

    local spine_res = 'jiesuanpoyi1_spine'
    local spine_2res = 'jiesuanpoyi2_spine'
    local poyi2AniName = self.bookRolePath
    local poyi1 = nil
	if cc.FileUtils:getInstance():isFileExist('spine/jiesuanpoyi/'..spine_res..".json") then
		poyi1 = sp.SkeletonAnimation:create('spine/jiesuanpoyi/'..spine_res..".json", 'spine/jiesuanpoyi/'..spine_res..".atlas", 1)
		poyi1:addTo(self.roleBookNode)
		poyi1:setVisible(false)

		local poyi2 = sp.SkeletonAnimation:create('spine/jiesuanpoyi/'..spine_2res..".json", 'spine/jiesuanpoyi/'..spine_2res..".atlas", 1)
		poyi2:addTo(self.roleBookNode)
		poyi2:setVisible(false)

		poyi1:registerSpineEventHandler(function (event)
			if event.eventData.name == 'jiesuanpoyi2_spine' then
				poyi2:setVisible(true)
				poyi2:setAnimation(0, GlobalFunctions:lAnimationName(poyi2AniName) ,false);
			end
		end, sp.EventType.ANIMATION_EVENT)

    end


	spine_res = 'jiesuanjihuo_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.bookSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.bookSpine:addTo(self.roleBookNode)
		self.bookSpine:setAnimation(0,'jiesuanjihuo_spine',false);

		local act = cc.Sequence:create( cc.DelayTime:create(1), cc.CallFunc:create(function()
			poyi1:setVisible(true)
			poyi1:setAnimation(0,'jiesuanpoyi1_spine',false);
		end))
		self.root:runAction( act )

	else
		print('error : 缺少spine 资源')
    end
	
end

return FightOverPanel;
