local FightPanel = class("FightPanel", require('app.layer.BaseLayer'))

local fonts = Enum:getTable("SystemFont")
local FightType = Enum:getTable("FightType");
local levelLockItemIds = Enum:getTable("levelLockItemIds");
local imgPath = 'image/fight/panel/';
local scheduler = cc.Director:getInstance():getScheduler();
local MessageBoxType = Enum:getTable('MessageBoxType')
local cvTypeEnum = Enum:getTable("cvTypeEnum");

-- 战斗界面 按钮tag，tag值与主界面按钮共享
local FightMenuTag = {
	TAG_ROUND_REWARD = 4,--目标
	TAG_CHAT 		 = 7,--聊天
	TAG_ITEM_USE	 = 8,--道具
	TAG_RANDOM_BOSS  = 5,--感知
	TAG_BARE_COLLECT = 16,--救世手册

	--TAG_MAIL		 = 6,--邮件

	TAG_ONLINE 		 = 206,--在线奖励
	TAG_FAST_FIGHT   = 207,--挂机
	TAG_RESTRAINT    = 208,--克制
	TAG_HARVERT   	 = 209,
	TAG_NEWGIFT		 = 210,--新手礼包
	TAG_FIGHTGIFT	 = 211,--战斗礼包

	TAG_ADMOB_BUFF 	 = 20, --广告 加速buff
	TAG_ADMOB_REWARD = 21, --广告 立即获得金币
	TAG_ADMOB_GACHA	 = 22, --广告 抽卡券
}
local btnNodePos = {
	[FightMenuTag.TAG_ROUND_REWARD] = cc.p(-260 , StageSize.height/2-268-NOTCH_HEIGHT),--任务
	[FightMenuTag.TAG_BARE_COLLECT] = cc.p(-260 , StageSize.height/2-368-NOTCH_HEIGHT),--救世手册
	[FightMenuTag.TAG_ITEM_USE] = cc.p(-260 , StageSize.height/2-468-NOTCH_HEIGHT),--道具
	[FightMenuTag.TAG_RANDOM_BOSS] = cc.p(-260 , StageSize.height/2-468-NOTCH_HEIGHT),--感知
	
	--[FightMenuTag.TAG_MAIL] = cc.p(266 , 270),--邮件
	[FightMenuTag.TAG_CHAT] = cc.p(266 , StageSize.height/2-253-NOTCH_HEIGHT),--聊天

	[FightMenuTag.TAG_ONLINE] = cc.p(266 , StageSize.height/2-553-NOTCH_HEIGHT),
	[FightMenuTag.TAG_NEWGIFT] = cc.p(266 , StageSize.height/2-673-NOTCH_HEIGHT),
	[FightMenuTag.TAG_FIGHTGIFT] = cc.p(266 , StageSize.height/2-793-NOTCH_HEIGHT),

	[FightMenuTag.TAG_FAST_FIGHT] = cc.p(-248 , -408),
	[FightMenuTag.TAG_RESTRAINT] = cc.p(-269 , -408),

	[FightMenuTag.TAG_ADMOB_BUFF] = cc.p(266 , StageSize.height/2-253-NOTCH_HEIGHT),
	[FightMenuTag.TAG_ADMOB_REWARD] = cc.p(266 , StageSize.height/2-353-NOTCH_HEIGHT),
	[FightMenuTag.TAG_ADMOB_GACHA] = cc.p(266 , StageSize.height/2-453-NOTCH_HEIGHT),
}
local btnDotPos = {
	[FightMenuTag.TAG_ROUND_REWARD] = cc.p(32 , 20),
	[FightMenuTag.TAG_RANDOM_BOSS] = cc.p(32 , 24),
	[FightMenuTag.TAG_CHAT] = cc.p(0 , 0),
	--[FightMenuTag.TAG_MAIL] = cc.p(19 , 19),
	[FightMenuTag.TAG_ITEM_USE] = cc.p(32 , 24),

	[FightMenuTag.TAG_ONLINE] = cc.p(0 , 0),
	[FightMenuTag.TAG_FAST_FIGHT] = cc.p(21 , 34),
	[FightMenuTag.TAG_RESTRAINT] = cc.p(-0 , -0),

}

local ADMOB_TYPE = {
	ADMOB_BUFF 		= 1, -- buff类广告 
	ADMOB_REWARD 	= 2, -- 立即获得类广告
	ADMOB_GACHA		= 3, -- 看完后获得抽卡券
}
function FightPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'FightPanel');

	self.useCCreator = false;
	--self.loadWait = true;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_ICON;
	self.haveSwallowLayer = true;
	self.showFooter = true;

	self.mapPanel = require('app.layer.MainScene.FightMapPanel'):create(scene, sceneName);
	self.scenePanel = require('app.layer.MainScene.FightScenePanel'):create(scene, sceneName);

	UIManager:AddLayer('FightMapPanel', self.mapPanel);
	UIManager:AddLayer('FightScenePanel', self.scenePanel);

	self.bInFight = false ; -- 是否挂机
	self.speedup = nil;
end

function FightPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.onlineSecondOff = -1;		-- 在线奖励 倒计时
	self.onlineShowGetAni = false;	-- 在线奖励 是否已播放ani3 切换可领取动画
	self.onlineGetAniTag = 1000;	-- 在线奖励 动画tag

	self:initPanels();
	self:registEvent();
	self.mapPanel:Initialize();
	self.scenePanel:Initialize();
end

function FightPanel:initPanels()
	--
	self.skillNamePosition = {
		[1] = cc.p(0, -200),
		[2] = cc.p(0, -250),
		[3] = cc.p(0, -300),
		[4] = cc.p(0, -350),
	}

	self.btnNodeName = {
		--[FightMenuTag.TAG_MAIL] = LANG.UI_MAIN_2,
		[FightMenuTag.TAG_CHAT] = LANG.UI_TITLE_CHAT,
		[FightMenuTag.TAG_ITEM_USE] = LANG.UI_TITLE_ITEM,
	
		[FightMenuTag.TAG_ROUND_REWARD] = LANG.UI_TITLE_MUBIAO,
		[FightMenuTag.TAG_RANDOM_BOSS] = LANG.UI_ROUNDBOSS_5,
		[FightMenuTag.TAG_BARE_COLLECT] = LANG.UI_TITLE_SHOUCE,

		[FightMenuTag.TAG_ADMOB_BUFF] =LANG.UI_ADMOB_1,
		[FightMenuTag.TAG_ADMOB_REWARD] = LANG.UI_ADMOB_2,
		[FightMenuTag.TAG_ADMOB_GACHA] = LANG.UI_ADMOB_6,
	
	}

	self.btnNode = cc.Node:create()
		:setPosition(0, 0)
		:addTo(self.root)

	self.btnList = {};
	self.btns = {};
	local leftTag = {FightMenuTag.TAG_ROUND_REWARD, FightMenuTag.TAG_BARE_COLLECT, --任务，救世手册
					FightMenuTag.TAG_ITEM_USE, FightMenuTag.TAG_RANDOM_BOSS} -- 目标，感知
	local len = #leftTag;
	for i=1, len do

		local btn = {}
		local tag = leftTag[i];
		self.btns[tag] = btn;
		-- local path = imgPath .. 'btn_' .. i .. '/';

		btn.node = cc.Node:create()
			:setPosition(btnNodePos[tag])
			:addTo(self.btnNode)

		btn.btn = ccui.Button:create('image/main/main_btn_bg.png');
		btn.btn:setTag(tag);
		btn.btn:addTo(btn.node);
		btn.btn:addClickEventListener(handler(self, self.btnClick));
		btn.btn:setAlphaTouchEnable(true);

		local btnBg = cc.Sprite:create('image/main/main_btn_'..tag..'.png')
		btnBg:addTo(btn.node);

		local btnName = cc.Label:createWithTTF( self.btnNodeName[tag], fonts.FONT_SIYUAN, 20)
			:enableOutline(cc.c3b(60, 60, 60), 2)
			:setPosition(0 , -btn.btn:getContentSize().height/2)
			:addTo(btn.node)

		self.btnList[tag] = btn.node
	end

	--	聊天,广告1,广告2
	local rightTag = {FightMenuTag.TAG_CHAT,FightMenuTag.TAG_ADMOB_BUFF,FightMenuTag.TAG_ADMOB_REWARD} --FightMenuTag.TAG_ADMOB_GACHA
	if global_branch.branch_open_chat ~= true  then
		rightTag = {FightMenuTag.TAG_ADMOB_BUFF,FightMenuTag.TAG_ADMOB_REWARD} --,FightMenuTag.TAG_ADMOB_GACHA
	end
	local len = #rightTag;
	for i=1, len do
	-- for i,v in ipairs(rightTag) do
		local btn = {}
		local tag = rightTag[i];

		self.btns[tag] = btn;

		btn.node = cc.Node:create();
		btn.node:addTo(self.btnNode)
		btn.node:setPosition(btnNodePos[tag])

		btn.btn = ccui.Button:create('image/main/main_btn_'..tag..'.png');
		btn.btn:setTag(tag);
		btn.btn:addTo(btn.node);
		btn.btn:addClickEventListener(handler(self, self.btnClick));
		btn.btn:setAlphaTouchEnable(true);

		if self.btnNodeName[tag] then
			local title = cc.Label:createWithTTF(self.btnNodeName[tag], fonts.FONT_SIYUAN, 20)
			:addTo(btn.node)
			:setPosition(0 , -25)
			:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)
		end

		btn.lock = cc.Sprite:create('image/lock/item_bg5.png')
		btn.lock:addTo(btn.node);
		btn.lock:setVisible(false);

		self.btnList[tag] = btn.node
	end

	if self.btns[FightMenuTag.TAG_RANDOM_BOSS] then --感知按钮相关
		self.randomBossProgressBg = cc.Sprite:create(imgPath.."bar_1_1.png");
		self.randomBossProgressBg:addTo(self.btns[FightMenuTag.TAG_RANDOM_BOSS].node);
		self.randomBossProgressBg:setPosition(0,-60);

		local progress = cc.Sprite:create(imgPath.."bar_1_2.png");
		self.randomBossProgress = cc.ProgressTimer:create(progress);
		self.randomBossProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR);
		self.randomBossProgress:setMidpoint(cc.p(0, 0));
		self.randomBossProgress:setBarChangeRate(cc.p(1, 0));
		self.randomBossProgress:setPosition(self.randomBossProgressBg:getContentSize().width/2,self.randomBossProgressBg:getContentSize().height/2);
		self.randomBossProgress:setPercentage(10);
		self.randomBossProgress:addTo(self.randomBossProgressBg);

		self.randomBossLabel = cc.Label:createWithTTF('999/9999', fonts.FONT_JP, 16);
		self.randomBossLabel:enableOutline(cc.c3b(60, 60, 60), 2);
		self.randomBossLabel:setPosition(0, -60);
		self.randomBossLabel:addTo(self.btns[FightMenuTag.TAG_RANDOM_BOSS].node);
		self.randomBossLabel:setVisible(false)
	end
	-- 广告角标                   
	local tempTag = {FightMenuTag.TAG_ADMOB_REWARD,FightMenuTag.TAG_ADMOB_BUFF,FightMenuTag.TAG_ADMOB_GACHA}
	for k,tag in pairs(tempTag) do
		if tag == FightMenuTag.TAG_ADMOB_GACHA then
			if self.btns[tag] then
				local tempNode = self.btns[tag].node
				local timebg = cc.Sprite:create('image/commonImg/admob_timebg.png')
					:addTo(tempNode)
					:setPosition(cc.p(0, 20))
				local timebar = cc.ProgressTimer
				local barImg = cc.Sprite:create('image/commonImg/admob_timebar.png')
				timebar = cc.ProgressTimer:create(barImg)
					:addTo(tempNode)
					:setAnchorPoint(cc.p(1, 0.5))
					:setPosition(cc.p(0, 20))
					:setBarChangeRate(cc.p(1, 0))
					:setType(cc.PROGRESS_TIMER_TYPE_BAR)
					:setMidpoint(cc.p(1, 0.5))
					:setPercentage(50)
				local clock = cc.Sprite:create('image/commonImg/admob_clock.png')
					:addTo(tempNode)
					:setPosition(cc.p(-20, 20))
				local getLabel = cc.Label:createWithTTF(LANG.UI_ADMOB_7, fonts.FONT_SIYUAN, 16)
					:enableOutline(cc.c3b(60, 60, 60), 2)
					:setPosition(0, -50)
					:addTo(tempNode)
				self.btns[tag].bar = timebar;
				self.btns[tag].switchGet = function(flag)
					getLabel:setVisible(flag)
					timebg:setVisible(not flag)
					timebar:setVisible(not flag)
					clock:setVisible(not flag)
				end
				self.btns[tag].switchGet(true)
			end
		else
			if self.btns[tag] then
				local tempNode = self.btns[tag].node
				local flagBg = cc.Sprite:create('image/commonImg/admob_flag.png')
					:addTo(tempNode)
					:setPosition(cc.p(-16,13 ))
				local timesLabel = cc.Label:createWithTTF('10', fonts.FONT_JP, 13)
					-- :enableOutline(cc.c3b(60, 60, 60), 2)
					:setPosition(-16,13 )
					:addTo(tempNode)
					-- :setVisible(false)
				self.btns[tag].times = timesLabel;
			end
		end
	end
	--每日奖励
	self:initNewOnlineNode()
	self:initNewUserGiftNode()
	self:initFightGiftNode();
	--阵型克制
	self:initRestraintNode();
	--
	self.bossWarning = cc.Sprite:create('image/fight/boss_warning.png');
	self.bossWarning:setVisible(false);
	self.bossWarning:setLocalZOrder(100);
	self.bossWarning:addTo(self.root);

	-- skill effect
	self.skillEffectNode = cc.Node:create();
	self.skillEffectNode:setVisible(false);
	self.skillEffectNode:setPosition(-StageSize.width/2+130, 163);
	self.skillEffectNode:addTo(self.root);

	-- boss icon
	self:initBossBloodNode()

	self:initDamageNode();

	self.fightPlayBtnNode = cc.Node:create()
	self.fightPlayBtnNode:addTo(self.root)
	self.fightPlayBtnNode:setPosition( cc.p(269 , 270) )
	self.fightPlayBtnNode:setVisible(false)

	local fightBtnsBg = cc.Sprite:create('image/main/fight_set_btn_bg.png')
	fightBtnsBg:addTo( self.fightPlayBtnNode )

	self:initTimeScaleBtn();

	self:initGotoBtn();
	self:initPlayDrawingBtn();

	self:initTimeNode();

	self:initRoleBookNode();

	self.helpBtn = ccui.Button:create('image/commonImg/help_1.png','image/commonImg/help_1.png')
	self.helpBtn:addTo(self.root);
	self.helpBtn:setPosition(266 , StageSize.height/2-120-NOTCH_HEIGHT)
	self.helpBtn:addClickEventListener(function() 
		UIManager:push('HelpPanel',self.name);
	end)

	self:initMultArenaNode();

	TimeManager:registUpdateEvent('FightPanel',handler(self,self.onEnterFrameUpdate), true);
end

function FightPanel:initBossBloodNode()
	self.bossBloodIconNode = cc.Node:create()
		:setVisible(false)
		:setPosition(StageSize.width/2-10, -35)
		:addTo(self.root)

	self.bloodNode = cc.Node:create()
	:addTo(self.bossBloodIconNode)

	self.guildBloodNode = cc.Node:create()
	:addTo(self.bloodNode)
	:setPosition(0, -25)
	local tmpBg = cc.Sprite:create(imgPath .. 'boss_guild_bg.png')
		:setAnchorPoint(cc.p(1,0.5))
		:addTo(self.guildBloodNode)
	local tmpLabel = cc.Label:createWithTTF(LANG.UI_L_BARMOR, fonts.FONT_SIYUAN, 12)
		:addTo(self.guildBloodNode)
		:setPosition( -50, -8 )


	local bossBloodBg = cc.Sprite:create(imgPath .. 'boss_hp_bg.png')
		:setAnchorPoint(cc.p(1,0.5))
		:addTo(self.bloodNode)

	local progressBar = cc.Sprite:create(imgPath .. 'boss_hp_bar.png')
	progressBar:setContentSize(cc.size(216, 14));
	self.bossBloodTimer = cc.ProgressTimer:create(progressBar)
		:addTo(self.bloodNode)
		:setAnchorPoint(cc.p(1, 0.5))
		:setPosition(cc.p(-5, -17))
		:setBarChangeRate(cc.p(1, 0))
		:setType(cc.PROGRESS_TIMER_TYPE_BAR)
		:setMidpoint(cc.p(1, 0.5))
		:setPercentage(50)

	self.guildBodyBN = cc.Node:create()
	:addTo(self.bossBloodIconNode)
	:setPosition(0, -25)
	:setVisible(true)
	local tmpBg = cc.Sprite:create(imgPath .. 'boss_damage_bg.png')
		:setAnchorPoint(cc.p(1,0.5))
		:addTo(self.guildBodyBN)
	local tmpLabel = cc.Label:createWithTTF(LANG.UI_L_OneDAMAGE, fonts.FONT_SIYUAN, 16)
		:addTo(self.guildBodyBN)
		:setPosition( -212, 0 )
	self.guildDamageLabel = cc.Label:createWithTTF('9999999999', fonts.FONT_SIYUAN, 20)
		:addTo(self.guildBodyBN)
		:setTextColor(cc.c3b(255,210,70))
		:enableOutline(cc.c3b(80, 50, 0), 2)
		:setPosition( -90, 0 )

	local bossIcon = cc.Sprite:create(imgPath .. 'boss_icon.png')
		:setPosition(-33, 10)
		:addTo(self.bossBloodIconNode)

	self.bossBuffNode = cc.Node:create()
		:addTo(self.bossBloodIconNode)
		:setPosition(-72, 17)

	self.buffItemList = {}

end

function FightPanel:updateFighterBuff( args )
	if self.bossBloodIconNode:isVisible() == false then
		return
	end
	if args == nil then
		return
	end
	if args.isEnemy then
		if args.isAdd then
			self:addBuff( args.bid, args.btype, args.icon )
		else
			self:removeBuff( args.btype )
		end
	end
	
end

function FightPanel:addBuff( bid, btype, icon )

	local idx = 0
	for k,v in pairs(self.buffItemList) do
		if v.type == btype then
			idx = k
			break
		end
	end

	if idx == 0 then
		local item = cc.Node:create()
		local bgName = 'buff_g.png'
		local txtColor = cc.c3b(215,255,200)
		local outLine = cc.c3b(35,120,35)
		if btype==2 or btype==4 or btype==6 or btype==7 or btype==11 or btype==14 or btype==18 then
			bgName = 'buff_r.png'
			txtColor = cc.c3b(255,200,200)
			outLine = cc.c3b(175,10,5)
		end
		local bg = cc.Sprite:create('image/fight/footernew/' .. bgName)
			:setPosition(0, 0)
			:addTo(item)

		if icon and icon ~= '' then
			local iconspr = cc.Sprite:create('image/icon_buff/' .. icon..'.png')
				:setPosition(0, -9)
				:addTo(item)
		end
		local label = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 13)
		:addTo(item)
		:setPosition(5 , -18)
		:setTextColor(txtColor)
		:enableOutline(outLine,2)

		item:addTo(self.bossBuffNode)

		table.insert( self.buffItemList, {type=btype, node=item, label = label, num = 1 } )

		self:updateBuffItemPos()

	else
		self.buffItemList[idx].num = self.buffItemList[idx].num + 1
		self.buffItemList[idx].label:setString( ''..self.buffItemList[idx].num )

	end

end

function FightPanel:removeBuff( btype )
	-- body
	local idx = 0
	for k,v in pairs(self.buffItemList) do
		if v.type == btype then
			idx = k
			break
		end
	end
	if idx == 0 then
		return
	end
	if self.buffItemList[idx] then
		self.buffItemList[idx].num = self.buffItemList[idx].num - 1
		if self.buffItemList[idx].num <= 0 then
			if self.buffItemList[idx].node then
				self.buffItemList[idx].node:removeFromParent()
			end
			table.remove( self.buffItemList, idx )
			self:updateBuffItemPos()
		elseif self.buffItemList[idx].num==1 then
			self.buffItemList[idx].label:setString( '' )
		else
			self.buffItemList[idx].label:setString( ''..self.buffItemList[idx].num )
		end
	end

end

function FightPanel:updateBuffItemPos()
	local idx = 0
	for k,v in pairs(self.buffItemList) do
		v.node:setPosition(0-24*idx,0)
		idx = idx + 1
	end
end


function FightPanel:initTimeScaleBtn()
	if tools.ConfigStateTools:checkLockById(Enum:getTable('ConfigState').FightSpeedUp) then
		return
	end
	self.speedNode = cc.Node:create();
	self.speedNode:addTo(self.fightPlayBtnNode)
	self.speedNode:setPosition( cc.p(0 , 88) )

	self.speedbtn = ccui.Button:create('image/main/main_f_btn_2.png');
	-- btn:setTag(tag);
	self.speedbtn:addTo(self.speedNode);
	self.speedbtn:addClickEventListener(handler(self, self.SpeedUp));
	self.speedbtn:setAlphaTouchEnable(true);

	self.SpeedUpTitle = cc.Label:createWithTTF('X1', fonts.FONT_SIYUAN, 20)
	:addTo(self.speedNode)
	:setPosition(0 , -25)
	:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)

end

function FightPanel:initGotoBtn()
	if tools.ConfigStateTools:checkLockById(Enum:getTable('ConfigState').FightGoToRoleShow) then
		return
	end
	self.gotoNode = cc.Node:create();
	self.gotoNode:addTo(self.fightPlayBtnNode)
	self.gotoNode:setPosition( cc.p(0 , -88) )

	self.gotobtn = ccui.Button:create('image/main/main_f_btn_1.png');
	-- btn:setTag(tag);
	self.gotobtn:addTo(self.gotoNode);
	self.gotobtn:addClickEventListener(handler(self, self.GoToView));
	self.gotobtn:setAlphaTouchEnable(true);

	-- local btnBg = cc.Sprite:create('image/main/main_f_btn_1.png')
	-- btnBg:addTo(self.gotoNode);

	self.gotolabel = cc.Label:createWithTTF(LANG.UI_L_FIGHT_AUTO[1], fonts.FONT_SIYUAN, 20)
	:addTo(self.gotoNode)
	:setPosition(0 , -25)
	:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)

end

function FightPanel:initPlayDrawingBtn()
	if tools.ConfigStateTools:checkLockById(Enum:getTable('ConfigState').FightGoToRoleShow) then
		return
	end
	self.drawNode = cc.Node:create();
	self.drawNode:addTo(self.fightPlayBtnNode)
	self.drawNode:setPosition( cc.p(0 , 0) )

	self.drawbtn = ccui.Button:create('image/main/main_f_btn_3.png');
	self.drawbtn:addTo(self.drawNode);
	self.drawbtn:addClickEventListener(handler(self, self.playDrawing));
	self.drawbtn:setAlphaTouchEnable(true);

	-- local btnBg = cc.Sprite:create('image/main/main_f_btn_1.png')
	-- btnBg:addTo(self.drawNode);

	self.drawlabel = cc.Label:createWithTTF(LANG.UI_L_FIGHT_SET[2], fonts.FONT_SIYUAN, 20)
	:addTo(self.drawNode)
	:setPosition(0 , -25)
	:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)

end

function FightPanel:initDamageNode()
	self.totalDamageNode = cc.Node:create()
		:setVisible(false)
		:setPosition(StageSize.width/2 - 80, 70)
		:setLocalZOrder(999)
		:addTo(self.root)
		
	local bg = cc.Sprite:create(imgPath .. 'lanjibg.png')
		:setPosition(0, 0)
		:addTo(self.totalDamageNode)

	self.totalDamLianjiSpr = cc.Sprite:create(imgPath .. 'lianji.png')
		:setPosition(-10, 40)
		:setAnchorPoint(0, 0.5)
		:addTo(self.totalDamageNode)

	self.totalDamSpr = cc.Sprite:create(imgPath .. 'shanghai.png')
		:setPosition(-45, -15)
		:setAnchorPoint(cc.p(1,0.5))
		:addTo(self.totalDamageNode)


	local fnt = 'font/num_n_dam.fnt';
	self.totalLianji = cc.Label:createWithBMFont(fnt, '')
		:addTo(self.totalDamageNode)
		:setAnchorPoint(cc.p(1,0.5))
		:setPosition(-15, 40)


	self.totalDamageLabel = cc.Label:createWithBMFont(fnt, '')
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:setAnchorPoint(cc.p(1, 0.5))
		:setPosition(60, -15)
		:setBMFontSize(26)
		:addTo(self.totalDamageNode)

	self.totalDamageFid = 0;
	self.totalDamageSkillId = 0;
	self.totalDamageValue = 0;
	self.totalCurValue = 0;
	self.totalDamageCnt = 0;
	self.totalDamList = {};

end

function FightPanel:initMultArenaNode()
	local topNode = cc.Node:create()
		:setVisible(false)
		:setPosition(0, StageSize.height/2-120-NOTCH_HEIGHT)
		:setLocalZOrder(999)
		:addTo(self.root)
	self.multArenaNode = topNode;

    local topBg = cc.Sprite:create('image/arena/arenaMult/am_ft_top.png')
    topBg:addTo(topNode)
    self.rank1Label = cc.Label:createWithTTF('排名 191' ,  fonts.FONT_SIYUAN , 18);
    self.rank1Label:addTo(topNode);
    self.rank1Label:setPosition(-250 , 0);
    self.rank1Label:enableOutline(cc.c3b(0,74,226),1);
    self.rank2Label = cc.Label:createWithTTF('排名 192' ,  fonts.FONT_SIYUAN , 18);
    self.rank2Label:addTo(topNode);
    self.rank2Label:setPosition(250 , 0);
    self.rank2Label:enableOutline(cc.c3b(226,0,96),1);

    self.name1Label = cc.Label:createWithTTF('我是一只小母猫' ,  fonts.FONT_SIYUAN , 18);
    self.name1Label:addTo(topNode);
    self.name1Label:setPosition(-100 , 0);
    -- self.name1Label:enableOutline(cc.c3b(0,74,226),1);
    self.name2Label = cc.Label:createWithTTF('天生我材必有用' ,  fonts.FONT_SIYUAN , 18);
    self.name2Label:addTo(topNode);
    self.name2Label:setPosition(100 , 0);
    -- self.name2Label:enableOutline(cc.c3b(226,0,96),1);

    local rightNode = cc.Node:create()
    rightNode:addTo(topNode)
    rightNode:setPosition(StageSize.width/2-120, -80)

    self.enemyTeamList = {}
    for i=1,3 do
        local teamItems = {}
        
        local teamNode = cc.Node:create()
        teamNode:addTo(rightNode)
        teamNode:setPosition(0, 0-(i-1)*60)

        local teamBg = cc.Sprite:create('image/arena/arenaMult/am_right_bg.png');
        teamBg:addTo(teamNode);

        local teamTicon = cc.Sprite:create('image/arena/arenaMult/am_'..i..'.png')
        teamTicon:addTo(teamNode)
        teamTicon:setPosition(105, 0)

        for j=1, TEAM_MAX_MEMBER do
            local icon = require('app.template.ItemTemplate'):create();
            icon:addTo(teamNode);
            icon:setScale(0.3)
            icon:setPosition((j-1)*40-45, 0)
            table.insert(teamItems, icon)
        end

        local shadowSpr = cc.Scale9Sprite:create('image/commonImg/list_mask_2_4_0__23_34.png');
        shadowSpr:addTo(teamNode);
        shadowSpr:setContentSize(cc.size(40, 240))
        shadowSpr:setRotation(-90)
        shadowSpr:setVisible(false)
        local rtSpr = cc.Sprite:create('image/arena/arenaMult/am_lose_icon.png');
        rtSpr:addTo(teamNode);
        rtSpr:setPosition(40,0)
        rtSpr:setVisible(false)

        self.enemyTeamList[i] = {teamItems=teamItems, shadowSpr=shadowSpr, rtSpr=rtSpr};

    end

    local leftNode = cc.Node:create()
    leftNode:addTo(topNode)
    leftNode:setPosition(-StageSize.width/2+120, -80)

    self.selfTeamList = {}
    self.selfTeamNodeList = {}
    for i=1,3 do
        local teamItems = {}

        local teamNode = cc.Node:create()
        teamNode:addTo(leftNode)
        teamNode:setPosition(0, 0-(i-1)*60)

        local teamBg = cc.Sprite:create('image/arena/arenaMult/am_left_bg.png');
        teamBg:addTo(teamNode);

        local teamTicon = cc.Sprite:create('image/arena/arenaMult/am_'..i..'.png')
        teamTicon:addTo(teamNode)
        teamTicon:setPosition(-105, 0)

        for j=1, TEAM_MAX_MEMBER do
            local icon = require('app.template.ItemTemplate'):create();
            icon:addTo(teamNode);
            icon:setScale(0.3)
            icon:setPosition((j-1)*40-75, 0)
            table.insert(teamItems, icon)
        end

        local shadowSpr = cc.Scale9Sprite:create('image/commonImg/list_mask_2_4_0__23_34.png');
        shadowSpr:addTo(teamNode);
        shadowSpr:setContentSize(cc.size(40, 240))
        shadowSpr:setRotation(90)
        shadowSpr:setVisible(false)
        local rtSpr = cc.Sprite:create('image/arena/arenaMult/am_lose_icon.png');
        rtSpr:addTo(teamNode);
        rtSpr:setPosition(-40,0)
        rtSpr:setVisible(false)
        self.selfTeamList[i] = {teamItems=teamItems, shadowSpr=shadowSpr, rtSpr=rtSpr};
    end

    local spine_res = 'xiaojiesuan_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.multArenaSp = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.multArenaSp:addTo(self.multArenaNode)
		self.multArenaSp:setVisible(false)
		self.multArenaSp:setPosition(0, -(StageSize.height/2-120-NOTCH_HEIGHT))
    end
end

function FightPanel:registEvent()
	-- event
	EventManager:RegisterEvent(EventType.FightRoleEnter, handler(self, self.roleEnter));
	EventManager:RegisterEvent(EventType.FightAddLabel, handler(self, self.AddLabel));
	EventManager:RegisterEvent(EventType.FightPanelClearDamage, handler(self, self.ClearDamage));
	EventManager:RegisterEvent(EventType.FightHpChange, handler(self, self.ChangeAllyHp));
	EventManager:RegisterEvent(EventType.FightPanelInitRoundInfo, handler(self, self.fightInitInfo));
	EventManager:RegisterEvent(EventType.FightBossWarning, handler(self, self.playBossWarning));
	--EventManager:RegisterEvent(EventType.RecountInit, handler(self, self.initRecount));
	--EventManager:RegisterEvent(EventType.RecountUpdate, handler(self, self.updateRecount));
	EventManager:RegisterEvent(EventType.FightNext, handler(self, self.backAutoFight));
	EventManager:RegisterEvent(EventType.FightBegin, handler(self, self.leaveAutoFight));
	EventManager:RegisterEvent(EventType.AutoFightCoinScatter, handler(self, self.playCoinScatter));
	EventManager:RegisterEvent(EventType.RoundBossUpdate, handler(self, self.updateRandomBossComing));
	-- EventManager:RegisterEvent(EventType.AutoFightTimeScale, handler(self, self.showBuffNode))

	EventManager:RegisterEvent(EventType.FightPanelBtnRedDot, handler(self, self.updateBtnRedDot))
	
    EventManager:RegisterEvent(EventType.NetworkRetOnlineReward, handler(self, self.showOnlineNode));
    EventManager:RegisterEvent(EventType.FightPanelRandomBossPoint, handler(self, self.showRandomBossInfo));
    --EventManager:RegisterEvent(EventType.FightPanelTeamRestraintChange, handler(self, self.showRestraintInfo));
    
	EventManager:RegisterEvent(EventType.BossDamageDown, handler(self, self.updateBossDamage));
	
    EventManager:RegisterEvent(EventType.UIEventUnlockChange, handler(self, self.updateLevelLock));
	EventManager:RegisterEvent(EventType.FightPanelBtnVisible, function(visible)
		self.btnNode:setVisible(visible)

	end)

	-- EventManager:RegisterEvent(EventType.FightOver, handler(self, self.ResetTimeScale));

	EventManager:RegisterEvent(EventType.FightUpdateTime, function(time)
		self.restTimeLabel:setString(Sec2Time(time))
	end)

	EventManager:RegisterEvent(EventType.FightFooterBuff, handler(self, self.updateFighterBuff));

	EventManager:RegisterEvent(EventType.Die, handler(self, self.setFighterDie));

	EventManager:RegisterEvent(EventType.NetworkRetAdmobReward, handler(self, self.retAdmobReward));
	-- EventManager:RegisterEvent(EventType.NetworkRetAdmobInfo, handler(self, self.retAdmobInfo));

	EventManager:RegisterEvent('FightPanel:showMultArenaSubRt', handler(self, self.showMultArenaSubRt));
	EventManager:RegisterEvent('FightPanel:noticeShowNewGiftNode', handler(self, self.noticeShowNewGiftNode));

	
end

function FightPanel:initTimeNode()
	self.timeNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(-240,400)
		:setVisible(false)

	local timeBg = cc.Sprite:create('image/commonImg/fight_time_bg.png')
		:addTo(self.timeNode)

	self.restTimeLabel = cc.Label:createWithTTF('00:00:00', fonts.FONT_SIYUAN, 20)
		:enableOutline(cc.c3b(60, 60, 60) , 2)
		:setPosition(15, -3)
		:addTo(self.timeNode)

	self.exitNode = cc.Node:create()
	:addTo(self.timeNode)
	:setPosition( -20, -80 )
	:setVisible(false)

	local exitBtn = ccui.Button:create('image/commonImg/exit_bg.png');
	exitBtn:addTo(self.exitNode);
	exitBtn:addClickEventListener(handler(self,self.ExitFight));

	local exitBg = cc.Sprite:create('image/commonImg/exit_btn.png')
	:addTo(self.exitNode)
	:setPosition(0,10)

	local exitLabel = cc.Label:createWithTTF(LANG.UI_MAIL_9,fonts.FONT_SIYUAN,18 )
	:addTo(self.exitNode)
	:setPosition(0, -20)
	:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);

end

function FightPanel:ExitFight(sender, event)
	AudioManager:playEffect(Conf.buttonClick);
	--11581 是否要退出当前战斗？
	MessageBox:create(LANG.UI_FIGHT_96 , MessageBoxType.CANCEL_OK,function()
		FightManager:ForceExitFight()
	end);

end

function FightPanel:initNewOnlineNode()
	self.onlineMNode = cc.Node:create();
	self.onlineMNode:addTo(self.root);
	self.onlineMNode:setPosition(btnNodePos[FightMenuTag.TAG_ONLINE]);--cc.p(StageSize.width/2 - 58,StageSize.height/2 - 60 - 71)

	local onlineBtn = ccui.Layout:create();
	onlineBtn:setTouchEnabled(true)
	onlineBtn:setContentSize(cc.size(73,73))
	onlineBtn:setAnchorPoint(cc.p(0.5,0.5))
	onlineBtn:addTo(self.onlineMNode);
	onlineBtn:addClickEventListener(handler(self,self.btnClick));
	onlineBtn:setTag(FightMenuTag.TAG_ONLINE);

	local spine_res = 'xianshilibao_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.onlineAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.onlineAnimation:addTo(self.onlineMNode)
	else
        print('error : 缺少spine 资源')
    end

	local mLabel = cc.Label:createWithTTF(LANG.UI_L_ONLINE_RWD,fonts.FONT_SIYUAN,20)
		:addTo(self.onlineMNode)
		:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2)
		:setPosition(0 , -33)

	self.onlineMTime = cc.Label:createWithTTF('99:00:00',fonts.FONT_SIYUAN,15)
		:addTo(self.onlineMNode)
		:enableOutline(cc.c4b(50 , 50 , 50 , 255) , 2)
		:setPosition(0 , -58)

end

function FightPanel:initNewUserGiftNode()
	self.newGiftTimeNum = 0;
	self.newGiftNode = cc.Node:create();
	self.newGiftNode:addTo(self.root);
	self.newGiftNode:setPosition(btnNodePos[FightMenuTag.TAG_NEWGIFT]);--cc.p(StageSize.width/2 - 58,StageSize.height/2 - 60 - 71)

	local nGiftBtn = ccui.Layout:create();
	nGiftBtn:setTouchEnabled(true)
	nGiftBtn:setContentSize(cc.size(73,73))
	nGiftBtn:setAnchorPoint(cc.p(0.5,0.5))
	nGiftBtn:addTo(self.newGiftNode);
	nGiftBtn:addClickEventListener(handler(self,self.btnClick));
	nGiftBtn:setTag(FightMenuTag.TAG_NEWGIFT);

	local bgSpr = cc.Sprite:create('image/onlineReward/online_nbtn.png')
	bgSpr:addTo(self.newGiftNode)
	local iconSpr = cc.Sprite:create('image/main/gift_new.png')
	iconSpr:addTo(self.newGiftNode)

	local mLabel = cc.Label:createWithTTF('新手礼包',fonts.FONT_SIYUAN,20)
		:addTo(self.newGiftNode)
		:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2)
		:setPosition(0 , -33)

	self.newGiftTime = cc.Label:createWithTTF('99:00:00',fonts.FONT_SIYUAN,15)
		:addTo(self.newGiftNode)
		:enableOutline(cc.c4b(50 , 50 , 50 , 255) , 2)
		:setPosition(0 , -58)

	local locked = tools.ConfigStateTools:checkLockById(1109);
	if locked == false then
		model.ShopModel:reqShopGiftRecommend( 1 );
	end

end

function FightPanel:initFightGiftNode()
	self.fightGiftTimeNum = 0;
	self.fightGiftNode = cc.Node:create();
	self.fightGiftNode:addTo(self.root);
	self.fightGiftNode:setPosition(btnNodePos[FightMenuTag.TAG_FIGHTGIFT]);

	local nGiftBtn = ccui.Layout:create();
	nGiftBtn:setTouchEnabled(true)
	nGiftBtn:setContentSize(cc.size(73,73))
	nGiftBtn:setAnchorPoint(cc.p(0.5,0.5))
	nGiftBtn:addTo(self.fightGiftNode);
	nGiftBtn:addClickEventListener(handler(self,self.btnClick));
	nGiftBtn:setTag(FightMenuTag.TAG_FIGHTGIFT);

	local bgSpr = cc.Sprite:create('image/onlineReward/online_nbtn.png')
	bgSpr:addTo(self.fightGiftNode)
	local iconSpr = cc.Sprite:create('image/main/gift_fight.png')
	iconSpr:addTo(self.fightGiftNode)

	local mLabel = cc.Label:createWithTTF('战斗礼包',fonts.FONT_SIYUAN,20)
		:addTo(self.fightGiftNode)
		:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2)
		:setPosition(0 , -33)

	self.fightGiftTime = cc.Label:createWithTTF('99:00:00',fonts.FONT_SIYUAN,15)
		:addTo(self.fightGiftNode)
		:enableOutline(cc.c4b(50 , 50 , 50 , 255) , 2)
		:setPosition(0 , -58)

end

function FightPanel:initRestraintNode( )
	-- body
	self.restrNode = cc.Node:create();
	self.restrNode:addTo(self.root);
	self.restrNode:setPosition( -260,210 ) ;

	-- local bg = cc.Sprite:create('image/fight/footernew/restrain_bg.png')
	-- bg:addTo(self.restrNode)
	
	local bgBtn = ccui.Button:create('image/fight/footernew/restrain_bg.png');
	bgBtn:addTo(self.restrNode);
	bgBtn:addClickEventListener(handler(self,self.btnClick));
	bgBtn:setTag(FightMenuTag.TAG_RESTRAINT);

	self.n_restrain_icon = cc.Sprite:create('image/fight/footernew/restrain_icon_1.png')
		:addTo(self.restrNode)
		:setPosition(cc.p(0, 25))

	self.n_restain_label = cc.Label:createWithTTF(LANG.UI_L_RESTRAINT_SUB ,  fonts.FONT_SIYUAN , 20)
		:addTo(self.restrNode)
		:setPosition(cc.p(0, -5))
		:setTextColor(cc.c3b(255, 255, 255))
		:enableOutline(cc.c4b(0, 0, 0 , 255) , 2)

	local tmpH = 10
	local tmpS = -23
	local barbg1 = cc.Sprite:create('image/fight/footernew/restrain_bar_bg.png')
		:addTo(self.restrNode)
		:setPosition(cc.p(0, tmpS))

	local barbg2 = cc.Sprite:create('image/fight/footernew/restrain_bar_bg.png')
		:addTo(self.restrNode)
		:setPosition(cc.p(0, tmpS-tmpH))

	local barbg3 = cc.Sprite:create('image/fight/footernew/restrain_bar_bg.png')
		:addTo(self.restrNode)
		:setPosition(cc.p(0, tmpS-tmpH*2))

	local tmpX = -22.5
	self.bar_r = cc.Sprite:create('image/fight/footernew/restrain_r2.png')
		:addTo(self.restrNode)
		:setAnchorPoint(cc.p(0,0.5))
		:setPosition(cc.p(tmpX, tmpS))

	self.bar_g = cc.Sprite:create('image/fight/footernew/restrain_g1.png')
		:addTo(self.restrNode)
		:setAnchorPoint(cc.p(0,0.5))
		:setPosition(cc.p(tmpX, tmpS-tmpH))

	self.bar_b = cc.Sprite:create('image/fight/footernew/restrain_b3.png')
		:addTo(self.restrNode)
		:setAnchorPoint(cc.p(0,0.5))
		:setPosition(cc.p(tmpX, tmpS-tmpH*2))

end

function FightPanel:updateRestrainData(rtype, red, green, blue)
	-- body
	self.bar_r:setScaleX( red/4 )
	self.bar_g:setScaleX( green/4 )
	self.bar_b:setScaleX( blue/4 )

	local value = model.TeamModel:getRestraintAmplificationFactor(1, rtype)*100

	if value == 0 then --未激活
		self.n_restrain_icon:setTexture('image/fight/footernew/restrain_icon_1.png')
		self.n_restain_label:setString(LANG.UI_L_RESTRAINT_SUB)
		self.n_restain_label:enableOutline(cc.c4b(0, 0, 0 , 255) , 2)
		self.n_restain_label:setTextColor(cc.c3b(255, 255, 255))

	else
		self.n_restrain_icon:setTexture('image/fight/footernew/restrain_icon_2.png')
		self.n_restain_label:setString(value..'%UP')
		self.n_restain_label:enableOutline(cc.c4b(0, 0, 0 , 255) , 2)
		self.n_restain_label:setTextColor(cc.c3b(247, 229, 144))

	end
end

function FightPanel:initRandomBossSpine( ... )
	local spine_res = 'boss_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.randomBossSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.randomBossSpine:addTo(self.root)
		self.randomBossSpine:setPosition(cc.p(320 , 124))
	else
		print('error : 缺少spine 资源')
	end  
end

function FightPanel:showRestraintInfo(  )

	local tid = 1
	if self.roundInfo then
		tid = model.TeamModel:getTeamIdByFightType( self.roundInfo.fightType, self.roundInfo );
	end

	local restraintStr = model.TeamModel:getTeamRestaintString(tid);
	local restraintType,restraintTypeStr = model.TeamModel:getTeamRestaintTypeByString(restraintStr);

	if restraintType == -1 then
		-- 阵型无克制！！！
		restraintType = 0
		restraintTypeStr = '0'
	end

	local red,green,blue = model.TeamModel:getTeamRestaintNum(restraintStr);
	self:updateRestrainData( tid, red, green, blue )

end

function FightPanel:showOnlineNode( bTrue  )
	-- body
	if bTrue == nil then
		bTrue = true 
	end
	if not bTrue then
		self.onlineMNode:setVisible(false);
		TimeManager:registUpdateEvent("FightPanelOnline",nil);
		return ;
	end

	local nextUnrewardId = model.OnlineModel:getNextUnRewardId();
	local nextStatus = model.OnlineModel:getStatusById(nextUnrewardId);
	if nextStatus == true then
		self.onlineMNode:setVisible(false);
		TimeManager:registUpdateEvent("FightPanelOnline",nil);
		return ;
	end

	self.onlineAnimationType = 0;
	
	TimeManager:registUpdateEvent('FightPanelOnline',handler(self,self.updateOnline));
	local visible = tools.ConfigStateTools:checkVisibleByType(Enum:getTable('ConfigStateType').FIGHT , FightMenuTag.TAG_ONLINE)
	self.onlineMNode:setVisible(visible);
end

function FightPanel:noticeShowNewGiftNode()
	if self.newGiftNode:isVisible() then
		self:showNewGiftNode()
	end
end

function FightPanel:showNewGiftNode(bTrue)
	if bTrue == nil then
		bTrue = true 
	end
	self.newGiftTimeNum = 0;
	self.fightGiftTimeNum = 0;
	if not bTrue then
		self.newGiftNode:setVisible(false);
		self.fightGiftNode:setVisible(false);
		return;
	end
	if model.ShopModel:getShopGiftData(3) then
		self.newGiftNode:setVisible(true);
		local newgData = model.ShopModel:getShopGiftData(3);
		self.newGiftTimeNum = newgData.timestamp;
	else
		self.newGiftNode:setVisible(false);
	end

	if #model.ShopModel:getFightGiftList()>0 then
		self.fightGiftNode:setVisible(true)
		local list = model.ShopModel:getFightGiftList();
		self.fightGiftTimeNum = list[1].gift_end_time;
		if self.newGiftNode:isVisible() then
			self.fightGiftNode:setPosition(btnNodePos[FightMenuTag.TAG_FIGHTGIFT]);
		else
			self.fightGiftNode:setPosition(btnNodePos[FightMenuTag.TAG_NEWGIFT]);
		end
	else
		self.fightGiftNode:setVisible(false);
	end
end

function FightPanel:onEnterFrameUpdate(dt)
	if self.beVisble == false then
		return;
	end
	local curTime = TimeManager:getCurTime()
	if self.newGiftTimeNum>0 then
		local secondOff = math.ceil(self.newGiftTimeNum - curTime)
		local timeInfo = secondOff>0 and Sec2Time(secondOff) or '';
		self.newGiftTime:setString(timeInfo)
	end

	if self.fightGiftTimeNum>0 then
		local secondOff = math.ceil(self.fightGiftTimeNum - curTime)
		local timeInfo = secondOff>0 and Sec2Time(secondOff) or '';
		self.fightGiftTime:setString(timeInfo)
	end

end

function FightPanel:showRandomBossInfo( )
	local locked,_ = self:checkClickLock(FightMenuTag.TAG_RANDOM_BOSS,true);
	self.randomBossProgressBg:setVisible(not locked);
	-- self.randomBossLabel:setVisible(not locked);
	if locked then
		return ;
	end


	local bossBuyNow,bossBuyMax = model.RoundBossModel:getBuyItemTotal()
	local progress = 0;
	if bossBuyNow >= bossBuyMax then
		progress = 100;
	else
		progress = bossBuyNow/bossBuyMax * 100;
	end
	self.randomBossProgress:setPercentage(progress);
	self.randomBossLabel:setString(tostring(bossBuyNow)..'/'..tostring(bossBuyMax));
end

function FightPanel:onEnter()
	UIManager:push('FightMapPanel');
	UIManager:push('FightScenePanel');
	--UIManager:push('FightFooterPanel');
	if not model.HarvestModel:IsOver() 
	and tools.ConfigStateTools:checkVisibleByType(Enum:getTable('ConfigStateType').FIGHT , FightMenuTag.TAG_HARVERT) then
		--UIManager:push('HarvestPanel');
	end


	self:updateBtnRedDot(FightMenuTag.TAG_ROUND_REWARD);
	self:updateBtnRedDot(FightMenuTag.TAG_ITEM_USE);
	self:updateBtnRedDot(FightMenuTag.TAG_RANDOM_BOSS);
	
	self:updateLock(FightMenuTag.TAG_ITEM_USE);
	self:updateLock(FightMenuTag.TAG_RANDOM_BOSS);
	self:updateLock(FightMenuTag.TAG_ADMOB_BUFF);
	self:updateLock(FightMenuTag.TAG_ADMOB_REWARD);

	self:showRandomBossInfo();
	self.bIsPlayAdmob = false;
	self.selectAdmobId = nil ;
	self:showAdmobInfo();
	EventManager:FireEvent(EventType.UIHeaderPanelBackEvent, handler(self, self.onClickback));

	self.bInFight = true ;
	if FightManager:getIsNotFight() then
		self.bInFight = false;
		self:showOnlineNode();
		self:showNewGiftNode();
		self.helpBtn:setVisible(true);
		self:updateBtnVisible();
		self.restrNode:setVisible(false);
		self.fightPlayBtnNode:setVisible(false)
		self.timeNode:setVisible(false)
	else
		self:showOnlineNode(false);
		self:showNewGiftNode(false);
		self.restrNode:setVisible(true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.helpBtn:setVisible(false);
	end
	EventManager:FireEvent(EventType.UIFooterUpdateSelect,3);
	self.root:setVisible(true);
	self.beVisble = true

	if FightManager:getAutoRelease() then
		self.gotobtn:loadTextureNormal('image/main/main_f_btn_1_0.png')
		self.gotolabel:setString(LANG.UI_L_FIGHT_AUTO[2])
	else
		self.gotobtn:loadTextureNormal('image/main/main_f_btn_1.png')
		self.gotolabel:setString(LANG.UI_L_FIGHT_AUTO[1])
	end	
	self:updatePlayDrawing();

end

function FightPanel:updateBtnVisible()
	tools.ConfigStateTools:setVisibleByType(Enum:getTable('ConfigStateType').FIGHT , self.btnList);
end

function FightPanel:onLeave()
	UIManager:pop('FightMapPanel');
	UIManager:pop('FightScenePanel');
	--UIManager:pop('FightFooterPanel');
	if not model.HarvestModel:IsOver() 
	and tools.ConfigStateTools:checkVisibleByType(Enum:getTable('ConfigStateType').FIGHT , FightMenuTag.TAG_HARVERT) then
--		UIManager:pop('HarvestPanel');
	end

	AudioManager:SetState(Enum:getTable('GameState').Main);
	AudioManager:playMusic(Conf.main, 2, true);
	self.root:setVisible(false);
	self.beVisble = false
end

function FightPanel:onClickback()
	UIManager:switch('MainPanel');
end


function FightPanel:fightInitInfo(info)
	--self.round_id = info.roundIndex;
	for k, v in pairs(self.btns) do
		self:showBtnNodeVisible(k,false);
		--v.node:setVisible(false);
	end
	self.bInFight = true;

	self.restrNode:setVisible(true);
	self:showOnlineNode(false);
	self:showNewGiftNode(false);
	self.helpBtn:setVisible(false);
	-- self:showBuffNode()

	self.fightPlayBtnNode:setVisible(false)
	self.timeNode:setVisible(false)
	self.exitNode:setVisible(false)
	self.multArenaNode:setVisible(false);

	self.timeNode:setPosition(-240,400)
	self.restrNode:setPosition( -260,210 ) ;
	-- self.btnList[FightMenuTag.TAG_BARE_COLLECT].node:setVisible(false)
	local exitNodeShow = not model.GuideModel:isInGuide(); -- 引导期间不能退出战斗

	if info.fightType == FightType.Boss then
		-- self.btns[FightMenuTag.TAG_CHAT].node:setVisible(true);
		-- self:showBtnNodeVisible(FightMenuTag.TAG_CHAT,true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.exitNode:setVisible(exitNodeShow) -- true 

	elseif info.fightType == FightType.RandomBoss then
		-- self:showBtnNodeVisible(FightMenuTag.TAG_CHAT,true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.exitNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

		self.bossBloodIconNode:setVisible(true)
		self.bloodNode:setVisible(true)
		self.guildBodyBN:setVisible(false)
		self.guildBloodNode:setVisible(false)

		self.buffItemList = {}
		self.bossBuffNode:removeAllChildren()

	elseif info.fightType == FightType.GuildBoss then
		-- self:showBtnNodeVisible(FightMenuTag.TAG_CHAT,true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		--self.exitNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

		if model.GuildModel:IsFightBossBody() then
			self.bloodNode:setVisible(false)
			self.guildBloodNode:setVisible(false)
			self.guildBodyBN:setVisible(true)
		else 
			self.bloodNode:setVisible(true)
			self.guildBloodNode:setVisible(true)
			self.guildBodyBN:setVisible(false)
		end

		self.bossBloodIconNode:setVisible(true)
		self.buffItemList = {}
		self.bossBuffNode:removeAllChildren()

	elseif info.fightType == FightType.TrialTower then
		-- self:showBtnNodeVisible(FightMenuTag.TAG_CHAT,true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.exitNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

	elseif info.fightType == FightType.Story then
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.exitNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

	elseif info.fightType == FightType.CopyRound then
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.exitNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

	elseif info.fightType == FightType.DailyRound then
		-- self:showBtnNodeVisible(FightMenuTag.TAG_CHAT,true);
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)

	elseif info.fightType == FightType.Event then
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)
	elseif info.fightType == FightType.Arena then
		self.fightPlayBtnNode:setVisible(true)
		self.timeNode:setVisible(true)
		
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)
	elseif info.fightType == FightType.MultArena then
		self.fightPlayBtnNode:setVisible(false)
		self.timeNode:setVisible(true)
		
		self.mapPanel:showCoverMap(info.bg)
		self.scenePanel:initBg(info.scene)
		self:initMultArenaInfo(info.subIndex);

		self.timeNode:setPosition(-240, StageSize.height/2-120-NOTCH_HEIGHT - 250)
		self.restrNode:setPosition(-260, StageSize.height/2-120-NOTCH_HEIGHT - 350)

	end

	self.roundInfo = info;

	if self.fightPlayBtnNode:isVisible() then
		self:ResetTimeScale()

		if model.GuideModel:isInGuide() then
			if self.roundInfo.roundIndex == 10101 then
				self.fightPlayBtnNode:setVisible(false)
			else
				local rVal = model.UserDefaultModel:getDataByKey('remind_FightPanelGotoView');
				if rVal ~= 1 then
					self.fightPlayBtnNode:runAction( 
						cc.Sequence:create( 
							cc.DelayTime:create(0.5),
							cc.CallFunc:create(function()
								local wpt = self.gotoNode:getParent():convertToWorldSpace(cc.p(self.gotoNode:getPosition()));
								EventManager:FireEvent(EventType.UIGuidePanel, wpt, LANG.UI_REMIND_2);
								GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu23);
								end) 
							) 
						)
				end
			end
		end
	end

	self:showRestraintInfo();
end

function FightPanel:showBtnNodeVisible( btnTag, bTrue )
	if bTrue and not tools.ConfigStateTools:checkVisibleByType(Enum:getTable('ConfigStateType').FIGHT , btnTag) then
		self.btns[btnTag].node:setVisible(false);
		return;
	end

	if self.btns[btnTag] and self.btns[btnTag].node ~= nil then
		self.btns[btnTag].node:setVisible(bTrue);
	end
end

function FightPanel:roleEnter(fighter, pos)
	if fighter then
		self.scenePanel:addRole(fighter, pos);
	end
end

function FightPanel:AddLabel(args)
	self.scenePanel:AddLabel(args);

	self:ShowTotalDamage( args );
end

function FightPanel:ClearDamage()
	-- body
	if self.totalDamageNode:isVisible() then
		self.totalDamageNode:runAction(
			cc.Sequence:create( cc.DelayTime:create(1), cc.CallFunc:create(function()
								self.totalDamageFid = 0;
								self.totalDamageSkillId = 0;
								self.totalDamageValue = 0;
								self.totalCurValue = 0;
								self.totalDamageCnt = 0;
								self.totalCurValue = 0;
								self.totalDamList = {};

								self:EndShowTotalDam();
								end) )
		)
	end
	
end

function FightPanel:ShowBeginTotalDam()
	self.totalDamageNode:stopAllActions()
	self.totalDamageNode:setVisible(true)
	self.totalDamageNode:setPosition(StageSize.width/2 + 220, 70)
	self.totalDamSpr:setOpacity(0)
	self.totalDamSpr:setPosition(-45-30, -15)

	self.totalDamLianjiSpr:setOpacity(0)
	self.totalDamLianjiSpr:setPosition(-10+30, 40)

	local move1 = cc.EaseBackOut:create( cc.MoveTo:create(0.24, cc.p(StageSize.width/2 - 80, 70)) )
	self.totalDamageNode:runAction( cc.Sequence:create( move1, cc.CallFunc:create(function()
			self.totalDamSpr:runAction( 
						cc.Spawn:create(
								cc.FadeIn:create(0.1),
								cc.MoveTo:create(0.1, cc.p(-45,-15))
								) 
				)

			self.totalDamLianjiSpr:runAction( 
						cc.Spawn:create(
								cc.FadeIn:create(0.1),
								cc.MoveTo:create(0.1, cc.p(-10,40))
								) 
				)
		end),
		cc.DelayTime:create(0.1),
		cc.CallFunc:create( function()
			self:playOnceDamage()
		end )

	 ) )

end

function FightPanel:EndShowTotalDam()
	-- body
	local move1 = cc.Spawn:create(
								cc.FadeOut:create(0.24),
								cc.MoveTo:create(0.24, cc.p(StageSize.width/2 + 300, 70))
								) 
	self.totalDamageNode:runAction( cc.Sequence:create( cc.DelayTime:create(0.5), move1, cc.CallFunc:create(function()
		self.totalDamageNode:setVisible(false)
		end) ) )
end

function FightPanel:totalLianjiAni(num, isLast)
	local showNum = ConvertStrToDigitUnit(num);
	self.totalLianji:setString( showNum );
	self.totalLianji:stopAllActions()
	local startScale, endScale = 1.2, 1.0;
	if isLast then
		startScale, endScale = 2.0, 1.5;
	end
	self.totalLianji:setScale(startScale)
	local move1 = cc.Spawn:create(
								cc.FadeIn:create(0.1),
								cc.ScaleTo:create(0.1, endScale)
								) 
	self.totalLianji:runAction( cc.Sequence:create(move1, cc.CallFunc:create( function()
		self:playOnceDamage()
	end ) )  )
end

function FightPanel:totalDamLabelAni()

	self.totalDamageLabel:stopAllActions()
	local idx = 1;
	local act = cc.Sequence:create( cc.DelayTime:create(0.01), cc.CallFunc:create(function()
		self.totalCurValue = self.totalCurValue + self.argDamageValue + math.random(-self.argDamageOffValue, self.argDamageOffValue)
		idx = idx + 1;
		if self.totalCurValue > self.totalDamageValue or idx==5 then
			self.totalCurValue = self.totalDamageValue;
		end

		local showNum = tostring(self.totalCurValue);
		self.totalDamageLabel:setString( showNum );

		self.totalDamSpr:setPositionX( 60-self.totalDamageLabel:getContentSize().width-5 )

		end))

	self.totalDamageLabel:runAction( cc.Repeat:create(act ,5) );

end

function FightPanel:playOnceDamage()
	-- body
	if #self.totalDamList == 0 then
		self.damagePlayEnd = true;
		self:ClearDamage();
		return;
	end
	local value = self.totalDamList[ #self.totalDamList ];
	table.remove( self.totalDamList );

	self.totalDamageNode:stopAllActions();
	self.totalDamageValue = self.totalDamageValue + value;
	self.argDamageValue = math.floor( (self.totalDamageValue - self.totalCurValue) / 5 );
	self.argDamageOffValue = math.floor( self.argDamageValue*0.2 );
	self.totalDamageCnt = self.totalDamageCnt + 1;
	self:totalDamLabelAni();
	local isLast = false;
	if #self.totalDamList==0 then
		isLast = true;
	end
	self:totalLianjiAni(self.totalDamageCnt, isLast);

end

function FightPanel:ShowTotalDamage( args )
	-- body
	if args.hitType and args.hitType==0 then
		return
	end
	if args.fid and args.skillId then
		if args.fid ~= self.totalDamageFid or args.skillId ~= self.totalDamageSkillId then
			self.totalDamageFid = args.fid;
			self.totalDamageSkillId = args.skillId;
			self.totalDamageValue = 0;
			self.totalDamageCnt = 0;
			self.totalCurValue = 0;
			self.totalDamList = {};

			self.damagePlayEnd = false;
		end

		table.insert(self.totalDamList, 1, args.value);
		if self.totalDamageNode:isVisible()==false then
			if #self.totalDamList == 1 then
				return;
			else
				self:ShowBeginTotalDam();
			end
		else 
			if self.damagePlayEnd then
				self:playOnceDamage();
			end
		end

	end 
end

function FightPanel:ChangeAllyHp(args)
end

function FightPanel:ResetTimeScale()
	local fspeed = FightManager:SpeedUp()
	if fspeed == 1 then
		self.speedup = 1
		self.SpeedUpTitle:setString('X1')
		self.speedbtn:loadTextureNormal('image/main/main_f_btn_2.png')
		AudioManager:changeSpeed(1)
	else 
		self.speedup = 2
		self.SpeedUpTitle:setString('X2')
		self.speedbtn:loadTextureNormal('image/main/main_f_btn_2_0.png')
		AudioManager:changeSpeed(2)
	end
end

function FightPanel:SpeedUp(sender, event)
	-- body
	if model.UserModel:getUserLevel() < 8 then
		Prompt:Show( LANG.UI_FIGHT_97 );
		return;
	end
	if self.speedup == 1 then
		self.speedup = 2
		self.SpeedUpTitle:setString('X2')
		self.speedbtn:loadTextureNormal('image/main/main_f_btn_2_0.png')
		AudioManager:changeSpeed(2)
	else 
		self.speedup = 1
		self.SpeedUpTitle:setString('X1')
		self.speedbtn:loadTextureNormal('image/main/main_f_btn_2.png')
		AudioManager:changeSpeed(1)
	end
	FightManager:setSpeedUp(1/self.speedup)
	-- cc.Director:getInstance():getScheduler():setTimeScale( self.speedup );
end

function FightPanel:GoToView(sender, event)
	-- body
	FightManager:setAutoRelease()
	if FightManager:getAutoRelease() then
		self.gotobtn:loadTextureNormal('image/main/main_f_btn_1_0.png')
		self.gotolabel:setString(LANG.UI_L_FIGHT_AUTO[2])
	else
		self.gotobtn:loadTextureNormal('image/main/main_f_btn_1.png')
		self.gotolabel:setString(LANG.UI_L_FIGHT_AUTO[1])
	end	

	if model.GuideModel:isInGuide() then
		local rVal = model.UserDefaultModel:getDataByKey('remind_FightPanelGotoView');
		if rVal ~= 1 then
			EventManager:FireEvent(EventType.UIGuidePanel);
			model.UserDefaultModel:setDataByKey('remind_FightPanelGotoView', 1);
		end
	end
end

function FightPanel:playDrawing(sender, event)
	FightManager:setPlayDrawing()
	self:updatePlayDrawing();
end

function FightPanel:updatePlayDrawing()
	if FightManager:isPlayDrawing() then
		self.drawbtn:loadTextureNormal('image/main/main_f_btn_3_0.png')
		self.drawlabel:setString(LANG.UI_L_FIGHT_SET[2])
	else
		self.drawbtn:loadTextureNormal('image/main/main_f_btn_3.png')
		self.drawlabel:setString(LANG.UI_L_FIGHT_SET[1])
	end	
end

function FightPanel:btnClick(sender, event)
	local tag = sender:getTag();
	AudioManager:playEffect(Conf.buttonClick);
	if self:checkClickLock(tag) then
		return 
	end

	AudioManager:playEffect(Conf.buttonClick)	
	if tag == FightMenuTag.TAG_ROUND_REWARD then
		UIManager:push('MissionPanel');

	elseif tag == FightMenuTag.TAG_CHAT then
		UIManager:push('ChatPanel');

	elseif tag == FightMenuTag.TAG_ITEM_USE then
		UIManager:push('FightItemPanel');

	elseif tag == FightMenuTag.TAG_RANDOM_BOSS then
		UIManager:push('RoundBossPanel');

	elseif tag == FightMenuTag.TAG_ONLINE then 
		UIManager:push('OnlineRewardPanel');

	elseif tag == FightMenuTag.TAG_NEWGIFT then
		UIManager:push('ShopGiftRecommendPanel')

	elseif tag == FightMenuTag.TAG_FIGHTGIFT then
		model.ShopModel:showFightGift(0);

	elseif tag == FightMenuTag.TAG_RESTRAINT then
		if model.GuideModel:isInGuide() then
    		Prompt:Show(LANG.COMMON_NOT);
    	else
        	if self.roundInfo then
				UIManager:push('TeamRestraintPanel',self.roundInfo.fightType,self.roundInfo.enemyRestraintString, self.roundInfo);
			end
        end
	elseif tag == FightMenuTag.TAG_BARE_COLLECT then
		UIManager:push('BareCollectPanel', 0);
	elseif tag == FightMenuTag.TAG_MAIL then
		UIManager:push('MailPanel');

	elseif tag == FightMenuTag.TAG_ADMOB_BUFF then 
		print('=================trigger admob buff ====   ')
		if tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').FIGHT , tag) then
			Prompt:Show(LANG.COMMON_CLOSE);
        	return;
		end
		self:onAdmobClick(ADMOB_TYPE.ADMOB_BUFF)
	elseif tag == FightMenuTag.TAG_ADMOB_REWARD then 
		print('=================trigger admob reward ====   ')
		if tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').FIGHT , tag) then
			Prompt:Show(LANG.COMMON_CLOSE);
        	return;
		end
		self:onAdmobClick(ADMOB_TYPE.ADMOB_REWARD)
	elseif tag == FightMenuTag.TAG_ADMOB_GACHA then
		print('=================trigger admob gacha ====   ')
		self:onAdmobClick(ADMOB_TYPE.ADMOB_GACHA)
	end
end
function FightPanel:checkClickLock( tag,notShowPrompt )
	local locked = false;
	local lockLv = 0;

	if tag == FightMenuTag.TAG_ROUND_REWARD then
		-- locked ,lockLv = model.RoundModel:getLockStateRoundReward();
	elseif tag == FightMenuTag.TAG_CHAT then

	elseif tag == FightMenuTag.TAG_ITEM_USE then
	 	locked,lockLv = model.ItemModel:getLockStateFightItemUse()
	elseif tag == FightMenuTag.TAG_RANDOM_BOSS then
	 	locked,lockLv = model.RoundBossModel:getLockStateRoundBoss()
	elseif tag == FightMenuTag.TAG_ONLINE then 
	end

	if locked and notShowPrompt ~= true  then
		Prompt:Show(string.format(LANG.UI_GUILD_43 , lockLv));
	end

	return locked

end


function FightPanel:updateBtnRedDot( index )

	local show = false;
	if index == FightMenuTag.TAG_ROUND_REWARD then
		show = model.MissionModel:getActiveState();
		tools.RedDotTools:setRedDotToNode(self.btns[index].node,show,nil,btnDotPos[index])
	elseif index == FightMenuTag.TAG_CHAT then

	elseif index == FightMenuTag.TAG_RANDOM_BOSS then
		show = model.RoundBossModel:getRedDotStateRoundBoss();
		tools.RedDotTools:setRedDotToNode(self.btns[index].node,show,nil,btnDotPos[index])
	elseif index == FightMenuTag.TAG_ONLINE then 
	end

end

function FightPanel:updateLock( index )

	if index == FightMenuTag.TAG_ROUND_REWARD then
		-- local locked ,lockLv = model.RoundModel:getLockStateRoundReward();
		-- tools.LevelLockTools:setLockToNode(self.btns[FightMenuTag.TAG_ROUND_REWARD].node,locked,lockLv,2,cc.p(0,0))
	elseif index == FightMenuTag.TAG_CHAT then

	elseif index == FightMenuTag.TAG_ITEM_USE then
		local locked,lockLv = model.ItemModel:getLockStateFightItemUse()
		tools.LevelLockTools:setLockToNode(self.btns[index].node,locked,lockLv,1,cc.p(0,0))
		-- self.btns[index].lock:setVisible(locked);
	elseif index == FightMenuTag.TAG_RANDOM_BOSS then
		local locked,lockLv = model.RoundBossModel:getLockStateRoundBoss()
		tools.LevelLockTools:setLockToNode(self.btns[index].node,locked,lockLv,2,cc.p(0,0),'image/main/main_btn_5_3.png')
	elseif index == FightMenuTag.TAG_ONLINE then

	elseif index == FightMenuTag.TAG_ADMOB_BUFF then
		if self.btns[index] then
			local locked = tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').FIGHT , index);
			tools.LevelLockTools:setLockToNode(self.btns[index].node,locked,1,1)
		end
	elseif index == FightMenuTag.TAG_ADMOB_REWARD then
		if self.btns[index] then
			local locked = tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').FIGHT , index);
			tools.LevelLockTools:setLockToNode(self.btns[index].node,locked,1,1)
		end
	end
end

function FightPanel:updateLevelLock( table_start_id )
	if table_start_id == levelLockItemIds.LEVLE_LOCK_2 then
	elseif table_start_id == levelLockItemIds.LEVLE_LOCK_5 then 
		self:updateLock(FightMenuTag.TAG_ITEM_USE);
	elseif table_start_id == levelLockItemIds.LEVLE_LOCK_7 then 
		self:updateLock(FightMenuTag.TAG_RANDOM_BOSS);
	end
end

function FightPanel:backAutoFight()
	self.scenePanel:switchState(1);

	self.scenePanel:initBg()
	self.mapPanel:hideCoverMap()

	self.bossBloodIconNode:setVisible(false)

	self:showAllBtns()
	
end

function FightPanel:showAllBtns()
	for k, v in pairs(self.btns) do
		self:showBtnNodeVisible(k,true);
		--v.node:setVisible(true);
	end

	-- 队伍克制 只在非挂机战显示
	-- 挂机 只在挂机战显示
	-- 在线奖励所有界面显示
	self.bInFight = false;
	-- self.btnList[FightMenuTag.TAG_BARE_COLLECT].node:setVisible(true)

	self.restrNode:setVisible(false);
	self.fightPlayBtnNode:setVisible(false)
	self.timeNode:setVisible(false)
	self.multArenaNode:setVisible(false)
	self:showOnlineNode();
	self:showNewGiftNode();
	self.helpBtn:setVisible(true);
	-- self:showBuffNode();
	--self:updateBtnVisible();

end

function FightPanel:leaveAutoFight()
	self.scenePanel:switchState(2);
end

function FightPanel:updateRandomBossComing( newRoundBoss )
	if newRoundBoss == nil or next(newRoundBoss) == nil then
		print('========error === no boss comming====')
		return 
	end
	-- dump(newRoundBoss)
	local _k,newBoss = next(newRoundBoss)
	if newBoss == nil then
		print('====error === no boss comming ===== 2222 ');
		return 
	end
	local pid = newBoss.pid;
	local bossData = model.RoundBossModel:getBossDataByPid( pid )
	if bossData == nil then
		print('========error === unKnown boss pid====',pid)
		return 
	end
	if  bossData.boss_difficulty < 4  then
		print('========no animations name boss',bossData.boss_difficulty,' in boss_spine');
		return 
	end
	-- print('========true ====== difficult == ',bossData.boss_difficulty)
	if self.randomBossSpine == nil then
		self:initRandomBossSpine();
	end
	local difficult = bossData.boss_difficulty;
	if difficult == 4 then
		difficult = 5;
	elseif difficult == 5 then
		difficult = 4;
	end
	local animationName = 'boss'..difficult
	self.randomBossSpine:setAnimation(0, GlobalFunctions:lAnimationName(animationName), false)


end

function FightPanel:playBossWarning()
	self.bossWarning:setVisible(true);
	self.bossWarning:setOpacity(0);
	local action1 = cc.FadeIn:create(0.5);
	local action2 = cc.FadeOut:create(0.5);
	local action3 = cc.Sequence:create(action1, action2);
	local action4 = cc.CallFunc:create(function()
		self.bossWarning:setVisible(false);
	end);
	self.bossWarning:runAction(cc.Sequence:create(action3, action3, action4));

end

function FightPanel:updateOnline( serverTime )

	-- self.onlineGetFrame
	local nextUnrewardId = model.OnlineModel:getNextUnRewardId();
	local nextStatus = model.OnlineModel:getStatusById(nextUnrewardId);
	if nextStatus == true  then
		--已领取！！
		-- self.onlineRewardGet:setVisible(false);
		self:showOnlineNode()
		return 
	end
	local nextSecond = model.OnlineModel:getSecondById(nextUnrewardId);
	local onlineSecond = model.OnlineModel:getOnlineSecondTotal()

	local secondOff = nextSecond - onlineSecond
	if secondOff < 0 then
		secondOff = 0
	end

	if self.onlineSecondOff ~= secondOff then
		--print("------------------updateOnline============",tostring(self.onlineSecondOff),'-----secondOff===',tostring(secondOff))
		self.onlineSecondOff = secondOff;
		if secondOff == 0 then
			self.onlineMTime:setString(LANG.UI_FIGHT_74)
			if self.onlineAnimationType ~= 1 then
				self.onlineAnimation:setAnimation(0,'get',true);
				self.onlineAnimationType = 1;
			end
		else
			secondOff = math.floor(secondOff);
			self.onlineMTime:setString(Sec2Time(secondOff))
			if self.onlineAnimationType ~= 2 then
				self.onlineAnimation:setAnimation(0,'idle',true);
				self.onlineAnimationType = 2;
			end
		end
	end
end

function FightPanel:updateBossDamage(damage, per, bossPer)
	self.bossBloodTimer:setPercentage(bossPer)

	self.guildDamageLabel:setString(  ConvertStrToDigitUnit(damage)  )
end

function FightPanel:showAdmobInfo(  )

	local admobItem = model.AdmobModel:getAdmobItem(ADMOB_TYPE.ADMOB_REWARD);
	if admobItem and admobItem.remain_times ~= -1 and self.btns[FightMenuTag.TAG_ADMOB_REWARD] and  self.btns[FightMenuTag.TAG_ADMOB_REWARD].times then
		local remainTimes = admobItem.remain_times;
		self.btns[FightMenuTag.TAG_ADMOB_REWARD].times:setString(tostring(remainTimes))
	end

	local admobItem = model.AdmobModel:getAdmobItem(ADMOB_TYPE.ADMOB_BUFF);
	if admobItem and admobItem.remain_times ~= -1 and self.btns[FightMenuTag.TAG_ADMOB_BUFF] and  self.btns[FightMenuTag.TAG_ADMOB_BUFF].times then
		local remainTimes = admobItem.remain_times;
		self.btns[FightMenuTag.TAG_ADMOB_BUFF].times:setString(tostring(remainTimes))
	end
	
	local admobItem = model.AdmobModel:getAdmobItem(ADMOB_TYPE.ADMOB_GACHA);
	if admobItem and admobItem.next_time and admobItem.max_time and self.btns[FightMenuTag.TAG_ADMOB_GACHA] then
		local curTime = TimeManager:getCurTime()
		if admobItem.next_time <= curTime then
			self.btns[FightMenuTag.TAG_ADMOB_GACHA].switchGet(true)
		else
			self.btns[FightMenuTag.TAG_ADMOB_GACHA].switchGet(false)
			local percent = math.floor((admobItem.next_time - curTime)/admobItem.max_time*100)
			self.btns[FightMenuTag.TAG_ADMOB_GACHA].bar:setPercentage(percent)
		end
	end

end
function FightPanel:onAdmobClick( admobId )
	model.AdmobModel:onAdmobClick(admobId)
	-- if self.bIsPlayAdmob == true  then
	-- 	return ;
	-- end
	-- local admobItem = model.AdmobModel:getAdmobItem(admobId)
	-- if admobItem == nil then
	-- 	GlobalFunctions:showErrorPromt('error --- not found admob info by id '..tostring(admobId));
	-- 	return 
	-- end
	-- if admobId == 3 then
	-- 	if admobItem.next_time > TimeManager:getCurTime() then
	-- 		local time = GlobalFunctions:ChangeTime(math.floor(admobItem.next_time - TimeManager:getCurTime()))
	-- 		Prompt:Show(string.format(LANG.UI_ADMOB_8, time))
	-- 		return
	-- 	end
	-- end
	-- self.bIsPlayAdmob = true;
	-- -- 1 请求服务器获取广告次数，2 请求SDK播放广告，3 请求服务器获取奖励
	-- local remainTimes = admobItem.remain_times;
	-- print('remainTimes ====     ' ,remainTimes );
	-- if remainTimes == 0  then
	-- 	-- Prompt:Show(LANG.UI_DAILY_ROUND_4);
	-- 	self.selectAdmobId = admobId;
	-- 	model.AdmobModel:reqAdmobInfo()
	-- 	return 
	-- end

	-- self:onAdmobSDKCall(admobId)
end

function FightPanel:retAdmobInfo( msg )
	-- if self.selectAdmobId ~= nil then
	-- 	local admobId = self.selectAdmobId;
		
	-- 	local admobItem = model.AdmobModel:getAdmobItem(admobId)
	-- 	if admobItem == nil then
	-- 		GlobalFunctions:showErrorPromt('error --- not found admob info by id '..tostring(admobId));
	-- 		self.bIsPlayAdmob = false;
	-- 		return 
	-- 	end
	
	-- 	local remainTimes = admobItem.remain_times;
	-- 	if remainTimes == 0  then
	-- 		Prompt:Show(LANG.UI_DAILY_ROUND_4);
	-- 		self.bIsPlayAdmob = false;
	-- 		return 
	-- 	end
	
	-- 	self:onAdmobSDKCall(admobId)
	-- end
	
	
end

function FightPanel:onAdmobSDKCall( admobId )
	-- self.selectAdmobId = nil;
	-- local function sdkPlayAdmobCallback( strData )
	-- 	WaitingPanel:Hide()

	-- 	local jsonData = Cjson.decode(strData)

	-- 	if jsonData.code ~= 0 then
	-- 		print("  sdk callback error  msg: "..tostring(jsonData.code))
	-- 		print("==== play admob over ==== ")
	-- 		self.bIsPlayAdmob = false;
	-- 		Prompt:Show(LANG.UI_ADMOB_5)
	-- 		return ;
	-- 	end

	-- 	model.AdmobModel:reqAdmobReward(admobId);
	-- end
	-- local args = {
	-- 	admob_id = model.AdmobModel:getNextAdmobId()
	-- }
	-- WaitingPanel:Show()
	-- sdkManager.SDKLuaHelper:playAdMob(args,sdkPlayAdmobCallback);
end

function FightPanel:retAdmobReward( msg )
	-- body
	self.bIsPlayAdmob = false;
	self:showAdmobInfo()
end

function FightPanel:isVisble()
	return self.beVisble
end


function FightPanel:initRoleBookNode()
	self.roleBookSprNode = cc.Node:create();
	self.roleBookSprNode:setVisible(true);
	self.roleBookSprNode:addTo(self.root);
end

function FightPanel:setFighterDie(fighter)
	if fighter:IsEnemy() then
		local bare_id = fighter:isShowRoleBook();
		if bare_id > 0 then
			self:playRoleBook(bare_id);
		end
	end
end

function FightPanel:playRoleBook(bareid)
	-- body
	local bookRolePath = ''
	local rolePath = ''
	local broken_sp = ''
	local bookInfo = ResTable:getRowData('bare_collect' , bareid);
	if bookInfo then
		bookRolePath = ResTable:getValue('role_resource', bookInfo.resource_id , 'card');
		broken_sp = ResTable:getValue('role_resource', bookInfo.resource_id , 'spine');
		local resourceId = ResTable:getValue('role' , bookInfo.role_id ,'resource_id');
		if resourceId then
	        rolePath = ResTable:getValue('role_resource', resourceId , 'card');
	    end
		
	end
	if bookRolePath == '' or rolePath == '' then
		return
	end

	self.roleBookSprNode:removeAllChildren()
	self.roleBookSprNode:setPosition( StageSize.width/2+200, -StageSize.width/2-200 )

	local move1 = cc.MoveTo:create(0.4, cc.p(20,-20))
	local move2 = cc.MoveTo:create(4, cc.p(-20,20))
	local move3 = cc.MoveTo:create(0.6, cc.p( -StageSize.width/2-200, StageSize.width/2+200 ))

	local spr1 = cc.Sprite:create('image/role/'..rolePath..'.png')
	spr1:addTo(self.roleBookSprNode)
	spr1:setOpacity(0)

	local spr2 = cc.Node:create()
	spr2:addTo(self.roleBookSprNode)
	spr2:setCascadeOpacityEnabled(true)
	spr2:setOpacity(0)

	if broken_sp and cc.FileUtils:getInstance():isFileExist('spine/'..broken_sp.."/"..broken_sp..".json") then
		local skAnimation = sp.SkeletonAnimation:create('spine/'..broken_sp.."/"..broken_sp..".json", 'spine/'..broken_sp.."/"..broken_sp..".atlas", 1)
		skAnimation:addTo(spr2)
		skAnimation:setAnimation(0, "idle", true)
	else
		local sprBroken = cc.Sprite:create('image/role/'..bookRolePath..'.png')
		sprBroken:addTo(spr2)
	end

	local playSpineFun = cc.CallFunc:create(function()
		local spine_res = 'poyi_spine'
		if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
			self.bookSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
			self.bookSpine:addTo(self.root)
			self.bookSpine:setAnimation(0,'poyi_spine',false);
		else
			print('error : 缺少spine 资源')
	    end
	end)

	local spr1ActSeq = cc.Sequence:create(
		-- cc.DelayTime:create(1),
		cc.FadeIn:create(0.2),
		cc.DelayTime:create(0.6),
		cc.FadeOut:create(0.4)
	)

	local spr2ActSeq = cc.Sequence:create(
		cc.DelayTime:create(0+1.2),
		cc.FadeIn:create(0.4),
		cc.DelayTime:create(2.8),
		cc.FadeOut:create(0.4)
	)

	local actSeq = cc.Sequence:create(
		move1,
		playSpineFun,
		move2,
		move3,
		cc.CallFunc:create(function()
			self.roleBookSprNode:removeAllChildren()
			FightManager:SetPlayRoleBookState(false)
		end)
	)

	FightManager:SetPlayRoleBookState(true)
	spr1:runAction(spr1ActSeq)
	spr2:runAction(spr2ActSeq)

	self.roleBookSprNode:runAction( actSeq )

	AudioManager:playCVStopBefor(bookInfo.role_id,nil ,cvTypeEnum.CV_BROKEN_FIGHT);

end

function FightPanel:initMultArenaInfo(subIndex)
	self.multArenaNode:setVisible(true)
	if subIndex>1 then
		return
	end

	local arenaData = model.ArenaModel:getMultArenaInfo();
	local name = model.UserModel:getUserName();
    self.name1Label:setString(tostring(name));
    if arenaData.rank <= 0 then
        self.rank1Label:setString('未上榜')
    else
        self.rank1Label:setString('第'..arenaData.rank..'名')
    end

    local enemyData = model.ArenaModel:getMultArenaEnemyPlayer();
    if enemyData.isRobot == 1 then
        local robotData = ResTable:getRowData('robot' , enemyData.uid);
        self.name2Label:setString(robotData.name)
    else
        self.name2Label:setString(enemyData.name)
    end
    self.rank2Label:setString('第'..enemyData.rank..'名')
    
    for i=1,3 do
    	self.enemyTeamList[i].shadowSpr:setVisible(false);
    	self.enemyTeamList[i].rtSpr:setVisible(false);

    	if enemyData then
	    	local tmpTeam = enemyData.team[i+10].arena_team;
	        if tmpTeam then
	            for j=1,TEAM_MAX_MEMBER do
	                if tmpTeam[j] then
	                    if enemyData.isRobot == 1 then
	                        self.enemyTeamList[i].teamItems[j]:Init(MONSTER_ROLE_TYPE, tmpTeam[j].no, tmpTeam[j].star_lv, tmpTeam[j].level)
	                    else
	                        self.enemyTeamList[i].teamItems[j]:Init(ROLE_TYPE, tmpTeam[j].no, tmpTeam[j].star_lv, tmpTeam[j].level, nil, tmpTeam[j].quality)
	                    end
	                else
	                    self.enemyTeamList[i].teamItems[j]:clearItem()
	                end
	            end
	        end
	    end

    end

    for i=1,3 do
    	local tid = model.ArenaModel:getMultArenaTeamIdByIdx( i )
    	self.selfTeamList[i].shadowSpr:setVisible(false);
    	self.selfTeamList[i].rtSpr:setVisible(false);

    	local team  = model.TeamModel:getTeamById(tid);
	    for j=1, TEAM_MAX_MEMBER do
	        local pid = team[j];
	        if pid ~=-1 then
	        	local card = model.CardModel:GetCard(pid);
	        	self.selfTeamList[i].teamItems[j]:Init(ROLE_TYPE, card.no, card.star_lv, card.level, team[j])
            	self.selfTeamList[i].teamItems[j]:updateAltarLevelColor(card.oriLv)
	        else
	        	self.selfTeamList[i].teamItems[j]:clearItem();
	        end
	    end
    end
end

function FightPanel:updateMultArenaFightInfo(data)
	local idx = data.challenge_no;
	local bwin = data.win;
	local bend = data.win_res;
	if bend~=-1 and idx<3 then
		self.selfTeamList[3].shadowSpr:setVisible(true)
		self.enemyTeamList[3].shadowSpr:setVisible(true)

		self.selfTeamList[3].rtSpr:setTexture('image/arena/arenaMult/am_un_icon.png')
		self.enemyTeamList[3].rtSpr:setTexture('image/arena/arenaMult/am_un_icon.png')
		self.selfTeamList[3].rtSpr:setVisible(true)
		self.enemyTeamList[3].rtSpr:setVisible(true)
	end

	self.selfTeamList[idx].shadowSpr:setVisible(true)
	self.enemyTeamList[idx].shadowSpr:setVisible(true)
	self.selfTeamList[idx].rtSpr:setTexture( bwin==1 and 'image/arena/arenaMult/am_win_icon.png' or 'image/arena/arenaMult/am_lose_icon.png')
	self.enemyTeamList[idx].rtSpr:setTexture(bwin==1 and 'image/arena/arenaMult/am_lose_icon.png' or 'image/arena/arenaMult/am_win_icon.png')
	self.selfTeamList[idx].rtSpr:setVisible(true)
	self.enemyTeamList[idx].rtSpr:setVisible(true)
end

function FightPanel:showMultArenaSubRt( msg )
	self:updateMultArenaFightInfo(msg)
	self.multArenaSp:setVisible(true)
	if msg.win == 1 then
		self.multArenaSp:setAnimation(0, GlobalFunctions:lAnimationName('shengli'), false)
	else
		self.multArenaSp:setAnimation(0, GlobalFunctions:lAnimationName('shibai'), false)
	end
	if msg.win_res==-1 and msg.challenge_no<3 then
		self.multArenaNode:runAction(cc.Sequence:create( 
								cc.DelayTime:create(3),
								cc.CallFunc:create(function()
									self.multArenaSp:setVisible(false);
									FightManager:onFightNext();
									model.ArenaModel:continueNextFight();
								end)
							))
	else
		UIManager:push('ArenaMultFightResult');
	end
end

-- 引导相关
function FightPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	if guideNo == 12 or guideNo == 14 then
		local btn = self.btns[FightMenuTag.TAG_ROUND_REWARD].node;
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));

	elseif guideNo == 78 then 
		local btn = self.btns[FightMenuTag.TAG_RANDOM_BOSS].node;
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	elseif guideNo == 83 then 
		local btn = self.btns[FightMenuTag.TAG_BARE_COLLECT].node;
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	end
	return pos;
end

return FightPanel;
