local FightScenePanel = class("FightScenePanel", require('app.layer.BaseLayer'));
local scheduler = cc.Director:getInstance():getScheduler();

local DirectionType = Enum:getTable("DirectionType");
local fonts = Enum:getTable("SystemFont");
local AnimationType = Enum:getTable("AnimationType");
local CastType = Enum:getTable('CastType');
local FightRestraintEnum = Enum:getTable('FightRestraintEnum')
local RestrainType = Enum:getTable('FightRestraintType')
local ArenaRoleMonsterType = Enum:getTable('ArenaRoleMonsterType');
local cvTypeEnum = Enum:getTable("cvTypeEnum");
local FightType = Enum:getTable('FightType');

local imgPath = 'image/fight/scene/';
local damageLabelPos = 
{
	[1] = {x = -10, y = 10},
	[2] = {x = -35, y = 40},
	[3] = {x = 35, y = 70},
	[4] = {x = -32, y = 100},
	[5] = {x = 45, y = 130},
	[6] = {x = -10, y = 160},
}
local damageLabelOffset = { x = 0, y = 80};


-- state : 1: auto; 2: boss; 3 ...
-- autoState: 1 move bg; 2 fight
function FightScenePanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'FightScenePanel');

	self.useCCreator = false;
	self.isPopup = true;
	self.haveSwallowLayer = false;
end

function FightScenePanel:Init()
	local bottomHeight = 268;

	self.sceneNodeFight = cc.Node:create()
		:setPosition(0, -StageSize.height/2)
		:setLocalZOrder(10)
		:addTo(self.root)

	self.sceneRealNode = cc.Node:create()
		:addTo(self.sceneNodeFight)

	self.sceneNodeBg = cc.Node:create()
		:setLocalZOrder(0)
		:addTo(self.sceneRealNode)


	self.effNode = cc.Node:create()
		:setLocalZOrder(888888)
		:addTo(self.sceneRealNode)

	self.labelNode = cc.Node:create()
		:setLocalZOrder(999999)
		:addTo(self.sceneRealNode)

	self.sceneNodeAuto = cc.Node:create()
		:setPosition(0, -StageSize.height/2)
		:setLocalZOrder(10)
		:addTo(self.root)
	

	self.roleZorder = 100;


	self.state = 1;
	-- #bg
	self.bgType = 0;
	self.lastBgType = 0;
	self.sceneBgList = {};


	-- #auto
	self.autoState = 1;
	-- time
	self.autoState1Time = 0;
	self.autoState2Time = 0;

	-- state1, move bg time
	self.autoState1MoveTime = 2;

	-- bullet speed
	self.bulletSpeedX = 1000;

	self.shakeTotalTime = 0;
	self.shakeTime = -1;
	self.shakeTarget = nil;
	self.shakeOriX = 0;
	self.shakeOriY = 0;

	--
	self.autoAllyList = {};
	self.autoEnemyList = {};
	self.autoEnemyDamageLabelList = {};
	self.bulletList = {};
	self.effectList = {};
	self.damageList = {};
	self.autoAllySkillList = {};
	self.coinList = {};
	self.ballList = {};

	self.timeScale = AUTOFIGHT_TIMESCALE_LIST[1];

	self.autoInit = false;

	-- #fight
	self.allyHandlerList = {}
	self.enemyHandlerList = {}

	self.allyBVList = {}
	self.enemyBVList = {}

	self.allyDamageList = {}
	self.enemyDamageList = {}

	self.aDamSkillLabelList = {}
	self.eDamSkillLabelList = {}

	self.autoEnemyAppearYList = {-140, -230, -320};

	-- event
	--EventManager:RegisterEvent(EventType.FightChangeCameraCaster, handler(self, self.cameraChangeToCaster));
	--EventManager:RegisterEvent(EventType.FightChangeCameraTarget, handler(self, self.cameraChangeToTarget));
	--EventManager:RegisterEvent(EventType.FightBackCamera, handler(self, self.cameraReset));
	EventManager:RegisterEvent(EventType.FightAddEffect, handler(self, self.addEffect));
	EventManager:RegisterEvent(EventType.FightDelEffect, handler(self, self.delEffect));
	--EventManager:RegisterEvent(EventType.SceneNext, handler(self, self.sceneNext));
	--EventManager:RegisterEvent(EventType.SceneRollCenter, handler(self, self.sceneRollCenter));
	--EventManager:RegisterEvent(EventType.SceneRollBack, handler(self, self.sceneRollBack));
	--EventManager:RegisterEvent(EventType.SceneEndRollCenter, handler(self, self.sceneEndRollCenter));
	--EventManager:RegisterEvent(EventType.FightPanelDestroyArmature, handler(self, self.destroyArmature));
	--EventManager:RegisterEvent(EventType.FightPanelStopRun, handler(self, self.stopRun));
	--EventManager:RegisterEvent(EventType.FightBuffLabelAdd, handler(self, self.addBuffLabel));
	--EventManager:RegisterEvent(EventType.FightBuffIconUpdate, handler(self, self.updateFighterIcon));
	EventManager:RegisterEvent(EventType.AutoFightAllyEnter, handler(self, self.autoFightAllyEnter));
	EventManager:RegisterEvent(EventType.AutoFightAllyLeave, handler(self, self.autoFightAllyLeave));
	EventManager:RegisterEvent(EventType.AutoFightEnemyEnter, handler(self, self.autoFightEnemyEnter));
	EventManager:RegisterEvent(EventType.AutoFightEnemyLeave, handler(self, self.autoFightEnemyLeave));
	EventManager:RegisterEvent(EventType.FightSceneBgMove, handler(self, self.moveBg));
	EventManager:RegisterEvent(EventType.FightSceneBgStop, handler(self, self.stopBg));
	EventManager:RegisterEvent(EventType.AutoFightEnemyReset, handler(self, self.autoEnemyReset));
	EventManager:RegisterEvent(EventType.AutoFightAddBullet, handler(self, self.addBullet));
	EventManager:RegisterEvent(EventType.AutoFightTimeScale, handler(self, self.setTimeScale));
	EventManager:RegisterEvent(EventType.AutoFightTeamChange, function()
		self.changeTeamFlag = true;
	end);
	EventManager:RegisterEvent(EventType.FightIncomeChange, handler(self, self.collectCoin));
	EventManager:RegisterEvent(EventType.FightIncomeChange, handler(self, self.collectBall));
	EventManager:RegisterEvent(EventType.FightOver, handler(self, self.clearPanel));
	EventManager:RegisterEvent(EventType.FightPanelUpdateScene, handler(self, self.initBg));
	EventManager:RegisterEvent(EventType.FightShowSkill, handler(self, self.showSkill));
	EventManager:RegisterEvent(EventType.FightShowSkillOver, handler(self, self.showSkillOver));
	EventManager:RegisterEvent(EventType.FightPreloadSkillEffect, handler(self, self.preloadSkillEffect));
	EventManager:RegisterEvent(EventType.Revive, handler(self, self.reviveFighter));

	EventManager:RegisterEvent(EventType.FightPanelScale, handler(self, self.runScaleTo));
	EventManager:RegisterEvent(EventType.FightPanelShake, handler(self, self.ShowShakeEff));

	EventManager:RegisterEvent(EventType.FightUpdateSkillIcon, handler(self, self.updateSkillTime));

	self.root:setVisible(false);

	self.fnt_ext = ''
	local ret = model.UserDefaultModel:getLanguageSet();
	if ret == 'kr' then
		self.fnt_ext = '_kr';
	elseif ret == 'jp' then
		self.fnt_ext = '_jp';
	elseif ret == 'en' then
		self.fnt_ext = '_en';
	end
end

function FightScenePanel:runScaleTo(fighter, skill)
	-- body
	if fighter == nil then
		self.sceneRealNode:runAction( 
			cc.Sequence:create(
				cc.Spawn:create(				
					cc.MoveTo:create(0.01, cc.p(0, 0)),
					cc.ScaleTo:create(0.01, 1.05)
				),
				cc.CallFunc:create(function()
					FightManager:setIsScaling(false)
				end)
				)
		);

	else
		self:showMaskAndHighActor(fighter, skill)

		-- local speedup = FightManager:SpeedUp()
		-- local idx = fighter:getFormationIndex();
		-- local pos;
		-- if fighter:IsEnemy() then
		-- 	pos = FightManager:getEnemyPosByFormaitionIndex(idx);
		-- else
		-- 	pos = FightManager:getRolePosByFormationIndex(idx);
		-- end
		-- self:showMaskAndHighActor(fighter, skill)
		-- self.sceneRealNode:runAction( 
		-- 	cc.Spawn:create(
		-- 		-- cc.MoveBy:create(0.28, cc.p(-pos.x, -pos.y-DesignSize.height/2)),
		-- 		cc.MoveBy:create(0.28*speedup, cc.p(-pos.x*2, -pos.y*2-DesignSize.height/2)),
		-- 		cc.ScaleTo:create(0.28*speedup, 2)
		-- 		)
		-- );
		-- FightManager:setIsScaling(true)

	end

end

function FightScenePanel:resetScale()
	-- body
	
	self.show_skill_mask:setVisible(false)
	self.sceneRealNode:runAction( 
			cc.Sequence:create(
				cc.Spawn:create(				
					cc.MoveTo:create(0.01, cc.p(0, 0)),
					cc.ScaleTo:create(0.01, 1)
				),
				cc.CallFunc:create(function()
					FightManager:setIsScaling(false)
				end)
				)
	);
end

function FightScenePanel:onEnter()
	self:clearAutoActor()
	self.autoInit = false
	self.resH = false
	self.resFH = false
	self.resDt = 0
	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.update), 0, false);

	self.monsterList = model.RoundModel:getRoundMonsterList();
	if DownloadControler:checkNeedMonsterRes( self.monsterList ) then
		self.resH = true
	end
	self.fighterModelList = {}

	if not self.autoInit then
		self.autoInit = true;
		self:autoAllyEnter();
		if self.resH then
			self:autoEnemyEnter();
		end

		-- bg
		local round = model.RoundModel:getRoundIndex();
		local bgType = ResTable:getValue('round', round, 'bg_id');
		self.bgType = bgType;
		self:initBg();

		if self.resH and self.resFH then
			self:switchAutoState(1);
		end
		self.changeTeamFlag = false;
	end

	self.root:setVisible(true);
end

function FightScenePanel:onLeave()
	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end

	self.root:setVisible(false);
end

function FightScenePanel:update( dt )
	if self.resH == false or self.resFH == false then
		self.resDt = self.resDt + dt
		if self.resDt>1 then
			if  self.resH==false and DownloadControler:checkNeedMonsterRes( self.monsterList ) then
				self.resH = true
				self:autoEnemyEnter();
			end 

			if self.resFH==false and DownloadControler:checkNeedFightRes( self.fighterModelList ) then
				self.resFH = true
				self:autoAllyEnter(); 
			end
			if self.resH and self.resFH then
				self:switchAutoState(1);
			end

		end
		if self.resDt > 1 then
			self.resDt = self.resDt - 1
		end
		return;
	end

	self:ShakeEff(dt);
	if self.state == 1 then
		if self.autoState == 1 then
			self.autoState1Time = self.autoState1Time + dt;
			self:moveBg(dt);
			if self.autoState1Time > self.autoState1MoveTime then
				self:switchAutoState(2);
			end
		elseif self.autoState == 2 then
			self.autoState2Time = self.autoState2Time + dt;
			self:checkAutoSkillCast();
			self:moveAutoEnemy(dt);
			self:updateDamage(dt)
			self:updateEffect(dt);
			self:updateBullet(dt);
			self:updateAllyAnimation(dt);
			if self:checkEnemyAlive() then
				self:switchAutoState(1);
			end
		end
	elseif self.state == 2 then
	else
		return;
	end
end

function FightScenePanel:switchAutoState(state)
	if state == 1 then
		if self.changeTeamFlag then
			self:autoAllyEnter();
			self.changeTeamFlag = false;
		end
		for _, v in pairs(self.autoAllyList) do
			v:SetAnimation(AnimationType.run, nil, self.timeScale*AUTOFIGHT_TIMESCALE_RUN);
		end

		for _, v in pairs(self.autoEnemyList) do
			v:clearEffect();
		end
		self:removeAllDamage();
		self:removeAllEffect();
		self:removeAllBullet();
		self.autoState1Time = 0;
	elseif state == 2 then
		for _, v in pairs(self.autoAllyList) do
			v:SetAnimation(AnimationType.idle, nil, self.timeScale);
		end
		for _, v in pairs(self.autoAllySkillList) do
			v.last_cast = v.pre_cd-v.cd;
		end
		for id, v in pairs(self.autoEnemyList) do
			v:SetAnimation(AnimationType.run, nil, self.timeScale*AUTOFIGHT_TIMESCALE_RUN);
		end
		self:resetAutoEnemy();
		self.autoState2Time = 0;
	else
		return;
	end
	self.autoState = state;
end

function FightScenePanel:switchState(state)
	if state == 1 then
		self.sceneNodeAuto:setVisible(true);
		self.sceneNodeFight:setVisible(false);

		self.sceneNodeBg:retain();
		self.sceneNodeBg:removeFromParent();
		self.sceneNodeBg:setPosition(0, 0)
        -- self.sceneNodeBg:setPosition(0, -StageSize.height/2)
        :setLocalZOrder(0)
        :addTo(self.sceneNodeAuto)
        self.sceneNodeBg:release();

        self.sceneNodeFight:setPosition(0, -StageSize.height/2)

        self:switchAutoState(2);

	elseif state == 2 then
		self.sceneNodeAuto:setVisible(false);
		self.sceneNodeFight:setVisible(true);

		self.sceneNodeBg:retain();
		self.sceneNodeBg:removeFromParent();
        self.sceneNodeBg:setPosition(0, 0)
        :setLocalZOrder(0)
        :addTo(self.sceneRealNode)
        self.sceneNodeBg:release();

        self.sceneNodeFight:setPosition(0, -StageSize.height/2+150)

	else
		return;
	end
	self.state = state;
end

function FightScenePanel:moveBg(dt)
	for k1, v1 in pairs(self.sceneBgList) do
		local num = #v1;
		for i=1, num do
			local posx, posy = v1[i].sp:getPosition();
			local speed = v1[i].speed*self.timeScale;
			local aimX = posx - speed*dt;
			if aimX < -StageSize.width then
				aimX = aimX + v1[i].width*num;
			end
			v1[i].sp:setPosition(aimX, posy);
		end
	end
end

function FightScenePanel:checkAutoSkillCast()
	for id, v in pairs(self.autoAllySkillList) do
		if self.autoState2Time > v.last_cast + v.cd/self.timeScale/AUTOFIGHT_TIMESCALE_CD then
			v.last_cast = self.autoState2Time;
			for _, v1 in pairs(v.event) do
				if v1.event_type == 1 then
					if v1.action == 1 then
						self.autoAllyList[id]:SetAnimation(AnimationType.attack, 1, self.timeScale*AUTOFIGHT_TIMESCALE_ATTACK);
					elseif v1.action == 2 then
						self.autoAllyList[id]:SetAnimation(AnimationType.skill, 1, self.timeScale*AUTOFIGHT_TIMESCALE_ATTACK);
					end
				end
			end
		end
	end
end

function FightScenePanel:moveAutoEnemy(dt)
	for k, v in pairs(self.autoEnemyList) do
		if not(v.isDead) then
			if self.autoState2Time > v.appearTime then
				local x, y = v:GetPosition();
				if x > 50 then
					local speed = v.speed*self.timeScale;
					local aimX = x - speed*dt;
					v:SetPosition(aimX, v.y);
				else
					v.isDead = true;
					v:SetAnimation(AnimationType.die, 1, self.timeScale);
					self:addCoinDrop(k);
					self:addExpDrop(k);
					self:addBallDrop(k);
				end
			end
		end
	end
end

function FightScenePanel:updateDamage(dt)
	local x, y, id = self:getBulletAimPos();
	if not id then
		return;
	end
	for _, v in pairs(self.damageList) do
		if not v.isShow then
			if v.showTime <= self.autoState2Time then
				v.isShow = true;
				self:showAutoDamage(id, v);
			end
		end
	end
end

function FightScenePanel:updateEffect(dt)
	local x, y, id = self:getBulletAimPos();
	if not id then
		return;
	end
	for _, v in pairs(self.effectList) do
		if not v.isShow then
			if v.showTime <= self.autoState2Time then
				v.isShow = true;
				local fatherNode, isEnemy;
				if v.target == -1 then
					isEnemy = true;
					fatherNode = self.autoEnemyList[id]:getEffectNode();
				else
					isEnemy = false;
					fatherNode = self.autoAllyList[v.target]:getEffectNode();
				end

				local effectName = ResTable:getValue(TableName.SkillEffect, v.effectID, 'name')
				DragonBonesManager:loadDragonBonesResources(effectName, 'Effect/db/')

				local effect = DragonBonesManager:getFactory():buildArmatureDisplay(effectName)
				local db_id = DragonBonesManager:AddDisplay(effect, effectName)
				if effect:getAnimation():hasAnimation(effectName) then
					effect:getAnimation():play(effectName, 1)
				else
					effect:getAnimation():play(FightRestraintEnum[v.restraintType], 1)
				end
				if isEnemy then
					effect:getArmature():setFlipX(true);
				end
--				effect:setScale(v.scale)
				if v.pos == 2 then
					effect:setPosition(0, -50)
				else
					effect:setPosition(0, -10)
				end
				effect:addTo(fatherNode)
				effect:getEventDispatcher():setEnabled(true)
				local listener = cc.EventListenerCustom:create("complete", function(event)
					effect:removeFromParent()
					DragonBonesManager:RemoveDisplay(db_id)
				end)
				effect:getEventDispatcher():addEventListenerWithFixedPriority(listener, 1);	
			end
		end
	end
end

function FightScenePanel:updateBullet(dt)
	local aimX, aimY, id = self:getBulletAimPos();
	for _, v in pairs(self.bulletList) do
		if not v.isShow then
			if v.showTime <= self.autoState2Time then
				v.isShow = true;
				v.sp:setVisible(true);
			end
		elseif not v.isRemove then
			if not aimX then
				v.sp:removeFromParent();
				v.isRemove = true;
			else
				local flyTime = self.autoState2Time - v.showTime;
				if flyTime >= 0.7 then
					v.sp:removeFromParent();
					v.isRemove = true;
				else
					local x, y = v.sp:getPosition();
					local next_x = x + self.bulletSpeedX*self.timeScale * dt;
					local next_y = v.beginY + flyTime/0.5*(aimY - v.beginY);
					if next_x >= aimX then
						v.sp:removeFromParent();
						v.isRemove = true;
					else
						v.sp:setPosition(next_x, next_y);
					end
				end
			end
		end
	end
end

function FightScenePanel:updateAllyAnimation(dt)
	for _, v in pairs(self.autoAllyList) do
		if (v.animationID == AnimationType.attack) or (v.animationID == AnimationType.hit) or (v.animationID == AnimationType.skill) then
			if v.armature:getAnimation():isCompleted() then
				v:SetAnimation(AnimationType.idle, nil, self.timeScale);
			end
		end
	end
end

function FightScenePanel:checkEnemyAlive()
	local aliveNum = 0;
	for id, v in pairs(self.autoEnemyList) do
		if not v.isDead then
			aliveNum = aliveNum + 1;
		end
	end
	if self.autoState2Time > 10 or aliveNum == 0 then
		return true;
	end
	return false;
end

function FightScenePanel:resetAutoEnemy()
	local time = 0;
	for _, v in pairs(self.autoEnemyList) do
		local r = math.random(5, 10);
		local layer = math.random(1,3);
		local y = self.autoEnemyAppearYList[layer];
		time = time + r/10;
		v.appearTime = time;
		v.y = y + DesignSize.height/2;
		v.x = StageSize.width/2+100;
		v.speed = math.random(150, 220)*self.timeScale;
		v.beHit = 0;
		v.isDead = false;
		v:SetPosition(v.x, v.y);
		v:SetLocalZOrder(layer);
	end
end

function FightScenePanel:setTimeScale( scale )
	scale = scale or 1;
	if not AUTOFIGHT_TIMESCALE_LIST[scale] then
		scale = 1;
	end
	
	if self.timeScale == AUTOFIGHT_TIMESCALE_LIST[scale] then
		return;
	end
	self.timeScale = AUTOFIGHT_TIMESCALE_LIST[scale];

	if not self:isVisible() then
		return;
	end
	if self.state == 1 then
		if self.autoState == 1 then
			for _, v in pairs(self.autoAllyList) do
				v:SetAnimation(AnimationType.run, nil, self.timeScale*AUTOFIGHT_TIMESCALE_RUN);
			end
		elseif self.autoState == 2 then
			for _, v in pairs(self.autoAllyList) do
				v:SetAnimation(AnimationType.idle, nil, self.timeScale);
			end
			for id, v in pairs(self.autoEnemyList) do
				if v:GetAnimation() ~= AnimationType.die then
					v:SetAnimation(AnimationType.run, nil, self.timeScale*AUTOFIGHT_TIMESCALE_RUN);
				end
			end
		end
	end
end

function FightScenePanel:clearAutoActor()
	for _, v in pairs(self.autoAllyList) do
		v:DestroyArmature();
		ActorManager:DestroyActor(v.id);
	end
	self.autoAllyList = {};
	self.autoAllySkillList = {};

	for _, v in pairs(self.autoEnemyList) do
		v:DestroyArmature();
		ActorManager:DestroyActor(v.id);
	end
	self.autoEnemyList = {};
	self.autoEnemyDamageLabelList = {};
	self.autoEnemyLastShowDamage = {};

end

function FightScenePanel:autoAllyEnter()
	for _, v in pairs(self.autoAllyList) do
		v:DestroyArmature();
		ActorManager:DestroyActor(v.id);
	end
	self.autoAllyList = {};
	self.autoAllySkillList = {};

	local team = model.TeamModel:getCurrentTeam();
	local curFormation = model.TeamModel:getCurrentTeamFormation()
	local teamPos = model.TeamModel:getTeamFormationPosById(curFormation);

	self.fighterModelList = {}
	for i=1, TEAM_MAX_MEMBER do
		local pid = team[i];
		if pid ~= -1 then
			local resId = model.CardModel:getCardResIdByPid(pid);
			local roleResData = ResTable:getRowData('role_resource', resId);
			table.insert( self.fighterModelList, roleResData.model )
		end
	end
	if DownloadControler:checkNeedFightRes( self.fighterModelList ) == false then
		return;
	end
	self.resFH = true

	for i=1, TEAM_MAX_MEMBER do
		local pid = team[i];
		if pid ~= -1 then
			-- actor
			local card = model.CardModel:GetCard(pid);
			local cardData = ResTable:getRowData('role', card.no);
			local actor = ActorManager:CreateActor(card.no, pid);
			local cardInfo = ResTable:getRowData('role_pre', cardData.role_id)
			self.autoAllyList[actor.id] = actor;

			-- animation
			local armature = actor:genArmature(DirectionType.right);
			armature:addTo(self.sceneNodeAuto);
			local formationIndex = teamPos[i];
			local pos = FightManager:getRolePosByFormationIndex(formationIndex);

			actor:SetPosition(pos.x, pos.y + DesignSize.height/2);
			actor:SetLocalZOrder(self.roleZorder + pos.z);

			-- skill
			local skill_id = model.CardModel:getCardSkillIdByPid(pid); --cardData.skill_id;
			local skill_no = ResTable:getValue('role_skill', skill_id, 'skill1_no');
			local skill_scr = ResTable:getValue('role_skill', skill_id, 'skill1_scr');
			local skill = ResTable:getRowData('skill', skill_no);
			if skill then
				local data = {};
				data.pre_cd = skill.pre_cd;
				data.cd = math.floor(skill.cd/cardData.aspd*100)/100;
				data.hit_type = ResTable:getValue('role_pre', cardData.role_id, 'def_type');
				data.last_cast = data.pre_cd-data.cd;
				data.damage = cardData.atk;
				data.critDamage = ConvertStrToDigitUnit(HpMultiply(ConvertDigitUnitToStr(cardData.atk), (200+cardData.crit_dam)/100));
				data.restraintDamage = ConvertStrToDigitUnit(HpMultiply(ConvertDigitUnitToStr(cardData.atk), 1.5));
				data.restraintCritDamage = ConvertStrToDigitUnit(HpMultiply(ConvertDigitUnitToStr(data.critDamage), 1.5));
				data.critRate = cardData.crit/10000;
				self.autoAllySkillList[actor.id] = data;

				data.event = {};
				local event1s = ResTable:getValue(TableName.SkillScript, skill_scr, 'event_1');
				local event2s = ResTable:getValue(TableName.SkillScript, skill_scr, 'event_2');
				local effectName, effect
				for k, v in pairs(event1s) do
					if v[2] == 1 then
						local event = {}
						event.time = v[1]
						event.event_type = v[2]
						event.action = v[3]
						event.effectID = v[4]
						table.insert(data.event, event)

						local effectID = event.effectID
						local slot
						if event.action == 1 then
							slot = 'attack'
						elseif event.action == 2 then
							slot = 'skill'
						end
						effectName = ResTable:getValue(TableName.SkillEffect, effectID, 'name')
						local fatherDB = actor:getArmatureDB()
						DragonBonesManager:loadDragonBonesResources(effectName, 'Effect/db/')

						effect = DragonBonesManager:getFactory():buildArmatureDisplay(effectName)
						fatherDB:getArmature():getSlot(slot):setChildArmature(effect:getArmature());
						if effect:getAnimation():hasAnimation('none') then
							effect:getAnimation():play('none', 1)
						end
					end
				end
				local callbacks = {}
				for k, v in pairs(event2s) do
					if v[2] == 2 or v[2] == 6 or v[2] == 7 then
						local event = {};
						event.time = v[1];
						event.event_type = v[2];
						event.events = {};
						for i=3, #v do
							event.events[i-2] = v[i];
						end
						table.insert(data.event, event);
						if event.event_type == 2 then
							table.insert(callbacks, function()
								self:addEventDamage(actor.id, data, event)
							end);
						elseif event.event_type == 6 then
							table.insert(callbacks, function()
								self:addEventEffect(actor.id, event, cardInfo.def_type);
							end)
						end
					end
				end

				actor:addArmatureEventListener(function()
					-- 1. play attack effect
					if effect:getAnimation():hasAnimation(effectName) then
						effect:getAnimation():play(effectName, 1)
					else
						effect:getAnimation():play(FightRestraintEnum[cardInfo.def_type], 1)
					end

					-- 2. add event callbacks
					for k, v in pairs(callbacks) do
						v()
					end
				end)
			end
		end
	end
end

function FightScenePanel:autoEnemyEnter()
	for _, v in pairs(self.autoEnemyList) do
		v:DestroyArmature();
		ActorManager:DestroyActor(v.id);
	end
	self.autoEnemyList = {};
	self.autoEnemyDamageLabelList = {};
	self.autoEnemyLastShowDamage = {};

	local round = model.RoundModel:getRoundIndex();
	local monsterData = ResTable:getValue('round', round, 'monster');
	self.enemyNum = 0;
	for i=1, #monsterData do
		if monsterData[i][1] then
			-- actor
			local actor = ActorManager:CreateActor(monsterData[i][1], 0);
			self.autoEnemyList[actor.id] = actor;
			self.autoEnemyDamageLabelList[actor.id] = {};
			self.autoEnemyLastShowDamage[actor.id] = 0;

			-- animation
			local armature = actor:genMonArmature(DirectionType.left);
			armature:addTo(self.sceneNodeAuto);
			local layer = math.random(1,3);
			local y = self.autoEnemyAppearYList[layer];
			actor:SetPosition(StageSize.width/2+100, y + DesignSize.height/2);
			local size = ResTable:getValue('monster', monsterData[i][1], 'size');
			actor:SetScale(size);

			self.enemyNum = self.enemyNum + 1;
		end
	end
end

function FightScenePanel:initBg(bgType)
	bgType = bgType or self.bgType;	
	
	local bgData = ResTable:getValue('round_bg', bgType, 'bg');
	if bgData == nil or next(bgData) == nil or #bgData==0 then
		self.sceneNodeBg:setVisible(false)
		return;
	end
	self.sceneNodeBg:setVisible(true)
	if self.lastBgType == bgType then
		return;
	end

	self.lastBgType = bgType;

	for k1, v1 in pairs(self.sceneBgList) do
		for k2, v2 in pairs(v1) do
			v2.sp:removeFromParent();
		end
	end
	self.sceneBgList = {};

	local roundLang = StageSize.width*2;
	local startX = -roundLang;
	for _, sp in pairs(bgData) do
		local spList = {};

		local data = {};
		local spPath = imgPath .. 'scene' .. sp[1] .. '.png';
		data.sp = cc.Sprite:create(spPath);
		data.y = sp[2];
		data.speed = sp[3];
		data.zorder = sp[4];
		data.sp:setPosition(startX, data.y);
		data.sp:addTo(self.sceneNodeBg);
		data.sp:setLocalZOrder(data.zorder);
		spList[1] = data;

		local size = data.sp:getContentSize();
		data.width = size.width;

		local maxNum = math.floor(roundLang*2/data.width) + 1;
		for i=2, maxNum do
			local dataOther = {};
			dataOther.sp = cc.Sprite:create(spPath);
			dataOther.y = sp[2];
			dataOther.speed = sp[3];
			dataOther.zorder = sp[4];
			dataOther.sp:setPosition(startX + data.width*(i-1), dataOther.y);
			dataOther.sp:addTo(self.sceneNodeBg);
			dataOther.sp:setLocalZOrder(dataOther.zorder);
			dataOther.width = data.width;
			spList[i] = dataOther;
		end
		table.insert(self.sceneBgList, spList);
	end
end

function FightScenePanel:getBulletAimPos()
	local min_id, aimX, aimY;
	for id, v in pairs(self.autoEnemyList) do
		if not(v.isDead) then
			local posX, posY = v:GetPosition();
			if not min_id then
				min_id = id;
				aimX = posX;
				aimY = posY;
			elseif posX < aimX then
				min_id = id;
				aimX = posX;
				aimY = posY;
			end
		end
	end
	return aimX, aimY, min_id;
end

function FightScenePanel:addEventDamage(id, damageInfo, eventInfo)
	local damage = {};

	damage.hit_type = damageInfo.hit_type;
	damage.isCrit = (math.random() < damageInfo.critRate);
	damage.damage = damageInfo.damage;
	damage.critDamage = damageInfo.critDamage;
	damage.restraintDamage = damageInfo.restraintDamage;
	damage.restraintCritDamage = damageInfo.restraintCritDamage;
	damage.showTime = self.autoState2Time+eventInfo.time/1000;
	damage.isShow = false;

	table.insert(self.damageList, damage);
end

function FightScenePanel:addEventEffect(id, effectInfo, restraintType)
	local effect = {};
	
	if effectInfo.events[1] == 1 then
		effect.target = id;
	else
		effect.target = -1;
	end
	effect.effectID = effectInfo.events[2];
	effect.pos = effectInfo.events[3];
	effect.scale = effectInfo.events[4];
	effect.showTime = self.autoState2Time+effectInfo.time/1000;
	effect.isShow = false;
	effect.restraintType = restraintType

	table.insert(self.effectList, effect);
end

function FightScenePanel:addEventBullet(id, bulletInfo)
	local bullet = {};

	bullet.isShow = false;
	bullet.isRemove = false;
	bullet.beginX, bullet.beginY = self.autoAllyList[id]:GetPosition();
	bullet.beginX = bullet.beginX + bulletInfo.events[5];
	bullet.beginY = bullet.beginY + bulletInfo.events[6];
	if bulletInfo.events[1] == 1 then
		bullet.sp = cc.Sprite:create(imgPath .. 'bullet' .. bulletInfo.events[2] .. '.png');
	else
		return;
	end
	bullet.sp:setLocalZOrder(self.autoAllyList[id]:GetLocalZOrder());
	bullet.sp:setPosition(bullet.beginX, bullet.beginY + DesignSize.height/2);
	bullet.sp:setScale(bulletInfo.events[8])
	bullet.sp:addTo(self.sceneNodeAuto);
	bullet.sp:setVisible(false);
	bullet.showTime = self.autoState2Time + bulletInfo.time/1000;
	bullet.duration = bulletInfo.events[4];
	bullet.trajectory = bulletInfo.events[3];

	table.insert(self.bulletList, bullet);
end

function FightScenePanel:removeAllDamage()
	self.damageList = {};
	for id, v in pairs(self.autoEnemyList) do
		self.autoEnemyLastShowDamage[id] = 0;
	end
end

function FightScenePanel:removeAllEffect()
	self.effectList = {};
end

function FightScenePanel:removeAllBullet()
	for _, v in pairs(self.bulletList) do
		if not v.isRemove then
			v.sp:removeFromParent();
		end
	end
	self.bulletList = {};
end

function FightScenePanel:showAutoDamage(id, damage)
	local delayTime;
	if (self.autoState2Time - self.autoEnemyLastShowDamage[id]) < 0.1 then
		delayTime = 0.1 - (self.autoState2Time - self.autoEnemyLastShowDamage[id]);
		self.autoEnemyLastShowDamage[id] = self.autoEnemyLastShowDamage[id] + 0.1;
	else
		delayTime = 0;
		self.autoEnemyLastShowDamage[id] = self.autoState2Time;
	end
	self:autoEnemyBeHit(id);
	local enemyID = self.autoEnemyList[id]:GetResID();
	local def_type = ResTable:getValue('monster', enemyID, 'def_type');

	local isRestraint = false;
	if (def_type - damage.hit_type) == 1 or (damage.hit_type - def_type == 2) then
		isRestraint = true;
	end

	local showNum = damage.damage;
	local fnt = 'font/num_1.fnt';
	if damage.isCrit and not isRestraint then
		showNum = '!' .. damage.critDamage;
		fnt = 'font/num_2'..self.fnt_ext..'.fnt';
	elseif not damage.isCrit and isRestraint then
		showNum = '!' .. damage.restraintDamage;
		fnt = 'font/num_3'..self.fnt_ext..'.fnt';
	elseif damage.isCrit and isRestraint then
		showNum = '!' .. damage.restraintCritDamage;
		fnt = 'font/num_4'..self.fnt_ext..'.fnt';
	else
		 showNum = '-' .. damage.damage;
	end

	local pos = -1;
	for i=1, 6 do
		if not self.autoEnemyDamageLabelList[id][i] then
			pos = i;
			break;
		end
	end
	if pos == -1 then
		return;
	end

	local numLabel = cc.Label:createWithBMFont(fnt, showNum);
	numLabel:addTo(self.autoEnemyList[id]:getLabelNode());
	self.autoEnemyDamageLabelList[id][pos] = numLabel;

	numLabel:setPosition(damageLabelPos[pos].x+damageLabelOffset.x, damageLabelPos[pos].y+damageLabelOffset.y);
	-- numLabel:runAction(cc.Sequence:create(
	-- 	cc.Spawn:create(
	-- 		cc.MoveBy:create(0.2, cc.p(200, 100)),
	-- 		cc.ScaleTo:create(0.2, 5)
	-- 		),
	-- 	cc.Spawn:create(
	-- 		cc.MoveBy:create(0.2, cc.p(-200, -100)),
	-- 		cc.ScaleTo:create(0.2, 2.5)
	-- 		),
	-- 	cc.MoveBy:create(0.5, cc.p(0, 80)),
	-- 	cc.DelayTime:create(0.2),
	-- 	cc.CallFunc:create(function()
	-- 		numLabel:removeFromParent();
	-- 		self.autoEnemyDamageLabelList[id][pos] = nil
	-- 	end)
	-- 	))

		numLabel:runAction(cc.Sequence:create(
		cc.Spawn:create(
			cc.MoveBy:create(0.1, cc.p(0, 20)),
			cc.ScaleTo:create(0.1, 1.6)
			),
		cc.Spawn:create(
			cc.MoveBy:create(0.1, cc.p(0, -10)),
			cc.ScaleTo:create(0.1, 1.2)
			),
		cc.DelayTime:create(0.1),
		cc.FadeOut:create(0.1),
		cc.CallFunc:create(function()
			numLabel:removeFromParent();
			self.autoEnemyDamageLabelList[id][pos] = nil
		end)
		))

end

function FightScenePanel:autoEnemyBeHit(id)
	local enemy = self.autoEnemyList[id];
	if not enemy then
		return;
	end
	local minHit, maxHit = 5, 10;

	enemy.beHit = enemy.beHit + 1;

	local dieRate = 0;
	if enemy.beHit <= minHit then
		dieRate = 0;
	elseif enemy.beHit > maxHit then
		dieRate = 100;
	else
		dieRate = (enemy.beHit - maxHit)* 20;
	end
	local x, y = enemy:GetPosition();

	if dieRate >= math.random(1, 100) and x < (StageSize.width/2-50) then
		enemy:SetAnimation(AnimationType.die, 1, self.timeScale);
		--EventManager:FireEvent(EventType.AutoFightCoinScatter);
		enemy.isDead = true;
		self:addCoinDrop(id);
		self:addExpDrop(id);
		self:addBallDrop(id);
	end
end

function FightScenePanel:addCoinDrop(id)
	self:dropItem(id, 'coin_effect_fly', self.coinList);
end

function FightScenePanel:addExpDrop(id)
	self:dropItem(id, 'exp_effect_fly', self.coinList);
end

function FightScenePanel:addBallDrop(id)
	if model.HarvestModel:IsOver() then
		return
	end
	if (math.random(1, 100) <= 40) then
		self:dropItem(id, 'ball_effect_fly', self.ballList);
	end
end

function FightScenePanel:dropItem(id, img, list)
	local x, y = self.autoEnemyList[id]:GetPosition();
	local coinNode = cc.Node:create();
	coinNode:setOpacity(0);
	coinNode:setCascadeOpacityEnabled(true);
	coinNode:setPosition(x, y+60);
	coinNode:setLocalZOrder(150);
	coinNode:addTo(self.sceneNodeAuto)

	local randomX = math.random(0, 350);
	local randomY = math.random(0, 70);
	local actionX = cc.MoveBy:create(3, cc.p(-randomX+50, randomY+50));
	local actionFade = cc.Sequence:create(cc.FadeIn:create(0.5), cc.DelayTime:create(2.5));
	local actionY = cc.Sequence:create(cc.MoveBy:create(0.25, cc.p(0, 50)), 
		cc.MoveBy:create(0.5, cc.p(0, -100)), 
		cc.MoveBy:create(0.5, cc.p(0, 90)), 
		cc.MoveBy:create(0.5, cc.p(0, -80)),
		cc.MoveBy:create(0.5, cc.p(0, 70)),
		cc.MoveBy:create(0.5, cc.p(0, -50)),
		cc.MoveBy:create(0.25, cc.p(0, 40))
	);
	coinNode:runAction(cc.Sequence:create(cc.Spawn:create(actionX, actionFade, actionY), cc.CallFunc:create(function()
		local lastX = math.random(40, 70); 
		local actionR = cc.Sequence:create(cc.Spawn:create(cc.MoveBy:create(0.5, cc.p(-lastX/20, 0)), cc.MoveBy:create(0.5, cc.p(0, -40))), cc.Spawn:create(cc.MoveBy:create(0.5, cc.p(-lastX/20, 0)), cc.MoveBy:create(0.5, cc.p(0, 40))));
		coinNode:runAction(cc.RepeatForever:create(actionR));
end)))
	table.insert(list, coinNode);

	local sp = cc.Sprite:create(imgPath .. img .. '.png');
	local scaleR = math.random(6, 10);
	sp:setScale(2*scaleR/10);
	sp:addTo(coinNode);
end

function FightScenePanel:collectCoin()
	for k, v in pairs(self.coinList) do
		if v.isRemove then
			v:removeFromParent();
			self.coinList[k] = nil
		else
			local key = math.random(1, 10);
			local tmpNode = cc.Node:create()
			tmpNode:addTo(v);

			tmpNode:runAction(cc.Sequence:create(cc.DelayTime:create(key/10), cc.CallFunc:create(function()
				v:stopAllActions();
				local x, y = v:getPosition();
				local endPos = self.sceneNodeAuto:convertToNodeSpace( cc.p( StageSize.width-70, 160 ) )
				local aimX, aimY = endPos.x, endPos.y; --StageSize.width/2-70, -StageSize.height/2+113;
				local bezier2 ={
					cc.p((aimX-x)/3+x-100,	(aimY-y)/3+y),
					cc.p((aimX-x)/3*2+x-150,(aimY-y)/3*2+y),
				    cc.p(aimX,			aimY)
				}
				v:runAction(cc.Sequence:create(cc.BezierTo:create(0.3, bezier2), cc.CallFunc:create(function()
					v.isRemove = true;
					v:setVisible(false)
				end)));
			end)));

		end
	end
end

function FightScenePanel:collectBall()
	for k, v in pairs(self.ballList) do
		if v.isRemove then
			v:removeFromParent();
			self.ballList[k] = nil
		else
			local key = math.random(1, 10);
			local tmpNode = cc.Node:create()
			tmpNode:addTo(v);

			tmpNode:runAction(cc.Sequence:create(cc.DelayTime:create(key/10), cc.CallFunc:create(function()
				v:stopAllActions();
				local x, y = v:getPosition();
				local endPos = self.sceneNodeAuto:convertToNodeSpace( cc.p( StageSize.width-70, 160 ) )
				local aimX, aimY = endPos.x, endPos.y;
				local bezier2 ={
					cc.p((aimX-x)/3+x-100,	(aimY-y)/3+y),
					cc.p((aimX-x)/3*2+x-150,(aimY-y)/3*2+y),
				    cc.p(aimX,			aimY)
				}
				v:runAction(cc.Sequence:create(cc.BezierTo:create(0.3, bezier2), cc.CallFunc:create(function()
					v.isRemove = true;
					v:setVisible(false);
				end)));
			end)));
		end
	end
end

function FightScenePanel:updateSkillTime(fighter, cdPercent)
	if fighter:IsEnemy() then
		if self.enemyBVList[fighter:GetPositionIndex()] then
			self.enemyBVList[fighter:GetPositionIndex()]:setSkillPercent(cdPercent);
		end
	else
		if self.allyBVList[fighter:GetPositionIndex()] then
			self.allyBVList[fighter:GetPositionIndex()]:setSkillPercent(cdPercent);
		end
	end
end

-- role
function FightScenePanel:addRole(fighter, pos)
	if fighter then
		local armature = fighter:genFightArmature();
		local bv = require('app.template.BloodVolume'):create(self.sceneRealNode)
		bv:showSkillBar()
		local role_pos
		if fighter:IsEnemy() then
			local formationIndex = fighter:getFormationIndex()
			role_pos = FightManager:getEnemyPosByFormaitionIndex(formationIndex);
			self.enemyHandlerList[pos] = armature
			self.enemyBVList[pos] = bv
			self.enemyDamageList[pos] = {}
			self.eDamSkillLabelList[pos] = {}

			-- bv:setLevel(model.CardModel:GetCard(fighter.pid).level)
			-- 预加载破衣CV
			local resId = fighter:GetResID()
			local allSkillData = ResTable:getRowData('monster', resId) or {};
			local bareid = allSkillData.show;
			if bareid and bareid > 0 then
				local bookInfo = ResTable:getRowData('bare_collect' , bareid);
				local role_id = ResTable:getValue('role', bookInfo.role_id, 'role_id')
				AudioManager:preloadCV(role_id) 
			end

			if FightManager:getFightType() == FightType.GuildBoss 
				or FightManager:getFightType() == FightType.RandomBoss then

				bv:setVisible(false)
			end
		else
			local formationIndex = fighter:getFormationIndex()
			role_pos = FightManager:getRolePosByFormationIndex(formationIndex);
			EventManager:FireEvent(EventType.FightInitSkillIcon, fighter, pos);
			self.allyHandlerList[pos] = armature
			self.allyBVList[pos] = bv
			-- bv:setLevel(model.CardModel:GetCard(fighter.pid).level)
			self.allyDamageList[pos] = {}
			self.aDamSkillLabelList[pos] = {}

			-- 预加载音效
			local resId = fighter:GetResID()
			local role_id = ResTable:getValue('role', resId, 'role_id')
			AudioManager:preloadCV(role_id) 
		end

		armature:setPosition(role_pos.x, role_pos.y + DesignSize.height/2);
		armature:setLocalZOrder(self.roleZorder + role_pos.z);
		armature.zorder = self.roleZorder + role_pos.z
		armature:addTo(self.sceneRealNode);
		armature:setName('role'..tostring(pos));

		bv:setPosition(role_pos.x, role_pos.y+50 + DesignSize.height/2);
		bv:setLocalZOrder(self.roleZorder + role_pos.z+1);
		bv:setMaxHp(fighter.m_propertyData:GetMaxHP());
		bv:setCurHp(fighter.m_propertyData:GetCurrentHP());
		bv:setDefType(fighter.fightAttribute);
		bv:setLevel(fighter:getFightLv());

		-- if fighter:GetArenaRoleMonsterType() ~= ArenaRoleMonsterType.NONE then
		-- 	local rankData = model.ArenaModel:getRankList(fighter.uid);
		-- 	local idx = fighter:GetPositionIndex()
		-- 	if rankData.arena_team and rankData.arena_team[idx] and rankData.arena_team[idx].no ~= -1 then
		-- 		bv:setLevel(rankData.arena_team[idx].level)
		-- 	end
		-- end
	end
end

function FightScenePanel:destroyArmature(isEnemy)
	if isEnemy then
		self:removeEnemy();
	else
		self:removeAlly();
	end
end

function FightScenePanel:removeEnemy()
	for pos, v in pairs(self.enemyHandlerList) do
		v:removeFromParent()
	end
end

function FightScenePanel:removeAlly()
	for pos, v in pairs(self.allyHandlerList) do
		v:removeFromParent()
	end
end

-- effect
function FightScenePanel:addEffect(effectSprite, nodeHandler)
	effectSprite:setName(tostring(nodeHandler:GetID()));
	effectSprite:addTo(self.effNode);
end

function FightScenePanel:delEffect(nodeHandler)
	local name = tostring(nodeHandler:GetID());
	if self.effNode:getChildByName(name) then
		self.effNode:removeChildByName(name);
	end
end

function FightScenePanel:reviveFighter( fighter )
	local bvList
	if fighter:IsEnemy() then
		bvList = self.enemyBVList
	else
		bvList = self.allyBVList
	end
	local pos = fighter:GetPositionIndex();
	if bvList[pos] == nil then
		return 
	end
	local curHp = fighter:getHpNum();
	if curHp and HpCompare(curHp, '>', '0') then
		bvList[pos]:setCurHp(curHp);
		bvList[pos]:setVisible(true);
	end
end

function FightScenePanel:AddLabel(args)
	local handlerList, bvList, damageList
	if args.isEnemy then
		handlerList = self.enemyHandlerList
		bvList = self.enemyBVList
		damageList = self.enemyDamageList
	else
		handlerList = self.allyHandlerList
		bvList = self.allyBVList
		damageList = self.allyDamageList
	end

	if bvList[args.pos] == nil then
		return 
	end
	if args.curHp then
		bvList[args.pos]:setCurHp(args.curHp);
		if HpCompare(args.curHp, '<=', '0') then
			bvList[args.pos]:removeFromParent();
			bvList[args.pos] = nil;
		end
	end

	if args.hitType and args.hitType > 0 then
		self:showSkillDamageLabel(args);
		return;
	end

	local showNum = ConvertStrToDigitUnit(args.value);
	local fnt = 'font/num_1.fnt';
	if args.cast_type == CastType.normal then
		showNum = '-' .. showNum;
		fnt = 'font/num_1.fnt';
	elseif args.cast_type == CastType.crit then
		showNum = '!' .. showNum;
		fnt = 'font/num_2'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.restrain then
		showNum = '!' .. showNum;
		fnt = 'font/num_3'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.restrainCrit then
		showNum = '!' .. showNum;
		fnt = 'font/num_4'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.heal then
		showNum = '+' .. showNum;
		fnt = 'font/num_5.fnt';
	elseif args.cast_type == CastType.absorb then
		fnt = 'font/num_1.fnt';
	elseif args.cast_type == CastType.reflect then
		fnt = 'font/num_1.fnt';
	end

	local index = -1;
	for i=1, 6 do
		if not damageList[args.pos][i] then
			index = i;
			break;
		end
	end
	if index == -1 then
		return;
	end
	
	local lx, ly = handlerList[args.pos]:getPosition(); --handlerList[args.pos]:getChildByName('wenben')
	local labelNode = cc.Node:create()
		:addTo(self.labelNode)
		:setPosition(lx+damageLabelPos[index].x+damageLabelOffset.x + 20, ly+damageLabelPos[index].y+damageLabelOffset.y)

	damageList[args.pos][index] = labelNode;

	local numLabel = cc.Label:createWithBMFont(fnt, showNum)
		:addTo(labelNode)

	local size = numLabel:getContentSize() 
	if args.cast_type == CastType.absorb then
		local sp = cc.Sprite:create('image/buff/shield.png')
			:addTo(labelNode)
		local spSize = sp:getContentSize()
		sp:setPosition(-size.width/2-spSize.width/2, 0)
	elseif args.cast_type == CastType.reflect then
		local sp = cc.Sprite:create('image/buff/reflect.png')
			:addTo(labelNode)
		local spSize = sp:getContentSize()
		sp:setPosition(-size.width/2-spSize.width/2, 0)
	else
		if args.icon and args.icon~='' then
			local icon = cc.Sprite:create('image/icon/'..args.icon..'.png')
			:setPosition( -size.width/2-12 , 0)
			:addTo(labelNode)
		end
	end
	local speedup = FightManager:SpeedUp();
	labelNode:runAction(cc.Sequence:create(
		cc.Spawn:create(
			cc.FadeIn:create(0.08*speedup),
			cc.ScaleTo:create(0.16*speedup, 1)
			),
		cc.DelayTime:create(0.48*speedup),
		cc.FadeOut:create(0.24*speedup),

		-- cc.Spawn:create(
		-- 	cc.MoveBy:create(0.1, cc.p(0, 50)),
		-- 	cc.ScaleTo:create(0.1, 1.6)
		-- 	),
		-- cc.Spawn:create(
		-- 	cc.MoveBy:create(0.1, cc.p(0, -50)),
		-- 	cc.ScaleTo:create(0.1, 1.2)
		-- 	),
		-- cc.DelayTime:create(0.1),
		-- cc.FadeOut:create(0.1),
		cc.CallFunc:create(function()
			if damageList[args.pos] and damageList[args.pos][index] then
				labelNode:removeFromParent();
				damageList[args.pos][index] = nil
			end
		end)
		))
	
end

function FightScenePanel:showSkillDamageLabel( args )
	-- body
	local handlerList, damageList
	if args.isEnemy then
		handlerList = self.enemyHandlerList
		damageList = self.eDamSkillLabelList
	else
		handlerList = self.allyHandlerList
		damageList = self.aDamSkillLabelList
	end

	local showNum = ConvertStrToDigitUnit(args.value);
	local fnt = 'font/num_1.fnt';
	if args.cast_type == CastType.normal then
		showNum = '-' .. showNum;
		fnt = 'font/num_1.fnt';
	elseif args.cast_type == CastType.crit then
		showNum = '!' .. showNum;
		fnt = 'font/num_2'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.restrain then
		showNum = '!' .. showNum;
		fnt = 'font/num_3'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.restrainCrit then
		showNum = '!' .. showNum;
		fnt = 'font/num_4'..self.fnt_ext..'.fnt';
	elseif args.cast_type == CastType.heal then
		showNum = '+' .. showNum;
		fnt = 'font/num_5.fnt';
	elseif args.cast_type == CastType.absorb then
		fnt = 'font/num_1.fnt';
	elseif args.cast_type == CastType.reflect then
		fnt = 'font/num_1.fnt';
	end
	
	local lx, ly = handlerList[args.pos]:getPosition(); --handlerList[args.pos]:getChildByName('wenben')
	local labelNode = cc.Node:create()
		:addTo(self.labelNode)
		:setScale(1.8)
		:setPosition(lx+damageLabelOffset.x + 20, ly+damageLabelOffset.y)

	for _,node in pairs(damageList[args.pos]) do
		local tx,ty = node:getPosition();
		node:setPosition( tx+30, ty+30 );
	end

	table.insert( damageList[args.pos], 1, labelNode )

	local numLabel = cc.Label:createWithBMFont(fnt, showNum)
		:addTo(labelNode)

	local size = numLabel:getContentSize()
	if args.icon and args.icon~='' then
		local icon = cc.Sprite:create('image/icon/'..args.icon..'.png')
		:setPosition( -size.width/2-12 , 0)
		:addTo(labelNode)
	end

	-- local size = numLabel:getContentSize() 
	-- if args.cast_type == CastType.absorb then
	-- 	local sp = cc.Sprite:create('image/buff/shield.png')
	-- 		:addTo(labelNode)
	-- 	local spSize = sp:getContentSize()
	-- 	sp:setPosition(-size.width/2-spSize.width/2, 0)
	-- elseif args.cast_type == CastType.reflect then
	-- 	local sp = cc.Sprite:create('image/buff/reflect.png')
	-- 		:addTo(labelNode)
	-- 	local spSize = sp:getContentSize()
	-- 	sp:setPosition(-size.width/2-spSize.width/2, 0)
	-- end
	local speedup = FightManager:SpeedUp();
	local act = nil
	if args.hitType == 2 then
		labelNode:setScale(1.0);
		act = cc.Sequence:create(
			cc.ScaleTo:create(0.12*speedup, 2),
			cc.ScaleTo:create(0.12*speedup, 1),
			cc.DelayTime:create(0.64*speedup),
			cc.Spawn:create(
				cc.MoveBy:create(0.16*speedup, cc.p(6, 6)),
				cc.FadeOut:create(0.16*speedup)
				),
			cc.CallFunc:create(function()
				if damageList[args.pos] then
					labelNode:removeFromParent();
					table.remove( damageList[args.pos] )
				end
			end)
			)
	else
		act = cc.Sequence:create(
			cc.Spawn:create(
				cc.FadeIn:create(0.08*speedup),
				cc.ScaleTo:create(0.16*speedup, 1)
				),
			cc.DelayTime:create(0.32*speedup),
			cc.Spawn:create(
				cc.MoveBy:create(0.24*speedup, cc.p(6, 6)),
				cc.FadeOut:create(0.24*speedup)
				),
			cc.CallFunc:create(function()
				if damageList[args.pos] then
					labelNode:removeFromParent();
					table.remove( damageList[args.pos] )
				end
			end)
			)
	end
	
	labelNode:runAction(act)

end

function FightScenePanel:clearPanel()
	for k, v in pairs(self.allyBVList) do
		v:removeFromParent()
	end
	self.allyBVList = {}

	for k, v in pairs(self.enemyBVList) do
		v:removeFromParent()
	end
	self.enemyBVList = {}
end

function FightScenePanel:preloadSkillEffect(fighterList)
	local skill_spine_begin = 'skill_spine'
	local skill_spine_bg = 'skill_spine_bg'
	local title_spine = 'title_spine'

	local effectStr = {'red', 'blue', 'green'}

	if not self.nSkillSpineNode then
		self.nSkillSpineNode = cc.Node:create()
		:addTo(self.root)
		:setLocalZOrder(99999)
		:setVisible(false)

		local mask = cc.LayerColor:create(cc.c4b(0, 0, 0, 255))
		mask:setContentSize( cc.size(StageSize.width*3, StageSize.height*3) );
		mask:setPosition(-StageSize.width*3/2, -StageSize.height*3/2);
		mask:addTo(self.nSkillSpineNode);

		-- if not self.spine_play then
		-- 	self.spine_play = {}
		-- 	for k, v in pairs(effectStr) do
		-- 		local spine_str = 'skill_spine_' .. v
		-- 		self.spine_play[v] = sp.SkeletonAnimation:create('spine/'..spine_str.."/"..spine_str..".json", 'spine/'..spine_str.."/"..spine_str..".atlas", 1)
		-- 		-- self.spine_play[v]:setPosition(0, StageSize.height/2)
		-- 		self.spine_play[v]:addTo(self.nSkillSpineNode)
		-- 		self.spine_play[v]:setVisible(false)
		-- 	end
		-- end

		if not self.nSkillSpineSk2 then
			self.nSkillSpineSk2 = sp.SkeletonAnimation:create("spine/skill_spine_n/skill_spine.json", "spine/skill_spine_n/skill_spine.atlas", 1)
			self.nSkillSpineSk2:addTo(self.nSkillSpineNode)
			self.nSkillSpineSk2:setVisible(false)
		end

		if not self.nSkillRoleNode then
			local stencil = cc.Sprite:create('image/fight/skill_clip_bg.png');
			local clipNode = cc.ClippingNode:create()
				:setStencil(stencil)
				:setInverted(false)
				:setAlphaThreshold(0.5)
				:addTo(self.nSkillSpineNode)

			self.nSkillRoleNode = cc.Node:create()
			:addTo(clipNode)
		end

		if not self.nSkillSpineSk then
			self.nSkillSpineSk = sp.SkeletonAnimation:create("spine/skill_spine_n/skill_spine.json", "spine/skill_spine_n/skill_spine.atlas", 1)
			self.nSkillSpineSk:addTo(self.nSkillSpineNode)
			self.nSkillSpineSk:setVisible(false)

			self.nSkillSpineSk:registerSpineEventHandler(function (event)
				if event.eventData.name == 'appear' then
					-- self.nSkillRoleNode:setVisible(true)
					self.nSkillNameNode:setOpacity(255)
					self.nSkillNameNode:setRotation( -20 )
					self.nSkillNameNode:setPosition(cc.p(630,-90))
					self.nSkillNameNode:runAction( cc.EaseBackOut:create(cc.MoveTo:create(0.22, cc.p(60, -290))) )

				elseif event.eventData.name == 'disappear' then
					-- self.nSkillRoleNode:runAction( cc.FadeOut:create(0.2) )
					print('fubin-----event.eventData.name ==disappear')
					self.nSkillNameNode:runAction( cc.FadeOut:create(0.33) )
				
				end
				
			end, sp.EventType.ANIMATION_EVENT)

			self.nSkillSpineSk:registerSpineEventHandler(function()
				self.nSkillSpineNode:setVisible(false)

				self.nSkillRoleNode:removeAllChildren();
				self.nSkillNameNode:removeAllChildren();

				-- self.spine_play['red']:setVisible(false)
				-- self.spine_play['blue']:setVisible(false)
				-- self.spine_play['green']:setVisible(false)

				EventManager:FireEvent(EventType.UIFightFooterPanelVisible, true)

			end, sp.EventType.ANIMATION_COMPLETE)
		end

		if not self.nSkillNameNode then
			self.nSkillNameNode = cc.Node:create()
			:addTo(self.nSkillSpineNode)
			:setPosition(cc.p(20000,0))
			:setCascadeOpacityEnabled(true);
		end

		if not self.nSkillSpineSk3 then
			self.nSkillSpineSk3 = sp.SkeletonAnimation:create("spine/skill_spine_n/skill_spine.json", "spine/skill_spine_n/skill_spine.atlas", 1)
			self.nSkillSpineSk3:addTo(self.nSkillSpineNode)
			self.nSkillSpineSk3:setVisible(false)
		end
	end
		
	if not self.spine_begin then
		self.spine_begin = sp.SkeletonAnimation:create('spine/'..skill_spine_begin.."/"..skill_spine_begin..".json", 'spine/'..skill_spine_begin.."/"..skill_spine_begin..".atlas", 1)
		self.spine_begin:setVisible(false)
		self.spine_begin:addTo(self.sceneRealNode)
		self.spine_begin:setLocalZOrder(200)

		self.spine_begin:registerSpineEventHandler(function (event)
			if event.eventData.name == skill_spine_begin then
				if self.spine_begin_callback then
					self.spine_begin_callback()
				end
			end
		end, sp.EventType.ANIMATION_EVENT)

		self.spine_begin:registerSpineEventHandler(function (event)
			if self.spine_end_callback then
				self.spine_end_callback()
			end
		end, sp.EventType.ANIMATION_COMPLETE)
	end
	

	if not self.show_skill_mask then
		self.show_skill_mask = cc.LayerColor:create(cc.c4b(0, 0, 0, 205))
		self.show_skill_mask:setContentSize( cc.size(StageSize.width*3, StageSize.height*3) );
		self.show_skill_mask:setPosition(-StageSize.width*3/2, -StageSize.height);
		self.show_skill_mask:setLocalZOrder(140);
		self.show_skill_mask:addTo(self.sceneRealNode);
		self.show_skill_mask:setVisible(false)
	end

	if not self.spine_bg then
		self.spine_bg = sp.SkeletonAnimation:create('spine/'..skill_spine_bg.."/"..skill_spine_bg..".json", 'spine/'..skill_spine_bg.."/"..skill_spine_bg..".atlas", 1)
		self.spine_bg:setLocalZOrder(142)
		self.spine_bg:setPosition(0, StageSize.height/2)
		self.spine_bg:addTo(self.sceneNodeFight)
		self.spine_bg:setVisible(false)

		self.spine_bg:registerSpineEventHandler(function()
			if self.spine_play_callback then
				self.spine_play_callback()
			end
		end, sp.EventType.ANIMATION_COMPLETE)
	end

	if not self.particle then
		self.particle = {};
		for k, v in pairs(effectStr) do
			self.particle[v] = {}
			for i=1, 3 do
				local particle_str = 'skill_' .. v .. '_' .. i;
				self.particle[v][i] = cc.ParticleSystemQuad:create('Effect/particle/'..particle_str..'.plist')
				self.particle[v][i]:addTo(self.sceneNodeFight)
				self.particle[v][i]:setLocalZOrder(149)
				self.particle[v][i]:setVisible(false)
				self.particle[v][i]:stopSystem()
			end
		end
	end

	if not self.title_spine then
		self.title_spine = sp.SkeletonAnimation:create('spine/'..title_spine.."/"..title_spine..".json", 'spine/'..title_spine.."/"..title_spine..".atlas", 1)
		self.title_spine:setLocalZOrder(14)
		self.title_spine:setPosition(0, -StageSize.height/2+118)
		self.title_spine:addTo(self.root)
		self.title_spine:setVisible(false)
	end

	for k, v in pairs(fighterList) do
		local navi
		if v:IsEnemy() and v:GetArenaRoleMonsterType() ~= ArenaRoleMonsterType.ROLE then
			navi = ResTable:getValue('monster', v.resID, 'Navi')
		else
			local res_id = ResTable:getValue('role', v.resID, 'resource_id')
			if v.pid and v.pid > 0 then
				res_id = model.CardModel:getCardResIdByPid(v.pid);
			end
			navi = ResTable:getValue('role_resource', res_id, 'card')
		end
		if navi and cc.FileUtils:getInstance():isFileExist('image/role/'..navi..'.png') then
			cc.Director:getInstance():getTextureCache():addImageAsync('image/role/'..navi..'.png', function() end)
		end
	end
end

function FightScenePanel:showMaskAndHighActor(fighter, skill)
	self.show_skill_mask:setVisible(true)

	if fighter:GetID() ~= -1 then -- not god

		local fighterIdx = fighter:GetPositionIndex();
		if fighter:IsEnemy() and self.enemyHandlerList[fighterIdx] then
			self.enemyHandlerList[fighterIdx]:setLocalZOrder(self.enemyHandlerList[fighterIdx].zorder + 50)
			if self.enemyBVList[fighterIdx] then
				self.enemyBVList[fighterIdx]:setLocalZOrder( self.enemyHandlerList[fighterIdx].zorder + 51 );
			end

		elseif not fighter:IsEnemy() and self.allyHandlerList[fighterIdx] then
			self.allyHandlerList[fighterIdx]:setLocalZOrder(self.allyHandlerList[fighterIdx].zorder + 50)
			if self.allyBVList[fighterIdx] then
				self.allyBVList[fighterIdx]:setLocalZOrder( self.allyHandlerList[fighterIdx].zorder + 51 );
			end
		end
		
	end


	for k, v in pairs(skill.targetList) do
		local idx = v:GetPositionIndex();
		if v:IsEnemy() and self.enemyHandlerList[idx] then
			self.enemyHandlerList[idx]:setLocalZOrder(self.enemyHandlerList[idx].zorder + 50)
			if self.enemyBVList[idx] then
				self.enemyBVList[idx]:setLocalZOrder( self.enemyHandlerList[idx].zorder + 51 );
			end
		end
		if not v:IsEnemy() and self.allyHandlerList[idx] then
			self.allyHandlerList[idx]:setLocalZOrder(self.allyHandlerList[idx].zorder + 50)
			if self.allyBVList[idx] then
				self.allyBVList[idx]:setLocalZOrder( self.allyHandlerList[idx].zorder + 51 );
			end
		end
	end

end

function FightScenePanel:showSkillNoFithter(fighter,dt)
	-- body
	local fightNode = fighter:getArmature()
	fightNode:runAction(cc.Sequence:create(

				cc.DelayTime:create(dt*FightManager:SpeedUp()),
				cc.CallFunc:create(function()
					-- self:runScaleTo(nil)
					FightShowSkillManager:setScenePanelShowOver()
					fighter:ContinuePlayAnimation(AnimationType.skill)
					SkillScriptManager:Continue()--技能脚本恢复继续执行				
				end)

				))
	
end

function FightScenePanel:playNewSkill( navi, color, skillName )
	-- body

	local role = cc.Sprite:create('image/role/'..navi..'.png')
		:addTo(self.nSkillRoleNode)
		:setPosition(0,-60)

	local name1Label = cc.Label:createWithTTF(tostring(skillName), fonts.FONT_SIYUAN, 50)
		:addTo(self.nSkillNameNode)
		:enableOutline(cc.c3b(0, 0, 0), 3)

	local speed = FightManager:SpeedUp();
	self.nSkillSpineNode:setVisible(true)
	-- self.spine_play[color]:setVisible(true)
	-- self.spine_play[color]:setAnimation(0, 'skill_spine_' .. color, false)
	-- self.spine_play[color]:setTimeScale(1/speed)

	-- self.nSkillNameNode:setOpacity(0)
	-- self.nSkillRoleNode:setOpacity(255)
	-- self.nSkillRoleNode:setVisible(false)

	self.nSkillSpineSk2:setVisible(true)
	self.nSkillSpineSk2:setAnimation(0, 'skill_spine2', false)
	self.nSkillSpineSk2:setTimeScale(1/speed)

	self.nSkillSpineSk3:setVisible(true)
	self.nSkillSpineSk3:setAnimation(0, 'skill_spine3', false)
	self.nSkillSpineSk3:setTimeScale(1/speed)

	self.nSkillSpineSk:setVisible(true)
	self.nSkillSpineSk:setAnimation(0, 'skill_spine1', false)
	self.nSkillSpineSk:setTimeScale(1/speed)
	EventManager:FireEvent(EventType.UIFightFooterPanelVisible, false)


	self.nSkillRoleNode:setPosition(-1700, 0)
	local m1 = cc.MoveTo:create(0.3*speed, cc.p(-25, 0))
	local m2 = cc.MoveTo:create(1.4*speed, cc.p(25, 0))
	local m3 = cc.MoveTo:create(0.3*speed, cc.p(1700, 0))
	self.nSkillRoleNode:runAction( cc.Sequence:create(m1,m2,m3) )

end

function FightScenePanel:showSkill(fighter, skill)

	local navi 
	if fighter:IsEnemy() and fighter:GetArenaRoleMonsterType() ~= ArenaRoleMonsterType.ROLE then
		navi = ResTable:getValue('monster', fighter.resID, 'Navi')
	else
		if fighter:GetID() == -1 then --god
			navi = 'M044_Navi' --测试用
		else
			local res_id = ResTable:getValue('role', fighter.resID, 'resource_id')
			if fighter.pid and fighter.pid > 0 then
				res_id = model.CardModel:getCardResIdByPid(fighter.pid);
			end
			navi = ResTable:getValue('role_resource', res_id, 'card')
		end
	end

	-- local color = FightRestraintEnum[fighter.fightAttribute]
	local color = FightRestraintEnum[1]
	if fighter:IsEnemy() then
		color = FightRestraintEnum[2]
	end
	if not color or not navi then
		return
	end

	if fighter:GetID() == -1 then --god
		-- self:playNewSkill(navi, color, skill.name)
		self.root:runAction(cc.Sequence:create(
				cc.DelayTime:create(4/3*FightManager:SpeedUp()),
				cc.CallFunc:create(function()
					FightShowSkillManager:setScenePanelShowOver()
					SkillScriptManager:Continue()--技能脚本恢复继续执行			
				end)

				))
		return;
	end
	
	local speed = FightManager:SpeedUp();
	-- spine_begin
	local posX, posY = fighter:getArmature():getPosition()
	self.spine_begin:setVisible(true)
	self.spine_begin:setTimeScale(1/speed)
	self.spine_begin:setPosition(posX, posY-40)
	self.spine_begin:setAnimation(0, 'skill_spine', false)

	self.spine_begin_callback = function()
		AudioManager:playCVStopBefor(fighter.resID ,nil,cvTypeEnum.CV_SKILL, nil,true );
	end

	self.spine_end_callback = function()
		local dt = 0;
		if FightManager:isPlayDrawing() then
			if not fighter:IsEnemy() then
				dt = 4/3;
				self:playNewSkill(navi, color, skill.name)
			else
				if FightManager:getFightType() == FightType.Arena then
					dt = 4/3;
					self:playNewSkill(navi, color, skill.name)
				end
			end
		end
		self:showSkillNoFithter(fighter,dt)
	end
end

function FightScenePanel:showSkillOver(fighter, skill)
	self.show_skill_mask:setVisible(false)

	self.sceneRealNode:setScale(1);

	for k, v in pairs(skill.targetList) do
		local idx = v:GetPositionIndex();
		if v:IsEnemy() and self.enemyHandlerList[idx] then
			self.enemyHandlerList[idx]:setLocalZOrder(self.enemyHandlerList[idx].zorder)
			if self.enemyBVList[idx] then
				self.enemyBVList[idx]:setLocalZOrder( self.enemyHandlerList[idx].zorder+1 );
			end
		end
		if not v:IsEnemy() and self.allyHandlerList[idx] then
			self.allyHandlerList[idx]:setLocalZOrder(self.allyHandlerList[idx].zorder)
			if self.allyBVList[idx] then
				self.allyBVList[idx]:setLocalZOrder( self.allyHandlerList[idx].zorder+1 );
			end

		end
	end
	if fighter:GetID() == -1 then --god
		return;
	end
	local fighterIdx = fighter:GetPositionIndex();
	if fighter:IsEnemy() and self.enemyHandlerList[fighterIdx] then
		self.enemyHandlerList[fighterIdx]:setLocalZOrder(self.enemyHandlerList[fighterIdx].zorder)
		if self.enemyBVList[fighterIdx] then
			self.enemyBVList[fighterIdx]:setLocalZOrder( self.enemyHandlerList[fighterIdx].zorder+1 );
		end

	elseif not fighter:IsEnemy() and self.allyHandlerList[fighterIdx] then
		self.allyHandlerList[fighterIdx]:setLocalZOrder(self.allyHandlerList[fighterIdx].zorder)
		if self.allyBVList[fighterIdx] then
			self.allyBVList[fighterIdx]:setLocalZOrder( self.allyHandlerList[fighterIdx].zorder+1 );
		end
	end
end

function FightScenePanel:ShowShakeEff(shakeType)
	-- body
	self.sceneRealNode:stopAllActions()
	self.sceneRealNode:setPosition(0,0)
	local seq = cc.Sequence:create( cc.MoveBy:create(1/30, cc.p(0,6)), cc.MoveBy:create(1/30, cc.p(0,-6)) )
	self.sceneRealNode:runAction( seq )

	-- if shakeType == 0 then
	-- 	--小震屏
	-- 	self.shakeTotalTime = 0.3;
	-- else
	-- 	--大震屏
	-- 	self.shakeTotalTime = 0.6;
	-- end
	-- self.shakeTotalTime = self.shakeTotalTime*FightManager:SpeedUp()
	-- self.shakeTime = 0;
	-- self.shakeTarget = self.sceneRealNode;
	-- self.shakeOriX, self.shakeOriY = self.shakeTarget:getPosition();
	-- FightShowSkillManager:setShakeEnd(1);
end

function FightScenePanel:ShakeEff(dt)
	if self.shakeTime == -1 then
		return;
	end
	if self.shakeTime >= self.shakeTotalTime then
		self.shakeTime = -1;
		self.shakeTarget:setPosition( self.shakeOriX, self.shakeOriY );
		FightShowSkillManager:setShakeEnd(0);
		return;
	end
	self.shakeTime = self.shakeTime + dt;

	--小震屏
	local min = -15;
	local max = 15;
	local miny = -60;
	local maxy = 60;
	if self.shakeTotalTime == 0.6 then
		--大震屏
		min = -40;
		max = 40;
		miny = -100;
		maxy = 100;
	end
	local rx = (math.random()*(max - min) + min)*dt*5;
	local ry = (math.random()*(maxy - miny) + miny)*dt*5;
	self.shakeTarget:setPosition( self.shakeOriX + rx, self.shakeOriY + ry );
	
end

return FightScenePanel;
