local GachaListPanel = class("GachaListPanel", require('app.layer.BaseLayer'));
local fonts = Enum:getTable("SystemFont");
local imgPath = 'image/chouka/main/';
local MessageBoxType = Enum:getTable('MessageBoxType')

function GachaListPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'GachaListPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.loadWait = true;
	self.showFooter = false;

	self.id = 1;
end

function GachaListPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.currentGachaIndex = 1;  --当前抽卡包

	self.mainIconArr = nil;		--当前背景角色数组
	self.currentIconIndex = 1;	--当前背景角色index

	self:initNewList()

	self:initResultPanel()

	EventManager:RegisterEvent(EventType.NetworkGachaResult, handler(self, self.retGachaResult));
	EventManager:RegisterEvent(EventType.NetworkGachaGetReward, handler(self, self.retGachaTimesReward));
	EventManager:RegisterEvent(EventType.NetworkRetAdmobReward, handler(self, self.updateTicks));
	EventManager:RegisterEvent(EventType.NetworkGachaInfo, handler(self, self.retCheckList));
	
end

function GachaListPanel:createTicketCell( itemId, num, img, parent )
	if self.tickList[itemId] == nil then
		self.tickList[itemId] = {}
	end
	local ticketNode = cc.Node:create();
	-- ticketNode:addTo(self.tickListNode);
	-- ticketNode:setPosition(cc.p(-StageSize.width/2+80+ self.tickIdx*150 , 0))
	ticketNode:addTo(parent)
	ticketNode:setPosition(cc.p(StageSize.width/2-120, 275))

	local ticketBg = cc.Sprite:create(imgPath..'gacha_ticket_bg.png')
	ticketBg:addTo(ticketNode);

	local ticketImg = cc.Sprite:create(img)
		:addTo(ticketNode)
		:setPosition(cc.p(-50,0))
		:setScale(0.5)

	local tickNum= cc.Label:createWithTTF(''..num, fonts.FONT_IMPACT, 20)
		:addTo(ticketNode)
		:setAnchorPoint(cc.p(1,0.5))
		:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2)
		-- :setTextColor(cc.c3b(230, 230, 0))
		:setPosition(cc.p(40 , 0));

	table.insert( self.tickList[itemId], tickNum );
	self.tickIdx = self.tickIdx+1
end

function GachaListPanel:initNewBarNode( parent )
	-- body
	local barNode = cc.Node:create()
	barNode:addTo( parent )
	barNode:setPosition(0, StageSize.height/2 -80 - NOTCH_HEIGHT)

	local bg = cc.Sprite:create(imgPath..'bar_node_bg.png')
	bg:addTo(barNode);

	local barBg = cc.Sprite:create(imgPath..'bar_bg.png')
	barBg:addTo(barNode);
	barBg:setPosition(-5, 0)

	self.barSpr = cc.Scale9Sprite:create(imgPath..'bar.png');
	self.barSpr:setContentSize(cc.size(460, 16))
	self.barSpr:addTo(barNode);
	self.barSpr:setAnchorPoint(0 , 0.5);
	self.barSpr:setPosition(-235 , 0);

	self.barLabel = cc.Label:createWithTTF('999/1000', fonts.FONT_IMPACT, 16)
		:addTo(barNode)
		:setPosition(-5, 0)
		:setAnchorPoint(cc.p(0.5,0.5))
		:enableOutline(cc.c3b(180 , 120 , 40) , 2)

	self.barBtn = ccui.Button:create(imgPath..'gacha_num_reward1.png', imgPath..'gacha_num_reward1.png', imgPath..'gacha_num_reward2.png');
	self.barBtn:addTo(barNode);
	self.barBtn:setPosition(270 , 0);
	self.barBtn:addClickEventListener(handler(self , self.gachaTimesRewardClick));
	self.barBtn:setEnabled(false)

	self.rwdLabel = cc.Label:createWithTTF(LANG.UI_FIGHT_5, fonts.FONT_IMPACT, 14)
		:addTo(barNode)
		:setPosition(270 , 0)
		:enableOutline(cc.c3b(190 , 60 , 15) , 2)

	local barIcon = cc.Sprite:create("image/icon/I11201.png")
	barIcon:addTo(barNode);
	barIcon:setScale(0.5)
	barIcon:setPosition(-280, 0);
	local barIconLabel = cc.Label:createWithTTF('x1', fonts.FONT_IMPACT, 16)
		:addTo(barNode)
		:setPosition(-260, -10)
		:enableOutline(cc.c3b(60 , 60 , 60) , 2)

	local barTipBtn = ccui.Button:create('image/commonImg/help_1.png');
	barTipBtn:addTo(barNode);
	barTipBtn:setPosition(310 , 0);
	barTipBtn:setVisible(false);
	barTipBtn:addClickEventListener(function() 
		UIManager:push('HelpPanel',self.name);
	end);

	local spine_res = 'double_time_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(barNode)
		spine:setPosition(-185, 15)
		spine:setAnimation(0, "double_time_spine", true)
		self.dbTimeSp = spine
	end
	self.dbTimeSp:setVisible(false)
	self.barSpr:setColor( cc.c3b(255,255,0) )

end

function GachaListPanel:initNewList()
	-- body
	local newRoot = cc.Node:create()
	newRoot:addTo(self.root)
	newRoot:setLocalZOrder(0)

	local bg = cc.Sprite:create(imgPath..'bg_3.png')
	bg:addTo(newRoot)

	local bottomHeight = StageSize.height-300;
	local scorllWidth = StageSize.width

	self.scorllview = ccui.ScrollView:create();
	self.scorllview:setTouchEnabled(true);
	self.scorllview:setBounceEnabled(true);
	self.scorllview:setDirection(ccui.ScrollViewDir.vertical);
	self.scorllview:setContentSize(cc.size(scorllWidth , bottomHeight));
	self.scorllview:setAnchorPoint(cc.p(0,0));
	self.scorllview:setScrollBarEnabled(false);
    self.scorllview:setPosition(-scorllWidth/2 , -bottomHeight/2);
    self.scorllview:addTo(newRoot);

 --    local mask1 = cc.Scale9Sprite:create(imgPath..'header_mask.png')
 --    :addTo(newRoot)
 --    :setCapInsets(cc.rect(0,0,16,83))
	-- :setContentSize(cc.size(640 , 226))
	-- :setAnchorPoint(0.5, 1)

	local mask2 = cc.Scale9Sprite:create(imgPath..'footer_mask.png')
    :addTo(newRoot)
	:setContentSize(cc.size(640 , 140))
	:setAnchorPoint(0.5, 0)
	:setPosition( 0 , -StageSize.height/2)

    self.tickIdx = 0;
    self.tickList = {}
    self.tickListNode = cc.Node:create()
    self.tickListNode:addTo( newRoot )
    self.tickListNode:setPosition(0, StageSize.height/2 -80 - NOTCH_HEIGHT)

    self.bagCellList = {}
    self.limitRwdCell = {}

    self:initNewBarNode( newRoot )
    self.initCell = {}
    self.initCell[1] = 1
    self.initCell[2] = 1
    self.initCell[3] = 1
    self:updateList(); --初始化一次 初始卡包
    self.initCell = {}

end

function GachaListPanel:retCheckList()
	if not model.GuideModel:isInGuide() then
    	self:updateList();
    end
end

function GachaListPanel:updateList()
	local totalH = 100
	local baseBagPos = 0;
	local scorllWidth = StageSize.width;
	local curTime = TimeManager:getCurTime();

	for k,v in pairs(model.GachaModel:getBagOrderList()) do
		local currentBagData = model.GachaModel:getContractData(v);
		if currentBagData then
			local checkData = model.GachaModel:getBagRateData( currentBagData.tab )
			local closeTime = -1;
			if checkData then
				closeTime = tonumber(checkData[3]);
			end

			if self.bagCellList[currentBagData.tab]==nil then
				if self.initCell[currentBagData.tab] or ( checkData and (closeTime == -1 or closeTime>= curTime) ) then
					local cellNode = cc.Node:create()
			    	cellNode:setPosition(scorllWidth/2, totalH)
			    	cellNode:addTo(self.scorllview)

			    	local ExtOffY = 0
			    	local isOpen = tools.ConfigStateTools:checkLockById(4902);
			    	if currentBagData.tab == 1 and isOpen==false then
			    		local extH = self:initAdmobGachaNode(cellNode)
			    		totalH = totalH + extH
			    		cellNode:setPosition(scorllWidth/2, totalH)
			    		ExtOffY = extH
					end

			    	local cellBg = cc.Sprite:create('image/chouka/main/'..currentBagData.img ..'.png')
			    	if cellBg == nil then
			    		cellBg = cc.Sprite:create('image/chouka/main/cell_bg_1.png')
			    	end
			    	cellBg:addTo(cellNode);
			    	cellBg:setAnchorPoint(0.5, 0);
			    	cellBg:setPosition(0, 80);

			    	local bgW = cellBg:getContentSize().width;
			    	local bgH = cellBg:getContentSize().height;

			    	local clipNode = cc.ClippingRectangleNode:create( cc.rect(-bgW/2,-bgH/2,bgW,bgH) )
			    	clipNode:addTo(cellBg)
			    	clipNode:setPosition(bgW/2, bgH/2)

			    	local aniName = ''
			    	local aniPos = cc.p(0,0)
					if currentBagData.tab == 3 then
						aniName = 'idle3'
						aniPos = cc.p(120,10)
					elseif currentBagData.tab == 2 then
						aniName = 'idle2'
						aniPos = cc.p(-5,15)
					elseif currentBagData.tab == 1 then
						aniName = 'idle1'
						aniPos = cc.p(-5,15)
					end

					if currentBagData.tab == 3 then
						local spine_res = 'R033_Spine';
						if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
							local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
							spine:addTo(clipNode)
							spine:setPosition(-170, -230)
							spine:setAnimation(0, "idle", true)
						end
					end

					if currentBagData.tab == 1 or currentBagData.tab == 2 or currentBagData.tab == 3 then
				    	local spine_res = 'slogan_spine';
						if aniName~='' and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
							local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
							spine:addTo(clipNode)
							spine:setPosition(aniPos)
							spine:setAnimation(0, GlobalFunctions:lAnimationName(aniName), true)
						end
					else
						local particle = cc.ParticleSystemQuad:create("Effect/particle/banner.plist")
						particle:addTo(clipNode);
						particle:setPositionType(cc.POSITION_TYPE_RELATIVE);
					end

			    	local btnNode = cc.Node:create()
			    	btnNode:addTo(cellNode)
			    	btnNode:setPosition(0, 45)
			    	self:createCellBtns(btnNode, currentBagData, currentBagData.tab)

			    	local probabilityBtn = ccui.Button:create('image/chouka/main/probability_bg_1.png');
					probabilityBtn:addTo(cellNode);
					probabilityBtn:setTag(currentBagData.tab);
					probabilityBtn:setPosition(scorllWidth/2-30 , 320);
					probabilityBtn:addClickEventListener(handler(self , self.showProbabilityPanelByTag));

					local tid = currentBagData.tab;
			    	self.bagCellList[tid].cellNode = cellNode
			    	self.bagCellList[tid].extOffY = ExtOffY
			    	self.bagCellList[tid].cellHeight = cellBg:getContentSize().height+80

			    	totalH = totalH + cellBg:getContentSize().height+80
			    	if currentBagData.tab == 1 then --基础卡包
			    		baseBagPos = totalH;
			    	end

			    	local rwdId = currentBagData.reward_id;
			    	if rwdId and rwdId>1 then
			    		self:createRwdBarNode(cellNode, tid, rwdId);
			    	end
			    end

			else
				local tid = currentBagData.tab;
				if checkData and (closeTime == -1 or closeTime>= curTime) then
					local extOffY = self.bagCellList[tid].extOffY;
					self.bagCellList[tid].cellNode:setPosition(scorllWidth/2, totalH+extOffY)
					totalH = totalH + extOffY + self.bagCellList[tid].cellHeight;

					if currentBagData.tab == 1 then --基础卡包
			    		baseBagPos = totalH;
			    	end
			    	self.bagCellList[tid].cellNode:setVisible(true)
			    else
			    	self.bagCellList[tid].cellNode:setVisible(false)
			    end
			end
		end
	end

	local footerHeight = 0
	local tmpTop = StageSize.height/2 -120 - NOTCH_HEIGHT
	local lastHeight = tmpTop + StageSize.height/2 - footerHeight; --剩余高度空间 120是footer高度

	self.scorllview:setInnerContainerSize(cc.size(scorllWidth, totalH));

	self.baseBagPos = 0;
	if totalH > lastHeight then
		self.scorllview:setPosition(-scorllWidth/2 , -StageSize.height/2 + footerHeight);
		self.scorllview:setContentSize(cc.size(scorllWidth , lastHeight));
		if baseBagPos >= lastHeight then
			self.baseBagPos = (totalH-baseBagPos)/(totalH-lastHeight)*100;
		else
			self.baseBagPos = 100;
		end

	else
		for k,v in pairs(self.bagCellList) do
			local x,y = v.cellNode:getPosition()
			v.cellNode:setPosition(x, y+(lastHeight-totalH))
		end
		totalH = lastHeight
		self.scorllview:setPosition(-scorllWidth/2 , -StageSize.height/2+(lastHeight-totalH) + footerHeight);
		self.scorllview:setContentSize(cc.size(scorllWidth , totalH));
		self.scorllview:setInnerContainerPosition(cc.p(0,0))
		
	end
	
    if model.GuideModel:isInGuide() then
    	self.scorllview:jumpToPercentVertical( self.baseBagPos );
    end

end

function GachaListPanel:createRwdBarNode( pNode,  tid, rwdId)
	-- body
	local rwdData = ResTable:getRowData('contract_reward', rwdId);

	local barNode = cc.Node:create()
	barNode:addTo( pNode )
	barNode:setPosition(0, 275+40)

	local barBg = cc.Sprite:create(imgPath..'lm_rwd_barbg.png')
	barBg:addTo(barNode);
	barBg:setPosition(0, 0)

	local clipNode = cc.ClippingRectangleNode:create( cc.rect(-100,-10,200,20) )
	clipNode:addTo(barNode)
	clipNode:setPosition(0, 0)
	-- clipNode:setClippingRegion( cc.rect(-100,-10,50,20) )

	local barSpr = cc.Scale9Sprite:create(imgPath..'lm_rwd_bar.png');
	barSpr:setContentSize(cc.size(200, 10))
	barSpr:addTo(clipNode);

	local spine_res = 'bannerlingqu_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(clipNode)
		spine:setAnimation(0, 'idle1', true)
	end

	local barLabel = cc.Label:createWithTTF('0/1', fonts.FONT_IMPACT, 16)
		:addTo(barNode)
		:setPosition(0, 0)
		:setAnchorPoint(cc.p(0.5,0.5))
		:enableOutline(cc.c3b(60 , 125 , 170) , 2)

	local headBg = cc.Sprite:create(imgPath..'lm_head_bg.png')
	headBg:setPosition(-120, 0)
	headBg:addTo(barNode)
	headBg:setScale(0.5)

	local rwdItem = {10010, 1};
	if rwdData.reward and rwdData.reward[1] then
		rwdItem = rwdData.reward[1]
	end

	local iconTemplate = template.ItemTemplate:create(2);
	iconTemplate:addTo(barNode);
	iconTemplate:setScale(0.5);
	iconTemplate:setPosition(-120, 0);
	iconTemplate:addItemClickEventListener(handler(self , self.getLimitRwd));
	iconTemplate:Init(rwdItem[1] , rwdItem[2]);
	iconTemplate:setItemTag(tid);

	local spine_res = 'bannerlingqu_spine';
	local headSpine = nil
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(barNode)
		spine:setPosition(-120, 0)
		spine:setAnimation(0, 'idle2', true)
		headSpine = spine;
	end

	self.limitRwdCell[tid] = {clipNode=clipNode, barLabel=barLabel, headSpine=headSpine, rwdId=rwdId, cntTimes=rwdData.times, rwdType=rwdData.type }

	self:updateRwdBarInfo(tid);

end

function GachaListPanel:getLimitRwd( sender,event )
	if self.isShowAction then
		return ;
	end
	local tag = sender:getTag();
	if self.limitRwdCell[tag] == nil then
		return
	end

	local num = model.GachaModel:getGachaTimesByType( self.limitRwdCell[tag].rwdType ) - model.GachaModel:getGachaRwdTimesByType( self.limitRwdCell[tag].rwdType )
	if num >= self.limitRwdCell[tag].cntTimes then
		model.GachaModel:reqGachaTimesReward( self.limitRwdCell[tag].rwdId )
	else
		Prompt:Show(LANG.UI_GACHA_31);
	end
end

function GachaListPanel:updateRwdBarInfo( tid )
	if self.limitRwdCell[tid] == nil then
		return
	end

	local curTimes = model.GachaModel:getGachaTimesByType(self.limitRwdCell[tid].rwdType) - model.GachaModel:getGachaRwdTimesByType( self.limitRwdCell[tid].rwdType )
	local totalTimes = self.limitRwdCell[tid].cntTimes;
	-- if curTimes > totalTimes then
	-- 	curTimes = totalTimes
	-- end
	local pro = math.min(curTimes/totalTimes, 1)
	local wBar = pro*200;
	self.limitRwdCell[tid].clipNode:setClippingRegion( cc.rect(-100,-10,wBar,20) )
	self.limitRwdCell[tid].barLabel:setString(curTimes..'/'..totalTimes)
	self.limitRwdCell[tid].headSpine:setVisible( curTimes>=totalTimes )
end


function GachaListPanel:update()
	local inEvent = model.OperateModel:isValidByType(14);
	if inEvent==true and self.dbTimeSp:isVisible()==false then
		local t1 = cc.TintTo:create(0.33, 255,0,145)
		local t2 = cc.TintTo:create(0.33, 220,78,255)
		local t3 = cc.TintTo:create(0.33, 0,255,255)
		local t4 = cc.TintTo:create(0.33, 0,128,255)
		local t5 = cc.TintTo:create(0.33, 136,0,255)
		local t6 = cc.TintTo:create(0.33, 255,0,145)
		self.barSpr:runAction( cc.RepeatForever:create( cc.Sequence:create(t1,t2,t3,t4,t5,t6) )  )

		self.dbTimeSp:setVisible(true)
	elseif inEvent==false and self.dbTimeSp:isVisible()==true then
		self.dbTimeSp:setVisible(false)
		self.barSpr:stopAllActions();
		self.barSpr:setColor( cc.c3b(255,255,0) )
	end

	self:onUpdateAdmobInfo();
end

function GachaListPanel:initAdmobGachaNode(parentNode)
	local node = cc.Node:create();
	node:addTo(parentNode)

	local adBg = cc.Sprite:create('image/chouka/main/admob_bg.png')
	adBg:addTo(node)
	adBg:setAnchorPoint(0.5, 0)

	local bg = cc.Sprite:create(imgPath..'btn_bg.png')
	bg:addTo(node);
	bg:setAnchorPoint(0.5, 1)

	local adBgW = adBg:getContentSize().width;
	local adBgH = adBg:getContentSize().height;
	node:setPosition(0, -adBgH+10)

	local clipNode = cc.ClippingRectangleNode:create( cc.rect(-adBgW/2,-adBgH/2,adBgW,adBgH) )
	clipNode:addTo(node)
	clipNode:setPosition(0, adBgH/2)
	local spine_res = 'slogan_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		spine:addTo(clipNode)
		spine:setAnimation(0, GlobalFunctions:lAnimationName('idle4'), true)
	end

	self.admobBtnNode = cc.Node:create()
	self.admobBtnNode:addTo(node)
	self.admobBtnNode:setPosition(0, -40)

	local btn = ccui.Button:create('image/chouka/main/admob_b.png','image/chouka/main/admob_b.png');
	btn:addClickEventListener(handler(self,self.onAdmobClick))
	btn:addTo(self.admobBtnNode);
	local admob_ico = cc.Sprite:create('image/chouka/main/admob_ico.png')
	admob_ico:addTo(self.admobBtnNode)
	admob_ico:setPosition(-50,0)
	local label1 = cc.Label:createWithTTF(LANG.UI_GACHA_29, fonts.FONT_SIYUAN, 18)
		:enableOutline(cc.c3b(95 , 60 , 0) , 1)
		:addTo(self.admobBtnNode)
		:setPosition(40, 0)

	self.admobTimeNode = cc.Node:create()
	self.admobTimeNode:addTo(node)
	self.admobTimeNode:setPosition(0, -40)

	local adTimeBg = cc.Sprite:create('image/chouka/main/admob_ub.png')
	adTimeBg:addTo(self.admobTimeNode)
	local admob_tbg = cc.Sprite:create('image/chouka/main/admob_ico1.png')
	admob_tbg:addTo(self.admobTimeNode)
	admob_tbg:setPosition(-50,0)
	local label2 = cc.Label:createWithTTF(LANG.UI_GACHA_30, fonts.FONT_SIYUAN, 16)
		:enableOutline(cc.c3b(160 , 160 , 160) , 1)
		:addTo(self.admobTimeNode)
		:setPosition(50, 0)

	self.adTimeLabel = cc.Label:createWithTTF('20:20:20', fonts.FONT_SIYUAN, 14)
		:enableOutline(cc.c3b(160 , 160 , 160) , 1)
		:addTo(self.admobTimeNode)
		:setPosition(-25, 0)

	local probabilityBtn = ccui.Button:create('image/chouka/main/probability_bg_1.png');
	probabilityBtn:addTo(node);
	probabilityBtn:setPosition(StageSize.width/2-30 , 235);
	probabilityBtn:addClickEventListener(function()
		UIManager:push('ItemProbabilityPanel' , 18004);
	end);

	return adBgH + 70

end

function GachaListPanel:onUpdateAdmobInfo()
	if self.admobTimeNode then
		local admobItem = model.AdmobModel:getAdmobItem(3);
		if admobItem and admobItem.next_time then
			local curTime = TimeManager:getCurTime()
			if admobItem.next_time <= curTime then
				if self.admobTimeNode:isVisible() then
					self.admobBtnNode:setVisible(true)
					self.admobTimeNode:setVisible(false)
				end
			else
				if self.admobBtnNode:isVisible() then
					self.admobBtnNode:setVisible(false)
					self.admobTimeNode:setVisible(true)
				end
				local lastTime = math.floor(admobItem.next_time - curTime)
				lastTime = lastTime > 2*3600 and 2*3600 or lastTime;
				self.adTimeLabel:setString( GlobalFunctions:ChangeTime(lastTime) )
			end
		end
	end
end

function GachaListPanel:onAdmobClick()
	model.AdmobModel:onAdmobClick(3)
end

function GachaListPanel:updateCellBtns( bagData, bagId )
	if self.bagCellList[bagId] == nil then
		return
	end
	local costSingle = bagData['cost']; 		--单抽消耗
	local costTen = bagData['multi_cost'] 		--十连消耗
	local proxy_cost = bagData['proxy_cost'];	--抽卡券 

	local costIcon = ''
	local costMoney = '';
	--如果有抽卡券显示抽卡券
	local proxyId = nil;
	local proxyTotalNum= 0
	if proxy_cost ~= nil and next(proxy_cost) then
		-- 11001,11002,11003...
		local _k,v = next(proxy_cost,_k)
		proxyId = v;
		while _k do 
			local num = model.ItemModel:getItemNum(v);
			if num > 0 then
				proxyId = v;
				-- print('抽卡券素材：id === '..v..".....num === "..tostring(num));
				proxyTotalNum = proxyTotalNum + num;
			end
			_k,v = next(proxy_cost,_k)
		end
	end
	local proxyIcon =   "image/icon/I"..tostring(ResTable:getValue('item',proxyId,'icon'))..".png";
	if proxyId == nil then
		proxyIcon =   "image/icon/I10002.png";
	end
	-- 当前抽卡券数量以及icon
	-- self.ticketImg:setTexture(proxyIcon)
	-- self.tickNum:setString(tostring(proxyTotalNum))
	if self.tickList[ proxyId ] and proxyId then
		for k,v in pairs(self.tickList[ proxyId ]) do
			v:setString( tostring(proxyTotalNum) )
		end
	end

	--单抽 10抽 默认使用抽卡券
	local costStr1,costStr10,costImg1,costImg10 = '1','10',proxyIcon,proxyIcon
	-- 只有普通抽卡可以消耗非抽卡券

	local diamondFst = 0
	if  proxyTotalNum < 1 and costSingle[1] ~= nil then
		local icon1 = ResTable:getValue('item',costSingle[1],'icon')
		costImg1 = "image/icon/I"..icon1..".png"; 
		costStr1 = costSingle[2];
		diamondFst=costSingle[2]
	end
	if self.bagCellList[bagId] and proxyId then
		self.bagCellList[bagId].singleMoney:setTexture(costImg1);
		self.bagCellList[bagId].btnSingle_label:setString(costStr1)

		self.bagCellList[bagId].tenMoney:setTexture(costImg10);
		self.bagCellList[bagId].btnTen_label:setString(costStr10);

		self.bagCellList[bagId].diamondFst = diamondFst;
	end

end

function GachaListPanel:createCellBtns( node, bagData, bagId )
	local costSingle = bagData['cost']; 		--单抽消耗
	local costTen = bagData['multi_cost'] 		--十连消耗
	local proxy_cost = bagData['proxy_cost'];	--抽卡券 

	local costIcon = ''
	local costMoney = '';
	--如果有抽卡券显示抽卡券
	local proxyId = nil;
	local proxyTotalNum= 0
	if proxy_cost ~= nil and next(proxy_cost) then
		-- 11001,11002,11003...
		local _k,v = next(proxy_cost,_k)
		proxyId = v;
		while _k do 
			local num = model.ItemModel:getItemNum(v);
			if num > 0 then
				proxyId = v;
				-- print('抽卡券素材：id === '..v..".....num === "..tostring(num));
				proxyTotalNum = proxyTotalNum + num;
			end
			_k,v = next(proxy_cost,_k)
		end
	end
	local proxyIcon =   "image/icon/I"..tostring(ResTable:getValue('item',proxyId,'icon'))..".png"; 
	-- 当前抽卡券数量以及icon
	print('fubin----createTicketCell:::', bagId, proxyId, proxyTotalNum,  proxyIcon, node)
	if proxyId then
		self:createTicketCell( proxyId, proxyTotalNum,  proxyIcon, node)
	else
		proxyIcon =   "image/icon/I10002.png";
	end
	-- self.ticketImg:setTexture(proxyIcon)
	-- self.tickNum:setString(tostring(proxyTotalNum))

	--单抽 10抽 默认使用抽卡券
	local costStr1,costStr10,costImg1,costImg10 = '1','10',proxyIcon,proxyIcon
	-- 只有普通抽卡可以消耗非抽卡券

	local diamondFst = 0;
	if  proxyTotalNum < 1 and costSingle[1] ~= nil then
		local icon1 = ResTable:getValue('item',costSingle[1],'icon')
		costImg1 = "image/icon/I"..icon1..".png"; 
		costStr1 = costSingle[2];
		diamondFst = costSingle[2];
	end

	local btnWidth = 285
	local btnSprPath = imgPath..'gacha_btn4.png'
	local offx = 20
	if costTen and costTen[2] and proxyId then --3个按钮
		btnWidth = 196
		btnSprPath = imgPath..'gacha_btn1.png'
		offx = 0
	end

	local bg = cc.Sprite:create(imgPath..'btn_bg.png')
	bg:addTo(node);

	-- 单抽次数按钮
	local btnLeftNode = cc.Node:create();
	btnLeftNode:addTo(node);
	btnLeftNode:setPosition(cc.p(- btnWidth/2 -20, 0))
	btnLeftNode:setCascadeOpacityEnabled(true);

	-- 单次广告按钮
	if bagData.tab and bagData.tab == 1 then
		local restTimes = model.GachaAdmobModel:getRestTimes(GACHA_ADMOB_TYPE_17);
		local maxTimes = model.GachaAdmobModel:getMaxTimes(GACHA_ADMOB_TYPE_17);
		if restTimes and maxTimes then
			local admobNode = {};
			admobNode.node = cc.Node:create()
				:setPosition(cc.p(-130, 60))
				:addTo(node)
			admobNode.admobBtn = ccui.Button:create('image/chouka/main/admob_a.png');
			admobNode.admobBtn:addTo(admobNode.node);
			admobNode.admobBtn:setZoomScale(0)
			admobNode.admobBtn:setPosition(cc.p(0, 0))
			admobNode.admobBtn:addClickEventListener(function()	
				local restTimes = model.GachaAdmobModel:getRestTimes(GACHA_ADMOB_TYPE_17);
				if restTimes <= 0 then
					return;
				end
				local args = {
					admob_index = 4
				}			
				sdkManager.SDKLuaHelper:playAdMob(args, function(strData)
					local jsonData = Cjson.decode(strData)
					jsonData.code = tonumber(jsonData.code)
			        if jsonData.code and jsonData.code ~= 0 and jsonData.code ~= 100 then
						if jsonData.code == 3 then
			                Prompt:Show(LANG.UI_ADMOB_9)
						else
			                Prompt:Show(LANG.UI_ADMOB_10)
						end
			            return
			        end
					local reqGacha = false
					if jsonData.code and jsonData.code == 0 then
						reqGacha = true
					end
					if not reqGacha then
						return
					end
					local msg = {
						no 		= 1,
						flag    = 4,
						is_release = sdkManager.SDKLuaHelper:retApkRelease(),
					}
					Network.NetworkManager:Send(Network.NetworkCmdType.req_gacha, msg);

					model.GachaAdmobModel:consume(GACHA_ADMOB_TYPE_17);
					local restTimes = model.GachaAdmobModel:getRestTimes(GACHA_ADMOB_TYPE_17);
					if restTimes <= 0 then
						admobNode.label1:setVisible(false)
						admobNode.icon1:setVisible(false)
						admobNode.label2:setVisible(true)
						admobNode.icon2:setVisible(true)
					else
						admobNode.label1:setString(LANG.UI_GACHA_32 .. string.format( "%d/%d", restTimes, maxTimes))
					end
				end)
			end)
			admobNode.label1 = cc.Label:createWithTTF(LANG.UI_GACHA_32 .. string.format( "%d/%d", restTimes, maxTimes), fonts.FONT_SIYUAN, 18)
				:setAnchorPoint(cc.p(0, 0.5))
				:enableOutline(cc.c3b(94 , 62 , 0) , 1)
				:setTextColor(cc.c3b(255,255,255))
				:addTo(admobNode.node)
				:setPosition(-65, 9)
			admobNode.icon1 = cc.Sprite:create('image/chouka/main/admob_ico_small.png')
				:addTo(admobNode.node)
				:setScale(1)
				:setPosition(-90, 8)
			admobNode.label2 = cc.Label:createWithTTF(string.format(LANG.UI_GACHA_33, '00:00:00'), fonts.FONT_SIYUAN, 18)
				:setAnchorPoint(cc.p(0, 0.5))
				:enableOutline(cc.c3b(94 , 62 , 0) , 1)
				:setTextColor(cc.c3b(255,255,255))
				:addTo(admobNode.node)
				:setPosition(-65, 9)
			admobNode.icon2 = cc.Sprite:create('image/chouka/main/admob_ico_small_disable.png')
				:addTo(admobNode.node)
				:setScale(1)
				:setPosition(-90, 8)
			TimeManager:registUpdateEvent('GachaAdmobCount',function()
				local rT2T = TimeManager:getTime2Tomorrow();
				local sec = rT2T % 60
				local min = ((rT2T - sec) / 60) % 60
				local hour = math.floor((rT2T - sec - min*60) / 3600)
				sec = tostring(sec)
				min = tostring(min)
				hour = tostring(hour)				
				if string.len(sec) == 1 then sec = '0'..sec end
				if string.len(min) == 1 then min = '0'..min end
				if string.len(hour) == 1 then hour = '0'..hour end
				admobNode.label2:setString(string.format(LANG.UI_GACHA_33, hour..':'..min..':'..sec))
			end,true);
			if restTimes > 0 then
				admobNode.label1:setVisible(true)
				admobNode.icon1:setVisible(true)
				admobNode.label2:setVisible(false)
				admobNode.icon2:setVisible(false)
			else
				admobNode.label1:setVisible(false)
				admobNode.icon1:setVisible(false)
				admobNode.label2:setVisible(true)
				admobNode.icon2:setVisible(true)
			end
		else
		end
	end

	local btnLeftImg = cc.Scale9Sprite:create(btnSprPath);
	-- btnLeftImg:setContentSize( cc.size(btnWidth, 70) )
	btnLeftImg:addTo(btnLeftNode);
	btnLeftImg:setPosition(0, -2)

	local btnLeftItembg = cc.Sprite:create(imgPath..'gacha_itemBg.png');
	btnLeftItembg:addTo(btnLeftNode);
	btnLeftItembg:setPosition(-30-offx,0)

	local btnLeft = ccui.Layout:create();
	btnLeft:addClickEventListener(handler(self,self.selectNewGachaCount));
	btnLeft:setTouchEnabled(true);
	btnLeft:setContentSize(cc.size(btnWidth , 54));
	btnLeft:setAnchorPoint(cc.p(0.5,0.5));
	btnLeft:setTag(bagId*10+1);
	btnLeft:addTo(btnLeftNode);

	local singleTimes = cc.Label:createWithTTF(LANG.UI_GACHA_22 , fonts.FONT_SIYUAN , 16)
		:enableOutline(cc.c3b(60 , 65 , 60) , 2)
		:addTo(btnLeftNode)
		:setPosition(cc.p(35+offx,0))

	local singleMoney = cc.Sprite:create(costImg1);
	singleMoney:addTo(btnLeftNode);
	singleMoney:setPosition(cc.p(-70-offx,0))
	singleMoney:setScale(0.4);

	local btnSingle_label = cc.Label:createWithTTF(costStr1, fonts.FONT_IMPACT, 14);
	btnSingle_label:setPosition(-20-offx, 0);
	btnSingle_label:addTo(btnLeftNode);

	-- 十抽次数按钮
	local btnRightNode = cc.Node:create();
	btnRightNode:addTo(node);
	btnRightNode:setPosition(cc.p(0, 0));
	btnRightNode:setCascadeOpacityEnabled(true)

	local btnRightImg = cc.Scale9Sprite:create(btnSprPath);
	-- btnRightImg:setContentSize( cc.size(btnWidth, 70) )
	btnRightImg:addTo(btnRightNode);
	btnRightImg:setPosition(0, -2)

	local btnRightItembg = cc.Sprite:create(imgPath..'gacha_itemBg.png');
	btnRightItembg:addTo(btnRightNode);
	btnRightItembg:setPosition(-30-offx,0)

	local btnRight = ccui.Layout:create();
	btnRight:addClickEventListener(handler(self,self.selectNewGachaCount));
	btnRight:setTouchEnabled(true);
	btnRight:setContentSize(cc.size(btnWidth , 54));
	btnRight:setAnchorPoint(cc.p(0.5,0.5));
	btnRight:setTag(bagId*10+2);
	btnRight:addTo(btnRightNode);

	local tenTimes = cc.Label:createWithTTF(LANG.UI_GACHA_23 , fonts.FONT_SIYUAN , 16)
		:enableOutline(cc.c3b(60 , 65 , 60) , 2)
		:addTo(btnRightNode)
		:setPosition(cc.p(35+offx,0))

	local tenMoney = cc.Sprite:create(costImg10);
	tenMoney:addTo(btnRightNode);
	tenMoney:setPosition(cc.p(-70-offx,0))
	tenMoney:setScale(0.4);

	local btnTen_label = cc.Label:createWithTTF(costStr10, fonts.FONT_IMPACT, 14);
	btnTen_label:setPosition(-20-offx, 0);
	btnTen_label:addTo(btnRightNode);

	if proxyId == nil and costTen and costTen[2] then --只使用钻石
		btnTen_label:setString( costTen[2]  )
		btnRight:setTag(bagId*10+3);
	end

	local diamondTrd = 0;
	if costTen and costTen[2] and proxyId then --3个按钮
		-- 十抽次数按钮
		local btnTrdNode = cc.Node:create();
		btnTrdNode:addTo(node);
		btnTrdNode:setPosition(cc.p( btnWidth+10, 0))
		btnTrdNode:setCascadeOpacityEnabled(true);

		local btnTrdImg = cc.Scale9Sprite:create(imgPath..'gacha_btn2.png');
		-- btnTrdImg:setContentSize( cc.size(btnWidth, 70) )
		btnTrdImg:addTo(btnTrdNode);
		btnTrdImg:setPosition(0, -2)

		local btnTrdItembg = cc.Sprite:create(imgPath..'gacha_itemBg.png');
		btnTrdItembg:addTo(btnTrdNode);
		btnTrdItembg:setPosition(-30,0)

		local btnTrd = ccui.Layout:create();
		btnTrd:addClickEventListener(handler(self,self.selectNewGachaCount));
		btnTrd:setTouchEnabled(true);
		btnTrd:setContentSize(cc.size(btnWidth , 54));
		btnTrd:setAnchorPoint(cc.p(0.5,0.5));
		btnTrd:setTag(bagId*10+3);
		btnTrd:addTo(btnTrdNode);

		local trdTimes = cc.Label:createWithTTF(LANG.UI_GACHA_23 , fonts.FONT_SIYUAN , 16)
			:enableOutline(cc.c3b(60 , 65 , 60) , 2)
			:addTo(btnTrdNode)
			:setPosition(cc.p(50,0))

		local trdMoney = cc.Sprite:create("image/icon/I10002.png");
		trdMoney:addTo(btnTrdNode);
		trdMoney:setPosition(cc.p(-70,0))
		trdMoney:setScale(0.4);

		local btnTrd_label = cc.Label:createWithTTF(costTen[2], fonts.FONT_SIYUAN, 14);
		btnTrd_label:setPosition(-20, 0);
		btnTrd_label:addTo(btnTrdNode);

		diamondTrd = costTen[2];

		btnRightNode:setPositionX(0)
		btnLeftNode:setPositionX(- btnWidth-10);

		singleTimes:setPosition(50, 0)
		tenTimes:setPosition(50, 0)

	else
		btnRightNode:setPositionX(btnWidth/2 - 10)
		btnLeftNode:setPositionX(- btnWidth/2 +10);

	end

	self.bagCellList[bagId] = {	singleMoney=singleMoney,
								btnSingle_label=btnSingle_label,
								tenMoney=tenMoney,
								btnTen_label=btnTen_label,
								diamondFst=diamondFst,
								diamondTrd=diamondTrd
							}
	
end

function GachaListPanel:showProbabilityPanelByTag( sender,event )
	if self.isShowAction then
		return ;
	end
	local tag = sender:getTag();
	UIManager:push('GachaProbabilityPanel' , tag);
end

function GachaListPanel:initResultPanel( ... )
	local ResultPanel = require('app/layer/MainScene/GachaResultPanel')
	self.resultNode = cc.Node:create();
	self.resultNode:addTo(self.root);
	self.resultLayer =  ResultPanel:create()
	self.resultLayer:Init(self.resultNode);
	self.resultNode:setLocalZOrder(10);
end

function GachaListPanel:updateGachaTimes(  )
	local leftTimes, rewardTimes, progress = model.GachaModel:getRewardTimes()

	local pro = leftTimes/rewardTimes;
	self.barSpr:setScaleX( math.min(pro, 1) )
	self.barBtn:setEnabled( progress >= 100 )
	if progress >= 100 then
		self.rwdLabel:setTextColor(cc.c3b(255,255,255))
		self.rwdLabel:enableOutline(cc.c3b(190 , 60 , 15) , 2)
	else
		self.rwdLabel:setTextColor(cc.c3b(60,60,60))
		self.rwdLabel:enableOutline(cc.c3b(255 , 255 , 255) , 2)
	end

	self.barLabel:setString(tostring(leftTimes)..'/'..tostring(rewardTimes))

	for k,v in pairs(self.limitRwdCell) do
		self:updateRwdBarInfo(k);
	end
	
end

function GachaListPanel:showAchievementResult( ... )
	self.isShowAction = false;

	model.CardModel:showNewCardIcon();
	UIManager:push('HeaderPanel','GachaListPanel',UIManager:getPanelHandler('GachaListPanel'));
end
function GachaListPanel:showGachaResulCallback(  )
	self:showAchievementResult()
end

function GachaListPanel:showGachaResult( gachaResults )
	if self.root:isVisible() then 
		local option = {
			gachaResults = gachaResults,
			callback = handler(self,self.showGachaResulCallback),
			fromName = 'GachaListPanel'
		}
		self.resultLayer:onEnter(option, self.bgPath);
	end	
end

function GachaListPanel:onEnter(enterIndex)
	self.currentGachaIndex = enterIndex or 1;
	self.isShowAction = false;
	model.GachaModel:checkBagListAndReq()
	self:onShowNew();

	self:updateGachaTimes();
	local len = model.GachaModel:getContractLength();
	local gachaList = model.GachaModel:getGachaList();
	for k,v in pairs(gachaList) do
		self:updateCellBtns(v, k)
	end
	self:updateTicks();

	EventManager:FireEvent(EventType.UIFooterUpdateSelect,6);
	self:update();
	TimeManager:registUpdateEvent('GachaListPanel',handler(self,self.update),true);
end

function GachaListPanel:updateTicks()
	for k,v in pairs(self.tickList) do
		if v then
			local num = model.ItemModel:getItemNum(k);
			for i,numLabel in ipairs(v) do
				numLabel:setString(''..num)
			end
		end
	end
end

function GachaListPanel:onShowNew()
	self.root:setOpacity(50);
	self.root:runAction(cc.FadeIn:create(0.2));
	self.root:setVisible(true);

	self.resultNode:setVisible(false)

end

function GachaListPanel:clearData(  )
	self.isShowAction = false;
	self.root:stopAllActions();
end
function GachaListPanel:onClickBack()
	-- if self.isShowAction then
	-- 	return ;
	-- end
	self:clearData();

	local actExit =  cc.CallFunc:create(function()
		UIManager:pop('GachaListPanel');
	end)
	self.root:setOpacity(255);
	self.root:runAction(cc.Sequence:create(cc.FadeTo:create(0.2 , 50) ,actExit));
end

function GachaListPanel:gachaTimesRewardClick(  )
	if self.isShowAction then
		return ;
	end
	local leftTimes, rewardTimes, progress = model.GachaModel:getRewardTimes()

	if progress < 100 then
		Prompt:Show(LANG.UI_GACHA_18);
		return 
	end
	-- 不在判断月卡状态
	model.GachaModel:reqGachaTimesReward(1)

	-- local monthCardData = model.RechargeModel:getMonthData()
	-- if monthCardData and monthCardData[4] and (monthCardData[4].state == 1 or monthCardData[4].state == 2) then
	-- 	model.GachaModel:reqGachaTimesReward()
	-- else
	-- 	-- Prompt:Show('激活月卡功能后才能领取奖励！')

	-- 	MessageBox:create(LANG.UI_GACHA_20 , MessageBoxType.CANCEL_OK,function()
	-- 		self:clearData();
	-- 		UIManager:push('ShopPanel',4)
	-- 	end);
	-- end
end

function GachaListPanel:retGachaTimesReward( msg )
	self:updateGachaTimes()
	self:updateTicks()
	UIManager:push('RewardPanel',msg);
end

function GachaListPanel:selectNewGachaCount( sender,event )
	local tag = sender:getTag();
	local tmpTag = tag % 10;
	local bagId = math.floor(tag/10)
	tag = tmpTag

	local count = 1;
	local flag = 1;
	if tag == 2 then
		count = 10;
		flag = 2;
	elseif tag == 3 then
		count = 11;
		flag = 3;
	else
	end

	AudioManager:playEffect(Conf.buttonClick);
	if self.isShowAction then
		return ;
	end
	if bagId == nil then
		return ;
	end

	local diamondCost = 0;
	if flag==1 and self.bagCellList[bagId].diamondFst~=0 then
		diamondCost = self.bagCellList[bagId].diamondFst;
	elseif flag==3 and self.bagCellList[bagId].diamondTrd~=0 then
		diamondCost = self.bagCellList[bagId].diamondTrd;
	end

	if diamondCost ~= 0 then
		MessageBox:create(string.format(LANG.UI_NOTICE_2,tostring(diamondCost)) , MessageBoxType.CANCEL_OK,function()
			self.currentGachaIndex = bagId;
			model.GachaModel:reqGacha(bagId,flag);
		end);
	else
		self.currentGachaIndex = bagId;
		model.GachaModel:reqGacha(bagId,flag);
	end
	
end

function GachaListPanel:retGachaResult(msg)
	-- dump(msg,"gachaResult ======= ")
	local gachaBagData = model.GachaModel:getContractData(self.currentGachaIndex);
	-- self:updateCellBtns(gachaBagData, self.currentGachaIndex);
	local gachaList = model.GachaModel:getGachaList();
	for k,v in pairs(gachaList) do
		self:updateCellBtns(v, k)
	end

	self:updateGachaTimes();

	self.isShowAction = true ;
	UIManager:pop('HeaderPanel');

	local action = cc.CallFunc:create(function (  )

		self:showGachaResult(msg.gachaResult)
	end)

	self.root:runAction(cc.Sequence:create(cc.DelayTime:create(0.1),action))
end

function GachaListPanel:onLeave()
	tools.RedDotTools:pushMainPanelRedDot(8);
	TimeManager:registUpdateEvent('GachaListPanel',nil);

	self:clearData()
	self.root:setVisible(false);
end

-- 引导相关
function GachaListPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	if guideNo == 2 or guideNo == 32 then

		local btn = self.bagCellList[1].btnSingle_label
		local bx, by = btn:getPosition();
		pos = cc.p(bx+20, by);

		pos = btn:getParent():convertToWorldSpace(pos);
	elseif guideNo == 3 or guideNo == 4 or guideNo == 33 or guideNo == 34 then
		pos = self.resultLayer:getGuidePosition();
	end
	return pos;
end

function GachaListPanel:guideCallback( guideNo )
	if guideNo == 3 or guideNo == 4 then
	self.resultLayer:backToInfo(guideNo == 4);
	end
end

return GachaListPanel;
