local GachaProbabilityPanel = class("GachaProbabilityPanel", require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

function GachaProbabilityPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'GachaProbabilityPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function GachaProbabilityPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels();
end

function GachaProbabilityPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(526,498));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , -22);

	local titleNode =  GlobalFunctions:createTitle(LANG.UI_L_GACHA_PROB_TITLE_1 , 526 , 46 , LANG.UI_L_GACHA_PROB_TITLE_2);
	titleNode:addTo(self.root);
	titleNode:setPosition(0 , 249);

	local infoBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	infoBg:addTo(self.root);
	infoBg:setPosition(0 , 84);
    infoBg:setCapInsets(cc.rect(11,11,34,28));
	infoBg:setContentSize(cc.size(502 , 230));

	local infoBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	infoBg:addTo(self.root);
	infoBg:setPosition(0 , -149);
    infoBg:setCapInsets(cc.rect(11,11,34,28));
	infoBg:setContentSize(cc.size(502 , 216));

	self.probabilityView = ccui.ScrollView:create();
	self.probabilityView:setTouchEnabled(true);
	self.probabilityView:setBounceEnabled(true);
	self.probabilityView:setDirection(ccui.ScrollViewDir.vertical);
	self.probabilityView:setContentSize(cc.size(482, 220));
	self.probabilityView:setAnchorPoint(cc.p(0.5, 0.5));
	self.probabilityView:setPosition(0 , 84);
	self.probabilityView:setScrollBarEnabled(false);
	self.probabilityView:addTo(self.root);

	local initData = {
        width = 502,
        height = 206,
        cellWidth = 502 ,
        cellHeight = 90,
		cellReloadFunc = handler(self,self.reloadCellPanel),
	}
	
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData) ;
    self.cardListView:getView():setPosition(cc.p(-251,-252)); -- 0,-149
	self.cardListView:getView():addTo(self.root)
end


function GachaProbabilityPanel:onEnter(index)
	self:addSwallowCallback(handler(self, self.onClickBack));
	local currentBagData = self:getGachaBagData(index);
	if not currentBagData then
		currentBagData = self:getGachaBagData(1);
	end

	local gachaNo = currentBagData.no;
	
	local rateList = {};
	local qualitys = {};
	-- for _k , tableNO in pairs(currentBagData.gacha_table) do
	-- 	local gachaTableData = model.GachaModel:getContractTableData(tableNO);
	-- 	if gachaTableData then
	-- 		for _k , contract in pairs(gachaTableData) do
	-- 			if contract.rate > 0 then
	-- 				local roleTableData = ResTable:getRowData('role' , contract.reward_no);
	-- 				if roleTableData then
	-- 					if not rateList[roleTableData.quality] then
	-- 						rateList[roleTableData.quality] = {};
	-- 						table.insert(qualitys,roleTableData.quality);
	-- 					end
	-- 					table.insert(rateList[roleTableData.quality] , contract);
	-- 				end
	-- 			end
	-- 		end
	-- 	end
	-- end

	local gachaTableData = model.GachaModel:getContractTableData(currentBagData.tab, currentBagData.random_type);
		if gachaTableData then
			for _k , contract in pairs(gachaTableData) do
				if contract.rate > 0 then
					local roleTableData = ResTable:getRowData('role' , contract.reward_no);
					if roleTableData then
						if not rateList[roleTableData.quality] then
							rateList[roleTableData.quality] = {};
							table.insert(qualitys,roleTableData.quality);
						end
						table.insert(rateList[roleTableData.quality] , contract);
					end
				end
			end
		end

	local qualityProbability = {};
	self.roleList = {};
	for quality , contract in pairs(rateList) do
		if not qualityProbability[quality] then
			qualityProbability[quality] = 0;
		end
		for _k , data in pairs(contract) do
			qualityProbability[quality] = qualityProbability[quality] + data.rate;
			table.insert(self.roleList , {no = data.reward_no , rate = data.rate});
		end
	end

	self.roleMaxNum = #self.roleList;
	local groupNum = math.ceil(self.roleMaxNum / 5);
	self.cardListView:setDataCount(groupNum);
	self.cardListView:reloadData();

	local num = #qualitys;
	if num > 1 then
		table.sort(qualitys , function(first , second)
			return first > second;
		end)
	end

	local maxHigth = 76 * num;
	maxHigth = maxHigth < 220 and 220 or maxHigth;

	self.probabilityView:removeAllChildren();
	self.probabilityView:setInnerContainerSize(cc.size(482 , maxHigth));

	for _k , quality in ipairs(qualitys) do
		local node = cc.Node:create();
		node:addTo(self.probabilityView);
		node:setPosition(241 , maxHigth-33-76*(_k-1));

		local bg = cc.Sprite:create('image/chouka/probability/probility_1.png');
		bg:addTo(node);

		local qualityImg = cc.Sprite:create('image/card/card_quality_'..quality..'.png');
		qualityImg:addTo(node);
		qualityImg:setAnchorPoint(0 , 0.5);
		qualityImg:setPosition(-221,0);

		for i = 1 , quality do
			local star = cc.Sprite:create('image/chouka/probability/probility_2.png');
			star:addTo(node);
			star:setPosition(-138 + 18*(i-1),0);
		end

		local rate = cc.Label:createWithTTF(tostring(qualityProbability[quality]/100)..'%' , fonts.FONT_SIYUAN,18);
		rate:addTo(node);
		rate:setAnchorPoint(1,0.5);
		rate:setPosition(224,0);
	end

	self.root:setVisible(true);
end

function GachaProbabilityPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 5 + 1;
	local curMaxNum = (idx+1) * 5;
	curMaxNum = math.min(curMaxNum , self.roleMaxNum);

	local children = cell:getChildren();

	if not children or #children <= 0 then
		for i = 1 , 5 do
			local x = 61 + 95 * (i - 1);
			local y = 45;
			local iconTemplate = template.ItemTemplate:create(2);
			iconTemplate:setPosition(cc.p(x , y));
			iconTemplate:addTo(cell);
			iconTemplate:setScale(0.8);
			if startNum <= curMaxNum then
				iconTemplate:Init(ROLE_TYPE,self.roleList[startNum].no);
				iconTemplate:setGachaProbability(tostring(self.roleList[startNum].rate/100)..'%');
				startNum = startNum + 1;
			else
				iconTemplate:setVisible(false);
			end
		end
	else
		for i = 1 , 5 do
			if startNum <= curMaxNum then
				children[i]:setVisible(true);
				children[i]:Init(ROLE_TYPE,self.roleList[startNum].no);
				children[i]:setGachaProbability(tostring(self.roleList[startNum].rate/100)..'%');

				startNum = startNum + 1;
			else
				children[i]:setVisible(false);
			end
		end
	end

	return cell;
end

function GachaProbabilityPanel:getGachaBagData( gachaType )
	
	local gachaList = model.GachaModel:getGachaList();
	if not gachaList or not gachaList[gachaType] then
		return nil;
	end

	local gachaBagData = model.GachaModel:getContractData(gachaType);
	return gachaBagData
end

function GachaProbabilityPanel:onClickBack()
	UIManager:pop('GachaProbabilityPanel');
end


function GachaProbabilityPanel:onLeave()
	self.root:setVisible(false);
end

return GachaProbabilityPanel;
