local GemInfoPanel = class("GemInfoPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

function GemInfoPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'GemInfoPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.isPopup = false;
end


function GemInfoPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()

end

function GemInfoPanel:initPanels()

	local bgImg = cc.Sprite:create('image/bg/Bg_14.png')
	bgImg:addTo(self.root)

	self.dataList = nil

	self:initTopNode();

	self:initInfoNode();

	self:initBottomNode()
end

function GemInfoPanel:initBottomNode()
	local btmNode = cc.Node:create()
	btmNode:addTo(self.root)
	btmNode:setPosition(0, -StageSize.height/2)

	local bg = cc.Sprite:create('image/footer/footer_bg_1.png')
		:setLocalZOrder(0)
		:setAnchorPoint(cc.p(0.5, 0))
		:setPosition(0, 0)
		:addTo(btmNode)

	local light = cc.Sprite:create('image/footer/footer_bg_4.png');
	light:addTo(btmNode);
	light:setPosition(0, bg:getContentSize().height)

	for i = 1 , 2 do
		local line = cc.Sprite:create('image/footer/footer_bg_3.png')
		line:addTo(btmNode);
		line:setPosition(-106 + (i-1)*214 , bg:getContentSize().height/2);
	end

	self.btnList = {};
	local btnPos = {
		cc.p(-213, 48),
		cc.p(0, 48),
		cc.p(213, 48),
	}
	local btnName = {
		'符文',
		'背包',
		'合成'
	}
	for i=1, 3 do
		local btn = {};
		self.btnList[i] = btn;

		btn.root = cc.Node:create()
			:setPosition(btnPos[i])
			:addTo(btmNode)
		btn.root:setCascadeOpacityEnabled(true);

		btn.select = cc.Scale9Sprite:create('image/footer/footer_bg_2.png')
		btn.select:addTo(btn.root);
		btn.select:setContentSize(cc.size( 210, 93 ) )
		btn.select:setVisible(i == 1);
		btn.select:setPosition(0 , -1);

		btn.btn = ccui.Button:create('image/footer/footer_gem_' .. i .. '.png')
		btn.btn:setZoomScale(0)
		btn.btn:setTag(i)
		btn.btn:setEnabled(i ~= 1)
		btn.btn:addClickEventListener(handler(self, self.clickFooter))
		btn.btn:addTo(btn.root)

		btn.label = cc.Label:createWithTTF(btnName[i] , fonts.FONT_SIYUAN , 20);
		btn.label:addTo(btn.root);
		btn.label:setPosition(0 , -29);
		btn.label:enableOutline(i== 1 and cc.c3b(30 , 80 , 140) or cc.c3b(0 , 0 , 0) , 2);

	end

	self.recastBtn = ccui.Button:create('image/commonImg/btn_1_169_55.png');
	self.recastBtn:addTo(self.root);
	self.recastBtn:setPosition(0 , -StageSize.height/2+ 200);
	self.recastBtn:setContentSize(cc.size(182 , 55))
	self.recastBtn:setScale9Enabled(true)
	self.recastBtn:addClickEventListener(handler(self , self.recastBtnClick));
	local label = cc.Label:createWithTTF('重铸', fonts.FONT_SIYUAN, 20)
    :enableOutline(cc.c3b(30, 60, 110), 2)
	self.recastBtn:setTitleLabel(label)
end

function GemInfoPanel:clickFooter(sender, eventType)
	local tag = sender:getTag()
	AudioManager:playEffect(Conf.buttonClick)

	UIManager:clearPopPanel();
	if tag == 2 then
		UIManager:push('GemListPanel');
	elseif tag == 3 then
		UIManager:push('GemComposePanel');
	end

end

function GemInfoPanel:initTopNode()
	local topNode = cc.Node:create();
	topNode:addTo(self.root)
	topNode:setPosition(0, StageSize.height/2 - 100)

	local bg = cc.Sprite:create('image/commonImg/icon_enemy_empty.png')
	bg:addTo(topNode)

	local l1 = cc.Sprite:create('image/fight/result/result_win_bg_1.png')
	l1:setAnchorPoint(1,0.5)
	l1:setPosition(-80, 0)
	l1:addTo(topNode)

	local l2 = cc.Sprite:create('image/fight/result/result_win_bg_1.png')
	l2:setAnchorPoint(1,0.5)
	l2:setScaleX(-1)
	l2:setPosition(80, 0)
	l2:addTo(topNode)

	local nameBg = cc.Scale9Sprite:create('image/commonImg/com_bg.png')
	nameBg:setContentSize(cc.size(120, 42))
	nameBg:setPosition(0, -80)
	nameBg:addTo(topNode)

	self.nameLabel = cc.Label:createWithTTF('符文名称' , fonts.FONT_SIYUAN , 20);
	self.nameLabel:addTo(topNode);
	self.nameLabel:setPosition(0 , -80);
	self.nameLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);

	self.gemIcon = cc.Sprite:create('image/icon/E83028.png')
	self.gemIcon:addTo(topNode)

end

function GemInfoPanel:initInfoNode()
	local infoNode = cc.Node:create()
	infoNode:addTo(self.root)

	local bg = cc.Scale9Sprite:create('image/commonImg/common_tips_bg.png')
	bg:setContentSize(cc.size(606, 520))
	bg:addTo(infoNode)

	local baseTNode = cc.Node:create()
	baseTNode:addTo(infoNode)
	baseTNode:setPosition(0, 300)
	local baseBg = cc.Scale9Sprite:create('image/commonImg/bg_3_200_42.png')
	baseBg:setContentSize(500, 42)
	baseBg:addTo(baseTNode)
	local baseArr = cc.Sprite:create('image/commonImg/arrow_5.png')
	baseArr:setPosition(-230, 0)
	baseArr:addTo(baseTNode)
	local baseLabel = cc.Label:createWithTTF('基础属性' , fonts.FONT_SIYUAN , 20);
	baseLabel:addTo(baseTNode);
	baseLabel:setAnchorPoint(0,0.5)
	baseLabel:setPosition(-200 , 0);
	baseLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);

	local atkNode = cc.Node:create()
	atkNode:addTo(infoNode)
	atkNode:setPosition(-150, 200)
	local atkBg1 = cc.Sprite:create('image/card/card_att_bg_2.png')
	atkBg1:addTo(atkNode)
	local atkBg2 = cc.Sprite:create('image/card/card_att_img_3.png')
	atkBg2:addTo(atkNode)
	atkBg2:setPosition(-110, 0)
	local atkLabel = cc.Label:createWithTTF('攻击' , fonts.FONT_SIYUAN , 20);
	atkLabel:addTo(atkNode);
	atkLabel:setAnchorPoint(0, 0.5)
	atkLabel:setPosition(-140 , 0);
	atkLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);
	self.curAtkLabel = cc.Label:createWithTTF('+99999999' , fonts.FONT_SIYUAN , 20);
	self.curAtkLabel:addTo(atkNode);
	self.curAtkLabel:setPosition(40 , 0);
	self.curAtkLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);


	local defNode = cc.Node:create()
	defNode:addTo(infoNode)
	defNode:setPosition(150, 200)
	local defBg1 = cc.Sprite:create('image/card/card_att_bg_2.png')
	defBg1:addTo(defNode)
	local defBg2 = cc.Sprite:create('image/card/card_att_img_2.png')
	defBg2:addTo(defNode)
	defBg2:setPosition(-110, 0)
	local defLabel = cc.Label:createWithTTF('防御' , fonts.FONT_SIYUAN , 20);
	defLabel:addTo(defNode);
	defLabel:setAnchorPoint(0, 0.5)
	defLabel:setPosition(-140 , 0);
	defLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);
	self.curDefLabel = cc.Label:createWithTTF('+99999999' , fonts.FONT_SIYUAN , 20);
	self.curDefLabel:addTo(defNode);
	self.curDefLabel:setPosition(40 , 0);
	self.curDefLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);


	local skillTNode = cc.Node:create()
	skillTNode:addTo(infoNode)
	skillTNode:setPosition(0, 50)
	local skillTBg = cc.Scale9Sprite:create('image/commonImg/bg_3_200_42.png')
	skillTBg:setContentSize(500, 42)
	skillTBg:addTo(skillTNode)
	local skillTArr = cc.Sprite:create('image/commonImg/arrow_5.png')
	skillTArr:setPosition(-230, 0)
	skillTArr:addTo(skillTNode)
	local skillTLabel = cc.Label:createWithTTF('符文技能' , fonts.FONT_SIYUAN , 20);
	skillTLabel:addTo(skillTNode);
	skillTLabel:setAnchorPoint(0,0.5)
	skillTLabel:setPosition(-200 , 0);
	skillTLabel:enableOutline(cc.c3b(30 , 80 , 140) , 2);
	local skillBtn = ccui.Button:create('image/commonImg/btn_2_103_36.png');
	skillBtn:addTo(skillTNode);
	skillBtn:setPosition(0 , 0);
	skillBtn:setContentSize(cc.size(100 , 40))
	skillBtn:setScale9Enabled(true)
	skillBtn:addClickEventListener(handler(self , self.skillBtnClick));
	local skBtnlabel = cc.Label:createWithTTF('技能预览', fonts.FONT_SIYUAN, 20)
    skBtnlabel:enableOutline(cc.c3b(30, 60, 110), 2)
    skBtnlabel:setPosition(30, 0)
	skillBtn:setTitleLabel(skBtnlabel)


	local skill1Node = cc.Node:create()
	skill1Node:addTo(infoNode)
	skill1Node:setPosition(-200, -50)
	local skill1Bg = cc.Sprite:create('image/commonImg/icon_item_add.png')
		:addTo(skill1Node)
	self.skill1Icon = cc.Sprite:create('image/skillicon/S1010.png')
		:addTo(skill1Node)
	self.skill1Name = cc.Label:createWithTTF('技能1名称', fonts.FONT_SIYUAN, 16)
		:addTo(skill1Node)
		:setPosition(65 , 36)
		:setAnchorPoint(0 , 0.5)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
	self.skill1Info = cc.Label:createWithTTF('肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻', fonts.FONT_SIYUAN, 16)
		:addTo(skill1Node)
		:setPosition(65 , 17)
		:setAnchorPoint(0 , 1)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setDimensions(300, 120)


	local skill2Node = cc.Node:create()
	skill2Node:addTo(infoNode)
	skill2Node:setPosition(-200, -200)
	local skill2Bg = cc.Sprite:create('image/commonImg/icon_item_add.png')
		:addTo(skill2Node)
	self.skill2Icon = cc.Sprite:create('image/skillicon/S1010.png')
		:addTo(skill2Node)
	self.skill2Name = cc.Label:createWithTTF('技能2名称', fonts.FONT_SIYUAN, 16)
		:addTo(skill2Node)
		:setPosition(65 , 36)
		:setAnchorPoint(0 , 0.5)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
	self.skill2Info = cc.Label:createWithTTF('肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻肥而不腻', fonts.FONT_SIYUAN, 16)
		:addTo(skill2Node)
		:setPosition(65 , 17)
		:setAnchorPoint(0 , 1)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setDimensions(300, 120)


end

function GemInfoPanel:onEnter(gemId)
	self.gemId = gemId;
	self.pid = -1

	self:updateInfo();

	self.root:setVisible(true);
end

function GemInfoPanel:recastBtnClick(sender , event)
	AudioManager:playEffect(Conf.buttonClick);

	UIManager:push('GemRecastPanel');
end

function GemInfoPanel:skillBtnClick(sender , event)
	AudioManager:playEffect(Conf.buttonClick);

	UIManager:push('GemSkillPanel');
end

function GemInfoPanel:updateInfo()
	

end

function GemInfoPanel:onClickBack()
	UIManager:pop('GemInfoPanel');
end

function GemInfoPanel:onLeave()
	self.root:setVisible(false);
end

return GemInfoPanel;
