local GuidePanel = class("GuidePanel" , require('app.layer.BaseLayer'));
local fonts = Enum:getTable("SystemFont")
local scheduler = cc.Director:getInstance():getScheduler();

function GuidePanel:ctor(scene, sceneName , name)
	self.super.ctor(self, scene, sceneName, 'GuidePanel');

	self.scene = scene;
	self.useCCreator = false;
	self.isPopup = true 

	self.isWait = false;
	self.guideType = 1; 
	self.showPanel = nil ;
	-- self.afterPanel = nil;
	self.touchTarget = false;

	self.touchRadius = 50; -- 触摸半径
	self.delayShowNum = -1; -- 延时引导帧数
end

function GuidePanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()

	self:registEvent();

end

function GuidePanel:registEvent()
end

function GuidePanel:initPanels()
	self.mainNode = cc.Node:create()
		:addTo(self.root)

	self.infoNode = cc.Node:create()
		:addTo(self.mainNode);

	local infoBg = cc.Sprite:create('image/guide/banzi.png')
		:addTo(self.infoNode)

	self.infoLabel = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 18)
	self.infoLabel:addTo(self.infoNode)
		:setAnchorPoint(cc.p(0.5 ,0.5))
		:setPosition(cc.p(0,0))
		:enableOutline(cc.c4b(50 , 50 , 50 , 255) , 2)
		:setMaxLineWidth(350)
		:setContentSize(cc.size(360 , 100))
		:setLineBreakWithoutSpace(true)	
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setVerticalAlignment(cc.VERTICAL_TEXT_ALIGNMENT_CENTER)
	
	self.hand = cc.Sprite:create('image/guide/shou.png')
		:addTo(self.mainNode)

end

function GuidePanel:addSwallowLayer()
	-- layer
	-- print('==============GuidePanel:addSwallowLayer============== swallCount == ',self.swallowCount)
	if self.swallowCount > 0 then
		return;
	end
	self.swallowCount = self.swallowCount + 1;

	self.swallowLayer = cc.LayerColor:create(cc.c4b(0, 0, 0, 0));
	self.swallowLayer:setContentSize( cc.size(StageSize.width, StageSize.height) );
	self.swallowLayer:setPosition(-StageSize.width/2, -StageSize.height/2);
	self.swallowLayer:setLocalZOrder(-100);
	self.swallowLayer:addTo(self.mainNode);

	-- listener
	self.swallowListener = cc.EventListenerTouchOneByOne:create();
	local onTouchBegan = function(touch, event)
		if self.mainNode:isVisible() == false then
			return true;
		end
		self.touchTarget = false;
		if self.isWait then
			return true ;
		end
		if self.guideType == 202 then
			-- 全屏触摸
			return true 
		end

		local target = event:getCurrentTarget()
		local touchPos = self.mainNode:convertToNodeSpace(touch:getLocation());
		local dis = cc.pGetDistance(self.particlePos,touchPos) ;
		-- print('touch begin 111 pos === ',touchPos.x,'....y == ',touchPos.y,'....dis == ',dis)
		if dis < self.touchRadius then
			self.touchTarget = true;
		end
        return true
	end
	local onTouchEnd = function(touch ,event )
		if self.mainNode:isVisible() == false then
			return true;
		end
		if self.isWait then
			return true ;
		end
		if self.touchTarget == true then
			self.touchTarget = false;
			self:showNext()
			return;
		end
		if self.guideType > 200 and self.guideType < 300 then
			self:showNextWithOutTouchPass()
		end

	end
	self.swallowListener:setSwallowTouches(true);
	self.swallowListener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN);
	self.swallowListener:registerScriptHandler(onTouchEnd, cc.Handler.EVENT_TOUCH_ENDED);
	local eventDispatcher = self.swallowLayer:getEventDispatcher();
	eventDispatcher:addEventListenerWithSceneGraphPriority(self.swallowListener, self.swallowLayer);
end

function GuidePanel:removeSwallowLayer()
	if self.swallowCount <= 0 then
		return;
	end
	self.swallowCount = self.swallowCount - 1;
	self.swallowLayer:getEventDispatcher():removeEventListener(self.swallowListener);
	self.swallowListener = nil;
	self.swallowCallback = nil;

	self.swallowLayer:removeFromParent();
end


function GuidePanel:showParticle( beginPos,bshow )

	
	if self.guideSpine == nil then
		local spine_res = 'guidering_spine'
		if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
			self.guideSpine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
			self.guideSpine:setAnimation(0, 'guidering_spine', true)
			self.guideSpine:addTo(self.mainNode);
		else
			print('error : 缺少spine 资源')
		end  
	end

	if self.guideSpine ~= nil then
		self.guideSpine:setPosition(beginPos)
		self.guideSpine:setVisible(bshow);
	end

	-- print('========self.touchRadius===== ',self.touchRadius);
	-- if self.testDrawNode  then
	-- 	self.testDrawNode:removeFromParent();
	-- 	self.testDrawNode = nil;
	-- end
	-- self.testDrawNode = cc.DrawNode:create();
	-- self.testDrawNode:addTo(self.root)
	-- self.testDrawNode:drawCircle(cc.p(0,0),self.touchRadius,0,20,false,cc.c4f(1,0,1,1))
	-- self.testDrawNode:setPosition(beginPos);
	-- self.testDrawNode:setLocalZOrder(10)


end

function GuidePanel:showHandPosition( particlePos,bshow )
	local handPos = cc.p(0,0)
	local handPosMoveBy = cc.p(0,0);
	local rotation = 0;
	local infoPos = cc.p(0,0)
	if particlePos == nil or next(particlePos) == nil then
		print('error ==================');
		return 
	end
	if particlePos.x < 0 then
		handPos.x = particlePos.x + 50 
		handPosMoveBy.x =  10;

		if particlePos.y > -280 then
			handPos.y = particlePos.y - 45
			handPosMoveBy.y =  -10

			infoPos.x = handPos.x + 200
			infoPos.y = handPos.y - 80
		else
			handPos.y = particlePos.y + 55 
			handPosMoveBy.y = 10
			
			rotation =  -90

			infoPos.x = handPos.x + 190
			infoPos.y = handPos.y + 80
		end

	else

		handPos.x = particlePos.x - 50 
		handPosMoveBy.x =  - 10;

		if particlePos.y > -280 then
			handPos.y = particlePos.y - 45
			handPosMoveBy.y =  - 10
			rotation =  90

			infoPos.x = handPos.x - 200
			infoPos.y = handPos.y - 80
		else
			handPos.y = particlePos.y + 55 
			handPosMoveBy.y = 10
			rotation =  180

			infoPos.x = handPos.x - 190
			infoPos.y = handPos.y + 80
		end
	end

	self.hand:setPosition(handPos)
	self.hand:setRotation(rotation)
	self.hand:setVisible(bshow)
	self.hand:stopAllActions()
	self.hand:runAction(cc.RepeatForever:create(cc.Sequence:create(
		cc.MoveBy:create(0.5,handPosMoveBy),
		cc.MoveBy:create(0.5,cc.p(-handPosMoveBy.x,-handPosMoveBy.y))
		)))
	
	-- print('infoPos === ',infoPos.x ,'.......... y = ',infoPos.y)
	if infoPos.x + 184 > StageSize.width/2 then
		infoPos.x = StageSize.width/2 - 190
	elseif infoPos.x - 184 < -StageSize.width/2 then 
		infoPos.x = 190 -StageSize.width/2
	end

	self.infoNode:setPosition(infoPos);


end

function GuidePanel:onEnter(enterFrom)
	self.mainNode:setVisible(false)
	if self.root:isVisible() == false then
		self.root:setVisible(true);

		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.Update), 0, false);
	end

	self.showPanel = enterFrom or 'MainPanel';
	self:delayOnShow()
end

function GuidePanel:updateGuideInfo( enterFrom )
	self.showPanel = enterFrom or 'MainPanel';

	self:delayOnShow()
end

function GuidePanel:onShow(  )
	self.mainNode:setVisible(true)
	local guideData = model.GuideModel:getGuideStepData();

	self.target = UIManager:getLayer(guideData.target) or nil

	self.infoLabel:setString(guideData.description);
	self.guideType = guideData.guideType or 0;
	
	self.callback = guideData.callback or nil ;
	self.callbackParam = guideData.callbackParam or nil ;

	self.isWait = false;
	if guideData.waitTime > 0 then
		self.mainNode:setVisible(false)
		self.isWait = true ;
		print('========wait time ==== ',guideData.waitTime/1000);
		self.mainNode:runAction(cc.Sequence:create(
			cc.DelayTime:create(guideData.waitTime/1000),
			cc.CallFunc:create(function (  )
				self.isWait = false;
				self.mainNode:setVisible(true)
				self:realShowInfo();
				print('=========wait end ======= ');
			end )
		))
	else
		self:realShowInfo();
	end

end

function GuidePanel:realShowInfo()
	-- body
	-- 引导坐标由对应引导页面获取,需要提供getGuidePosition
	EventManager:FireEvent(EventType.ShieldGuide, false);

	local bShow = true ;
	local guideData = model.GuideModel:getGuideStepData();
	-- local worldPos = cc.p(0,0)
	local worldPos = cc.p( StageSize.width/2, StageSize.height/2 )
	if self.target ~= nil and self.target['getGuidePosition'] ~= nil then
		worldPos = self.target['getGuidePosition'](self.target,guideData.id);
	end
	self.particlePos = self.mainNode:convertToNodeSpace(worldPos);

	self:showParticle(self.particlePos,bShow);
	self:showHandPosition(self.particlePos,bShow);

	local g_id = guideData.id
	if g_id == 6 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu1);
	elseif g_id == 7 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu2);
	elseif g_id == 8 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu3);
	elseif g_id == 9 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu4);
	elseif g_id == 10 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu5);
	elseif g_id == 11 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu6);
	elseif g_id == 12 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu10);
	elseif g_id == 15 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu11);
	elseif g_id == 16 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu12);
	elseif g_id == 17 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu13);
	elseif g_id == 18 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu14);
	elseif g_id == 19 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu15_levelup);
	elseif g_id == 20 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu16);
	elseif g_id == 21 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu17);
	elseif g_id == 22 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu18);
	elseif g_id == 23 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu19);
	elseif g_id == 24 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu20);
	elseif g_id == 25 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu21);
	elseif g_id == 26 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu25);
	elseif g_id == 32 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu26_card);
	elseif g_id == 33 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu27);
	elseif g_id == 34 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu28);
	elseif g_id == 35 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu29);
	elseif g_id == 36 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu30);
	elseif g_id == 37 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu31);
	elseif g_id == 38 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu32);
	elseif g_id == 41 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu33);
	elseif g_id == 42 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu34);
	elseif g_id == 43 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu35);
	elseif g_id == 44 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu36);
	elseif g_id == 45 or g_id == 47 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu39);
	elseif g_id == 48 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu40);
	elseif g_id == 49 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu41);
	elseif g_id == 52 then
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu42);
	end


	self.infoNode:setVisible(true)
	if guideData.description == nil or  string.len( guideData.description ) == 0 then
		self.infoNode:setVisible(false)
	end

end

function GuidePanel:showNext()
	self.mainNode:setVisible(false)
	EventManager:FireEvent(EventType.ShieldGuide, true);
	-- print('fubin--GuidePanel-click----1:')
	
	local isSendGuideStep = false --此步引导 点击 是否向后台发送 保存引导数据
	if self.callback ~= nil and self.target ~= nil  then 
		if self.target[self.callback] then
			if self.callbackParam == nil or type(self.callbackParam) ~= 'table' then

				if model.GuideModel:isWillSaveGuide() then
					-- print('fubin--GuidePanel-click----2:')
					model.GuideModel:switchIsNeedSaveStep(true)--开启标志
					isSendGuideStep = true
				end

				self.target[self.callback](self.target,self.callbackParam);

				if isSendGuideStep==false then
					-- print('fubin--GuidePanel-click----3:')
					model.GuideModel:setNextGuideStep()
					-- self:delayOnShow();
				end

				return 
			end
			local _k1,_v1 = next(self.callbackParam,_k1)
			if _k1 == nil then
				print('===error==== ,   callbackParam need be null or not empty')
			end
			if type(_k1) == "number" then
				-- 纯数组
				-- 暂时未处理数组型 多参
				-- local _k2,_v2 = next(self.callbackParam,_k1)
				-- local _k3,_v3 = next(self.callbackParam,_k2)
				-- self.target[self.callback](self.target,sender,_v1);
			else
				-- 需要传节点，根据tag值处理
				local sender = cc.Node:create();
				local senderEventType = ccui.TouchEventType.ended
				if self.callbackParam['tag'] then
					sender:setTag(self.callbackParam['tag'])
				end

				if model.GuideModel:isWillSaveGuide() then
					model.GuideModel:switchIsNeedSaveStep(true)--开启标志
					isSendGuideStep = true
					self.mainNode:setVisible(false)
					-- print('fubin--GuidePanel-click----4:')
				end

				self.target[self.callback](self.target,sender,senderEventType);

			end
		end
	end 

	if isSendGuideStep==false then
		-- print('fubin--GuidePanel-click----5:')
		model.GuideModel:setNextGuideStep()
		-- self:delayOnShow();
	end

end

-- 不传递触摸，同时执行事件
function GuidePanel:showNextWithOutTouchPass( ... )
	-- print('============showNextWithOutTouchPass  ==== ,self.guideType == ',self.guideType);
	if self.guideType == 202 then
		-- 强制跳转到MainPanel
		EventManager:FireEvent(EventType.ShieldGuide, true);
		self.mainNode:setVisible(false)
	   	UIManager:switch('MainPanel');
	   	model.GuideModel:setNextGuideStep()
	end
end

function GuidePanel:Update()
	if self.delayShowNum == -1 then
		return
	end
	if self.delayShowNum >= 2 then
		self:onShow();
		self.delayShowNum = -1;
	else
		self.delayShowNum = self.delayShowNum + 1;
	end
end

function GuidePanel:delayOnShow()
	-- self:onShow();
	self.delayShowNum = 0;
end


function GuidePanel:onClickBack()
	UIManager:pop('GuidePanel');
end

function GuidePanel:onLeave()
	self.hand:stopAllActions()

	if self.m_scheduleHandler then
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil
	end
	self.root:setVisible(false);

end

return GuidePanel;
