local GuildBossPanel = class("GuildBossPanel",require('app.layer.BaseLayer'))

local FightType = Enum:getTable('FightType');
local fonts = Enum:getTable('SystemFont')
local MessageBoxType = Enum:getTable('MessageBoxType')
local TeamType = Enum:getTable('TeamType')
local imagePath = 'image/guild/boss/'
local scheduler = cc.Director:getInstance():getScheduler();

function GuildBossPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'GuildBossPanel');

	self.useCCreator = false;
	self.isPopup = false;
	self.haveSwallowLayer = false;

	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function GuildBossPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:InitPanels();
	self:registEvent();
end

function GuildBossPanel:InitPanels()

	local bossBg = cc.Sprite:create(imagePath..'guild_boss_bg.png')
	:addTo(self.root)

	local spine_res = 'guildboss_spine'
	local spine = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
	spine:addTo(self.root)
	spine:setPosition(0, 0)
	spine:setAnimation(0, "guildboss_spine", true)

	local particle = cc.ParticleSystemQuad:create("Effect/particle/huoxing.plist")
	particle:addTo(self.root);
	particle:setPosition(0, 0)

	self.roleNode = cc.Node:create()
	:addTo(self.root)

	local boss_monster_id = Conf.guildBossMonsterId;
	self.roleArmature = DragonBonesManager:GetArmature(ResTable:getValue('monster', boss_monster_id ,'model'))
	self.roleArmature:addTo(self.roleNode);
	self.roleArmature:setPosition(cc.p(4,25));
	self.roleArmature:getAnimation():play("idle");
	self.roleArmature:getArmature():setFlipX(true);
	self.roleArmature:setTag(boss_monster_id);
	self.roleArmature:setCascadeOpacityEnabled(true);

	local topNode = cc.Node:create()
	:addTo(self.root)
	:setPosition(0, 448)

	self.bossNameNode = cc.Node:create()
		:addTo(topNode)
		:setPosition(0 , -7) -- 0,-7
	self.colorBg = cc.Sprite:create(imagePath..'guild_boss_14.png')
		:addTo(self.bossNameNode)
		:setScale(0.6)
	self.bossColor = cc.Sprite:create()
		:addTo(self.colorBg)
		:setPosition(self.colorBg:getContentSize().width/2 , self.colorBg:getContentSize().height/2)
		:setScale(0.95)

	self.bossName = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
		:addTo(self.bossNameNode)

	self.bossDamageNode = cc.Node:create()
		:addTo(topNode)
		:setPosition(0 , -31)
	local bg = cc.Sprite:create(imagePath..'guild_boss_1.png')
		:addTo(self.bossDamageNode)
	local bg = cc.Sprite:create(imagePath..'guild_boss_2.png')
		:addTo(self.bossDamageNode)
		:setPosition(0 , -9)

	--护甲状态
	self.bossArmorNode = cc.Node:create()
		:addTo(self.bossDamageNode)
		:setPosition(0,-9)
	
	self.bossArmorProgress = cc.ProgressTimer:create(cc.Sprite:create(imagePath..'guild_boss_3.png'));
	self.bossArmorProgress:addTo(self.bossArmorNode);
	self.bossArmorProgress:setType(cc.PROGRESS_TIMER_TYPE_BAR);
	self.bossArmorProgress:setMidpoint(cc.p(0, 0));
	self.bossArmorProgress:setBarChangeRate(cc.p(1, 0));
	self.bossArmorProgress:setPercentage(60);

	local bg = cc.Sprite:create(imagePath..'guild_boss_4.png')
		:addTo(self.bossArmorNode)
		:setPosition(0 , -14)
	local armorLabel = cc.Label:createWithTTF(LANG.UI_L_ARMOR, fonts.FONT_SIYUAN, 14)
		:setPosition(cc.p(-27,-14))
		:addTo(self.bossArmorNode)
		:enableOutline(cc.c3b(0 , 0 , 0),2)
	self.bossArmorHp = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 14)
		:setPosition(cc.p(33,-14))
		:addTo(self.bossArmorNode)
		:enableOutline(cc.c3b(0 , 0 , 0),2)
	
	local tipBg = cc.Sprite:create(imagePath..'guild_boss_7.png')
		:addTo(self.bossArmorNode)
		:setPosition(0 , -54)
	local tipImg = cc.Sprite:create('image/commonImg/tip_2_19_19.png')
		:addTo(self.bossArmorNode)
		:setPosition(0 , -54)
	local tipLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_13, fonts.FONT_SIYUAN, 15)
		:setPosition(cc.p(0,-54))
		:addTo(self.bossArmorNode)
		:setTextColor(cc.c3b(255,255,0))
	
	local width = tipImg:getContentSize().width + 10;
	local tipWidth = tipLabel:getContentSize().width+10;
	local posX = -tipWidth / 2

	tipImg:setPositionX(posX);
	tipLabel:setPositionX(posX + width / 2 + tipWidth / 2);

	--无护甲状态
	self.bossArmorTimeNode = cc.Node:create()
		:addTo(self.bossDamageNode)
		:setPosition(0 , -10)
	local bg = cc.Sprite:create(imagePath..'guild_boss_5.png')
		:addTo(self.bossArmorTimeNode)

	self.bossArmorTime = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 14)
		:setPosition(cc.p(-40,0))
		:addTo(self.bossArmorTimeNode)
		:enableOutline(cc.c3b(0 , 0 , 0),2)

	local bossArmorTimeLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_10, fonts.FONT_SIYUAN, 14)
		:setPosition(29,0)
		:addTo(self.bossArmorTimeNode)
		:enableOutline(cc.c3b(0 , 0 , 0),2)

	local tipBg = cc.Sprite:create(imagePath..'guild_boss_9.png')
		:addTo(self.bossArmorTimeNode)
		:setPosition(0 , -52)

	local tipLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_12, fonts.FONT_SIYUAN, 15)
		:setPosition(cc.p(0,-52))
		:addTo(self.bossArmorTimeNode)
		:setTextColor(cc.c3b(255,255,0))

	--boss技能
	self.bossSkillNode = cc.Node:create()
		:addTo(topNode)
		:setPosition(-137 , -133) --open: -137,-193
	local tipBg = cc.Sprite:create(imagePath..'guild_boss_6.png')
		:addTo(self.bossSkillNode)
		:setPosition(-105 , 13)
	--self.bossSkillBg = cc.Sprite:create()
	--	:addTo(self.bossSkillNode)
	--	:setPosition(-105 , 13)
	self.bossSkillIcon = cc.Sprite:create()
		:addTo(self.bossSkillNode)
		:setPosition(-105 , 13)
	
	self.bossSkillName = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 16)
		:addTo(self.bossSkillNode)
		:setPosition(-55 , 36)
		:setAnchorPoint(0 , 0.5)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
	
	self.bossSkillInfo = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 16)
		:addTo(self.bossSkillNode)
		:setPosition(-55 , 17)
		:setAnchorPoint(0 , 1)
		:enableOutline(cc.c3b(50 , 20 , 0),2)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setDimensions(198, 160)

	--下次开启倒计时
	self.notOpenTimeNode = cc.Node:create()
		:addTo(topNode)
		:setPosition(111,-116)

	local notOpenTimeBg = cc.Sprite:create('image/commonImg/time_down_bg_1.png')
		:addTo(self.notOpenTimeNode)

	local notOpenLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_14, fonts.FONT_SIYUAN, 15)
		:setPosition(cc.p(-9,-4))
		:addTo(self.notOpenTimeNode)
	
	local notOpenTimeBg = cc.Sprite:create(imagePath..'guild_boss_13.png')
		:addTo(self.notOpenTimeNode)
		:setPosition(98,-3)

	self.notOpenTime = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 14)
		:setPosition(98,-3)
		:addTo(self.notOpenTimeNode)
		:enableShadow(cc.c4b(90,30,30,125))

	--已开启倒计时
	self.openTimeNode = cc.Node:create()
		:addTo(topNode)
		:setPosition(111,-175)

	local openTimeBg = cc.Sprite:create('image/commonImg/time_down_bg_1.png')
		:addTo(self.openTimeNode)

	self.openTime = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 17)
		:setPosition(16.5,-4)
		:addTo(self.openTimeNode)
	
	local openTimeBg = cc.Sprite:create(imagePath..'guild_boss_12.png')
		:addTo(self.openTimeNode)
		:setPosition(124,-4)
	local openLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_9, fonts.FONT_SIYUAN, 16)
		:setPosition(cc.p(124,-2))
		:addTo(self.openTimeNode)
		:enableShadow(cc.c4b(95,30,30,125))

	--右侧按钮
	local rwdBtnNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(248 , 178)
	local rwdbtn = ccui.Button:create('image/main/main_btn_bg.png')
		:addTo(rwdBtnNode)
		:addClickEventListener(handler(self, self.bossRewardBtnClick))
	
	local btnIcon = cc.Sprite:create('image/main/main_btn_17.png')
		:addTo(rwdBtnNode)
	
	local rwdBtnlabel = cc.Label:createWithTTF(LANG.UI_EVENT_2, fonts.FONT_SIYUAN, 20)
		:addTo(rwdBtnNode)
		:setPosition(0 , -35)
		:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)
	self.rewardListBtn = rwdBtnNode;

	local rankBtnNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(248, 86)
	local rankbtn = ccui.Button:create('image/main/main_btn_bg.png')
		:addTo(rankBtnNode)
		:addClickEventListener(handler(self, self.OpenRankPanel))
	
	local btnIcon = cc.Sprite:create('image/main/main_btn_12.png')
		:addTo(rankBtnNode)
	
	local rwdBtnlabel = cc.Label:createWithTTF(LANG.UI_GUILD_RANK[1], fonts.FONT_SIYUAN, 20)
		:addTo(rankBtnNode)
		:setPosition(0 , -35)
		:enableOutline(cc.c4b(60 , 60 , 60 , 255),2)

	--伤害
	local totalDamgeBg = cc.Sprite:create(imagePath..'guild_boss_11.png')
		:addTo(self.root)
		:setPosition(-2 , -104)
	local totalDamgeImg = cc.Sprite:create(imagePath..'guild_boss_15.png')
		:addTo(self.root)
		:setPosition(-122 , -103)
	local totalDamagelabel = cc.Label:createWithTTF(LANG.UI_L_ATK, fonts.FONT_SIYUAN, 18)
		:addTo(self.root)
		:setPosition(-78 , -104)
	self.totalDamage = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
		:addTo(self.root)
		:setPosition(54 , -104)
		:enableOutline(cc.c4b(90 , 30 , 30 , 255),2)

	--编队
	local teamNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0 , -198)
	local teamNodeBg = cc.Sprite:create(imagePath..'guild_boss_8.png')
		:addTo(teamNode)
	
	self.headListNode = cc.Node:create()
		:addTo(teamNode)
		:setPosition(-15,0)

	self.teamBtn = ccui.Button:create('image/commonImg/team_icon.png' , 'image/commonImg/team_icon.png')
	self.teamBtn:addTo(teamNode)
	self.teamBtn:setPosition(189,0)
	self.teamBtn:addClickEventListener(handler(self , self.OpenTeamPanel))

	--挑战次数
	
	local challengeNumNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0 , -331)

	self.buyTime = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
		:setPosition(cc.p(0,40))
		:addTo(challengeNumNode)
		:setTextColor(cc.c3b(255 , 255, 0))

	local challengeNumBg = cc.Sprite:create(imagePath..'guild_boss_10.png')
		:addTo(challengeNumNode)

	local challengeNumNumLabel = cc.Label:createWithTTF(LANG.UI_GUILD_BOSS_3, fonts.FONT_SIYUAN, 18)
		:setPosition(cc.p(-74,0))
		:addTo(challengeNumNode)
	
	self.challengeNum = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 18)
		:setPosition(cc.p(83,0))
		:setAnchorPoint(1, 0.5)
		:addTo(challengeNumNode)
	
	local addChallengeNumBtn = ccui.Button:create(imagePath..'guild_boss_16.png' , imagePath..'guild_boss_16.png')
		:addTo(challengeNumNode)
		:setPosition(115,0)
		:addClickEventListener(handler(self , self.addChallengeNumBtnClick))

	local btnLabel = cc.Label:createWithTTF(LANG.UI_GUILD_58, fonts.FONT_SIYUAN, 18)
		:enableOutline(cc.c4b(40 , 40 , 100 , 255) , 1)

	self.fightBtn = ccui.Button:create('image/commonImg/btn_9_219_55.png')
	self.fightBtn:setPosition(cc.p(0 ,-399))
	self.fightBtn:addTo(self.root)
	self.fightBtn:setTitleLabel(btnLabel)
	self.fightBtn:addClickEventListener(handler(self , self.fightBtnClick))
	
end

function GuildBossPanel:OpenTeamPanel()
	AudioManager:playEffect(Conf.confirmButton)
	UIManager:push('TeamPanel', TeamType.GUILD_BOSS)
end

function GuildBossPanel:OpenRankPanel()
	UIManager:push('GuildRaidRankPanel');
end

function GuildBossPanel:registEvent()
	EventManager:RegisterEvent(EventType.NetworkGuildMissionRetInfo , handler(self , self.onShow));
	-- EventManager:RegisterEvent(EventType.NetworkGuildMissionRank , handler(self , self.retGuildMissionRank));
	EventManager:RegisterEvent(EventType.FightEnableChallenge , handler(self , self.enableBtnFight));
	EventManager:RegisterEvent(EventType.NetworkGuildBossKillBonusGet , handler(self , self.retGuildBossKillBonusGet));
	EventManager:RegisterEvent(EventType.NetworkGuildBossBuyChallengeNum , handler(self , self.retBuyChallengeNum));
	EventManager:RegisterEvent(EventType.NetworkGuildBossRedDotShow, handler(self, self.updateRedDotShow))
	EventManager:RegisterEvent(EventType.NetworkGuildBossCheckPoint, handler(self, self.retGuildBossCheckPoint))
	EventManager:RegisterEvent(EventType.UIUpateGuildBonusRedDot, handler(self, self.updateBonusRedDot))


	EventManager:RegisterEvent(EventType.UpdateGuildBossInfoPanel, handler(self, self.UpdateGuildBossInfoPanel))

end

function GuildBossPanel:UpdateGuildBossInfoPanel()
	-- body
	--更新编队信息
	self:updateBonusRedDot()
    self.headListNode:removeAllChildren()
    local roleList = model.TeamModel:getGuildTeam();
    self.isHaveTeam = false
	for i=1, TEAM_MAX_MEMBER do
		local roleNode = template.ItemTemplate:create()
		roleNode:addTo(self.headListNode);
		roleNode:setScale(0.7)
		roleNode:setPosition( -115 +(i-1)*80 , 0);

		local pid = roleList[i]
        if pid ~= -1 then
        	self.isHaveTeam = true
        	local card = model.CardModel:GetCard(pid);
			roleNode:Init(ROLE_TYPE , card.no,card.star_lv,card.level,pid);
			roleNode:updateAltarLevelColor(card.oriLv)
        end
	end

	self.isReqGuildBoss = false;

	self.openState = model.GuildModel:getGuildBossOpenState();
	if self.openState == 0 then
		self:onEventNotOpen();
	elseif self.openState == -1 then
		self:onEventOver()
		return;
	elseif self.openState == 1 then
		self.bossNameNode:setVisible(true);
		self.bossNameNode:setPosition(0 , 18);
		self.bossSkillNode:setPosition(-137,-193);
		self.fightBtn:setVisible(true)
		self.bossDamageNode:setVisible(true);
		self.buyTime:setVisible(true);

		self:updateRedDotShow()
	end
	
	self:onEnterFrame()

	local guildBossInfo = model.GuildModel:getGuildBossInfo()
	local raidMonsterData = ResTable:getRowData('guild_raid_order',guildBossInfo.raid.order_no);
	self.totalDamage:setString(tostring(guildBossInfo.raid.all_damage));
	
	local monsterId = 0;
	if self.bossArmorTimeNode:isVisible() then --无护甲状态
		monsterId = raidMonsterData.monster_body
	else
		monsterId = raidMonsterData.monster_armor
	end

	local monsterData = ResTable:getRowData('monster', monsterId)

	local panret = self.roleArmature:getParent();
	self.roleArmature:removeFromParent();
	self.roleArmature = nil ;
	self.roleArmature = DragonBonesManager:GetArmature(monsterData.model)
	self.roleArmature:addTo(panret);
	self.roleArmature:setPosition(cc.p(4,25));
	self.roleArmature:getAnimation():play("idle");
	self.roleArmature:getArmature():setFlipX(true);
	self.roleArmature:setTag(monsterId);
    self.roleArmature:setCascadeOpacityEnabled(true);

    self.bossName:setString( monsterData.name )

    local skillData = ResTable:getRowData('skill', monsterData.skill_2)
    self.bossSkillName:setString( skillData.name )
    self.bossSkillInfo:setString( skillData.info )

	--self.bossSkillBg:setTexture('image/card/cardSkill/card_skill_type_'..skillData.skill_type..'.png');
    self.bossSkillIcon:setTexture( 'image/skillicon/'..skillData.icon..'.png' )

    self.bossColor:setTexture( 'image/commonImg/def_type_'..monsterData.def_type..'.png' )
   
	local width = self.colorBg:getBoundingBox().width;
	local nameWidth = self.bossName:getContentSize().width+10;
	local posX = -nameWidth/2;
	self.colorBg:setPositionX(posX);
	self.bossName:setPositionX(posX + width/2 + nameWidth/2)


	self:updateButtonState()
	self:updateRedDot()
end

function GuildBossPanel:updateRedDot()
	local show = model.TeamModel:checkTeamFormationRedDotState();
	tools.RedDotTools:setRedDotToNode(self.teamBtn,show,nil,cc.p(38,38))
end

function GuildBossPanel:onEventOver()
	
	--self.bossArmorTimeNode:setVisible(false)
	--self.bossArmorNode:setVisible(false)
	self.bossDamageNode:setVisible(false)

	self.notOpenTimeNode:setVisible(true);
	self.openTimeNode:setVisible(false)
    self.notOpenTime:setString(LANG.UI_L_OVER)

    self.fightBtn:setVisible(false)

    self.bossName:setString( LANG.UI_L_OVER )
    self.bossSkillName:setString( LANG.UI_L_OVER )
	self.bossSkillInfo:setString( LANG.UI_L_OVER )
	self.challengeNum:setString('0');
	self.buyTime:setVisible(false);

	self.bossNameNode:setVisible(false);
	self.bossSkillNode:setPosition(-137,-133);
end

function GuildBossPanel:onEventNotOpen()
	self.bossNameNode:setVisible(true);
	self.notOpenTimeNode:setVisible(true);
	self.openTimeNode:setVisible(false);
	self.fightBtn:setVisible(false)
	self.bossDamageNode:setVisible(false);
	self.challengeNum:setString('0');
	self.bossNameNode:setPosition(0 , -7);
	self.bossSkillNode:setPosition(-137,-133);
	self.buyTime:setVisible(false);
	--self.bossArmorTimeNode:setVisible(false)
	--self.bossArmorNode:setVisible(false)

	--self.notOpenTime:setString(LANG.UI_GUILD_BOSS_14)
end

function GuildBossPanel:onEnterFrame()
	self.open = false

	local guildBossInfo = model.GuildModel:getGuildBossInfo()

	if self.openState == 0 then
		local raidMonsterData = ResTable:getRowData('guild_raid_order',guildBossInfo.raid.order_no);
		local startTable = os.date('*t', guildBossInfo.raid.open_time);
		local msg = string.format( LANG.UI_EVENT_29,startTable.month,startTable.day ).." "..string.format('%02d:%02d', startTable.hour, startTable.min)..' '..LANG.UI_GUILD_129;
		self.notOpenTime:setString(tostring(msg));

		local serverTime = math.floor(TimeManager:getCurTime());
		if serverTime >= guildBossInfo.raid.open_time and not self.isReqGuildBoss then
			self:onEventOver();
			self:reqGuildBoss();
			self.isReqGuildBoss = true;
		end
		return;
	end

	if self.openState == -1 then
		return
	end

	local raidMonsterData = ResTable:getRowData('guild_raid_order',guildBossInfo.raid.order_no);
	local serverTime = math.floor(TimeManager:getCurTime());
	local lastTime = guildBossInfo.raid.armor_cd - serverTime;
	if lastTime > 0 then
		self.bossArmorTimeNode:setVisible(true)
		self.bossArmorNode:setVisible(false)
		self.bossArmorTime:setString( Sec2Time(lastTime) )

	else
		self.bossArmorTimeNode:setVisible(false)
		self.bossArmorNode:setVisible(true)

		local armor_hp = guildBossInfo.raid.armor_hp
		armor_hp = armor_hp == 0 and raidMonsterData.armor_value or armor_hp;
		self.bossArmorHp:setString(armor_hp)

		local pro = armor_hp/raidMonsterData.armor_value * 100;
		pro = pro > 100 and 100 or pro;
		self.bossArmorProgress:setPercentage(pro)
	end
	
	if serverTime < guildBossInfo.raid.open_time then
		self.openTimeNode:setVisible(false)
		self.notOpenTimeNode:setVisible(true);
		local startTable = os.date('*t', guildBossInfo.raid.open_time)
        local endTable = os.date('*t', guildBossInfo.raid.close_time)

        local msg = string.format( LANG.UI_EVENT_29,startTable.month,startTable.day ).."--"..string.format( LANG.UI_EVENT_29,endTable.month,endTable.day )

		self.notOpenTime:setString( msg )

	elseif serverTime >= guildBossInfo.raid.open_time and serverTime < guildBossInfo.raid.close_time then
		self.openTimeNode:setVisible(true)
		self.notOpenTimeNode:setVisible(false);

		local lastTime = guildBossInfo.raid.close_time - serverTime;
		local day = math.floor(lastTime / 86400);
		if day > 0 then
			local time = lastTime - day * 86400;
			self.openTime:setString(day..LANG.UI_CARD_41..Sec2Time(time) )
		else
			self.openTime:setString( Sec2Time(lastTime) )
		end
		
        self.open = true
	elseif not self.isReqGuildBoss then
		self:onEventOver();
		self:reqGuildBoss();
		self.isReqGuildBoss = true;
	end

end

function GuildBossPanel:retGuildBossCheckPoint( msg )--扫荡暂时关闭了
	-- 
	self:onShow()
	UIManager:push('RewardPanel',msg)
end

function GuildBossPanel:fightBtnClick()

	-- local open_state = tonumber(model.GuildModel:getGuildBossOpen());
	-- if open_state == 0 or open_state == nil then
	-- 	Prompt:Show(LANG.UI_GUILD_BOSS_6);
	-- end 
	if self.open == false or self.openState ~= 1 then
		Prompt:Show(LANG.UI_GUILD_BOSS_6);
		return
	end

	local guildBossInfo = model.GuildModel:getGuildBossInfo()

	if guildBossInfo.raidUser.guild_raid_challenge_num <= 0 then
		Prompt:Show(LANG.UI_GUILD_BOSS_7);
		return
	end

	if self.isHaveTeam == false then --没有编队信息 --是否使用关卡队伍进行挑战？
		local box = MessageBox:create(LANG.UI_GUILD_BOSS_15 , MessageBoxType.CANCEL_OK,
			function() --去编队
				UIManager:push('TeamPanel', TeamType.GUILD_BOSS)
			end,
			function() --立即挑战
				--保存关卡编队到 工会boss编队
				local guildTeam = {};
				local topTeam = model.TeamModel:getTopFpTeam();
				for i=1, TEAM_MAX_MEMBER do
					guildTeam[i] = topTeam[i].pid;
				end
		        model.TeamModel:sendChangeTeam(guildTeam, 2);
				if FightManager:enterFight(FightType.GuildBoss) then
					self.fightBtn:setTouchEnabled(false);
				end
			end);
		box:setOkBtnName(LANG.UI_GUILD_BOSS_17)--去编队
		box:setCancelBtnName(LANG.UI_GUILD_BOSS_16)--立即挑战
		return
	end
	if FightManager:enterFight(FightType.GuildBoss) then
		self.fightBtn:setTouchEnabled(false);
	end
end

function GuildBossPanel:enableBtnFight()
	self.fightBtn:setTouchEnabled(true);
end

function GuildBossPanel:retGuildBossKillBonusGet(  )--没用a
	self:updateButtonState();
end

function GuildBossPanel:retGuildMissionRank()--没用b
end

function GuildBossPanel:bossRewardBtnClick(  )
	-- boss完成奖励
	UIManager:push('GuildBossBonusListPanel');
end

function GuildBossPanel:onEnter()
	self:onEventOver()
	self:reqGuildBoss();
	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.onEnterFrame), 1, false);

	self.root:setVisible(true);
	if IsKr then
		EventManager:FireEvent(EventType.UIFooterUpdateSelect,4);
	else
		EventManager:FireEvent(EventType.UIFooterUpdateSelect,5);
	end
end

function GuildBossPanel:reqGuildBoss()
	model.GuildModel:reqGuildBossInfo();
end

function GuildBossPanel:onShow()
	-- local bossStage = model.GuildModel:getBossStage() 
	-- if bossStage == -1 then
	-- 	Prompt:Show(LANG.UI_GUILD_56);
	-- 	return
	-- end
	
	-- self.guildBossTableData = ResTable:getRowData('guild_boss',bossStage);

	self:updateBonusRedDot()
	self.root:setVisible(true);
end

function GuildBossPanel:updateButtonState(  )
	if self.openState ~= 1 then
		return
	end

	local guildBossInfo = model.GuildModel:getGuildBossInfo()
	self.challengeNum:setString(tostring(guildBossInfo.raidUser.guild_raid_challenge_num)) --挑战次数
	self.buyTime:setString(string.format(LANG.UI_GUILD_BOSS_18,guildBossInfo.raidUser.guild_raid_last_buy_num)); --购买次数

end

function GuildBossPanel:updateRedDotShow(  )
	--通关奖励
	local redDotBonus = model.GuildModel:getNewGuildBossRewardState();
	tools.RedDotTools:setRedDotToNode(self.rewardListBtn,redDotBonus,nil,cc.p(30,30))
	-- 挑战次数
	local redDotChallenge = model.GuildModel:getNewGuildBossFightState()
	tools.RedDotTools:setRedDotToNode(self.fightBtn,redDotChallenge,nil,cc.p(209,45))

end

function GuildBossPanel:onLeave()
	self.root:setVisible(false);

	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
end

function GuildBossPanel:addChallengeNumBtnClick()
	--local nextCoin = model.GuildModel:getNextCoin();
	if self.openState ~= 1 then
		Prompt:Show(LANG.UI_GUILD_122);
		return;
	end

	local costName = ResTable:getValue('item' , COLOR_DIA , 'name');

	MessageBox:create(string.format(LANG.UI_GUILD_57 , tostring(20), tostring(costName)) , MessageBoxType.CANCEL_OK,function()
		model.GuildModel:reqGuildBossFightTimes();
	end);
end

function GuildBossPanel:retBuyChallengeNum()
	self:updateButtonState();
	self:updateRedDotShow();
	Prompt:Show(LANG.UI_GUILD_59)

end

function GuildBossPanel:updateBonusRedDot()
	local state = model.GuildModel:getNewGuildBossRewardState();
	tools.RedDotTools:setRedDotToNode(self.rewardListBtn,state,nil,cc.p(30 , 30));
end

function GuildBossPanel:onClickBack()
	UIManager:pop('GuildBossPanel');
	tools.RedDotTools:pushGuildRedDot(2)
end

return GuildBossPanel;
