local HarvestPanel = class("HarvestPanel", require('app.layer.BaseLayer'))

local fonts = Enum:getTable("SystemFont")
local imgPath = 'image/harvest/'

function HarvestPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'HarvestPanel');

	self.useCCreator = false;
	self.isPopup = true;
	self.haveSwallowLayer = false;
end

function HarvestPanel:Init(root)
	self.root:setVisible(false);

	self:initPanels();
	self:registEvent();

	self.state = 1;
	self.showEffect = false
end

function HarvestPanel:initPanels()
	self.topNodePos = cc.p(0, 300);

	self.harvest = {};

	self.harvest[1] = cc.Sprite:create()
		:setLocalZOrder(10)
		:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		:addTo(self.root)
		:setBlendFunc(cc.blendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA));

	self.harvest[2] = cc.Sprite:create()
		:setLocalZOrder(10)
		:setVisible(false)
		:setPosition(self.topNodePos)
		:addTo(self.root)

	self.harvest[3] = cc.Sprite:create()
		:setLocalZOrder(10)
		:setVisible(false)
		:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		:addTo(self.root)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.harvest[4] = cc.Sprite:create(imgPath .. 'Harvest_4.png')
		:setLocalZOrder(0)
		:setVisible(false)
		:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		:addTo(self.root)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.harvest[5] = cc.Sprite:create()
		:setLocalZOrder(0)
		:setVisible(false)
		:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		:addTo(self.root)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.harvest[6] = cc.Sprite:create()
		:setLocalZOrder(15)
		:setVisible(false)
		:setPosition(self.topNodePos)
		:addTo(self.root)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.numLabel = cc.Label:createWithTTF('', fonts.FONT_HUAKANG, 20)
		:setLocalZOrder(20)
		:enableOutline(cc.c4b(60, 60, 60, 255), 2)
		:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		:addTo(self.root)

	self.clickLayout = ccui.Layout:create()
	self.clickLayout:setContentSize(cc.size(130, 130))
		:setLocalZOrder(40)
		:setPosition(HARVEST_NODE_X-65, HARVEST_NODE_Y-65)
		:addTo(self.root)
		:setTouchEnabled(true)
		:addClickEventListener(function()
			if model.HarvestModel:isFull() then
				self:playHarvestEffect();
			end
		end)

	self.ShowLayer = cc.LayerColor:create(cc.c4b(0, 0, 0, 154))
		:setContentSize(cc.size(StageSize.width, StageSize.height))
		:addTo(self.root)
		:setVisible(false)
		:setPosition(cc.p(-StageSize.width/2, -StageSize.height/2))
		:setLocalZOrder(-10)
	
	self.touchLayer = ccui.Layout:create()
	self.touchLayer:setContentSize(cc.size(StageSize.width, StageSize.height))
		:setLocalZOrder(300)
		:addTo(self.root)
		:setVisible(false)
		:setAnchorPoint(cc.p(0.5, 0.5))
		:setTouchEnabled(true)
		:setSwallowTouches(true)
	
	self.ClickLayer = ccui.Layout:create()
	self.ClickLayer:setContentSize(cc.size(StageSize.width, StageSize.height))
		:setLocalZOrder(200)
		:addTo(self.root)
		:setVisible(false)
		:setAnchorPoint(cc.p(0.5, 0.5))
		:setPosition(cc.p(0, 0))
		:setTouchEnabled(true)
		:addTouchEventListener(function ( sender,eventType )
			if eventType == ccui.TouchEventType.began then
				self:stopClickEffect()
				model.HarvestModel:tapScreen();
				self:playHarvestCountEffect(model.HarvestModel:getTap())

				if self.touchEffectNode then
					local tapEffect2 = cc.ParticleSystemQuad:create('Effect/particle/harvest_coin_drop.plist')
						:setTexture(cc.Director:getInstance():getTextureCache():addImage('image/icon/I10001.png'))
						:setAutoRemoveOnFinish(true)
						--:setGravity(cc.p(0, -StageSize.height))
						:setDuration(0.1)
						:addTo(self.touchEffectNode)
						:setPosition(self.topNodePos);

					local tapEffect3 = cc.ParticleSystemQuad:create('Effect/particle/harvest_dia_drop.plist')
						:setAutoRemoveOnFinish(true)
						--:setGravity(cc.p(0, -StageSize.height))
						:setTexture(cc.Director:getInstance():getTextureCache():addImage('image/icon/I10002.png'))
						:setDuration(0.1)
						:addTo(self.touchEffectNode)
						:setPosition(self.topNodePos);
				end
			end	
		end)
	
	self.clickNode = cc.Node:create()
		:setVisible(false)
		:addTo(self.root)
	
	self.click1 = cc.Sprite:create(imgPath .. 'click_1.png')
		:addTo(self.clickNode)

	self.click2 = cc.Label:createWithTTF(LANG.UI_L_HAVESTCLICK,  fonts.FONT_IMPACT , 46);
	self.click2:addTo(self.clickNode);
	self.click2:setTextColor(cc.c4b(36, 153, 218 , 255));
	self.click2:enableBold();

	self.clickEffect1 = cc.Sprite:create()
		:addTo(self.clickNode)

	self.clickEffect2 = cc.Sprite:create()
		:setFlipX(true)
		:addTo(self.clickNode)

	self.CountNode = cc.Node:create()
		:setPosition(0, 50)
		:setLocalZOrder(500)
		:setVisible(false)
		:addTo(self.root)

	self.comboNode1 = cc.Node:create()
		:setVisible(false)
		:addTo(self.CountNode)

	self.combo_1_1 = cc.Sprite:create('image/harvest/combo_1_1.png')
		:addTo(self.comboNode1);
	
	self.combo_1_2 = cc.Sprite:create()
		:addTo(self.comboNode1)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.combo_1_num = cc.Label:createWithTTF('', fonts.FONT_JP, 50)
		:setTextColor(cc.c3b(9, 176, 246))
		:enableOutline(cc.c4b(60, 60, 60, 255), 3)
		:setPosition(100, -2)
		:addTo(self.comboNode1)

	self.comboNode2 = cc.Node:create()
		:setVisible(false)
		:addTo(self.CountNode)

	self.combo_2_3 = cc.Sprite:create()
		:addTo(self.comboNode2)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.combo_2_2 = cc.Sprite:create('image/harvest/combo_2_2.png')
		:addTo(self.comboNode2)

	self.combo_2_1 = cc.Sprite:create()
		:addTo(self.comboNode2)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));
	
	self.combo_2_num = cc.Label:createWithTTF('', fonts.FONT_JP, 50)
		:setTextColor(cc.c3b(255, 222, 0))
		:enableOutline(cc.c4b(60, 60, 60, 255), 3)
		:setPosition(100, -2)
		:addTo(self.comboNode2)

	self.comboNode3 = cc.Node:create()
		:setVisible(false)
		:addTo(self.CountNode)

	self.combo_3_3 = cc.Sprite:create()
		:addTo(self.comboNode3)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));

	self.combo_3_2 = cc.Sprite:create('image/harvest/combo_2_2.png')
		:addTo(self.comboNode3)

	self.combo_3_1 = cc.Sprite:create()
		:addTo(self.comboNode3)
		:setBlendFunc(cc.blendFunc(GL_SRC_ALPHA , GL_ONE));
	
	self.combo_3_num = cc.Label:createWithTTF('', fonts.FONT_JP, 50)
		:setTextColor(cc.c3b(255, 67, 67))
		:enableOutline(cc.c4b(60, 60, 60, 255), 3)
		:setPosition(100, -2)
		:addTo(self.comboNode3)
end

function HarvestPanel:registEvent()
	EventManager:RegisterEvent(EventType.FightNext, handler(self, self.backAutoFight));
	EventManager:RegisterEvent(EventType.FightBegin, handler(self, self.leaveAutoFight));
	EventManager:RegisterEvent(EventType.HarvestPowerGen, handler(self, self.playCollectEffect))
end

function HarvestPanel:onEnter()
	self.root:setVisible(true)
	self:beginHarvestEffect(true)
end

function HarvestPanel:onLeave()
	self.root:setVisible(false)
	self:stopEffect()
end

function HarvestPanel:beginHarvestEffect(flag)
	if self.state ~= 1 then
		return;
	end
	for k, v in pairs(self.harvest) do
		v:setVisible(false)
		v:stopAllActions()
	end
	self.harvest[1]:setVisible(true)
	self.harvest[1]:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('Harvest_1')))

	local curPower = model.HarvestModel:getCurPower();
	local maxPower = model.HarvestModel:getMaxPower();
	if flag then 
		self.numLabel:setString(string.format('%s/%s', tostring(curPower), tostring(maxPower)));
	end
end

function HarvestPanel:stopEffect()
	for k, v in pairs(self.harvest) do
		v:stopAllActions()
	end
end

function HarvestPanel:playHarvestCountEffect(count)
	local v1, v2, v3 = HARVEST_TIMES1, HARVEST_TIMES2, HARVEST_TIMES3
	if count == v1 then
		self.CountNode:setVisible(true);
		self.comboNode1:setVisible(true);
		self.comboNode2:setVisible(false);
		self.comboNode3:setVisible(false);

		self.combo_1_1:setOpacity(0);
		self.combo_1_1:runAction(cc.FadeIn:create(0.3));

		self.combo_1_2:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_1_2')));

		self.combo_1_num:setString(tostring(count))
	elseif count > v1 and count < v2 then
		self.combo_1_num:setString(tostring(count))
	elseif count == v2 then
		self.combo_1_2:stopAllActions()
		self.comboNode1:setVisible(false)
		self.comboNode2:setVisible(true)

		self.combo_2_2:setOpacity(0)
		self.combo_2_2:runAction(cc.FadeIn:create(0.3))

		self.combo_2_1:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_2_1')))
		self.combo_2_3:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_2_3')))
		self.combo_2_num:setString(tostring(count))
	elseif count > v2 and count < v3 then
		self.combo_2_num:setString(tostring(count))
	elseif count == v3 then
		self.combo_2_1:stopAllActions()
		self.combo_2_3:stopAllActions()
		self.comboNode2:setVisible(false)
		self.comboNode3:setVisible(true)

		self.combo_3_1:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_3_1')))
		self.combo_3_2:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_3_2')))
		self.combo_3_3:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('combo_3_3')))
		self.combo_3_num:setString(tostring(count))
	elseif count > v3 then
		self.combo_3_num:setString(tostring(count))
	end
end

function HarvestPanel:playHarvestEffect()
	self.showEffect = true
	self.touchLayer:setVisible(true)

	model.HarvestModel:beginTap();
	self.numLabel:setVisible(false);
	self.harvest[1]:setVisible(false)

	self.harvest[5]:setVisible(true)
	self.harvest[5]:runAction(cc.Sequence:create(
		FrameEffectManager:createAction('Harvest_5'),
		cc.CallFunc:create(function()
			self.harvest[5]:setVisible(false)
		end),
		cc.DelayTime:create(0.2),
		cc.CallFunc:create(function()
			self.harvest[2]:setVisible(true)
		end)
		))

	self.harvest[4]:setVisible(false)
	self.harvest[4]:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.5),
		cc.CallFunc:create(function()
			self.harvest[4]:setVisible(true)
			self.harvest[4]:setPosition(HARVEST_NODE_X, HARVEST_NODE_Y)
		end),
		cc.MoveTo:create(0.5, self.topNodePos),
		cc.CallFunc:create(function()
			self.harvest[4]:setVisible(false)
		end)
		))

	self.ShowLayer:setVisible(true);
	self.harvest[6]:setVisible(false)
	self.harvest[6]:runAction(cc.Sequence:create(
		cc.DelayTime:create(1),
		cc.CallFunc:create(function()
			self.harvest[6]:setVisible(true)
		end),
		FrameEffectManager:createAction('Harvest_6'),
		cc.CallFunc:create(function()
			self:showClickEffect()
			if not self.touchEffectNode then
				self.touchEffectNode = cc.Node:create()
					:setLocalZOrder(70)
					:addTo(self.root)
			end
			self.harvest[6]:setVisible(false)
			self.touchLayer:setVisible(false)
			self.ClickLayer:setVisible(true);
		end),
		cc.DelayTime:create(8),
		cc.CallFunc:create(function()
			self.harvest[1]:setVisible(true)
			self.harvest[2]:stopAllActions()
			self.harvest[2]:setVisible(false)
			self.ShowLayer:setVisible(false)
			self.ClickLayer:setVisible(false)
			self.CountNode:setVisible(false)
			self.numLabel:setVisible(true)
			self:beginHarvestEffect(true)
			self.showEffect = false
			model.HarvestModel:sendTapResult();
			if self.touchEffectNode then
				self.touchEffectNode:removeFromParent()
				self.touchEffectNode = nil;
			end
		end)
		))

	self.harvest[2]:setVisible(false)
	self.harvest[2]:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('Harvest_2')))
	
end

function HarvestPanel:playCollectEffect()
	if self.showEffect then
		return;
	end
	if self.state ~= 1 then
		return;
	end
	local callback = function()
		local curPower = model.HarvestModel:getCurPower();
		local maxPower = model.HarvestModel:getMaxPower();
		self.numLabel:setString(string.format('%s/%s', tostring(curPower), tostring(maxPower)));
	end;

	self.harvest[3]:setVisible(true)
	self.harvest[3]:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.5),
		FrameEffectManager:createAction('Harvest_3'),
		cc.CallFunc:create(callback)
	))
end

function HarvestPanel:backAutoFight()
	self.state = 1;
	self.harvest[1]:setVisible(true);
	self.numLabel:setVisible(true);
	self.clickLayout:setVisible(true);
	self:beginHarvestEffect(true)
end

function HarvestPanel:leaveAutoFight()
	self.state = 2;
	self.harvest[1]:setVisible(false);
	self.numLabel:setVisible(false);
	self.clickLayout:setVisible(false);
end

function HarvestPanel:showClickEffect()
	self.clickEffectFlag = true;
	self.clickNode:setVisible(true)
	self.clickNode:runAction(cc.Sequence:create(
		cc.DelayTime:create(3),
		cc.CallFunc:create(handler(self, self.stopClickEffect))
		))

	self.click1:setScaleX(1)
	self.click1:setScaleY(0)
	self.click1:runAction(cc.Sequence:create(
		cc.ScaleTo:create(0.16, 1, 0.8),
		cc.ScaleTo:create(0.12, 1, 0.75)
		))

	self.click2:setOpacity(0);
	self.click2:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.12),
		cc.FadeIn:create(0.24)
		))

	self.clickEffect1:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('Harvest_7')));
	self.clickEffect2:runAction(cc.RepeatForever:create(FrameEffectManager:createAction('Harvest_7')));
end

function HarvestPanel:stopClickEffect()
	if not self.clickEffectFlag then
		return;
	end
	self.clickEffectFlag = false;
	self.click1:stopAllActions()
	self.click2:stopAllActions()
	self.clickEffect1:stopAllActions()
	self.clickEffect2:stopAllActions()
	self.clickNode:stopAllActions()
	self.clickNode:setVisible(false)
end

return HarvestPanel;
