local HeadSelectPanel = class("HeadSelectPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont")
local MessageBoxType = Enum:getTable('MessageBoxType')

function HeadSelectPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'HeadSelectPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function HeadSelectPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function HeadSelectPanel:initPanels()

	local bg = cc.Sprite:create('image/bg/Bg_5.png');
	bg:addTo(self.root);

	local healdBg = cc.Sprite:create('image/setup/head_select_bg_1.png');
	healdBg:addTo(self.root);
	healdBg:setPosition(0 , 403);

	self.headIcon = template.ItemTemplate:create();
	self.headIcon:addTo(self.root);
	self.headIcon:setPosition(0 , 403);
	self.headIcon:setVisible(false);

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -71);
	bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(604 , 701));

	local initData = {
        width = 618,
        height = 685,
        cellWidth = 618 ,
        cellHeight = 131,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

    }
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData);
    self.cardListView:getView():setPosition(cc.p(-309,-410.5));
	self.cardListView:getView():addTo(self.root)

	local texture = 'image/commonImg/menu_left_71_46.png';
	local texture = 'image/commonImg/menu_left_71_46.png';
	local untexture = 'image/commonImg/menu_left_71_46_u.png';

	self.menuList = {};
    for i = 1 , 2 do
		
		local bgBtn = ccui.Button:create(texture,texture,unTexture);
        bgBtn:addTo(self.root);
		bgBtn:setTag(i);
		bgBtn:setPosition(-151 + 302 * (i - 1) , 302);
		bgBtn:setScaleX(i == 2 and -1 or 1);
		bgBtn:setScale9Enabled(true)
		bgBtn:setCapInsets(cc.rect(10 , 0 , 51 , 46));
		bgBtn:setContentSize(cc.size(302 , 45));
		bgBtn:addClickEventListener(handler(self , self.menuClick));
		self.menuList[i] = bgBtn;

		local label = cc.Label:createWithTTF(LANG.UI_SETTING_KANBAN_MENU[i] , fonts.FONT_SIYUAN , 20);
        label:setScaleX(i == 2 and -1 or 1);
        bgBtn:setTitleLabel(label)

	end
	local btnSize = self.menuList[1]:getContentSize();
	self.light = cc.Sprite:create('image/commonImg/menu_light_1.png');
	self.light:addTo(self.root);
	self.light:setPosition(-151 , 302+btnSize.height/2);

	--self.cardView = ccui.ScrollView:create();
	--self.cardView:setTouchEnabled(true);
	--self.cardView:setBounceEnabled(true);
	--self.cardView:setDirection(ccui.ScrollViewDir.vertical);
	--self.cardView:setContentSize(cc.size(618 , 685));
	--self.cardView:setAnchorPoint(0.5 , 0.5);
	--self.cardView:setPosition(0 , -68);
	--self.cardView:setScrollBarEnabled(false);
	--self.cardView:addTo(self.root);
end

function HeadSelectPanel:onEnter()

	self.selectNo = nil;
	self.no = model.UserModel:getHeadIcon();
	self:switchList(1);
	local headData = GlobalFunctions:getHeadIconData(self.no);
	self:setHead(headData);
	self.root:setVisible(true);
end
function HeadSelectPanel:menuClick(sender , event)
	local tag = sender:getTag()
	self:switchList(tag);
end

function HeadSelectPanel:switchList(index)
	self.cardIndexList = {};

	for _k , menu in pairs(self.menuList) do
		local label = menu:getTitleLabel();
		if _k == index then
			menu:setBright(true);
			label:setTextColor(cc.c3b(255 , 255 , 255));
			self.light:setPositionX(menu:getPositionX());
		else
			menu:setBright(false);
			label:setTextColor(cc.c3b(180 , 180 , 180));
		end
	end

	if index == 1 then
		local cards = model.CardModel:getAllGetCard();

		if not cards then
			return;
		end
		self.cards = {};
		for roleId , _ in pairs(cards) do
			local roleData = ResTable:getRowData('role', roleId);
			if roleData then
				local data = {no = roleId}
				local cardNos=model.CardModel:getCardByNo(roleId);
				local maxQualityCard = nil;
				for pid,_v in pairs(cardNos) do
					local card = model.CardModel:GetCard(pid);
					if card and (not maxQualityCard or card.quality > maxQualityCard.quality) then
						maxQualityCard = card;
					end
				end
				if maxQualityCard then
					data.quality = maxQualityCard.quality;
					data.starLv = maxQualityCard.star_lv;
				end
				table.insert(self.cards , data);
			end
		end
	
		if #self.cards > 1 then
			table.sort(self.cards , function(firstData , secondData)
				local firstQuality = firstData.quality;
				local secondQuality = secondData.quality;

				if not firstQuality then
					firstQuality = ResTable:getValue('role', firstData.no,'quality');
				end
				if not secondQuality then
					secondQuality = ResTable:getValue('role', secondData.no,'quality');
				end

				local firstStar = firstData.starLv;
				local secondStar = secondData.starLv;
				if not firstStar then
					firstStar = ResTable:getValue('role_pre', firstData.no , 'initial_star');
				end
				if not secondStar then
					secondStar = ResTable:getValue('role_pre', secondData.no , 'initial_star');
				end

				if firstStar ~= secondStar then
					return firstStar > secondStar;
				end
				if firstQuality ~= secondQuality then
					return firstQuality > secondQuality;
				end
				return firstData.no < secondData.no;
			end);
		end
	else
		local avatars = model.CardModel:getAllAvatar();
		self.cards = {};
		local haveId = {};
		for _k , avatar in pairs(avatars) do
			if avatar.overdue_time == 0 and not haveId[avatar.no] then
				local data = {no = avatar.no};
				table.insert(self.cards , data);
				haveId[avatar.no] = 1;
			end
		end

		if #self.cards > 1 then
			table.sort(self.cards , function(first , second)
				local firstNo = first.no;
				local secondNo = second.no;

				local firstRoleId = ResTable:getValue('role_avatar' , firstNo,'role_id');
				local secondRoleId = ResTable:getValue('role_avatar' , secondNo,'role_id');

				local firstData = ResTable:getRowData('role', firstRoleId);
				local secondData = ResTable:getRowData('role', secondRoleId);

				if firstData.quality ~= secondData.quality then
					return firstData.quality > secondData.quality;
				end

				return firstNo < secondNo;
			end);
		end
	end

	for idx , card in ipairs(self.cards) do
		local index = math.ceil(idx / 5);
		if not self.cardIndexList[index] then
			self.cardIndexList[index] = {};
		end
		table.insert(self.cardIndexList[index] , card);
	end

	self.cardListView:setDataCount(#self.cardIndexList)
	self.cardListView:reloadData();
end
function HeadSelectPanel:cellItemTouch(table , cell)

	if self.selectNo then
		if self.no == self.selectNo then
			return;
		end

		self.no = self.selectNo;

		for index , cardList in ipairs(self.cardIndexList) do
			local cell = self.cardListView:cellAtIndex(index - 1);
			if cell then
				local children = cell:getChildren();
				for _idx , child in ipairs(children) do
					if child:getItemTag() == self.selectNo then
						child:setSelect(true);
						if cardList[_idx] then
							self:setHead(cardList[_idx]);
						end
					else
						child:setSelect(false);
					end
				end
			end
		end

		self.selectNo = nil;
		--[[
		local fileUtils = cc.FileUtils:getInstance();
		local fileName = model.UserModel:SelfSettingConfig();
		local settingConfig = {};
		if fileUtils:isFileExist(fileName) then
			local fileData = fileUtils:getStringFromFile(fileName);
			if fileData ~= '' then
				settingConfig = Cjson.decode(fileData);
			end
		end

		settingConfig.headIcon = self.no;
		fileUtils:writeStringToFile(Cjson.encode(settingConfig), fileName);

		model.UserModel:setHeadIcon(self.no);

		local roleId = self.no;
		if self.no < 1000 then
			roleId = ResTable:getValue('role' , self.no , 'role_id');
		else
			roleId = ResTable:getValue('role_avatar' , self.no , 'role_id');
		end
		
		local name = ResTable:getValue('role_info' , roleId , 'name');
		Prompt:Show(string.format(LANG.UI_USER_1,(name and tostring(name) or '')));
		EventManager:FireEvent(EventType.UIHeaderPanelUpdateIcon);
		]]
		AudioManager:playEffect(Conf.buttonClick);
	end
end

function HeadSelectPanel:reloadCellPanel(cell , idx)

	local cardList = self.cardIndexList[idx+1];

	if not cardList then
		return;
	end

	local children = cell:getChildren();
	if not children or #children <= 0 then
		for i = 1 , 5 do
			local x = 65 + 122 * (i - 1);
			local y = 65;
			local iconTemplate = template.ItemTemplate:create(2);
			iconTemplate:addTo(cell);
			iconTemplate:setPosition(cc.p(x , y));
			iconTemplate:addItemClickEventListener(handler(self , self.cardClick));
			local cardData = cardList[i]
			if cardData then
				if cardData.no < 1000 then
					if cardData.quality then
						iconTemplate:Init(ROLE_TYPE , cardData.no,cardData.starLv,nil,nil,cardData.quality);
					else
						iconTemplate:Init(ROLE_TYPE , cardData.no);
					end
				else
					iconTemplate:Init(ROLE_AVATAR_TYPE , cardData.no);
				end
				iconTemplate:setItemTag(cardData.no);
				iconTemplate:setSelect(self.no ==cardData.no);
			else
				iconTemplate:setVisible(false);
			end
		end
	else
		for i = 1 , 5 do
			local cardData = cardList[i]
			if cardData then
				children[i]:setVisible(true);
				if cardData.no < 1000 then
					if cardData.quality then
						children[i]:Init(ROLE_TYPE , cardData.no,cardData.starLv,nil,nil,cardData.quality);
					else
						children[i]:Init(ROLE_TYPE , cardData.no);
					end
				else
					children[i]:Init(ROLE_AVATAR_TYPE , cardData.no);
				end
				children[i]:setItemTag(cardData.no);
				children[i]:setSelect(self.no ==cardData.no);
			else
				children[i]:setVisible(false);
			end
		end
	end

	return cell;
end

function HeadSelectPanel:cardClick(sender , event)
	self.selectNo = sender:getTag();
end

function HeadSelectPanel:setHead(data)
	self.headIcon:stopAllActions();
	self.headIcon:setOpacity(0);
	self.headIcon:setVisible(true);
	if data.no < 1000 then
		if data.quality then
			self.headIcon:Init(ROLE_TYPE , data.no,data.starLv,nil,nil,data.quality);
		else
			self.headIcon:Init(ROLE_TYPE , data.no);
		end
	else
		self.headIcon:Init(ROLE_AVATAR_TYPE , data.no);
	end

	self.headIcon:runAction(cc.FadeIn:create(0.3));
end
function HeadSelectPanel:onClickBack()
	if self:checkHeadChange(function()
			UIManager:pop('HeadSelectPanel');
		end) then
		return;
	else
		UIManager:pop('HeadSelectPanel');
	end
end

function HeadSelectPanel:checkHeadChange(callback)
	local curNo = model.UserModel:getHeadIcon();
	if self.no ~= curNo then
		MessageBox:create(LANG.UI_USER_3,MessageBoxType.CANCEL_OK,function (  )
			local msg = {};
			msg.no = self.no;
			Network.NetworkManager:Send(Network.NetworkCmdType.req_user_set_head, msg);

			if callback then
				callback();
			end
		end, function()
			if callback then
				callback();
			end
		end)
		return true;
	end

	return false;
end

function HeadSelectPanel:onLeave()
	self.root:setVisible(false);
end
return HeadSelectPanel;
