local HelpPanel = class("HelpPanel" , require('app.layer.BaseLayer'))
local HelpShow = Enum:getTable('HelpShow');
local fonts = Enum:getTable("SystemFont")
local imagePath = 'image/help/'
function HelpPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'HelpPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function HelpPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels();
end

function HelpPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_c9_1_2.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(18,50,123,73));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(579 , 744));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , 7);

	local bg = ccui.Scale9Sprite:create("image/commonImg/bg_c9_2_3.png")
		:setCapInsets(CCRectMake(10,10,117,94))
		:setContentSize(cc.size(546 , 595))
		:setPosition(cc.p(0 , 25))
		:addTo(self.root)

	self.helpListView = ccui.ListView:create()
		:ignoreContentAdaptWithSize(false)
		:setAnchorPoint(cc.p(0.5,0.5))
		:setBounceEnabled(true)
		:setDirection(ccui.ScrollViewDir.vertical)
		:setContentSize(cc.size(540 , 580))
		:setScrollBarEnabled(false)
		:setPosition(cc.p(0 , 25))
		:addTo(self.root)

	local btnLabel = cc.Label:createWithTTF(LANG.COMMON_OK , fonts.FONT_SIYUAN , 20);
	btnLabel:enableOutline(cc.c4b(50 , 85 , 150 , 255) , 2);

	self.okBtn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	self.okBtn:addTo(self.root);
	self.okBtn:setPosition(0 , -320);
	self.okBtn:addClickEventListener(function() self:onClickBack() end);
	self.okBtn:setTitleLabel(btnLabel);
end

function HelpPanel:onEnter(curPanelName)
	self:addSwallowCallback(handler(self , self.onClickBack));
	local helpData = HelpShow[curPanelName];
	if not helpData then
		UIManager:pop('HelpPanel');
	end

	local lg = model.UserDefaultModel:getLanguageSet();
	local tableName = 'language_help_'..lg;

	self.helpListView:removeAllChildren();
	local info = nil
	if helpData then
		info = ResTable:getValue(tableName,helpData[1],'info');
	end
	if info then

		local formatStr , colorPos = formatColorText(info);
		local info = cc.Label:createWithTTF(tostring(formatStr) , Enum:getTable("SystemFont").FONT_HUAKANG , 18);
		info:setLineBreakWithoutSpace(true);
		info:setMaxLineWidth(540)

		local strLen = getStrLen(formatStr);

		for i = 0 , strLen-1 do
			local label = info:getLetter(i);
			if label then
				local changeColor = false
				for _k , pos in pairs(colorPos) do
					if i >= pos.sPos-1 and i <= pos.ePos-1 then
						label:setColor(cc.c3b(pos.color[1] , pos.color[2] , pos.color[3]));
						changeColor = true;
						break;
					end
				end
				if not changeColor then
					label:setColor(cc.c3b(255 , 255 , 255));
				end
			end
		end

		local size = info:getContentSize();
		local layout = ccui.Layout:create();
		layout:setContentSize(size);
		info:addTo(layout);
		info:setPosition(size.width/2 , size.height/2);

		self.helpListView:pushBackCustomItem(layout);
	end
	
	self:updateZOrder(curPanelName);

	self.root:setVisible(true);
end

function HelpPanel:updateZOrder(curPanelName)
	local panelZOder = UIManager:getPanelLocalZOrder(curPanelName) or 1;
	panelZOder = panelZOder * PANEL_ZORDER + 30;
	self.root:setLocalZOrder(panelZOder);
end

function HelpPanel:onClickBack()
	UIManager:pop('HelpPanel');
end

function HelpPanel:onLeave()
	self.root:setVisible(false);
end

return HelpPanel;
