local ItemProbabilityPanel = class("ItemProbabilityPanel", require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

function ItemProbabilityPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'ItemProbabilityPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function ItemProbabilityPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels();
end

function ItemProbabilityPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(526,498));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , -22);

	local titleNode =  GlobalFunctions:createTitle(LANG.UI_L_GACHA_PROB_TITLE_1 , 526 , 46 , LANG.UI_L_GACHA_PROB_TITLE_2);
	titleNode:addTo(self.root);
	titleNode:setPosition(0 , 249);

	local infoBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	infoBg:addTo(self.root);
	infoBg:setPosition(0 , 0);
    infoBg:setCapInsets(cc.rect(11,11,34,28));
	infoBg:setContentSize(cc.size(502 , 420));

	local initData = {
        width = 502,
        height = 410,
        cellWidth = 502 ,
        cellHeight = 110,
		cellReloadFunc = handler(self,self.reloadCellPanel),
	}
	
    self.cardListView = template.TableViewTemplate:create()
    self.cardListView:initWithSizeAndItem(initData) ;
    self.cardListView:getView():setPosition(cc.p(-251, -205)); -- 0,-149
	self.cardListView:getView():addTo(self.root)
end


function ItemProbabilityPanel:onEnter(box_id)
	self:addSwallowCallback(handler(self, self.onClickBack));

	local drop_id = ResTable:getValue('box', box_id, 'drop_id');

	self.itemList = {};

	local boxDrop = ResTable:getTable('box_drop').contain;
	if boxDrop then
		for _k , data in pairs(boxDrop) do
			if data.drop_id == drop_id and data.rate > 0 and data.reward and #data.reward>0 then
				table.insert(self.itemList , {no = data.reward[1][1] , num = data.reward[1][2], rate = data.rate});
			end
		end
	end

	self.roleMaxNum = #self.itemList;
	local groupNum = math.ceil(self.roleMaxNum / 5);
	self.cardListView:setDataCount(groupNum);
	self.cardListView:reloadData();

	self.root:setVisible(true);
end

function ItemProbabilityPanel:reloadCellPanel(cell , idx)
	local startNum = idx * 5 + 1;
	local curMaxNum = (idx+1) * 5;
	curMaxNum = math.min(curMaxNum , self.roleMaxNum);

	local children = cell:getChildren();

	if not children or #children <= 0 then
		for i = 1 , 5 do
			local x = 61 + 95 * (i - 1);
			local y = 60;
			local iconTemplate = template.ItemTemplate:create(2);
			iconTemplate:setPosition(cc.p(x , y));
			iconTemplate:addTo(cell);
			iconTemplate:setScale(0.8);
			if startNum <= curMaxNum then
				iconTemplate:Init(self.itemList[startNum].no, self.itemList[startNum].num);
				iconTemplate:setGachaProbability(tostring(self.itemList[startNum].rate/100)..'%', true);
				startNum = startNum + 1;
			else
				iconTemplate:setVisible(false);
			end
		end
	else
		for i = 1 , 5 do
			if startNum <= curMaxNum then
				children[i]:setVisible(true);
				children[i]:Init(self.itemList[startNum].no, self.itemList[startNum].num);
				children[i]:setGachaProbability(tostring(self.itemList[startNum].rate/100)..'%', true);

				startNum = startNum + 1;
			else
				children[i]:setVisible(false);
			end
		end
	end

	return cell;
end

function ItemProbabilityPanel:onClickBack()
	UIManager:pop('ItemProbabilityPanel');
end

function ItemProbabilityPanel:onLeave()
	self.root:setVisible(false);
end

return ItemProbabilityPanel;
