local LoadingPanel = class("Loading")
local scheduler = cc.Director:getInstance():getScheduler();
local fonts = Enum:getTable("SystemFont")

function LoadingPanel:ctor()
	self.isPopup = true;
end

function LoadingPanel:Init(root, scene)
	self.root = root;
	self.scene = scene;
	self.root:setVisible(false);

	self.root:setLocalZOrder(10000);

	local stageSize = cc.Director:getInstance():getVisibleSize();


	self.leftPart = cc.Sprite:create('image/loading/loading_bg1.png');
	self.leftPart:setPosition(-self.leftPart:getContentSize().width/2, self.leftPart:getContentSize().height/2);
	self.root:addChild(self.leftPart);
	self.rightPart = cc.Sprite:create('image/loading/loading_bg2.png');
	self.rightPart:setPosition(stageSize.width + self.rightPart:getContentSize().width/2, self.leftPart:getContentSize().height/2);
	self.root:addChild(self.rightPart);

	self.bottomNode = cc.Node:create();
	self.bottomNode:setPosition(stageSize.width/2, 100);
	self.bottomNode:setVisible(false);
	self.root:addChild(self.bottomNode);

	self.progressBarBg1 = ccui.Scale9Sprite:create("image/loading/loading_bar_bg2.png");
	self.progressBarBg1:setCapInsets(CCRectMake(1,1,10,88));
	self.progressBarBg1:setContentSize(cc.size(stageSize.width, 100)); 
    self.progressBarBg1:setPosition(cc.p(0,0));
	self.bottomNode:addChild(self.progressBarBg1);

	self.progressBarBg2 = cc.Sprite:create("image/loading/loading_bar_bg.png");
    self.progressBarBg2:setPosition(cc.p(0,0));
	self.bottomNode:addChild(self.progressBarBg2);

	self.progressBar = cc.ProgressTimer:create(cc.Sprite:create("image/loading/loading_bar_fg.png"));
    self.progressBar:setType(cc.PROGRESS_TIMER_TYPE_BAR);
    self.progressBar:setMidpoint(cc.p(0, 0));
    self.progressBar:setBarChangeRate(cc.p(1, 0));
    self.progressBar:setPosition(0,3);
	self.bottomNode:addChild(self.progressBar);

	self.progressLabel = cc.Label:createWithTTF("0", fonts.FONT_SIYUAN, 30);
	self.progressLabel:setPosition(cc.p(300, -17));
	self.progressLabel:setAnchorPoint(cc.p(1, 1));	
	self.bottomNode:addChild(self.progressLabel);

	self.progressLabelWord = cc.Label:createWithTTF(LANG.UI_LOADING_1,fonts.FONT_SIYUAN, 22); --
	self.progressLabelWord:setPosition(cc.p(0, -30));
	self.bottomNode:addChild(self.progressLabelWord);
	
	self.effectNode = cc.Node:create();
	self.effectNode:setPosition(-586/2, 25);
	self.effectNode:setScale(0.2);
	self.bottomNode:addChild(self.effectNode);
	self.effect = DragonBonesManager:GetUIEffect('A_UI_Loading_004');
	self.effectNode:addChild(self.effect);

	self.speed = 25;
	self.progressPhase1 = 30;
	self.barWidth = 586;
	self.effectNodeSpeed = self.barWidth / (100/self.speed);
end

--=================================================
function LoadingPanel:onEnter()
	self:onPhase1();
end

function LoadingPanel:onPhase1()
	self.phase = 1;
	local stageSize = cc.Director:getInstance():getVisibleSize();
	self.leftPart:setPosition(-self.leftPart:getContentSize().width/2, self.leftPart:getContentSize().height/2);
	self.rightPart:setPosition(stageSize.width + self.rightPart:getContentSize().width/2, self.leftPart:getContentSize().height/2);

	self.root:setVisible(true);
	local actionLeft = cc.MoveBy:create(0.2, cc.p(self.leftPart:getContentSize().width, 0));
	local actionRight = cc.MoveBy:create(0.2, cc.p(-self.rightPart:getContentSize().width, 0));
	
	self.leftPart:runAction(actionLeft);
	self.rightPart:runAction(actionRight);

	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.update2), 0.25, false);
end

function LoadingPanel:onPhase2()
	self.phase = 2;
	self.progressBar:setPercentage(0);
	self.progressLabel:setString('0%');
	self.effectNode:setPosition(-586/2, 25);
	self.bottomNode:setVisible(true);
	local time = (100/self.speed)*(self.progressPhase1/100);
	self.progressBar:runAction(cc.Sequence:create(cc.ProgressTo:create(time, self.progressPhase1), cc.CallFunc:create(handler(self, self.onPhase3))));
	self.effect:getAnimation():play('play');
	self.effectNode:runAction(cc.MoveBy:create(time, cc.p(self.barWidth*self.progressPhase1/100, 0)))
end

function LoadingPanel:onPhase3()
	local time = (100/self.speed)*(1-self.progressPhase1/100);
	self.progressBar:runAction(cc.Sequence:create(cc.ProgressTo:create(time, 100), cc.CallFunc:create(function() self.phase = 3; end)));
	self.effectNode:runAction(cc.MoveBy:create(time, cc.p(self.barWidth*(1-self.progressPhase1/100), 0)))
end

function LoadingPanel:onPhase4()
	self.phase = 4;
	self.effect:getAnimation():stop('play');
	self.bottomNode:setVisible(false);
	local actionLeft = cc.MoveBy:create(0.2, cc.p(-self.leftPart:getContentSize().width, 0));
	local actionRight = cc.MoveBy:create(0.2, cc.p(self.rightPart:getContentSize().width, 0));
	
	self.leftPart:runAction(cc.Sequence:create(actionLeft, cc.CallFunc:create(handler(self, self.onPhase5))));
	self.rightPart:runAction(actionRight);
end

function LoadingPanel:onPhase5()
	self.phase = 5;
end

function LoadingPanel:onLeave()
	self.root:setVisible(false);
end

function LoadingPanel:update( dt )
	if self.phase == 2 or self.phase == 3 then
		self.progressLabel:setString(string.format("%d", math.floor(self.progressBar:getPercentage()))..'%');
	end
end

function LoadingPanel:update2()
	self:onPhase2();
	scheduler:unscheduleScriptEntry(self.m_scheduleHandler);
end

return LoadingPanel;
