local PlotPanel = class("PlotPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/plot/'
local bgPath = 'image/plot/'
local fonts = Enum:getTable("SystemFont")
local scheduler = cc.Director:getInstance():getScheduler();

function PlotPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'PlotPanel');

	self.useCCreator = false;
	self.haveSwallowLayer = true;

	self.posX = 0;
	self.posY = 0;
end

function PlotPanel:Init(root)
	self.root:setVisible(false);

	self:initPanels();
	self:registEvent();
end

function PlotPanel:initPanels()

	self.bgStr = 'BG_polt_1';
	self.bg = cc.Sprite:create(imgPath .. 'BG_polt_1.png')
		:addTo(self.root)

	self.picSp = cc.Sprite:create()
		:addTo(self.root)

	self.skeletonNode = cc.Node:create()
		:addTo(self.root)

	self.sp2 = cc.Sprite:create(imgPath .. 'sp2.png')
		:setPosition(0, -StageSize.height/2+160)
		:addTo(self.root)

	self.nameLabel = cc.Label:createWithTTF('name', fonts.FONT_SIYUAN, 30)
		:setTextColor(cc.c3b(255, 255, 255))
		:setAnchorPoint(cc.p(0, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setPosition(-StageSize.width/2+105, -StageSize.height/2+298)
		:enableOutline(cc.c3b(0, 0, 0), 2)
		:addTo(self.root)

	self.sp1 = cc.Sprite:create(imgPath .. 'sp1.png')
		:setAnchorPoint(cc.p(0, 0.5))
		:setPosition(-StageSize.width/2+60, -StageSize.height/2+268)
		:addTo(self.root)

	self.skipBtn = ccui.Button:create(imgPath .. 'btn.png')
	self.skipBtn:setPosition(StageSize.width/2-65, StageSize.height/2-65)
	self.skipBtn:addClickEventListener(handler(self, self.clickSkip))
	self.skipBtn:addTo(self.root)

	self.skipSp = cc.Sprite:create(imgPath .. 'skip.png')
		:setPosition(StageSize.width/2-65, StageSize.height/2-65)
		:addTo(self.root)		

	local paddingH, paddingV = 60, 0
	local size = cc.size(StageSize.width, 240)
	self.text = ccui.Text:create('', fonts.FONT_SIYUAN, 30)
	self.text:setTextAreaSize(cc.size(size.width - paddingH * 2, size.height - paddingV * 2))
	self.text:setTextVerticalAlignment(cc.VERTICAL_TEXT_ALIGNMENT_TOP)
	self.text:setTextHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
	self.text:setAnchorPoint(0.5, 0)
	self.text:enableOutline(cc.c3b(0, 0, 0), 2)
	self.text:setPosition(0, -StageSize.height/2)
	self.text:addTo(self.root)
	self.text:setTouchEnabled(true)

	self.labelBlink = cc.Label:createWithTTF('|', fonts.FONT_SIYUAN, 30)
		:setPosition(258, -StageSize.height/2+155)
		:setVisible(false)
		:addTo(self.root)
end

function PlotPanel:registEvent()
end

function PlotPanel:onEnter(plot_type, id, option, blink)
	-- 0 role plot
	-- 1 new player plot
	-- 2 round plot
	-- 3 love
	-- others?
	self.option_love = nil 
	self.dialog_index = 1 -- 剧情顺序

	bgPath = 'image/plot/'
	self.plot_type = plot_type or 0;
	AudioManager:SetState(Enum:getTable('GameState').Plot);

	local plot_id, dialogData;
	if self.plot_type == 0 then
		plot_id = id;
		dialogData = ResTable:getTable('dialog_role');
	elseif self.plot_type == 1 then
		plot_id = id;
		dialogData = ResTable:getTable('dialog_0')
	elseif self.plot_type == 2 then
		local round_id = model.RoundModel:getRoundIndex();
		local data = ResTable:getValue('round', round_id, 'task_id');
		if data[option] > 0 then
			plot_id = data[option];
		else
			EventManager:FireEvent(EventType.PlotFinish);
			return;
		end
		local chapter = model.RoundModel:getChapter()
		dialogData = ResTable:getTable('dialog_'..chapter);
	elseif self.plot_type == 3 then
		-- option = {
		-- 	love_lv = roleLove.love_lv,
		-- 	role_id = roleLove.role_id
		-- }
		self.option_love = option;
		plot_id = id;
		dialogData = ResTable:getTable('dialog_love');
		bgPath = 'image/cardLove/'
	else
		return;
	end

	-- init current plot
	local isEmpty = true;
	self.plotList = {};
	self.currentID = nil;
	for _, data in pairs(dialogData.contain) do
		if data['task_id'] == plot_id then
			local id = data['no'];
			self.plotList[id] = data;
			isEmpty = false;
			if not self.currentID then
				self.currentID = id;
			elseif id < self.currentID then
				self.currentID = id;
			end
		end
	end
	
	if self.plot_type == 0 and isEmpty then
		return;
	end

	-- init flag
	self.playingDialogue = false;

	self.showRole = 0;

	-- show panel
	self.root:setVisible(true)

	-- init dialogure
	if self.m_scheduleHandler == nil  then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.updateFrameDialog), 0, false);
	end

	if blink == true then
		local whiteSpr = cc.Scale9Sprite:create('image/blood/nblood_white.png')
		:addTo(self.root)
		:setContentSize(cc.size(StageSize.width, StageSize.height))
		local callfunc = cc.CallFunc:create(function() 
			
		end)
		whiteSpr:runAction( cc.Sequence:create(cc.FadeOut:create(1), callfunc ,cc.RemoveSelf:create()) )
	end
	
	if self:initDialogue() then
		self:playDialogue()
	else
		self:plotFinish()
	end

	EventManager:FireEvent(EventType.LoadingGoOn);
	self:addSwallowCallback(handler(self, self.clickDialogue))
end

function PlotPanel:onLeave()
	self.playingDialogue = false;
	self.text:stopAllActions()
	self.root:setVisible(false)
	self.skeletonNode:removeAllChildren();
	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
	if self.plot_type == 2 then
		AudioManager:SetState(Enum:getTable('GameState').Fight);
	elseif self.plot_type == 3 then
		AudioManager:SetState(Enum:getTable('GameState').Main);
		AudioManager:playMusic(Conf.main, 2, true);
	end
end

-- 初始化对话
function PlotPanel:initDialogue()
	if not self.currentID then
		return false;
	end
	local currentDialog = self.plotList[self.currentID];
	if not currentDialog then
		return false;
	end

	--bgm
	if currentDialog.bgm and currentDialog.bgm ~= '' then
		AudioManager:playMusic(Conf[tostring(currentDialog.bgm)], 4, true);
	end
	-- dialog_love cv 
	if self.plot_type == 3 then
		if self.option_love == nil then 
			self.option_love = {} 
		end 
		
		local roleId = self.option_love.role_id ;
		local love_lv = self.option_love.love_lv 
		AudioManager:playDialogLoveCV(roleId,love_lv,self.dialog_index)
	end
	-- bg 
	if self.bgStr ~= currentDialog.background then
		self.bgStr = currentDialog.background;
		self.bg:setTexture(bgPath..self.bgStr..".png");
	end

	-- name
	local name = '';
	currentDialog.whosay = tonumber(currentDialog.whosay);
	if currentDialog.whosay ~= -1 and currentDialog.whosay ~= 0 then 
		if currentDialog.whosay == 991 then
			name = tostring(model.UserModel:getUserData().name);
		else
			name = ResTable:getValue('npc', currentDialog.whosay, 'name');
		end
	end
	self.nameLabel:setString(name);

	-- show role
	local aimShowOne;
	for _, v in pairs (currentDialog.npc) do
		if v >= 0 then
			aimShowOne = v;
			break;
		end
	end
	if aimShowOne and aimShowOne > 100 and aimShowOne < 200 then
		aimShowOne = aimShowOne - 100
	end
	if aimShowOne then
		if aimShowOne == 0 then
		elseif aimShowOne ~= self.showRole then
			local data = ResTable:getRowData('npc', aimShowOne);
			self.showRole = aimShowOne;
			local spine_res = data['live2d'];
			if spine_res ~= nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
				self.skeletonNode:removeAllChildren();
				local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1);
				skAnimation:addTo(self.skeletonNode)		
				skAnimation:setAnimation(0, "idle", true)
				self.picSp:setVisible(false);
			else
				self.picSp:setVisible(true);
				self.picSp:setTexture('image/role/'..data['navi']..'.png');
			end
		end
	end
	return true
end

-- 播放对话
function PlotPanel:playDialogue()
	local currentDialog = self.plotList[self.currentID];
	self.playingDialogue = true
	self.text:setString(" ")
	-- self.text:runAction(self:createTextActionSequence(self.text))

	self:clickDialogue();

	-- self:createTextAction()
end

function PlotPanel:nextDialog()
	local currentDialog = self.plotList[self.currentID];
	if not currentDialog then
		self:plotFinish()
		return;
	end
	if currentDialog['type'] == 1 then
		self.currentID = nil;
	elseif currentDialog['type'] == 0 then
		self.currentID = currentDialog.follow[1];
	else
		self.currentID = nil;	
	end
	self.dialog_index = self.dialog_index + 1
	
	self.labelBlink:stopAllActions()
	self.labelBlink:setVisible(false)

	if self:initDialogue() then
		self:playDialogue()
	else
		self:plotFinish()
	end
end

function PlotPanel:plotFinish()
	if self.plot_type == 1 then
		App:run("LoginScene");
	elseif self.plot_type == 2 then
		EventManager:FireEvent(EventType.PlotFinish);
	else
		UIManager:pop('PlotPanel');
	end
end

function PlotPanel:clickDialogue()
	if self.playingDialogue then
		-- self.text:stopAllActions()
		local currentDialog = self.plotList[self.currentID]
		local str = currentDialog.plotinfo
		str = string.gsub(str, "\\n", "\n")
		str = string.gsub(str, "\\t", "\t")
		self.text:setString(str)
		self:dialogureEndFunc()
	else
		self:nextDialog();
	end
end

function PlotPanel:clickSkip()
	self:plotFinish()
end

function PlotPanel:updateFrameDialog( dt )
	-- body
	if self.playingDialogue==false then
		return;
	else
		if self.delayTime == 0 then
			local current = self:nextUtf8strIndex(self.dialogStr, self.showDigPos)

			self.text:setString( string.sub(self.dialogStr, 1, current - 1) )

			self.showDigPos = current;

			if self.showDigPos >= self.diloglength then
				self:dialogureEndFunc();
			end

		end

		self.delayTime = self.delayTime + dt;
		if self.delayTime >= self.pTime then
			self.delayTime = 0;
		end
	end

end

function PlotPanel:createTextAction( time )
	local currentDialog = self.plotList[self.currentID];
	self.dialogStr = currentDialog.plotinfo;
	self.diloglength = string.len(self.dialogStr);
	self.pTime = time or 0.15;
	self.showDigPos = 1;
	self.delayTime = 0;

end

-- 打字机效果
function PlotPanel:createTextActionSequence(target, time)
	local currentDialog = self.plotList[self.currentID];

	time = time or 0.15
	local str = currentDialog.plotinfo
	
	str = string.gsub(str, '\\n', '\n');
	str = string.gsub(str, '\\t', '\t');

	local actions = {}
	local length = string.len(str)
	local index = 1

	-- 一个一个蹦字
	while index <= length do
		local current = self:nextUtf8strIndex(str, index)
		table.insert(actions, cc.DelayTime:create(time))
		table.insert(actions, cc.CallFunc:create(
			function()
				target:setString(string.sub(str, 1, current - 1))
			end
			))
		index = current
	end
	table.insert(actions, cc.CallFunc:create(handler(self, self.dialogureEndFunc)))

	return cc.Sequence:create(actions)
end

function PlotPanel:dialogureEndFunc()
	self.playingDialogue = false
	self.labelBlink:setVisible(true)
	self.labelBlink:runAction(cc.RepeatForever:create(cc.Sequence:create(
		cc.DelayTime:create(0.4),
		cc.FadeIn:create(0.1),
		cc.DelayTime:create(0.4),
		cc.FadeOut:create(0.1)
		)))
end

function PlotPanel:nextUtf8strIndex(str, current)
	-- 汉字
	if string.byte(str, current) > 127 then
		return current + 3
	-- 字母
	elseif string.byte(str, current) < 128 then
		return current + 1
	end
end

function PlotPanel:onClickBack()
	self:plotFinish();
end

return PlotPanel;
