local RewardPanel = class("RewardPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/reward/';
local fonts = Enum:getTable("SystemFont");
local RewardType = Enum:getTable('RewardType');

local ACT_STEP = {
	STEP_NULL 			= -1,
	STEP_SHOW_ITEM 		= 1,
	STEP_SHOW_PARTNER 	= 2,
	STEP_SHOW_BOX	 	= 3,
	STEP_END 			= 10,
}

function RewardPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'RewardPanel');

	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;
	self.isPopup = true;

	self.callback = nil;

end

function RewardPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels();
	self:initPartnerPanel();
	self:initBoxPanel();

	EventManager:RegisterEvent(EventType.NetworkRetOpenBox,handler(self,self.retOpenBox));
end

function RewardPanel:initPanels()

	--self.particleWin = cc.ParticleSystemQuad:create("Effect/particle/winp.plist")
	--self.particleWin:addTo(self.root);
	--self.particleWin:setPosition(cc.p(0,170))
	--self.particleWin:stopSystem();

	local spine_res = 'RewardPanelEffect'
	self.effect = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
   	self.effect:setPosition(0 , 32);
	self.effect:addTo(self.root);

	self.infoNode = cc.Node:create()
		:setCascadeOpacityEnabled(true)
		:addTo(self.root);

	self.headIcon = template.ItemTemplate:create();
	self.headIcon:addTo(self.infoNode);
	self.headIcon:setPosition(-193, 117);

	self.rewardLabel = cc.Label:createWithTTF('LV 999', fonts.FONT_SIYUAN, 20)
		:setPosition(-127, 125)
		:setAnchorPoint(0, 0.5)
		:addTo(self.infoNode)

	self.rewardLevel = cc.Label:createWithTTF('LV 999', fonts.FONT_SIYUAN, 20)
		:setPosition(-127, 100)
		:setAnchorPoint(0, 0.5)
		:addTo(self.infoNode)

	local expBg = cc.Sprite:create(imgPath..'exp_bg.png');
	expBg:addTo(self.infoNode);
	expBg:setPosition(57 , 77);

	local expBar = cc.Sprite:create(imgPath .. 'progress_fg.png');
	expBar:setContentSize(cc.size(373, 6));
	self.expProgress = cc.ProgressTimer:create(expBar)
		:setAnchorPoint(cc.p(0, 0.5))
		:setPosition(-130, 77)
		:setMidpoint(cc.p(0, 0))
		:setBarChangeRate(cc.p(1, 0))
		:setType(cc.PROGRESS_TIMER_TYPE_BAR)
		:setPercentage(30)
		:addTo(self.infoNode)


	self.rewardNode = cc.Node:create();
	self.rewardNode:addTo(self.root);
	self.rewardNode:setPosition(0, -24);

	local rewardBg = cc.Sprite:create(imgPath..'item_bg.png');
	rewardBg:addTo(self.rewardNode);

	self.rewardList = ccui.ScrollView:create();
	self.rewardList:setDirection(ccui.ScrollViewDir.horizontal)
					:setContentSize(cc.size(497, 118))
					:setScrollBarEnabled(false)
					:addTo(self.rewardNode)
					:setAnchorPoint(cc.p(0.5,0.5))
					:setPosition(cc.p(0,0))
					:setCascadeOpacityEnabled(true)							
end

function RewardPanel:initPartnerPanel()
	self.newPartnerNode = cc.Node:create();
	self.newPartnerNode:addTo(self.root);
	self.newPartnerNode:setLocalZOrder(10);

	local NewPartnerPanel = require('app/layer/MainScene/GachaResultPanel');
	self.newPartnerPanel = NewPartnerPanel:create()
	self.newPartnerPanel:Init(self.newPartnerNode);
end

function RewardPanel:initBoxPanel()
	self.boxNode = cc.Node:create()
	:addTo(self.root)
	
	self.boxCountdown = cc.Label:createWithTTF('',fonts.FONT_SIYUAN,20)
	:addTo(self.boxNode)
	:setPosition(0,-125)
	:setTextColor(cc.c3b(120,170,200))

	local boxTip = cc.Sprite:create('image/commonImg/tip_2.png')
	:addTo(self.boxNode)

	local tipLabel = cc.Label:createWithTTF(LANG.UI_BOX_2 , fonts.FONT_SIYUAN , 20)
	:addTo(self.boxNode)
	:setTextColor(cc.c3b(255 , 200 , 0))

	local width = boxTip:getContentSize().width+10;
	local labelWidth = tipLabel:getContentSize().width;
	local posX = -labelWidth / 2;
	boxTip:setPosition(posX , -163);
	tipLabel:setPosition(posX + width/2+labelWidth/2 , -163);
end

function RewardPanel:onEnter(msg , option)	

	self.root:setVisible(true);
	self:addSwallowCallback(handler(self, self.onClickBack));

	self.isLevelUp = false
	self.levelUpData = nil ;
	
	self.rewardType = nil ;
	self.callback = nil;
	self.effectName = nil;
	self.incomeTime = nil;
	if option ~= nil then
		self.rewardType = option.rewardType;
		self.callback = option.callback;
		self.effectName = option.effectName;
		self.incomeTime = option.incomeTime
	end
	self:onShow(msg)
end

function RewardPanel:onShow(msg)
	self.actStep = ACT_STEP.STEP_NULL;
	self.boxs = {};

	self.boxNode:stopAllActions()
	self.isReqBoxReward = false;
	TimeManager:registUpdateEvent('RewardPanel',nil,true);

	local index = 1;
	local itemSize = cc.size(114,118)
	local itemList = {}

	self.boxNode:setVisible(false);
	self.newPartnerPanel:onClickBack();
	self.rewardList:removeAllChildren();

	local money = msg.money 
	if money ~= nil and  money.change and string.find(money.change , '-') == nil then
		if itemList then
			self.actStep = ACT_STEP.STEP_SHOW_ITEM;
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(MONEY_ID, money.change);
			index = index + 1;
		end
	end

	local exp = msg.userExp;
	if exp ~= nil and exp.change then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		local item = require('app.template.ItemTemplate'):create(2);
		item:addTo(self.rewardList)
		item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
		item:setCanShowInfo(true);
		item:Init(EXP_ID, exp.change);
		index = index + 1;
	end

	if msg.userExp and msg.userExp.lv_up  then --and msg.userExp.lv_up.explain
		if not self.isLevelUp then
			self.levelUpData =msg.userExp.lv_up;
		else
			self.levelUpData.change = self.levelUpData.change + msg.userExp.lv_up.change;
			for i = 1 , #msg.userExp.lv_up.reward do
				table.insert(self.levelUpData.reward,msg.userExp.lv_up.reward[i]);
			end
		end
		self.isLevelUp = true
	end
	

	itemList = msg.itemChange
	if itemList then
		for k, v in ipairs(itemList) do
			if tonumber(v.addNum) > 0  then
				self.actStep = ACT_STEP.STEP_SHOW_ITEM;
				local item = require('app.template.ItemTemplate'):create(2);
				item:addTo(self.rewardList)
				item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
				item:setCanShowInfo(true);
				item:Init(v.no, v.addNum);
				index = index + 1;
			end
			--消耗道具，消耗金币...不做显示
		end
	end

	itemList = msg.newPartner
	if itemList  then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(ROLE_TYPE,v.no);
			index = index + 1;
		end
	end


	itemList = msg.equipChange
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			if tonumber(v.addNum) > 0 then
				local item = require('app.template.ItemTemplate'):create(2);
				item:addTo(self.rewardList)
				item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
				item:setCanShowInfo(true);
				item:Init(v.no,tonumber(v.addNum));
				index = index + 1;
			end
		end
	end

	itemList = msg.newAvatar
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(ROLE_AVATAR_TYPE , v.no);
			index = index + 1;
		end
	end

	itemList = msg.dailyMissiontItem
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			if tonumber(v) > 0 then
				local item = require('app.template.ItemTemplate'):create(2);
				item:addTo(self.rewardList)
				item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
				item:setCanShowInfo(true);
				item:Init(DAILY_MISSION_ITEM , v);
				index = index + 1;
			end
		end
	end

	itemList = msg.battlepassPoint
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			if tonumber(v) > 0 then
				local item = require('app.template.ItemTemplate'):create(2);
				item:addTo(self.rewardList)
				item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
				item:setCanShowInfo(true);
				item:Init(BATTLEPASS_POINT , v);
				index = index + 1;
			end
		end
	end

	itemList = msg.BgActivate
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(v.no , v.addNum or 1);
			index = index + 1;
		end
	end

	--[[
	 	客户端自定义显示内容！！ 
		msg.tempReward = {
			{10001,100},{...}
		}
	]]
	itemList = msg.tempReward
	if itemList then
		self.actStep = ACT_STEP.STEP_SHOW_ITEM;
		for _k , v  in pairs(itemList) do
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(v[1],v[2]);
			index = index + 1;
		end
	end

	if msg.box and #msg.box>0 then
		self.boxs = msg.box
		if self.actStep == ACT_STEP.STEP_NULL then
			self:showBoxNode()
		end
	end
	if self.actStep == ACT_STEP.STEP_NULL then
		UIManager:pop('RewardPanel');
		if self.callback then --主要用于 迷宫功能里
			self.callback();
		end
		return;
	end
	if self.actStep == ACT_STEP.STEP_SHOW_ITEM then
		local maxWidth = (index-1)*itemSize.width
		if maxWidth < 497 then
			maxWidth = 497
		end
		self.rewardList:setInnerContainerSize(cc.size(maxWidth,118))

		self.effect:setAnimation(0, GlobalFunctions:lAnimationName((self.effectName or 'RewardPanelEffect')) , false)
		AudioManager:playEffect(Conf.getReward2);

		self.root:runAction(cc.Sequence:create(
		cc.FadeIn:create(0.5),
		cc.CallFunc:create(function()
			if msg.newPartner then
				self:showNewPartner(msg);
			elseif #self.boxs>0 then
				self:showBoxNode()
			else
				self.actStep = ACT_STEP.STEP_END
			end
		end)));
	end

	self.rewardNode:setOpacity(0);
	self.rewardNode:runAction(cc.FadeIn:create(0.3))

	if self.rewardType == RewardType.REWARD_TYPE_COMMON then
		self.infoNode:setVisible(false);
		self.rewardNode:setPosition(0 , 35);
	else
		self.infoNode:setVisible(true);
		self.infoNode:setOpacity(0);
		self.infoNode:runAction(cc.FadeIn:create(0.3));

		self.rewardNode:setPosition(0 , -24);

		local percent = math.floor(model.UserModel:getExpPercent());
		self.expProgress:setPercentage(percent);
		
		self.rewardLabel:setVisible(false);
		if self.rewardType == RewardType.REWARD_TYPE_FAST_FIGHT then
			self.rewardLabel:setVisible(true);
			self.rewardLabel:setString(LANG.UI_REWARD_2)
		elseif self.rewardType == RewardType.REWARD_TYPE_FIGHT_INCOME then
			self.rewardLabel:setVisible(true);
			self.rewardLabel:setString(string.format(LANG.UI_REWARD_3, tostring(Sec2Time(self.incomeTime or 0))))
		end
		self.rewardLevel:setString(string.format('Lv %s', tostring(model.UserModel:getUserLevel())));

		local no = model.UserModel:getHeadIcon() or 1;

		local iconData = GlobalFunctions:getHeadIconData(no)
		if iconData.quality then
			self.headIcon:Init(iconData.roleType, iconData.no,iconData.starLv,nil,nil,iconData.quality);
		else
			self.headIcon:Init(iconData.roleType, iconData.no);
		end
	end
end

function RewardPanel:showBoxNode()
	if self.boxs then
		self.actStep = ACT_STEP.STEP_SHOW_BOX;
		local itemSize = cc.size(114,118)
		local index = 0;
		local tBoxs = {};
		for _k , v in ipairs(self.boxs) do
			if not tBoxs[v.no] then
				tBoxs[v.no] = 1;
			else
				tBoxs[v.no] = tBoxs[v.no] + 1;
			end
		end
		self.rewardList:removeAllChildren();
		for no , num in pairs(tBoxs) do
			index = index + 1;
			local item = require('app.template.ItemTemplate'):create(2);
			item:addTo(self.rewardList)
			item:setPosition(index*itemSize.width - itemSize.width/2, itemSize.height/2);
			item:setCanShowInfo(true);
			item:Init(no,num);
		end

		local maxWidth = index*itemSize.width
		if maxWidth < 497 then
			maxWidth = 497
		end
		self.rewardList:setInnerContainerSize(cc.size(maxWidth,118))

		self.effect:setAnimation(0, GlobalFunctions:lAnimationName('RewardPanelEffect') , false)
		AudioManager:playEffect(Conf.getReward2);
		
		local countdown = 4;
		TimeManager:registUpdateEvent('RewardPanel',function()
			countdown = countdown - 1;
			self.boxCountdown:setString(string.format(LANG.UI_BOX_1 , countdown));
			if countdown == 0 then
				self:reqBoxReward()
				TimeManager:registUpdateEvent('RewardPanel',nil,true);
			end
		end,true);

		self.infoNode:setVisible(false);
		self.rewardNode:setPosition(0 , 35);
		
		self.boxNode:setVisible(true);
	end
end

function RewardPanel:showNewPartner(msg)
	if msg.newPartner then
		self.actStep = ACT_STEP.STEP_SHOW_PARTNER;
		local gachaResults = {};
		for _k , newPartner in pairs(msg.newPartner) do
			local data = {
				partner = {
					no = newPartner.no,
					num = 1,
				}
			};
			table.insert(gachaResults,data);
		end
		self.newPartnerPanel:onEnter({gachaResults = gachaResults,callback = function() 
			if self.boxs and #self.boxs>0 then
				self:showBoxNode();
			else
				self.actStep = ACT_STEP.STEP_NULL;
			end
		end},'image/chouka/main/bg_1.png',true);
	end
end

function RewardPanel:reqBoxReward()
	if self.isReqBoxReward then
		return;
	end

	self.isReqBoxReward = true;
	self.boxNode:runAction(cc.Sequence:create(cc.DelayTime:create(5) , cc.CallFunc:create(function() 
		self.isReqBoxReward = false;
	end)))

	local bids = {};
	for _k , data in pairs(self.boxs) do
		table.insert(bids,data.bid);
	end
	local msg = {
		bids = bids
	};
	Network.NetworkManager:Send(Network.NetworkCmdType.req_open_box , msg);
end

function RewardPanel:retOpenBox(msg)
	self:onShow(msg);
end

function RewardPanel:onLeave()
	self.root:setVisible(false);
	self.rewardList:removeAllChildren()
	TimeManager:registUpdateEvent('RewardPanel',nil,true);
end

function RewardPanel:onClickBack()

	if self.actStep == ACT_STEP.STEP_END or self.actStep == ACT_STEP.STEP_NULL then
		--self.particleWin:stopSystem();
		UIManager:pop('RewardPanel');
		if self.isLevelUp then
			local option = nil ;
			if self.callback then
				option = {
					callback = self.callback;
					guide = true;
				}
				print('RewardPanel:onClickBack========guide=======>');
			end
			UIManager:push('LevelUpPanel',self.levelUpData,option);
		else
			if self.callback then
				self.callback();
			end
		end
	elseif self.actStep == ACT_STEP.STEP_SHOW_BOX then
		self:reqBoxReward();
	end
end

return RewardPanel;
