local RoundTargetRewardPanel = class("RoundTargetRewardPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local RoundTargetRewardType = Enum:getTable("RoundTargetRewardType");
local MessageBoxType = Enum:getTable('MessageBoxType')


function RoundTargetRewardPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'RoundTargetRewardPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function RoundTargetRewardPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function RoundTargetRewardPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(606,761));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , -42);

	local titleNode = GlobalFunctions:createTitle(LANG.UI_ROUND_13,606,63,nil,true);
	titleNode:addTo(self.root);
	titleNode:setPosition(0,370);

	local listBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	listBg:addTo(self.root);
	listBg:setPosition(0 , -39);
    listBg:setCapInsets(cc.rect(11,11,34,28));
	listBg:setContentSize(cc.size(582 , 736));

	local initData = {
        width = 564,
        height = 720,
        cellWidth = 564,
        cellHeight = 132,
		cellReloadFunc = handler(self,self.reloadCellPanel),
	}
	
	self.roundRewardTableView = template.TableViewTemplate:create()
    self.roundRewardTableView:initWithSizeAndItem(initData) ;
    self.roundRewardTableView:getView():setPosition(cc.p(-282,-399));
    self.roundRewardTableView:getView():addTo(self.root)

	--self.cancelBtn = ccui.Button:create('image/commonImg/btn_bg2.png' , 'image/commonImg/btn_bg2.png');
	--self.cancelBtn:addTo(self.root);
	--self.cancelBtn:setPosition(-117 , -451);
	--self.cancelBtn:addClickEventListener(handler(self , self.onClickBack));

	--local cancelLabel = cc.Label:createWithTTF(LANG.UI_ROUND_14 , fonts.FONT_SIYUAN , 20);
	--cancelLabel:addTo(self.cancelBtn);
	--cancelLabel:setPosition(self.cancelBtn:getContentSize().width/2 , self.cancelBtn:getContentSize().height/2);
	--cancelLabel:setTextColor(cc.c3b(60 , 60 , 60));
	--cancelLabel:enableOutline(cc.c4b(255 , 255 , 255 , 255) , 2);

	--self.oneKeyBtn = ccui.Button:create('image/commonImg/btn_bg1.png' , 'image/commonImg/btn_bg1.png');
	--self.oneKeyBtn:addTo(self.root);
	--self.oneKeyBtn:setPosition(117 , -451);
	--self.oneKeyBtn:addClickEventListener(handler(self , self.oneKeyBtnClick));

	--local oneKeyLabel = cc.Label:createWithTTF(LANG.UI_ROUND_15 , fonts.FONT_SIYUAN , 20);
	--oneKeyLabel:addTo(self.oneKeyBtn);
	--oneKeyLabel:setPosition(self.oneKeyBtn:getContentSize().width/2 , self.oneKeyBtn:getContentSize().height/2);
	--oneKeyLabel:setTextColor(cc.c3b(255 , 255 , 255));
	--oneKeyLabel:enableOutline(cc.c4b(19 , 93 , 188 , 255) , 2);

	EventManager:RegisterEvent(EventType.NetwordRetRoundTargetReward, handler(self, self.retRoundReward));
end

function RoundTargetRewardPanel:onEnter(rewardType)

	self.roundRewardType = rewardType or RoundTargetRewardType.ROUND;

	self:addSwallowCallback(handler(self , self.onClickBack));

	self:updateData();

	self.root:setVisible(true);
end

function RoundTargetRewardPanel:updateData()
	self.roundRewards = {};

	if self.roundRewardType == RoundTargetRewardType.ROUND then
		for _k , reward in pairs(ResTable:getTable('round_reward').contain) do
			table.insert(self.roundRewards , reward);
		end

		if #self.roundRewards > 1 then
			table.sort(self.roundRewards , function(first , second)
				local firstGetReward = model.RoundModel:getRoundRewarded(first.id);
				local secondGetReward = model.RoundModel:getRoundRewarded(second.id);
				-- print(fristGetReward , secondGetReward);
				if firstGetReward == nil and secondGetReward ~= nil then
					return true;
				end
				if firstGetReward ~= nil and secondGetReward == nil then
					return false;
				end
				return first.id < second.id;
			end);
		end
	elseif self.roundRewardType == RoundTargetRewardType.EVENT then
		for _k , reward in pairs(ResTable:getTable('event_reward').contain) do
			table.insert(self.roundRewards , reward);
		end
		
		if #self.roundRewards > 1 then
			table.sort(self.roundRewards , function(first , second)
				local firstGetReward = model.EventModel:getEventRewardState(first.id);
				local secondGetReward = model.EventModel:getEventRewardState(second.id);
				if firstGetReward == nil and secondGetReward ~= nil then
					return true;
				end
				if firstGetReward ~= nil and secondGetReward == nil then
					return false;
				end
				return first.id < second.id;
			end);
		end
	elseif self.roundRewardType == RoundTargetRewardType.TRIAL then
		for _k , reward in pairs(ResTable:getTable('trial_reward').contain) do
			table.insert(self.roundRewards , reward);
		end
		
		if #self.roundRewards > 1 then
			table.sort(self.roundRewards , function(first , second)
				local firstGetReward  = model.TrialModel:getStatusById(first.id);
				local secondGetReward  = model.TrialModel:getStatusById(second.id);
	
				if firstGetReward == nil and secondGetReward ~= nil then
					return true;
				end
	
				if firstGetReward ~= nil and secondGetReward == nil then
					return false;
				end
	
				return first.id < second.id
			end);
		end
	end

	self.roundRewardTableView:reloadData(self.roundRewards);
end

function RoundTargetRewardPanel:reloadCellPanel(cell , idx)

	if self.roundRewards[idx + 1] then
		local child = cell:getChildByTag(999);
		if not child then
			child = template.RoundTargetRewardTemplate:create();
			child:addTo(cell);
			child:setPosition(282 , 66);
			child:addBtnClickEventListener(handler(self , self.rewardClick))
			child:setTag(999);
		end
		local reward = self.roundRewards[idx + 1];

		if self.roundRewardType == RoundTargetRewardType.ROUND then
			child:Init(reward);
		elseif self.roundRewardType == RoundTargetRewardType.EVENT then
			child:InitEventReward(reward);
		elseif self.roundRewardType == RoundTargetRewardType.TRIAL then
			child:InitTrial(reward);
		end

		child:setBtnTag(reward.id);
	end
	
	return cell;
end

function RoundTargetRewardPanel:oneKeyBtnClick()
	self:rewardClick(0 , 1);
end

function RoundTargetRewardPanel:rewardClick(id , flag)

	if flag == 1 then
		local msg = {};
		msg.id = id;
		if self.roundRewardType == RoundTargetRewardType.ROUND then
			Network.NetworkManager:Send(Network.NetworkCmdType.req_round_reward , msg);
		elseif self.roundRewardType == RoundTargetRewardType.TRIAL then
			model.TrialModel:reqTrialReward(id);
		end
	else
		if self.roundRewardType == RoundTargetRewardType.TRIAL then
			Prompt:Show(LANG.UI_TRIAL_9);
		else
			Prompt:Show(LANG.UI_ROUND_16);
		end
		self:onClickBack();
	end
end

function RoundTargetRewardPanel:retRoundReward(msg)
	if msg.id and #msg.id > 0 then
		self:updateData();

		if self.roundRewardType ~= RoundTargetRewardType.EVENT then
			tools.RedDotTools:pushFightSceneRedDot(4)
		end
	else
		if self.roundRewardType == RoundTargetRewardType.TRIAL then
			Prompt:Show(LANG.UI_TRIAL_10);
		else
			Prompt:Show(LANG.UI_ROUND_17);
		end
	end

	UIManager:push('RewardPanel' , msg);

end

function RoundTargetRewardPanel:onClickBack()
	UIManager:pop('RoundTargetRewardPanel');

end

function RoundTargetRewardPanel:onLeave()
	if self.roundRewardType == RoundTargetRewardType.ROUND then
		EventManager:FireEvent(EventType.FightPanelBtnRedDot , 4);
	elseif self.roundRewardType == RoundTargetRewardType.EVENT then
		EventManager:FireEvent(EventType.UIEventRewardUpdate)
	elseif self.roundRewardType == RoundTargetRewardType.TRIAL then
		EventManager:FireEvent(EventType.UITrialPanelUpdateRewardTips);
	end
	
	self.root:setVisible(false);
end

function RoundTargetRewardPanel:guideRewardClick( )
	self:rewardClick(1 , 1);
end
return RoundTargetRewardPanel;
