local ScreenEffectPanel = class("ScreenEffectPanel", require('app.layer.BaseLayer'))
local imgPath = 'image/screen/';

function ScreenEffectPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'ScreenEffectPanel');

	self.useCCreator = false;
	self.haveSwallowLayer = true;
	self.isPopup = true;
end

function ScreenEffectPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanel();

	EventManager:RegisterEvent(EventType.FightDoorOpen, handler(self, self.playDoorClose2Open));
end

function ScreenEffectPanel:initPanel()
	self.fightDoorNode = cc.Node:create();
	self.fightDoorNode:setVisible(false);
	self.fightDoorNode:addTo(self.root);	

	self.crossing = cc.Sprite:create(imgPath .. 'crossing.png')
		:setLocalZOrder(150)
		:addTo(self.fightDoorNode)
	
	self.fightMask = cc.LayerColor:create(cc.c4b(0, 0, 0, 255))
	self.fightMask:setContentSize( cc.size(StageSize.width, StageSize.height) )
	self.fightMask:setPosition(-StageSize.width/2, -StageSize.height/2)
	self.fightMask:setLocalZOrder(100)
	self.fightMask:addTo(self.fightDoorNode)

	self.roundSelectNode = cc.Node:create();
	self.roundSelectNode:setVisible(false);
	self.roundSelectNode:addTo(self.root);

	self.selectMark = cc.Sprite:create(imgPath .. 'select_mark.png');
	self.selectMark:addTo(self.roundSelectNode);

	self.crossingBeginX = -1608
	self.crossingPauseX = 0
	self.crossingEndX	= 1608
end

function ScreenEffectPanel:onEnter(flag, option)
	if not flag then
		return;
	end
	self.root:setVisible(true);
	self.callback = nil;
	self.background = nil;
	if option then
		self.background = option.background;
		self.callback = option.callback;
	end

	if flag == 'door' then
		self:playDoorClose()
	elseif flag == 'newRound' then
		self:playRoundSelect();
	elseif flag == 'newChapter' then
		self:playNewChapterClose();
	elseif flag == 'doorOpen' then
		self:playDoorOpen();
	end
end

function ScreenEffectPanel:onLeave()
	self.root:setVisible(false);
end

function ScreenEffectPanel:playDoorClose()
	self.fightDoorNode:setVisible(true);

	self.crossing:setPosition(self.crossingBeginX, 0)
	self.crossing:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.5, cc.p(self.crossingPauseX-50, 0)),
		cc.MoveTo:create(0.4, cc.p(0, 0)),
		cc.CallFunc:create(function() 
			if self.callback then
				self.callback()
			end
		end),
		cc.MoveTo:create(0.4, cc.p(50, 0))))
	
	self.fightMask:setOpacity(0)
	self.fightMask:runAction(cc.FadeIn:create(0.35))
end

function ScreenEffectPanel:playDoorClose2Open(callback)
	self.crossing:stopAllActions()
	self.crossing:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.5, cc.p(self.crossingEndX, 0)),
		cc.CallFunc:create(function() 
			if callback then
				callback()
			end
			self.fightDoorNode:setVisible(false);
			UIManager:pop('ScreenEffectPanel');
		end)))
	
	self.fightMask:runAction(cc.Sequence:create(
		cc.DelayTime:create(0.15),
		cc.FadeOut:create(0.35)))
end

function ScreenEffectPanel:playDoorOpen()
	self.fightDoorNode:setVisible(true);

	self.crossing:setPosition(self.crossingBeginX, 0)
	self.crossing:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.5, cc.p(self.crossingPauseX-50, 0)),
		cc.MoveTo:create(0.8, cc.p(50, 0)),
		cc.MoveTo:create(0.5, cc.p(self.crossingEndX, 0)),
		cc.CallFunc:create(function() 
			if self.callback then
				self.callback()
			end
			self.fightDoorNode:setVisible(false);
			UIManager:pop('ScreenEffectPanel');
		end)))
	
	self.fightMask:setOpacity(0)
	self.fightMask:runAction(cc.Sequence:create(
		cc.FadeIn:create(0.35),
		cc.DelayTime:create(0.6),
		cc.FadeOut:create(0.35)))
end

function ScreenEffectPanel:playRoundSelect()
	self.roundSelectNode:setVisible(true);

	local chapter = model.RoundModel:getChapter();
	local chapterData = ResTable:getRowData('chapter', chapter);
	local pathData = ResTable:getRowData('round_path', chapterData.path)

	local roundIndex = model.RoundModel:getRoundIndex() % 100;

	local x, y = pathData['coordination'][roundIndex][1], pathData['coordination'][roundIndex][2]

	self.selectMark:setPosition(x, y);
	self.selectMark:setScale(3);
	self.selectMark:runAction(cc.Sequence:create(cc.ScaleTo:create(0.5, 1), cc.CallFunc:create(function()
		if self.callback then
			self.callback();
		end
		self.roundSelectNode:setVisible(false);
		UIManager:pop('ScreenEffectPanel');
	end)))
	--[[
	self.selectMask:setPosition(0, 200);
	self.selectMask:setScale(5);
	self.selectMask:runAction(cc.ScaleTo:create(2, 0.5));
	--]]
end

function ScreenEffectPanel:playNewChapterClose()
	self.fightDoorNode:setVisible(true);

	self.crossing:setPosition(self.crossingBeginX, 0)
	self.crossing:runAction(cc.Sequence:create(
		cc.MoveTo:create(0.5, cc.p(self.crossingPauseX-50, 0)),
		cc.CallFunc:create(function() 
		if self.callback then
			self.callback();
		end
		end),
		cc.MoveTo:create(0.8, cc.p(50, 0)),
		cc.MoveTo:create(0.5, cc.p(self.crossingEndX, 0)),
		cc.CallFunc:create(function() 
			self.fightDoorNode:setVisible(false);
			UIManager:pop('ScreenEffectPanel');
		end)))

	self.fightMask:setOpacity(0)
	self.fightMask:runAction(cc.Sequence:create(
		cc.FadeIn:create(0.35),
		cc.DelayTime:create(0.6),
		cc.FadeOut:create(0.35)))
end

return ScreenEffectPanel;
