local SettingAudioPanel = class("SettingAudioPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local imagePath = 'image/setup/'
local PushType = Enum:getTable("PushType");
local MessageBoxType = Enum:getTable('MessageBoxType')
local MIN_PERCENT = 8

function SettingAudioPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'SettingAudioPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function SettingAudioPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
	self:registerHandler()
end

function SettingAudioPanel:initPanels()
	self.bMove = false;

	local bg = ccui.Layout:create();
	bg:addTo(self.root);
	bg:setBackGroundImage('image/commonImg/bg_2_11_4_18_21.png');
	bg:setBackGroundImageScale9Enabled(true);
	bg:setBackGroundImageCapInsets(cc.rect(11,4,18,21));
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(604,319));
	bg:setAnchorPoint(0.5,0.5);
	bg:setPosition(0 , 30);

	local teamBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	teamBg:addTo(self.root);
	teamBg:setPosition(0 , 62);
    teamBg:setCapInsets(cc.rect(11,11,34,28));
	teamBg:setContentSize(cc.size(582 , 219));

	--GlobalFunctions:createTitleNode( bg, LANG.UI_SETTING[4], LANG.UI_SETTING_EN[4],  bg:getContentSize().width/2, bg:getContentSize().height -36 );
	
	local texture = 'image/commonImg/menu_left_71_46.png';
	local untexture = 'image/commonImg/menu_left_71_46_u.png';

	self.vollabel1 = cc.Label:createWithTTF(LANG.UI_SETTING_11 , fonts.FONT_SIYUAN , 18)
	self.volBtn = ccui.Button:create(texture,texture,unTexture);
	self.volBtn:addTo(self.root);
	self.volBtn:setTitleLabel(self.vollabel1)
	self.volBtn:setScale9Enabled(true)
	self.volBtn:setBright(true)
	self.volBtn:setPosition(-150 , 210)
	self.volBtn:setCapInsets(cc.rect(10 , 0 , 51 , 46));
	self.volBtn:setContentSize(cc.size(302 , 42));
	self.volBtn:addClickEventListener(handler(self , self.changeShowVolNode));

	self.lanlabel2 = cc.Label:createWithTTF(LANG.UI_SETTING_18 , fonts.FONT_SIYUAN , 18)
	self.lanlabel2:setScaleX(-1);
	self.lanBtn = ccui.Button:create(texture,texture,unTexture);
	self.lanBtn:addTo(self.root);
	self.lanBtn:setScaleX(-1);
	self.lanBtn:setTitleLabel(self.lanlabel2)
	self.lanBtn:setScale9Enabled(true)
	self.lanBtn:setBright(false)
	self.lanBtn:setPosition(150 , 210)
	self.lanBtn:setCapInsets(cc.rect(10 , 0 , 51 , 46));
	self.lanBtn:setContentSize(cc.size(302 , 42));
	self.lanBtn:addClickEventListener(handler(self , self.changeShoLanwNode));

	self.selBtnTag = cc.Sprite:create('image/commonImg/menu_light_1.png')
	:addTo(self.root)
	:setPosition(-150 , 231)


	local function volumeSliderChangedEvent(sender,eventType)
		local tag = sender:getTag();
		local percent = sender:getPercent();
		percent = percent-MIN_PERCENT;
		local volume = percent / (100-MIN_PERCENT);

		if volume < 0  then
			self.volumeSliderList[tag]:setPercent(MIN_PERCENT);
			return;
		end

		if tag == 1 then
			AudioManager:setMusicVolume(volume);
			self.musicVolume = volume;
		elseif tag == 2 then
			AudioManager:setEffectsVolume(volume);
			self.effectsVolume = volume;
		end

		if eventType == 2 then
			AudioManager:playEffect(Conf.confirmButton);
		end
	end
	self.volumeNode = cc.Node:create()
		:addTo(self.root)
	self.volumeSliderList = {};
	self.pushSwichBtn = {}
	local title = {LANG.UI_SETTING_13,LANG.UI_SETTING_14,LANG.UI_SETTING_16,LANG.UI_SETTING_29}
	local bgImg = {'ui_audio_bg_5.png','ui_audio_bg_6.png','ui_audio_bg_9.png','ui_audio_bg_9.png'}
	for i = 1 , 4 do
		local node = cc.Node:create();
		node:addTo(self.volumeNode);
		node:setPosition(0 , 125 + 50 - 50*i);

		local bg = cc.Sprite:create( imagePath..bgImg[i]);
		bg:addTo(node);
		bg:setPosition(-250, 0)

		local label = cc.Label:createWithTTF(title[i], fonts.FONT_SIYUAN , 18);
		label:addTo(node);
		label:setAnchorPoint(cc.p(0, 0.5));
		label:setPosition(-226 , 0);

		if i < 3 then
			local volumeSlider = ccui.Slider:create()
			volumeSlider:setTouchEnabled(true)
			volumeSlider:loadBarTexture(imagePath.."ui_audio_bg_1.png")
			volumeSlider:loadSlidBallTextures(imagePath.."ui_audio_bg_3.png", imagePath.."ui_audio_bg_3.png", "")
			volumeSlider:loadProgressBarTexture(imagePath.."ui_audio_bg_8.png")
			volumeSlider:setPosition(66 , 0)
			volumeSlider:addEventListener(volumeSliderChangedEvent)
			volumeSlider:setPercent(0)
			volumeSlider:setTag(i);
			volumeSlider:addTo(node);
	
			volumeSlider:setScale9Enabled(true)
			volumeSlider:setCapInsetProgressBarRenderer( cc.rect(14,0,8,28) )
			self.volumeSliderList[i] = volumeSlider;
		else
			local tag = i-2
			local btn = ccui.Button:create( imagePath.. 'ui_swich_control_bg.png')
				:addTo(node)
				:setPosition(cc.p(0,0))
				:setTag(tag)
				btn:addClickEventListener(handler(self , self.onPushSwichCall));

			local swichOnNode = cc.Node:create()
				:addTo(btn)
				:setPosition(cc.p(48,13.5))
				:setTag(1)
			local onBg = cc.Sprite:create( imagePath.. 'ui_swich_control_on.png')
				:addTo(swichOnNode)
				:setPosition(cc.p(-21,0))
			local onLabel = cc.Label:createWithTTF('ON' , fonts.FONT_SIYUAN , 15)
				:setPosition(cc.p(19,2))
				:addTo(swichOnNode)
			local swichOffNode = cc.Node:create()
				:addTo(btn)
				:setTag(2)
				:setVisible(false)
				:setPosition(cc.p(48,13.5))
			local offBg = cc.Sprite:create( imagePath.. 'ui_swich_control_off.png')
				:addTo(swichOffNode)
				:setPosition(cc.p(21,0))
			local offLabel = cc.Label:createWithTTF('OFF' , fonts.FONT_SIYUAN , 15)
				:setPosition(cc.p(-20,2))
				:addTo(swichOffNode)

			self.pushSwichBtn[tag] = btn;

		end

	end

	local label = cc.Label:createWithTTF(LANG.UI_SETTING_15 , fonts.FONT_SIYUAN , 20);
	label:enableOutline(cc.c3b(37 , 73 , 176) , 2);

	local saveBtn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	saveBtn:addTo(self.root);
	saveBtn:setPosition(0 , -87);
	saveBtn:addClickEventListener(handler(self , self.saveBtnClick));
	saveBtn:setTitleLabel(label);

	self:initLanguageNode()
	
end

function SettingAudioPanel:initLanguageNode()
	-- body
	self.countrySprList = {}
	self.dataArr = {}
	local dataCnt = 0;
	for k, v in pairs(ResTable:getTable('language_change').contain) do
		self.dataArr[v.country] = v;
		dataCnt = dataCnt + 1
	end

	self.LangNode = cc.Node:create()
		:addTo(self.root)
		:setVisible(false)

	local curLn = model.UserDefaultModel:getLanguageSet();

	local subLangNode = cc.Node:create()
	subLangNode:addTo(self.LangNode)
	subLangNode:setPosition(0, 45)
	self.subLangNode = subLangNode;

	self.scorllNode = cc.Node:create()
	self.scorllNode:addTo(subLangNode)
	self.scorllNode:setPosition(0, -110)
	self.scorllNode:setVisible(false)

	local lnode = cc.Node:create()
		:addTo(subLangNode)
		:setPosition(0, 62)
	local bgBtn = ccui.Button:create(imagePath..'ui_lg_btn.png' , imagePath..'ui_lg_btn.png');
	bgBtn:addTo(lnode);
	bgBtn:addClickEventListener(handler(self , self.openLangView));

	local label1 = cc.Label:createWithTTF(LANG.UI_SETTING_L1, fonts.FONT_SIYUAN , 20);
	label1:addTo(lnode);
	label1:setAnchorPoint(cc.p(1,0.5));
	label1:setPosition(-140 , 0);

	local label = cc.Label:createWithTTF('选择语言', fonts.FONT_SIYUAN , 20);
	label:addTo(lnode);
	label:setAnchorPoint(cc.p(0,0.5));
	label:setPosition(-120 , 0);
	self.curLlabel =  label;

	local bg = cc.Sprite:create(imagePath..'ui_lg_bg.png')
	bg:addTo(self.scorllNode)
	self.scorllview = ccui.ScrollView:create();
	self.scorllview:setTouchEnabled(true);
	self.scorllview:setBounceEnabled(true);
	self.scorllview:setDirection(ccui.ScrollViewDir.vertical);
	self.scorllview:setContentSize(cc.size(500 , 280));
	self.scorllview:setAnchorPoint(cc.p(0,0));
	self.scorllview:setScrollBarEnabled(false);
    self.scorllview:setPosition(-250 , -140);
    self.scorllview:addTo(self.scorllNode);



	-- local icon = cc.Sprite:create(imagePath..ln..'_icon.png');
	-- icon:addTo(node);
	-- icon:setPosition( -210, 0 )
	local cellH = 50
	local totalHeight = dataCnt * cellH
	if totalHeight < 280 then
		totalHeight = 280
	end
	self.scorllview:setInnerContainerSize(cc.size(500, totalHeight));

	local idx = 0
	for k,v in pairs(self.dataArr) do
		
		local ln = k;
		local node = cc.Node:create()
			:addTo(self.scorllview)
			:setPosition(250, totalHeight - idx*cellH - cellH/2)
		local cellLine = cc.Sprite:create(imagePath..'ui_lg_cell_line.png')
		cellLine:addTo(node)
		cellLine:setPosition(0, -20)

		local bgBtn = ccui.Button:create(imagePath..'ui_lg_cell_bg.png' , imagePath..'ui_lg_cell_bg.png');
		bgBtn:addTo(node);
		bgBtn:setName(ln)
		bgBtn:setOpacity(0)
		bgBtn:setSwallowTouches(false)
		bgBtn:addClickEventListener(handler(self , self.btnLanguageClick));

		local text = v.name
		-- local font = 'font/SourceHanSansCN-Medium.otf';
		-- if i == 2 then
		-- 	font = fonts.FONT_SIYUAN;
		-- end
		local selspr = cc.Sprite:create(imagePath..'ui_lg_cell_bg.png');
		selspr:addTo(node);
		selspr:setPosition( 0, 0 )
		selspr:setVisible(false)

		local label = cc.Label:createWithTTF(text, fonts.FONT_SIYUAN , 20);
		label:addTo(node);
		label:setAnchorPoint(cc.p(0,0.5));
		label:setPosition(-200 , 0);

		self.countrySprList[ ln ] = selspr
		if curLn == ln then
			selspr:setVisible(true)
			self.curLlabel:setString(text)
		end
		idx = idx + 1

	end



	local cvLangNode = cc.Node:create()
		:addTo(self.LangNode)
		:setPosition(0, -45)
		:setLocalZOrder(-1)
	self.cvLangNode = cvLangNode;

	self.scorllCvNode = cc.Node:create()
	self.scorllCvNode:addTo(cvLangNode)
	self.scorllCvNode:setPosition(0, -110)
	self.scorllCvNode:setVisible(false)

	local lnCVode = cc.Node:create()
		:addTo(cvLangNode)
		:setPosition(0, 62)
	local bgBtn = ccui.Button:create(imagePath..'ui_lg_btn.png' , imagePath..'ui_lg_btn.png');
	bgBtn:addTo(lnCVode);
	bgBtn:addClickEventListener(handler(self , self.openCVLangView));

	local label1 = cc.Label:createWithTTF(LANG.UI_SETTING_L2, fonts.FONT_SIYUAN , 20);
	label1:addTo(lnCVode);
	label1:setAnchorPoint(cc.p(1,0.5));
	label1:setPosition(-140 , 0);

	local label = cc.Label:createWithTTF('CV语言', fonts.FONT_SIYUAN , 20);
	label:addTo(lnCVode);
	label:setAnchorPoint(cc.p(0,0.5));
	label:setPosition(-120 , 0);
	self.curCVLlabel =  label;

	local bg = cc.Sprite:create(imagePath..'ui_lg_bg.png')
	bg:addTo(self.scorllCvNode)
	self.CVscorllview = ccui.ScrollView:create();
	self.CVscorllview:setTouchEnabled(true);
	self.CVscorllview:setBounceEnabled(true);
	self.CVscorllview:setDirection(ccui.ScrollViewDir.vertical);
	self.CVscorllview:setContentSize(cc.size(500 , 280));
	self.CVscorllview:setAnchorPoint(cc.p(0,0));
	self.CVscorllview:setScrollBarEnabled(false);
    self.CVscorllview:setPosition(-250 , -140);
    self.CVscorllview:addTo(self.scorllCvNode);

    self.CVDataList = {'jp','kr'}
    self.CVName = {jp=LANG.UI_LANGUAGE_NAME[1] , kr=LANG.UI_LANGUAGE_NAME[2]}
    self.CVSprList = {}
    local dataCnt = #self.CVDataList
    local cellH = 50
	local totalHeight = dataCnt * cellH
	if totalHeight < 280 then
		totalHeight = 280
	end
	self.CVscorllview:setInnerContainerSize(cc.size(500, totalHeight));
	local curLn = model.UserDefaultModel:getCVLanguageSet();

	local idx = 0
	for k,v in pairs(self.CVDataList) do
		
		local ln = v;
		local node = cc.Node:create()
			:addTo(self.CVscorllview)
			:setPosition(250, totalHeight - idx*cellH - cellH/2)
		local cellLine = cc.Sprite:create(imagePath..'ui_lg_cell_line.png')
		cellLine:addTo(node)
		cellLine:setPosition(0, -20)

		local bgBtn = ccui.Button:create(imagePath..'ui_lg_cell_bg.png' , imagePath..'ui_lg_cell_bg.png');
		bgBtn:addTo(node);
		bgBtn:setName(ln)
		bgBtn:setOpacity(0)
		bgBtn:setSwallowTouches(false)
		bgBtn:addClickEventListener(handler(self , self.btnCVLanguageClick));

		local text = self.CVName[ ln ]
		local selspr = cc.Sprite:create(imagePath..'ui_lg_cell_bg.png');
		selspr:addTo(node);
		selspr:setPosition( 0, 0 )
		selspr:setVisible(false)

		local label = cc.Label:createWithTTF(text, fonts.FONT_SIYUAN , 20);
		label:addTo(node);
		label:setAnchorPoint(cc.p(0,0.5));
		label:setPosition(-200 , 0);

		self.CVSprList[ ln ] = selspr
		if curLn == ln then
			selspr:setVisible(true)
			self.curCVLlabel:setString(text)
		end
		idx = idx + 1

	end
end

function SettingAudioPanel:registerHandler()

	self.swallowLayer = cc.LayerColor:create(cc.c4b(0, 0, 0, 0));
	self.swallowLayer:setContentSize( cc.size(StageSize.width, StageSize.height) );
	self.swallowLayer:setPosition(-StageSize.width/2, -StageSize.height/2);
	self.swallowLayer:setLocalZOrder(100);
	self.swallowLayer:addTo(self.root);

	-- listener
	self.swallowListener = cc.EventListenerTouchOneByOne:create();
	local onTouchBegan = function(touch, event)
		self.bMove = false;
        return true
	end
	local onTouchMove = function(touch,event)
		local posEnd = touch:getLocation();
		local posStart = touch:getStartLocation()
		local dis = cc.pGetDistance(posStart,posEnd)
		if dis > 20 then
			self.bMove = true;
		end
	end
	local onTouchEnd = function(touch ,event )
	end
	self.swallowListener:setSwallowTouches(false);
	self.swallowListener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN);
	self.swallowListener:registerScriptHandler(onTouchMove, cc.Handler.EVENT_TOUCH_MOVED);
	self.swallowListener:registerScriptHandler(onTouchEnd, cc.Handler.EVENT_TOUCH_ENDED);
	local eventDispatcher = self.swallowLayer:getEventDispatcher();
	eventDispatcher:addEventListenerWithSceneGraphPriority(self.swallowListener, self.swallowLayer);
end

function SettingAudioPanel:onPushSwichCall( sender )
	local tag = sender:getTag()

	local btnState = tag == 1 and 0 or 1

	local function callback( newState )
		self.pushState = newState;
		self.pushSwichBtn[1]:getChildByTag(1):setVisible(self.pushState ~= 0)
		self.pushSwichBtn[1]:getChildByTag(2):setVisible(self.pushState == 0)
	
		self.pushSwichBtn[2]:getChildByTag(1):setVisible(self.pushState == 1)
		self.pushSwichBtn[2]:getChildByTag(2):setVisible(self.pushState ~= 1)

		if self.pushState == 2 then
			MessageBox:create(LANG.UI_SETTING_30,MessageBoxType.OK, function () end)
		end

	end
	local changeState = 0;
	if tag == 1 then
		changeState = self.pushState == 0 and 1 or 0
	else
		if self.pushState == 0 then
			Prompt:Show(LANG.UI_SETTING_31)
			return 
		end
		changeState = self.pushState == 1 and 2 or 1
	end
	model.UserModel:changePushState(changeState,callback)

end
function SettingAudioPanel:btnLanguageClick(sender , event)
	if self.bMove then
		return
	end
	local name = sender:getName();
	-- print('SettingAudioPanel--click--language:', name)
	for k,v in pairs(self.countrySprList) do
		self.countrySprList[ k ]:setVisible(false)
		if k == name then
			self.countrySprList[ k ]:setVisible(true)
			self.curSelLG = k;
			self.curLlabel:setString(self.dataArr[k].name)
		end
	end
	-- tools.SensitiveTools:clearAll() -- 切换敏感词库

	AudioManager:playEffect(Conf.confirmButton);

    -- EventManager:FireEvent(EventType.MainSceneReloadGame);
end

function SettingAudioPanel:onEnter(bareId , pos)

	self:addSwallowCallback(handler(self , self.onClickBack));

	self:updateBtnVisible();

	local fileUtils = cc.FileUtils:getInstance();
	local fileName = device.writablePath..'/setting_config.json';
	local settingConfig = {};
	self.musicVolume = 1;
	self.effectsVolume = 1;

	self.pushState = model.UserDefaultModel:getPushEventState()
	self.pushSwichBtn[1]:getChildByTag(1):setVisible(self.pushState ~= 0)
	self.pushSwichBtn[1]:getChildByTag(2):setVisible(self.pushState == 0)

	self.pushSwichBtn[2]:getChildByTag(1):setVisible(self.pushState == 1)
	self.pushSwichBtn[2]:getChildByTag(2):setVisible(self.pushState ~= 1)

	if fileUtils:isFileExist(fileName) then
		local fileData = fileUtils:getStringFromFile(fileName);
		if fileData ~= '' then
			settingConfig = Cjson.decode(fileData);

			if settingConfig.musicVolume then
				self.musicVolume = settingConfig.musicVolume;
			end

			if settingConfig.effectsVolume then
				self.effectsVolume = settingConfig.effectsVolume;
			end
		end
	end
	
	local maxPercent = 100-MIN_PERCENT;
	local percent = self.musicVolume * maxPercent;
	percent = percent > maxPercent and maxPercent or percent;
	self.volumeSliderList[1]:setPercent(MIN_PERCENT + percent);

	percent = self.effectsVolume * maxPercent;
	percent = percent > maxPercent and maxPercent or percent;
	self.volumeSliderList[2]:setPercent(MIN_PERCENT + percent);

	self.root:setVisible(true);
end

function SettingAudioPanel:updateBtnVisible()
	local package_name = sdkManager.SDKLuaHelper:getPackageName()
	if not tools.ConfigStateTools:checkVisibleByName('SettingAudioPanelLanguage') and package_name == 'com.nani.ego.kr.one' then
		-- self.lanBtn:setVisible(false);
		-- self.volBtn:setPositionX(0)
		-- self.selBtnTag:setPositionX(0)
		-- self.volBtn:setContentSize(cc.size(604 , 42));

		self.subLangNode:setVisible(false)
		self.cvLangNode:setPosition(0, 0)
	end
	self.curSelLG = model.UserDefaultModel:getLanguageSet();

	self.curSelCV = model.UserDefaultModel:getCVLanguageSet();
end

function SettingAudioPanel:saveBtnClick(sender , event)
	self:saveVolume();
	UIManager:pop('SettingAudioPanel');

	if self.curSelLG ~= model.UserDefaultModel:getLanguageSet() then
		model.UserDefaultModel:setLanguageSet( self.curSelLG )
		EventManager:FireEvent(EventType.MainSceneReloadGame);
		cc.exports.CURRENCY = GlobalFunctions:getTokenByLan(model.UserDefaultModel:getLanguageSet())
	end

	if self.curSelCV ~= model.UserDefaultModel:getCVLanguageSet() then
		model.UserDefaultModel:setCVLanguageSet( self.curSelCV )
		AudioManager:initCVPath()
	end
end

function SettingAudioPanel:saveVolume()
	local fileUtils = cc.FileUtils:getInstance();
	local fileName = device.writablePath..'/setting_config.json';
	local settingConfig = {};
	if fileUtils:isFileExist(fileName) then
		local fileData = fileUtils:getStringFromFile(fileName);
		if fileData ~= '' then
			settingConfig = Cjson.decode(fileData);
		end
	end
	settingConfig.musicVolume = self.musicVolume;
	settingConfig.effectsVolume =	self.effectsVolume;
	fileUtils:writeStringToFile(Cjson.encode(settingConfig), fileName);
end

function SettingAudioPanel:onClickBack()
	if self.LangNode:isVisible() and self.scorllNode:isVisible() then
		self.scorllNode:setVisible(false)
		return
	end
	if self.LangNode:isVisible() and self.scorllCvNode:isVisible() then
		self.scorllCvNode:setVisible(false)
		return
	end
	self:saveVolume();

	UIManager:pop('SettingAudioPanel');
end

function SettingAudioPanel:onLeave()
	self.root:setVisible(false);
end

function SettingAudioPanel:changeShowVolNode()
	self.volBtn:setBright(true)
	self.lanBtn:setBright(false)
	self.vollabel1:setTextColor(cc.c3b(255 , 255 , 255))
	self.lanlabel2:setTextColor(cc.c3b(120 , 120 , 120))

	self.LangNode:setVisible(false)
	self.volumeNode:setVisible(true)
	self.selBtnTag:setPositionX(-150)

	AudioManager:playEffect(Conf.buttonClick);
end

function SettingAudioPanel:changeShoLanwNode()
	self.volBtn:setBright(false)
	self.lanBtn:setBright(true)
	self.vollabel1:setTextColor(cc.c3b(120 , 120 , 120))
	self.lanlabel2:setTextColor(cc.c3b(255 , 255 , 255))

	self.LangNode:setVisible(true)
	self.volumeNode:setVisible(false)
	self.selBtnTag:setPositionX(150)

	AudioManager:playEffect(Conf.buttonClick);
end

function SettingAudioPanel:openLangView()
	self.scorllNode:setVisible( not self.scorllNode:isVisible() )
	if self.scorllNode:isVisible() then
		self.scorllCvNode:setVisible(false)
	end
end

function SettingAudioPanel:openCVLangView()
	self.scorllCvNode:setVisible( not self.scorllCvNode:isVisible() )
	if self.scorllCvNode:isVisible() then
		self.scorllNode:setVisible(false)
	end
end

function SettingAudioPanel:btnCVLanguageClick(sender , event)
	if self.bMove then
		return
	end
	local name = sender:getName();
	for k,v in pairs(self.CVSprList) do
		self.CVSprList[ k ]:setVisible(false)
		if k == name then
			self.CVSprList[ k ]:setVisible(true)
			self.curSelCV = k;
			self.curCVLlabel:setString(self.CVName[ k ])
		end
	end
	AudioManager:playEffect(Conf.confirmButton);

end

return SettingAudioPanel;
