local ShopGiftRecommendPanel = class("ShopGiftRecommendPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');
local imagePath = 'image/event/';
local MessageBoxType = Enum:getTable('MessageBoxType')
local CommonShopType = Enum:getTable('CommonShopType');

function ShopGiftRecommendPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'ShopGiftRecommendPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function ShopGiftRecommendPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function ShopGiftRecommendPanel:initPanels()

	-- local bg = cc.Scale9Sprite:create('image/commonImg/common_tips_bg.png');
	-- bg:addTo(self.root);
	-- bg:setContentSize(cc.size(600 , 800));
	-- bg:setPosition(0,-10)

	self.bg = cc.Sprite:create()
	self.bg:addTo(self.root)

	self.lastTimeLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
		:setPosition(-52, -73)
		:addTo(self.root)

	-- GlobalFunctions:createTitleNode( bg, LANG.UI_MAZE_10, '',  bg:getContentSize().width/2, bg:getContentSize().height -40 );

    local itemNode = cc.Node:create()
    itemNode:addTo(self.root)
    itemNode:setPosition(0, -150)

	self.rewardView = ccui.ScrollView:create();
	self.rewardView:setTouchEnabled(true);
	self.rewardView:setBounceEnabled(true);
	self.rewardView:setDirection(ccui.ScrollViewDir.horizontal);
	self.rewardView:setContentSize(cc.size(520, 120));
	self.rewardView:setAnchorPoint(cc.p(0.5, 0.5));
	self.rewardView:setScrollBarEnabled(false);
	self.rewardView:addTo(itemNode);


	local btnNode = cc.Node:create()
	btnNode:addTo(self.root)
	btnNode:setPosition(0, -300)

	local btn = ccui.Button:create('image/commonImg/btn_1_219_55.png' ,'image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	btn:addTo(btnNode);
	btn:addClickEventListener(handler(self , self.onClickBuy));

	local btnIcon = cc.Sprite:create('image/shop/currency_icon.png')
	btnIcon:addTo(btnNode)
	btnIcon:setPosition(-50, 0)

	self.priceLabel = cc.Label:createWithTTF(LANG.UI_MAZE_12, fonts.FONT_SIYUAN, 20)
		:enableOutline(cc.c3b(55,85,155), 2)
		:setPosition(10, 0)
		:addTo(btnNode)

	local closeBtn = ccui.Button:create('image/commonImg/close_btn_1.png' , 'image/commonImg/close_btn_1.png');
	closeBtn:addTo(self.root);
	closeBtn:setPosition(240 , 366);
	closeBtn:addClickEventListener(function()
		UIManager:pop('ShopGiftRecommendPanel')
		if self.callback then
			self.callback();
		end
	end);
end

function ShopGiftRecommendPanel:onEnter( tp,callback )
	self.tp = tp or 3;
	local giftData = model.ShopModel:getShopGiftData( self.tp );
	if not giftData then
		UIManager:pop('ShopGiftRecommendPanel');
		return
	end

	self.itemData = giftData;
	self.itemId = giftData.no;
	self.callback = callback;

	-- local name = giftData.giftname or ResTable:getLangByNum(giftData.giftname_no);
    -- self.itemName:setString(tostring(name));
    self.priceLabel:setString(tostring(giftData.price));
    self.bg:setTexture('image/shop/'..giftData.resource..'.png')

    self:setReward(giftData.item);

    TimeManager:registUpdateEvent('ShopGiftRecommendPanel',handler(self,self.updateTime),true);

	self.root:setVisible(true);
	-- self:addSwallowCallback(handler(self , self.onClickBack));

end

function ShopGiftRecommendPanel:setReward(reward)
	local rewardNum = #reward;

	local maxWidth = rewardNum * 110;
	maxWidth = maxWidth < 520 and 520 or maxWidth;

	self.rewardView:removeAllChildren();
	self.rewardView:setInnerContainerSize(cc.size(maxWidth, 87));

	for i=1, rewardNum do
		local r = reward[i];
		local layout = ccui.Layout:create();
		layout:addTo(self.rewardView);
		local icon = template.ItemTemplate:create(2);
		icon:addTo(layout);
		icon:Init(r[1] , r[2]);
		icon:setPosition(55 + 110 * (i - 1) , 55);
	end
	self.rewardView:setTouchEnabled(rewardNum > 4);
end

function ShopGiftRecommendPanel:onClickBack()
	-- UIManager:pop('ShopGiftRecommendPanel');
end

function ShopGiftRecommendPanel:onClickBuy()
    local buyItem = {
        no = self.itemId,
        type = 6
    }
    sdkManager.PaymentHelper:onPayment(buyItem,handler(self,self.giftPaymentCallback));

end

function ShopGiftRecommendPanel:giftPaymentCallback( msg )
	model.ShopModel:removeShopGiftData(self.itemId)
	UIManager:pop('ShopGiftRecommendPanel');
	if self.tp == 3 then
		EventManager:FireEvent('FightPanel:noticeShowNewGiftNode');
	end
    if self.callback then
    	self.callback(msg)
    end
end

function ShopGiftRecommendPanel:updateTime()
	if not self.itemData or not self.itemData.timestamp then
		return;
	end

	local curTime = TimeManager:getCurTime()
    local lastTime = self.itemData.timestamp - curTime;

    local day = math.floor(lastTime / 86400);
    local lastSeconds = lastTime - day * 86400;
    local timeStr = GlobalFunctions:ChangeTime(lastSeconds);
    if day > 0 then
        timeStr = string.format(LANG.UI_ARENA_3 , day)..tostring(timeStr);
    end

    self.lastTimeLabel:setString(tostring(timeStr));
    -- self.lastTimeLabel:setString(string.format(LANG.UI_ARENA_17,tostring(timeStr)));

    if lastTime <= 0 then
        UIManager:pop('ShopGiftRecommendPanel')
		if self.callback then
			self.callback();
		end
    end
end

function ShopGiftRecommendPanel:onLeave()
	TimeManager:registUpdateEvent('ShopGiftRecommendPanel',nil,true);
	self.root:setVisible(false);
end

return ShopGiftRecommendPanel;
