local StoryCardPanel = class("StoryCardPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable('SystemFont');
local cardImagePath = 'image/card/';

local cardMenuPos = {
	cc.p(-151 , 425),
	cc.p(150 , 425),
}

function StoryCardPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'StoryCardPanel');

	self.useCCreator = false;
	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;

end

function StoryCardPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);
	self:initPanels()
end

function StoryCardPanel:initPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_5.png');
	self.bg:addTo(self.root);

	local topHight = NOTCH_HEIGHT + 143
	local menuHeight = 46;
	local menuPosY = StageSize.height/2-topHight;

	local viewBgHeight = StageSize.height-topHight-menuHeight/2-FOOTER_HEIGHT-20;
	local viewPosY = menuPosY-menuHeight/2-viewBgHeight/2;

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.root);
	bg:setPosition(0 , viewPosY-20);
	bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(604 , viewBgHeight));

	local layout = ccui.Layout:create()
    layout:setContentSize(cc.size(604,viewBgHeight-38))
	layout:addTo(self.root)
	layout:setPosition(-302,viewPosY-viewBgHeight/2+36-20);
    layout:setClippingEnabled(true);

	self.tableView = cc.TableView:create( cc.size(604,viewBgHeight-38));
	self.tableView:addTo(layout);
	self.tableView:setVerticalFillOrder(cc.TABLEVIEW_FILL_TOPDOWN);
	self.tableView:setAnchorPoint(0.5 , 0.5);
	--self.tableView:setPosition(-309 , -401);
    self.tableView:setDirection(cc.SCROLLVIEW_DIRECTION_VERTICAL);
    self.tableView:setDelegate()
	self.tableView:registerScriptHandler(handler(self,self.numberOfCellsInTableView),cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
	self.tableView:registerScriptHandler(handler(self,self.scrollViewDidScroll),cc.SCROLLVIEW_SCRIPT_SCROLL)
	self.tableView:registerScriptHandler(handler(self,self.tableCellTouched),cc.TABLECELL_TOUCHED)
    self.tableView:registerScriptHandler(handler(self,self.cellSizeForTable),cc.TABLECELL_SIZE_FOR_INDEX)
	self.tableView:registerScriptHandler(handler(self,self.tableCellAtIndex),cc.TABLECELL_SIZE_AT_INDEX)

	self.memberLabel = cc.Label:createWithTTF('请选择要出战的角色 (0/8)' , fonts.FONT_SIYUAN , 30)
    self.memberLabel:enableOutline(cc.c3b(0 , 0 , 0) , 2)
    self.memberLabel:addTo(self.root)
    self.memberLabel:setAnchorPoint(0,0.5)
    self.memberLabel:setPosition(-302,viewPosY+viewBgHeight/2+10)

	local bottomNode = cc.Node:create()
		:addTo(self.root)
		:setPosition(0,viewPosY-viewBgHeight/2-20);

	local line = cc.Sprite:create('image/commonImg/line.png');
	line:addTo(bottomNode);
	line:setContentSize(cc.size(604,1))
	line:setPosition(cc.p(0,33))

	local cardNumBg = cc.Sprite:create('image/commonImg/num_bg.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(211 , 23);

	cardNumBg = cc.Sprite:create('image/commonImg/num_icon.png');
	cardNumBg:addTo(bottomNode);
	cardNumBg:setPosition(209 , 16);
	
	self.curCardNum = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.curCardNum:addTo(bottomNode);
	self.curCardNum:setAnchorPoint(1 , 0.5);
	self.curCardNum:setPosition(289 , 16);

end

function StoryCardPanel:onEnter()
	self:onShow(1);
end

function StoryCardPanel:onShow(cardListIndex)
	--self.bg:setTexture('image/bg/Bg_1.png');
	cardListIndex = cardListIndex or 1;

	self.cardTemplateList = {}

	self.cards = {};
	local cardIndex = 0;
	for k, card in pairs(model.CardModel:GetAllCard()) do
		cardIndex = cardIndex + 1;
		self.cards[cardIndex] = card;
	end

	if #self.cards > 1 then
		model.CardModel:sortFuc2( self.cards )
	end

	self.curCardNum:setString(tostring(cardIndex)..'/'..tostring(CARD_MAX_NUM));

	self.cardListIndex = cardListIndex;
	self:updateData();

	self.root:setOpacity(50);
	self.root:runAction(cc.FadeIn:create(0.2))

	EventManager:FireEvent(EventType.UIFooterUpdateSelect,2);
	self.root:setVisible(true);
end

function StoryCardPanel:onResume()
	local offset = self.tableView:getContentOffset();
	local offsetX = offset.x;
	local offsetY = offset.y;

	self:onShow(self.cardListIndex);

	local maxInset = self.tableView:maxContainerOffset();
    local minInset = self.tableView:minContainerOffset();

    offsetX = math.min(offsetX, maxInset.x);
    offsetX = math.max(offsetX, minInset.x);
    offsetY = math.min(offsetY, maxInset.y);
    offsetY = math.max(offsetY, minInset.y);

    self.tableView:setContentOffsetInDuration(cc.p(offsetX , offsetY) , 0);
end

function StoryCardPanel:scrollViewDidScroll(table)

end

function StoryCardPanel:tableCellTouched(table)
	if self.curSelectPid then
		-- if self.cardListIndex==1 then
		-- 	UIManager:push('CardPanel' , self.curSelectPid, true, self.cards[self.cardListIndex]);
		-- else
		-- 	UIManager:push('CardPanel' , self.curSelectPid, true, nil);
		-- end
		
		AudioManager:playEffect(Conf.buttonClick);
		self.curSelectPid = nil;
	end
end

function StoryCardPanel:numberOfCellsInTableView(table)
	return self.tableViewCellMaxNum;
end

function StoryCardPanel:cellSizeForTable(table)
	return 604 , 425;
end

function StoryCardPanel:tableCellAtIndex(table,idx)
	
	local startNum = idx * 3 + 1;
	local curMaxNum = (idx+1) * 3;
	curMaxNum = curMaxNum > self.cardMaxNum and self.cardMaxNum or curMaxNum;
	local cell = table:dequeueCell()
	if not cell then
		cell = cc.TableViewCell:new();
		for i = 1 , 3 do
			local x = 101 + 201*(i-1);
			local y = 212.5;
			local bigCard = template.CardTemplate:create(cell);
			bigCard:setTouchEnabled(false);
			if startNum <= curMaxNum then
				local card = self.cards[startNum];
				bigCard:Init(card.pid);
				bigCard:checkRedDot();
				bigCard:checkFlag(card.no);
				if model.TeamModel:CurrentTeamHaveCard(card.pid) then
					bigCard:initCardInTeamState({Enum:getTable('CardTeamState').FIGHT});
				else
					bigCard:initCardInTeamState();
				end
	
				bigCard:setPosition(x, y);
				bigCard:addCardClickEventListener(handler(self, self.enterCardInfo));
				bigCard:setCardTag(card.pid);
				startNum = startNum + 1;

				local state = model.StoryModel:isInTempCardList( card.pid ) and 'select' or ''
				bigCard:setState(state)
				self.cardTemplateList[card.pid] = bigCard;
			else
				bigCard:setPosition(x, y);
				bigCard:setVisible(false);
			end
		end
	else
		local children = cell:getChildren();
		for i = 1 , 3 do
			local bigCard = children[i];
			if startNum <= curMaxNum then
				local card = self.cards[startNum];
				bigCard:setVisible(true);
				bigCard:Init(card.pid);
				bigCard:checkRedDot();
				bigCard:checkFlag(card.no);
				if model.TeamModel:CurrentTeamHaveCard(card.pid) then
					bigCard:initCardInTeamState({Enum:getTable('CardTeamState').FIGHT});
				else
					bigCard:initCardInTeamState();
				end
				bigCard:addCardClickEventListener(handler(self, self.enterCardInfo));
				bigCard:setCardTag(card.pid);

				local state = model.StoryModel:isInTempCardList( card.pid ) and 'select' or ''
				bigCard:setState(state)
				self.cardTemplateList[card.pid] = bigCard;

				startNum = startNum + 1;
			else
				bigCard:setVisible(false);
			end
		end
	end
	return cell;
end

function StoryCardPanel:updateData()

	self.cardList = {};
	self.cardMaxNum = #self.cards;
	self.tableViewCellMaxNum = math.ceil(self.cardMaxNum/3);

	self.tableView:reloadData();
end

function StoryCardPanel:enterCardInfo(sender , event)
	local pid = sender:getTag();
	if self.cardTemplateList[pid] then
		local ret, cnt = model.StoryModel:setTempCardList(pid)
		if ret == 1 then
			self.cardTemplateList[pid]:setState('select')
		elseif ret == 2 then
			self.cardTemplateList[pid]:setState('')
		end

		self.memberLabel:setString('请选择要出战的角色 ('..cnt..'/8)')
	end
end

function StoryCardPanel:onClickBack()
	UIManager:pop('StoryCardPanel');
end

function StoryCardPanel:onLeave()
	self.root:setVisible(false);
end

return StoryCardPanel;
