local StoryPlotPanel = class("StoryPlotPanel", require('app.layer.BaseLayer'))

local imgPath = 'image/plot/'
local bgPath = 'image/plot/'
local fonts = Enum:getTable("SystemFont")
local scheduler = cc.Director:getInstance():getScheduler();

function StoryPlotPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'StoryPlotPanel');
	self.useCCreator = false;
	-- self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;

	self.haveSwallowLayer = true;

	self.posX = 0;
	self.posY = 0;
end

function StoryPlotPanel:Init(root)
	self.root:setVisible(false);

	self:initPanels();
	self:registEvent();
end

function StoryPlotPanel:initPanels()

	self.bgStr = 'BG_polt_1';
	self.bg = cc.Sprite:create(imgPath .. 'BG_polt_1.png')
		:addTo(self.root)

	self.picSp = cc.Sprite:create()
		:addTo(self.root)

	self.skeletonNode = cc.Node:create()
		:addTo(self.root)

	self.sp2 = cc.Sprite:create(imgPath .. 'sp2.png')
		:setPosition(0, -StageSize.height/2+160)
		:addTo(self.root)

	self.nameLabel = cc.Label:createWithTTF('name', fonts.FONT_SIYUAN, 30)
		:setTextColor(cc.c3b(255, 255, 255))
		:setAnchorPoint(cc.p(0, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setPosition(-StageSize.width/2+105, -StageSize.height/2+298)
		:enableOutline(cc.c3b(0, 0, 0), 2)
		:addTo(self.root)

	self.sp1 = cc.Sprite:create(imgPath .. 'sp1.png')
		:setAnchorPoint(cc.p(0, 0.5))
		:setPosition(-StageSize.width/2+60, -StageSize.height/2+268)
		:addTo(self.root)

	-- self.skipBtn = ccui.Button:create(imgPath .. 'btn.png')
	-- self.skipBtn:setPosition(StageSize.width/2-65, StageSize.height/2-65)
	-- self.skipBtn:addClickEventListener(handler(self, self.clickSkip))
	-- self.skipBtn:addTo(self.root)

	-- self.skipSp = cc.Sprite:create(imgPath .. 'skip.png')
	-- 	:setPosition(StageSize.width/2-65, StageSize.height/2-65)
	-- 	:addTo(self.root)

	self.btnNode = cc.Node:create()
	self.btnNode:addTo(self.root)
	self.btnNode:setPosition(0, 200)

	local paddingH, paddingV = 60, 0
	local size = cc.size(StageSize.width, 240)
	self.text = ccui.Text:create('', fonts.FONT_SIYUAN, 30)
	self.text:setTextAreaSize(cc.size(size.width - paddingH * 2, size.height - paddingV * 2))
	self.text:setTextVerticalAlignment(cc.VERTICAL_TEXT_ALIGNMENT_TOP)
	self.text:setTextHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
	self.text:setAnchorPoint(0.5, 0)
	self.text:enableOutline(cc.c3b(0, 0, 0), 2)
	self.text:setPosition(0, -StageSize.height/2)
	self.text:addTo(self.root)
	self.text:setTouchEnabled(true)

	self.labelBlink = cc.Label:createWithTTF('|', fonts.FONT_SIYUAN, 30)
		:setPosition(258, -StageSize.height/2+155)
		:setVisible(false)
		:addTo(self.root)

	self.msgDialogId = 0;
end

function StoryPlotPanel:registEvent()
end

function StoryPlotPanel:onEnter(plotId, id, option, blink)
	self.option_love = nil;
	self.unableClick = false;
	self.msgDialogId = 0;
	self.delayTime = 0;

	bgPath = 'image/plot/'
	-- self.plot_type = plot_type or 0;
	AudioManager:SetState(Enum:getTable('GameState').Plot);

	local plot_id = plotId;
	local plotData = ResTable:getRowData('dungeon_plot', plotId);
	self.currentID = plotData.start_dialogue;

	-- init flag
	self.playingDialogue = false;

	self.showRole = 0;

	-- show panel
	self.root:setVisible(true)

	-- init dialogure
	if self.m_scheduleHandler == nil  then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.updateFrameDialog), 0, false);
	end

	if blink == true then
		local whiteSpr = cc.Scale9Sprite:create('image/blood/nblood_white.png')
		:addTo(self.root)
		:setContentSize(cc.size(StageSize.width, StageSize.height))
		local callfunc = cc.CallFunc:create(function() 
			
		end)
		whiteSpr:runAction( cc.Sequence:create(cc.FadeOut:create(1), callfunc ,cc.RemoveSelf:create()) )
	end
	
	if self:initDialogue() then
		self:playDialogue()
	else
		self:plotFinish()
	end

	self:addSwallowCallback(handler(self, self.clickDialogue))
end

function StoryPlotPanel:onLeave()
	self.playingDialogue = false;
	self.text:stopAllActions()
	self.root:setVisible(false)
	self.skeletonNode:removeAllChildren();
	if self.m_scheduleHandler then 
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil 
	end
	-- if self.plot_type == 2 then
	-- 	AudioManager:SetState(Enum:getTable('GameState').Fight);
	-- elseif self.plot_type == 3 then
	-- 	AudioManager:SetState(Enum:getTable('GameState').Main);
	-- 	AudioManager:playMusic(Conf.main, 2, true);
	-- end
end

-- 初始化对话
function StoryPlotPanel:initDialogue()
	if not self.currentID then
		return false;
	end
	local currentDialog = ResTable:getRowData('dungeon_dialogue', self.currentID);
	if not currentDialog then
		return false;
	end
	self.btnNode:removeAllChildren()
	
	self.currentDialog = currentDialog;
	self:createTextAction()
	-- print('fubin---self.currentDialog :', self.currentDialog.content)

	-- --bgm
	-- if currentDialog.bgm and currentDialog.bgm ~= '' then
	-- 	AudioManager:playMusic(Conf[tostring(currentDialog.bgm)], 4, true);
	-- end
	
	-- bg 
	if self.bgStr ~= currentDialog.show_bg then
		self.bgStr = currentDialog.show_bg;
		self.bg:setTexture(bgPath..self.bgStr..".png");
	end

	-- name
	local name = '';
	-- currentDialog.whosay = tonumber(currentDialog.whosay);
	-- if currentDialog.whosay ~= -1 and currentDialog.whosay ~= 0 then 
	-- 	if currentDialog.whosay == 991 then
	-- 		name = tostring(model.UserModel:getUserData().name);
	-- 	else
	-- 		name = ResTable:getValue('npc', currentDialog.whosay, 'name');
	-- 	end
	-- end
	name = tostring(model.UserModel:getUserData().name);
	self.nameLabel:setString(name);

	-- show role

	-- local aimShowOne;
	-- for _, v in pairs (currentDialog.npc) do
	-- 	if v >= 0 then
	-- 		aimShowOne = v;
	-- 		break;
	-- 	end
	-- end
	-- if aimShowOne and aimShowOne > 100 and aimShowOne < 200 then
	-- 	aimShowOne = aimShowOne - 100
	-- end
	-- if aimShowOne then
	-- 	if aimShowOne == 0 then
	-- 	elseif aimShowOne ~= self.showRole then
	-- 		local data = ResTable:getRowData('npc', aimShowOne);
	-- 		self.showRole = aimShowOne;
	-- 		local spine_res = data['live2d'];
	-- 		if spine_res ~= nil and cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
	-- 			self.skeletonNode:removeAllChildren();
	-- 			local skAnimation = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1);
	-- 			skAnimation:addTo(self.skeletonNode)		
	-- 			skAnimation:setAnimation(0, "idle", true)
	-- 			self.picSp:setVisible(false);
	-- 		else
	-- 			self.picSp:setVisible(true);
	-- 			self.picSp:setTexture('image/role/'..data['navi']..'.png');
	-- 		end
	-- 	end
	-- end

	self.picSp:setVisible(true);
	self.picSp:setTexture('image/role/'..'R035_Navi'..'.png');
	return true
end

-- 播放对话
function StoryPlotPanel:playDialogue()
	self.playingDialogue = true
	self.text:setString(" ")
	self:clickDialogue();
end

function StoryPlotPanel:nextDialog()
	local nextDialogList = self.currentDialog.next;
	if not nextDialogList or #nextDialogList<=0 then
		self:plotFinish()
		return;
	end

	if #nextDialogList == 1 then
		self.currentID = nextDialogList[1]
	
		self.labelBlink:stopAllActions()
		self.labelBlink:setVisible(false)

		if self:initDialogue() then
			self:playDialogue()
		else
			self:plotFinish()
		end

	else
		self:createSelBtns( nextDialogList )
	end
	
end

function StoryPlotPanel:createSelBtns( dialogList )
	self.btnNode:removeAllChildren()
	local i = 0
	for k,v in pairs(dialogList) do
		local dialogData = ResTable:getRowData('dungeon_dialogue', v);
		local btn = ccui.Button:create('image/storyLine/plot_btn.png')
		btn:setPosition(0, i*90)
		btn:addTo(self.btnNode)
		btn:setTag(v)
		btn:addClickEventListener(handler(self, self.clickSelPlot))
		local label = cc.Label:createWithTTF(dialogData.content, fonts.FONT_SIYUAN, 18)
		label:enableOutline(cc.c4b(125 , 125 , 125, 255) , 2);
		btn:setTitleLabel(label)
		i = i+1
	end
	self.unableClick = true;
end

function StoryPlotPanel:clickSelPlot(sender , event)
	local did = sender:getTag();
	self.currentDialog = ResTable:getRowData('dungeon_dialogue', did);
	self.msgDialogId = did;
	self:nextDialog()
	self.unableClick = false;
end

function StoryPlotPanel:plotFinish()
	local storyid = model.StoryModel:getCurStoryId();
	local storyData = model.StoryModel:getStoryDataById( storyid )
	local msg = {};
	msg.dungeon_no = self.msgDialogId;
	msg.resid = storyData.storyRoundId;
	msg.storyid = storyid;
	msg.type = 2;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_story_sel_plot , msg);

	UIManager:pop('StoryPlotPanel');
end

function StoryPlotPanel:clickDialogue()
	if self.unableClick == true then
		return
	end
	if self.playingDialogue then
		local str = self.currentDialog.content;
		str = string.gsub(str, "\\n", "\n")
		str = string.gsub(str, "\\t", "\t")
		self.text:setString(str)
		self:dialogureEndFunc()
	else
		self:nextDialog();
	end
end

function StoryPlotPanel:clickSkip()
	self:plotFinish()
end

function StoryPlotPanel:updateFrameDialog( dt )
	-- body
	if self.playingDialogue==false then
		return;
	else
		if self.delayTime == 0 then
			local current = self:nextUtf8strIndex(self.dialogStr, self.showDigPos)

			self.text:setString( string.sub(self.dialogStr, 1, current - 1) )

			self.showDigPos = current;

			if self.showDigPos >= self.diloglength then
				self:dialogureEndFunc();
			end

		end

		self.delayTime = self.delayTime + dt;
		if self.delayTime >= self.pTime then
			self.delayTime = 0;
		end
	end

end

function StoryPlotPanel:createTextAction( time )
	self.dialogStr = self.currentDialog.content;
	self.diloglength = string.len(self.dialogStr);
	self.pTime = time or 0.15;
	self.showDigPos = 1;
	self.delayTime = 0;

end

-- 打字机效果
function StoryPlotPanel:createTextActionSequence(target, time)
	time = time or 0.15
	local str = self.currentDialog.content;
	
	str = string.gsub(str, '\\n', '\n');
	str = string.gsub(str, '\\t', '\t');

	local actions = {}
	local length = string.len(str)
	local index = 1

	-- 一个一个蹦字
	while index <= length do
		local current = self:nextUtf8strIndex(str, index)
		table.insert(actions, cc.DelayTime:create(time))
		table.insert(actions, cc.CallFunc:create(
			function()
				target:setString(string.sub(str, 1, current - 1))
			end
			))
		index = current
	end
	table.insert(actions, cc.CallFunc:create(handler(self, self.dialogureEndFunc)))

	return cc.Sequence:create(actions)
end

function StoryPlotPanel:dialogureEndFunc()
	self.playingDialogue = false
	self.labelBlink:setVisible(true)
	self.labelBlink:runAction(cc.RepeatForever:create(cc.Sequence:create(
		cc.DelayTime:create(0.4),
		cc.FadeIn:create(0.1),
		cc.DelayTime:create(0.4),
		cc.FadeOut:create(0.1)
		)))
end

function StoryPlotPanel:nextUtf8strIndex(str, current)
	-- 汉字
	if string.byte(str, current) > 127 then
		return current + 3
	-- 字母
	elseif string.byte(str, current) < 128 then
		return current + 1
	end
end

function StoryPlotPanel:onClickBack()
	self:plotFinish();
end

return StoryPlotPanel;
