local TeamListPanel = class("TeamListPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");
local LockItems = Enum:getTable("levelLockItemIds");
local imagePath = 'image/mission/'
local TeamType = Enum:getTable('TeamType')

local DAILY_PROGRESS_MAX_WIDTH = 484;

function TeamListPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'TeamListPanel');

	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = false;
	self.bMove = false;
end

function TeamListPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function TeamListPanel:initPanels()

	local bg = cc.Sprite:create('image/bg/Bg_5.png');
	bg:addTo(self.root);

	local viewBg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	viewBg:addTo(self.root);
	viewBg:setPosition(0 , 0);
	viewBg:setCapInsets(cc.rect(11,4,18,21));
	viewBg:setContentSize(cc.size(604 , 904));

	-- local bg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	-- bg:addTo(self.roundRewardNode);
	-- bg:setPosition(0 , 9);
	-- bg:setCapInsets(cc.rect(11,11,34,28));
	-- bg:setContentSize(cc.size(583,877));

	local newRoot = cc.Node:create()
    newRoot:addTo(self.root)
    newRoot:setLocalZOrder(0)

    local bottomHeight = 900 --StageSize.height-400- NOTCH_HEIGHT;
    local scorllWidth = 600

    self.scorllview = ccui.ScrollView:create();
    self.scorllview:setTouchEnabled(true);
    self.scorllview:setBounceEnabled(true);
    self.scorllview:setDirection(ccui.ScrollViewDir.vertical);
    self.scorllview:setContentSize(cc.size(scorllWidth , bottomHeight));
    self.scorllview:setAnchorPoint(cc.p(0,0));
    self.scorllview:setScrollBarEnabled(false);
    self.scorllview:setPosition(-scorllWidth/2 , -bottomHeight/2);
    self.scorllview:addTo(newRoot);

    self.teamList = nil;

end

function TeamListPanel:onEnter()
	self:updateList();
	self.root:setVisible(true);
end

function TeamListPanel:updateList()
	local list = model.TeamModel:getAllTeamIdList();
	if not self.teamList then 
		self.teamList = {}
		local cnt = 0
		for k,v in pairs(list) do
			cnt = cnt+1
		end
		if cnt%2==1 then
			cnt = cnt+1
		end
		local totalHeight = 900;
		if math.ceil(cnt/2)*150 > totalHeight then
			totalHeight = math.ceil(cnt/2)*150;
		end
		self.scorllview:setInnerContainerSize( cc.size(StageSize.width , totalHeight) );

		local idx = 0;
		for k,v in pairs(list) do
			local teamItems = {}

			local row = math.ceil(cnt/2)
			local col = cnt%2;

			local node = cc.Node:create()
			node:addTo(self.scorllview)
			node:setPosition(col*300+150, totalHeight-(math.floor(idx/2))*150-75)
			local btn = ccui.Button:create('image/arena/arenaMult/am_left_tmbg.png')
			btn:addTo(node)
			btn:setTag(v)
			btn:setSwallowTouches(false)
			btn:addClickEventListener(handler(self , self.openBtnClick))

			local nameLabel = cc.Label:createWithTTF(model.TeamModel:getTeamNameById(v) ,  fonts.FONT_SIYUAN , 18);
	        nameLabel:addTo(node);
	        nameLabel:setPosition(0 , 25);

	        for j=1, TEAM_MAX_MEMBER do
	            local icon = require('app.template.ItemTemplate'):create();
	            icon:addTo(node);
	            icon:setScale(0.55)
	            -- icon:Init(ROLE_TYPE, card.no, card.star_lv, card.level, team[i])
	            -- icon:updateAltarLevelColor(card.oriLv)
	            icon:setPosition((j-1)*62-92, -30)
	            table.insert(teamItems, icon)
	        end
	        self.teamList[v] = {teamItems=teamItems}

	        cnt = cnt-1;
	        idx = idx+1;
		end
	end

	for k,v in pairs(list) do
        local teamInfo = model.TeamModel:getTeamById( v )
        for j=1, TEAM_MAX_MEMBER do
            if teamInfo[j] and teamInfo[j] > 0 then
                local card = model.CardModel:GetCard(teamInfo[j])
                if card then
	                self.teamList[v].teamItems[j]:Init(ROLE_TYPE, card.no, card.star_lv, card.level, teamInfo[j])
	                self.teamList[v].teamItems[j]:updateAltarLevelColor(card.oriLv)
	            else
	            	self.teamList[v].teamItems[j]:clearItem()
	            end
            else
                self.teamList[v].teamItems[j]:clearItem()
            end

        end
    end
end

function TeamListPanel:registerHandler()

	self.swallowLayer = cc.LayerColor:create(cc.c4b(0, 0, 0, 0));
	self.swallowLayer:setContentSize( cc.size(StageSize.width, StageSize.height) );
	self.swallowLayer:setPosition(-StageSize.width/2, -StageSize.height/2);
	self.swallowLayer:setLocalZOrder(100);
	self.swallowLayer:addTo(self.root);

	-- listener
	self.swallowListener = cc.EventListenerTouchOneByOne:create();
	local onTouchBegan = function(touch, event)
		self.bMove = false;
        return true
	end
	local onTouchMove = function(touch,event)
		local posEnd = touch:getLocation();
		local posStart = touch:getStartLocation()
		local dis = cc.pGetDistance(posStart,posEnd)
		if dis > 20 then
			self.bMove = true;
		end
	end
	local onTouchEnd = function(touch ,event )
	end
	self.swallowListener:setSwallowTouches(false);
	self.swallowListener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN);
	self.swallowListener:registerScriptHandler(onTouchMove, cc.Handler.EVENT_TOUCH_MOVED);
	self.swallowListener:registerScriptHandler(onTouchEnd, cc.Handler.EVENT_TOUCH_ENDED);
	local eventDispatcher = self.swallowLayer:getEventDispatcher();
	eventDispatcher:addEventListenerWithSceneGraphPriority(self.swallowListener, self.swallowLayer);
end


function TeamListPanel:openBtnClick(sender , event)
	if self.bMove then
		return
	end
	local tag = sender:getTag()

	local locked,lockLv = false, 0;
	local teamType = model.TeamModel:getTeamTypeById(tag)
	if teamType == TeamType.ARENA then
		locked,lockLv = tools.LevelLockTools:checkLockById(LockItems.LEVLE_LOCK_20)
	elseif teamType == TeamType.GUILD_BOSS then
		locked,lockLv = tools.LevelLockTools:checkLockById(LockItems.LEVLE_LOCK_10)
	elseif teamType == TeamType.EVENT then
		locked,lockLv = tools.LevelLockTools:checkLockById(LockItems.LEVLE_LOCK_23)
	elseif teamType == TeamType.TRIAL_R or teamType == TeamType.TRIAL_G or teamType == TeamType.TRIAL_B then
		locked,lockLv = tools.LevelLockTools:checkLockById(LockItems.LEVLE_LOCK_9)
		if locked==false then
			if model.TrialModel:getFinshTrialId() < 150 then
				Prompt:Show(string.format(LANG.UI_TRIAL_28 , 150));
				return;
			end
		end
	end
	if locked then
		Prompt:Show(string.format(LANG.UI_GUILD_43 , lockLv));
		return;
	end
	UIManager:push('TeamPanel', teamType)
	EventManager:FireEvent(EventType.UITeamUpdateRoundInfoOption , nil);

end

function TeamListPanel:onClickBack()
	UIManager:pop('TeamListPanel');

end

function TeamListPanel:onLeave()
	self.root:setVisible(false);
end

return TeamListPanel;
