local TeamPanel = class("TeamPanel", require('app.layer.BaseLayer'))
local scheduler = cc.Director:getInstance():getScheduler();

local TeamType = Enum:getTable('TeamType')
local fonts = Enum:getTable('SystemFont');
local MessageBoxType = Enum:getTable('MessageBoxType')
local RestrainType = Enum:getTable("FightRestraintType")
local FightType = Enum:getTable('FightType');

local imgPath = 'image/team/';

local rolePos = {
	[1] = {x = 172, y = 291, zorder = 80},
	[2] = {x = 0, y = 382, zorder = 80},
	[3] = {x = 0, y = 221, zorder = 100},
	[4] = {x = -159, y = 291, zorder = 50},
}

function TeamPanel:ctor(scene, sceneName)
	self.super.ctor(self, scene, sceneName, 'TeamPanel');
	
	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;

	self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
	self.showFooter = true;
end

function TeamPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self.roundInfoOption = nil;
	self.curTeam = {};
	self.tempTeam = {};
	self.roleViewList = {};
	self.teamType = 0;
	self.teamChangeCheck = false;
	self.selFormation = -1;
	self:initPanels();
	self:registEvent();
	
end

function TeamPanel:initPanels()

	-- bg
	self.bg = cc.Sprite:create('image/bg/Bg_5.png')
		:setLocalZOrder(-10)
		:addTo(self.root)

	-- team
	self.showRoleNode = cc.Node:create()
		:setPosition(0, 0)
		:setLocalZOrder(100)
		:addTo(self.root)
	
	self.showRoleList = {};
	for i=TEAM_MAX_MEMBER, 1, -1 do
		local template = require('app.template.TeamRoleTemplate'):create(self.showRoleNode);
		template:setPosition(rolePos[i].x, rolePos[i].y);
		template:setLocalZOrder(rolePos[i].zorder);
		template:Init(i);
		self.showRoleList[i] = template;
	end

	local touchListener = cc.EventListenerTouchOneByOne:create()
	touchListener:registerScriptHandler(function(touch, event)
		if not self.root:isVisible() then
			return false
		end
		local locationInNode = self.showRoleNode:convertToNodeSpace(touch:getLocation());
		for i=1, TEAM_MAX_MEMBER do
			local rectR = cc.rect(rolePos[i].x-50, rolePos[i].y, 100, 100)
			if cc.rectContainsPoint(rectR, locationInNode) then
				if self.tempTeam[i] ~= -1 then
					self.touchRole = i
					self.touchOffsetX = locationInNode.x - rolePos[i].x
					self.touchOffsetY = locationInNode.y - rolePos[i].y
					self.showRoleList[self.touchRole]:setLocalZOrder(200)
					self.moveOutPos = false
					return true
				end
			end
		end
		return false
	end, cc.Handler.EVENT_TOUCH_BEGAN)
	touchListener:registerScriptHandler(function(touch, event)
		local locationInNode = self.showRoleNode:convertToNodeSpace(touch:getLocation())
		if self.showRoleList[self.touchRole] then
			self.showRoleList[self.touchRole]:move(locationInNode.x-rolePos[self.touchRole].x-self.touchOffsetX, locationInNode.y-rolePos[self.touchRole].y-self.touchOffsetY)
		end
		if math.abs((rolePos[self.touchRole].x + self.touchOffsetX) - locationInNode.x) > 7 or
			math.abs((rolePos[self.touchRole].y + self.touchOffsetY) - locationInNode.y) > 7 then
			self.moveOutPos = true
		end
	end, cc.Handler.EVENT_TOUCH_MOVED)
	touchListener:registerScriptHandler(function(touch, event)
		if not self.showRoleList[self.touchRole] then
			return
		end
		local locationInNode = self.showRoleNode:convertToNodeSpace(touch:getLocation())
		local changePos
		for i=1, TEAM_MAX_MEMBER do
			if i ~= self.touchRole then
				if math.abs((rolePos[i].x + self.touchOffsetX) - locationInNode.x) <= 30 and
					math.abs((rolePos[i].y + self.touchOffsetY) - locationInNode.y) <= 30 then
					changePos = i
					break
				end
			end
		end
		if changePos then
			AudioManager:playEffect(Conf.teamingSwipe);
			self:switchRole(self.touchRole, changePos)
		elseif not self.moveOutPos then
			self:setRoleViewCardUnSelect(self.touchRole);
			self:onOffRole(self.touchRole)
		end
		self.showRoleList[self.touchRole]:move(0, 0)
		self.showRoleList[self.touchRole]:setLocalZOrder(rolePos[self.touchRole].zorder)
	end, cc.Handler.EVENT_TOUCH_ENDED)
	local eventDispatcher = self.showRoleNode:getEventDispatcher()
	eventDispatcher:addEventListenerWithSceneGraphPriority(touchListener, self.showRoleNode)

	-- equip btn
	self.equipBtn = ccui.Button:create('image/commonImg/btn_5_132_45.png')
	self.equipBtn:setPosition(230, StageSize.height/2-90- NOTCH_HEIGHT)
	self.equipBtn:addTo(self.root)
	self.equipBtn:addClickEventListener(function()
		if self:checkTeamChange(function() 
			UIManager:push('EquipWearPanel', self.teamType); 
		end) then
			return;
		end
		UIManager:push('EquipWearPanel', self.teamType);
	end)
	local label = cc.Label:createWithTTF(LANG.UI_EQUIP_7, fonts.FONT_SIYUAN, 18)
	label:enableOutline(cc.c4b(125 , 125 , 125, 255) , 2);
	self.equipBtn:setTitleLabel(label)

	-- info node
	self.infoNode = cc.Node:create();
	self.infoNode:addTo(self.root);

	self.infoBg = cc.Sprite:create(imgPath .. 'info_bg.png')
		:setPosition(0, 40)
		:addTo(self.infoNode)

	self.fpBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(-272, 64.5)
		:addTo(self.infoNode)

	self.hpBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(-272 , 15)
		:addTo(self.infoNode)

	self.atkBg = cc.Sprite:create('image/commonImg/arrow_3_bg.png')
		:setPosition(22, 15)
		:addTo(self.infoNode)

	self.fpWord = cc.Label:createWithTTF(LANG.UI_TEAM_21, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setPosition(-230, 65.5)
		:addTo(self.infoNode)

	self.fpLabel = cc.Label:createWithTTF('123.23MK', fonts.FONT_IMPACT, 20)
		:setPosition(276, 64.5)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)

	self.atkWord = cc.Label:createWithTTF(LANG.UI_TEAM_13, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setPosition(-230, 15)
		:addTo(self.infoNode)

	self.atkLabel = cc.Label:createWithTTF('233.43AJ', fonts.FONT_IMPACT, 20)
		:setPosition(-18, 15)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)

	self.hpWord = cc.Label:createWithTTF(LANG.UI_TEAM_14, fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(160, 200, 255))
		:setPosition(63, 15)
		:addTo(self.infoNode)

	self.hpLabel = cc.Label:createWithTTF('123.45K', fonts.FONT_IMPACT, 20)
		:setPosition(276, 15)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)
	--[[
	self.defWord = cc.Label:createWithTTF(LANG.UI_TEAM_22, fonts.FONT_IMPACT, 20)
		:enableOutline(cc.c3b(76, 76, 76), 2)
		:setPosition(-240, -45)
		:addTo(self.infoNode)

	self.defLabel = cc.Label:createWithTTF('123.45K', fonts.FONT_IMPACT, 20)
		:setTextColor(cc.c3b(76, 76, 76))
		:setPosition(-50, -45)
		:setAnchorPoint(cc.p(1, 0.5))
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_RIGHT)
		:addTo(self.infoNode)
	]]
	self.eventInfoNode = cc.Node:create()
		:setPosition(90, -500)
		:setVisible(false)
		:addTo(self.infoNode)

	self.eventMark = cc.Sprite:create('image/event/event_bg_12.png')
		:setPosition(-20, 0)
		:addTo(self.eventInfoNode)

	self.eventWord = cc.Label:createWithTTF(LANG.UI_EVENT_17, fonts.FONT_SIYUAN, 20)
		:setTextColor(cc.c3b(250, 0, 100))
		:setPosition(0, 0)
		:setAnchorPoint(0,0.5)
		:addTo(self.eventInfoNode)
	self.eventInfoNode:setPosition( StageSize.width/2 - self.eventWord:getContentSize().width - 20, -30 );

	-- role view 
	self.roleviewBg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	self.roleviewBg:addTo(self.root);
	self.roleviewBg:setPosition(0 , -274);
	self.roleviewBg:setCapInsets(cc.rect(11,4,18,21));
	self.roleviewBg:setContentSize(cc.size(604,364));

	local initData = {
		width = 594,
		height = 346,
		cellWidth = 158 ,
		cellHeight = 346,
		direction = 0,
		cellReloadFunc = handler(self,self.reloadCellPanel),
		cellTouchFunc = handler(self,self.cellItemTouch) ,

	}
	self.roleView = template.TableViewTemplate:create()
	self.roleView:initWithSizeAndItem(initData);
	self.roleView:getView():setPosition(cc.p(-297, -447));
	self.roleView:getView():addTo(self.root)

	-- btns
	self.btnOk = ccui.Button:create(imgPath .. 'btn_ok.png');
	self.btnOk:setPosition(269, 201);
	self.btnOk:addClickEventListener(handler(self, self.onOkClick));
	self.btnOk:addTo(self.root);

	local btnOkLabel =  cc.Label:createWithTTF(LANG.UI_L_SAVE, fonts.FONT_SIYUAN, 20);
	btnOkLabel:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);
	self.btnOk:setTitleLabel( btnOkLabel );

	self.btnAuto = ccui.Button:create(imgPath .. 'btn_auto.png');
	self.btnAuto:setPosition(197, 201);
	self.btnAuto:addClickEventListener(handler(self, self.onAutoRole));
	self.btnAuto:addTo(self.root);

	local btnAutoLabel =  cc.Label:createWithTTF(LANG.UI_L_AUTO, fonts.FONT_SIYUAN, 20);
	btnAutoLabel:enableOutline(cc.c4b(20 , 90 , 190 , 255) , 2);
	self.btnAuto:setTitleLabel( btnAutoLabel );

	-- 编队阵型
	local teamFormationNode = cc.Node:create();
	teamFormationNode:addTo(self.root);
	teamFormationNode:setPosition(-StageSize.width/2+82, 205)
	self.teamFormationNode = teamFormationNode;

	local formationBtn = ccui.Button:create(imgPath .. 'formation/team_formation_1.png',imgPath .. 'formation/team_formation_1.png');
	formationBtn:addClickEventListener(handler(self, self.onFormation));
	formationBtn:addTo(teamFormationNode);

	local formationImg = cc.Sprite:create('image/team/formation/team_formation_2.png')
	formationImg:addTo(teamFormationNode);
	formationImg:setPosition(-27,55);

	self.formationSp = cc.Sprite:create('image/team/formation/item_formation_1_2.png')
	self.formationSp:addTo(teamFormationNode);
	self.formationSp:setPosition(0,10);

	self.formationName =  cc.Label:createWithTTF(LANG.UI_TEAM_24, fonts.FONT_SIYUAN, 20);
	self.formationName:addTo(teamFormationNode);
    self.formationName:setPosition(cc.p(0,-35))
	self.formationName:setTextColor(cc.c4b(50, 50, 50, 255));
	self.formationName:enableOutline(cc.c4b(255 , 255 , 255 , 255) , 2);

	--local bg = cc.Sprite:create(imgPath..'formation/team_change.png');
	--bg:addTo(teamFormationNode);
	--bg:setPosition(-29 , 45);

	-- btn 
	local menuPos = {-226.5 , -75.5 , 75.5 , 226.5} --151
	self.btnList = {};
	local height = -70 -- -70
	for i=0, 3 do
		self.btnList[i] = {};
		local unTexture = 'image/commonImg/menu_1_151_46_u.png';
		local texture = 'image/commonImg/menu_1_151_46.png';
		
		local unSelect = ccui.Button:create(unTexture , unTexture);
		unSelect:addTo(self.root);
		unSelect:setTag(i);
		unSelect:addClickEventListener(handler(self , self.clickFightAttribute));
		unSelect:setPosition(menuPos[i+1] , height);
		self.btnList[i].unSelect = unSelect;

		local select = ccui.Button:create(texture , texture);
		select:addTo(self.root);
		select:setPosition(menuPos[i+1] , height);
		self.btnList[i].select = select;

		local light = cc.Sprite:create('image/commonImg/menu_light_1.png');
		light:addTo(select);
		light:setPosition(select:getContentSize().width/2 , select:getContentSize().height/2+23);

		if i == 0 then
			local label = cc.Label:createWithTTF('ALL' , fonts.FONT_SIYUAN , 20);
			label:addTo(unSelect);
			label:setPosition(unSelect:getContentSize().width / 2 , unSelect:getContentSize().height/2);
			label:setTextColor(cc.c3b(180 , 180 , 180));

			label = cc.Label:createWithTTF('ALL' , fonts.FONT_SIYUAN , 20);
			label:addTo(select);
			label:setPosition(select:getContentSize().width / 2 , select:getContentSize().height/2);
		else
			local defType = cc.Sprite:create('image/commonImg/def_type_'..i..'.png');
			defType:addTo(self.root);
			defType:setScale(0.5);
			defType:setPosition(menuPos[i+1] , height);
		end
	end

	local dropTip = cc.Node:create();
	dropTip:addTo(self.root);
	dropTip:setPosition(0 , 138);
	dropTip:setLocalZOrder(110);

	local dropBg = cc.Sprite:create('image/commonImg/tip_bg_1_547_33.png');
	dropBg:addTo(dropTip);

	local dropImg = cc.Sprite:create('image/commonImg/tip_2.png');
	dropImg:addTo(dropTip);
	self.dropImg = dropImg;

	local dropLabel = cc.Label:createWithTTF(LANG.UI_SETTING_24 , fonts.FONT_SIYUAN , 20);
	dropLabel:addTo(dropTip);
	dropLabel:setTextColor(cc.c3b(255 , 200 , 0));
	self.dropLabel = dropLabel;

	local width = dropImg:getContentSize().width+10;
	local labelWidth = dropLabel:getContentSize().width;
	local posX = -labelWidth / 2;
	dropImg:setPosition(posX , 0);
	dropLabel:setPosition(posX + width/2+labelWidth/2 , 0);

	self:initChangeTeamNode()

	--[[
	local allBtnLayout = ccui.Layout:create();
	allBtnLayout:setContentSize(cc.size(148, 98));
	allBtnLayout:setTag(0);
	allBtnLayout:setTouchEnabled(true);
	allBtnLayout:addClickEventListener(handler(self, self.clickRank));
	allBtnLayout:setPosition(StageSize.width/2-390-148, -StageSize.height/2+5);
	allBtnLayout:addTo(self.root);

	self.allBtn1 = cc.Sprite:create(imgPath .. 'allbtn1.png');
	self.allBtn1:setPosition(74, 49);
	self.allBtn1:addTo(allBtnLayout);
	self.allBtn2 = cc.Sprite:create(imgPath .. 'allbtn2.png');
	self.allBtn2:setPosition(74, 49);
	self.allBtn2:addTo(allBtnLayout);
	self.btnList = {};
	self.btnList[0] = {
		img1 = self.allBtn1,
		img2 = self.allBtn2,
	}
	--]]
end

function TeamPanel:initChangeTeamNode()
	self.changeTeamNode = cc.Node:create()
	self.changeTeamNode:addTo(self.root)
	self.changeTeamNode:setPosition(0, 400)

	local leftBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	leftBtn:addTo(self.changeTeamNode);
	leftBtn:setTag(1);
	leftBtn:addClickEventListener(handler(self , self.clickChangeTeam));
	leftBtn:setPosition(-270 , 0);

	local rightBtn = ccui.Button:create('image/card/avatar_arr.png' , 'image/card/avatar_arr.png');
	rightBtn:addTo(self.changeTeamNode);
	rightBtn:setTag(2);
	rightBtn:setScaleX(-1);
	rightBtn:addClickEventListener(handler(self , self.clickChangeTeam));
	rightBtn:setPosition(270 , 0);

	local nameBg = cc.Sprite:create('image/commonImg/tip_bg_1_309_28.png');
	nameBg:addTo(self.changeTeamNode);
	nameBg:setPosition(0, 150)

	local nameLabel = cc.Label:createWithTTF(LANG.UI_SETTING_24 , fonts.FONT_SIYUAN , 20);
	nameLabel:addTo(self.changeTeamNode);
	nameLabel:setTextColor(cc.c3b(255 , 200 , 0));
	nameLabel:setPosition(0, 150)
	self.nameLabel = nameLabel;
end

function TeamPanel:registEvent(  )
    EventManager:RegisterEvent(EventType.UITeamFormationChange , handler(self , self.eventChangeFormation));
    EventManager:RegisterEvent(EventType.UITeamUpdateRoundInfoOption , handler(self , self.updateRoundInfoOption));
    -- EventManager:RegisterEvent(EventType.UITeamOnShow , handler(self , self.onShow));
    EventManager:RegisterEvent(EventType.UITeamOnShow , handler(self , self.fromFormationPanel));
end

function TeamPanel:clickChangeTeam(sender,event)
	local tmpType = self.teamType;
	local t = sender:getTag();
	if t == 1 then
		tmpType = self.teamType - 1 
	else
		tmpType = self.teamType + 1 
	end
	if tmpType<11 then
		tmpType = 13
	elseif tmpType>13 then
		tmpType = 11
	end

	if self:checkTeamChange(function()
		self:onEnter(tmpType)
	end) then
	else
		self:onEnter(tmpType)
	end
	
end

function TeamPanel:onEnter(teamType)
	if teamType == TeamType.EVENT then
		model.EventModel:addRobotToCardModel()
	end
	self.root:setVisible(true);
	self.selFormation = -1;

	self.teamType = teamType or TeamType.COMMON;
	self.select_key = select_key;
	self.teamChangeCheck = true;
	self.roundInfoOption = nil;
	self.waitDownloadList = {}
	self:onShow();

	if self.teamType == TeamType.ARENA_1 or self.teamType == TeamType.ARENA_2 or self.teamType == TeamType.ARENA_3 then
		self.changeTeamNode:setVisible(true)
		self.nameLabel:setString(LANG.UI_TEAM_N7)
		if self.teamType == TeamType.ARENA_2 then
			self.nameLabel:setString(LANG.UI_TEAM_N8)
		elseif self.teamType == TeamType.ARENA_3 then
			self.nameLabel:setString(LANG.UI_TEAM_N9)
		end
	else
		self.changeTeamNode:setVisible(false)
	end

end

function TeamPanel:fromFormationPanel( data )
	--{formation=self.select_id, team=self.teamInfo, fchange=formationChange, tchange=teamChange }
	if data.tchange == true then
		self.tempTeam = data.team;
		for i=1, TEAM_MAX_MEMBER do
			if self.teamType == TeamType.EVENT then
				self.showRoleList[i]:setInfo(self.tempTeam[i], model.CardModel:getEventHp(self.tempTeam[i]))
			else
				self.showRoleList[i]:setInfo(self.tempTeam[i])
			end
		end
	end

	if data.fchange == true then
		self.selFormation = data.formation;

		local formationTexture = 'image/team/formation/item_formation_'..self.selFormation..'_2.png'
		self.formationSp:setTexture(formationTexture)

		local formationList = model.TeamModel:getTeamFormationList();
		for _k , itemData in pairs(formationList) do
			if itemData.id == self.selFormation then
				self.formationName:setString(tostring(itemData.name));
			end
		end
	end
end

function TeamPanel:onShow()
	self.teamIndex = 1;
	self.allRoleData = nil;
	self.pidInTeam = nil;

	self:updateTeamRole();
	self:updateBottomView();
	local attIdx = 0;
	local dropMsg = LANG.UI_SETTING_24;
	local sc = 1.0
	local dropIcon = 'image/commonImg/tip_2.png'
	if self.teamType == TeamType.TRIAL_R then
		attIdx = 2;
		dropMsg = LANG.UI_TRIAL_25;
		dropIcon = 'image/commonImg/def_type_2.png'
		sc = 0.5
	elseif self.teamType == TeamType.TRIAL_G then
		attIdx = 3;
		dropMsg = LANG.UI_TRIAL_25;
		dropIcon = 'image/commonImg/def_type_3.png'
		sc = 0.5
	elseif self.teamType == TeamType.TRIAL_B then
		attIdx = 1;
		dropMsg = LANG.UI_TRIAL_25;
		dropIcon = 'image/commonImg/def_type_1.png'
		sc = 0.5
	end
	self.dropImg:setTexture(dropIcon)
	self.dropImg:setScale(sc)
	self.dropLabel:setString(dropMsg)
	self.dropImg:setPosition( -self.dropLabel:getContentSize().width/2-20, 0)

	self:chooseFightAttribute(attIdx);
	self:updateEquipBtnRedDot();

	if self.teamType == TeamType.EVENT then
		self.eventInfoNode:setVisible(true)
	else
		self.eventInfoNode:setVisible(false)
	end
	self:eventChangeFormation()
end

function TeamPanel:updateEquipBtnRedDot()
	local state = model.EquipModel:getTeamSlotEquipActiveState(self.teamType);
	tools.RedDotTools:setRedDotToNode(self.equipBtn,state,nil,cc.p(120 , 33));
end

function TeamPanel:updateRoundInfoOption(option)
	self.roundInfoOption = option;
end

function TeamPanel:updateInitDownLoad()
	-- body
	for k,v in pairs(self.waitDownloadList) do
		if DownloadControler:checkNeedDownloadSpineModel(v) == false then
		else
			if self.teamType == TeamType.EVENT then
				self.showRoleList[k]:setInfo(self.curTeam[k], model.CardModel:getEventHp(self.curTeam[k]))
			else
				self.showRoleList[k]:setInfo(self.curTeam[k])
			end
			self.waitDownloadList[k] = nil
			break;
		end
	end

	if not next(self.waitDownloadList) then
		scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
		self.m_scheduleHandler = nil;
		WaitingPanel:Hide();
	end
end

function TeamPanel:updateTeamRole()
	local tid = model.TeamModel:getTeamIdByTeamType( self.teamType )
	self.curTeam = model.TeamModel:getTeamById(tid);

	for i=1, TEAM_MAX_MEMBER do
		local modelRes = ''
		if self.curTeam[i] and self.curTeam[i] ~= -1 then
			local resId = model.CardModel:getCardResIdByPid(self.curTeam[i]);
			local roleResData = ResTable:getRowData('role_resource', resId);
			modelRes = roleResData.model
		end
		if modelRes~='' and DownloadControler:checkNeedDownloadSpineModel(modelRes) == false then
			
			self.waitDownloadList[i] = modelRes
			
		else
			if self.teamType == TeamType.EVENT then
				self.showRoleList[i]:setInfo(self.curTeam[i], model.CardModel:getEventHp(self.curTeam[i]))
			else
				self.showRoleList[i]:setInfo(self.curTeam[i])
			end
		end
		self.tempTeam[i] = self.curTeam[i];
	end

	self:updateTop();

	if next(self.waitDownloadList) then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.updateInitDownLoad), 0, false);
		WaitingPanel:Show();
	end
end

-- 
function TeamPanel:setRoleTeamState( pid ,state,index )
	local stateStr = 'teamChange'
	if state == true  then
		if self.teamType == TeamType.EVENT then
			stateStr = 'eventTeamChange'
		else
			stateStr = 'teamChange'
		end
	else
		stateStr = 'none'
		index = -1
	end
	--if self.roleViewList[pid] then
	--	self.roleViewList[pid]:setSelect(state);
	--	self.roleViewList[pid]:setState(stateStr);
	--end
	self:checkInTeam(pid,index)

end
-- return 1,2,3,4..TEAM_MAX_MEMBER or -1
function TeamPanel:checkInTeam( pid,value )
	if self.pidInTeam == nil then
		local curTeam = self.tempTeam --model.TeamModel:getCurrentTeam();
		self.pidInTeam = {}
		for i=1, TEAM_MAX_MEMBER do
			self.pidInTeam[curTeam[i]] = i;
			-- print('=======checkInTeam   init === pid ',curTeam[i],'... index = ',i)
		end
	end

	if pid == nil or pid == -1 then
		return -1
	end

	if value ~= nil then
		self.pidInTeam[pid] = value;
		-- print('================checkInTeam == change == pid',pid,'   ..value = ',value)
	end

	if self.pidInTeam[pid] == nil  then
		self.pidInTeam[pid] = -1;
	end
	-- self.pidInTeam[pid] 
	return  self.pidInTeam[pid];

end

function TeamPanel:initAllRoleData(  )
	-- if needRefresh ~= true and  self.allRoleData ~= nil then
	-- 	return self.allRoleData
	-- end
	self:checkInTeam();
	-- local pidInTeam = {} -- 当前队伍pid
	
	local levelLimit = 1
	if self.teamType == TeamType.EVENT then
		levelLimit = 60
	elseif self.teamType == TeamType.GUILD_MISSION then
	else
		levelLimit = 1
	end


	local cardTmp = {};
	self.allRoleDataDefType = {} -- 按红绿蓝分组
	self.allRoleDataDefType[RestrainType.RED_BASE] = {}
	self.allRoleDataDefType[RestrainType.GREEN_BASE] = {}
	self.allRoleDataDefType[RestrainType.BLUE_BASE] = {}
	
	self.allRoleIndexByPid = {}	-- 按pid:index

	for k, v in pairs(model.CardModel:GetAllCard()) do
		local st = true;
		if self.teamType==TeamType.ARENA_1 or self.teamType==TeamType.ARENA_2 or self.teamType==TeamType.ARENA_3 then
			local b,a,tid = model.TeamModel:isRoleInArenaMulTeam(v.pid)
			if b and tid~=self.teamType then
				st = false;
			end
		end

		if self.teamType==TeamType.STORY then
			local b = model.StoryModel:isInCardList( v.pid )
			if not b then
				st = false;
			end
		end
		
		if st and v.level >= levelLimit then
			local pid = v.pid;
			local cardData = model.CardModel:GetCard(pid);
			local roleData = ResTable:getRowData('role', cardData.no);
			local rolePre = ResTable:getRowData('role_pre', roleData.role_id);
			-- local name = ResTable:getValue('role_info',roleData.role_id, 'name');

			local tempCardData = {
				pid 		= v.pid,
				no 			= cardData.no,
				role_id 	= roleData.role_id,
				rank 		= rolePre.rank,
				def_type 	= rolePre.def_type,
				level		= cardData.level,
				quality		= cardData.quality,
	
				-- quality 	= roleData.quality,
				-- name 		= name,
			}
			table.insert(cardTmp,tempCardData);
			-- self.allRoleDataByPid[pid] = tempCardData;
		end
	end
	model.CardModel:sortFuc2( cardTmp, self.teamType )

	self.allRoleData = cardTmp;

	for i=1,#cardTmp do
		local item = cardTmp[i]
		table.insert(self.allRoleDataDefType[item.def_type],item);

		-- print('===========defType == ',item.def_type);
		self.allRoleIndexByPid[item.pid] = i;

		--判断并下载 龙骨动画资源
		local resId = model.CardModel:getCardResIdByPid(item.pid);
		local roleResData = ResTable:getRowData('role_resource', resId);
		DownloadControler:checkNeedDownloadSpineModel(roleResData.model)

	end
	-- print(' len 1 = ',#self.al)

	-- for i,v in ipairs(self.allRoleDataDefType) do
	-- 	print(i,'........',#v);
	-- end

end

function TeamPanel:getRoleDataByPid( pid )
	if self.allRoleData == nil  then
		self:initAllRoleData();
	end
	if pid == nil or pid == -1 then
		return nil;
	end
	local index  = self.allRoleIndexByPid[pid]
	
	if index == nil then
		return nil;
	end

	return self.allRoleData[index];
end

function TeamPanel:getRoleDataByType( def_type )
	if self.allRoleData == nil  then
		self:initAllRoleData();
	end
	if def_type == nil or def_type == 0 then
		return self.allRoleData;
	end

	return self.allRoleDataDefType[def_type];
end

function TeamPanel:updateBottomView(  )
	self.cardSize = cc.size(158, 346);
	self.roleViewList = {};
	if self.allRoleData == nil  then
		self:initAllRoleData();
	end
end

function TeamPanel:reloadCellPanel( cell,index )
	local cellNode  = cell:getChildByTag(999);
	if nil == cellNode then
	 	cellNode = template.CardTemplate:create(cell);
		cellNode:setTouchEnabled(false);
		cellNode:setPosition(self.cardSize.width/2, self.cardSize.height/2);
		cellNode:setTag(999);
		cellNode:setScale(0.81);
	end
	local allCardData = self:getRoleDataByType(self.curDefType);

	local tempCardData = allCardData[index + 1];
	local pid = tempCardData.pid;

	cellNode:Init(pid);
	if self:checkInTeam(pid) > 0 then
		if self.teamType == TeamType.EVENT then
			cellNode:setState('eventTeamChange')
		else
			cellNode:setState('teamChange');
		end
		cellNode:setSelect(true);
	else
		cellNode:setState('none');
		cellNode:setSelect(false);
	end
	local lastTag = cell:getTag();
	-- print('..lastTag == ',lastTag,'... tag = ',pid)
	--self.roleViewList[lastTag] = nil;
	cell:setTag(pid);
	--self.roleViewList[pid] = cellNode;

	if self.teamType == TeamType.EVENT then
		local hpPercent = model.CardModel:getEventHp(pid) or 0;
		local hp = CardProManager:GetCardPro(pid).hp;
		cellNode:updateHp(hpPercent/10000, hp)
	else
		cellNode:hideHpNode()
	end
end

function TeamPanel:cellItemTouch( table,cell )
	-- body
	-- print("cell touched at tag: " .. cell:getTag())
	-- print('cell index == ',cell:getIdx())
	-- self.select_key = cell:getTag();
	self:chooseRole(cell:getIdx())
end

function TeamPanel:setRoleViewCardUnSelect(pos)
	if not self.showRoleList[pos] then
		return
	end
	local pid = self.showRoleList[pos]:getPid();
	if pid == -1 then
		return;
	end

	local allCardData = self:getRoleDataByType(self.curDefType);
	local allIndicesIdx = self.roleView:getAllIndicesCellIdx();
	for _k , idx in pairs(allIndicesIdx) do
		local card = allCardData[idx+1];
		if pid == card.pid then
			local cell = self.roleView:cellAtIndex(idx);
			if cell then
				local cellNode = cell:getChildByTag(999);
				if cellNode then
					cellNode:setState('none');
					cellNode:setSelect(false);
				end
			end
		end
	end
end

function TeamPanel:chooseRole( cellIndex )
	AudioManager:playEffect(Conf.teaming);
	local cell = self.roleView:cellAtIndex(cellIndex);
	local dataIndex = cellIndex + 1;

	local allCardData = self:getRoleDataByType(self.curDefType);
	local tempCardData = allCardData[dataIndex];
	-- dump(tempCardData,'chooseRole tempCardData = ');

	local pid = tempCardData.pid;
	local role_id = tempCardData.role_id;
	local card_no = tempCardData.no;
	local lastPos = self:checkInTeam(pid); -- -1 or teamIndex
	if lastPos ~= -1 then
		local ret = self:onOffRole(lastPos)
		if ret and cell then
			local cellNode = cell:getChildByTag(999);
			if cellNode then
				cellNode:setState('none');
				cellNode:setSelect(false);
			end
		end
		return
	end

	-- get aim pos
	local front2Back = nil
	for i=1, TEAM_MAX_MEMBER do
		if self.tempTeam[i] == -1 then
			front2Back = i
			break
		end
	end
	if not front2Back then
		Prompt:Show(LANG.UI_TEAM_32)
		return;
	end
	local back2Front = nil
	for i=TEAM_MAX_MEMBER, 1, -1 do
		if self.tempTeam[i] == -1 then
			back2Front = i
			break
		end
	end
	local job = ResTable:getValue('role_pre', tempCardData.role_id, 'job_type')

	local aim_pos
	if job == 1 then
		aim_pos = front2Back
	else
		aim_pos = back2Front
	end

	-- 判断roleId是否已在队伍
	for index=1, TEAM_MAX_MEMBER do
		local indexPid = self.showRoleList[index]:getPid();
		local indexData = self:getRoleDataByPid(indexPid);
		if indexData ~= nil and indexData.role_id == role_id then
			Prompt:Show(LANG.UI_TEAM_45);
			return;
		end
	end

	local resId = model.CardModel:getCardResIdByPid(pid);
	local roleResData = ResTable:getRowData('role_resource', resId);
	if DownloadControler:checkNeedDownloadSpineModel(roleResData.model) == false then
		Prompt:Show('this model is down loading ...');
		return;
	end

	local name = ResTable:getValue('role_info', role_id, 'name');
	--local quality = ResTable:getValue('role', card_no, 'quality');
	Prompt:Show(string.format(LANG.UI_TEAM_17, tostring(name), LANG.UI_CARD_QUALITY[tempCardData.quality]));


	local roleId = role_id--ResTable:getValue('role' , card_no, 'role_id');
	-- AudioManager:playCVStopBefor(roleId, 'Teaming');

	self.tempTeam[aim_pos] = pid;
	self.showRoleList[aim_pos]:setShowActType(1)
	if self.teamType == TeamType.EVENT then
		self.showRoleList[aim_pos]:setInfo(pid, model.CardModel:getEventHp(pid))
	else
		self.showRoleList[aim_pos]:setInfo(pid)
	end

	if cell then
		local cellNode = cell:getChildByTag(999);
		if cellNode then
			if self.teamType == TeamType.EVENT then
				cellNode:setState('eventTeamChange')
			else
				cellNode:setState('teamChange');
			end
			cellNode:setSelect(true);
		end
	end
	
	self:setRoleTeamState(pid,true,aim_pos)

	self:updateTop();

end

function TeamPanel:switchRole(pos1, pos2)
	if not self.showRoleList[pos1] or not self.showRoleList[pos2] then
		return
	end

	local pid1 = self.showRoleList[pos1]:getPid();
	local pid2 = self.showRoleList[pos2]:getPid();
	self.showRoleList[pos1]:checkStopShowAction();
	self.showRoleList[pos2]:checkStopShowAction();

	if self.teamType == TeamType.EVENT then
		self.showRoleList[pos1]:setInfo(pid2, model.CardModel:getEventHp(pid2));
		self.showRoleList[pos2]:setInfo(pid1, model.CardModel:getEventHp(pid1));
	else
		self.showRoleList[pos1]:setInfo(pid2);
		self.showRoleList[pos2]:setInfo(pid1);
	end

	self.tempTeam[pos1] = pid2;
	self.tempTeam[pos2] = pid1;

	if pid2 ~= -1 then
		self:setRoleTeamState(pid2, true, pos1)
	else
		self:setRoleTeamState(pid2, false, -1)
	end
	if pid1 ~= -1 then
		self:setRoleTeamState(pid1, true, pos2)
	else
		self:setRoleTeamState(pid1, false, -1)
	end

	self:updateTop();
end

function TeamPanel:onOffRole(pos)
	if not self.showRoleList[pos] then
		return false;
	end
	local pid = self.showRoleList[pos]:getPid();
	if pid == -1 then
		return false;
	end

	--if not self:canQuit(pid) then
	--	Prompt:Show(LANG.UI_TEAM_19);
	--	return false;
	--end
	self.showRoleList[pos]:setShowActType(2)
	self.showRoleList[pos]:setInfo(-1);
	self.tempTeam[pos] = -1;
	self:setRoleTeamState(pid,false,-1)

	self:updateTop();
	return true;
end

function TeamPanel:onAutoRole()
	-- temp sort
	local queue = {};
	for i=1, TEAM_MAX_MEMBER do
		queue[i] = {
			pid = -1,
			fp = '0',
			role_id = -1,
		};
	end
	local levelLimit = 1
	local colorLimit = 0;
	if self.teamType == TeamType.EVENT then
		levelLimit = 60
	elseif self.teamType == TeamType.TRIAL_R then
		colorLimit = RestrainType.RED_BASE;
	elseif self.teamType == TeamType.TRIAL_G then
		colorLimit = RestrainType.GREEN_BASE;
	elseif self.teamType == TeamType.TRIAL_B then
		colorLimit = RestrainType.BLUE_BASE;
	elseif self.teamType == TeamType.GUILD_MISSION then
		levelLimit = 1
	else
		levelLimit = 1
	end

	local count = 0
	local cardNoList = {};

	if self.teamType == TeamType.TRIAL_R or self.teamType == TeamType.TRIAL_G or self.teamType == TeamType.TRIAL_B then
		
		for k, v in pairs(self.allRoleDataDefType[colorLimit]) do
			count = count + 1
			local fp = ConvertDigitUnitToStr(CardProManager:GetCardPro(v.pid).fp);
			if not cardNoList[v.no] then
				cardNoList[v.no] = {
					fp = fp,
					pid = v.pid
				};
			else
				if HpCompare(cardNoList[v.no].fp, '<', fp) then
					cardNoList[v.no].fp = fp;
					cardNoList[v.no].pid = v.pid;
				end
			end
		end
	else
		for k, v in pairs(model.CardModel:GetAllCard()) do
			local hpLimit = true
			if self.teamType == TeamType.EVENT then
				hpLimit = (model.CardModel:getEventHp(v.pid) > 0)
			end
			
			if self.teamType == TeamType.STORY then
				local b , hp = model.StoryModel:isInCardList( v.pid )
				if not b or hp<=0 then
					hpLimit = false;
				end
			end

			if self.teamType==TeamType.ARENA_1 or self.teamType==TeamType.ARENA_2 or self.teamType==TeamType.ARENA_3 then
				local b,a,tid = model.TeamModel:isRoleInArenaMulTeam(v.pid)
				if b and tid~=self.teamType then
					hpLimit = false;
				end
			end
			if v.level >= levelLimit and hpLimit then
				count = count + 1
				local fp = ConvertDigitUnitToStr(CardProManager:GetCardPro(v.pid).fp);

				if not cardNoList[v.no] then
					cardNoList[v.no] = {
						fp = fp,
						pid = v.pid
					};
				else
					if HpCompare(cardNoList[v.no].fp, '<', fp) then
						cardNoList[v.no].fp = fp;
						cardNoList[v.no].pid = v.pid;
					end
				end
			end
		end
	end

	local noList = {};
	for no,data in pairs(cardNoList) do
		if HpCompare(data.fp, '>', queue[4].fp) then
			local aim_pos = 1;
			for i=TEAM_MAX_MEMBER-1, 1, -1 do
				if HpCompare(data.fp, '<=', queue[i].fp) then
					aim_pos = i+1;
					break;
				end
			end
			if aim_pos < TEAM_MAX_MEMBER then
				for i=TEAM_MAX_MEMBER, aim_pos+1, -1 do
					queue[i].pid = queue[i-1].pid;
					queue[i].fp = queue[i-1].fp;
					queue[i].role_id = queue[i-1].role_id;
				end
			end
			queue[aim_pos].pid = data.pid;
			queue[aim_pos].fp = data.fp;
			queue[aim_pos].role_id = no;
		end
	end

	if count <= 0 then
		Prompt:Show(LANG.UI_TEAM_29);
		return
	end

	local someoneAlive = false;
	for i=1, TEAM_MAX_MEMBER do
		if queue[i].pid ~= -1 then
			someoneAlive = true;
			break;
		end
	end
	if not someoneAlive then
		Prompt:Show(LANG.UI_TEAM_20);
		return;
	end	

	for i=1, TEAM_MAX_MEMBER do
		if self.tempTeam[i] ~= -1  then
			self:setRoleTeamState(self.tempTeam[i],false,-1)
		end
	end

	for i=1, TEAM_MAX_MEMBER do
		queue[i].index = i
	end

	table.sort(queue, function(first, second)
		if first.pid ~= -1 and second.pid == -1 then
			return true
		elseif first.pid == -1 and second.pid ~= -1 then
			return false
		elseif first.pid == -1 and second.pid == -1 then
			return first.index < second.index
		else
			local no1 = model.CardModel:GetCard(first.pid).no
			local role_id1 = ResTable:getValue('role', no1, 'role_id');
			local job1 = ResTable:getValue('role_pre', role_id1, 'job_type')
			local no2 = model.CardModel:GetCard(second.pid).no
			local role_id2 = ResTable:getValue('role', no2, 'role_id');
			local job2 = ResTable:getValue('role_pre', role_id2, 'job_type')
			if job1 < job2 then
				return true
			elseif job1 > job2 then
				return false
			else
				return first.index < second.index
			end
		end
	end)

	--for i=1, TEAM_MAX_MEMBER do --判断前4个动画资源是否下载完，如果不存在 就暂时不能自动生成编队
	--	local resId = model.CardModel:getCardResIdByPid(queue[i].pid);
	--	local roleResData = ResTable:getRowData('role_resource', resId);
	--	if DownloadControler:checkNeedDownloadSpineModel(roleResData.model) == false then
	--		Prompt:Show('this model is down loading ...');
	--		return;
	--	end
	--end

	local inTeamPid = {};
	for i=1, TEAM_MAX_MEMBER do
		self.tempTeam[i] = queue[i].pid;
		self.showRoleList[i]:setShowActType(1)
		if self.teamType == TeamType.EVENT then
			self.showRoleList[i]:setInfo(queue[i].pid, model.CardModel:getEventHp(queue[i].pid));
		else
			self.showRoleList[i]:setInfo(queue[i].pid);
		end
		if self.tempTeam[i] ~= -1  then
			self:setRoleTeamState(self.tempTeam[i],true,i)
			inTeamPid[self.tempTeam[i]] = 1;
		end
	end

	local allCardData = self:getRoleDataByType(self.curDefType);
	local allIndicesIdx = self.roleView:getAllIndicesCellIdx();
	for _k , idx in pairs(allIndicesIdx) do
		local card = allCardData[idx+1];
		if card then
			local cell = self.roleView:cellAtIndex(idx);
			if cell then
				local cellNode = cell:getChildByTag(999);
				if cellNode then
					if inTeamPid[card.pid] then
						if self.teamType == TeamType.EVENT then
							cellNode:setState('eventTeamChange')
						else
							cellNode:setState('teamChange');
						end
						cellNode:setSelect(true);
						inTeamPid[card.pid] = nil;
					else
						cellNode:setState('none');
						cellNode:setSelect(false);
					end
				end
			end
		end
	end
	self:updateTop();
	Prompt:Show(LANG.UI_TEAM_18);
end

function TeamPanel:onFormation( sender,eventType )
	UIManager:push('TeamFormationPanel', self.tempTeam, self.teamType);
end

function TeamPanel:updateTop()
	local curRestraint = model.TeamModel:getRestraintAmplificationByInfo( self.tempTeam )
	local fp = '0';
	local atk = '0';
	local hp = '0';
	for i=1, TEAM_MAX_MEMBER do
		if self.tempTeam[i] ~= -1 then
			local pro = CardProManager:GenProInTeam(self.tempTeam[i], nil, nil, self.teamType, {curRestraint=curRestraint, pos=i});
			fp = ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.fp), ConvertDigitUnitToStr(fp)));
			atk = ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.atk), ConvertDigitUnitToStr(atk)));
			hp = ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.hp), ConvertDigitUnitToStr(hp)));
		end
	end
	self.fpLabel:setString(fp);
	self.atkLabel:setString(atk);
	self.hpLabel:setString(hp);
end

function TeamPanel:onLeave()
	if self.teamType == TeamType.EVENT then
		model.EventModel:removeRobotFromCardModel()
	end
	self.root:setVisible(false);
end

function TeamPanel:clickRoleNode(pos)
	self:onOffRole(pos)
end

function TeamPanel:setFp(value)
	value = math.floor(value);
	for i=1, TEAM_MAX_MEMBER do
		if value ~= 0 then
			self.fpLabelList[i]:setVisible(true);
			local num = value % 10;
			self.fpLabelList[i]:setString(tostring(num));
			value = math.floor(value/10);
		else
			self.fpLabelList[i]:setVisible(false);
		end
	end
end

function TeamPanel:eventChangeFormation(  )
	local curFormation;
	if self.teamType == TeamType.EVENT then
		curFormation = model.TeamModel:getEventTeamFormation()
	elseif self.teamType == TeamType.ARENA then
		curFormation = model.TeamModel:getCurrentTeamFormation(6)
	elseif self.teamType == TeamType.GUILD_BOSS then
		curFormation = model.TeamModel:getCurrentTeamFormation(2)
	elseif self.teamType == TeamType.TRIAL_R or self.teamType == TeamType.TRIAL_G 
		or self.teamType == TeamType.TRIAL_B or self.teamType == TeamType.ARENA_1
		or self.teamType == TeamType.ARENA_2 or self.teamType == TeamType.ARENA_3 
		or self.teamType == TeamType.STORY then
		curFormation = model.TeamModel:getCurrentTeamFormation(self.teamType)
	else
		curFormation = model.TeamModel:getCurrentTeamFormation()
	end
	local formationTexture = 'image/team/formation/item_formation_'..curFormation..'_2.png'
	self.formationSp:setTexture(formationTexture)

	local formationList = model.TeamModel:getTeamFormationList();
	for _k , itemData in pairs(formationList) do
		if itemData.id == curFormation then
			self.formationName:setString(tostring(itemData.name));
		end
	end

	local show = model.TeamModel:checkTeamFormationRedDotState();
	tools.RedDotTools:setRedDotToNode(self.teamFormationNode,show,nil,cc.p(0 , 30))
end

function TeamPanel:onOkClick()
	AudioManager:playEffect(Conf.buttonClick);

	local isEmpty = true;
	for i = 1 , TEAM_MAX_MEMBER do
		if self.tempTeam[i] ~= -1 then
			isEmpty = false;
		end
	end
	if isEmpty then
		Prompt:Show(LANG.UI_TEAM_19);
		return
	end
	if self.selFormation>0 then
		model.TeamModel:reqChangeFormation(self.selFormation, self.teamType , self.tempTeam);
	else
		local teamId = model.TeamModel:getTeamIdByTeamType( self.teamType )
		local team = model.TeamModel:getTeamById(teamId);

		local change = false;
		if team then
			for i = 1 , TEAM_MAX_MEMBER do
				if team[i] ~= self.tempTeam[i] then
					change = true;
					break;
				end
			end
		end
		if change then
			model.TeamModel:sendChangeTeam(self.tempTeam, teamId);
		end
	end

	Prompt:Show(LANG.UI_TEAM_23);
	self:onOkBack();
end

function TeamPanel:onOkBack()
	UIManager:pop('TeamPanel')
	if self.teamType ~= TeamType.ARENA and self.teamType ~= TeamType.TRIAL and self.teamType ~= TeamType.CARD and self.teamType ~= TeamType.GUILD_BOSS then
		if self.teamType == TeamType.EVENT then
			UIManager:push('EventRoundInfoPanel',self.roundInfoOption);
		elseif self.teamType == TeamType.STORY then

		else
			if self.roundInfoOption then
				if self.roundInfoOption.fightType == FightType.CopyRound then
					UIManager:push('NewRoundInfoPanel',self.roundInfoOption);
				elseif self.roundInfoOption.fightType == FightType.MultArena then
					UIManager:push('ArenaMultTeamPanel');
				else
					UIManager:push('RoundInfoPanel',self.roundInfoOption);
				end
			end
		end
	end
	self.roundInfoOption = nil;
end

function TeamPanel:canQuit(pid)
	local totalNum = 0;
	for i=1, TEAM_MAX_MEMBER do
		if self.tempTeam[i] ~= -1 then
			totalNum = totalNum + 1;
		end
	end

	if totalNum <= 1 then
		return false;
	else
		return true;
	end
end

function TeamPanel:clickFightAttribute(sender, event)
	if self.curDefType == sender:getTag() then
		return;
	end
	if self.teamType == TeamType.TRIAL_R or self.teamType == TeamType.TRIAL_G or self.teamType == TeamType.TRIAL_B then
		Prompt:Show(LANG.UI_TRIAL_25)
		return
	end
	local btnTag = sender:getTag();
	self:chooseFightAttribute(btnTag);
end

function TeamPanel:chooseFightAttribute(attribute)

	for k, v in pairs(self.btnList) do
		v.unSelect:setVisible(k ~= attribute);
		v.select:setVisible(k == attribute);
	end

	self.curDefType = attribute or 0;
	local allCardData = self:getRoleDataByType(self.curDefType);
	self.roleView:reloadData(allCardData);
end

function TeamPanel:onClickBack()
	if self:checkTeamChange(function()
		UIManager:pop('TeamPanel');
	end) then
		return;
	end
	UIManager:pop('TeamPanel');
end

function TeamPanel:checkTeamChange(callback)
	local change = false;
	local tid = model.TeamModel:getTeamIdByTeamType( self.teamType )
	local team = model.TeamModel:getTeamById(tid);

	for i = 1 , TEAM_MAX_MEMBER do
		if team[i] ~= self.tempTeam[i] then
			change = true;
			break;
		end
	end

	if change or self.selFormation>0 then
		local isEmpty = true;
		for i = 1 , TEAM_MAX_MEMBER do
			if self.tempTeam[i] ~= -1 then
				isEmpty = false;
			end
		end
		if not isEmpty then
			MessageBox:create(LANG.UI_TEAM_15 , MessageBoxType.CANCEL_OK, function()
				if self.selFormation>0 then
					model.TeamModel:reqChangeFormation(self.selFormation, self.teamType , self.tempTeam);
				else
					local teamId = model.TeamModel:getTeamIdByTeamType( self.teamType )
					model.TeamModel:sendChangeTeam(self.tempTeam, teamId);
				end

				if callback then
					callback();
				end
			end, function()
				if callback then
					callback();
				end
			end);
		else
			MessageBox:create(LANG.UI_TEAM_46 , MessageBoxType.OK, function()
				if callback then
					callback();
				end
			end);
		end
		return true;
	end
	return false;
end

function TeamPanel:guideChoseRole( guide_step )
	local index = 1;
	if guide_step == 2 then
		index = 1;
	elseif guide_step == 9 then 
		index = 2;
	-- elseif guide_step == 4 then 
	-- 	index = 3;
	end
	self:chooseRole(index)
end

function TeamPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	if guideNo == 9 then
		local btn = self.roleView:cellAtIndex(1):getChildByTag(999);
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	elseif guideNo == 10 or guideNo == 43 then 
		local btn = self.btnOk;
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));
	elseif guideNo == 42 then 
		local btn = self.roleView:cellAtIndex(2):getChildByTag(999);
		pos = btn:getParent():convertToWorldSpace(cc.p(btn:getPosition()));

	end
	return pos;
end

return TeamPanel;
