local TipsEquipPanel = class("TipsEquipPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

function TipsEquipPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'TipsEquipPanel');

	self.useCCreator = false;
	self.isPopup = true;

end

function TipsEquipPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function TipsEquipPanel:initPanels()

	local bg = cc.Scale9Sprite:create('image/commonImg/bg_2_11_4_18_21.png');
	bg:addTo(self.root);
	bg:setPosition(6 , 8.5);
    bg:setCapInsets(cc.rect(11,4,18,21));
	bg:setContentSize(cc.size(484 , 563));

	local titleNode = GlobalFunctions:createTitle( LANG.UI_L_EQUIP_INFO_1,484);
	titleNode:addTo(self.root);
	titleNode:setPosition(6,312.5);

	self.title = titleNode:getChildByName('title');

	local nameBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	nameBg:addTo(self.root);
	nameBg:setPosition(5 , 245);
    nameBg:setCapInsets(cc.rect(11,11,34,28));
	nameBg:setContentSize(cc.size(462 , 59));

	local nameImg = cc.Sprite:create('image/commonImg/arrow_bg.png');
	nameImg:addTo(self.root);
	nameImg:setPosition(-188 , 245);

	self.equipName = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.equipName:addTo(self.root);
	self.equipName:setAnchorPoint(0 , 0.5);
	self.equipName:setPosition(-169 , 245);
	self.equipName:setTextColor(cc.c4b(160 , 200 , 255 , 255));

	self.equipIcon = template.ItemTemplate:create();
	self.equipIcon:setPosition(6 , 149);
	self.equipIcon:addTo(self.root);

	local infoBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	infoBg:addTo(self.root);
	infoBg:setPosition(8 , -49);
    infoBg:setCapInsets(cc.rect(11,11,34,28));
	infoBg:setContentSize(cc.size(462 , 257));

	self.equipInfo = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.equipInfo:addTo(self.root);
	self.equipInfo:setAnchorPoint(0 , 1);
	self.equipInfo:setContentSize(cc.size(423 , 124));
	self.equipInfo:setLineBreakWithoutSpace(true);
	self.equipInfo:setMaxLineWidth(423);
	self.equipInfo:setLineHeight(30);
	self.equipInfo:setPosition(-206 , 66);

	local attBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	attBg:addTo(self.root);
	attBg:setPosition(5 , -118);
    attBg:setCapInsets(cc.rect(11,11,34,28));
	attBg:setContentSize(cc.size(418 , 59));

	local attImg = cc.Sprite:create('image/commonImg/arrow_3_bg.png');
	attImg:addTo(self.root);
	attImg:setPosition(-180 , -118);

	self.attName = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.attName:addTo(self.root);
	self.attName:setAnchorPoint(0 , 0.5);
	self.attName:setTextColor(cc.c3b(160 , 200 , 255));
	self.attName:setPosition(-167 , -118);

	self.attValue = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 20);
	self.attValue:addTo(self.root);
	self.attValue:setAnchorPoint(1 , 0.5);
	self.attValue:setPosition(204 , -118);

	self.btnLabel = cc.Label:createWithTTF(LANG.UI_EQUIP_36 , fonts.FONT_SIYUAN , 20);
	self.btnLabel:enableOutline(cc.c3b(50 , 85 , 150) , 2);

	self.btn = ccui.Button:create('image/commonImg/btn_1_219_55.png' , 'image/commonImg/btn_1_219_55.png');
	self.btn:addTo(self.root);
	self.btn:setPosition(7 , -223);
	self.btn:addClickEventListener(handler(self , self.btnClick));
	self.btn:setTitleLabel(self.btnLabel);

end

function TipsEquipPanel:onEnter(data)
	if not data then
		return;
	end

	self.teamSlot = data.teamSlot;
	self.slot = data.slot;
	self.no = data.no;
	self.enterType = data.enterType;
	
	local equipItem = ResTable:getRowData('item' , self.no);
	
	if not equipItem then
		return;
	end

	local equipData = ResTable:getRowData('equip' , self.no);

	if not equipData then
		return;
	end
	
	self.btn:setTag(self.enterType);

	if self.enterType ~= 3 then
		self.btn:setVisible(true);
		self.btnLabel:setString(self.enterType == 1 and LANG.UI_EQUIP_36 or LANG.UI_EQUIP_37);
		
		self.title:setString(LANG.UI_EQUIP_7);
	else
		self.title:setString(LANG.UI_L_EQUIP_INFO_1);
		self.btnLabel:setString(LANG.COMMON_OK);
	end

	self.equipIcon:Init(self.no);
	self.equipName:setString(tostring(equipItem.name));
	self.equipInfo:setString(tostring(equipItem.info));

	local att = GetMathsValue(string.format(tostring(equipData['value1']), 1));
	if math.ceil(att) > 0 then
		self.attName:setString(tostring(LANG.UI_CARAD_ATT_TYPE[equipData['attribute1']]));
		local attFlag = (equipData['attribute1'] == 1 or equipData['attribute1'] == 2) and ' + ' or '';
		self.attValue:setString(attFlag..tostring(att));
	end
	
	self:addSwallowCallback(handler(self , self.onClickBack));
	
	self.root:setVisible(true);
end

function TipsEquipPanel:btnClick(sender , event)
	local tag = sender:getTag();

	if tag == 1 then
		local msg = {};
		msg.teamSlot = self.teamSlot;
		msg.slot = self.slot;
		msg.no = self.no;
		--NetworkBeforeValue.OldFp = model.TeamModel:getCurrentTeamFp();
		Network.NetworkManager:Send(Network.NetworkCmdType.req_equip_wear , msg);
		UIManager:pop('TipsEquipPanel');
		UIManager:pop('EquipWearSelectPanel');

	elseif tag == 2 then
		local msg = {};
		msg.teamSlot = self.teamSlot;
		msg.slot = self.slot;
		msg.no = self.no;
		--NetworkBeforeValue.OldFp = model.TeamModel:getCurrentTeamFp();
		Network.NetworkManager:Send(Network.NetworkCmdType.req_equip_takeoff , msg);
		UIManager:pop('TipsEquipPanel');

	else
		UIManager:pop('TipsEquipPanel');
	end
end

function TipsEquipPanel:onClickBack()
	UIManager:pop('TipsEquipPanel');
end

function TipsEquipPanel:onLeave()
	self.root:setVisible(false);
end

return TipsEquipPanel;
