--[[
local TrialRewardPanel = class("TrialRewardPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont");

local MessageBoxType = Enum:getTable('MessageBoxType')


function TrialRewardPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'TrialRewardPanel');

	self.useCCreator = false;
	self.isPopup = true;
end

function TrialRewardPanel:Init(root)
	self.root = root;
	self.root:setVisible(false);

	self:initPanels()
end

function TrialRewardPanel:initPanels()

	local bg = ccui.Layout:create();
	bg:setBackGroundImage('image/common/reward_bg1.png');
	bg:setTouchEnabled(true);
	bg:setContentSize(cc.size(636 , 1013));
	bg:setAnchorPoint(cc.p(0.5,0.5));
	bg:setPosition(1, -4);
	bg:addTo(self.root);

	bg = cc.Scale9Sprite:create('image/common/common_bg2.png'); --587 , 143
    bg:setCapInsets(CCRectMake(100,40,300,80));
	bg:setContentSize(cc.size(619, 861));
	bg:setPosition(cc.p(-1,24));
	bg:addTo(self.root);
	
	local titleLabel = cc.Label:createWithTTF(LANG.UI_TRIAL_6 , fonts.FONT_SIYUAN , 20);
	titleLabel:addTo(self.root);
	titleLabel:setPosition(0 , 488);
	titleLabel:setTextColor(cc.c3b(255 , 255 , 255));

	titleLabel = cc.Label:createWithTTF('- PRESENT BOX -' , fonts.FONT_SIYUAN , 9);
	titleLabel:addTo(self.root);
	titleLabel:setPosition(0 , 469);
	titleLabel:setTextColor(cc.c3b(0 , 142 , 219));

	self.trailRewardView = ccui.ScrollView:create();
	self.trailRewardView:setTouchEnabled(true);
	self.trailRewardView:setBounceEnabled(true);
	self.trailRewardView:setDirection(ccui.ScrollViewDir.vertical);
	self.trailRewardView:setContentSize(cc.size(619, 841));
	self.trailRewardView:setAnchorPoint(cc.p(0.5, 0.5));
	self.trailRewardView:setPosition(-1, 24);
	self.trailRewardView:setScrollBarEnabled(false);
	self.trailRewardView:addTo(self.root);

	self.cancelBtn = ccui.Button:create('image/commonImg/btn_bg2.png' , 'image/commonImg/btn_bg2.png');
	self.cancelBtn:addTo(self.root);
	self.cancelBtn:setPosition(-117 , -451);
	self.cancelBtn:addClickEventListener(handler(self , self.onClickBack));

	local cancelLabel = cc.Label:createWithTTF(LANG.UI_TRIAL_7 , fonts.FONT_SIYUAN , 20);
	cancelLabel:addTo(self.cancelBtn);
	cancelLabel:setPosition(self.cancelBtn:getContentSize().width/2 , self.cancelBtn:getContentSize().height/2);
	cancelLabel:setTextColor(cc.c3b(60 , 60 , 60));
	cancelLabel:enableOutline(cc.c4b(255 , 255 , 255 , 255) , 2);

	self.oneKeyBtn = ccui.Button:create('image/commonImg/btn_bg1.png' , 'image/commonImg/btn_bg1.png');
	self.oneKeyBtn:addTo(self.root);
	self.oneKeyBtn:setPosition(117 , -451);
	self.oneKeyBtn:addClickEventListener(handler(self , self.oneKeyBtnClick));

	local oneKeyLabel = cc.Label:createWithTTF(LANG.UI_TRIAL_8 , fonts.FONT_SIYUAN , 20);
	oneKeyLabel:addTo(self.oneKeyBtn);
	oneKeyLabel:setPosition(self.oneKeyBtn:getContentSize().width/2 , self.oneKeyBtn:getContentSize().height/2);
	oneKeyLabel:setTextColor(cc.c3b(255 , 255 , 255));
	oneKeyLabel:enableOutline(cc.c4b(19 , 93 , 188 , 255) , 2);

	EventManager:RegisterEvent(EventType.UITrialPanelGetReward, handler(self, self.retRoundReward));
end

function TrialRewardPanel:onEnter()
	self:updateData();
	self.root:setVisible(true);
end

function TrialRewardPanel:updateData()
	local roundNum = 0;
	self.roundRewards = {};
	
	for _k , reward in pairs(ResTable:getTable('trial_reward').contain) do
		table.insert(self.roundRewards , reward);
		roundNum = roundNum + 1;
	end
	
	if roundNum > 1 then
		table.sort(self.roundRewards , function(first , second)
			local firstGetReward  = model.TrialModel:getStatusById(first.id);
			local secondGetReward  = model.TrialModel:getStatusById(second.id);

			if firstGetReward == nil and secondGetReward ~= nil then
				return true;
			end

			if firstGetReward ~= nil and secondGetReward == nil then
				return false;
			end

			return first.id < second.id
		end);
	end

	local maxHeight = roundNum * 190;
	maxHeight = maxHeight < 841 and 841 or maxHeight;

	self.roundRewardList = {};

	self.trailRewardView:removeAllChildren();
	self.trailRewardView:setInnerContainerSize(cc.size(619, maxHeight));

	for _k , reward in ipairs(self.roundRewards) do
		local roundRewardTemplate = template.RoundTargetRewardTemplate:create(self.trailRewardView);
		roundRewardTemplate:InitTrial(reward);
		roundRewardTemplate:setTag(reward.id);
		roundRewardTemplate:setPosition(309.5 , maxHeight - 86.5 - 190 * (_k - 1));
		roundRewardTemplate:addClickEventListener(handler(self , self.rewardClick));
		self.roundRewardList[reward.id] = roundRewardTemplate;
	end
end

function TrialRewardPanel:oneKeyBtnClick()
	self:rewardClick(0 , 1);
end

function TrialRewardPanel:rewardClick(id , flag)
	if flag == 1 then
		model.TrialModel:reqTrialReward(id);
	else
		Prompt:Show(LANG.UI_TRIAL_9);
		self:onClickBack();
	end
end

function TrialRewardPanel:retRoundReward(msg)
	if msg.id and #msg.id > 0 then
		for _k , id in pairs(msg.id) do
			self.roundRewardList[id]:setGetRewardBtnVisible(false);
			tools.RedDotTools:pushFightSceneRedDot(4)
		end
		UIManager:push('RewardPanel',msg)
	else
		Prompt:Show(LANG.UI_TRIAL_10);
	end
end

function TrialRewardPanel:onClickBack()
	EventManager:FireEvent(EventType.UITrialPanelUpdateRewardTips , msg);
	UIManager:pop('TrialRewardPanel');
end

function TrialRewardPanel:onLeave()
	self.root:setVisible(false);
end

return TrialRewardPanel;
]]