local TrialSelPanel = class("TrialSelPanel" , require('app.layer.BaseLayer'))
local fonts = Enum:getTable("SystemFont")
local MessageBoxType = Enum:getTable('MessageBoxType')

function TrialSelPanel:ctor(scene, sceneName, name)
	self.super.ctor(self, scene, sceneName, 'TrialSelPanel');
	self.scene = scene;
	self.useCCreator = false;
	self.haveSwallowLayer = true;
    self.showHeaderType = Enum:getTable('HeaderType').HEAD_TITLE;
    self.showFooter = true;
end

function TrialSelPanel:Init(root)
	  -- body
      self.root = root;
      self.root:setVisible(false);
      self:InitPanels();
end

function TrialSelPanel:InitPanels()

	self.bg = cc.Sprite:create('image/bg/Bg_5.png');
	self.bg:addTo(self.root);

	local topHeight = NOTCH_HEIGHT+100
    local titltPosY = StageSize.height/2-topHeight;

	local viewBg = cc.Scale9Sprite:create('image/commonImg/bg_1_11_11_34_28.png');
	viewBg:addTo(self.root);
	viewBg:setAnchorPoint(0.5, 1)
	-- viewBg:setCapInsets(cc.rect(11,4,18,21));
	viewBg:setContentSize(cc.size(604 , 884));
	viewBg:setPosition(0, titltPosY + 20)

	local mainNode = cc.Node:create()
	mainNode:addTo(self.root)
	mainNode:setPosition(0, titltPosY)

	self.cellList = {}

	local posY = 0
	local trialList = GlobalFunctions:sortTableByKey(ResTable:getTable('trial_type').contain);
	for k,data in pairs(trialList) do
		local cellNode = cc.Node:create()
		cellNode:addTo(mainNode)

		local cellBtn = ccui.Button:create('image/trial/'..data.small_icon..'.png');--
		cellBtn:addTo(cellNode);
		cellBtn:setTag(data.no)
		cellBtn:addClickEventListener(handler(self , self.selClick));

		local bgHeight = cellBtn:getContentSize().height;
		cellNode:setPosition(0, posY-bgHeight/2)
		posY = posY - bgHeight - 20;

		
		local cellTitle = cc.Label:createWithTTF(data.name, fonts.FONT_SIYUAN, 30)
	        :setPosition(0, bgHeight/2-50)
	        :setAnchorPoint( cc.p(0.5, 0.5) )
	        :addTo(cellNode)

	    if data.no == 1 then
	    	cellTitle:setTextColor(cc.c3b(255, 255, 255))
	    	cellTitle:enableOutline(cc.c3b(0 , 0 , 0) , 2)
	    elseif data.no == 2 then
	    	cellTitle:setTextColor(cc.c3b(175, 215, 255))
	    	cellTitle:enableOutline(cc.c3b(0 , 40 , 145) , 2)
	    elseif data.no == 3 then
	    	cellTitle:setTextColor(cc.c3b(255, 180, 160))
	    	cellTitle:enableOutline(cc.c3b(165 , 0 , 0) , 2)
	    elseif data.no == 4 then
	    	cellTitle:setTextColor(cc.c3b(140, 255, 125))
	    	cellTitle:enableOutline(cc.c3b(10 , 80 , 0) , 2)
	    end

	    if data.color_icon ~= 0 then
			local cellIcon = cc.Sprite:create('image/commonImg/'..data.color_icon..'.png')--
			cellIcon:addTo(cellNode)
			cellIcon:setScale(0.5)
			cellIcon:setPosition(-20-cellTitle:getContentSize().width/2, bgHeight/2-50)
		end

		local openNode = nil;
		local openMsg = ''
		local lockMsg = ''
	    if data.week and data.week[1] ~= 0 then
		    openNode = cc.Node:create()
		    openNode:addTo(cellNode)
		    openNode:setPosition(0,-10)
		    -- local openBg = cc.Sprite:create('image/commonImg/btn_black_1_219_55.png')
		    -- openBg:addTo(openNode)
		    local bg3 = ccui.Scale9Sprite:create("image/commonImg/bg_c9_2_3.png")
				-- :setCapInsets(CCRectMake(10,10,117,94))
				:setContentSize(cc.size(545 , 190))
				:setPosition(cc.p(0 , 10))
				:addTo(openNode)
			for i=1,#data.week do
				openMsg = openMsg..LANG.UI_DATE_WEEK[data.week[i]]
				if i ~= #data.week then
					openMsg = openMsg..'/'
				end
			end
		    lockMsg = table.concat(data.week,"/")
		    local openLabel = cc.Label:createWithTTF(string.format(LANG.UI_TRIAL_27, openMsg), fonts.FONT_SIYUAN, 20)
		        :addTo(openNode)
		        :setTextColor(cc.c3b(255, 255, 0))
	    		:enableOutline(cc.c3b(0 , 0 , 0) , 2)

	    	local lockSpr = cc.Sprite:create('image/card/cardfuture/future_lock.png')
				:addTo(openNode)
				:setPosition(-40-openLabel:getContentSize().width/2,0)
		end
		local labelNode = cc.Node:create()
		labelNode:addTo(cellNode)
		labelNode:setPosition(0, -bgHeight/2+25)
		local labelBg = cc.Sprite:create('image/commonImg/time_bg_1.png')
		labelBg:addTo(labelNode)
		local curLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
	        :addTo(labelNode)
	        :setTextColor(cc.c3b(255, 255, 0))
	    	:enableOutline(cc.c3b(0 , 0 , 0) , 2)

	    self.cellList[ data.no ] = {curLabel=curLabel, openNode=openNode, openWeek=openMsg, lockMsg=lockMsg}
	end

	viewBg:setContentSize(cc.size(604 , -posY+20));

end

function TrialSelPanel:onEnter()
	-- self:addSwallowCallback(handler(self , self.onClickBack));
	self.root:setVisible(true);

	local w = os.date("%w",TimeManager:getCurTime());
	if w=='0' then
		w = '7'
	end
	for k,v in pairs(self.cellList) do
		if v.openNode then
			if string.find(v.lockMsg, w, 1) ~= nil then
				v.openNode:setVisible(false)
			else
				v.openNode:setVisible(true)
			end
		end
		local trial_id = model.TrialModel:getCurrentTrialId(k)
		v.curLabel:setString(string.format( LANG.UI_TRIAL_17,trial_id ))
	end
end

function TrialSelPanel:onLeave()
	self.root:setVisible(false);
end

function TrialSelPanel:onClickBack()
	UIManager:pop('TrialSelPanel');
end

function TrialSelPanel:selClick(sender , event)
    local tag = sender:getTag();
    if self.cellList[tag].openNode and self.cellList[tag].openNode:isVisible() then
    	Prompt:Show(string.format(LANG.UI_TRIAL_27, self.cellList[tag].openWeek));
		return;
	end
    UIManager:push('TrialPanel', tag);
end

function TrialSelPanel:getGuidePosition( guideNo )
	local pos = cc.p(0,0);
	self.guideTag = 1;
	if guideNo == 88 then
		for k,v in pairs(self.cellList) do
			if v.openNode and v.openNode:isVisible() == false then
				pos = v.openNode:getParent():convertToWorldSpace(cc.p(v.openNode:getPosition()));
				self.guideTag = k;
				break;
			end
		end
	end
	return pos;
end

function TrialSelPanel:clickGuide()
    UIManager:push('TrialPanel', self.guideTag);
end

return TrialSelPanel;
