local AccountModel = class("AccountModel")
local SDKLoginType = Enum:getTable("SDKLoginType")
local LoginPackageType = Enum:getTable("LoginPackageType")

-- 该模块记录内容包括
-- 账号相关:
--      账号类型
--      游客: 游客账号值
--      第三方账号: sdk id; sdk token; email
--      uid
--      aid
-- 默认服务器
-- 版本
-- 
function AccountModel:ctor()
    self:clearAll();
end

function AccountModel:clearAll()
    self.account_guest = nil;            -- 游客account值
    self.platform_id = nil;              -- sdk 唯一id 由服务器返回 默认为空
    self.platform_token = nil;           -- sdk token
    self.email = nil;


    self.unregtime = 0; -- 第三方登录 注销结束时间
    self.defaultServer = -1;
    self.uid = -1;
    self.aid = -1;
    self.gameVersion = '1.0';
end

function AccountModel:getAccountID()
    return self.aid
end

function AccountModel:getAccountUid( ... )
    return self.uid;
end

function AccountModel:getAccountServer()
    return self.server
end

function AccountModel:getAccountGuest()
    return self.account_guest or 'guestTest'
end

function AccountModel:getUnregisterTime(  )
    return self.unregtime or 0;
end

function AccountModel:getPlatformEmail( ... )
    return self.email or nil;
end

function AccountModel:setSDKLoginInfo( data )
    self.email = data.email
	self.platform_token = data.platform_token

    dump(jsonData,'setSDKLoginInfo === ')
end

function AccountModel:setGameVersion(version)
    if version then
        self.gameVersion = tostring(version);
    end
end

function AccountModel:getGameVersion()
    return self.gameVersion
end

function AccountModel:getGameCode()
    local packageName = sdkManager.SDKLuaHelper:getPackageName();
    return Enum:getTable('PackageNameCode')[tostring(packageName)] or 14;
end

function AccountModel:getOSVersion()
    local osVersion = cc.Application:getInstance():getOSVersion();
    return osVersion or 'default';
end

function AccountModel:getDeviceModel()
    local deviceModel = cc.Application:getInstance():getDeviceModel();
    return deviceModel or 'default';
end

function AccountModel:getServerInfo()
    return self.serverInfo
end

function AccountModel:saveCustomAccount( account,password )
    if account == '' then
        account = '0000000000';
    end
    if password == '' then
        password = '0000000000';
    end
    self.account = account;
    self.password = password
end

function AccountModel:sdkSignOut(platform_type, callback)
    if platform_type == SDKLoginType.LOGIN_GUEST or platform_type == SDKLoginType.LOGIN_ACCOUNT then
        return
    end
    sdkManager.SDKLuaHelper:signOut(platform_type, function()
        if callback then
            callback()
        end
    end)
end

--================================== req =====================================

function AccountModel:reqAnnouncement(  )
    local ret = model.UserDefaultModel:getLanguageSet()
    Network.NetworkManager:Send(Network.NetworkCmdType.req_announcement, {lg = ret} , true);
end

function AccountModel:reqServerInfo()
    Network.NetworkManager:Send(Network.NetworkCmdType.req_server_info, {});
end

function AccountModel:retServerInfo(msg)
    self.serverInfo = msg.serverInfo
end

function AccountModel:reqEnterServer(server)
    if not server then
        return
    end
    local msg = {};
    local platform_type = model.UserDefaultModel:getPlatformType();
    if platform_type == SDKLoginType.LOGIN_GUEST then
        if not self.account_guest then
            self.account_guest = model.UserDefaultModel:getGuestAccount();
        end
		msg = {
	        account_guest = self.account_guest,
        }
        if not msg.account_guest then
            msg.account_guest = '';
        end
	elseif platform_type == SDKLoginType.LOGIN_GOOGLE or
		platform_type == SDKLoginType.LOGIN_FACEBOOK or
		platform_type == SDKLoginType.LOGIN_IOS then
		msg = {
	        platform_token = self.platform_token,
	    }
        if not msg.platform_token then
            return false;
        end
	else
		return false;
	end
    msg.server = server;    
    msg.account_type = model.UserDefaultModel:getPlatformType()    
    msg.package_name = sdkManager.SDKLuaHelper:getPackageName()
    msg.packageLoginType = 1001;
    if LoginPackageType[msg.package_name] and LoginPackageType[msg.package_name][msg.account_type] then
        msg.packageLoginType = LoginPackageType[msg.package_name][msg.account_type]
    end

    self.server = server
    Network.NetworkManager:Send(Network.NetworkCmdType.req_enter_server, msg);
    return true;
end

function AccountModel:retAccountInfo(msg)
    self.account_type = msg.account_type
    if msg.account_type == SDKLoginType.LOGIN_GUEST then
        if msg.account_guest then
            model.UserDefaultModel:setGuestAccount(msg.account_guest)
            self.account_guest = msg.account_guest
        end
    else
        if msg.platform_id then
            self.platform_id = msg.platform_id
        end
        if msg.platform_email then
            self.email = msg.platform_email
        end
    end
end

function AccountModel:reqSDKBind( platform_type )
	local msg = {};

    msg.platform_token = self.platform_token
	msg.account_type = platform_type
    msg.account_guest = self.account_guest
    msg.package_name = sdkManager.SDKLuaHelper:getPackageName()

    cc.exports.platform_type = platform_type;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_guest_bind_platform, msg);
end

function AccountModel:retGuestBindPlatform(msg)
    cc.exports.platform_type = nil
    self.platform_id = msg.platform_id
    self.email = msg.platform_email
    model.UserDefaultModel:setPlatformType(msg.account_type);
	EventManager:FireEvent(EventType.UIRetGuestBind);
end

-- state 1 注销 ，其他 取消注销
function AccountModel:reqUnregisterSDK( state )
    local platformType = model.UserDefaultModel:getPlatformType()
    if platformType >=  SDKLoginType.LOGIN_ACCOUNT then
        Prompt:Show('当前登录方式未提供注销功能');
        return 
    end
	
	local req = {};
	req.uid = self.uid;
	req.account_type = platformType; -- 1 sdk google 2 sdk facebook 3 sdk ios,99 account,100 guest 
    req.platform_id = self.platform_id;
    req.state = state

	Network.NetworkManager:Send(Network.NetworkCmdType.req_unregister_platform, req);
end

function AccountModel:retServerUnregister( msg )
    self.unregtime = msg.unregtime or 0;
end

function AccountModel:reqLoginUid( server, uid )

	local req = {};
    req.uid = uid or -1;
    req.aid = self.aid;
    req.server = server or 1;
    if self.accountInfo and type(self.accountInfo) == 'table' then
        for k, v in pairs(self.accountInfo) do
            if v.server == req.server then
                req.uid = v.uid;
                break
            end
        end
    end
    req.package_name = sdkManager.SDKLuaHelper:getPackageName(); -- 本次登录包名，用于区分支付渠道

	Network.NetworkManager:Send(Network.NetworkCmdType.req_check_uid, req);
end

function AccountModel:retServerLoginUID( msg )
    self.aid = msg.aid;
    self.uid = msg.uid;
    model.UserDefaultModel:setDefaultServer(msg.server, '')
end

function AccountModel:reqChangeName(name)
end

function AccountModel:reqCreateNewAccount(name)
    local msg = {}
    msg.account_type = self.account_type
    if msg.account_type == SDKLoginType.LOGIN_GUEST then
        msg.account = self.account_guest
    else
        msg.account = self.platform_id
    end
    msg.server = self.server

    msg.name = name
    msg.country = sdkManager.SDKLuaHelper:getCountry()

    local package_name = sdkManager.SDKLuaHelper:getPackageName()
    msg.package_name = package_name
    msg.packageLoginType = 1001;
    if LoginPackageType[package_name] and LoginPackageType[package_name][msg.account_type] then
        msg.packageLoginType = LoginPackageType[package_name][msg.account_type]
    end

    Network.NetworkManager:Send(Network.NetworkCmdType.req_create_new_user, msg);
end

function AccountModel:reqRecoverAccount(account)
    local msg = {}
    msg.account = account;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_recover_account, msg);
end

return AccountModel;
