local AnnouncementModel = class("AnnouncementModel")
local scheduler = cc.Director:getInstance():getScheduler();

function AnnouncementModel:ctor()
	self.other_id = 0
	self.gm_id = 0
	self.other_msg_last_show_id = 0
end

function AnnouncementModel:Init(msg)
	self.gm_msg = {}
	self.other_msg = {}
	
	if self.m_scheduleHandler == nil then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.tryShowNextAnnouncement), 1, false);
	end
end

function AnnouncementModel:tryShowNextAnnouncement()
	-- 1 other 2 gm not important 3 gm important
	local curTime = math.floor(TimeManager:getCurTime())

	-- gm
	local show_gm_id = 0
	for k, v in pairs(self.gm_msg) do
		if v.end_time > curTime and v.begin_time < curTime then
			if v.id > show_gm_id then
				show_gm_id = v.id
			end
		end
	end
	if show_gm_id > 0 then
		local frequency = self.gm_msg[show_gm_id].frequency or 300
		if (curTime - self.gm_msg[show_gm_id].begin_time) % frequency == 0 then
			if self.gm_msg[show_gm_id].important == 1 then
				EventManager:FireEvent(EventType.AddImportantAnnouncement, self.gm_msg[show_gm_id].content)
			else
				EventManager:FireEvent(EventType.AddNewAnnouncement, self.gm_msg[show_gm_id].content, true)
			end
		end
	end

	-- other
	if self.other_msg_last_show_id < self.other_id then
		for id=self.other_msg_last_show_id+1, self.other_id do
			if self.other_msg[id] then
				self.other_msg_last_show_id = id
				self.canShowNextOtherMsg = false
				EventManager:FireEvent(EventType.AddNewAnnouncement, self.other_msg[id], false)
				break
			end
		end
	end
end

function AnnouncementModel:getAnnouncementGM()
	return self.gm_id
end

function AnnouncementModel:getLastID()
	return self.other_id
end

function AnnouncementModel:addAnnouncement(msg)
	if msg.announcement then
		for _, v in pairs(msg.announcement) do
			if v.id > self.other_id then
				self.other_id = v.id
			end

			local word = ''
			
			if v.t == 1 then
				local roleData = ResTable:getRowData('role', v.value1)
				local roleName = ResTable:getValue('role_info' , roleData.role_id , 'name')

				word = string.format(LANG.INFO_Announcement_WORD[1], v.name, roleName)
				
			elseif v.t == 2 then
				local skinData = ResTable:getRowData('role_avatar', v.value1)
				local roleName = ResTable:getValue('role_info', skinData.role_id, 'name')
				local skinName = skinData.name

				word = string.format(LANG.INFO_Announcement_WORD[2], v.name, roleName, skinName)

			elseif v.t == 3 then
				local bareCollectData = ResTable:getRowData('bare_collect', v.value1)
				local roleName = ResTable:getValue('role_info' , bareCollectData.role_id , 'name')

				word = string.format(LANG.INFO_Announcement_WORD[3], v.name, roleName)

			elseif v.t == 4 then
				word = string.format(LANG.INFO_Announcement_WORD[4], v.name, v.value1)

			elseif v.t == 5 then
				local roundName = ResTable:getValue('round', v.value1, 'name')
				word = string.format(LANG.INFO_Announcement_WORD[5], v.name, roundName)

			end
			if not self.other_msg[v.id] then
				self.other_msg[v.id] = word;
			end
		end
	end
	if msg.announcement_gm then
		for _, v in pairs(msg.announcement_gm) do
			if v.id > self.gm_id then
				self.gm_id = v.id
			end
			if not self.gm_msg[v.id] then
				self.gm_msg[v.id] = v;
			end
		end
	end
end

return AnnouncementModel;

