local ArenaModel = class("ArenaModel")
local restrainType = Enum:getTable("FightRestraintType")
local FightType = Enum:getTable('FightType');
function ArenaModel:ctor()
    self:clearAll();
end

function ArenaModel:clearAll()
    self.season = 0;
    self.score = 0;
    self.top_score = 0;
    self.challenge_num = 0;
    self.close_time = 0;
    self.arenaRankTableData = {};
    self.curArenaRankTabelData = nil;
    self.nextArenaRankTableData = nil;
    self.topArenaRankTableData = nil;
    self.rankList = {};
    self.arenaRewardTableSortData = {};
    self.rank_reward_stage = {};
    self.changeRankState = 0;
    self.initRedDotState = 0;
    self.nextSeasonData = nil;

    self.bInit = false;

    self.multArenaFightIdx = 0;
    self.multSelArenaFighterId = 0;
    self.multArenaFightResult = {};
    self.multArenaFightPower = {};
    self.multArenaFightOrders = {};
    self.multArenaHistory = {}
    self.multArenaHistoryId = 0;

    self.multArenaInfo = nil;
    self.oldArenaRankInfo = nil;
end

function ArenaModel:reqArenaInfo(  )
	Network.NetworkManager:Send(Network.NetworkCmdType.req_arena_info, {});
end

function ArenaModel:initArenaRedDot(initRedDotState)
    self.initRedDotState = initRedDotState or 0;
end

function ArenaModel:initData(data)
    print_ro(data)
    self:clearAll();

    for _k , tData in pairs(ResTable:getTable('arena_rank').contain) do
        table.insert(self.arenaRankTableData , tData);
    end

    if #self.arenaRankTableData > 1 then
        table.sort(self.arenaRankTableData , function(first , second)
            return first.point < second.point;
        end);
    end
    
    if data then
        self.bInit = true 
        if data.season then
            self.season = data.season;
        end

        if data.score then
            self.score = data.score;
        end

        if data.challenge_num then
            self.challenge_num = data.challenge_num;
        end

        if data.rank_reward_stage then
            for _k , no in pairs(data.rank_reward_stage) do
                self.rank_reward_stage[tonumber(no)] = 2;
            end
        end

        if data.top_score then
            self.top_score = data.top_score;
        end

        if data.rankList then
            self:updateRankList(data.rankList);
        end

        if data.close_time then
            self.close_time = data.close_time;
        end

        if data.next_season then
            self.nextSeasonData = data.next_season;
        end

        if data.fight_refresh_times then
            self.fightRefreshTimes = data.fight_refresh_times;
        end

        self:updateCurArenaRankData();
    end

end

function ArenaModel:hasInit(  )
    return self.bInit;
end

function ArenaModel:setInit(isInit)
    self.bInit = isInit
end

function ArenaModel:getAranaLastTime()
    local serverTime = math.floor(TimeManager:getCurTime());
    local lastTime , timeType = 0 , 0;
    if self.nextSeasonData then
        if self.nextSeasonData.next_open_time > 0 then
            lastTime = self.nextSeasonData.next_open_time - serverTime;
		    timeType = 2;
        end
    else
        if self.close_time > 0 then
            lastTime = self.close_time - serverTime;
            timeType = 1;
        end
    end
    return lastTime , timeType;
end

function ArenaModel:updateCurArenaRankData(refreshRankState)
    self.nextArenaRankTableData = nil;
    self.topArenaRankTableData = nil;
    local curArenaRankTabelData = nil
    local topScore = self.top_score < self.score and self.score or self.top_score;

    for _k , tData in ipairs(self.arenaRankTableData) do
        if self.score >= tData.point then
            curArenaRankTabelData = tData;
        elseif self.score < tData.point and not self.nextArenaRankTableData then
            self.nextArenaRankTableData = tData;
        end
    
        if topScore >= tData.point then
            self.topArenaRankTableData = tData;
        end
    end
    if not self.nextArenaRankTableData then
        self.nextArenaRankTableData = curArenaRankTabelData
    end

    if not self.topArenaRankTableData then
        self.topArenaRankTableData = curArenaRankTabelData
    end

    self.changeRankState = 0;
    if refreshRankState then
        if curArenaRankTabelData.rank > self.curArenaRankTabelData.rank then
            self.changeRankState = 1;
        elseif curArenaRankTabelData.order > self.curArenaRankTabelData.order then
            self.changeRankState = 2;
        end
    end

    self.curArenaRankTabelData = curArenaRankTabelData;
end

function ArenaModel:getArenaRankData()
    return self.arenaRankTableData;
end

function ArenaModel:getCurArenaRankData()
    return self.curArenaRankTabelData;
end

function ArenaModel:getNextArenaRankData()
    return self.nextArenaRankTableData;
end

function ArenaModel:getTopArenaRankData()
    return self.topArenaRankTableData;
end

function ArenaModel:getChangeRankState()
    return self.changeRankState;
end

function ArenaModel:setChangeRankState(state)
    self.changeRankState = state
end

function ArenaModel:getSeason()
    return self.season;
end

function ArenaModel:getScore()
    return self.score
end

function ArenaModel:getChallengeNum()
    return self.challenge_num;
end

function ArenaModel:getRankRewardState(no)
    return self.rank_reward_stage[no];
end

function ArenaModel:updateScore(score)
    self.score = score
    self:updateCurArenaRankData(true);
end

function ArenaModel:updateChallengeNum(challengeNum)
    if challengeNum then
        self.challenge_num = challengeNum
    end
end

function ArenaModel:updateRankRewardState(states)
    if states then
        self.rank_reward_stage = {};

        for _k , no in pairs(states) do
            self.rank_reward_stage[tonumber(no)] = 2;
        end
    end
end

function ArenaModel:getRankList(uid)
    if uid then
        return self.rankList[uid];
    end
    return self.rankList;
end

function ArenaModel:updateRankList(rankList)
    if rankList then
        self.rankList = {};
        for _k , data in pairs(rankList) do
            self.rankList[data.uid] = data;
            for _k , tData in ipairs(self.arenaRankTableData) do
                if data.score >= tData.point then
                    self.rankList[data.uid].rankData = tData;
                elseif data.score < tData.point then
                    break;
                end
            end

            if data.isRobot == 1 then
                local robotData = ResTable:getRowData('robot' , data.uid);
                data.name = robotData.name
                self.rankList[data.uid].arena_team = {};
                for _k , rData in ipairs(robotData.monster) do
                    local monsterStar = ResTable:getValue('monster_attribute' , rData[2] , 'star');
                    local monsterData = {
                        no =  rData[1],
                        att =  rData[2],
                        level =  robotData.lv,
                        star_lv =  monsterStar or 1,
                    };
                    table.insert(self.rankList[data.uid].arena_team , monsterData);
                end
            else
                local a_team = {}
                for k, v in pairs(self.rankList[data.uid].arena_team) do
                    if v and v.no then
                        a_team[tonumber(k)] = v
                    else
                        a_team[tonumber(k)] = {}
                        a_team[tonumber(k)]['no'] = -1
                    end
                end
                self.rankList[data.uid].arena_team = a_team

                local a_pro = {}
                for k, v in pairs(self.rankList[data.uid].pro) do
                    a_pro[tonumber(k)] = v
                end
                self.rankList[data.uid].pro = a_pro
            end
            
        end
    end
end

function ArenaModel:getFightRefreshTimes()
    return self.fightRefreshTimes;
end

function ArenaModel:setFightRefreshTimes(fightRefreshTimes)
    self.fightRefreshTimes = fightRefreshTimes
end

function ArenaModel:rewardRedDotActiveState(no)
    if self.curArenaRankTabelData==nil then
        return false;
    end

    if no then
        local point = ResTable:getValue('arena_reward' , no , 'point');
        return not self.rank_reward_stage[no] and self.curArenaRankTabelData.point >= point;
    end

    local rewardDatas = ResTable:getTable('arena_reward').contain.order;
    for _k , data in ipairs(rewardDatas) do
        if self.curArenaRankTabelData.point >= data.point and not self.rank_reward_stage[data.no] then
            return true
        end
    end
    return false;
end

function ArenaModel:reqArenaChallenge(uid)

    local itemNum = model.ItemModel:getItemNum(ARENA_ITEM);
    if self.challenge_num <= 0 and itemNum < Conf.arena_cost_item_num then
        Prompt:Show(LANG.UI_ARENA_10);
        return;
    end

    local arenaTeam = model.TeamModel:getArenaTeam();

    local haveTeam = false;
    for _k , pid in pairs(arenaTeam) do
        if pid ~= -1 then
            haveTeam = true;
            break;
        end
    end

    if not haveTeam then
        local defTeam = model.TeamModel:getCurrentTeam();
        model.TeamModel:sendChangeTeam(defTeam, 6);
    end

    --战斗请求
    local option = {};
    option.id = uid;
    FightManager:enterFight(Enum:getTable('FightType').Arena, option);
end


function ArenaModel:getRedDotStateArena(  )
	if self:getLockStateArena() then
		return false;
    end

    if not self.bInit  then
        if self.initRedDotState == 1 then
            return true;
        end
    end
    
	return false or self:rewardRedDotActiveState()
end
function ArenaModel:getLockStateArena()
    if tools.ConfigStateTools:checkLockByName('ArenaPanel') then
		return true
    end
    return self:getLockLevelStateArena(  );
end
function ArenaModel:getLockLevelStateArena(  )
    return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_20);		
end

function ArenaModel:getMultArenaInfo()
    if self.multArenaInfo==nil then
        Network.NetworkManager:Send(Network.NetworkCmdType.req_multarena_info, {});
    end
    return self.multArenaInfo;
end
function ArenaModel:onRetMultArenaInfo(msg)
    -- print_ro(msg, 'fubin--arenamultinfo')
    self.multArenaInfo = msg.arena;
    self:updateMultArenaMonster();
    EventManager:FireEvent('ArenaMultPanel:onShow');
end

function ArenaModel:getMultArenaSeason()
    return self.multArenaInfo.season;
end

function ArenaModel:refreshMultArenaEnemyList(bfree)
    local msg = {}
    msg.season = self.multArenaInfo.season;
    msg.bfree = bfree
    msg.rank = self.multArenaInfo.rank;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_multarena_refresh, msg);
end

function ArenaModel:onRetMultArenaRefresh(msg)
    if msg.arena and msg.arena.fight_refresh_num then
        self.multArenaInfo.fight_refresh_num = msg.arena.fight_refresh_num;
    end

    if msg.arena and msg.arena.rank then
        self.multArenaInfo.hour_update_time = msg.arena.hour_update_time;
        self.multArenaInfo.hour_reward = msg.arena.hour_reward;
        self.multArenaInfo.rank = msg.arena.rank;
        self.multArenaInfo.hour_reward_level = msg.arena.hour_reward_level;

        local bUpdataTop = false
        if msg.arena.top_rank then --更新前三名
            self.multArenaInfo.top_rank = msg.arena.top_rank;
            bUpdataTop = true;
        end
        EventManager:FireEvent('ArenaMultPanel:updateRankInfo', bUpdataTop);
        Prompt:Show(LANG.UI_ARENA_MU_23)
    end

    if msg.arena and msg.arena.rankList then
        self.multArenaInfo.rankList = msg.arena.rankList;
        self:updateMultArenaMonster();

        EventManager:FireEvent('ArenaMultListPanel:updateRankList');
    end
    
end

function ArenaModel:updateMultArenaMonster()
    -- body
    local keys = {'monster', 'monster_1', 'monster_2'}
    local fkeys = {'formation', 'formation_1', 'formation_2'}
    for k,v in pairs(self.multArenaInfo.rankList) do
        if v.isRobot == 1 then
            local robotData = ResTable:getRowData('robot' , v.uid);
            v.name = robotData.name
            v.team = {};
            for i=1,3 do
                v.team[i+10] = {arena_team={}, formation=v[ fkeys[i] ], monsterList=robotData[ keys[i]] }
                for _k , rData in ipairs(robotData[ keys[i] ]) do
                    local monsterStar = ResTable:getValue('monster_attribute' , rData[2] , 'star');
                    local monsterData = {
                        no =  rData[1],
                        att =  rData[2],
                        level =  robotData.lv,
                        star_lv =  monsterStar or 1,
                    };
                    table.insert(v.team[i+10].arena_team , monsterData);
                end
            end
        else
            for i=1,3 do
                v.team[i+10] = v.team[ tostring(i+10) ]
                v.team[ tostring(i+10) ] = nil;

                local team = v.team[i+10].arena_team;
                local restraint = {}
                restraint[restrainType.RED_BASE] = 0;
                restraint[restrainType.GREEN_BASE] = 0;
                restraint[restrainType.BLUE_BASE] = 0;
                for _k , t in pairs(team) do
                    if t.no ~= -1 then
                        local roleData = ResTable:getRowData('role', t.no);
                        local def_type = ResTable:getValue('role_pre', roleData.role_id, 'def_type');
                        restraint[def_type] = restraint[def_type] + 1;
                    end
                end
                local restraintString = tostring(restraint[restrainType.BLUE_BASE])..
                                        tostring(restraint[restrainType.RED_BASE])..
                                        tostring(restraint[restrainType.GREEN_BASE]);
                local restraintFactor = {}
                restraintFactor[1] = 0
                restraintFactor[2] = 0
                for k, v in pairs(ResTable:getTable('formation_attribute').contain) do
                    if restraint[v.color] == v.value then
                        for k1, v1 in pairs(v.attribute) do
                            if not restraintFactor[v1[1]] then
                                restraintFactor[v1[1]] = v1[2]
                            else
                                restraintFactor[v1[1]] = restraintFactor[v1[1]] + v1[2]
                            end
                        end
                    end
                end
                v.team[i+10].restraintFactor = restraintFactor
                v.team[i+10].restraintString = restraintString
            end
            
        end
    end

    self:makeEnemysFightPower()
    -- print_ro(self.multArenaInfo.rankList, 'fubin---mult')

end

function ArenaModel:onRetMultArenaRwd(msg)
    if msg.arena then
        self.multArenaInfo.hour_update_time = msg.arena.hour_update_time;
        self.multArenaInfo.hour_reward = msg.arena.hour_reward;
    end
    EventManager:FireEvent('ArenaMultPanel:updateRewardItemProgress');
    UIManager:push('RewardPanel',msg)
end

function ArenaModel:getMultArenaHistory()
    return self.multArenaHistory;
end
function ArenaModel:onRetMultArenaHistory(msg)
    if msg.noticeList then
        local cnt = #msg.noticeList;
        for i=1,cnt do
            table.insert(self.multArenaHistory, 1, msg.noticeList[cnt-i+1])
            if msg.noticeList[cnt-i+1].id > self.multArenaHistoryId then
                self.multArenaHistoryId = msg.noticeList[cnt-i+1].id;
            end
        end
        -- self.multArenaHistory = msg.noticeList;
    end
    EventManager:FireEvent('ArenaMultHistoryPanel:onShow');
end
function ArenaModel:getMultArenaHistoryId()
    return self.multArenaHistoryId
end

function ArenaModel:getMultAranaLastTime()
    local serverTime = math.floor(TimeManager:getCurTime());
    local lastTime , timeType = 0 , 0;
    if self.multArenaInfo then
        if self.multArenaInfo.next_season then
            if self.multArenaInfo.next_season.next_open_time > 0 then
                lastTime = self.multArenaInfo.next_season.next_open_time - serverTime;
                timeType = 2;
            end
        else
            if self.multArenaInfo.close_time > 0 then
                lastTime = self.multArenaInfo.close_time - serverTime;
                timeType = 1;
            end
        end
    end
    return lastTime , timeType;
end


function ArenaModel:setMultArenaFightOrders( orders )
    self.multArenaFightOrders = orders;
    self.multArenaFightIdx = 0;
    self.multArenaFightResult = {}
    self.multArenaFightPower = {}
    self:continueNextFight();
end
function ArenaModel:getMultArenaTeamIdByIdx( idx )
    return self.multArenaFightOrders[idx]
end

function ArenaModel:setMultArenaFight( id )
    self.multSelArenaFighterId = id;
end
function ArenaModel:getMultArenaEnemyPlayer()
    for _k , rankData in pairs(self.multArenaInfo.rankList) do
        if rankData and rankData.uid==self.multSelArenaFighterId then
            return rankData;
        end
    end
    return nil;
end

function ArenaModel:getMultArenaEnemysTeam(enemyId, idx)
    local ret = {}
    for k,v in pairs(self.multArenaInfo.rankList) do
        if v.uid == enemyId then
            ret.arena_team =  v.team[idx+10].arena_team
            ret.formation =  v.team[idx+10].formation
            ret.isRobot = v.isRobot;
            if v.isRobot == 0 then
                ret.restraintString = v.team[idx+10].restraintString
                ret.restraintFactor = v.team[idx+10].restraintFactor
                ret.equip = v.equip;
                ret.baseskill = v.baseskill;
            end
        end
    end
    return ret;
end

function ArenaModel:makeEnemysFightPower()
    local x = 1
    for k,v in pairs(self.multArenaInfo.rankList) do
        local powerList = {}
        for i=1,3 do
            local teamData = v.team[i+10];
            local tmpTeam = teamData.arena_team;
            local monsterList = teamData.monsterList;
            local totalFp = '0';
            local fp = 0;
            if tmpTeam then
                local restraintFactor,enemyRestraintString, roleListData = nil, '000', nil;
                if v.isRobot == 1 and monsterList then
                    restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(monsterList);
                else
                    roleListData = self:getMultArenaEnemysTeam(v.uid, i);
                end
                
                for j=1,TEAM_MAX_MEMBER do
                    if tmpTeam[j] and tmpTeam[j].no>0 then
                        if v.isRobot == 1 then
                            local pro = CardProManager:GenMonsterPro(tmpTeam[j].no, tmpTeam[j].att,restraintFactor);
                            if pro then
                                totalFp = HpPlus(totalFp, ConvertDigitUnitToStr(pro.fp));
                            end
                        else
                            local pro = CardProManager:GenArenaRoleMonsterPro(roleListData , j);
                            fp = fp + tonumber(ConvertDigitUnitToStr(pro.fp))
                        end
                    end
                end

                if v.isRobot == 1 then
                    powerList[i+10] = tonumber(totalFp)
                    -- self.enemyPFLabelList[i]:setString('战斗力:'..totalFp)
                else
                    powerList[i+10] = fp
                    -- self.enemyPFLabelList[i]:setString('战斗力:'..fp)
                end
            end

        end
        v.powerList = powerList;

    end

end

function ArenaModel:setMultArenaResult(v)
    table.insert(self.multArenaFightResult, v)
end

function ArenaModel:getMultArenaResult()
    return self.multArenaFightResult;
end

function ArenaModel:setMultArenaPower(v)
    table.insert(self.multArenaFightPower, v)
end

function ArenaModel:getMultArenaPower()
    return self.multArenaFightPower;
end

function ArenaModel:continueNextFight()
    -- body
    self.multArenaFightIdx = self.multArenaFightIdx+1;
    FightManager:enterFight(FightType.MultArena, {id = self.multSelArenaFighterId, 
                                                subIndex = self.multArenaFightIdx, 
                                                selfTeamId=self.multArenaFightOrders[self.multArenaFightIdx]})
end

function ArenaModel:getOldMultArenaInfo()
    return self.oldArenaRankInfo;
end

function ArenaModel:getNumPerHourByRank( rank )
    -- body
    for k, v in pairs(ResTable:getTable('multiple_arena_rank_reward').contain) do
        if v.rank[1] <= rank and rank<=v.rank[2] then
            return v.reward;
        end
    end
    return 0
end

function ArenaModel:setMultAFightResult(msg)
    if msg.arena and msg.win_res==1 then
        self.oldArenaRankInfo = {}
        self.oldArenaRankInfo.rank = self.multArenaInfo.rank;
        self.oldArenaRankInfo.hour_reward_level = self.multArenaInfo.hour_reward_level;

        self.multArenaInfo.hour_update_time = msg.arena.hour_update_time;
        self.multArenaInfo.hour_reward = msg.arena.hour_reward;
        self.multArenaInfo.rank = msg.arena.rank;
        self.multArenaInfo.hour_reward_level = msg.arena.hour_reward_level;
    end
    local bUpdataTop = false
    if msg.arena and msg.arena.top_rank then --更新前三名
        self.multArenaInfo.top_rank = msg.arena.top_rank;
        bUpdataTop = true;
    end
    if msg.arena and msg.arena.challenge_num then --更新前三名
        self.multArenaInfo.challenge_num = msg.arena.challenge_num;
    end
    
    -- if msg.arena and msg.arena.rankList then
    --     self.multArenaInfo.rankList = msg.arena.rankList;
    --     self:updateMultArenaMonster();
    -- end
    EventManager:FireEvent('ArenaMultPanel:updateRankInfo', bUpdataTop);
end


return ArenaModel
