local BattlepassModel = class("BattlepassModel")
local scheduler = cc.Director:getInstance():getScheduler();

BattlepassModel.BattlepassMenuType = {
	DAILY 	= 1,
	WEEK 	= 2,
	ULTIMATE= 3,
}

function BattlepassModel:ctor()
	self:clearAll()
end

function BattlepassModel:clearAll(  )
	self.hasInit = false;
	self.battlepassId = 0;
	self.rewardState = {};
	self.battlepassMissionData = {};
	self.rewardData = {};
	self.battlepassOpenTime = 0;
	self.battlepassCloseTime = 0;
	self.battlepassShowTime = 0;
	self.specialState = 0;
	self.curBattlepassRewardNo = 0;
	self.curBattlepassPoint = 0;
end

function BattlepassModel:isInit()
	return self.hasInit;
end

function BattlepassModel:setInit(Init)
	self.hasInit = init
end

function BattlepassModel:Init(data)
	if data.battlepass_id then
		self.battlepassId = data.battlepass_id;
	end

	if data.special_state then
		self.specialState = data.special_state;
	end

	if data.battlepass_open_time then
		self.battlepassOpenTime = data.battlepass_open_time
	end

	if data.battlepass_close_time then
		self.battlepassCloseTime = data.battlepass_close_time
	end

	if data.battlepass_show_time then
		self.battlepassShowTime = data.battlepass_show_time
	end

	if data.battlepass_point then
		self.curBattlepassPoint = data.battlepass_point
	end

	local rewardFreeState = {};
	if data.reward_free_state then
		for _k,no in pairs(data.reward_free_state) do
			rewardFreeState[no] = 1;
		end
	end

	local rewardSpecialState = {};
	if data.reward_special_state then
		for _k,no in pairs(data.reward_special_state) do
			rewardSpecialState[tonumber(no)] = 1;
		end
	end

	local reward = ResTable:getTable('battlepass_reward').contain;
	local rewardData = {};
	local num = 0;
	for _k,data in pairs(reward) do
		if data.battlepass_id == self.battlepassId then
			table.insert(rewardData,data);
			num = num + 1;
		end
	end

	if num > 1 then
		table.sort(rewardData , function(first , second)
			return first.total_point < second.total_point;
		end)
	end

	self.rewardData = {};
	for _k , data in pairs(rewardData) do
		if not self.rewardData[data.no] then
			self.rewardData[data.no] = {};
		end
	
		self.rewardData[data.no] = data;
		self.rewardData[data.no].reward_free_state = rewardFreeState[data.no] or 0;
		self.rewardData[data.no].reward_special_state = rewardSpecialState[data.no] or 0;
		self.rewardData[data.no].level = _k;
	end

	local battlePass = {};
	if data.battlepass_mission then
		for _k , bPass in pairs(data.battlepass_mission) do
			if bPass then
				for no , data in pairs(bPass) do
					battlePass[tonumber(no)] = data;
				end
			end
		end
	end

	self.battlepassMissionData = {};
	local mission = ResTable:getTable('battlepass_mission').contain;
	for _k , data in pairs(mission) do
		if not self.battlepassMissionData[data.time_type] then
			self.battlepassMissionData[data.time_type] = {};
		end
		self.battlepassMissionData[data.time_type][data.no] = data;
		self.battlepassMissionData[data.time_type][data.no].num = 0;
		self.battlepassMissionData[data.time_type][data.no].state = 0; --0不能领取，1可领取，2已领取
		if battlePass[data.no] then
			self.battlepassMissionData[data.time_type][data.no].num = battlePass[data.no][1] or 0;
			self.battlepassMissionData[data.time_type][data.no].state = battlePass[data.no][2] or 0;
		end
		local tData = self.battlepassMissionData[data.time_type][data.no];
		if tData.state ~= 1 then
			if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
				local curNum = ConvertDigitUnitToStr(tostring(tData.num));
				local comp = HpCompare(curNum , ">=" , tostring(tData.times));
				if comp then
					self.battlepassMissionData[tData.time_type][tData.no].state = 2;
				end
			else
				if tData.num >= tData.times then
					self.battlepassMissionData[tData.time_type][tData.no].state = 2;
				end
			end
		end

		if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
			local curNum = ConvertDigitUnitToStr(tostring(tData.num));
			self.battlepassMissionData[tData.time_type][tData.no].percent = HpDivide(curNum,tData.times);
		else
			self.battlepassMissionData[tData.time_type][tData.no].percent = tData.num/tData.times;
		end
	end

	if data.lastTime then
		self.lastTime = data.lastTime;
	end

	self:updateCurBattlepassRewardNo()

	self.nextDayRefreshTime = TimeManager:getNextDayTime();
	self.hasInit = true;
	
	--tools.RedDotTools:pushMainPanelRedDot(12);
	--tools.RedDotTools:pushMainPanelRedDot(14);
	--EventManager:FireEvent(EventType.UIUpdateBtnShow,14);
	--EventManager:FireEvent(EventType.UIUpdateBattlepassRewardLevel);
end

function BattlepassModel:getBattlepassMissionData(timeType , no)
	if self.battlepassMissionData[timeType] then
		return self.battlepassMissionData[timeType][no];
	end
	return nil;
end

function BattlepassModel:updateCurBattlepassRewardNo()
	local maxNo = 0;
	self.curBattlepassRewardNo = 0;
	for _k , data in pairs(self.rewardData) do
		if self.curBattlepassPoint < data.total_point then
			if self.curBattlepassRewardNo == 0 or self.rewardData[self.curBattlepassRewardNo].total_point > data.total_point then
				self.curBattlepassRewardNo = data.no;
			end
		end
		if maxNo < data.no then
			maxNo = data.no;
		end
	end

	if self.curBattlepassRewardNo == 0 then
		self.curBattlepassRewardNo = maxNo;
	end
end

function BattlepassModel:getCurBattlepassRewardNo()
	return self.curBattlepassRewardNo;
end

function BattlepassModel:getSpecialState()
	return self.specialState;
end

function BattlepassModel:updateSpecialState(state)
	self.specialState = state;
end

function BattlepassModel:updateBattlepassPoint(battlepassPoint)
	if battlepassPoint then
		for _k , addNum in pairs(battlepassPoint) do
			self.curBattlepassPoint = self.curBattlepassPoint+addNum
		end
		self:updateCurBattlepassRewardNo();
	end
end

function BattlepassModel:getCurBattlepassPoint()
	return self.curBattlepassPoint;
end

function BattlepassModel:updateBattlepassMission(battlepassMission)
	if battlepassMission then
		for _k , data in pairs(battlepassMission) do
			if self.battlepassMissionData[data[1]] and self.battlepassMissionData[data[1]][data[2]] then
				self.battlepassMissionData[data[1]][data[2]].num = data[3][1];
				self.battlepassMissionData[data[1]][data[2]].state = data[3][2];
				local tData = self.battlepassMissionData[data[1]][data[2]];
				if tData.state ~= 1 then
					if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
						local curNum = ConvertDigitUnitToStr(tostring(tData.num));
						local comp = HpCompare(curNum , ">=" , tostring(tData.times));
						if comp then
							self.battlepassMissionData[tData.time_type][tData.no].state = 2;
						end
					else
						if tData.num >= tData.times then
							self.battlepassMissionData[tData.time_type][tData.no].state = 2;
						end
					end
				end

				if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
					local curNum = ConvertDigitUnitToStr(tostring(tData.num));
					self.battlepassMissionData[tData.time_type][tData.no].percent = HpDivide(curNum,tData.times);
				else
					self.battlepassMissionData[tData.time_type][tData.no].percent = tData.num/tData.times;
				end
			end
		end
	end
end

function BattlepassModel:getBattlepassReward(no)
	if no then
		return self.rewardData[no]
	end
	return self.rewardData;
end

function BattlepassModel:updateBattlepassReward(rewardType , no)
	if self.rewardData[no] then
		if rewardType == 2 then
			self.rewardData[no].reward_special_state = 1;
		else
			self.rewardData[no].reward_free_state = 1;
		end
	end
end

function BattlepassModel:getBattlepassId()
	return self.battlepassId;
end

function BattlepassModel:getBattlepassCountDown(countDownType)
	if self.battlepassId ~= 0 then
		local serverTime = math.floor(TimeManager:getCurTime());
		if countDownType == 1 then --每日
			local nextDayTime = self.nextDayRefreshTime - serverTime;
			return nextDayTime < 0 and 0 or nextDayTime;
		elseif countDownType == 2 then --每周
			local weekTime = TimeManager:GetNextWeekTime();
			weekTime = weekTime - serverTime;
			return weekTime < 0 and 0 or weekTime;
		else
			local closeTime = self.battlepassCloseTime - serverTime;
			return closeTime < 0 and 0 or closeTime;
		end
	end
	return 0;
end

function BattlepassModel:getBattlepassByTimeType(timeType)
	return self.battlepassMissionData[timeType];
end

function BattlepassModel:updateBattlepassState(timeType , no ,state)
	if self.battlepassMissionData[timeType] and self.battlepassMissionData[timeType][no] then
		self.battlepassMissionData[timeType][no].state = state;
	end
end

function BattlepassModel:getBattlepassActiveState()

	if not self.hasInit then
		return;
	end

	if self:getLockStateBattlepass() then
		return false 
	end

	local state = self:getTaskActiveState();

	for _k , s in pairs(state) do
		if s then
			return true;
		end
	end

	return false;
end

function BattlepassModel:getTaskActiveState()
	local stateData = {false,false,false};

	for time_type,battlepassMissionData in pairs(self.battlepassMissionData) do
		if not tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').BATTLEPASS , tType) then
			for no , data in pairs(battlepassMissionData) do
				if data.state == 2 then
					stateData[time_type] = true;
					break;
				end
			end
		end
	end
	
	return stateData;
end

function BattlepassModel:getRewardActiveState()
	local curPointNum = model.ItemModel:getItemNum(BATTLEPASS_POINT);
	for no , data in pairs(self.rewardData) do
		if curPointNum >= data.total_point and ((data.reward_state == 0 and data.reward and #data.reward > 0) 
			or (data.reward_special_state == 0 and self.specialState == 1 and data.special_reward and #data.special_reward > 0)) then
			return true;
		end
	end
	return false;
end

function BattlepassModel:getLockStateBattlepass(  )
	return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_19);		
end

function BattlepassModel:getLockStateBattlepassReward()
	return self:getBattlepassCountDown() <= 0;
end

return BattlepassModel;
