local CardCommentModel = class("CardCommentModel")

function CardCommentModel:ctor()
    self:clearAll();
end

function CardCommentModel:clearAll()
    self.commentData = {};
    self.commentLikeState = {};
    self.commentCardLikeState = {};
end

function CardCommentModel:reqCardCommentData(no , reset)

    reset = reset or 0;

    local ids = {};

    if reset == 0 and self.commentData[no] then
        local idList = self.commentData[no].commentIdList or {};
        local reqIndex = self.commentData[no].reqIndex or 0;
        
        local totalNum = #idList;
        if reqIndex >= totalNum then
            return;
        end

        local endIndex = reqIndex + 4;
        endIndex = endIndex > totalNum and totalNum or endIndex
        
        for i = reqIndex + 1 , endIndex do
            table.insert(ids , idList[i]);
        end
    end

    local msg = {};
    msg.no = no;
    msg.reset = reset;
    msg.ids = ids;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_card_comment_info, msg);
end

function CardCommentModel:reqCardCommentCardLike(no)

    if not self.commentData[no] or self.commentCardLikeState[no] then
        return;
    end

    local msg = {};
    msg.no = no;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_card_comment_card_like, msg);
end

function CardCommentModel:reqCardCommentLike(tag , no , cid)
    local msg = {};
    msg.no = no;
    msg.cid = cid;
    msg.tag = tag;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_card_comment_like, msg);
end

function CardCommentModel:reqCardComment(no,comment)
    local msg = {};
    msg.no = no;
    msg.comment = comment;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_card_comment, msg);
end

function CardCommentModel:setCommentData(data)

    if data.reset == 1 then
        self.commentData[data.no] = nil;
    end

    if not self.commentData[data.no] then
        self.commentData[data.no] = {};
        self.commentData[data.no].comment = {};
        self.commentData[data.no].commentIdList = {};
        self.commentData[data.no].reqIndex = 0;
        self.commentData[data.no].likeNum = 0;
    end

    if not self.commentLikeState[data.no] then
        self.commentLikeState[data.no] = {};
    end

    if data.card_comment_id and #data.card_comment_id > 1 then
        table.sort(data.card_comment_id , function(first , second)
            return first > second;
        end);

        self.commentData[data.no].commentIdList = data.card_comment_id;
    end
    
    if data.card_comment then
        local comment = self.commentData[data.no].comment;
        local newComment = {};
        for _cid , cData in pairs(data.card_comment) do
            _cid = tonumber(_cid);
            if not comment[_cid] then
                comment[_cid] = {};
                comment[_cid].no = data.no;
                comment[_cid].cid = _cid;
                comment[_cid].uid = cData[1];
            end
            comment[_cid].name = cData[2];                                                                                                                                                                                                                                                              
            comment[_cid].comment = cData[3];
            comment[_cid].like = cData[4];
            comment[_cid].notLike = cData[5];
            comment[_cid].likeTime = cData[6];
            
            self.commentData[data.no].comment[_cid] = comment[_cid];

            self.commentData[data.no].reqIndex = self.commentData[data.no].reqIndex + 1;
        end
    end

    if data.like_num then
        self.commentData[data.no].likeNum = data.like_num;
    end
end

function CardCommentModel:updateCommentCardLike(data)
    if self.commentData[data.no] then
        if data.like then
            self.commentData[data.no].likeNum = data.like;
            self.commentCardLikeState[data.no] = 1;
        end
    end
end

function CardCommentModel:updateCommentLike(data)
    if self.commentData[data.no] then
        if self.commentData[data.no].comment[data.cid] then
            if data.like then
                self.commentData[data.no].comment[data.cid].like = data.like;
            end

            if data.not_like then
                self.commentData[data.no].comment[data.cid].notLike = data.not_like;
            end

            self.commentLikeState[data.no][data.cid] = 1;
        end
    end
end

function CardCommentModel:updateComment(data)
    if not self.commentData[data.no] then
        self.commentData[data.no] = {}
        self.commentData[data.no].like = 0;
        self.commentData[data.no].comment = {};
        self.commentData[data.no].commentIdList = {};
        self.commentData[data.no].reqIndex = 0;
    else
        self.commentData[data.no].comment = self.commentData[data.no].comment or {};
    end

    self.commentData[data.no].comment[data.cid] = {};
    self.commentData[data.no].comment[data.cid].no = data.no;
    self.commentData[data.no].comment[data.cid].cid = data.cid;
    self.commentData[data.no].comment[data.cid].uid = data.uid;
    self.commentData[data.no].comment[data.cid].name = data.name;
    self.commentData[data.no].comment[data.cid].comment = data.comment;
    self.commentData[data.no].comment[data.cid].like = data.comment_like;
    self.commentData[data.no].comment[data.cid].notLike = data.comment_not_like;
    self.commentData[data.no].comment[data.cid].likeTime = data.like_time;

    table.insert(self.commentData[data.no].commentIdList , 1 , data.cid);
    
    if #self.commentData[data.no].commentIdList > 50 then
        table.remove(self.commentData[data.no].commentIdList);
    end
    
    self.commentData[data.no].reqIndex = self.commentData[data.no].reqIndex + 1;
end
function CardCommentModel:getCommentDataByNo(no)
    return self.commentData[no];
end

function CardCommentModel:getCommentCardLikeState(no)
    return self.commentCardLikeState[no] or 0;
end

function CardCommentModel:getCommentLikeState(no,cid)
    if self.commentLikeState[no] then
        return self.commentLikeState[no][cid] or 0;
    end
    return 0;
end

function CardCommentModel:getCommentTotalNum(no)
    if self.commentData[no] and self.commentData[no].commentIdList then
        return #self.commentData[no].commentIdList
    end

    return 0;
end
return CardCommentModel
