local CardProManager = class("CardProManager")
local Attribute = Enum:getTable('Attribute');
local restrainType = Enum:getTable("FightRestraintType")
local TeamType = Enum:getTable('TeamType')

function CardProManager:ctor()
	self.proList = {};
	self.baseParameter = {};
end

function CardProManager:GetParameter()

	self.baseParameter = {};
	for i=1, 3 do
		self.baseParameter[i] = {};
		for k, v in pairs(Attribute) do
			self.baseParameter[i][v] = 0;
		end
		self.baseParameter[i]['hp'] = 1;
		self.baseParameter[i]['atk'] = 1;
	end

	--未加入公会不计算公会科技属性
	if not model.GuildModel:isJoinGuild() then
		return;
	end

	local baseskill = model.HomeModel:getBaseSkillLevel();
	if baseskill then
		for skillId, level in pairs(baseskill) do
			if level > 0 then
				local data = ResTable:getMultiValue('base_skill_lv', skillId, level);
				if data then
					local s_type = data['type'];
					if s_type >= 1 and s_type <= 2 then
						self.baseParameter[data.def_type][Attribute[s_type]] = self.baseParameter[data.def_type][Attribute[s_type]] + (tonumber(data.value) / 10000);
					elseif s_type >= 3 and s_type <= 6 then
						self.baseParameter[data.def_type][Attribute[s_type]] = self.baseParameter[data.def_type][Attribute[s_type]] + tonumber(data.value);
					end
				end
			end
		end
	end
end
-- index : nil->all; 1 ; 2 ; 3 ;
function CardProManager:GenCardPro(index)

	self.proList = {};
	
	self:GetParameter();
		
	local cardList = model.CardModel:GetAllCard();
	for pid, card in pairs(cardList) do
		if index then
			local role_id = ResTable:getValue('role', card.no, 'role_id');
			if ResTable:getValue('role_pre', role_id, 'def_type') == index then
				self.proList[pid] = self:genPro(card);
			end
		else
			self.proList[pid] = self:genPro(card);
		end
	end
end

function CardProManager:GetCardPro( pid )
	if not self.proList[pid] then
		local cardData = model.CardModel:GetCard(pid);
		if not cardData then
			return nil;
		end
		self.proList[pid] = self:genPro(cardData);
	end
	return self.proList[pid];
end

function CardProManager:setNeedUpdateGenPro(pid,isDirectUpdate)
	if isDirectUpdate then
		local cardData = model.CardModel:GetCard(pid);
		if not cardData then
			return nil;
		end
		self.proList[pid] = self:genPro(cardData);
	else
		self.proList[pid] = nil
	end
end

function CardProManager:setNeedUpdateGenProByNo(no,isDirectUpdate)
	local cards =  model.CardModel:getCardByNo(no)
	if cards then
		for pid , _ in pairs(cards) do
			self:setNeedUpdateGenPro(pid,isDirectUpdate)
		end
	end
end

function CardProManager:setNeedUpdateAllGenPro(isDirectUpdate)
	if isDirectUpdate then
		self:GenCardPro()
	else
		self.proList = {};
		self:GetParameter();
	end
end

function CardProManager:genPro(data , roleStar , breakLv , lv)
	local roleData = ResTable:getRowData('role', data.no);
	local rolePreData = ResTable:getRowData('role_pre', data.no);

	--card level
	lv = lv or data.level;
	local cardLevelPro = {
		hp = '0',
		atk = '0',
	};
	local lvUpData = ResTable:getMultiValue('role_lv', data.no, lv)
	if lvUpData then
		cardLevelPro['hp'] = tostring(lvUpData.hp)
		cardLevelPro['atk'] = tostring(lvUpData.atk)
	end

	-- card star
	local roleStar = roleStar or data.star_lv;
	local cardStarPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	local cardStarAttributeData = ResTable:getMultiValue('role_star_attribute', data.no, roleStar)
	if cardStarAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardStarPro[v] = cardStarAttributeData[v] and tostring(cardStarAttributeData[v]) or '0'
			else
				cardStarPro[v] = cardStarAttributeData[v] or 0
			end
		end
	end

	--card break
	local breakLv = breakLv or data.break_lv
	local cardBreakAttributeData = ResTable:getMultiValue('role_grade_attribute', breakLv , rolePreData.initial_star)
	local cardBreakPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	if cardBreakAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardBreakPro[v] = cardBreakAttributeData[v] and tostring(cardBreakAttributeData[v]) or '0'
			else
				cardBreakPro[v] = cardBreakAttributeData[v] or 0
			end
		end
	end

	--card skill
	local roleSkillPro = {};
	for k, v in pairs(Attribute) do
		if not roleSkillPro[v] then
			roleSkillPro[v] = {};
		end
		roleSkillPro[v].addValue = 0;
		if k >= 1 and k <= 2 then
			roleSkillPro[v].multValue = 0;
		else
			roleSkillPro[v].multValue = 1;
		end
	end

	local skillItemId = 0;
	if data.pid and data.pid > 0 then
		skillItemId = model.CardModel:getCardSkillIdByPid(pid);
	end
	if skillItemId == 0 then
		skillItemId = roleData.skill_id;
	end
	local roleSkill = ResTable:getRowData('role_skill' , skillItemId);
	local cardBreakAttributeTabelData = {};
	for _k , breakAttributeData in pairs(ResTable:getTable('role_grade').contain) do
		if breakLv >= breakAttributeData.grade and breakAttributeData.skill_open and breakAttributeData.skill_open~= '' then
			local skillIndex = string.sub(breakAttributeData.skill_open , -1 , -1);
			if roleSkill['skill'..skillIndex..'_no'] and roleSkill['skill'..skillIndex..'_no'] ~= '' then
				local skillResid = roleSkill['skill'..skillIndex..'_no'];
				local skillData = ResTable:getRowData('skill', skillResid);
				if skillData and skillData.skill_type == 3 then
					for _k , skillDamage in pairs(skillData.skill_damage_value) do
						local s_type = Attribute[skillDamage[1]];
						roleSkillPro[s_type].addValue = roleSkillPro[s_type].addValue + skillDamage[2];
						roleSkillPro[s_type].multValue = roleSkillPro[s_type].multValue + skillDamage[3];
					end
				end
			end
		end
	end

	local zfHp, zfAtk = model.BareCollectModel:getBlessHpAndAtk();

	-- pro calculate
	local pro = {};
	pro.hp			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['hp']) ,
						tostring(cardLevelPro['hp'])) , 
						ConvertDigitUnitToStr(cardStarPro['hp'])) , 
						ConvertDigitUnitToStr(cardBreakPro['hp'])) , 
						tostring(roleSkillPro['hp']['addValue'])) ,
						tostring(zfHp)));
	pro.atk			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['atk']) , 
						tostring(cardLevelPro['atk'])) , 
						ConvertDigitUnitToStr(cardStarPro['atk'])) , 
						ConvertDigitUnitToStr(cardBreakPro['atk'])) , 
						tostring(roleSkillPro['atk']['addValue'])) ,
						tostring(zfAtk)));
	pro.def			= roleData['def'] + cardStarPro['def'] + cardBreakPro['def'] + roleSkillPro['def']['addValue'];
	pro.aspd		= roleData['aspd'] + cardStarPro['aspd'] + cardBreakPro['aspd'] + roleSkillPro['aspd']['addValue'];
	pro.crit		= roleData['crit'] + cardStarPro['crit'] + cardBreakPro['crit'] + roleSkillPro['crit']['addValue'];
	pro.crit_dam	= roleData['crit_dam'] + cardStarPro['crit_dam'] + cardBreakPro['crit_dam'] + roleSkillPro['crit_dam']['addValue'];
	pro.fp			= '0';
	pro.lv			= lv;
	
	pro.hp			= ConvertStrToDigitUnit(HpMultiply(pro.hp, (self.baseParameter[rolePreData.def_type]['hp'] + roleSkillPro['hp']['multValue'])));
	pro.atk			= ConvertStrToDigitUnit(HpMultiply(pro.atk, (self.baseParameter[rolePreData.def_type]['atk'] + roleSkillPro['atk']['multValue'])));
	pro.def			= (pro.def + self.baseParameter[rolePreData.def_type]['def']) * (roleSkillPro['def']['multValue']);
	pro.aspd		= (pro.aspd + self.baseParameter[rolePreData.def_type]['aspd']) * (roleSkillPro['aspd']['multValue']);
	pro.crit		= (pro.crit + self.baseParameter[rolePreData.def_type]['crit']) * (roleSkillPro['crit']['multValue']);
	pro.crit_dam	= (pro.crit_dam + self.baseParameter[rolePreData.def_type]['crit_dam']) * (roleSkillPro['crit_dam']['multValue']);

	pro.fp			= ConvertStrToDigitUnit(HpPlus(HpMultiply(ConvertDigitUnitToStr(pro.hp), 0.1), HpMultiply(ConvertDigitUnitToStr(pro.atk), 1.5)));

	return pro;
end

function CardProManager:GenProInTeam(pid , roleStar , breakLv, teamType, tmpInfo)
	local data = model.CardModel:GetCard(pid);
	local roleData = ResTable:getRowData('role', data.no);
	local rolePreData = ResTable:getRowData('role_pre', data.no);
	local isInTeam, pos = model.TeamModel:isRoleInTeam(pid,model.TeamModel:getTeamIdByTeamType(teamType));
	if tmpInfo then
		pos = tmpInfo.pos
	end

	--card level
	local lv = data.level;
	local cardLevelPro = {
		hp = '0',
		atk = '0',
	};
	local lvUpData = ResTable:getMultiValue('role_lv', data.no, lv)
	if lvUpData then
		cardLevelPro['hp'] = tostring(lvUpData.hp)
		cardLevelPro['atk'] = tostring(lvUpData.atk)
	end

	-- card star
	local roleStar = roleStar or data.star_lv;
	local cardStarPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	local cardStarAttributeData = ResTable:getMultiValue('role_star_attribute', data.no, roleStar)
	if cardStarAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardStarPro[v] = cardStarAttributeData[v] and tostring(cardStarAttributeData[v]) or '0'
			else
				cardStarPro[v] = cardStarAttributeData[v] or 0
			end
		end
	end

	--card break
	local breakLv = breakLv or data.break_lv
	local cardBreakAttributeData = ResTable:getMultiValue('role_grade_attribute', breakLv , rolePreData.initial_star)
	local cardBreakPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	if cardBreakAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardBreakPro[v] = cardBreakAttributeData[v] and tostring(cardBreakAttributeData[v]) or '0'
			else
				cardBreakPro[v] = cardBreakAttributeData[v] or 0
			end
		end
	end

	--card skill
	local roleSkillPro = {};
	for k, v in pairs(Attribute) do
		if not roleSkillPro[v] then
			roleSkillPro[v] = {};
		end
		roleSkillPro[v].addValue = 0;
		if k >= 1 and k <= 2 then
			roleSkillPro[v].multValue = 0;
		else
			roleSkillPro[v].multValue = 1;
		end
	end

	local skillItemId = model.CardModel:getCardSkillIdByPid(pid);
	if skillItemId == 0 then
		skillItemId = roleData.skill_id;
	end
	local roleSkill = ResTable:getRowData('role_skill' , skillItemId);
	local cardBreakAttributeTabelData = {};
	for _k , breakAttributeData in pairs(ResTable:getTable('role_grade').contain) do
		if data.break_lv >= breakAttributeData.grade and breakAttributeData.skill_open and breakAttributeData.skill_open~= '' then
			local skillIndex = string.sub(breakAttributeData.skill_open , -1 , -1);
			if roleSkill['skill'..skillIndex..'_no'] and roleSkill['skill'..skillIndex..'_no'] ~= '' then
				local skillResid = roleSkill['skill'..skillIndex..'_no'];
				local skillData = ResTable:getRowData('skill', skillResid);
				if skillData and skillData.skill_type == 3 then
					for _k , skillDamage in pairs(skillData.skill_damage_value) do
						local s_type = Attribute[skillDamage[1]];
						roleSkillPro[s_type].addValue = roleSkillPro[s_type].addValue + skillDamage[2];
						roleSkillPro[s_type].multValue = roleSkillPro[s_type].multValue + skillDamage[3];
					end
				end
			end
		end
	end
	--love
	local lovePro = {};
	for k, v in pairs(Attribute) do
		if not lovePro[v] then
			lovePro[v] = {};
		end
		if k == 1 or k == 2 then
			lovePro[v].addValue = '0';
		else
			lovePro[v].addValue = 0;
		end
		lovePro[v].multValue = 0;
	end

	if data.love_id and data.love_id > 0 then
		local loveData = model.CardModel:getLoveData(data.love_id);
		if loveData then
			if loveData.loveLevel >= 5 then
				local data = loveData.loveData[loveData.loveLevel];
				for _k , effect in pairs(data.effect) do
					local attType = effect[1];
					local attValue = effect[2];
					local multValue = effect[3];
					if attType == 1 or attType == 2 then
						lovePro[Attribute[attType]].addValue = attValue and tostring(attValue) or '0';
					else
						lovePro[Attribute[attType]].addValue = attValue or 0;
					end
					lovePro[Attribute[attType]].multValue = multValue or 0;
				end
			end
		end
	end

	-- team restraint pro 
	local roleRestraintPro = {}
	local teamBareColorAll = nil
	if tmpInfo and tmpInfo.curRestraint then --只在编队面板时 使用，为当前编队的 组队属性加成
		roleRestraintPro.hp = tmpInfo.curRestraint[1]
		roleRestraintPro.atk = tmpInfo.curRestraint[2]

		teamBareColorAll = model.TeamModel:getTmpTeamColorAtt(rolePreData.def_type)
	else
		local tid = 1
		if teamType then
			tid = model.TeamModel:getTeamIdByTeamType( teamType );
		end
		roleRestraintPro.hp = model.TeamModel:getRestraintAmplificationFactor(1,tid)
		roleRestraintPro.atk = model.TeamModel:getRestraintAmplificationFactor(2,tid)

		teamBareColorAll = model.TeamModel:getTeamColorAtt(tid, rolePreData.def_type)
	end
	if teamBareColorAll == nil then
		teamBareColorAll = {};
		for k, v in pairs(Attribute) do
			if not teamBareColorAll[v] then
				teamBareColorAll[v] = {};
			end
			if k == 1 or k == 2 then
				teamBareColorAll[v].addValue = 0;
				teamBareColorAll[v].multValue = 0;
			end
		end
	end

	-- equip pro
	local equipParameter = {};
	for k, v in pairs(Attribute) do
		equipParameter[v] = 0;
	end
	equipParameter['hp'] = 1;
	equipParameter['atk'] = 1;

	local equipEffectNum = 0;
	local equipSlot = model.EquipModel:getEquipSlotByIndex(pos);
	for i=1, 4 do
		if equipSlot and equipSlot[i] ~= 0 then
			local equipData = ResTable:getRowData('equip', equipSlot[i]);
			if equipData then
				for i=1, 3 do
					local att = GetMathsValue(string.format(tostring(equipData['value'..i]), 1));
					if math.ceil(att) > 0 then
						local e_type = equipData['attribute'..i];
						equipParameter[Attribute[e_type]] = equipParameter[Attribute[e_type]] + att;
					end
				end
			end
		end
	end

	--bare collect
	local bareCollectPro = {};
	for k, v in pairs(Attribute) do
		if not bareCollectPro[v] then
			bareCollectPro[v] = {};
		end
		if k == 1 or k == 2 then
			bareCollectPro[v].addValue = '0';
		else
			bareCollectPro[v].addValue = 0;
		end
		bareCollectPro[v].multValue = 0;
	end

	
	if data.bare_id and data.bare_id > 0 then
		local bareData = model.BareCollectModel:getBareData(data.bare_id);
		if bareData then
			local attType = bareData.attribute_effect[1];
			local attValue = bareData.attribute_effect[2];
			local multValue = bareData.attribute_effect[3];
			if attType == 1 or attType == 2 then
				bareCollectPro[Attribute[attType]].addValue = attValue and tostring(attValue) or '0';
			else
				bareCollectPro[Attribute[attType]].addValue = attValue or 0;
			end
			bareCollectPro[Attribute[attType]].multValue = multValue or 0;
		end
	end
	if data.bare_id_2 and data.bare_id_2 > 0 then
		local bareData = model.BareCollectModel:getBareData(data.bare_id_2);
		if bareData then
			local attType = bareData.attribute_effect[1];
			local attValue = bareData.attribute_effect[2] or 0;
			local multValue = bareData.attribute_effect[3] or 0;
			if attType == 1 or attType == 2 then
				bareCollectPro[Attribute[attType]].addValue = tostring(attValue + tonumber(bareCollectPro[Attribute[attType]].addValue));
			else
				bareCollectPro[Attribute[attType]].addValue = attValue + bareCollectPro[Attribute[attType]].addValue;
			end
			bareCollectPro[Attribute[attType]].multValue = multValue + bareCollectPro[Attribute[attType]].multValue;
		end
	end

	local maze = teamType == TeamType.EVENT or false
	local mazePro = {};
	mazePro['hp'] = 0;
	mazePro['atk'] = 0;
	if maze then
		local color = rolePreData.def_type;
		local job = rolePreData.job_type;
		mazePro['hp'] = model.EventModel:getBuffAttByColorAndJob(color, job, 'hp')
		mazePro['atk'] = model.EventModel:getBuffAttByColorAndJob(color, job, 'atk')
	end

	local story = teamType == TeamType.STORY or false
	local storyPro = {};
	storyPro['hp'] = 0;
	storyPro['atk'] = 0;
	if story then
		-- local color = rolePreData.def_type;
		-- local job = rolePreData.job_type;
		-- mazePro['hp'] = model.EventModel:getBuffAttByColorAndJob(color, job, 'hp')
		-- mazePro['atk'] = model.EventModel:getBuffAttByColorAndJob(color, job, 'atk')
	end

	--Gem
	local gemPro = {};
	for k, v in pairs(Attribute) do
		if not gemPro[v] then
			gemPro[v] = {};
		end
		if k == 1 or k == 2 then
			gemPro[v].addValue = '0';
		else
			gemPro[v].addValue = 0;
		end
		gemPro[v].multValue = 0;
	end
	if false then
	end

	local zfHp, zfAtk = model.BareCollectModel:getBlessHpAndAtk();

	-- pro calculate
	local pro = {};
	pro.hp			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['hp']) ,
						tostring(cardLevelPro['hp'])) ,
						ConvertDigitUnitToStr(cardStarPro['hp'])) , 
						ConvertDigitUnitToStr(cardBreakPro['hp'])) , 
						tostring(roleSkillPro['hp']['addValue'])), 
						ConvertDigitUnitToStr(bareCollectPro['hp']['addValue'])),
						tostring(lovePro['hp']['addValue'])),
						tostring(gemPro['hp']['addValue'])),
						tostring(zfHp)),
						tostring(teamBareColorAll['hp']['addValue'])));
	pro.atk			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['atk']) , 
						tostring(cardLevelPro['atk'])) , ConvertDigitUnitToStr(cardStarPro['atk'])) , 
						ConvertDigitUnitToStr(cardBreakPro['atk'])) , 
						tostring(roleSkillPro['atk']['addValue'])), 
						ConvertDigitUnitToStr(bareCollectPro['atk']['addValue'])),
						tostring(lovePro['atk']['addValue'])),
						tostring(gemPro['atk']['addValue'])),
						tostring(zfAtk)),
						tostring(teamBareColorAll['atk']['addValue'])));
	pro.def			= roleData['def'] + cardStarPro['def'] + cardBreakPro['def'] + roleSkillPro['def']['addValue']+bareCollectPro['def']['addValue']+lovePro['def']['addValue'];
	pro.aspd		= roleData['aspd'] + cardStarPro['aspd'] + cardBreakPro['aspd'] + roleSkillPro['aspd']['addValue']+bareCollectPro['aspd']['addValue']+lovePro['aspd']['addValue'];
	pro.crit		= roleData['crit'] + cardStarPro['crit'] + cardBreakPro['crit'] + roleSkillPro['crit']['addValue']+bareCollectPro['crit']['addValue']+lovePro['crit']['addValue'];
	pro.crit_dam	= roleData['crit_dam'] + cardStarPro['crit_dam'] + cardBreakPro['crit_dam'] + roleSkillPro['crit_dam']['addValue']+bareCollectPro['crit_dam']['addValue']+lovePro['crit_dam']['addValue'];
	pro.fp			= '0';
	pro.lv			= lv;
	
	pro.hp			= ConvertStrToDigitUnit(HpMultiply(pro.hp, (self.baseParameter[rolePreData.def_type]['hp'] + roleSkillPro['hp']['multValue'] + roleRestraintPro.hp + bareCollectPro['hp']['multValue'] +lovePro['hp']['multValue'] +mazePro['hp'] +gemPro['hp']['multValue']+teamBareColorAll['hp']['multValue'])));
	pro.atk			= ConvertStrToDigitUnit(HpMultiply(pro.atk, (self.baseParameter[rolePreData.def_type]['atk'] + roleSkillPro['atk']['multValue'] + roleRestraintPro.atk + bareCollectPro['atk']['multValue']+lovePro['atk']['multValue'] +mazePro['atk']+gemPro['atk']['multValue']+teamBareColorAll['atk']['multValue'])));
	pro.def			= (pro.def + self.baseParameter[rolePreData.def_type]['def']) * (roleSkillPro['def']['multValue']+ bareCollectPro['def']['multValue']+lovePro['def']['multValue']);
	pro.aspd		= (pro.aspd + self.baseParameter[rolePreData.def_type]['aspd']) * (roleSkillPro['aspd']['multValue']+ bareCollectPro['aspd']['multValue']+lovePro['aspd']['multValue']);
	pro.crit		= (pro.crit + self.baseParameter[rolePreData.def_type]['crit']) * (roleSkillPro['crit']['multValue']+ bareCollectPro['crit']['multValue']+lovePro['crit']['multValue']);
	pro.crit_dam	= (pro.crit_dam + self.baseParameter[rolePreData.def_type]['crit_dam']) * (roleSkillPro['crit_dam']['multValue']+ bareCollectPro['crit_dam']['multValue']+lovePro['crit_dam']['multValue']);

	equipParameter.hp = math.floor(equipParameter.hp);
	equipParameter.atk = math.floor(equipParameter.atk);
	pro.hp		= ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.hp), tostring(equipParameter.hp)));
	pro.atk		= ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.atk), tostring(equipParameter.atk)));
	pro.def		= pro.def + equipParameter.def;
	pro.aspd	= pro.aspd + equipParameter.aspd;
	pro.crit	= pro.crit + equipParameter.crit;
	pro.crit_dam= pro.crit_dam + equipParameter.crit_dam;

	pro.fp		= ConvertStrToDigitUnit(HpPlus(HpMultiply(ConvertDigitUnitToStr(pro.hp), 0.1), HpMultiply(ConvertDigitUnitToStr(pro.atk), 1.5)));

	return pro;
end

function CardProManager:GenMonsterPro( id1, id2 ,restraintFactor)
	-- init
	local pro = {};
	pro.hp = 0;		
	pro.atk = 0;	
	pro.def = 0;	
	pro.crit = 0;	
	pro.crit_dam = 0;
	pro.aspd = 0;
	pro.fp = 0;
	pro.lv = 0;

	local data = ResTable:getRowData('monster_attribute', id2);
	local monsterData = ResTable:getRowData('monster', id1);
	pro.def_type = monsterData['def_type'];
	pro.hp = tostring(data['hp']);
	pro.atk = tostring(data['atk']);
	pro.def = data['def'];
	pro.crit = data['crit'];
	pro.crit_dam = data['crit_dam'];
	pro.aspd = data['aspd'];
	pro.lv = data['lv'] or 0;

	if not restraintFactor then
		restraintFactor = {};
		restraintFactor[1] = 0;
		restraintFactor[2] = 0;
	end

	local roleRestraintPro = {}
	roleRestraintPro.hp = restraintFactor[1] or 0;
	roleRestraintPro.atk = restraintFactor[2] or 0;

	-- id加成属性
	local idData = ResTable:getValue('monster', id1, 'attribute_rate');
	pro.hp = ConvertStrToDigitUnit(HpMultiply(ConvertDigitUnitToStr(pro.hp), (idData[1] + roleRestraintPro.hp)));
	pro.atk	= ConvertStrToDigitUnit(HpMultiply(ConvertDigitUnitToStr(pro.atk), (idData[2] + roleRestraintPro.atk)));
	pro.def	= pro.def * idData[3];
	pro.crit = pro.crit * idData[4];
	pro.crit_dam = pro.crit_dam * idData[5];
	pro.aspd = pro.aspd * idData[6];

	pro.fp = ConvertStrToDigitUnit(HpPlus(HpMultiply(ConvertDigitUnitToStr(pro.hp), 0.1), HpMultiply(ConvertDigitUnitToStr(pro.atk), 1.5)));
	
	return pro;
end

function CardProManager:GenGuildBossPro( boss_id, boss_no )
	-- init
	local pro = {};
	pro.hp = 0;		
	pro.atk = 0;	
	pro.def = 0;	
	pro.crit = 0;	
	pro.crit_dam = 0;
	pro.aspd = 0;
	pro.lv = boss_id;

	pro.def_type = ResTable:getValue('monster', boss_no, 'def_type');

	local order_no = model.GuildModel:getOrderNo()
	local guildBossInfo = model.GuildModel:getGuildBossInfo()
	local raidMonsterData = ResTable:getRowData('guild_raid_order',order_no);

	pro.hp = tostring( guildBossInfo.raid.armor_hp );
	pro.atk = tostring(  raidMonsterData.atk );
	pro.def = 0;
	pro.crit = 0;
	pro.crit_dam = 0;
	pro.aspd = 1;

	return pro;
end

function CardProManager:GenArenaRoleMonsterPro(rankData , pos)

	local data = rankData.arena_team[pos];

	local roleData = ResTable:getRowData('role', data.no);
	local rolePreData = ResTable:getRowData('role_pre', data.no);

	--card level
	local lv = data.level;
	local cardLevelPro = {
		hp = '0',
		atk = '0',
	};
	local lvUpData = ResTable:getMultiValue('role_lv', data.no, lv)
	if lvUpData then
		cardLevelPro['hp'] = tostring(lvUpData.hp)
		cardLevelPro['atk'] = tostring(lvUpData.atk)
	end

	-- card star
	local roleStar = data.star_lv;
	local cardStarPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	local cardStarAttributeData = ResTable:getMultiValue('role_star_attribute', data.no, roleStar)
	if cardStarAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardStarPro[v] = cardStarAttributeData[v] and tostring(cardStarAttributeData[v]) or '0'
			else
				cardStarPro[v] = cardStarAttributeData[v] or 0
			end
		end
	end

	--card break
	local breakLv = data.break_lv
	local cardBreakAttributeData = ResTable:getMultiValue('role_grade_attribute', breakLv , rolePreData.initial_star)
	local cardBreakPro = {
		hp = '0',
		atk = '0',
		def = 0,
		aspd = 0,
		crit = 0,
		crit_dam = 0,
	};
	if cardBreakAttributeData then
		for _k, v in pairs(Attribute) do
			if _k == 1 or _k == 2 then
				cardBreakPro[v] = cardBreakAttributeData[v] and tostring(cardBreakAttributeData[v]) or '0'
			else
				cardBreakPro[v] = cardBreakAttributeData[v] or 0
			end
		end
	end

	--card skill
	local roleSkillPro = {};
	for k, v in pairs(Attribute) do
		if not roleSkillPro[v] then
			roleSkillPro[v] = {};
		end
		roleSkillPro[v].addValue = 0;
		if k >= 1 and k <= 2 then
			roleSkillPro[v].multValue = 0;
		else
			roleSkillPro[v].multValue = 1;
		end
	end

	local roleSkill = ResTable:getRowData('role_skill' , roleData.skill_id);
	local cardBreakAttributeTabelData = {};
	for _k , breakAttributeData in pairs(ResTable:getTable('role_grade').contain) do
		if data.break_lv >= breakAttributeData.grade and breakAttributeData.skill_open and breakAttributeData.skill_open~= '' then
			local skillIndex = string.sub(breakAttributeData.skill_open , -1 , -1);
			if roleSkill['skill'..skillIndex..'_no'] and roleSkill['skill'..skillIndex..'_no'] ~= '' then
				local skillResid = roleSkill['skill'..skillIndex..'_no'];
				local skillData = ResTable:getRowData('skill', skillResid);
				if skillData and skillData.skill_type == 3 then
					for _k , skillDamage in pairs(skillData.skill_damage_value) do
						local s_type = Attribute[skillDamage[1]];
						roleSkillPro[s_type].addValue = roleSkillPro[s_type].addValue + skillDamage[2];
						roleSkillPro[s_type].multValue = roleSkillPro[s_type].multValue + skillDamage[3];
					end
				end
			end
		end
	end

	-- team restraint pro 
	local roleRestraintPro = {}
	roleRestraintPro.hp = rankData.restraintFactor[1];
	roleRestraintPro.atk = rankData.restraintFactor[2];

	-- equip pro
	local equipParameter = {};
	for k, v in pairs(Attribute) do
		equipParameter[v] = 0;
	end
	equipParameter['hp'] = 1;
	equipParameter['atk'] = 1;

	for _k, equip in pairs(rankData.equip) do
		local equipData = ResTable:getRowData('equip', equip.no);
		if equipData then
			-- for i=1, 3 do
			-- 	local att = GetMathsValue(string.format(tostring(equipData['value'..i]), equip.strenlevel));
			-- 	if math.ceil(att) > 0 then
			-- 		local e_type = equipData['attribute'..i];
			-- 		equipParameter[Attribute[e_type]] = equipParameter[Attribute[e_type]] + att;
			-- 	end
			-- end
		end
	end

	--baseSkill
	local baseParameter = {};
	for i=1, 3 do
		baseParameter[i] = {};
		for k, v in pairs(Attribute) do
			baseParameter[i][v] = 0;
		end
		baseParameter[i]['hp'] = 1;
		baseParameter[i]['atk'] = 1;
	end

	if rankData.baseskill then 
		for skillId, level in pairs(rankData.baseskill) do
			if level > 0 then
				local data = ResTable:getMultiValue('base_skill_lv', skillId, level);
				if data then
					local s_type = data['type'];
					if s_type >= 1 and s_type <= 2 then
						baseParameter[data.def_type][Attribute[s_type]] = baseParameter[data.def_type][Attribute[s_type]] + (tonumber(data.value) / 100);
					elseif s_type >= 3 and s_type <= 6 then
						baseParameter[data.def_type][Attribute[s_type]] = baseParameter[data.def_type][Attribute[s_type]] + tonumber(data.value);
					end
				end
			end
		end
	end

	-- pro calculate
	local pro = {};
	pro.hp			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['hp']) , tostring(cardLevelPro['hp'])) , ConvertDigitUnitToStr(cardStarPro['hp'])) , ConvertDigitUnitToStr(cardBreakPro['hp'])) , tostring(roleSkillPro['hp']['addValue'])));
	pro.atk			= tostring(HpPlus(HpPlus(HpPlus(HpPlus(ConvertDigitUnitToStr(roleData['atk']) , tostring(cardLevelPro['atk'])) , ConvertDigitUnitToStr(cardStarPro['atk'])) , ConvertDigitUnitToStr(cardBreakPro['atk'])) , tostring(roleSkillPro['atk']['addValue'])));
	pro.def			= roleData['def'] + cardStarPro['def'] + cardBreakPro['def'] + roleSkillPro['def']['addValue'];
	pro.aspd		= roleData['aspd'] + cardStarPro['aspd'] + cardBreakPro['aspd'] + roleSkillPro['aspd']['addValue'];
	pro.crit		= roleData['crit'] + cardStarPro['crit'] + cardBreakPro['crit'] + roleSkillPro['crit']['addValue'];
	pro.crit_dam	= roleData['crit_dam'] + cardStarPro['crit_dam'] + cardBreakPro['crit_dam'] + roleSkillPro['crit_dam']['addValue'];
	pro.fp			= '0';
	pro.lv			= lv;
	
	pro.hp			= ConvertStrToDigitUnit(HpMultiply(pro.hp, (baseParameter[rolePreData.def_type]['hp'] + roleSkillPro['hp']['multValue'] + roleRestraintPro.hp)));
	pro.atk			= ConvertStrToDigitUnit(HpMultiply(pro.atk, (baseParameter[rolePreData.def_type]['atk'] + roleSkillPro['atk']['multValue'] + roleRestraintPro.atk)));
	pro.def			= (pro.def + baseParameter[rolePreData.def_type]['def']) * roleSkillPro['def']['multValue'];
	pro.aspd		= (pro.aspd + baseParameter[rolePreData.def_type]['aspd']) * roleSkillPro['aspd']['multValue'];
	pro.crit		= (pro.crit + baseParameter[rolePreData.def_type]['crit']) * roleSkillPro['crit']['multValue'];
	pro.crit_dam	= (pro.crit_dam + baseParameter[rolePreData.def_type]['crit_dam']) * roleSkillPro['crit_dam']['multValue'];

	equipParameter.hp = math.floor(equipParameter.hp);
	equipParameter.atk = math.floor(equipParameter.atk);
	pro.hp		= ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.hp), tostring(equipParameter.hp)));
	pro.atk		= ConvertStrToDigitUnit(HpPlus(ConvertDigitUnitToStr(pro.atk), tostring(equipParameter.atk)));
	pro.def		= pro.def + equipParameter.def;
	pro.aspd	= pro.aspd + equipParameter.aspd;
	pro.crit	= pro.crit + equipParameter.crit;
	pro.crit_dam= pro.crit_dam + equipParameter.crit_dam;

	pro.fp		= ConvertStrToDigitUnit(HpPlus(HpMultiply(ConvertDigitUnitToStr(pro.hp), 0.1), HpMultiply(ConvertDigitUnitToStr(pro.atk), 1.5)));

	return pro;
end

function CardProManager:GetMonsterRestraintFactor(monsterList)
	local restraintFactor = {}
	restraintFactor[1] = 0
	restraintFactor[2] = 0

	local restraintString = '000';
	
	if monsterList then
		local restraint = {}
		restraint[restrainType.RED_BASE] = 0;
		restraint[restrainType.GREEN_BASE] = 0;
		restraint[restrainType.BLUE_BASE] = 0;
	
		for i=1, 9 do
			local data = monsterList[i];
			if data and data[1] then
				local def_type = ResTable:getValue('monster', data[1] , 'def_type');
				restraint[def_type] = restraint[def_type] + 1;
			end
		end

		restraintString = tostring(restraint[restrainType.BLUE_BASE])..tostring(restraint[restrainType.RED_BASE])..tostring(restraint[restrainType.GREEN_BASE]);
	
		for k, v in pairs(ResTable:getTable('formation_attribute').contain) do
			if restraint[v.color] == v.value then
				for k1, v1 in pairs(v.attribute) do
					if not restraintFactor[v1[1]] then
						restraintFactor[v1[1]] = v1[2]
					else
						restraintFactor[v1[1]] = restraintFactor[v1[1]] + v1[2]
					end
				end
			end
		end
	end
	return restraintFactor , restraintString;
end

cc.exports.CardProManager = CardProManager:create();
