local ChatModel = class("ChatModel")
local scheduler = cc.Director:getInstance():getScheduler();

--[[ * msg_type ( channel )
--		1: change world channel
--		2: world channel msg
--		3: system channel msg
--		4: guild msg
--]]	

function ChatModel:ctor()
	self:clearAll()
end

function ChatModel:clearAll(  )
	self.running = false;
	EventManager:UnregisterEvent(EventType.ChatReceiveMsg)

end

function ChatModel:Init()
	self.room = 1;
	self.msg = {};
	self.msgNum = {};
	for i=1, 4 do
		self.msg[i] = {};
		self.msgNum[i] = 0;
	end
	self.closeTime = 0

	EventManager:RegisterEvent(EventType.ChatReceiveMsg, handler(self, self.receiveMsg));
end

function ChatModel:setPanelShow(flag)
	self.panelShowFlag = flag;
	if not flag then
		self.closeTime = TimeManager:getCurTime()
	end
end

function ChatModel:open()
	Webproxy:Init(define.chat_url)
	self:setRunning(true)
	if not self.close_scheduleHandler then
		self.close_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.checkClose), 10, false);
	end
end

function ChatModel:checkClose()
	local time = TimeManager:getCurTime()
	if self.running and not self.panelShowFlag and self.closeTime + Conf.chat_close_time < time then
		Webproxy:close()
		self.room = 1;
		self.msg = {};
		self.msgNum = {};
		for i=1, 4 do
			self.msg[i] = {};
			self.msgNum[i] = 0;
		end
		self.running = false
	end
end

function ChatModel:sendBaseInfo()
	local msg = {};
	msg.msg_type = 0;
	msg.gid = model.GuildModel:getGuildInfo().gid;
	msg.uid = model.UserModel:getUserUid();

	Webproxy:sendMessage(msg);
end

function ChatModel:getMsgs(channel)
	return self.msg[channel+1];
end

function ChatModel:getMsgNum(channel)
	return self.msgNum[channel+1];
end

function ChatModel:receiveMsg(msg)
	if self.msgNum[msg.msg_type] then
		self.msgNum[msg.msg_type] = self.msgNum[msg.msg_type] + 1;
	end
	if msg.msg_type == 1 then
		self.room = msg.data;
		if self.running then
			EventManager:FireEvent(EventType.ChatChangeRoom);
		end
	elseif msg.msg_type == 2 then
		table.insert(self.msg[msg.msg_type], msg);
		if self.running then
			EventManager:FireEvent(EventType.ChatGenMsg, 1, msg);
		end
	elseif msg.msg_type == 3 then
		-- TODO?????

	elseif msg.msg_type == 4 then
		table.insert(self.msg[msg.msg_type], msg);
		if self.running then
			EventManager:FireEvent(EventType.ChatGenMsg, 2, msg);
		end
	end
end

function ChatModel:sendMessage(info, channel)
	local msg = {};
	msg.data = info;
	msg.uid = model.UserModel:getUserUid();
	msg.name = model.UserModel:getUserName();
	msg.level = model.UserModel:getUserLevel();
	local headNo = model.UserModel:getHeadIcon()
	msg.head = headNo;

	if channel == 1 then
		msg.msg_type = 2;
		msg.room_id = self.room;
	elseif channel == 2 then
		local gid = model.GuildModel:getGuildInfo().gid;
		if gid and gid ~= 0 then
			msg.msg_type = 4;
			msg.gid = gid;
		else
			return;
		end
	else
		return;
	end

	Webproxy:sendMessage(msg);
end

function ChatModel:changeRoom(newRoom)
	local msg = {}
	msg.msg_type = 1;
	msg.old_room = self.room;
	msg.new_room = newRoom;

	Webproxy:sendMessage(msg);
end

function ChatModel:getRoom()
	return self.room;
end

function ChatModel:isRunning()
	return self.running;
end

function ChatModel:setRunning(flag)
	self.running = flag;
end

return ChatModel;
