local EntrustModel = class("EntrustModel")
function EntrustModel:ctor()
	self.entrustData = {};
	self.initFlag = false;
end

function EntrustModel:isInit()
	return self.initFlag;
end

function EntrustModel:Init(msg)
	self.initFlag = true;
	self.entrustData.daily = {};
	for k, v in pairs(msg.common_delegation_info) do
		local data = {};
		data.id = v.task_id;
		data.state = v.state;
		data.restTime = v.last_time;
		data.roleList = {};
		data.rate = v.percent;
		self.entrustData.daily[tonumber(k)] = data;
	end
	self.entrustData.urgent = {};
	for k, v in pairs(msg.emergency_delegation_info) do
		local data = {};
		data.id = v.task_id;
		data.state = v.state;
		data.restTime = v.last_time;
		data.roleList = {};
		data.rate = v.percent;
		self.entrustData.urgent[tonumber(k)] = data;
	end
	self.urgentTimes = msg.emergency_times or 0;
	self.dailyTimes = msg.common_times or 0;
end
function EntrustModel:getDataByIndex(index)
	if not index then
		return;
	end
	if index >= 0 and index <= 4 then
		return self.entrustData.daily[tonumber(index)];
	elseif index >= 5 and index <= 8 then
		return self.entrustData.urgent[tonumber(index)];
	end
end
function EntrustModel:updateUrgentTimes(urgentTimes)
	self.urgentTimes = urgentTimes;
end
function EntrustModel:updateDailyTimes(dailyTimes)
	self.dailyTimes = dailyTimes;
end
function EntrustModel:getUrgentTimes()
	return self.urgentTimes;
end
function EntrustModel:getDailyTimes()
	return self.dailyTimes;
end
function EntrustModel:getDailyLogistics()
	return self.entrustData.daily;
end

function EntrustModel:getUrgentLogistics()
	return self.entrustData.urgent;
end

function EntrustModel:updateLogistics(info, index)
	local aimItem = nil
	if self.entrustData.urgent[index] then
		aimItem = self.entrustData.urgent[index]
	end
	if self.entrustData.daily[index] then
		aimItem = self.entrustData.daily[index]
	end

	if aimItem then
		aimItem.id = info.task_id;
		aimItem.state = info.state;
		aimItem.restTime = info.last_time;
		aimItem.rate = info.percent;
	end
end

function EntrustModel:setReplaceID(id)
	self.replaceID = id;
end

function EntrustModel:addNewLogistics(info)
	if not self.replaceID then
		return;
	end

	if tonumber(infoself.replaceID) > 100 then
		for k, v in pairs(self.entrustData.urgent) do
			if v.id == self.replaceID then
				v.state = info.state;
				v.restTime = info.last_time;
				v.roleList = {};
				v.rate = info.percent;				
				break;
			end
		end
	else
		for k, v in pairs(self.entrustData.daily) do
			if v.id == self.replaceID then
				v.state = info.state;
				v.restTime = info.last_time;
				v.roleList = {};
				v.rate = info.percent;				
				break;
			end
		end
	end
	self.replaceID = nil;
end

function EntrustModel:ntNewEntrustModel(msg)
	if msg then
		return
	end
	if msg.index then
		if msg.index >= 0 and msg.index <= 4 then
			if msg.delegation_info then
				if not self.entrustData.daily then
					self.entrustData.daily = {};
				end
				local data = {};
				data.id = msg.delegation_info.task_id;
				data.state = msg.delegation_info.state;
				data.restTime = msg.delegation_info.last_time;
				data.roleList = {};
				data.rate = msg.delegation_info.percent;
				self.entrustData.daily[tonumber(msg.index)] = data;
			end
		elseif msg.index >= 5 and msg.index <= 8 then
			if msg.delegation_info then
				if not self.entrustData.urgent then
					self.entrustData.urgent = {};
				end
				local data = {};
				data.id = msg.delegation_info.task_id;
				data.state = msg.delegation_info.state;
				data.restTime = msg.delegation_info.last_time;
				data.roleList = {};
				data.rate = msg.delegation_info.percent;
				self.entrustData.urgent[tonumber(msg.index)] = data;
			end
		end
	end	
end

return EntrustModel; 
