local EquipModel = class("EquipModel")
local TeamType = Enum:getTable('TeamType')
function EquipModel:ctor()
	self:clearAll();
end

function EquipModel:clearAll(  )
	self.equipList = {};
	self.equipWear = {};
	self.equipComposesTableData = {};
end

function EquipModel:Init(equips, equip_wear)
	self:clearAll();
	for i = 1 , TEAM_MAX_MEMBER do
		self.equipWear[i] = {};
		for j = 1 , 4 do
			self.equipWear[i][j] = 0;
		end
	end
	if equips then
		for k, v in pairs(equips) do
			self.equipList[v.no] = v.num;
		end
	end
	if equip_wear then
		for k, v in pairs(equip_wear) do
			self.equipWear[v.team_slot][v.slot] = v.no;
		end
	end

	local equipCompose = ResTable:getTable('equip_compose').contain;
	for _k , eCompose in pairs(equipCompose) do
		local equipType = ResTable:getValue('equip' , eCompose.equip_id , 'type');
		if not self.equipComposesTableData[equipType] then
			self.equipComposesTableData[equipType] = {};
		end
		table.insert(self.equipComposesTableData[equipType] , eCompose);
	end
end

function EquipModel:getEquipComposeTableDataByType(equipType)
	return self.equipComposesTableData[equipType];
end

function EquipModel:getAllEquip()
	return self.equipList;
end

function EquipModel:getAllEquipWear()
	return self.equipWear;
end

function EquipModel:getEquipNum(no)
	if self.equipList[no] then
		return self.equipList[no]
	end
	return 0;
end

function EquipModel:onChangeEquips(msg)
	if type(msg) ~= 'table' then
		return;
	end
	for k, v in pairs(msg) do
		self.equipList[v.no] = v.num;
	end
end

function EquipModel:onChangeEquipWear(msg)
	if type(msg) ~= 'table' then
		return;
	end
	for k, v in pairs(msg) do
		self.equipWear[v.teamSlot][v.slot] = v.no;
	end
end

function EquipModel:getEquipSlotByIndex(index)
	return self.equipWear[index];
end

function EquipModel:getTeamSlotEquipActiveState(teamType)
	local team = {};
	teamType = teamType or TeamType.COMMON;
	local tid = model.TeamModel:getTeamIdByTeamType( teamType )
	team = model.TeamModel:getTeamById(tid);
	
	for i=1, TEAM_MAX_MEMBER do
		local qualityEquips = self:getActiveState(i);
		for slot, haveBetter in pairs(qualityEquips) do
			if haveBetter then
				return true;
			end
		end

		local pid = team[i];
		if pid > 0 and not model.CardModel:isEventRobotCard(pid) then
			local data = model.CardModel:GetCard(pid);
			if data and ((data.bare_id and data.bare_id <= 0) or (data.bare_id_2 and data.bare_id_2 <= 0 and data.star_lv>=10)) then
				local isHaveAllActiveBare,isHaveCanWear = model.BareCollectModel:isHaveAllActiveBare()
				if isHaveAllActiveBare and isHaveCanWear then
					return true;
				end
			end

			local data = model.CardModel:GetCard(pid);
			if data and data.love_id <= 0 then
				local loveDatas = model.CardModel:getLoveDataByRoleId(data.no);
				if loveDatas then
					for _k , data in pairs(loveDatas) do
						if data.loveLevel >= 5 then
							return true;
						end
					end
				end
			end
		end
	end
	
	return false;
end

function EquipModel:getActiveState(teamSlot)

	local equipQaulity = {};

	if not self.equipWear[teamSlot] then
		return equipQaulity;
	end

	for slot, no in pairs(self.equipWear[teamSlot]) do
		if no == 0 then
			local haveBetter = false
			for compareNo, num in pairs(self.equipList) do
				local equip_slot = ResTable:getValue('equip', compareNo, 'type');
				if equip_slot == slot and num > 0 then
					haveBetter = true;
					break;
				end
			end
			equipQaulity[slot] = haveBetter;
		else
			local quality = ResTable:getValue('item', no, 'quality');
			local haveBetter = false
			for compareNo, num in pairs(self.equipList) do
				local equip_slot = ResTable:getValue('equip', compareNo, 'type');
				if equip_slot == slot and num > 0  then
					local compareQu = ResTable:getValue('item', compareNo, 'quality');
					if compareQu > quality then
						haveBetter = true;
						break;
					end
				end
			end
			equipQaulity[slot] = haveBetter;
		end
	end
	return equipQaulity;
end

function EquipModel:getEquipComposeActiveState(equipType)
	local equipTypes = {};

	if equipType then
		equipTypes[equipType] = false; 
	else 
		equipTypes = {false , false , false , false};
	end

	for eType , state in pairs(equipTypes) do
		local equipCompose = self.equipComposesTableData[eType];
		if equipCompose then
			for _k , compose in pairs(equipCompose) do
				local haveNum = self:getEquipNum(compose.equip_cost[1]);

				local costNum =  #compose.equip_cost;
				if haveNum >= costNum then
					equipTypes[eType] = true;
					break;
				end
			end
		end
	end
	return equipTypes;
end

return EquipModel;
