local EventModel = class("EventModel")
local MessageBoxType = Enum:getTable('MessageBoxType')
local Attribute = Enum:getTable('Attribute');

EventModel.EVENT_MAP_ITEM_TYPES = {
	EVENT_MAP_ITEM_TYPE_NULL = 0;	-- kong
	EVENT_MAP_ITEM_TYPE_1 = 1;		-- 门
	EVENT_MAP_ITEM_TYPE_2 = 2;		-- 钥匙（钥匙怪，boss）（doorOpen == 0 显示怪物/boss， doorOpen == 1 显示钥匙）
	EVENT_MAP_ITEM_TYPE_3 = 3;		-- 普通怪
	EVENT_MAP_ITEM_TYPE_4 = 4;		-- 事件buff类型1 增益 手动触发
	EVENT_MAP_ITEM_TYPE_5 = 5;		-- 事件buff类型2 减益 立即触发
	EVENT_MAP_ITEM_TYPE_6 = 6; 		-- 宝箱类型1
	EVENT_MAP_ITEM_TYPE_7 = 7;		-- 宝箱类型2
	EVENT_MAP_ITEM_TYPE_8 = 8;		-- 宝箱类型3
	EVENT_MAP_ITEM_TYPE_9 = 9;		-- 事件buff类型3 复活道具 
	
}
EventModel.EVENT_MAP_COL = 5;
EventModel.EVENT_MAP_ROW = 7;
EventModel.EVENT_MAP_MAX_TIER = 5; -- 默认层级上限5 

EventModel.EVENT_ITEMID_1 = 13001; -- 特殊道具id 

function EventModel:ctor()
	self.hasInit = false;
	
	self.round_reward = {};

	self.event_id = 0;
	self.start_time = 0; --赛季起始时间
	self.close_time = 0; --赛季结束时间

	self.buff = {};
	self.att_buff = {};
	self.roleSkillBuff = {};
	self.godSkillBuff = {};
	self.sp5buff = nil;
	self.sp5Num = 0;
	self.sp6buff = {};
	self.sp7buff = nil;

	self.active_flag = 0;
	self.resetTime = 0; --再次进入的时间
	self.curPos = 0;
	self.movePoints = 0; --行动点，每移动一个格子 扣一点


	self.hard = 0;--难度id
	self.tier = 0;--层数

	self.thing_map = {};--随机事件
	self.tileList = {};--地块信息
	self.item_use = {};

	self.tile_buff = {};--地块上选择buff
	self.tile_robot = {};--地块上选择机器人
	self.tile_shop = {};--地块上旅行商人的售卖物品


	self.event_money = 0;
	self.color = -1;
	
	self.event_hard_log = {};
	self.event_welfare = {};

	self.eventMap = nil;
	self.selectValue = -1;
	self.selectPos = -1;

	self.endPos = 0;

	self.endFightBuffPos = 0;

	self.resultData = nil;

	self.eventBPPoint = 0;
	self.eventBPData = nil;
	self.eventBPRwdList = {};

end

function EventModel:isEnd()
	return self.curPos == self.endPos;
end

function EventModel:isInit()
	return self.hasInit;
end

function EventModel:initData(data)
	if data == nil then
		return ;
	end
	self.hasInit = true
	self:updateEventInfo(data);

	if data.buff_times and data.buff_times['5'] then
		self.sp5Num = data.buff_times['5'];
	end
end

function EventModel:reqEventMapInfo( tier )
	local msg = {};
    msg.openTier = tier;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_map_info, msg);
end

function EventModel:retEventMapInfo(data)
	self:updateEventInfo(data)
	EventManager:FireEvent(EventType.NetworkEventGetMapInfo);
end

function EventModel:updateEventInfo( data )
	self.tile_buff = {};--地块上选择buff
	self.tile_robot = {};--地块上选择机器人
	self.tile_shop = {};--地块上旅行商人的售卖物品
	
	if data.maze_id then
		self.event_id = data.maze_id;
	end

	if data.hard_no then
		self.hard = data.hard_no;
	end

	if data.tier then
		self.tier = data.tier;
	end

	if data.maze_point then
		self.movePoints = data.maze_point;
	end

	if data.buff then
		self.buff = data.buff;
		self:updateAttBuff()
	end

	if data.cur_pos then
		self.curPos = data.cur_pos
	end

	if data.thing_map then
		self.thing_map = data.thing_map
	end

	if data.open_state then
		self.tileList = data.open_state
	end

	if data.item_used_times then
		self.item_use = data.item_used_times
	end

	if data.natural_days_refresh_time then
		self.resetTime = data.natural_days_refresh_time
	end

	if data.maze_robot then
		self.maze_robot = data.maze_robot
	end

	if data.user_maze_bp then
		self.eventBPPoint = data.user_maze_bp.battlepass_point;
	end
	if data.maze_bp_open then
		self.eventBPData = data.maze_bp_open
		if data.user_maze_bp then
			self.eventBPData.reward_state = data.user_maze_bp.reward_state;
			self.eventBPData.reward_special_state = data.user_maze_bp.reward_special_state;
			self.eventBPData.special_state = data.user_maze_bp.special_state;
		end
		self:initBPRwdList();
	end

	self:updateRoundMapInfo()

	-- 再updateRoundMapInfo 之后执行
	if data.monster_hp then --monster_hp={"1"={23, {10000,10000,10000,10000}, {8990, 10000, 1212, 22}},...};
		for k,v in pairs(data.monster_hp) do
			local pos = tonumber(k)
			if self.mapInfo[pos] then
				if v[2] and #v[2]>0 then
					self.mapInfo[pos].monster_hp = v[2]
					self.mapInfo[pos].monster_cd = v[3]
				else
					self.mapInfo[pos].monster_hp = {10000,10000,10000,10000}
					self.mapInfo[pos].monster_cd = {0,0,0,0}
				end
				

				local monsterData = ResTable:getRowData('maze_monster',v[1]);
				self.mapInfo[pos].monster = monsterData.monster
				self.mapInfo[pos].formation = monsterData.formation
			end
		end
	end

	self.resultData = nil;
	
	
end

function EventModel:baseDataUpdata( data )
	if data.maze_point then
		self.movePoints = data.maze_point;
	end
	if data.cur_pos and data.cur_pos ~= self.curPos then
		self.curPos = data.cur_pos
		-- if self.curPos == self.endPos then
		-- 	if self.tier == 1 then
		-- 		self:reqEventMapInfo( self.tier+1 )
		-- 	elseif self.tier == 2 then
		-- 		MessageBox:create('选择关卡：选3 还是 选4？' , MessageBoxType.CANCEL_OK, function()
		-- 			self:reqEventMapInfo( self.tier+1 )
		-- 		end, function()
		-- 			self:reqEventMapInfo( self.tier+2 )
		-- 		end, function()
		-- 		end);
		-- 	end
		-- end
	end
	if data.pos_state and data.nextPos then
		local pos = data.nextPos;
		self.tileList[pos] = data.pos_state

		local open = 0
		if data.pos_state ~= 0 then
			open = 1
		end
		self.mapInfo[pos].open = open
		self.mapInfo[pos].state = data.pos_state
	end

end

function EventModel:checkShowPanel()
	if self.endFightBuffPos > 0 then
		UIManager:push('EventBuffSelectPanel', self.endFightBuffPos);
		self.endFightBuffPos = 0;
	end
end

function EventModel:retFightExitData(data)
	if data.pos_state == 2 and data.nextPos then
		local pos = data.nextPos;
		if self.mapInfo[pos].state == 1 then
			self.endFightBuffPos = pos;
		end
	end

	self:baseDataUpdata(data)
	if data.buff_times and data.buff_times['5'] then
		self.sp5Num = data.buff_times['5'];
	end

	if data.monsterTeam and data.nextPos then --更新怪物血量
		self.mapInfo[ data.nextPos ].monster_hp = data.monsterTeam[2]
		self.mapInfo[ data.nextPos ].monster_cd = data.monsterTeam[3]
	end

	if data.itemFirstUse and  data.itemFirstUse[1]==EventModel.EVENT_ITEMID_1 then
		self.item_use[''..EventModel.EVENT_ITEMID_1][2]=data.itemFirstUse[0]
	end
	EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);

end

function EventModel:getMovePoints()
	return self.movePoints
end

function EventModel:getFightBuffsById( tid )
	if self.tile_buff[tid] == nil then
		self:reqEventTileInfo(tid);
		return nil
	else
		return self.tile_buff[tid];
	end
end

function EventModel:getFightShopById( tid )
	if self.tile_shop[tid] == nil then
		self:reqEventTileInfo(tid);
		return nil
	else
		return self.tile_shop[tid];
	end
end

function EventModel:getFightRobotsById( tid )
	if self.tile_robot[tid] == nil then
		self:reqEventTileInfo(tid);
		return nil
	else
		return self.tile_robot[tid];
	end
end

function EventModel:reqEventTileInfo( pos )
	local msg = {};
    msg.nextPos = pos;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_tile_info, msg);
end

function EventModel:retEventTileInfo(data)
	-- ret = {
	-- 	nextPos = 34//请求的位置
	-- 	maze_point = 19//剩余行动点
	-- 	cur_pos = 34//当前位置
	-- 	pos_state = 3//当前位置的状态
	-- 	tingId = 10,//只有走到有事件的点才会返回次参数
	-- }
	-- print_ro(data)

	self:baseDataUpdata(data)

	if data.drop_buff and data.nextPos then
		self.tile_buff[ data.nextPos ] = data.drop_buff
		EventManager:FireEvent(EventType.UIUpdateEventBuff)
	end

	if data.employ and data.nextPos then
		self.tile_robot[ data.nextPos ] = data.employ
		EventManager:FireEvent(EventType.UIUpdateEventRobot)
	end

	if data.store and data.nextPos then
		self.tile_shop[ data.nextPos ] = data.store
		EventManager:FireEvent(EventType.UIUpdateEventShop)
	end

	if data.partnerHp then --血量更新
		local names = ''
		local bHaveDead = false;
		for k,v in pairs(data.partnerHp) do
			local tmp = model.CardModel:setEventHp(v[1], v[2])
			if bHaveDead == false then
				bHaveDead = tmp
			end

			local card = model.CardModel:GetCard(v[1]);
			local tmpName = ResTable:getValue('role_info', card.no, 'name');
			if k == 1 then
				names = tmpName;
			else
				names = names..','..tmpName;
			end
		end
		local tileData = self:getMapCellInfo(data.nextPos)
		if tileData then
			if tileData.type == 3 then --复活
				if names=='' then
					Prompt:Show( LANG.UI_MAZE_24 )
				else
					if bHaveDead then
						Prompt:Show( string.format(LANG.UI_MAZE_22, tostring(names)) )
					else
						Prompt:Show( string.format(LANG.UI_MAZE_23, tostring(names)) )
					end
				end
				
			elseif tileData.type == 4 then --泉水
				-- if names=='' then
				-- 	Prompt:Show( LANG.UI_MAZE_24 )
				-- else
				-- 	Prompt:Show( LANG.UI_MAZE_25 )
				-- end
				-- Prompt:Show( string.format(LANG.UI_MAZE_23, tostring(names)) )
			end
		end
	end

	local opt = {callback=function()
		EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);
	end}

	UIManager:push('RewardPanel' , data, opt); --不一定打开奖励面板
end

function EventModel:getEventId()
	return self.event_id or 0;
end

function EventModel:getCurPos()
	return self.curPos
end

function EventModel:getEventStartTime(  )
	if self.start_time == 0  then
		local eventOpen = ResTable:getRowData('maze_open',self.event_id);
		if eventOpen ~= nil then
			self.start_time = eventOpen.open_time
			self.close_time = eventOpen.close_time
		end
	end
	return self.start_time or 0;	
end

function EventModel:updateRoundMapInfo()
	-- body
	if self.tier == 0 then
		return
	end

	local hardData = ResTable:getRowData('maze_hard',self.hard);
	local roundNo = hardData['tier_'..self.tier]
	local monsterAtt = hardData['monster_attribute_'..self.tier]

	local roundData = ResTable:getRowData('maze_round',roundNo);

	local deployment = roundData.deployment;
	local deployment_monster = roundData.deployment_monster;
	local rewardId = roundData.reward;

	self.mapInfo = {}
	for k,v in pairs(self.tileList) do
		local open = 0
		if v ~= 0 then
			open = 1
		end
		self.mapInfo[k] = {pos=k, type=1, open=open, state=v, rwd=0}
	end

	for k,v in pairs(deployment) do
		local pos = v[3];
		if self.mapInfo[ pos ] then
			self.mapInfo[pos].thingId = v[1];
			self.mapInfo[pos].type = ResTable:getValue('maze_thing', v[1], 'type')
			if v[4] then
				self.mapInfo[pos].rwd = v[4]
			end
		end
	end

	for k,v in pairs(deployment_monster) do
		local pos = v[3];
		local monsterIdx = v[5]
		if self.mapInfo[ pos ] then
			self.mapInfo[pos].thingId = v[1];
			local tType = ResTable:getValue('maze_thing', v[1], 'type');
			self.mapInfo[pos].type = tType
			if v[4] then
				self.mapInfo[pos].rwd = v[4]
			end

			-- local monsterData = ResTable:getRowData('maze_monster',monsterIdx);
			-- self.mapInfo[pos].monster = monsterData.monster
			-- self.mapInfo[pos].formation = monsterData.formation
			if monsterAtt[ monsterIdx ] then
				self.mapInfo[pos].att = monsterAtt[ monsterIdx ]
			else
				self.mapInfo[pos].att = monsterAtt[ 1 ]
			end

			if tType == 10 then
				self.endPos = pos;
			end
			
		end
	end

	for k,v in pairs(self.thing_map) do
		if self.mapInfo[ v[1] ] then
			self.mapInfo[ v[1] ].thingId = v[2];
			self.mapInfo[ v[1] ].type = ResTable:getValue('maze_thing', v[2], 'type')
		end
	end
end

function EventModel:getMapInfo()
	return self.mapInfo;
end

function EventModel:getMapCellInfo( pos )
	return self.mapInfo[pos]
end

function EventModel:getCanClickTiles()
	-- body
end

function EventModel:getBPData()
	return self.eventBPData;
end

function EventModel:getBPRwdList(no)
	if no then
		for k,v in pairs(self.eventBPRwdList) do
			if v.no == no then
				return self.eventBPRwdList[k]
			end
		end
	end
	return self.eventBPRwdList;
end

function EventModel:updateBattlepassReward(msg)
	local rewardType = msg.rewardType;
	local no = msg.no
	for k,v in pairs(self.eventBPRwdList) do
		if v.no == no then
			if rewardType == 2 then
				self.eventBPRwdList[k].reward_special_state = 1;
			else
				self.eventBPRwdList[k].reward_free_state = 1;
			end
		end
	end
	EventManager:FireEvent('EventBattlePassPanel:retBattlepassReward' , msg);
end

function EventModel:initBPRwdList()
	self.eventBPRwdList = {};
	local bpId = self.eventBPData.battlepass_id;
    local mzBpRwds = ResTable:getTable('maze_bp_reward').contain;
    for k,v in pairs(mzBpRwds) do
        if v.battlepass_id == bpId then
            table.insert(self.eventBPRwdList, clone(v))
        end
    end
    local num = #self.eventBPRwdList;
    if num > 1 then
        table.sort(self.eventBPRwdList , function(first , second)
            return first.total_point > second.total_point;
        end);
    end

    for k,v in pairs(self.eventBPData.reward_state) do
    	for _,rwdData in pairs(self.eventBPRwdList) do
    		if rwdData.no == v then
    			rwdData.reward_free_state = 1
    			break;
    		end
    	end
    end

    for k,v in pairs(self.eventBPData.reward_special_state) do
    	for _,rwdData in pairs(self.eventBPRwdList) do
    		if rwdData.no == v then
    			rwdData.reward_special_state = 1
    			break;
    		end
    	end
    end

    -- table.insert(self.eventBPRwdList, {no=0})
    self:updateBPCurRwd();
end

function EventModel:updateBPCurRwd()
    local maxPoint = 0;
    local curLimitRwdId = 0;
    for k,v in pairs(self.eventBPRwdList) do
        if v.no>0 and self.eventBPPoint <= v.total_point then
            if maxPoint > v.total_point or maxPoint == 0 then
                maxPoint = v.total_point;
                curLimitRwdId = v.no;
            end
        end
    end
    self.eventBPData.curLimitRwdId = curLimitRwdId;
end

function EventModel:getEventBPPoint()
	return self.eventBPPoint
end

function EventModel:updateMazeBattlepassPoint(battlepassPoint)
	if battlepassPoint then
		for _k , addNum in pairs(battlepassPoint) do
			self.eventBPPoint = self.eventBPPoint+addNum
		end
		self:updateBPCurRwd();
	end
end

function EventModel:updateMazeBPActiveState( v )
	self.eventBPData.special_state = v;
	EventManager:FireEvent('EventBattlePassItemCell:openAcitve');
end

function EventModel:checkCanClick( pos )
	-- body
	if self.curPos+5 == pos or self.curPos-5 == pos then
		return true
	end

	local row1 = math.floor((self.curPos-1)/EventModel.EVENT_MAP_COL );
	local row2 = math.floor((pos-1)/EventModel.EVENT_MAP_COL );
	if row1==row2 and (self.curPos-1==pos or self.curPos+1==pos) then
		return true
	end
	return false
end

function EventModel:getResetTime()
	return self.resetTime;
end

function EventModel:isNeedRest()
	if self.resetTime < TimeManager:getCurTime() then
		return true
	end
end

function EventModel:reqEventMapFightBuff( buffid, tid, selBuffIdx )
	local msg = {};
	msg.buff = buffid;
	msg.nextPos = tid
	self.selBuffIdx = selBuffIdx;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_select_buff , msg);
end

function EventModel:retEventMapFightBuff( data )
	-- ret = {
	-- 	nextPos 	= 1,
	-- 	thingId 	= 1,
	-- 	maze_point 	= 1,
	-- 	cur_pos 	= 1,
	-- 	pos_state 	= 1,
	-- 	buff        = 1,
	-- }
	self:baseDataUpdata(data)

	self:updateBuff(data.buff);

	EventManager:FireEvent(EventType.NetworkEventSelectBuff, data.nextPos); --self.selBuffIdx

	EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);
end

function EventModel:updateBuff(buff)
	self.buff = buff; --返回当前全部buff
	self:updateAttBuff()
	-- for _k , _buff in pairs(self.buff) do
	-- 	if _buff == buff then
	-- 		return;
	-- 	end
	-- end

	-- table.insert(self.buff , buff);
end

function EventModel:updateAttBuff() --更新属性类型的buff
	self.att_buff = {};
	self.roleSkillBuff = {};
	self.godSkillBuff = {};
	self.sp5buff = nil;
	self.sp6buff = {};
	self.sp7buff = nil;

	for k, v in pairs(self.buff) do

		local buffData = ResTable:getRowData('maze_buff', v)
		if buffData.type == 1 then
			local color = buffData.color
			local job = buffData.job
			local key = 'ALL'
			if color==4 and job==0 then
				key = 'ALL'
			elseif color==4 then
				key = 'ALL_'..job
			elseif job==0 then
				key = ''..color..'_ALL'
			else
				key = ''..color..'_'..job
			end
			local attKey = Attribute[ buffData.value[1] ];
			local value = buffData.value[2];

			if not self.att_buff[key] then
				self.att_buff[key] = {}
			end

			if not self.att_buff[key][attKey] then
				self.att_buff[key][attKey] = value;
			else
				self.att_buff[key][attKey] = self.att_buff[key][attKey] + value;
			end
		elseif buffData.type == 3 then
			local roleNo = buffData.value[1];
			local skillType = buffData.value[2];
			local skillId = buffData.value[3];
			self.roleSkillBuff[ ''..roleNo..'_'..skillType ] = skillId;
		elseif buffData.type == 2 or buffData.type == 4 then
			table.insert(self.godSkillBuff, buffData.value);
		elseif buffData.type == 5 then
			self.sp5buff = {}
			self.sp5buff.maxNum = buffData.value[1];
			local attCnt = (#buffData.value-1) / 2;
			local idx = 1
			for i=1,attCnt do
				local attKey = Attribute[ buffData.value[idx+1] ];
				local value = buffData.value[idx+2];
				self.sp5buff[attKey] = value;
				idx = idx+2
			end

		elseif buffData.type == 6 then
			local attCnt = #buffData.value / 2;
			local idx = 1
			for i=1,attCnt do
				local attKey = Attribute[ buffData.value[idx] ];
				local value = buffData.value[idx+1];
				if self.sp6buff[attKey] == nil then
					self.sp6buff[attKey] = value;
				else
					self.sp6buff[attKey] = self.sp6buff[attKey] + value;
				end
				idx = idx+2
			end

		elseif buffData.type == 7 then
			self.sp7buff = buffData.value[1];
			table.insert(self.godSkillBuff, buffData.value);
		end
	end

end

function EventModel:getSpBuffByType(t)
	if t == 7 then
		return self.sp7buff;
	end
	return nil;
end

function EventModel:getBuffAttByColorAndJob(color, job, att)
	local key = ''..color..'_'..job;
	local v1 = 0
	if self.att_buff[key] and self.att_buff[key][att] then
		v1 = self.att_buff[key][att]
	end
	local v2 = 0
	local key1 = 'ALL_'..job;
	local key2 = ''..color..'_ALL';
	if self.att_buff[key1] and self.att_buff[key1][att] then
		v2 = v2+self.att_buff[key1][att]
	end
	if self.att_buff[key2] and self.att_buff[key2][att] then
		v2 = v2+self.att_buff[key2][att]
	end
	if self.att_buff['ALL'] and self.att_buff['ALL'][att] then
		v2 = v2+self.att_buff['ALL'][att]
	end
	if att=='atk' and self.item_use[''..EventModel.EVENT_ITEMID_1] and self.item_use[''..EventModel.EVENT_ITEMID_1][2]==1 then
		v2 = v2+0.3
	end
	local v3 = 0
	if self.sp6buff and self.sp6buff[att] and (self.tier==3 or self.tier==4) then
		v3 = self.sp6buff[att]
	end
	local v4 = 0
	if self.sp5buff and self.sp5Num>0 then
		local num = self.sp5buff.maxNum > self.sp5Num and self.sp5Num or self.sp5buff.maxNum;
		v4 = self.sp5buff[att] * num;
	end
	return v1+v2+v3+v4;
end

function EventModel:getReplaceSkill(roleNo, skillType)
	local key = ''..roleNo..'_'..skillType;
	if self.roleSkillBuff[key] then
		return self.roleSkillBuff[key]
	end
	return nil;
end

function EventModel:getGodSkillList()
	return self.godSkillBuff;
end

function EventModel:getBuff()
	return self.buff;
end

function EventModel:getEventResultData()
	return self.resultData;
end

function EventModel:reqEventResultData()
	local msg = {};
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_result , msg); 
end

function EventModel:retEventResult( data )
	if data then
		self.resultData = data;
		EventManager:FireEvent('EventType.EventResultPanel');
	end
end

function EventModel:reqGiveUpShopOnce(tid)
	local msg = {};
	msg.nextPos = tid
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_giveup_shop , msg);
end

function EventModel:retGiveUpShopOnce(data)
	self:baseDataUpdata(data)
	EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);
end

function EventModel:reqEventMapShopOnce( itemId, tid )
	local msg = {};
	msg.items = {itemId};
	msg.nextPos = tid
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_select_shoponce , msg);
end

function EventModel:retEventMapShopOnce( data )
	-- ret = {
	-- 	nextPos 	= 1,
	-- 	thingId 	= 1,
	-- 	maze_point 	= 1,
	-- 	cur_pos 	= 1,
	-- 	pos_state 	= 1,
	-- 	items       = 1,
	-- }
	self:baseDataUpdata(data)
	if data.items and data.nextPos then
		self.tile_shop[ data.nextPos ] = data.items
	end
	EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);
	EventManager:FireEvent(EventType.UIUpdateEventShop);

	UIManager:push('RewardPanel',data)
end

function EventModel:reqEventMapEmployRobot( robotId, tid )
	local msg = {};
	msg.robotNo  = robotId;
	msg.nextPos = tid
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_employ_robot , msg);
end

function EventModel:retEventMapEmployRobot( data )
	-- robot = {
	-- 		rid 	= 1,
	-- 		rno 	= 1,
	-- 		level 	= 1,
	-- 		break_lv = 1,
	-- 		star_lv = 1,
	-- 		aid 	= 1,
	-- 		event_hp = 1,
	-- 		event_cd = 1,
	-- 	},
	self:baseDataUpdata(data)
	if data.robot then
		model.CardModel:addEventRobotCard( { data.robot } )
		if self.maze_robot == nil then
			self.maze_robot = {}
		end
		table.insert(self.maze_robot, data.robot)
	end
	EventManager:FireEvent(EventType.UIUpdateEventPos, data.nextPos);
	Prompt:Show(LANG.UI_MAZE_27)

end

function EventModel:addRobotToCardModel()
	model.CardModel:addEventRobotCard( self.maze_robot );
	model.TeamModel:updateTeamBareAllAtt( 5 )
end

function EventModel:removeRobotFromCardModel()
	self.maze_robot = model.CardModel:removeAllEventRobotCard();
end

function EventModel:useItem( itemId )
	local itemData = model.ItemModel:getItem(itemId);
	if itemData and itemData.num<=0 then
		Prompt:Show(LANG.ERROR_INFO[9063]);
		return
	end
	local msg = {};
	msg.itemNo = itemId;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_use_item , msg);
end

function EventModel:retUseItem( data )
	if data then
		if data.itemNo == EventModel.EVENT_ITEMID_1 then
			model.CardModel:resetEventALLCardHP();
			self.item_use[''..data.itemNo] = {data.useTimes, data.useFirst}
		else
		end
	end
end

function EventModel:isHaveDeadOrBadRole()
	local cards = model.CardModel:GetAllCard()
	for k, v in pairs(cards) do
		if v.level >= 60 and v.event_hp>0 and v.event_hp<10000 then
			return true
		end
	end
	return false;
end

function EventModel:isHaveDeadRole()
	local cards = model.CardModel:GetAllCard()
	local st = 0
	for k, v in pairs(cards) do
		if v.level >= 60 then
			if v.event_hp<=0 then
				st = 1
				return st;
			elseif v.event_hp>0 and v.event_hp<10000 then
				st = 2
			end
		end
	end
	return st;
end

function EventModel:getMonsterFpLv( tid )
	local fightInfo = self:getMapMonsterInfo(tid)
	local enemyInfo = {}
	for k, v in pairs(fightInfo.monster) do
		table.insert(enemyInfo, {[1] = v, [2] = fightInfo.att})
	end
	local totalFp = '0';
	local maxLv = 0;
	local restraintFactor,enemyRestraintString = CardProManager:GetMonsterRestraintFactor(enemyInfo);
	for _k , data in pairs(enemyInfo) do
		local pro = CardProManager:GenMonsterPro(data[1], data[2],restraintFactor);
		if pro then
			totalFp = HpPlus(totalFp, ConvertDigitUnitToStr(pro.fp));
		end
		local attributeData = ResTable:getRowData('monster_attribute', data[2]);
		if attributeData.lv > maxLv then
			maxLv = attributeData.lv
		end
	end
	return totalFp, maxLv;
end

------------------------fubin--------------------

function EventModel:getActiveFlag()
	return self.active_flag;
end

function EventModel:getFpNo()
	return self.fp_no;
end

function EventModel:getHard()
	return self.hard;
end

function EventModel:getTier()
	return self.tier;
end



function EventModel:getEventWelfareData()
	return self.event_welfare;
end

function EventModel:getEventWelfareDataById(id)
	return self.event_welfare[id]
end


function EventModel:getEventCloseTime(  )
	-- if not self:isInit()  then
	-- 	self:reqEventInfo()
	-- 	return 0;
	-- end
	return self.close_time or 0;	
end


function EventModel:getHardLog()
	return self.event_hard_log;
end

function EventModel:getEventTotalMoney()
	return self.event_money or 0;
end

function EventModel:updateEventTotalMoney(eventMoney)
	if eventMoney and self.event_money then
		for _k , money in ipairs(eventMoney) do
			self.event_money = money.totalNum;
		end
	end
end	

function EventModel:getEventRewardState(id)
	return self.round_reward[id];
end

function EventModel:updateEventRewardState(id)
	self.round_reward[id] = 1;
end

function EventModel:setEventId(event_id)
	self.event_id = event_id
end

function EventModel:reqEventInfo(  )
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_get_info, {});
end

function EventModel:retEventActive(msg)
	if msg.fp_no then
		self.fp_no = msg.fp_no;
	end

	if msg.hard then
		self.hard = msg.hard;
	end

	if msg.tier then
		self.tier = 1;
	end

	if msg.active_flag then
		self.active_flag = msg.active_flag;
	end

	if msg.color then
		self.color = msg.color
	end

	if msg.buff then
		self.buff = msg.buff;
		self:updateAttBuff();
	end
end

function EventModel:getLockStateEvent(  )
	return self.event_id  == -1;
end

function EventModel:isShowEventBp()
	local ret = false;
    if self.eventBPData then
    	local endTime = self.eventBPData.show_time;
		local lastTime = endTime - TimeManager:getCurTime()
		if lastTime > 0 then
			ret = true;
		end
    end
    return ret;
end

----------------------------------------------------------------event   map ---------------------------------

function EventModel:hasMapInit(  )
	return self.eventMap ~= nil and  self.eventMap.hasInit == true ;
end

function EventModel:reqEventReset(  )
	local msg = {};
	self.selectValue = -1;
	msg.event_id = self.event_id
	Network.NetworkManager:Send(Network.NetworkCmdType.req_event_reset, msg);
end

function EventModel:retEventRest( msg )
	self:updateEventInfo(msg.event);
	if  msg.newFlag == 1  then
		-- 通关当前层 ,进入新层级
		self:retEventMapInfo(msg);
	else
		-- 通关当前难度 / 重置 难度
		self.eventMap.hasInit = false ;
	end
end

function EventModel:getEventMapInfo(  )
	return self.eventMap;
end


function EventModel:getEventMapDoorState(  )
	return self.eventMap.doorOpen ;
end

function EventModel:getEventMapDoorPos( )
	return self.endPos;
end

function EventModel:getMapMonsterInfo( pos )
	return self.mapInfo[pos]
	-- if self.eventMap.monsterHash[pos] == nil then
	-- 	return nil;
	-- end

	-- return self.eventMap.fight[self.eventMap.monsterHash[pos]];
end

--------------------------------------------------------------------------------------------------------------

function EventModel:updateHardLog(eventHardLog)
	if eventHardLog then
		self.event_hard_log = eventHardLog;
	end
end
return EventModel;
