local GachaModel = class("GachaModel")
-- local scheduler = cc.Director:getInstance():getScheduler();

function GachaModel:ctor()
	self:clearAll()
end

function GachaModel:clearAll(  )
	self.hasInit = false;
	self.gachaTimes = {};
	self.gachaRwdTimes = {};
	self.gachaRewardTimes = 0; -- gacha[0][0].reward_times
	self.nextGachaTimesRewardTimes = 100 -- 抽卡次数奖励，固定50次领取
	self.gachaLimitTimes = 0; -- 每日剩余抽卡次数

	self.contractBagTableData = {};
	self.bagOrderList = {}
	self.bagRateList = {}
	--self.contractTypeMax = {};
	self.contractLength = 0;
end
function GachaModel:IsInit()
	return self.hasInit;
end

function GachaModel:Init(gacha)
	self:clearAll()

	self:setGachaTimes(gacha.gachaTimes);
	self.gachaLimitTimes = gacha.day_limit_times or 0;
	local contractBag = ResTable:getTable('contract_bag').contain;
	if contractBag then
		for _k , data in pairs(contractBag) do
			if tonumber(data.start_time) ==-1 or data.end_time > TimeManager:getCurTime() then

				if not self.contractBagTableData[data.tab] then
					self.contractBagTableData[data.tab] = {};
					self.contractLength = self.contractLength + 1;
				end
				self.contractBagTableData[data.tab] = data;
				table.insert(self.bagOrderList, data.tab);

				--if self.contractTypeMax[data.tab] == nil or self.contractTypeMax[data.tab] < data.no  then
				--	self.contractTypeMax[data.tab] = data.no;
				--end
			end
		end
	end

	table.sort(self.bagOrderList , function(first , second)
		return self.contractBagTableData[first].no > self.contractBagTableData[second].no;
	end);

	self.contractTableData = {};
	local contractTable = ResTable:getTable('contract').contain;
	if contractTable then
		for _k , contract in pairs(contractTable) do
			if not self.contractTableData[contract.contract_no] then
				self.contractTableData[contract.contract_no] = {};
			end
			table.insert(self.contractTableData[contract.contract_no] , contract);
		end
	end

	self.hasInit = true;

end

function GachaModel:checkBagListAndReq()
	Network.NetworkManager:Send(Network.NetworkCmdType.req_gacha_info, {});
end

function GachaModel:retGachaInfo(msg)
	self.bagRateList = {}
	for k,v in pairs(msg.contract_validity) do
		self.bagRateList[ v[1] ] = v
	end

	if msg.operates then
		model.OperateModel:updateData(msg.operates);
	end

	EventManager:FireEvent(EventType.NetworkGachaInfo);
end

function GachaModel:getBagRateData( tab )
	return self.bagRateList[tab];
end

function GachaModel:getBagOrderList()
	return self.bagOrderList;
end

function GachaModel:getContractData( gachaType,gachaNo )
	-- body

	if self.contractBagTableData[gachaType] == nil then
		return nil;
	end
	return self.contractBagTableData[gachaType];

end

function GachaModel:getContractTableData(tableNO, sp)
	if self.bagRateList[tableNO] then
		local contractNo = self.bagRateList[tableNO][2]

		if not self.contractTableData[contractNo] and sp==2 then
			self.contractTableData[contractNo] = {};

			local ctrQaTable = ResTable:getTable('quality_bag').contain;
			local roleBagTable = ResTable:getTable('role_bag').contain;
			if ctrQaTable and roleBagTable then
				for _k , ctrBag in pairs(ctrQaTable) do
					if ctrBag.bag_no == contractNo then

						for k,v in pairs(roleBagTable) do
							if v.quality_no == ctrBag.quality_no then
								local data = {}
								data.id = v.id;
								data.contract_no = contractNo;
								data.reward_no = v.role_id;
								data.quantity = v.quantity;
								data.rate = v.role_rate * ctrBag.quality_rate * 0.0001;
								table.insert(self.contractTableData[contractNo] , data);
							end
						end
					end
				end
			end

		end

		return self.contractTableData[contractNo];
	else
		return nil
	end
end


function GachaModel:getGachaList()
	return self.contractBagTableData;
end

function GachaModel:getContractLength( ... )
	return self.contractLength
end
--[[
function GachaModel:getContractTypeMaxID( gachaType )
	return self.contractTypeMax[gachaType];
end
]]

function GachaModel:getCurrentGachaTimesRewardTimes(  )
	return self.nextGachaTimesRewardTimes;
end

function GachaModel:setGachaTimes(gachaTimes)
	if gachaTimes == nil then
		return 
	end
	local rewardTimes = 0
	self.gachaTimes = {}
	self.gachaRwdTimes = {}
	if gachaTimes then
		for _, data in pairs(gachaTimes) do 
			if data['type'] and data['reward_times'] then
				self.gachaTimes[tonumber(data['type'])] = data['times'];
				self.gachaRwdTimes[tonumber(data['type'])] = data['reward_times'];
				if tonumber(data['type']) == 0 then
					rewardTimes = data['reward_times'];
				end
			end
		end
	end
	self.gachaRewardTimes = rewardTimes;
end

function GachaModel:getGachaTimes()
	return self.gachaTimes;
end

function GachaModel:getGachaLimitTimes(  )
	return self.gachaLimitTimes
end
function GachaModel:getGachaTimesByType( gachaType )
	if not self.gachaTimes or not self.gachaTimes[gachaType] then
		return 0;
	end
	return self.gachaTimes[gachaType];
end

function GachaModel:getGachaRwdTimesByType( gachaType )
	if not self.gachaRwdTimes or not self.gachaRwdTimes[gachaType] then
		return 0;
	end
	return self.gachaRwdTimes[gachaType];
end

-- 获取已抽卡并未领取次数 领取单位次数 进度
function GachaModel:getRewardTimes()
	local totalTimes = self:getGachaTimesByType(0);
	local leftTimes = totalTimes - self.gachaRewardTimes;
	local progress = 100;
	if leftTimes < self.nextGachaTimesRewardTimes then
		progress = math.floor(leftTimes / self.nextGachaTimesRewardTimes*100)
	end
	return leftTimes, self.nextGachaTimesRewardTimes, progress;
end

function GachaModel:setGachaAchievement( achievementPiece )
	self.achievementPiece = achievementPiece;
end

function GachaModel:getGachaAchievement(  )
	return self.achievementPiece;
end

function GachaModel:preloadCV(gachaResults)

	local cvLoadList = {};
	for i=1, #gachaResults do
		local info = {};
		local gachaItem = gachaResults[i];
		if gachaItem['partner'] then
			local no = gachaItem['partner'].no;
			cvLoadList[no] = true;
		end
	end
	AudioManager:preloadCVList(cvLoadList);

	-- if not self.loadCVHandler then
	-- 	self.time = 0;
	-- 	self.loadCVHandler = scheduler:scheduleScriptFunc(handler(self, self.update), 0, false);
	-- end
	-- self.preloadCVFlag = true;

	-- self.coroutine = coroutine.create(function()
	-- 	for k, v in pairs(self.cvLoadList) do
	-- 		local role_id = ResTable:getValue('role', k, 'role_id');
	-- 		AudioManager:loadCV(role_id, 'Gacha');
	-- 		coroutine.yield();
	-- 	end
	-- 	self.preloadCVFlag = false;
	-- 	if self.loadCVHandler then 
	-- 		scheduler:unscheduleScriptEntry(self.loadCVHandler)
	-- 		self.loadCVHandler = nil 
	-- 	end
	-- end)
end

-- function GachaModel:update(dt)
-- 	self.time = self.time + 1;
-- 	if self.time % 5 == 1 and self.preloadCVFlag then
-- 		if self.coroutine then
-- 			coroutine.resume(self.coroutine);
-- 		end
-- 	end
-- end


function GachaModel:reqGacha( no,flag )
	local msg = {
		no 	= no,
		flag    = flag,
	}
 	local	guideStep = model.GuideModel:getGuideStep() ; -- 新手引导阶段
	if guideStep == model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_1 or guideStep == model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_7 then
		msg.guideStep = guideStep;
	end
	 Network.NetworkManager:Send(Network.NetworkCmdType.req_gacha, msg);
end

function GachaModel:retGachaResult( msg )
	self.gachaLimitTimes = msg.day_limit_times or 0;

	-- self:setGachaResult(msg.gachaResult);
	self:setGachaTimes(msg.gachaTimes);
	self:setGachaAchievement(msg.achivementPiece);
	self:preloadCV(msg.gachaResult);
end

function GachaModel:reqGachaTimesReward( rwdId )
	

	local msg = {
	}
	if rwdId then
		msg.id = rwdId
	end
	Network.NetworkManager:Send(Network.NetworkCmdType.req_gacha_get_reward, msg);

end

function GachaModel:retGachaTimesReward( msg )
	-- dum(msg,)
	if msg == nil or msg.gacha == nil  then
		return 
	end
	self:setGachaTimes(msg.gacha.gachaTimes);
end

function GachaModel:getRedDotStateGacha(  )
	if self:getLockStateGacha() then
        return false;
    end
	if not self:IsInit() then
		return false;
	end


	-- 1 有抽卡次数奖励
	local _, _, progress = self:getRewardTimes();
	if progress >= 100 then
		return true ;
	end

	-- 2 有抽卡券
	local allTickets = {11001,11101,11201}
	for k,v in pairs(allTickets) do
		local num = model.ItemModel:getItemNum(v);
		if num > 0 then
			return true ;
		end
	end

	return false;
end

function GachaModel:getLockStateGacha(  )
    return tools.LevelLockTools:checkLockById(1000);		
end

return GachaModel;
