local GuideModel = class('GuideModel')
local scheduler = cc.Director:getInstance():getScheduler();

local STEP_FORCE_MAX = 100 -- 强制引导阶段最大值< 1-99>
						   -- 弱引导阶段 <101-199>
local STEP_ROUND_MIN = 200 -- 关卡引导阶段<201-299> < level作为关卡使用>

GuideModel.GUIDE_STEP_ENUM = {
	GUIDE_ENUM_1	= 1, 	-- 抽卡，角色6
	-- GUIDE_ENUM_2	= 2,	
	-- GUIDE_ENUM_3	= 3,
	-- GUIDE_ENUM_4	= 4,
	GUIDE_ENUM_6	= 6,	-- 随机商店
	GUIDE_ENUM_7	= 7,	-- 抽卡2，角色49
	GUIDE_ENUM_8	= 8,	-- 挂机奖励
	GUIDE_ENUM_ROUNDBOSS  = 107,	-- 随机boss
	GUIDE_ENUM_ARENA = 106,	-- 竞技场

}

GuideModel.START_GUIDE_ID = 6 -- 起始引导ID

function GuideModel:ctor( ... )
	self:clearAll();
end

function GuideModel:clearAll( ... )
	self.showGuide = true;

	self.guide = {}

	self.guideEnterPath = {};

	self.guide_weak_complete = {};

	self.isNeedSaveStep = false;--判断当前引导步骤 是否需要保存数据
	self.willSaveStep = 0;--正在保存到后台的 引导id

	--waiteFightOver 在战斗开始的时候 此标记为 true， 战斗结束后 为 false，
	--用于战斗 开始时 不向后台存储引导数据， 战斗结束后 向后台存储数据
	self.waiteFightOver = false;

	self.bUseOtherPath = false;

	self.curGuideId = 0; --当前引导id
	self.lastStep = 0;
	self.bJustEnd = false;
	self.curGuideData = nil;
	self.bfind = false;
	self.bStart = false;

	self.tmpStopGuide = false;
	self.timeOut = 0;
end

function GuideModel:Init( guideNo,weakComplete ,userLv)

	-- guideNo = 47;
	if guideNo == nil or guideNo == 0 then
		guideNo = GuideModel.START_GUIDE_ID; --从 编队开始 引导
	end
	self.curGuideId = guideNo;

	-- 已完成的弱引导
	if weakComplete == nil or type(weakComplete) ~= 'table' then
		weakComplete = {}
	end
	for k,v in pairs(weakComplete) do
		self.guide_weak_complete[v] = 1;
	end
	local tmpGD = self:getGuideStepData()
	if tmpGD and self.guide_weak_complete[tmpGD.no] then
		if tmpGD.order == 1 then
			self.guide_weak_complete[tmpGD.no] = nil;
		end
	end
	
	self:updateGuide(userLv);

end

--[[
	guide_newer = {
		no  		阶段 
					1-99 	强引导 限定角色等级 新角色必要引导步骤 顺序进行
					100-199 弱引导	 解锁后，点击进入对应界面 或者 从主界面进入 开启引导
		level		0 不做引导，直接跳过，
		order 		当前阶段 对应进度
		flag 		0 过场进度
					1 重要进度，需要保存，<强退再次进入时，则直接跳过后续进度，执行下一阶段>
					2 次要进度，需要保存，<强退再次进入时，需要重新引导此阶段，并跳过flag=2进度，快速定位到下一进度>
			无效了	-1 可省略进度，<起始进度不在主界面， 强退再次进入时，需要从主界面开始引导>
		guideType 	类型： 
			0 默认，传递触摸事件，执行下一步引导或结束引导
			
			101 传递触摸，直接结束引导
			102 传递触摸，返回主页面

			201 不传递触摸，直接下一步引导
			202 不传递触摸，强制返回主界面
			
			-- 特殊处理
			111
			211 212 213
		waitTime 	添加触摸屏蔽时间，默认0 
		showPanel 		
		afterPanel
		coor 以640*1136 为基准，距离屏幕中心坐标位置
		touchRadius 触摸半径
		adaptationType 适配坐标方案 0 相对屏幕中心调节坐标，1 相对底部调节坐标，2 相对顶部调节坐标
	}

]]

function GuideModel:updateGuide( userLv )

	local curGuideItem = ResTable:getRowData('guide_newer',self.curGuideId);
	if curGuideItem == nil then
		self.curGuideId = 0;
		return ;
	end

	--校正引导的 新方法
	if curGuideItem.order == 1 then --记录的第一步 不做变动
	else
		--记录的不是第一步
		local stfind = true
		while stfind do
			if curGuideItem.next_id == nil or curGuideItem.next_id == 0 then
				stfind = false;
				self.curGuideId = 0;
				break;
			else
				curGuideItem = ResTable:getRowData('guide_newer', curGuideItem.next_id);

				if curGuideItem.order == 1 then
					stfind = false;
					self.curGuideId = curGuideItem.id;
				else
					if curGuideItem.otherPath and #curGuideItem.otherPath>0 then --新入口支线
						stfind = false
						self.guideEnterPath = clone(curGuideItem.otherPath)
						table.insert( self.guideEnterPath, curGuideItem.id )
						self.curGuideId = table.remove( self.guideEnterPath, 1)
						
						self.bUseOtherPath = true
					else --非配置错误
						-- self.curGuideId = 0;
						-- --定位选择 本节 第一步   这种情况是不存在的，
						-- --因为 保存的步骤 flag肯定为 1 或者 2, 
						-- --如果为 1，本阶段结束，查找下一个阶段 order==1的，如果没有则 以为着整个引导结束
						-- --如果为 2，则本阶段 后面还有 flag等于1的 未执行
					end
				end

			end
			
		end
	end

	self:checkLevelUpGuide( userLv )

end

function GuideModel:checkLevelUpGuide( lv )
	if self.curGuideId == 0 then
		for k, v in pairs(ResTable:getTable('guide_newer').contain) do
			if v.no>STEP_FORCE_MAX and v.no<STEP_ROUND_MIN and lv == v.level and v.order == 1 then -- 未完成
				if self.guide_weak_complete[v.no] ~= 1 then
					self.curGuideId = v.id
					self:reqSaveGuideStep(self.curGuideId)
					break;
				end
			end
		end
	end

	if self.curGuideId ~= 0 then
		self.lastStep = self.curGuideId;
		self.curGuideData = self:getGuideStepData()
	end

	if self.m_scheduleHandler == nil then
		self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.findTargetOnEnterFrame), 0.3, false);
	end

end

function GuideModel:checkRoundGuide( round )
	if self.curGuideId == 0 then
		for k, v in pairs(ResTable:getTable('guide_newer').contain) do
			if v.no>STEP_ROUND_MIN and round == v.level and v.order == 1 then
				self.curGuideId = v.id
				break;
			end
		end

		if self.curGuideId ~= 0 then
			self.curGuideData = self:getGuideStepData()
			if self.m_scheduleHandler == nil then
				self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.findTargetOnEnterFrame), 0.3, false);
			end
		end
	end

end

function GuideModel:triggerGuide( gid )
	if self.curGuideId == 0 then
		self.curGuideId = gid

		if self.curGuideId ~= 0 then
			self.lastStep = self.curGuideId;
			self.curGuideData = self:getGuideStepData()
			if self.m_scheduleHandler == nil then
				self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.findTargetOnEnterFrame), 0.3, false);
			end
		end
	end

end

function GuideModel:stopGuide()
	self.tmpStopGuide = true
	EventManager:FireEvent(EventType.ShieldGuide, false);
	UIManager:pop('GuidePanel');
end

function GuideModel:resumeGuide()
	self.tmpStopGuide = false
	if self:isInGuide() then
		EventManager:FireEvent(EventType.ShieldGuide, true);
	end
end

function GuideModel:findTargetOnEnterFrame()
	-- print('fubin-------------------findTargetOnEnterFrame-------------------: ',self.curGuideId)
	if self.tmpStopGuide then --暂时关闭引导
		-- print('fubin--1-------: ',self.curGuideId)
		return;
	end
	if self.curGuideId == 0 then
		-- print('fubin--2-------: ',self.curGuideId)
		if self.m_scheduleHandler then
			scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
			self.m_scheduleHandler = nil
		end
		return;
	end
	if self.bStart == false then
		-- print('fubin--3-------: ',self.curGuideId)
		return
	end
	if self.bfind then
		-- print('fubin--4-------: ',self.curGuideId)
		return;
	end

	local target = UIManager:getLayer(self.curGuideData.target) or nil
	local aniEnd = true
	if self.curGuideData.target == 'FooterPanel' and target then
		aniEnd = target:isAniEnd()
	end
	if aniEnd and target and UIManager:panelIsVisible( self.curGuideData.target ) then --找到目标了
		-- print('fubin--5-------: ',self.curGuideId)
		self.bfind = true;

		if self.curGuideData.guideType == 112 or self.curGuideData.guideType == 113 then
			UIManager:switch('FightPanel');
		elseif self.curGuideData.guideType == 214 then 
			UIManager:switch('MainPanel');
		end
		self:checkTrackEvent()
		UIManager:push('GuidePanel')
		self.timeOut = 0;
	else
		self.timeOut = self.timeOut + 1
		if self.timeOut >= 30 then --超时后 解除屏蔽层
			-- print('fubin--guide time out-------: ',self.curGuideId)
			self.bStart = false;
			EventManager:FireEvent(EventType.ShieldGuide, false);
			UIManager:pop('GuidePanel');
		end
		-- print('fubin--6-------: ',self.curGuideId)
	end

end

function GuideModel:checkTrackEvent( ... )
	if self.curGuideData == nil then
		return
	end
	local guideId = self.curGuideData.id
	local eventName = nil
	if guideId == GuideModel.START_GUIDE_ID then -- 引导开启
		eventName = 'TRACK_TU_START'
	elseif guideId == 11 then -- 关卡1
		eventName = 'TRACK_TU7_STAGE1'
	elseif guideId == 19 then  -- 角色升级
		eventName = 'TRACK_TU15_LEVELUP'
	elseif guideId == 25 then 	-- 关卡2
		eventName = 'TRACK_TU22_STAGE2'
	elseif guideId == 32 then 	-- 抽卡
		eventName = 'TRACK_TU26_CARD'
	elseif guideId == 44 then 	-- 关卡3
		eventName = 'TRACK_TU37_STAGE3'
	elseif guideId == 52 then 	-- 关卡4
		eventName = 'TRACK_TU43_STAGE4'
	end
	if eventName ~= nil then
		--[[
		local args = {
			eventName = eventName
		}
		sdkManager.SDKLuaHelper:trackEvent(args)
		]]
		GlobalFunctions:trackEvent(eventName)
	end


end

function GuideModel:isInGuide()
	return self.curGuideId > 0 ;
end

function GuideModel:initCheckGuide(  )
end

function GuideModel:getGuideStepData( guideId )
	local tmpid = guideId or self.curGuideId;
	if tmpid == 0 then
		return nil;
	else
		return ResTable:getRowData('guide_newer',tmpid);
	end 
end

function GuideModel:onSaveGuideNotice()
	-- print('fubin-----onSaveGuideNotice----:', self.curGuideId)
	self:switchIsNeedSaveStep(false)
	self.willSaveStep = 0;
	self:setNextGuideStep();
end

function GuideModel:switchWaiteFightOver()
	if self.isNeedSaveStep then --本次需要保留引导数据，直到战斗结束才发送  req_enter_fight
		self.waiteFightOver = true
		self:stopGuide() --暂时关闭引导，战斗结束后，并且点击完面板，再次开启
	else
		if self.waiteFightOver then --战斗结束了，并且这时是需要传输 保存引导数据的  req_exit_fight
			self.waiteFightOver = false --此标记 生命周期结束
			self.isNeedSaveStep = true
		end
	end
end
function GuideModel:switchIsNeedSaveStep(open)
	self.isNeedSaveStep = open
end
function GuideModel:isWillSaveGuide()
	local guideData = self:getGuideStepData()
	if guideData and (guideData.flag==1 or guideData.flag==2) then
		return true
	end
	return false
end
function GuideModel:getSaveGuideStepData()
	if self.waiteFightOver then
		return nil
	end
	if self.isNeedSaveStep then
		local guideData = self:getGuideStepData()
		if guideData and (guideData.flag==1 or guideData.flag==2) then
			self.willSaveStep = guideData.id
			return guideData.id
		else
			return nil
		end
	else
		return nil
	end
end

function GuideModel:setNextGuideStep(   )
	
	-- print('fubin--setNextGuideStep--111 ===cur: ',self.curGuideId);
	local guideData = self:getGuideStepData()
	self.curGuideId = guideData.next_id
	-- print('fubin--setNextGuideStep--222 ===next: ',self.curGuideId);

	self.bUseOtherPath = false
	if self.guideEnterPath and #self.guideEnterPath>0 then --新入口支线引导步骤
		self.curGuideId = table.remove( self.guideEnterPath, 1)
		-- print('fubin--setNextGuideStep--333 ===oth: ', self.curGuideId);

		self.bUseOtherPath = true
	end

	if self.curGuideId ~= 0 then
		local nextGuideData = self:getGuideStepData(self.curGuideId);
		if nextGuideData == nil then
			self.curGuideId = 0;
		end
	end

	-- print('fubin--setNextGuideStep--444 ===end: ',self.curGuideId);
	self.bfind = false;
	if self.curGuideId == 0 then
		-- 引导结束
		self.bStart = false;
		EventManager:FireEvent(EventType.ShieldGuide, false);
		UIManager:pop('GuidePanel');
		
		GlobalFunctions:sendFunnel(Enum:getTable('FunnelType').tu_finish);

		if self.lastStep ~= 0 then
			self.lastStep = 0;
			self.bJustEnd = true;
		end

	else
		self.curGuideData = self:getGuideStepData()
		self.lastStep = self.curGuideId;
	end

end

function GuideModel:getGuideStep(  )
	if self.curGuideId == 0 then
		return -1,-1
	end
	local curGuideData = self:getGuideStepData()
	if curGuideData then
		return curGuideData.no, curGuideData.order;
	end
	return -1,-1;
end

function GuideModel:reqSaveGuideStep( id )
	local msg = {};
	msg.id = id 
	Network.NetworkManager:Send(Network.NetworkCmdType.req_user_save_guide, msg);

end

function GuideModel:startTriggerGuide()
	self.bStart = true;
	self.lastStep = self.curGuideId;
end

function GuideModel:getBJustEnd()
	return self.bJustEnd;
end
function GuideModel:setBJustEnd(v)
	self.bJustEnd = v;
end


return GuideModel;
