local MissionModel = class("MissionModel")
local scheduler = cc.Director:getInstance():getScheduler();

MissionModel.BattlePassMenuType = {
	CAREER 	= 1,
	DAILY 	= 2,
	WEEK 	= 3,
	ULTIMATE= 4,
}

function MissionModel:ctor()
	self:clearAll()
end

function MissionModel:clearAll(  )
	self.hasInit = false;
	self.rewardState = {};
	self.dailyMissionData = {};
	self.rewardData = {};
	self.curBattlePassRewardNo = 0;

	self.lifeMissionData = {};
	self.lifeMissionGoalData = {}

	self.lifeMissionGoalTotalComplateTimes = 0;
end

function MissionModel:isInit()
	return self.hasInit;
end

function MissionModel:Init(data)
	local dailyMission = {};
	if data.daily_mission then
		for _k , mData in pairs(data.daily_mission) do
			if mData then
				for no , data in pairs(mData) do
					dailyMission[tonumber(no)] = data;
				end
			end
		end
	end

	self.dailyMissionData = {};
	local mission = ResTable:getTable('daily_mission').contain;
	for _k , data in pairs(mission) do
		if not self.dailyMissionData[data.time_type] then
			self.dailyMissionData[data.time_type] = {};
		end
		self.dailyMissionData[data.time_type][data.no] = data;
		self.dailyMissionData[data.time_type][data.no].num = 0;
		self.dailyMissionData[data.time_type][data.no].state = 0; --0不能领取，1已领取，3可以领取
		if dailyMission[data.no] then
			self.dailyMissionData[data.time_type][data.no].num = dailyMission[data.no][1];
			self.dailyMissionData[data.time_type][data.no].state = dailyMission[data.no][2];
		end
		local tData = self.dailyMissionData[data.time_type][data.no];
		if tData.state ~= 1 then
			if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
				local curNum = ConvertDigitUnitToStr(tostring(tData.num));
				local comp = HpCompare(curNum , ">=" , tostring(tData.times));
				if comp then
					self.dailyMissionData[tData.time_type][tData.no].state = 2;
				end
			else
				if tData.num >= tData.times then
					self.dailyMissionData[tData.time_type][tData.no].state = 2;
				end
			end
		end
		if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
			local curNum = ConvertDigitUnitToStr(tostring(tData.num));
			self.dailyMissionData[tData.time_type][tData.no].percent = HpDivide(curNum,tData.times);
		else
			self.dailyMissionData[tData.time_type][tData.no].percent = tData.num/tData.times;
		end
	end

	local dailyMissionRewardState = {};
	if data.daily_mission_reward_state then
		for _k , no in pairs(data.daily_mission_reward_state) do
			dailyMissionRewardState[no] = 1;
		end
	end

	self.dailyMissionRewardMaxValue = 0;
	self.dailyMissionReward = {};
	local dailyMissionReward = ResTable:getTable('daily_mission_reward').contain;
	for _k , reward in pairs(dailyMissionReward) do
		self.dailyMissionReward[reward.no] = reward;
		self.dailyMissionReward[reward.no].state = dailyMissionRewardState[reward.no] or 0;
		if self.dailyMissionRewardMaxValue < reward.value then
			self.dailyMissionRewardMaxValue = reward.value;
		end
	end

	self.curDailyMissionItemNum = 0;
	if data.daily_mission_item_num then
		self.curDailyMissionItemNum = data.daily_mission_item_num;
	end

	local lifeMission = {};
	if data.life_mission then
		for _k , lData in pairs(data.life_mission) do
			if lData then
				for no , times in pairs(lData) do
					lifeMission[tonumber(no)] = times;
				end
			end
		end
	end

	local rewardComplate = {}
	if data.reward_complete then
		for _k , rComplate in pairs(data.reward_complete) do
			rewardComplate[tonumber(_k)] = rComplate;
		end
	end


	self.lifeMissionData = {};

	self.lifeMissionGoalTotalComplateTimes = 0;

	local lifemissions = ResTable:getTable('lifemission').contain;
	for _k , data in pairs(lifemissions) do
		if not self.lifeMissionData[data.no] then
			self.lifeMissionData[data.no] = {};
		end
		self.lifeMissionData[data.no] = data;
		self.lifeMissionData[data.no].num = lifeMission[data.no] and lifeMission[data.no] or 0;
		self.lifeMissionData[data.no].complateTimes = rewardComplate[data.no] or 0;

		self:updateLifeMissionDataByNo(data.no);

		self.lifeMissionGoalTotalComplateTimes = self.lifeMissionGoalTotalComplateTimes + self.lifeMissionData[data.no].complateTimes;
	end

	local lifeMissionRewardState = {};
	if data.life_mission_reward_state then
		for _k , no in pairs(data.life_mission_reward_state) do
			lifeMissionRewardState[tonumber(no)] = 1;
		end
	end

	self.lifeMissionGoalData = {}
	local lifemissiongoals = ResTable:getTable('lifemissiongoal').contain;
	for _k , data in pairs(lifemissiongoals) do
		if not self.lifeMissionGoalData[data.no] then
			self.lifeMissionGoalData[data.no] = {};
		end

		self.lifeMissionGoalData[data.no] = data;
		self.lifeMissionGoalData[data.no].state = lifeMissionRewardState[data.no] or 0;
	end

	self.nextDayRefreshTime = TimeManager:getNextDayTime();

	self.hasInit = true;
	
	tools.RedDotTools:pushMainPanelRedDot(12);
	tools.RedDotTools:pushMainPanelRedDot(14);
	EventManager:FireEvent(EventType.UIUpdateBtnShow,14);
	EventManager:FireEvent(EventType.UIUpdateBattlePassRewardLevel);
end

function MissionModel:getNextDayRefreshTime()
	return self.nextDayRefreshTime;
end

function MissionModel:updateLifeMissionDataByNo(no)
	local data = self.lifeMissionData[no];
	local nextComplateTimes = data.complateTimes+1
	if data.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
		local curNum = ConvertDigitUnitToStr(tostring(data.num));
	
		local nextNum = '0';
		if tostring(data.pre_times[1]) ~= '0' then
			local preTimesNum = #data.pre_times;
			if nextComplateTimes <= preTimesNum then
				nextNum = data.pre_times[nextComplateTimes];
			else
				local mulNum = nextComplateTimes - preTimesNum;
				nextNum = HpPlus(tostring(data.pre_times[preTimesNum]),HpMultiply(tostring(data.times) , tostring(mulNum)));
			end
		else
			nextNum = data.times * nextComplateTimes;
		end

		self.lifeMissionData[no].num = curNum;
		self.lifeMissionData[no].nextNum = nextNum;
		self.lifeMissionData[no].state = HpCompare(curNum,">=",nextNum) and 2 or 0;
		self.lifeMissionData[no].percent = HpDivide(curNum,nextNum);
	else
		local nextNum = 0;
		if data.pre_times[1] > 0 then
			local preTimesNum = #data.pre_times;
			if nextComplateTimes <= preTimesNum then
				nextNum = data.pre_times[nextComplateTimes];
			else
				nextNum = data.pre_times[preTimesNum] + (nextComplateTimes - preTimesNum) * data.times;
			end
		else
			nextNum = data.times * nextComplateTimes;
		end

		self.lifeMissionData[data.no].nextNum = nextNum;
		self.lifeMissionData[data.no].state = self.lifeMissionData[data.no].num >= nextNum and 2 or 0;
		self.lifeMissionData[data.no].percent = self.lifeMissionData[data.no].num / nextNum;
	end
end

function MissionModel:getDailyMissionData(timeType , no)
	if self.dailyMissionData[timeType] then
		return self.dailyMissionData[timeType][no];
	end
	return nil;
end

function MissionModel:getDailyMissionReward(no)
	if no then
		return self.dailyMissionReward[no];
	end
	return self.dailyMissionReward;
end

function MissionModel:getDailyMissionRewardMaxValue()
	return self.dailyMissionRewardMaxValue;
end

function MissionModel:getDailyMissionItemNum()
	return self.curDailyMissionItemNum;
end

function MissionModel:updateDailyMissionItem(addDailyMissionNums)
	for _k , num in pairs(addDailyMissionNums) do
		self.curDailyMissionItemNum = self.curDailyMissionItemNum + num;
	end
end

function MissionModel:updateDailyMissionRewardState(no)
	if self.dailyMissionReward[no] then
		self.dailyMissionReward[no].state = 1;
	end
end

function MissionModel:getLifeMissionGoalData(no)
	if no then
		return self.lifeMissionGoalData[no];
	end
	return self.lifeMissionGoalData;
end

function MissionModel:getLifeMissionGoalTotalComplateTimes()
	return self.lifeMissionGoalTotalComplateTimes;
end

function MissionModel:getLifeMissionData(no)
	if no then
		return self.lifeMissionData[no];
	else
		return self.lifeMissionData;
	end
end

function MissionModel:updateLifeMissionGetTimes(no , complateTimes , totalTimes)
	if self.lifeMissionData[no] then
		self.lifeMissionData[no].complateTimes = complateTimes;
		self:updateLifeMissionDataByNo(no);
	end

	if totalTimes then
		self.lifeMissionGoalTotalComplateTimes = totalTimes;
	end
end

function MissionModel:updateLifeMissionGoalState(no)
	if self.lifeMissionGoalData[no] then
		self.lifeMissionGoalData[no].state = 1;
	end
end


function MissionModel:updateLifeMissionData(lifeMissionData)
	for _k , data in pairs(lifeMissionData) do
		local no , num  = data[1] , data[2];
		if self.lifeMissionData[no] then
			self.lifeMissionData[no].num = num or self.lifeMissionData[no].num;
			self:updateLifeMissionDataByNo(no);
		end
	end
end

function MissionModel:getSpecialState()
	return self.specialState;
end

function MissionModel:updateSpecialState(state)
	self.specialState = state;
end

function MissionModel:updateDailyMissionData(dailyMission)
	if dailyMission then
		for _k , data in pairs(dailyMission) do
			if self.dailyMissionData[data[1]] and self.dailyMissionData[data[1]][data[2]] then
				self.dailyMissionData[data[1]][data[2]].num = data[3][1];
				self.dailyMissionData[data[1]][data[2]].state = data[3][2];
				local tData = self.dailyMissionData[data[1]][data[2]];
				if tData.state ~= 1 then
					if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
						local curNum = ConvertDigitUnitToStr(tostring(tData.num));
						local comp = HpCompare(curNum , ">=" , tostring(tData.times));
						if comp then
							self.dailyMissionData[tData.time_type][tData.no].state = 2;
						end
					else
						if tData.num >= tData.times then
							self.dailyMissionData[tData.time_type][tData.no].state = 2;
						end
					end
				end

				if tData.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
					local curNum = ConvertDigitUnitToStr(tostring(tData.num));
					self.dailyMissionData[tData.time_type][tData.no].percent = HpDivide(curNum,tData.times);
				else
					self.dailyMissionData[tData.time_type][tData.no].percent = tData.num/tData.times;
				end
			end
		end
	end
end

function MissionModel:getDailyMissionByTimeType(timeType)
	return self.dailyMissionData[timeType];
end

function MissionModel:updateDailyMissionState(timeType , no ,state)
	if self.dailyMissionData[timeType] and self.dailyMissionData[timeType][no] then
		self.dailyMissionData[timeType][no].state = state;
	end
end

function MissionModel:getActiveState(activeType)
	if activeType == 1 then
		return model.RoundModel:getRoundActiveState()
	elseif activeType == 2 then
		return self:getLifeMissionActiveState();
	elseif activeType == 3 then
		return self:getDailyMissionActiveState();
	else
		if not model.RoundModel:getRoundActiveState() then
			if not self:getLifeMissionActiveState() then
				return self:getDailyMissionActiveState();
			else
				return true
			end
		else
			return true;
		end
	end

	return false;
end

function MissionModel:getLifeMissionActiveState()
	if not tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').MISSION , 2) then
		local locked,lockLv = self:getLockStatMission();
		if locked then
			return false;
		end
		local totalTimes = self:getLifeMissionGoalTotalComplateTimes();
    	for no , data in pairs(self.lifeMissionGoalData) do
			if data.state == 0 and data.times <= totalTimes then
				return true;
			end
		end

		for no , data in pairs(self.lifeMissionData) do
			if data.state == 2 then
				return true
			end
		end
	end

	return false;
end

function MissionModel:getDailyMissionActiveState()
	if not tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').MISSION , 3) then
		local locked,lockLv = model.MissionModel:getLockStatMission();
		if locked then
			return false;
		end

    	for no , data in pairs(self.dailyMissionData[1]) do
			if data.state == 2 then
				return true;
			end
		end

		for no , data in pairs(self.dailyMissionReward) do
			if data.state == 0 and self.curDailyMissionItemNum >= data.value then
				return true
			end
		end
	end

	return false;
end

function MissionModel:getLockStatMission(  )
	return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_19);		
end

return MissionModel;
