local OnlineModel = class("OnlineModel")
function OnlineModel:ctor()
    self:clearAll();
end

function OnlineModel:clearAll()
    self.hasInit = false;

    self.rewardStatus = {}
    self.onlineLoginTimestamp = 0;  -- 上次登录时 服务器今日在线时长
    self.onlineLoginInterval = 0;   -- 上次登录时，服务器时间戳
    self.onlineTotal = 0;         -- 今日之前，已在线时长
    self.onlineRewards = nil ;

    self.nextRewardId = 1; --下一个 未到时间的奖励id

    self.nextUnRewardId = 1; --下一个 未领取的奖励
    ResTable:setIsLoadFalse("online_reward")
end

function OnlineModel:initOnline( online_data,curTime )
    print('online Model init')
    print_ro(online_data)
    print(curTime)
    self:clearAll();
    
    if online_data == nil then
        return;
    end
    self.rewardStatus = online_data.reward or {}; -- 1,2,4 已领取奖励id
    self.onlineLoginTimestamp = online_data.today_online or 0; --今日已在线时长
    self.onlineTotal = online_data.total_online or 0; --已在线时长
    self.onlineLoginInterval = curTime + 5 -- 客户端减慢5秒

    self.hasInit = true ;

  
end

function OnlineModel:hasInitOk(  )
    return self.hasInit;
end

function OnlineModel:getRewardStatus(  )
    return self.rewardStatus;
end

function OnlineModel:getOnlineRewards(  )
    if self.onlineRewards == nil then
        local reward = ResTable:getTable('online_reward').contain;
        self.onlineRewards = GlobalFunctions:sortTableByKey(reward);
        -- dump(self.onlineRewards,"self.onlineRewards====")
        --更新已领取状态
        -- dump(self.rewardStatus,"========rewardStataus")
        for i=1,#self.rewardStatus do
            self.onlineRewards[self.rewardStatus[i]].status = true 
        end
    end

    return self.onlineRewards;
end

function OnlineModel:getNextUnRewardId(  )
    local status = self:getStatusById(self.nextUnRewardId);
    if status == true and self.nextUnRewardId ~= #self.onlineRewards then

        local checkNextUnReward = false;
        for i=self.nextUnRewardId,#self.onlineRewards do
            local item = self.onlineRewards[i];
            --下一个未领取的奖励
            if checkNextUnReward == false and item.status ~= true  then
                checkNextUnReward = true ;
                self.nextUnRewardId = item.id 
            end
        end
    end
    return self.nextUnRewardId;
end

function OnlineModel:getNextId()
    local second = self:getSecondById(self.nextRewardId);
    local secondClient = self:getOnlineSecondTotal()
    if second < secondClient then
        local checkNext = false;
        local nextTime = 0;
        self:getOnlineRewards();
        for i=self.nextRewardId,#self.onlineRewards do
            local item = self.onlineRewards[i];
            --下一个未到时 的奖励
            if checkNext == false and  secondClient < item.time  then
                checkNext = true 
                self.nextRewardId = item.id;
                nextTime = item.time
            end
        end

    end
    return self.nextRewardId;
end

-- 今日一在线时长
function OnlineModel:getOnlineSecond(  )
    -- return self.onlineSecondClient;
    local serverTime = math.floor( TimeManager:getCurTime() )
     --  登录时今日已在线时间 + （ 当前时间 - 登录时间 - 后台无效时间）
     local todayTimes = self.onlineLoginTimestamp + ( serverTime - self.onlineLoginInterval - TimeManager:getBackGroundTime() )
     if todayTimes < 0 then
        todayTimes = 0
    end
    return todayTimes
end
-- 总游戏时长
function OnlineModel:getOnlineSecondTotal(  )
    local onlineToday = self:getOnlineSecond()
    return onlineToday + self.onlineTotal;
end
function OnlineModel:getStatusById( id )
    self:getOnlineRewards();
    local item = self.onlineRewards[id];
    if item == nil then
        print('===error=========not find item in online_reward by id = '..tostring(id));
        return -1;
    end
    return item.status;
end

function OnlineModel:getSecondById( id )
    self:getOnlineRewards();
    local item = self.onlineRewards[id];
    if item == nil then
        print('===error=========not find item in online_reward by id = '..tostring(id));
        return -1;
    end
    return item.time;
end

function OnlineModel:getRewardById( id )
    self:getOnlineRewards();
    local item = self.onlineRewards[id];
    if item == nil then
        print('===error=========not find item in online_reward by id = '..tostring(id));
        return -1;
    end
    return item.reward;
end
--[[
function OnlineModel:reqOnlineInfo(  )
    -- local msg = {}
    -- Network.NetworkManager:Send(Network.NetworkCmdType.req_online_get_info, msg)	        
end
function OnlineModel:onRetOnlineGetInfo( msg )
    -- self.rewardStatus = msg.online_data.reward;
    -- self.onlineLoginTimestamp = msg.online_data.today_time;
    -- self.onlineLoginInterval = TimeManager:getCurTime()
end
]]
function OnlineModel:reqOnlineReward( id )
    local status = self:getStatusById(id)
    if status == -1 then
        return 
    end
    if status == true then
        return 
    end

    local msg = {}
    msg.id = id;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_online_get_reward, msg)	        
end

function OnlineModel:onRetOnlineGetReward( msg )
    -- dump(msg,'OnlineModel=======retGetReward == ');
    -- print()
    local msg_state = msg.state or 0;
    if msg_state == 1 then
        local id = msg.id;
        self.onlineRewards[id].status = true;
    else
        self.nextRewardId = 1;
    end
    self.rewardStatus = msg.onlineReward.reward or self.rewardStatus -- 1,2,4 已领取奖励id


end




return OnlineModel;
