local RankModel = class("RankModel")
local RANK_ENUM = Enum:getTable('RANK_ENUM');

function RankModel:ctor()
    self:clearAll();
end

function RankModel:clearAll()
    self.rankList = {}; --所有类型的 各自类型排行榜
    self.NO1Item = {}; --每一类排行榜的第一名
    self.selfRankItem = {}; --每一类排行中 自己的名次

    self.reqRankLastTime = {}; --每一类排行榜 最后一次请求数据的时间

    self.isLastMonthRank = false; --竞技场 月排行，是否为 上个月排行
    self.rankHash = {}; --每一类排行的哈希值

end

function RankModel:initData(data)
    if data then
        self.NO1Item = data;
    end

end

function RankModel:IsLastMonthRank()
    return self.isLastMonthRank;
end

function RankModel:canRequestRankListByType( rankType )
    if self.reqRankLastTime[ rankType ] then
        local tmpTime = TimeManager:getCurTime() - self.reqRankLastTime[ rankType ];
        local limitTime = 60;
        if rankType == RANK_ENUM.ENUM_1 or rankType == RANK_ENUM.ENUM_3 then
            limitTime = 300;
        end
        if tmpTime > limitTime then
            return true;
        else
            return false;
        end
    else
        return true;
    end
end

function RankModel:reqRankListByType( rankType )
    local msg = {};
    msg.rankType = rankType;
    msg.hash = self.rankHash[rankType] or -1;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_rank_list, msg);
end

function RankModel:retRankList( data )

    if data.selfRank and next(data.selfRank) then
        self.selfRankItem[ data.rankType ] = data.selfRank;
        local tmpName = self.selfRankItem[ data.rankType ].name;
        if tmpName==nil or tmpName=='' then
            self.selfRankItem[ data.rankType ].name = model.UserModel:getUserName();
        end
    end    

    if data.lastMonth ~= nil then
        self.isLastMonthRank = data.lastMonth;
    end

    if data.hash ~= nil then
        self.rankHash[ data.rankType ] = data.hash;
    end

    self.reqRankLastTime[ data.rankType ] = TimeManager:getCurTime();

    if data.rankList and next(data.rankList) then
        if self.rankList[ data.rankType ] == nil then
            self.rankList[ data.rankType ] = data.rankList;
        else
            for k,v in pairs(data.rankList) do
                if v.role == -1 then--某些情况下清空数据
                    self.rankList[ data.rankType ][v.rank] = nil
                else
                    self.rankList[ data.rankType ][v.rank] = v
                end
            end
        end
    end

    if self.rankList[ data.rankType ] and next(self.rankList[ data.rankType ]) then
        self.NO1Item[ data.rankType ] = self.rankList[ data.rankType ][1];

    else
        Prompt:Show(LANG.UI_L_RANK_10);
        self.rankList[ data.rankType ] = {}
    end

    EventManager:FireEvent(EventType.UIRankListView);
    
end

function RankModel:getRankNameByType( rankType )
    if rankType==RANK_ENUM.ENUM_1 then
        return LANG.UI_L_RANK_2;

    elseif rankType==RANK_ENUM.ENUM_2 then
        return LANG.UI_L_RANK_3;

    elseif rankType==RANK_ENUM.ENUM_3 then
        return LANG.UI_L_RANK_4;
    elseif rankType==RANK_ENUM.ENUM_4 then
        return LANG.UI_GUILD_RANK[1];

    end
    return '';
end

function RankModel:getSelfRankByType( rankType )
    return self.selfRankItem[ rankType ];
end

function RankModel:getNo1RankByType( rankType )
    local ret = self.NO1Item[ rankType ];
    if ret == nil then
        ret = self.NO1Item[ ''..rankType ];
    end
    return ret;
end

function RankModel:getRankListByType( rankType )
    return self.rankList[ rankType ];
end

function RankModel:getLockStateRank()
   return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_22);
end

return RankModel
