local RecountModel = class("RecountModel")

function RecountModel:ctor()
	self:clearAll()
end

function RecountModel:clearAll(  )
	self.damagePool = {};
	self.total = {
		damage = 0;
		heal = 0;
	}
	EventManager:UnregisterEvent(EventType.RecountInit)
	EventManager:UnregisterEvent(EventType.RecountAdd)
end

function RecountModel:Init()
	self:clearAll()

	EventManager:RegisterEvent(EventType.RecountInit, handler(self, self.start));
	EventManager:RegisterEvent(EventType.RecountAdd, handler(self, self.add));
end

function RecountModel:reset()
	self.damagePool = {};
	self.total.damage = 0;
	self.total.heal = 0;
end

function RecountModel:start(team)
	self:reset();
	local rank = 1;
	for i=1, TEAM_MAX_MEMBER do
		local roleInfo = {
			damage = {
				value = 0,
				percent = 0,
				rank = rank,
			},
			heal = {
				value = 0,
				percent = 0,
				rank = rank,
			},
			pid = team[i];
		}
		self.damagePool[i] = roleInfo;
		rank = rank + 1;
	end
end

function RecountModel:add(args)
	local roleInfo = self.damagePool[args.pos];
	if not roleInfo then
		return;
	end
	local key, value;
	if args.heal then
		key = 'heal';
		value = args.heal;
	elseif args.damage then
		key = 'damage';
		value = args.damage;
	else
		return;
	end
	--print(key);
	roleInfo[key].value = roleInfo[key].value + value;
	self.total[key] = self.total[key] + value;
	self:calculatePercent();
	self:rank(args.pos, key);
	--print('recount add', json.encode(args))
	EventManager:FireEvent(EventType.RecountUpdate, self.damagePool);
end

function RecountModel:calculatePercent()
	for _, v in pairs(self.damagePool) do
		v.heal.percent = (self.total.heal == 0) and 0 or (math.floor(v.heal.value/self.total.heal*1000)/1000);
		v.damage.percent = (self.total.damage == 0) and 0 or (math.floor(v.damage.value/self.total.damage*1000)/1000);
	end
end

function RecountModel:rank(aimPos, key)
	local aimRole = self.damagePool[aimPos][key];
	local upRank = 0;
	for pos, v in pairs(self.damagePool) do
		if pos ~= aimPos then
			if (v[key].value < aimRole.value) and (v[key].rank < aimRole.rank) then
				v[key].rank = v[key].rank + 1;
				upRank = upRank + 1;
			end
		end
	end
	aimRole.rank = aimRole.rank - upRank;
end

return RecountModel;
