local RoundBossModel = class('RoundBossModel')

local scheduler = cc.Director:getInstance():getScheduler();

function RoundBossModel:ctor(  )
    self:clearAll()
end

function RoundBossModel:clearAll(  )
    self.roundBossList = {} ;

    self.buyCoinMax = 2000              -- 感知度 最大上限
    self.buy_boss_coin = {10007,1800}; -- 购买boss 道具ID， 消耗数量

    self.refresh_ticket_cost = 1        -- 刷新boss 消耗道具数量 固定1
    self.refresh_ticket_id = 15001      -- 刷新boss 道具id 
    self.refresh_boss_coin = { 10002,100} -- 刷新boss 货币ID，货币数量（如果道具不足，则使用货币）
    self.dbTimes = 0;
    self.dbActInfo = nil;
end

--==============================--
--@roundBossList:
-- {
    -- boss_difficulty,
    -- boss_no,
    -- monster_id,
    -- reward,
    -- round_bossinfo_id,

    -- id,
    -- end_times = {end_time1,end_time2}


    -- end_time,
    -- key = (id+ end_time)
-- }
--==============================--
function RoundBossModel:getroundBossList(  )
    return self.roundBossList;
end

function RoundBossModel:hasBoss(  )
    return #self.roundBossList > 0
end

function RoundBossModel:Init( roundBossInfo )
    self:clearAll()
    self.buy_boss_coin = roundBossInfo.buy_roundboss_coin 
    self.refresh_boss_coin = roundBossInfo.reset_roundboss_coin 
    -- self.roundBossList = roundBossInfo.round_boss;
    self:initList(roundBossInfo.newRoundBoss)
end

function RoundBossModel:initList( roundBossList1 )
    
    self.roundBossList = {};
    if roundBossList1 == nil or #roundBossList1 == 0 then
        return 
    end
    for k,roundItem in pairs(roundBossList1) do
        self:pushItem(roundItem)
    end
    self:sortRoundBossList();
end

function RoundBossModel:updateBossItemInfo( bossInfo )
    if bossInfo == nil then
        return
    end
    local item,index = self:getBossDataByPid(bossInfo.pid);
    if item == nil then
        self:pushItem(bossInfo)
    else
        if bossInfo.kill_state == 1 then
            self:removeItem(index)
        else
            self:updateItem(item,bossInfo)
        end
    end
end

function RoundBossModel:getBossDataByPid( pid )
    local len = #self.roundBossList;
    for i=1,len do
        local item = self.roundBossList[i];
        if item.pid == pid then
            return item,i;
        end
    end
    return nil ;
end

function RoundBossModel:getBossDataByKey( key )
    return self.roundBossList[key]
end

function RoundBossModel:pushItem( roundItem )
    if roundItem == nil or roundItem.pid == nil or roundItem.round_bossinfo_id == nil then
        return ;
    end
    local roundData = {};
    roundData['pid']                = roundItem['pid'];
    -- roundData['timeout']            = roundItem['timeout'];
    roundData['kill_state']         = roundItem['kill_state'];
    -- roundData['end_time']           = roundItem['end_time'];
    roundData['round_bossinfo_id']  = roundItem['round_bossinfo_id']; 

    local itemInfo = ResTable:getRowData('round_bossinfo',roundItem['round_bossinfo_id']);
    roundData['boss_no']            = itemInfo['no'];
    roundData['boss_difficulty']    = itemInfo['difficulty'];
    roundData['monster_id']         = itemInfo['monster_id'];
    roundData['reward']             = itemInfo['reward'];
    roundData['boss_icon']          = itemInfo['icon'];
    roundData['fp']                 = itemInfo['fp'];
    roundData['info']               = itemInfo['info'];
    roundData['attribute_id']       = itemInfo['attribute_id'];
    roundData['boss_name'] = ResTable:getValue('monster',itemInfo.monster_id,'name')
    roundData['def_type'] = ResTable:getValue('monster',itemInfo.monster_id,'def_type')

    table.insert( self.roundBossList,roundData );
end

function RoundBossModel:updateItem( itemData,updateData )
    itemData.kill_state = updateData.kill_state;
    if itemData.kill_state == 1  then
        
        return
    end
    if itemData.round_bossinfo_id ~= updateData.round_bossinfo_id then
        local itemInfo = ResTable:getRowData('round_bossinfo',updateData.round_bossinfo_id);
        itemData['boss_no']         = itemInfo['no'];
        itemData['boss_difficulty'] = itemInfo['difficulty'];
        itemData['monster_id']      = itemInfo['monster_id'];
        itemData['reward']          = itemInfo['reward'];
        itemData['boss_icon']       = itemInfo['icon'];
        itemData['fp']              = itemInfo['fp'];
        itemData['info']            = itemInfo['info'];
        itemData['boss_name'] = ResTable:getValue('monster',itemInfo.monster_id,'name')
        roundData['def_type'] = ResTable:getValue('monster',itemInfo.monster_id,'def_type')

    end
end

function RoundBossModel:removeItem( index )
    table.remove( self.roundBossList,index )
end

function RoundBossModel:sortRoundBossList()
    -- body
    table.sort( self.roundBossList,function ( item1,item2 )
        if item1.boss_difficulty ~= item2.boss_difficulty then
            return item1.boss_difficulty > item2.boss_difficulty;
        end

        return item1.fp < item2.fp;
    end )

end
-- ret 当前数量，上限数量
function RoundBossModel:getBuyItemTotal(  )
    local ticketNum = model.ItemModel:getItemNum(self.buy_boss_coin[1])
    return ticketNum,self.buyCoinMax;
end
-- ret 当前拥有，本次消耗，货币类型
function RoundBossModel:getCurrentBuy(  )
    -- body
    local ticketNum = model.ItemModel:getItemNum(self.buy_boss_coin[1])
    return {ticketNum,self.buy_boss_coin[2],self.buy_boss_coin[1]};
end

-- ret 当前拥有，本次消耗，货币类型   
function RoundBossModel:getCurrentRefresh(  )

    --免费
    if self.refresh_boss_coin[1] == 0 then
        return {0,0,0}; 
    end

    local ticketNum = model.ItemModel:getItemNum(self.refresh_ticket_id)
    -- print('ticketNum === '..tostring(ticketNum))
    if ticketNum > 0 then
        return {ticketNum,self.refresh_ticket_cost,self.refresh_ticket_id};
    end

    local moneyHave = 0;
    if self.refresh_boss_coin[1] == MONEY_ID   then
        moneyHave = model.MoneyModel:getMoney()
    elseif self.refresh_boss_coin[1] == COLOR_DIA then
        moneyHave = model.ItemModel:getItemNum(Conf.REDDIAMOND);
    end
    -- print("moneyHave === "..tostring(moneyHave))
    return {moneyHave, self.refresh_boss_coin[2],self.refresh_boss_coin[1]};  
end

function RoundBossModel:getRedDotStateRoundBoss(  )
    if self:getLockStateRoundBoss() then
		return false;
    end
    local ticketNum = model.ItemModel:getItemNum(self.buy_boss_coin[1])
    local buyEnough = tonumber(ticketNum) > tonumber(self.buy_boss_coin[2])

    return buyEnough 
end

function RoundBossModel:getLockStateRoundBoss(  )
    return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_7);		
end

function RoundBossModel:reqRefreshAll( costType,costNum )
    
    local refresh = self:getCurrentRefresh();
    local moneyHave ,moneyEnough = nil,nil;
    if refresh[3] == Conf.GAMEMONEY   then
        moneyHave = model.MoneyModel:getMoney()
        moneyEnough = model.MoneyModel:isEnough(refresh[2])
    else
        moneyHave = model.ItemModel:getItemNum(refresh[3]);
        moneyEnough = moneyHave >= refresh[2]
    end
    if not moneyEnough then
        Prompt:Show(LANG.UI_SHOP_56);
        return 
    end
    local msg = {}
    msg.count = 1;
    msg.num = refresh[2];
    msg.type = refresh[3];
    local	guideStep = model.GuideModel:getGuideStep() ; -- 新手引导阶段
	if guideStep == model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_ROUNDBOSS  then
		msg.guideStep = model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_ROUNDBOSS;
    end
    Network.NetworkManager:Send(Network.NetworkCmdType.req_roundboss_refreshAll, msg)	        

end

function RoundBossModel:retRefreshAll( msg )
    self.refresh_boss_coin = msg.reset_roundboss_coin or {10002,100}
    self:initList(msg.newRoundBoss)

end

function RoundBossModel:reqBuyBoss(  )
    -- if #self.roundBossList > 5 then
    --     Prompt:Show('隐藏BOSS上限已满！')
    --     return
    -- end
    local msg = {}
    msg.count = 1;
    local	guideStep = model.GuideModel:getGuideStep() ; -- 新手引导阶段
	if guideStep == model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_ROUNDBOSS  then
		msg.guideStep = model.GuideModel.GUIDE_STEP_ENUM.GUIDE_ENUM_ROUNDBOSS;
	end
    Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_roundboss, msg)	        
end

function RoundBossModel:retbuyBoss( msg )
    if msg == nil or msg.newRoundBoss == nil then
        return 
    end
    local _k,newItem = next(msg.newRoundBoss)
    self:updateBossItemInfo(newItem);
    self:sortRoundBossList();
end

function RoundBossModel:retFightOver( msg )
    if msg and msg.round_boss_double_reward and msg.round_boss_double_reward.remaining_times then
        self:setDBTimes(msg.round_boss_double_reward.remaining_times)
    end

    if msg == nil or msg.newRoundBoss == nil then
        return 
    end
    local _k,newItem = next(msg.newRoundBoss)
    self:updateBossItemInfo(newItem);

end

function RoundBossModel:getDBTimes()
    return self.dbTimes;
end
function RoundBossModel:setDBTimes( v )
    self.dbTimes = v;
end

function RoundBossModel:reqDoubleActInfo()
    local curTime = TimeManager:getCurTime();
    if self.dbActInfo and next(self.dbActInfo) and curTime>=self.dbActInfo.open_time and curTime<=self.dbActInfo.close_time then
        return true;
    end
    local msg = {}
    msg.type = 16;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_roundboss_dbInfo, msg)  
    return false;
end

function RoundBossModel:retRoundBossDBInfo(msg)
    if msg and msg.round_boss_double_reward and next(msg.round_boss_double_reward) then
        self.dbActInfo = msg.round_boss_double_reward;
        self.dbTimes = self.dbActInfo.remaining_times;
    end
    EventManager:FireEvent('RoundBossPanel:onUpdateDbInfo');
end

function RoundBossModel:getDBActId()
    if self.dbActInfo then
        return self.dbActInfo.operate_id;
    end
    return 0;
end

function RoundBossModel:isValidDBAct()
    local curTime = TimeManager:getCurTime();
    if self.dbActInfo and next(self.dbActInfo) and curTime>=self.dbActInfo.open_time and curTime<=self.dbActInfo.close_time then
        return true;
    end
    return false;
end

return RoundBossModel;


