local RoundModel = class("RoundModel")
function RoundModel:ctor()
	self:clearAll()
end

function RoundModel:clearAll(  )
	self.passData = {};
	self.chapterStar = {};
	self.roundGetRewarded = {};
	self.chapterStarRewardState = {};
	self.allEnd = false;

	self.copyRoundId = 0;
	self.copyRoundPassAll = false;
	self.copyRoundOpenData = nil;
	self.copyRoundWin = false;

	self.roundMisson = {};
end

function RoundModel:Init(round)
	self:clearAll()

	local myRound = round;
	if(round.round_id ~= nil ) then
		myRound = round.round_id;
	end

	self.round_id = myRound;
	self.chapter = 1;
	local data = ResTable:getRowData('round', self.round_id);
	if data then
		self.chapter = data.chapter;
	end
	--self.chapter = tonumber(string.sub(tostring(self.round_id), -4, -3));
	self.checkpoint = tonumber(string.sub(tostring(self.round_id), -2, -1));

	self.roundList = {};
	for _, v in pairs(ResTable:getTable('round').contain) do
		local chapter = tonumber(string.sub(tostring(v['no']), -4, -3));
		local checkpoint = tonumber(string.sub(tostring(v['no']), -2, -1));
		if not self.roundList[chapter] then
			self.roundList[chapter] = {};
		end
		self.roundList[chapter][checkpoint] = {};
		self.roundList[chapter][checkpoint].round_id = v['no'];
		self.roundList[chapter][checkpoint].chapter = chapter;
		self.roundList[chapter][checkpoint].checkpoint = checkpoint;
		self.roundList[chapter][checkpoint].isCurrent = (v['no'] == self.round_id);
		self.roundList[chapter][checkpoint].isComplete = (v['no'] < self.round_id);
	end

	if round.round_reward then
		for _k , getRewardedId in pairs(round.round_reward) do
			self.roundGetRewarded[getRewardedId] = 1;
		end
	end

	self:roundMonsterList()
	self:checkAllEnd();
end

function RoundModel:checkAllEnd()
	local data = ResTable:getRowData('round', self.round_id);
	if data==nil or data.name == '9999' then
		self.allEnd = true;
	else
		self.allEnd = false;
	end
end

function RoundModel:IsAllEnd()
	return self.allEnd;
end

function RoundModel:getInfo(chapter, checkpoint)
	if self.passData[chapter] and self.passData[chapter][checkpoint] then
		return self.passData[chapter][checkpoint];
	end
	return {chapter=1, checkpoint=1, isCurrent=false, isComplete=false};
end

function RoundModel:setNewRound(round_id)
	self.round_id = round_id;
	local data = ResTable:getRowData('round', self.round_id);
	if data then
		self.chapter = data.chapter;
	end
	-- self.chapter = tonumber(string.sub(tostring(self.round_id), -4, -3));
	self.checkpoint = tonumber(string.sub(tostring(self.round_id), -2, -1));
	for chapter, v in pairs(self.passData) do
		for checkpoint, v2 in pairs(v) do
			v2.isCurrent = (v2.round_id == round_id);
			v2.isComplete = (v2.round_id < self.round_id);
		end
	end 	

	self:roundMonsterList()
	DownloadControler:checkNeedMonsterRes( self.monsterList )
	self:checkAllEnd();
end

function RoundModel:getRoundIndex()
	return self.round_id;
end

function RoundModel:getChapter()
	return self.chapter;
end

function RoundModel:getCheckpoint()
	return self.checkpoint;
end

function RoundModel:getRoundRewarded(id)
	return self.roundGetRewarded[id];
end

function RoundModel:updateRoundReward(ids)
	if ids and #ids > 0 then
		for _k , id in pairs(ids) do
			self.roundGetRewarded[id] = 1;
		end
	end
end

-- 每5关奖励
function RoundModel:reqRoundBreakReward( curRound )
	local msg = {}
	msg.round_id = curRound or self.round_id - 1
	Network.NetworkManager:Send(Network.NetworkCmdType.req_round_break_reward, msg);

end

function RoundModel:getRoundActiveState()
	if not tools.ConfigStateTools:checkLockByType(Enum:getTable('ConfigStateType').MISSION , 1) then
        for _k , reward in pairs(ResTable:getTable('round_reward').contain) do
			if not self:getRoundRewarded(reward.id) and self.round_id > reward.round_id then
				return true;
			end
		end
	end
	return false;
end

function RoundModel:getChapterData(chapter , point)
	if chapter then
		if self.roundList[chapter] and point then
			return self.roundList[chapter][point];
		else
			return self.roundList[chapter];
		end
	end
	return nil;
end

-- bool,lockLv
function RoundModel:getLockStateRound(  )
	local roundIndex = self:getRoundIndex()
	local roundLimit = ResTable:getValue('round', roundIndex , 'level');
	local userLv = model.UserModel:getUserLevel();
	print(userLv, roundLimit)
	return userLv < roundLimit,roundLimit
end

function RoundModel:getRedDotStateRoundReward( ... )
	if self:getLockStateRoundReward() then
		return false;
	end
    print("未完成！！！！！！！！！！！！！")

	return  true ;
end

function RoundModel:getRoundMonsterList()
	return self.monsterList
end

function RoundModel:roundMonsterList()
	-- body
	self.monsterList = {}
	local roundData = ResTable:getRowData('round', self.round_id);
	if roundData == nil then
		return
	end
	for i,bossInfo in ipairs(roundData.boss) do
		local isHave = false
		for j,v in ipairs(self.monsterList) do
			if v == bossInfo[1] then
				isHave = true
				break;
			end
		end
		if isHave == false then
			table.insert(self.monsterList, bossInfo[1]);
		end
		
	end

	for i,bossInfo in ipairs(roundData.monster) do
		local isHave = false
		for j,v in ipairs(self.monsterList) do
			if v == bossInfo[1] then
				isHave = true
				break;
			end
		end
		if isHave == false then
			table.insert(self.monsterList, bossInfo[1]);
		end
		
	end

end

-- bool,lockLv
function RoundModel:getLockStateRoundReward(  )
    return tools.LevelLockTools:checkLockById(1000);		
end

function RoundModel:updateCopyRoundInfo( msg )
	if msg then
		self.copyRoundId = msg.next_round_no;
		self.copyRoundPassAll = msg.is_pass_all;
	end
end
function RoundModel:updateCopyRoundTime( msg )
	self.copyRoundOpenData = msg;
end
function RoundModel:isCopyRoundPassAll()
	return self.copyRoundPassAll;
end

function RoundModel:getCurCopyRoundId()
	return self.copyRoundId;
end

function RoundModel:isOpenCopyRound()
	if self.copyRoundId <= 0 then
		return false;
	end
	if self.copyRoundOpenData == nil then
		return false;
	end
    local curTime = TimeManager:getCurTime();
    if curTime>=self.copyRoundOpenData.open_time and curTime<=self.copyRoundOpenData.show_time then
    	return true;
    else
        return false;
    end
end

function RoundModel:getCopyRoundOpenInfo()
	return self.copyRoundOpenData;
end

function RoundModel:isCopyRoundNewWin()
	return self.copyRoundWin;
end
function RoundModel:setCopyRoundNewWin( v )
	self.copyRoundWin = v;
end


function RoundModel:initRoundMission( bank_missions )
	self.roundMisson = {};
	if bank_missions then
		for k,v in pairs(bank_missions) do
			self.roundMisson[ v.season_id ] = v;
		end
	end
end

function RoundModel:getRoundMissionById( id )
	if self.roundMisson[id] then
		return self.roundMisson[id];
	end
	return {}
end

function RoundModel:getRoundMissionTaskById(sid, taskno)
	if self.roundMisson[sid] then
		for k,v in pairs( self.roundMisson[sid].round_bank_missions ) do
			if v.no == taskno then
				return v;
			end
		end
	end
	return nil;
end

function RoundModel:updateRoundMissionState( bank_missions )
	for k,v in pairs(bank_missions) do
		if self.roundMisson[ v.season_id ] then
			for _,task in pairs(v.round_bank_missions) do
				for mk,mv in pairs( self.roundMisson[ v.season_id ].round_bank_missions ) do
					if mv.no == task.no then
						if task.status_1 then
							mv.status_1 = task.status_1;
						end
						if task.status_2 then
							mv.status_2 = task.status_2;
						end
						if task.progress then
							mv.progress = task.progress;
						end
						break;
					end
				end
			end
		end
	end
end

function RoundModel:retBuyRoundMission( recharge_season_id )
	-- body
	if self.roundMisson[recharge_season_id] then
		self.roundMisson[recharge_season_id].is_recharge = 1;
		EventManager:FireEvent('BankTaskItem:onRechargeUpdate', 1);
	end
end
function RoundModel:retGetRoundMissionRwd( data )
	-- self:updateRoundMissionState(data)
	UIManager:push('RewardPanel',data)

	EventManager:FireEvent('BankTaskItem:onRwdUpdate');
end

return RoundModel;
