local ShopModel = class("ShopModel")
local CommonShopType = Enum:getTable('CommonShopType');
--[[
local tableList = 
{
	[CommonShopType.SHOP] = 'shop',
	[CommonShopType.GUILD] = 'guild_rewardshop',
	[CommonShopType.RECHARGE] = 'recharge',
	[CommonShopType.GIFT] = 'shop_gift',
}
]]
function ShopModel:ctor()
	self:clearAll()
end

function ShopModel:clearAll( ... )
	self.shopData = {};
	self.randomShopInfo = nil;
	self.eventShopData = {};
	self.shopInit = {};
	self.eventShopInit = false;
	self.shopRedDot = {};
	self.shopGiftList = {};
	self.fightGiftList = {};
end

function ShopModel:shopHasInit(index)
	return self.shopInit[index];
end

function ShopModel:Init(shop)

	if not shop then
		return;
	end

	if shop.shop then
		self.shopData[CommonShopType.SHOP] = {};
		for _k , v in ipairs(shop.shop) do
			self.shopData[CommonShopType.SHOP][v.no] = v;
		end
		self.shopInit[CommonShopType.SHOP] = true;
	end

	if shop.gift then
		self.shopRedDot[CommonShopType.GIFT] = false;
		self.shopData[CommonShopType.GIFT] = {};
		for _k , v in pairs(shop.gift) do
			-- if v.time_stamp then
			-- 	v.days = math.ceil(v.time_stamp / 86400);
			-- end
			if v.price then
				v.price = tonumber(v.price)
			end
			self.shopData[CommonShopType.GIFT][v.no] = v;
			if v.price == 0 and v.times ~= 0 then
				self.shopRedDot[CommonShopType.GIFT] = true
			end
		end
		self.shopInit[CommonShopType.GIFT] = true;
	end

	if shop.activity_gift then
		self.shopData[CommonShopType.GIFT_ACTIVITY] = {};
		for _k , v in pairs(shop.activity_gift) do
			if v.price then
				v.price = tonumber(v.price)
			end
			v.days = math.ceil(v.time_stamp / 86400);
			self.shopData[CommonShopType.GIFT_ACTIVITY][v.no] = v;
		end

		self.shopInit[CommonShopType.GIFT_ACTIVITY] = true;
	end

	if shop.recharge then
		if not self.shopData[CommonShopType.RECHARGE] then
			self.shopData[CommonShopType.RECHARGE] = {};
		end
		self.shopData[CommonShopType.RECHARGE] = {};
		for _k , v in ipairs(shop.recharge) do
			if v.price then
				v.price = tonumber(v.price)
			end
			self.shopData[CommonShopType.RECHARGE][v.no] = v;
		end
		self.shopInit[CommonShopType.RECHARGE] = true;
	end

	--if shop.recharge_online then
	--	if not self.shopData[CommonShopType.RECHARGE] then
	--		self.shopData[CommonShopType.RECHARGE] = {};
	--	end
	--	self.shopData[CommonShopType.RECHARGE][2] = {};
	--	for _k , v in ipairs(shop.recharge_online) do
	--		self.shopData[CommonShopType.RECHARGE][2][v.no] = v;
	--	end
	--	self.shopInit[CommonShopType.RECHARGE] = true;
	--end

	if shop.month then
		if not self.shopData[CommonShopType.MONTH_CARD] then
			self.shopData[CommonShopType.MONTH_CARD] = {};
		end
		for _k , v in ipairs(shop.month) do
			if v.price then
				v.price = tonumber(v.price)
			end
			self.shopData[CommonShopType.MONTH_CARD][v.no] = v;
		end
		self.shopInit[CommonShopType.MONTH_CARD] = true;
	end

	if shop.guild_items then
		self.shopData[CommonShopType.GUILD] = {};
		for _k , v in ipairs(shop.guild_items) do
			self.shopData[CommonShopType.GUILD][v.no] = v;
		end
		self.shopInit[CommonShopType.GUILD] = true;
	end

	if shop.resolve_items then
		self.shopData[CommonShopType.RESOLVE] = {};
		for _k , v in ipairs(shop.resolve_items) do
			self.shopData[CommonShopType.RESOLVE][v.no] = v;
		end
		self.shopInit[CommonShopType.RESOLVE] = true;
	end
	if shop.role_items then
		self.shopData[CommonShopType.ROLE] = {};
		for _k , v in ipairs(shop.role_items) do
			self.shopData[CommonShopType.ROLE][v.no] = v;
		end
		self.shopInit[CommonShopType.ROLE] = true;
	end
	if shop.equip_items then
		self.shopData[CommonShopType.EQUIP] = {};
		for _k , v in ipairs(shop.equip_items) do
			self.shopData[CommonShopType.EQUIP][v.no] = v;
		end
		self.shopInit[CommonShopType.EQUIP] = true;
	end

	if shop.avatar_items then
		self.shopData[CommonShopType.AVATAR] = {};
		for _k , v in ipairs(shop.avatar_items) do
			self.shopData[CommonShopType.AVATAR][v.no] = v;
		end
		self.shopInit[CommonShopType.AVATAR] = true;
	end

	if shop.maze_items then
		self.shopData[CommonShopType.MAZE] = {};
		for _k , v in ipairs(shop.maze_items) do
			self.shopData[CommonShopType.MAZE][v.no] = v;
		end
		self.shopInit[CommonShopType.MAZE] = true;
	end

	if shop.battlepass1_items then
		self.shopData[CommonShopType.BATTLEPASS_1] = {};
		for _k , v in ipairs(shop.battlepass1_items) do
			self.shopData[CommonShopType.BATTLEPASS_1][v.no] = v;
		end
		self.shopInit[CommonShopType.BATTLEPASS_1] = true;
	end

	if shop.battlepass2_items then
		self.shopData[CommonShopType.BATTLEPASS_2] = {};
		for _k , v in ipairs(shop.battlepass2_items) do
			self.shopData[CommonShopType.BATTLEPASS_2][v.no] = v;
		end
		self.shopInit[CommonShopType.BATTLEPASS_2] = true;
	end

	if shop.arena_items then
		self.shopData[CommonShopType.ARENA] = {};
		for _k , v in ipairs(shop.arena_items) do
			self.shopData[CommonShopType.ARENA][v.no] = v;
		end
		self.shopInit[CommonShopType.ARENA] = true;
	end

	if shop.multiple_arena_items then
		self.shopData[CommonShopType.MULT_ARENA] = {};
		for _k , v in ipairs(shop.multiple_arena_items) do
			self.shopData[CommonShopType.MULT_ARENA][v.no] = v;
		end
		self.shopInit[CommonShopType.MULT_ARENA] = true;
	end

	if shop.adventure1_items then
		self.shopData[CommonShopType.ADVENTURE1] = {};
		for _k , v in ipairs(shop.adventure1_items) do
			self.shopData[CommonShopType.ADVENTURE1][v.no] = v;
		end
		self.shopInit[CommonShopType.ADVENTURE1] = true;
	end

	if shop.adventure2_items then
		self.shopData[CommonShopType.ADVENTURE2] = {};
		for _k , v in ipairs(shop.adventure2_items) do
			self.shopData[CommonShopType.ADVENTURE2][v.no] = v;
		end
		self.shopInit[CommonShopType.ADVENTURE2] = true;
	end

	if shop.special_items then
		self.shopData[CommonShopType.SPECIAL] = {};
		for _k , v in ipairs(shop.special_items) do
			self.shopData[CommonShopType.SPECIAL][v.no] = v;
		end
		self.shopInit[CommonShopType.SPECIAL] = true;
	end

	if shop.bubble_items then
		self.shopData[CommonShopType.BUBBLE] = {};
		for _k , v in ipairs(shop.bubble_items) do
			self.shopData[CommonShopType.BUBBLE][v.no] = v;
		end
		self.shopInit[CommonShopType.BUBBLE] = true;
	end

	if shop.operates then
		model.OperateModel:updateData(shop.operates)
	end
end

function ShopModel:resetShopInit()
	self.shopInit = {};
end

function ShopModel:reqShopInfo( index )
	local msg = {}
	msg.index = index or CommonShopType.SHOP
	msg.lan = model.UserDefaultModel:getLanguageSet()
	Network.NetworkManager:Send(Network.NetworkCmdType.req_shop_info , msg);
end

function ShopModel:sendBuyShopItem(id , count,price)
	local data = model.ShopModel:getShopDataByIndex(CommonShopType.SHOP)[id];
	if data and data.times == 0 then
		Prompt:Show(LANG.UI_SHOP_95)
		return;
	end
	local msg = {
		id = id,
		count = count,
		price = price 
	}
	Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_shop_item, msg)	        
end

function ShopModel:sendBuyGift(id,shopType)
	local data = model.ShopModel:getShopDataByIndex(shopType)[id];
	if data and data.times == 0 then
		Prompt:Show(LANG.UI_SHOP_95)
		return;
	end

	local msg = {
		id = id,
		shopType = shopType 
	}
	Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_gift,msg);
end

function ShopModel:reqStoreBuy(storeType, id , count , price)
	if storeType == CommonShopType.CLIENT then
		local msg = {};
		msg.storeType = storeType;
	    msg.item_id = id;
	    msg.count = count;
	    msg.price = price
	    Network.NetworkManager:Send(Network.NetworkCmdType.req_play_ball_exchange , msg);
		return;
	end
	local data = model.ShopModel:getShopDataByIndex(storeType)[id];
	if data and data.times == 0 then
		Prompt:Show(LANG.UI_SHOP_95)
		return;
	end
	local msg = {};
	msg.storeType = storeType;
    msg.item_id = id;
    msg.count = count;
    msg.price = price
    Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_store_item , msg);
end

function ShopModel:reqBattlepassBuy(shopType , id,count)
	local data = model.ShopModel:getShopDataByIndex(shopType)[id];
	if data and data.times == 0 then
		Prompt:Show(LANG.UI_SHOP_95)
		return;
	end
	local msg = {};
	msg.shop_type = shopType;
    msg.item_id = id;
    msg.count = count;
    Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_battlepass , msg);
end

function ShopModel:reqBuyAvatar(id)
	local msg = {
		id = id
	}
	Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_avatar,msg);
end

function ShopModel:getSortShopData(index)

	if not self.shopData[index] then
		return nil;
	end

	local sortTable = {}
	for _k , value in pairs(self.shopData[index]) do
		table.insert(sortTable,value);
	end
	if #sortTable > 1 then
		table.sort(sortTable,function (v_1 , v_2)
			local sData_1 = ResTable:getRowData(tableList[index],v_1.no)
			local sData_2 = ResTable:getRowData(tableList[index],v_2.no)
			return (sData_1.order or v_1.no) < (sData_2.order or v_2.no)
		end);
	end

	return sortTable;
end
function ShopModel:getShopDataByIndex(index , _type)
	if index then
		if _type then
			return self.shopData[index][_type];
		else
			return self.shopData[index];
		end
	end
	return nil;
end

function ShopModel:retBuyShopItem(msg)
	if not msg then
		return;
	end

	if msg.shop then
		-- 商店数据刷新
		self:Init(msg.shop);
		return ;
	end

	if not self.shopData[CommonShopType.SHOP] or not self.shopData[CommonShopType.SHOP][msg.id] then
		return
	end
	local shopData = self.shopData[CommonShopType.SHOP][msg.id];
	if shopData and shopData.times > 0 then
		self.shopData[CommonShopType.SHOP][msg.id].times = msg.rest_times;
	end
end

function ShopModel:retStoreBuy(msg)
	if not msg then
		return;
	end
	if msg.shop then
		-- 商店数据刷新
		self:Init(msg.shop);
		return ;
	end
	if not self.shopData[msg.storeType] or not self.shopData[msg.storeType][msg.item_id] then
		return
	end
	local shopData = self.shopData[msg.storeType][msg.item_id];
	if shopData and shopData.times > 0 then
		self.shopData[msg.storeType][msg.item_id].times = msg.rest_times;
	end
end

function ShopModel:retBuyGift(msg)
	if not msg or not msg.id then
		return;
	end

	if not self.shopData[CommonShopType.GIFT] or not self.shopData[CommonShopType.GIFT][msg.id] then
		return;
	end

	if msg.times and self.shopData[CommonShopType.GIFT][msg.id].times ~= -1 then
		self.shopData[CommonShopType.GIFT][msg.id].times = msg.times;
		if self.shopData[CommonShopType.GIFT][msg.id].price == 0 
		  and self.shopData[CommonShopType.GIFT][msg.id].times == 0 then
			self.shopRedDot[CommonShopType.GIFT] = false;
		end
	end
end

function ShopModel:retAvatarBuy(msg)
	if self.shopData[CommonShopType.AVATAR] and 
		self.shopData[CommonShopType.AVATAR][msg.id] and
		self.shopData[CommonShopType.AVATAR][msg.id].times>0 then
		self.shopData[CommonShopType.AVATAR][msg.id].times = msg.times;
	end
end

function ShopModel:retBattlepassBuy(msg)
	if self.shopData[msg.shop_type] then
		if msg.times ~= -1 then
			self.shopData[msg.shop_type][msg.id].times = msg.times;
		end
	end
end

function ShopModel:changeFstBuyState(recharge_state)
	for k,v in pairs(recharge_state) do
		if self.shopData[CommonShopType.RECHARGE] and self.shopData[CommonShopType.RECHARGE][v] then
			self.shopData[CommonShopType.RECHARGE][v].is_recharged = 1
		end
	end
end

------- recahrge_online ------
-- rechage_online 次数变更
function ShopModel:retRechageOnlineTimes( rechageOnline )
	if not self.shopData[CommonShopType.RECHARGE] then
		return
	end
	if not self.shopData[CommonShopType.RECHARGE][2] then
		return
	end
	for _k , v in pairs(rechageOnline) do
		self.shopData[CommonShopType.RECHARGE][2][tonumber(_k)].times = self.shopData[CommonShopType.RECHARGE][2][tonumber(_k)].total_times - v;
	end
end

function ShopModel:setShopRedDot(shopType,state)
	self.shopRedDot[shopType] = state == 1;
end

function ShopModel:getShopActiveState(shopType)
	if shopType == nil then
		if self.shopRedDot[CommonShopType.GIFT] then
			return true
		-- elseif model.RechargeModel:getGrowupRedDot() then
		-- 	return true;
		end
		return model.RechargeModel:getMonthRedDot()
	elseif shopType == CommonShopType.GIFT then
		return self.shopRedDot[CommonShopType.GIFT];
	elseif shopType == CommonShopType.GROWUP then
		return model.RechargeModel:getGrowupRedDot()
	elseif shopType == CommonShopType.MONTH_CARD then
		return model.RechargeModel:getMonthRedDot()
	end
end

-------- random shop ----------
function ShopModel:reqRandomShopInfo(  )
	--Network.NetworkManager:Send(Network.NetworkCmdType.req_random_shop_info, {})	        
end

function ShopModel:retRandomShopInfo( msg )
	-- self.randomShopInfo = {}
	if msg == nil then
		return
	end
	self.randomShopInfo = {};
	self:updateRandomShopInfo(msg)
end

function ShopModel:reqRandomShopRefresh(  )
	local msg = {}
	msg.count = 0;
	if self.randomShopInfo.free_num < 1 then
		msg.count = self.randomShopInfo.nextResetCoin[2]
	end
	--Network.NetworkManager:Send(Network.NetworkCmdType.req_random_shop_refresh,msg)	        
end


function ShopModel:retRandomShopRefresh( msg )
	self.randomShopInfo = {};
	self:updateRandomShopInfo(msg)
end

function ShopModel:reqRandomShopBuyItem( no,discount )
	local msg = {}
	msg.no = no;
	msg.discount = discount;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_random_shop_item, msg)	        
end


function ShopModel:retRandomShopBuy( msg )
	local no = msg.no
	local item = self:getRandomShopItemByNo(no)
	if item then
		item.buy = item.buy - 1;
	end
	local costName = ResTable:getValue('item' , item.price[1] , 'name');
	local msg = string.format('消耗 %s %s 获得[%s]' , tostring(math.ceil( item.price[2]*item.discount/100 )), tostring(costName),tostring(item.name))
	Prompt:Show(msg)
end



function ShopModel:hasRandomShopInit(  )

	return self.randomShopInfo ~= nil and self.randomShopInfo.hasInitList == true ;
end

function ShopModel:updateRandomShopInfo( data )
	self.randomShopInfo.cout_donw_timestamp_end = data.max_time; -- 倒计时结束时间戳
	self.randomShopInfo.free_num = data.free_num;
	self.randomShopInfo.reset_num = data.reset_num;
	self.randomShopInfo.nextResetCoin = data.nextResetCoin;
	self.randomShopInfo.item_list = data.item_list;

	self.randomShopInfo.free_num_max = 5;
	self.randomShopInfo.count_down_interval =  60*60*2 
	self.randomShopInfo.count_down_timestamp_now =  data.currentTime or math.floor( TimeManager:getCurTime() )
	self.randomShopInfo.cout_donw_timestamp_end = self.randomShopInfo.cout_donw_timestamp_end - self.randomShopInfo.count_down_timestamp_now + math.floor( TimeManager:getCurTime() ) 

	self.randomShopInfo.hasInitList = false
	self:updateRandomShopList();
end
--[[
	item_list = {
		no,discount,buy,   服务器数据(no,折扣（0-100），可购买次数)
		name,item,price		本地数据（名字，购买项内容（{10007，10}），价格（{货币类型，货币数量}） ）
	}

]]
function ShopModel:updateRandomShopList(  )
	if self.randomShopInfo == nil then
		return 
	end
	if self.randomShopInfo.hasInitList ~= true and self.randomShopInfo.item_list ~= nil then
		for i=1,#self.randomShopInfo.item_list do
			local item = self.randomShopInfo.item_list[i]

			local itemData  = ResTable:getRowData('shop_renovate',item.no);
			self.randomShopInfo.item_list[i].name = itemData.name;
			self.randomShopInfo.item_list[i].item = itemData.item;
			self.randomShopInfo.item_list[i].price = itemData.price;
			-- self.randomShopInfo.item_list[i].sign = itemData.sign;
			
		end

		self.randomShopInfo.hasInitList = true 
	end
end

function ShopModel:getRandomShopList(  )
	if self.randomShopInfo == nil or self.randomShopInfo.item_list == nil then
		return nil
	end
	return self.randomShopInfo.item_list
end

function ShopModel:getRandomShopItemByIndex( index )
	if self.randomShopInfo == nil or self.randomShopInfo.item_list == nil then
		return nil
	end
	return self.randomShopInfo.item_list[index];
end

function ShopModel:getRandomShopItemByNo( no )
	if self.randomShopInfo == nil or self.randomShopInfo.item_list == nil then
		return nil
	end
	for i=1,#self.randomShopInfo.item_list do
		if self.randomShopInfo.item_list[i].no == no then
			return self.randomShopInfo.item_list[i]
		end
	end
end

function ShopModel:getRandomShopCountDownInterVal(  )
	return self.randomShopInfo.count_down_interval;
end
-- 剩余免费次数
-- 如果不足5，则计算当前倒计时
function ShopModel:getRandomShopFreeNum( fix )
	local fixCount = 0
	if fix == true then
		fixCount = 1;
	end
	if self.randomShopInfo.free_num < self.randomShopInfo.free_num_max then

		local curCountDown = self:getRandomShopCountDown()
		if curCountDown <= 0 then
			self.randomShopInfo.free_num = self.randomShopInfo.free_num_max
		else
			local freeNum2 = math.ceil( (curCountDown - fixCount )/ self.randomShopInfo.count_down_interval)
			self.randomShopInfo.free_num = self.randomShopInfo.free_num_max - freeNum2 
		end

	end
	return self.randomShopInfo.free_num;
end
-- {货币类型，货币数量}
function ShopModel:getRandomShopNextCoin(  )
	return self.randomShopInfo.nextResetCoin;
end

function ShopModel:getRandomShopCountDown(  )
	if self.randomShopInfo.cout_donw_timestamp_end == 0 then
		return 0;
	end
    local serverTime = math.floor( TimeManager:getCurTime() )
	return self.randomShopInfo.cout_donw_timestamp_end - serverTime-- (serverTime - self.randomShopInfo.count_down_timestamp)


end

function ShopModel:getRedDotStateRandomShop(  )
	if self:getLockStateRandomShop() then
		return false;
	end
	if self.randomShopInfo == nil then
		self:reqRandomShopInfo()
		return false;
	end
	return self:getRandomShopFreeNum() >= self.randomShopInfo.free_num_max 
end
	
function ShopModel:getLockStateRandomShop(  )
    return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_1);		
end

function ShopModel:getLimitBuyInfo( ltype, val )
	if ltype == 1 then
		return string.format(LANG.UI_SHOP_91,val)
	elseif ltype == 2 then
		return string.format(LANG.UI_SHOP_92,val)
	elseif ltype == 3 then
		return string.format(LANG.UI_SHOP_93,val)
	else
		return string.format(LANG.UI_SHOP_94,val)
	end
end

function ShopModel:reqShopGiftRecommend( opt )
	local msg = {}
	msg.opt_type = opt;
	msg.lan = model.UserDefaultModel:getLanguageSet()
	Network.NetworkManager:Send(Network.NetworkCmdType.req_shop_gift_rec , msg);
end

function ShopModel:retShopGiftData(msg)
	if not msg.novice_gift or not next(msg.novice_gift) then
		return;
	end
	table.insert(self.shopGiftList, msg.novice_gift)
end

function ShopModel:insertShopGiftData(msg)
	local locked = tools.ConfigStateTools:checkLockById(1109);
	if locked == true then
		return;
	end
	local isEx = false;
	for k,v in pairs(self.fightGiftList) do
		if v.gift_data.no == msg.gift_data.no then
			isEx = true;
			break;
		end
	end
	if isEx == false then
		table.insert(self.fightGiftList, clone(msg))
	end
end
function ShopModel:getFightGiftList()
	return self.fightGiftList;
end
function ShopModel:showFightGift(idx)
	local showIdx = idx+1;
	if showIdx > #self.fightGiftList then
		return;
	end
	UIManager:push('SpecialOfferPanel', true, {}, showIdx);
end
function ShopModel:removeFightGift(no)
	for k,v in pairs(self.fightGiftList) do
		if v.gift_data and v.gift_data.no == no then
			table.remove(self.fightGiftList, k)
			return;
		end
	end
end

function ShopModel:getShopGiftData( tp )
	for k,v in pairs(self.shopGiftList) do
		if v.type == tp then
			return v;
		end
	end
	return nil;
end

function ShopModel:removeShopGiftData(no)
	for k,v in pairs(self.shopGiftList) do
		if v.no == no then
			table.remove(self.shopGiftList, k)
			return;
		end
	end
end

return ShopModel
