local SkillLevelUpModel = class("SkillLevelUpModel")


function SkillLevelUpModel:ctor()
	self:clearAll()
end

function SkillLevelUpModel:clearAll( ... )
	self.slotData = {};
end

function SkillLevelUpModel:resetSlotData(slot)
	self.slotData[slot] = {};
	self.slotData[slot].pid 			= -1;
	self.slotData[slot].slotId 			= slot;
	self.slotData[slot].skillId 		= -1;
	self.slotData[slot].skillResid 		= -1;
	self.slotData[slot].skillLevel 		= -1;
	self.slotData[slot].skillExp 		= -1;
	self.slotData[slot].skillState 		= -1;
	self.slotData[slot].skillLastTime 	= -1;
	self.slotData[slot].consumeItme 	= -1;
	self.slotData[slot].isSendFinsh 	= false;
end
function SkillLevelUpModel:initSlotData(msg)
	if not msg then
		return;
	end
	for i = 1 , 2 do
		self:resetSlotData(i);
	end

	if not msg.skill_lvlup_info then
		return
	end
	for _k , data in pairs(msg.skill_lvlup_info) do
		self.slotData[data.slot_pos] = {};
		self.slotData[data.slot_pos].pid 			= data.pid;
		self.slotData[data.slot_pos].slotId 		= data.slot_pos;
		self.slotData[data.slot_pos].skillId 		= data.skill_id;
		self.slotData[data.slot_pos].skillResid 	= data.skill_resid;
		self.slotData[data.slot_pos].skillLevel 	= data.skill_lv;
		self.slotData[data.slot_pos].skillExp 		= data.skill_exp;
		self.slotData[data.slot_pos].skillState 	= data.skill_state;
		self.slotData[data.slot_pos].skillLastTime 	= data.skill_learn_time
		self.slotData[data.slot_pos].consumeItme 	= data.consume_time;
		self.slotData[data.slot_pos].isSendFinsh 	= false;
	end
	--print_ro(msg.skill_lvlup_info);
end
function SkillLevelUpModel:updateTime()
	for _k , slot in pairs(self.slotData) do
		if slot.pid ~= -1 then
			slot.skillLastTime = (slot.skillLastTime - 1) > 0 and (slot.skillLastTime - 1) or 0;
		end
	end
end
function SkillLevelUpModel:getLevelUpState()

	local emptyCount = 0;
	local isFinish = false;
	for _k , slot in pairs(self.slotData) do
		if slot.pid == -1 then
			emptyCount = emptyCount + 1;
		end
		if slot.skillState == 1 and slot.skillLastTime == 0 then
			isFinish = true;
		end
	end

	if isFinish then
		return 1;
	end

	if emptyCount == 2 then
		return 2;
	end

	if emptyCount == 0 then
		return 3;
	end

	return 0;
end
function SkillLevelUpModel:getAllData()
	return self.slotData;
end
function SkillLevelUpModel:getDataBySlot(slot)
	return self.slotData[slot];
end
function SkillLevelUpModel:getSkillInfoByPid(pid_)
	local card = model.CardModel:GetCard(pid_);
	local skill_list = card.skill_list;
	local infos = {};
	for _k,skill in pairs(skill_list) do
		local max_lv = self:getSkillMaxLevel(skill.resid)
		if max_lv ~= -1 and skill.level < max_lv then
			local skl 		= {};
			skl.sid 	= skill.sid;
			skl.resid 	= skill.resid;
			skl.level 	= skill.level;
			skl.exp 	= skill.exp;
			table.insert(infos , skl);
		end
	end
	table.sort(infos, function(skillA, skillB)
		return tonumber(skillA.resid) > tonumber(skillB.resid)
	end)
	return infos;
end
function SkillLevelUpModel:getSkillMaxLevel(skill_resid)
	local maxLevel = 0;
	local skillType = ResTable:getValue('skill', skill_resid , 'skill_type');
	local skillLevelTableData = ResTable:getTable('skill_lv').contain;
	for _k , skillLevelData in pairs(skillLevelTableData) do
		if skillLevelData.type == skillType then
			if maxLevel < skillLevelData.lv then
				maxLevel = skillLevelData.lv;
			end
		end
	end

	return maxLevel;
end
--角色
function SkillLevelUpModel:isInSkillLevelUp(pid)
	for _k , data in pairs(self.slotData) do
		if data.pid == pid then
			return true;
		end
	end
	return false;
end
--技能
function SkillLevelUpModel:isInLevelUp(skillId)
	for _k , data in pairs(self.slotData) do
		if data.skillId == skillId then
			return true;
		end
	end
	return false;
end
function SkillLevelUpModel:getSpeedCostMoney(slot)
	local data = self.slotData[slot];
	if data.pid ~= -1 then
		return math.ceil(data.skillLastTime/600) * 5;
	end

	return -1;
end
function SkillLevelUpModel:getEmptySlot()
	for _k , data in pairs(self.slotData) do
		if data.pid == -1 then
			return _k;
		end
	end
	return -1;
end



function SkillLevelUpModel:onRetStartLearn(msg)
	if not msg then
		return;
	end
	self.slotData[msg.slot_pos] = {};
	self.slotData[msg.slot_pos].pid 				= msg.pid;
	self.slotData[msg.slot_pos].slotId 				= msg.slot_pos;
	self.slotData[msg.slot_pos].skillId 			= msg.skill_id;
	self.slotData[msg.slot_pos].skillState 			= msg.skill_state;
	self.slotData[msg.slot_pos].skillLastTime 		= msg.skill_learn_time;
	self.slotData[msg.slot_pos].consumeItme 		= msg.consume_time;
	self.slotData[msg.slot_pos].isSendFinsh 		= false;

	local role = model.CardModel:GetCard(msg.pid);
	local skill_list = role.skill_list;
	for _k , skill in pairs(skill_list) do
		if skill.sid == msg.skill_id then
			self.slotData[msg.slot_pos].skillResid 	= skill.resid;
			self.slotData[msg.slot_pos].skillLevel 	= skill.level;
			self.slotData[msg.slot_pos].skillExp 	= skill.exp;
			break;
		end
	end
end

function SkillLevelUpModel:onRetSkillLearnFinish(msg)
	if not msg then
		return;
	end

	local role = model.CardModel:GetCard(msg.skill_lvlup_info.pid);

	for _k,skill in pairs(role.skill_list) do
		if skill.sid == msg.skill_lvlup_info.skill_id then
			skill.level = msg.skill_lvlup_info.skill_lv
			skill.exp = msg.skill_lvlup_info.skill_exp
			skill.resid = msg.skill_lvlup_info.skill_resid;
			self:resetSlotData(msg.skill_lvlup_info.slot_pos);
			break;
		end
	end
	
end
function SkillLevelUpModel:onRetSkillLevelCancel(msg)
	self:resetSlotData(msg.slot_pos);
end
function SkillLevelUpModel:onRetSkillLevelSpeed(msg)
	if not msg then
		return;
	end

	local role = model.CardModel:GetCard(msg.skill_lvlup_info.pid);

	for _k,skill in pairs(role.skill_list) do
		if skill.sid == msg.skill_lvlup_info.skill_id then
			skill.level = msg.skill_lvlup_info.skill_lv;
			skill.exp = msg.skill_lvlup_info.skill_exp;
			skill.resid = msg.skill_lvlup_info.skill_resid;
			self:resetSlotData(msg.skill_lvlup_info.slot_pos);
			break;
		end
	end
end
return SkillLevelUpModel
