local TeamModel = class("TeamModel")

local restrainType = Enum:getTable("FightRestraintType")
local Attribute = Enum:getTable("Attribute")
local TeamType = Enum:getTable('TeamType')
local FightType = Enum:getTable("FightType");

function TeamModel:ctor()
	self:clearAll()
end

function TeamModel:clearAll( ... )
	self.formation_state_lv = 1;
	self.formationRedDotState = {};
end

function TeamModel:Init(msg)
	self.team = {};
	self.default_tid = 1;
	self.event_tid = 5;
	self.arena_tid = 6;
	self.guild_tid = 2;
	self.trialRed_tid = 8;
	self.trialGreen_tid = 9;
	self.trialBlue_tid = 10;
	self.arena1_tid = 11;
	self.arena2_tid = 12;
	self.arena3_tid = 13;
	self.story_tid = 14;
	self.tidList = {self.default_tid, self.arena_tid, self.guild_tid,
		self.trialRed_tid, self.trialGreen_tid, self.trialBlue_tid,
		self.arena1_tid, self.arena2_tid, self.arena3_tid
	}
	self.oldFp = '0';
	self:initTeamInfo(msg);

end

function TeamModel:getAllTeamIdList()
	local isLock = tools.ConfigStateTools:checkLockById(2920);
	if isLock then
		return {self.default_tid, self.arena_tid, self.guild_tid,
				self.trialRed_tid, self.trialGreen_tid, self.trialBlue_tid
				}
	else
		return self.tidList;
	end
end

function TeamModel:getTeamNameById(tid)
	local teamName = LANG.UI_TEAM_N1;
	if tid == self.arena_tid then
		teamName = LANG.UI_TEAM_N3;
	elseif tid == self.guild_tid then
		teamName = LANG.UI_TEAM_N2;
	elseif tid == self.trialRed_tid then
		teamName = LANG.UI_TEAM_N4;
	elseif tid == self.trialGreen_tid then
		teamName = LANG.UI_TEAM_N6;
	elseif tid == self.trialBlue_tid then
		teamName = LANG.UI_TEAM_N5;
	elseif tid == self.event_tid then
		teamName = LANG.UI_TEAM_N10
	elseif tid == self.arena1_tid then
		teamName = LANG.UI_TEAM_N7;
	elseif tid == self.arena2_tid then
		teamName = LANG.UI_TEAM_N8;
	elseif tid == self.arena3_tid then
		teamName = LANG.UI_TEAM_N9;
	end
	return teamName;

end

function TeamModel:initTeamInfo(msg)
	for k, v in pairs(msg) do
		self.team[k] = {};
		self.team[k].team = {};
		for i=1, TEAM_MAX_MEMBER do
			local pid = tonumber(v['pid'..i]);
			local card = model.CardModel:GetCard(pid);
			if not card then
				if pid>=5000 and pid<6000 then --event 机器人卡牌
				else
					pid = -1;
				end
			end

			self.team[k].team[i] = pid;
		end
		self.team[k].captain = tonumber(v.team_leader_pid);
		self.team[k].formation = tonumber(v.formation);
		self.team[k].formation = self.team[k].formation > 0 and self.team[k].formation or 1
		self.team[k].restraintString = '000'; --[blue][red][green]
		self:updateRestraintTypeByTeamId(k)
		self:updateTeamBareAllAtt(k)
	end
	
	local key = {self.event_tid, self.guild_tid, 
				self.trialRed_tid,self.trialGreen_tid,
				self.trialBlue_tid,self.arena1_tid,
				self.arena2_tid,self.arena3_tid, self.story_tid};
	for k,v in pairs(key) do
		if self.team[v] == nil then
			self.team[v] = {}
			self.team[v].team = {-1,-1,-1,-1}
			self.team[v].formation = 1;
			self.team[v].restraintString = '000'; --[blue][red][green]
			self.team[v].colorAtt = {}
		end
	end

end

function TeamModel:updateRestraintTypeByTeamId( teamId )
	if teamId == self.event_tid then
		return
	end
	local team = self.team[teamId].team
	local restraint = {}
	restraint[restrainType.RED_BASE] = 0;
	restraint[restrainType.GREEN_BASE] = 0;
	restraint[restrainType.BLUE_BASE] = 0;
	for i=1, TEAM_MAX_MEMBER do
		local pid = team[i];
		if pid ~= -1 and not(pid>=5000 and pid<6000) then
			local data = model.CardModel:GetCard(pid);
			local roleData = ResTable:getRowData('role', data.no);
			local def_type = ResTable:getValue('role_pre', roleData.role_id, 'def_type');
			restraint[def_type] = restraint[def_type] + 1;
		end
	end
	self.team[teamId].restraintString = tostring(restraint[restrainType.BLUE_BASE])..tostring(restraint[restrainType.RED_BASE])..tostring(restraint[restrainType.GREEN_BASE]);
	-- print('==========================team teamId = '..teamId..' ==========restraintString =  ',self.team[teamId].restraintString)
	self:updateRestraintAmplification(teamId)
end

function TeamModel:updateEventRestraint(  )
	local teamId = self.event_tid;
	local team = self.team[teamId].team
	local restraint = {}
	restraint[restrainType.RED_BASE] = 0;
	restraint[restrainType.GREEN_BASE] = 0;
	restraint[restrainType.BLUE_BASE] = 0;
	for i=1, TEAM_MAX_MEMBER do
		local pid = team[i];
		if pid ~= -1 then
			local data = model.CardModel:GetCard(pid);
			local roleData = ResTable:getRowData('role', data.no);
			local def_type = ResTable:getValue('role_pre', roleData.role_id, 'def_type');
			restraint[def_type] = restraint[def_type] + 1;
		end
	end
	self.team[teamId].restraintString = tostring(restraint[restrainType.BLUE_BASE])..tostring(restraint[restrainType.RED_BASE])..tostring(restraint[restrainType.GREEN_BASE]);
	-- print('==========================team teamId = '..teamId..' ==========restraintString =  ',self.team[teamId].restraintString)
	self:updateRestraintAmplification(teamId)
end

-- restraintString :[0-4][0-4][0-4]  每位相加不大于4
function TeamModel:getTeamRestaintTypeByString( restraintString )
	-- body
	if restraintString == nil then
		return restrainType.NULL,0;
	end

	local redNum = tonumber(string.sub(restraintString,restrainType.RED_BASE,restrainType.RED_BASE)) 
	local greenNum = tonumber(string.sub(restraintString,restrainType.GREEN_BASE,restrainType.GREEN_BASE)) 
	local blueNum = tonumber(string.sub(restraintString,restrainType.BLUE_BASE,restrainType.BLUE_BASE)) 

	local tempRestrainType,tempStr = restrainType.NULL,'0'
	if redNum >= 2 then
		if greenNum == 2 then
			return restrainType.RED_GREEN,'2/2';
		elseif blueNum == 2 then 
			return restrainType.RED_BLUE,'2/2';
		else
			return restrainType.ALL_RED,tostring(redNum);
		end
	end

	if greenNum >= 2 then
		if blueNum == 2 then
			return restrainType.GREEN_BLUE,'2/2';
		else
			return restrainType.ALL_GREEN,tostring(greenNum);
		end
	end
	
	if blueNum >= 2 then
		return restrainType.ALL_BLUE,tostring(blueNum);
	end
	return restrainType.NULL,'0';

end

function TeamModel:getTeamRestaintNum(restraintString)
	local redNum = tonumber(string.sub(restraintString,restrainType.RED_BASE,restrainType.RED_BASE)) 
	local greenNum = tonumber(string.sub(restraintString,restrainType.GREEN_BASE,restrainType.GREEN_BASE)) 
	local blueNum = tonumber(string.sub(restraintString,restrainType.BLUE_BASE,restrainType.BLUE_BASE))
	return redNum, greenNum, blueNum
end

function TeamModel:updateRestraintAmplification(tid)
	local restraintString = self.team[tid].restraintString
	self.team[tid].restraintFactor = {}
	self.team[tid].restraintFactor[1] = 0
	self.team[tid].restraintFactor[2] = 0
	local restraintNum = {}
	restraintNum[restrainType.RED_BASE]  = tonumber(string.sub(restraintString,restrainType.RED_BASE,restrainType.RED_BASE)) 
	restraintNum[restrainType.GREEN_BASE]  = tonumber(string.sub(restraintString,restrainType.GREEN_BASE,restrainType.GREEN_BASE)) 
	restraintNum[restrainType.BLUE_BASE]  = tonumber(string.sub(restraintString,restrainType.BLUE_BASE,restrainType.BLUE_BASE)) 
	for k, v in pairs(ResTable:getTable('formation_attribute').contain) do
		if restraintNum[v.color] == v.value then
			for k1, v1 in pairs(v.attribute) do
				if not self.team[tid].restraintFactor[v1[1]] then
					self.team[tid].restraintFactor[v1[1]] = v1[2]
				else
					self.team[tid].restraintFactor[v1[1]] = self.team[tid].restraintFactor[v1[1]] + v1[2]
				end
			end
		end
	end
end

function TeamModel:getRestraintAmplificationFactor(factorType, tid)
	tid = tid or self.default_tid;
	if not self.team[tid].restraintFactor then
		self:updateRestraintAmplification(tid)
	end
	return self.team[tid].restraintFactor[factorType]
end

function TeamModel:getRestraintAmplificationByInfo( curTeam )

	local restraint = {}
	restraint[restrainType.RED_BASE] = 0;
	restraint[restrainType.GREEN_BASE] = 0;
	restraint[restrainType.BLUE_BASE] = 0;
	for k,v in pairs(curTeam) do
		local pid = v;
		if pid ~= -1 then
			local data = model.CardModel:GetCard(pid);
			local roleData = ResTable:getRowData('role', data.no);
			local def_type = ResTable:getValue('role_pre', roleData.role_id, 'def_type');
			restraint[def_type] = restraint[def_type] + 1;
		end
	end
	local restraintString = tostring(restraint[restrainType.BLUE_BASE])..tostring(restraint[restrainType.RED_BASE])..tostring(restraint[restrainType.GREEN_BASE]);
	
	local restraintFactor = {}
	restraintFactor[1] = 0
	restraintFactor[2] = 0
	local restraintNum = {}
	restraintNum[restrainType.RED_BASE]  = tonumber(string.sub(restraintString,restrainType.RED_BASE,restrainType.RED_BASE)) 
	restraintNum[restrainType.GREEN_BASE]  = tonumber(string.sub(restraintString,restrainType.GREEN_BASE,restrainType.GREEN_BASE)) 
	restraintNum[restrainType.BLUE_BASE]  = tonumber(string.sub(restraintString,restrainType.BLUE_BASE,restrainType.BLUE_BASE)) 
	for k, v in pairs(ResTable:getTable('formation_attribute').contain) do
		if restraintNum[v.color] == v.value then
			for k1, v1 in pairs(v.attribute) do
				if not restraintFactor[v1[1]] then
					restraintFactor[v1[1]] = v1[2]
				else
					restraintFactor[v1[1]] = restraintFactor[v1[1]] + v1[2]
				end
			end
		end
	end

	self.tmpColorAtt = {}
	for i=1, TEAM_MAX_MEMBER do
		if curTeam[i] > 0 then
			local data = model.CardModel:GetCard( curTeam[i] );
			local bareIdList = {}
			if data then
				if data.bare_id and data.bare_id > 0 then
					table.insert(bareIdList, data.bare_id)
				end
				if data.bare_id_2 and data.bare_id_2 > 0 then
					table.insert(bareIdList, data.bare_id_2)
				end
			end
			if #bareIdList > 0 then
				for k,v in pairs(bareIdList) do
					local bareData = model.BareCollectModel:getBareData(v);
					if bareData and bareData.attribute_color then
						local bareCollectPro = nil
						if self.tmpColorAtt[bareData.attribute_color] then
							bareCollectPro = self.tmpColorAtt[bareData.attribute_color]
						else
							bareCollectPro = {}
							bareCollectPro[Attribute[1]]={addValue=0, multValue=0}
							bareCollectPro[Attribute[2]]={addValue=0, multValue=0}
						end
						local attType = bareData.attribute_effect2[1];
						local attValue = bareData.attribute_effect2[2];
						local multValue = bareData.attribute_effect2[3];
						if attType == 1 or attType == 2 then
							bareCollectPro[Attribute[attType]].addValue = attValue + bareCollectPro[Attribute[attType]].addValue;
							bareCollectPro[Attribute[attType]].multValue = multValue + bareCollectPro[Attribute[attType]].multValue;
						end
						self.tmpColorAtt[bareData.attribute_color] = bareCollectPro;
					end
				end
			end
		end
	end

	return restraintFactor
end

-- 队伍克制类型（400） :[0-4][0-4][0-4]  每位相加不大于4
function TeamModel:getCurrentTeamRestraintString(  )
	return self.team[self.default_tid].restraintString;
end

function TeamModel:getTeamRestaintString(tid)
	if self.team[tid] then
		return self.team[tid].restraintString;
	end
end

function TeamModel:getCurrentTeam()
	return self.team[self.default_tid].team;
end

function TeamModel:getEventTeam()
	return self.team[self.event_tid].team
end

function TeamModel:getArenaTeam()
	return self.team[self.arena_tid].team
end

function TeamModel:getGuildTeam()
	return self.team[self.guild_tid].team
end

function TeamModel:getTeamById( tid )
	if self.team[tid] then
		return self.team[tid].team;
	else
		return {}
	end
end

function TeamModel:CurrentTeamHaveCard(pid)
	local flag = false;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[1].team[i] == pid then
			flag = true;
			break;
		end
	end
	return flag;
end

-- function TeamModel:ChangeTeam(index, pid, tid)
-- 	tid = tid or self.default_tid;
-- 	local team = self.team[tid];
-- 	if pid == -1 then
-- 		team[index] = -1;
-- 	else
-- 		team[index] = pid;
-- 	end
-- 	self:sendInfo();
-- end

-- function TeamModel:onChangeTeam(msg)
-- 	for k, v in pairs(msg) do
-- 		if self.team[tonumber(k)] then
-- 			for k1, v1 in pairs(v) do
-- 				if self.team[tonumber(k)][tonumber(k1)] then
-- 					self.team[tonumber(k)][tonumber(k1)] = tonumber(v1);
-- 				end
-- 			end
-- 		end
-- 	end
-- end

-- function TeamModel:changeTeam(index , pid , tid)
-- 	tid = tid or self.default_tid;
-- 	if self.team[tid] and self.team[tid][index] then
-- 		self.team[tid][index] = pid;
-- 	end
-- end

function TeamModel:sendChangeTeam(team, tid)
	tid = tid or self.default_tid
	self.oldFp = self:getCurrentTeamFp(tid);
	for i = 1 , TEAM_MAX_MEMBER do
		self.team[tid].team[i] = team[i] or self.team[tid].team[i];
	end
	self:updateTeamBareAllAtt(tid);
	self:sendInfo(tid);
end

function TeamModel:sendInfo(tid)
	local team = {};
	for i=1, TEAM_MAX_MEMBER do
		team[i] = self.team[tid].team[i];
	end

	local msg = {};
    msg.tid             =   tid;
    msg.team_pid        =   team;
    msg.team_leader_pid =   self.team[tid].captain;
	msg.formation 		= 	self.team[tid].formation;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_save_battle_team, msg);
end

function TeamModel:getTeamIdByFightType( fightType, data )
	local tid = self.default_tid
	if fightType == FightType.Event then
		tid = self.event_tid
	elseif fightType == FightType.Arena then
		tid = self.arena_tid;
	elseif fightType == FightType.GuildBoss then
		tid = self.guild_tid;
	elseif fightType == FightType.TrialTower then
		if data and data.trial_type then
			tid = model.TrialModel:getTrialTeamIdByType(data.trial_type)
		end
	elseif fightType == FightType.MultArena then
		if data and data.subIndex then
			tid = model.ArenaModel:getMultArenaTeamIdByIdx( data.subIndex );
		end
	elseif fightType == FightType.Story then
		tid = self.story_tid;
	else
		
	end
	return tid;
end

function TeamModel:getTeamIdByTeamType( teamType )
	local tid
	if teamType == TeamType.EVENT then
		tid = self.event_tid
	elseif teamType == TeamType.ARENA then
		tid = self.arena_tid;
	elseif teamType == TeamType.GUILD_BOSS then
		tid = self.guild_tid;
	elseif teamType == TeamType.TRIAL_R then
		tid = self.trialRed_tid;
	elseif teamType == TeamType.TRIAL_G then
		tid = self.trialGreen_tid;
	elseif teamType == TeamType.TRIAL_B then
		tid = self.trialBlue_tid;
	elseif teamType == TeamType.ARENA_1 then
		tid = self.arena1_tid;
	elseif teamType == TeamType.ARENA_2 then
		tid = self.arena2_tid;
	elseif teamType == TeamType.ARENA_3 then
		tid = self.arena3_tid;
	elseif teamType == TeamType.STORY then
		tid = self.story_tid;
	else
		tid = self.default_tid
	end
	return tid;
end

function TeamModel:reqChangeFormation( formation, teamType , team)
	local tid
	if teamType == TeamType.EVENT then
		tid = self.event_tid
	elseif teamType == TeamType.ARENA then
		tid = self.arena_tid;
	elseif teamType == TeamType.GUILD_BOSS then
		tid = self.guild_tid;
	elseif teamType == TeamType.TRIAL_R then
		tid = self.trialRed_tid;
	elseif teamType == TeamType.TRIAL_G then
		tid = self.trialGreen_tid;
	elseif teamType == TeamType.TRIAL_B then
		tid = self.trialBlue_tid;
	elseif teamType == TeamType.ARENA_1 then
		tid = self.arena1_tid;
	elseif teamType == TeamType.ARENA_2 then
		tid = self.arena2_tid;
	elseif teamType == TeamType.ARENA_3 then
		tid = self.arena3_tid;
	elseif teamType == TeamType.STORY then
		tid = self.story_tid;
	else
		tid = self.default_tid
	end

	self.oldFp = self:getCurrentTeamFp(tid);

	local msg = {};
	if team then--阵型和编队都要更换
		msg.team_pid = {};
		for i = 1 , TEAM_MAX_MEMBER do
			local pid = team[i] or self.team[tid].team[i];
			self.team[tid].team[i] = pid;
			msg.team_pid[i] = pid;
		end
		self.team[tid].formation = formation;
	end

    msg.tid             =   tid;
    -- msg.team_pid        =   team;
    msg.team_leader_pid =   self.team[tid].captain;
	msg.formation 		= 	formation;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_save_battle_team, msg);
end

function TeamModel:retChangeTeam( msg )

	if msg.team_info ~= nil  then
		local tid = msg.team_info.tid;
		self.team[tid].formation = tonumber(msg.team_info.formation) or self.team[tid].formation;
		if msg.team_info.team_pid then
			for i=1, TEAM_MAX_MEMBER do
				self.team[tid].team[i] = msg.team_info.team_pid[i];
			end
		end

		if tid == self.event_tid then
			self:updateEventRestraint()
		else
			self:updateRestraintTypeByTeamId(tid)
		end
		--EventManager:FireEvent(EventType.FightPanelTeamRestraintChange);

		self:checkTeamFpUp(self.oldFp,tid);
	end
	tools.RedDotTools:pushCardRedDot(1);
end

function TeamModel:getTeamFormationList(  )
	if self.teamFormationList == nil  then
		self.teamFormationList = GlobalFunctions:sortTableByKey(ResTable:getTable('formation').contain);
		self:updateFormationStateLv();
		-- self.formation_state_lv = 2;
	end
	return self.teamFormationList
end

function TeamModel:updateFormationStateLv(  )
	if self.formation_state_lv == -1 then
		return ;
	end
	local userLv = model.UserModel:getUserLevel()
	for i=1,#self.teamFormationList do
		self.formation_state_lv = self.teamFormationList[i].lv
		if userLv < self.formation_state_lv then
			return
		end
	end
	self.formation_state_lv = -1; -- 阵型已全部获得
end

function TeamModel:updateFormationState()
	self:getTeamFormationList(  )
	if self.formation_state_lv == -1 then
		return;
	end

	local userLv = model.UserModel:getUserLevel()
	for i=1,#self.teamFormationList do
		local data = self.teamFormationList[i];
		if data.lv >= self.formation_state_lv then
			if userLv >= data.lv then
				self.formationRedDotState[data.id] = 1;
			else
				self.formation_state_lv = data.lv;
				return;
			end
		end
	end
	self.formation_state_lv = -1;
end

function TeamModel:clearTeamFormationRedDotState()
	self.formationRedDotState = {};
end

function TeamModel:getTeamFormationRedDotState()
	return self.formationRedDotState;
end

function TeamModel:checkTeamFormationRedDotState()
	return #table.keys(self.formationRedDotState) > 0;
end

function TeamModel:getCurrentTeamFormation( tid )
	tid = tid or self.default_tid
	return self.team[tid].formation;
end

function TeamModel:getEventTeamFormation()
	return self.team[self.event_tid].formation;
end

function TeamModel:getTeamFormationPosById( formation )
	self:getTeamFormationList()
	return self.teamFormationList[formation].position;
end

function TeamModel:getTeamTypeById(tid)
	local teamType = TeamType.COMMON
	if tid == self.event_tid then
		teamType = TeamType.EVENT
	elseif tid == self.arena_tid then
		teamType = TeamType.ARENA
	elseif tid == self.guild_tid then
		teamType = TeamType.GUILD_BOSS
	elseif tid == self.trialRed_tid then
		teamType = TeamType.TRIAL_R
	elseif tid == self.trialGreen_tid then
		teamType = TeamType.TRIAL_G
	elseif tid == self.trialBlue_tid then
		teamType = TeamType.TRIAL_B
	elseif tid == self.default_tid then
		teamType = TeamType.COMMON
	elseif tid == self.arena1_tid then
		teamType = TeamType.ARENA_1;
	elseif tid == self.arena2_tid then
		teamType = TeamType.ARENA_2;
	elseif tid == self.arena3_tid then
		teamType = TeamType.ARENA_3;
	elseif tid == self.story_tid then
		teamType = TeamType.STORY;
	end
	return teamType;
end

function TeamModel:getCurrentTeamFp(tId)
	local totalFp = '0';
	tId = tId or self.default_tid;
	local teamType = self:getTeamTypeById(tId)
	for i=1, TEAM_MAX_MEMBER do
		if self.team[tId].team[i] ~= -1 then
			local pro = CardProManager:GenProInTeam(self.team[tId].team[i], nil, nil, teamType);
			if pro then
				totalFp = HpPlus(totalFp, ConvertDigitUnitToStr(pro.fp));
			end
		end
	end
	return totalFp;
end

function TeamModel:getFormationStateLv(  )
	return self.formation_state_lv;
end

function TeamModel:isRoleInTeam(pid,tId)
	tId = tId or self.default_tid;
	local flag , pos = false , -1;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[tId].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
	end
	return flag , pos;
end

function TeamModel:isRoleInEventTeam(pid)
	local flag , pos = false , -1;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.event_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
	end
	return flag , pos;
end

function TeamModel:isRoleInArenaTeam(pid)
	local flag , pos = false , -1;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.arena_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
	end
	return flag , pos;
end

function TeamModel:isRoleInGuildTeam(pid)
	local flag , pos = false , -1;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.guild_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
	end
	return flag , pos;
end

function TeamModel:isRoleInTrialTeam(pid)
	local flag , pos = false , -1;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.trialRed_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
		if self.team[self.trialGreen_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
		if self.team[self.trialBlue_tid].team[i] == pid then
			flag = true;
			pos = i;
			break;
		end
	end
	return flag , pos;
end

function TeamModel:isRoleInArenaMulTeam(pid)
	local flag , pos, tid = false , -1, 0;
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.arena1_tid].team[i] == pid then
			flag = true;
			pos = i;
			tid = self.arena1_tid;
			break;
		end
		if self.team[self.arena2_tid].team[i] == pid then
			flag = true;
			pos = i;
			tid = self.arena2_tid;
			break;
		end
		if self.team[self.arena3_tid].team[i] == pid then
			flag = true;
			pos = i;
			tid = self.arena3_tid;
			break;
		end
	end
	return flag , pos , tid;
end

function TeamModel:canEventFight()
	local canFight = false;
	for i=1, TEAM_MAX_MEMBER do
		local pid = self.team[self.event_tid].team[i];
		if pid ~=-1 then
			if model.CardModel:getEventHp(pid) <= 0 then
				canFight = false;
				break;
			else
				canFight = true;
			end
			
		end
	end
	return canFight;
end

function TeamModel:checkTeamFpUp(oldFp , tid)
	if not oldFp or oldFp == '' then
		return;
	end

	local newFp = self:getCurrentTeamFp(tid);
	newFp = ConvertDigitUnitToStr(tostring(newFp));
	oldFp = ConvertDigitUnitToStr(tostring(oldFp))
	if true or HpCompare(newFp , '>' , oldFp) then
	  FpUpPanel:Show(oldFp , newFp);
	end

	if tid == self.default_tid then
		model.UserModel:checkTopPower(newFp);
	end
end


-- 判断是否有新阵型解锁，
function TeamModel:getRedDotStateFormation(  )

	if self.formation_state_lv == nil then
		self:updateNextFormationLv();
	end	

	if self.formation_state_lv == -1 then
		-- 阵型已全部获得
		return false;
	end
	local lv = model.UserModel:getUserLevel()
	return  self.formation_state_lv < lv;
end

function TeamModel:checkEventTeam()
	local hasMember = false
	for i=1, TEAM_MAX_MEMBER do
		if self.team[self.event_tid].team[i] ~= -1 then
			hasMember = true
			break
		end
	end
	return hasMember
end

function TeamModel:checkTeamByTid(tid)
	if not self.team[tid] then
		return false
	end
	local hasMember = false
	for i=1, TEAM_MAX_MEMBER do
		if self.team[tid].team[i] ~= -1 then
			hasMember = true
			break
		end
	end
	return hasMember
end

function TeamModel:getTopFpTeam()
	local team = {};
	local count = 0
	local cardNoList = {};
	for i=1, TEAM_MAX_MEMBER do
		team[i] = {
			pid = -1,
			fp = '0',
			role_id = -1,
		};
	end
	for k, v in pairs(model.CardModel:GetAllCard()) do
		count = count + 1
		local fp = ConvertDigitUnitToStr(CardProManager:GetCardPro(v.pid).fp);
		if not cardNoList[v.no] then
			cardNoList[v.no] = {
				fp = fp,
				pid = v.pid
			};
		else
			if HpCompare(cardNoList[v.no].fp, '<', fp) then
				cardNoList[v.no].fp = fp;
				cardNoList[v.no].pid = v.pid;
			end
		end
	end

	local noList = {};
	for no,data in pairs(cardNoList) do
		if HpCompare(data.fp, '>', team[4].fp) then
			local aim_pos = 1;
			for i=TEAM_MAX_MEMBER-1, 1, -1 do
				if HpCompare(data.fp, '<=', team[i].fp) then
					aim_pos = i+1;
					break;
				end
			end
			if aim_pos < TEAM_MAX_MEMBER then
				for i=TEAM_MAX_MEMBER, aim_pos+1, -1 do
					team[i].pid = team[i-1].pid;
					team[i].fp = team[i-1].fp;
					team[i].role_id = team[i-1].role_id;
				end
			end
			team[aim_pos].pid = data.pid;
			team[aim_pos].fp = data.fp;
			team[aim_pos].role_id = no;
		end
	end

	return team;
end

function TeamModel:updateWearOrTakeoffBareAllAtt(bareId, pid)
	local bareData = model.BareCollectModel:getBareData(bareId)
	if not bareData then
		return
	end
	if bareData.attribute_effect2 and (bareData.attribute_effect2[2]>0 or bareData.attribute_effect2[3]>0) then
		local needUpTids = {}
		for k,v in pairs(self.team) do
			for i=1, TEAM_MAX_MEMBER do
				if v.team[i] == pid and pid>0 then
					table.insert(needUpTids, k)
					break;
				end
			end
		end
		for _,tid in pairs(needUpTids) do
			self:updateTeamBareAllAtt(tid)
		end
	end
end

function TeamModel:updateTeamBareAllAtt( tId )
	self.team[tId].colorAtt = {}
	for i=1, TEAM_MAX_MEMBER do
		if self.team[tId].team[i] > 0 then
			local data = model.CardModel:GetCard( self.team[tId].team[i] );
			local bareIdList = {}
			if data then
				if data.bare_id and data.bare_id > 0 then
					table.insert(bareIdList, data.bare_id)
				end
				if data.bare_id_2 and data.bare_id_2 > 0 then
					table.insert(bareIdList, data.bare_id_2)
				end
			end
			if #bareIdList > 0 then
				for k,v in pairs(bareIdList) do
					local bareData = model.BareCollectModel:getBareData(v);
					if bareData and bareData.attribute_color then
						local bareCollectPro = nil
						if self.team[tId].colorAtt[bareData.attribute_color] then
							bareCollectPro = self.team[tId].colorAtt[bareData.attribute_color]
						else
							bareCollectPro = {}
							bareCollectPro[Attribute[1]]={addValue=0, multValue=0}
							bareCollectPro[Attribute[2]]={addValue=0, multValue=0}
						end
						local attType = bareData.attribute_effect2[1];
						local attValue = bareData.attribute_effect2[2];
						local multValue = bareData.attribute_effect2[3];
						if attType == 1 or attType == 2 then
							bareCollectPro[Attribute[attType]].addValue = attValue + bareCollectPro[Attribute[attType]].addValue;
							bareCollectPro[Attribute[attType]].multValue = multValue + bareCollectPro[Attribute[attType]].multValue;
						end
						self.team[tId].colorAtt[bareData.attribute_color] = bareCollectPro;
					end
				end
			end
		end
	end
end

function TeamModel:getTeamColorAtt(tid, color)
	if self.team[tid] and self.team[tid].colorAtt[color] then
		return self.team[tid].colorAtt[color];
	end
	return nil;
end

function TeamModel:getTmpTeamColorAtt(color)
	if self.tmpColorAtt and self.tmpColorAtt[color] then
		return self.tmpColorAtt[color];
	end
	return nil;
end

return TeamModel;
