local TrialModel = class("TrialModel")
-- local TRIAL_AUTO_NUM_MAX = 2 --挑战次数上限
local FightType = Enum:getTable('FightType');

function TrialModel:ctor()
	self:clearAll()
end

function TrialModel:clearAll(  )
	self.trialList = nil;
	self.trial_rewards_status = nil;

	self.trial_no = 0; --已通关关卡
	self.auto_num_now 	= 0;
	self.auto_num_total = 0;
	self.auto_num_buy = {10001,0,0} --购买挑战次数 货币类型，数量，剩余次数
	self.trialRewardState = {}
	self.type_trial_info = {}
	ResTable:setIsLoadFalse("trial_round")

end

function TrialModel:InitData(trial_round_msg)
	self:clearAll()

	self.trial_rewards_status = trial_round_msg.round_reward or {}; 	--已领取奖励
	self.trial_no 		= trial_round_msg.trial_no or 0;		 		--当前关卡
	self.auto_num_now 	= trial_round_msg.auto_num; 				--剩余扫荡次数
	self.auto_num_total = trial_round_msg.total_num; 			--扫荡上限
	self.auto_num_buy 	= trial_round_msg.buy_next_coin; 
	if trial_round_msg.type_trial_info then
		self.type_trial_info = trial_round_msg.type_trial_info;
	end

end

function TrialModel:getTrialList(  )
	if self.trialList == nil  then
		-- self.trialList = GlobalFunctions:sortTableByKey(ResTable:getTable('trial_round').contain);
		-- for i=1,#self.trial_rewards_status do
		-- 	self.trialList[self.trial_rewards_status[i]].status = 1;
		-- end

		self.trialList = {}
		for k,v in pairs(ResTable:getTable('trial_round').contain) do
			if self.trialList[v.type] == nil then
				self.trialList[v.type] = {}
			end
			self.trialList[v.type][v.no] = v
		end
		for i=1,#self.trial_rewards_status do
			self.trialList[1][self.trial_rewards_status[i]].status = 1;
		end
	end
	return self.trialList
end

function TrialModel:getTrialTeamIdByType( tp )
	local tid = 1;
	if tp==2 then
		tid = 10;
	elseif tp==3 then
		tid = 8;
	elseif tp==4 then
		tid = 9;
	end
	return tid;
end

function TrialModel:updatTrialListStatus(  )
	self:getTrialList()
	for i=1,#self.trial_rewards_status do
		self.trialList[1][self.trial_rewards_status[i]].status = 1;
	end
end

function TrialModel:getTrialRewardList( )
	return self.trial_rewards_status;
end

--获取当前已通关最大关卡
function TrialModel:getFinshTrialId(tp)
	if tp and self.type_trial_info[''..tp] then
		return self.type_trial_info[''..tp].no;
	else
		return self.trial_no;
	end
end

-- 获取当前未通过关卡
function TrialModel:getCurrentTrialId(tp)
	if tp and self.type_trial_info[''..tp] then
		return self.type_trial_info[''..tp].no + 1;
	else
		return self.trial_no + 1;
	end
end

function TrialModel:getLimitFightNum(tp)
	if tp and self.type_trial_info[''..tp] then
		return self.type_trial_info[''..tp].remaining_times;
	else
		return -1;
	end
end

function TrialModel:getItemDataById( id, tp )
	local t = tp or 1
	self:getTrialList()
	return self.trialList[t][id];
end

function TrialModel:getStatusById( id )
	self:getTrialList()
	return self.trialList[1][id].status;
end

--请求试练塔信息
function TrialModel:reqTrialInfo(  )
	-- 
	
	local msg = {}
	Network.NetworkManager:Send(Network.NetworkCmdType.req_trial_info , msg);
end

function TrialModel:retTrialInfo( msg )
	self:InitData(msg);
	self:updatTrialListStatus();
end

-- 购买挑战次数
-- function TrialModel:reqTrialBuyAutoNum()
-- 	local msg = {}
-- 	msg.count = 1;
-- 	msg.num = self.auto_num_buy[2] or 0
-- 	Network.NetworkManager:Send(Network.NetworkCmdType.req_buy_trial_auto_num , msg);
-- end

function TrialModel:retTrialBuyAutoNum( msg )
	local trialData =  msg.trial_round;
	if trialData then
		self.auto_num_now = trialData.auto_num;
		self.auto_num_buy = trialData.buy_next_coin;
	end

end
-- 请求战斗
-- 0 挑战
-- 1 扫荡
function TrialModel:reqTrialFight(tp)
	local opation = {}
	opation.auto = 0;
	opation.trial_no = self:getCurrentTrialId(tp)
	opation.trial_type = tp;
	FightManager:enterFight(FightType.TrialTower , opation)	
end

function TrialModel:retTrialFightOver( msg )
	if msg.trial_round ~= nil then
		self:InitData(msg.trial_round);
	end
	-- GlobalFunctions:showRewardPrompt(msg , LANG.UI_ACTIVITY_16)

end

function TrialModel:reqTrialReward( id )
	
	if id == nil then
		return 
	end
	if id ~= 0 then
		if id > self.trial_no then
			print('==== 当前关卡尚未通过！！ ');
		end

		local status = self:getStatusById(id);
		if status == 1 then
			print("==== 奖励已领取====")
			return
		end
	end

	local msg = {}
	msg.id = id;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_trial_get_reward , msg);
end

function TrialModel:retTrialReward( msg )
	self.trial_rewards_status = msg.round_reward
	if msg.id and  #msg.id > 0 then
		for _k , id in pairs(msg.id) do
			self.trialList[1][id].status = 1;
			-- self:updatTrialListStatus();
			GlobalFunctions:showRewardPrompt(msg , LANG.UI_ACTIVITY_16)
		end
	end

end

function TrialModel:getActiveState()
	local curRound = self:getFinshTrialId();
	for _k , reward in pairs(ResTable:getTable('trial_reward').contain) do
		if not self:getStatusById(reward.id) and curRound >= reward.round_id then
			return true;
		end
	end
	return false;
end

function TrialModel:getRedDotStateTrial( ... )
	if self:getLockStateTrial() then
		return false;
	end

	return  self:getActiveState() ;
end

-- bool,lockLv
function TrialModel:getLockStateTrial(  )
    return tools.LevelLockTools:checkLockById(Enum:getTable('levelLockItemIds').LEVLE_LOCK_9);		
end

return TrialModel;



--- 1 
