local UserDefaultModel = class("UserDefaultModel")
local SDKLoginType = Enum:getTable("SDKLoginType")

local PLATFORM_TYPE_KEY     = 'platform_type' --userdefault 登录类型key
local GUEST_ACCOUNT_KEY     = 'guest_account' --userdefault 游客账号
local CLAUSE_SET_KEY        = 'clause_set' --userdefault 条款设置
local LANGUAGE_SET_KEY      = 'lg_set' --userdefault language_set
local CV_SET_KEY      = 'cv_set' --userdefault language_set
local DEFAULT_SER_id_KEY    = 'default_server_id' -- 默认服务器id
local DEFAULT_SER_NAME_KEY  = 'default_server_name' -- 默认服务器名称
local PLAY_VIDEO_ST         = 'play_video_st' -- 播放开场视频
local PUSH_STATE_KEY        = 'pushEventName' -- 推送开启状态 0，1，2
local FUNEEL                = 'funnel' --  打点

-- local PUSH_

function UserDefaultModel:ctor()
    self:clearAll();
end

function UserDefaultModel:clearAll()

    self.clause_set_state = nil
    self.play_video_state = nil
    self.language_set = nil 
    self.guestAccount = nil 
    self.platform_type = -1;
    self.defaultServerID = 0;
    self.defaultServerName = '';

    self.pushEventState = -1
end

function UserDefaultModel:clearAboutUserData()
    self.play_video_state = nil
end

-- 同一设备共享推送状态
-- ret 0 off, 1 推送 on  夜间推送 on,2 推送 on  夜间推送 off
function UserDefaultModel:getPushEventState(  )
    if self.pushEventState == -1 then
        self.pushEventState = cc.UserDefault:getInstance():getIntegerForKey(PUSH_STATE_KEY);
    end
    return self.pushEventState;

end
function UserDefaultModel:setPushEventState(  state )
    if self.pushEventState ~= state then
        cc.UserDefault:getInstance():setIntegerForKey(PUSH_STATE_KEY, state);
		cc.UserDefault:getInstance():flush();
    end
    self.pushEventState = state 

end

function UserDefaultModel:setClauseSetState( state )
    if self.clause_set_state ~= state then
		cc.UserDefault:getInstance():setBoolForKey(CLAUSE_SET_KEY, state);
		cc.UserDefault:getInstance():flush();
    end
    self.clause_set_state = state 

end

function UserDefaultModel:getClauseSetState(  )
    if self.clause_set_state == nil then
        self.clause_set_state = cc.UserDefault:getInstance():getBoolForKey(CLAUSE_SET_KEY);
    end
    return self.clause_set_state
end

function UserDefaultModel:setPlayVideoState( state )
    if self.play_video_state ~= state then
        local uid = model.AccountModel:getAccountUid()
        cc.UserDefault:getInstance():setBoolForKey(PLAY_VIDEO_ST..'_'..uid, state);
        cc.UserDefault:getInstance():flush();
    end
    self.play_video_state = state 

end

function UserDefaultModel:getPlayVideoState(  )
    if self.play_video_state == nil then
        local uid = model.AccountModel:getAccountUid()
        self.play_video_state = cc.UserDefault:getInstance():getBoolForKey(PLAY_VIDEO_ST..'_'..uid);
    end
    return self.play_video_state
end

function UserDefaultModel:setLanguageSet( lg_set )
    if self.language_set ~= lg_set then
		cc.UserDefault:getInstance():setStringForKey(LANGUAGE_SET_KEY, lg_set);
		cc.UserDefault:getInstance():flush();
    end
    self.language_set = lg_set 
end

function UserDefaultModel:getLanguageSet(  )
    if self.language_set == nil then
        local defLg = 'kr'
        if device.language == 'cn' or device.language=='tw' then
            defLg = 'cn'
        elseif device.language == 'kr' or device.language=='jp' then
            defLg = device.language
        else
            defLg = 'en'
        end
        self.language_set = cc.UserDefault:getInstance():getStringForKey(LANGUAGE_SET_KEY, defLg);
        -- self.language_set = cc.UserDefault:getInstance():getStringForKey(LANGUAGE_SET_KEY,'kr');
    end
    local package_name = sdkManager.SDKLuaHelper:getPackageName()
    if package_name == 'com.nani.ego.kr.one' then
        self.language_set = 'kr';
    end
    -- self.language_set = 'kr';
    return self.language_set
end

function UserDefaultModel:setCVLanguageSet( lg_set )
    if self.cv_set ~= lg_set then
        cc.UserDefault:getInstance():setStringForKey(CV_SET_KEY, lg_set);
        cc.UserDefault:getInstance():flush();
    end
    self.cv_set = lg_set 
end

function UserDefaultModel:getCVLanguageSet()
    self.cv_set = cc.UserDefault:getInstance():getStringForKey(CV_SET_KEY, 'jp');
    return self.cv_set;
end

function UserDefaultModel:setPlatformType( tag )
    if self.platform_type ~= tag then
		cc.UserDefault:getInstance():setIntegerForKey(PLATFORM_TYPE_KEY, tag);
		cc.UserDefault:getInstance():flush();
    end
    self.platform_type = tag 

end

function UserDefaultModel:getPlatformType(  )
    if self.platform_type == SDKLoginType.NULL then
        self.platform_type = cc.UserDefault:getInstance():getIntegerForKey(PLATFORM_TYPE_KEY);
    end
    return self.platform_type
end

function UserDefaultModel:setGuestAccount( account )
    if self.platform_type ~= SDKLoginType.LOGIN_ACCOUNT then
        if self.guestAccount ~= account then
            cc.UserDefault:getInstance():setStringForKey(GUEST_ACCOUNT_KEY, account);
            cc.UserDefault:getInstance():flush();
        end
        self.guestAccount = account;
	end
end

function UserDefaultModel:getGuestAccount(  )
    if self.guestAccount == nil then
        self.guestAccount = cc.UserDefault:getInstance():getStringForKey(GUEST_ACCOUNT_KEY);
    end
    return self.guestAccount
end

function UserDefaultModel:setDataByKey(key, value)
    -- body
    local fileUtils = cc.FileUtils:getInstance();
    local fileName = model.UserModel:SelfSettingConfig();
    local settingConfig = {};
    if fileUtils:isFileExist(fileName) then
        local fileData = fileUtils:getStringFromFile(fileName);
        if fileData ~= '' then
            settingConfig = Cjson.decode(fileData);
        end
    end

    settingConfig[key] = value
    -- print('fubin---settingconfig:', key, value)
    print_ro(settingConfig)
    fileUtils:writeStringToFile(Cjson.encode(settingConfig), fileName);
end

function UserDefaultModel:getDataByKey(key)
    -- body
    if model.UserModel:getUserLevel() > 2 then
        if key== 'remind_FightPanelGotoView' or key=='remind_SkillCastTemplateNew' then
            return 1;
        end
    end
    local fileUtils = cc.FileUtils:getInstance();
    local fileName = model.UserModel:SelfSettingConfig();
    local settingConfig = {};
    if fileUtils:isFileExist(fileName) then
        local fileData = fileUtils:getStringFromFile(fileName);
        if fileData ~= '' then
            settingConfig = Cjson.decode(fileData);
        end
    end
    return settingConfig[key];
end

function UserDefaultModel:getDefaultServer()
    if self.defaultServerID == 0 then
        self.defaultServerID = cc.UserDefault:getInstance():getIntegerForKey(DEFAULT_SER_id_KEY);
    end
    if self.defaultServerName == '' then
        self.defaultServerName = cc.UserDefault:getInstance():getStringForKey(DEFAULT_SER_NAME_KEY);
    end
    return self.defaultServerID, self.defaultServerName
end

function UserDefaultModel:setDefaultServer(id, name)
    if not id or not name then
        return
    end
	cc.UserDefault:getInstance():setIntegerForKey(DEFAULT_SER_id_KEY, id);
	cc.UserDefault:getInstance():setStringForKey(DEFAULT_SER_NAME_KEY, name);
	cc.UserDefault:getInstance():flush();
    self.defaultServerID = id
    self.defaultServerName = name
	model.UserModel:setServer(id)
end

function UserDefaultModel:getFunnel(id)
    if not self.funnel then
        self.funnel = {}
    end    
    id = tonumber(id)
    if not self.funnel[id] then
        local key = FUNEEL .. id
        self.funnel[id] = cc.UserDefault:getInstance():getIntegerForKey(key, 0);
    end
    return (self.funnel[id] == 1)
end

function UserDefaultModel:setFunnel(id)
    if not self.funnel then
        self.funnel = {}
    end    
    self.funnel[id] = 1
    local key = FUNEEL .. id
	cc.UserDefault:getInstance():setIntegerForKey(key, 1);
end

return UserDefaultModel;
