--当前用户信息
local UserModel = class("UserModel")
local UserStatus = Enum:getTable('UserStatus');
local cvTypeEnum = Enum:getTable("cvTypeEnum");
local KanbanType = Enum:getTable("KanbanType");

local DEF_MAIN_BG_ID = 1;

function UserModel:ctor()
	self:clearAll()
end

function UserModel:clearAll(  )
    self.user = {
		exp = 0,
		level = 1,
		name = '',
		uid = 1,
		kanbanType = 1,
		kanban = 13,
		headIcon = 13,
		mainBg = DEF_MAIN_BG_ID,
		ui = 1,
		top_power = 0,
		guide_no = 1,
		guide_weak_complete = {},
		createTime = 0,
		mainBgList = {},
		cvTypeIndex = {};
		curCvType = cvTypeEnum.CV_NULL;
		loginDays = 1,
		pop_notice_today = 0,
		noticeId = 0;
	};
end
function UserModel:Init(msg)
	self:clearAll()

	self:setUserData(msg)

	model.GuideModel:Init(self.user.guide_no,self.user.guide_weak_complete,self.user.level);
end

function UserModel:getUserData()
    return self.user
end 

function UserModel:setUserData(msg)
	if msg.exp then
		self.user.exp = msg.exp 
	end
	if msg.level then
		self.user.level = tonumber(msg.level)
	end
	if msg.name then
		self.user.name = msg.name
	end
	if msg.uid then
		self.user.uid = msg.uid
	end
	--if msg.kanban then
	--	self.user.kanban = msg.kanban
	--end
	if msg.head_icon then
		self.user.headIcon = msg.head_icon
	end
	--if msg.mainBg then
	--	self.user.mainBg = msg.mainBg
	--end
	if msg.ui then
		self.user.ui = msg.ui
	end
	if msg.top_power then
		self.user.top_power = msg.top_power
	end
	if msg.guide_no then
		self.user.guide_no = msg.guide_no
	end
	if msg.guide_weak_complete then
		self.user.guide_weak_complete = msg.guide_weak_complete
	end

	if msg.created then
		self.user.createTime = msg.created;
	end
	if msg.deviceToken then
		self.user.deviceToken = msg.deviceToken
	end

	if msg.userStatus and msg.userStatus.status then
		self.user.userStatus = msg.userStatus.status
	else
		self.user.userStatus = UserStatus.NORMAL
	end
	if msg.userStatus and msg.userStatus.time then
		self.user.userStatusTime = msg.userStatus.time
	else
		self.user.userStatusTime = 0
	end
	if msg.pop_notice_today then
		self.pop_notice_today = msg.pop_notice_today;
	end

	if msg.login_days then
		self.user.loginDays = msg.login_days;
	end

	if msg.noticeId then
		self.user.noticeId = msg.noticeId;
	end

	self:initBg(msg);
end 

function UserModel:getKanbanRoleAndAvatar(  )
	local kType , no = self:getKanban();

	if not kType then
		kType = 1;
	end

	if not no then
		no = 13;
	end

	local roleId,avatarId = nil,nil;

	if kType == KanbanType.KANBAN_ROLE then
		roleId = no;
	elseif kType == KanbanType.KANBAN_AVATAR then
		roleId = ResTable:getValue('role_avatar' , no , 'role_id');
		avatarId = no;
	elseif kType == KanbanType.KANBAN_LOVE then
		roleId = ResTable:getValue('role_love' , no , 'role_id');
	elseif kType == KanbanType.KANBAN_BARE then
		roleId = ResTable:getValue('bare_collect' , no , 'role_id');
	end

	return roleId,avatarId;
end

function UserModel:getKanbanTypeAndResourceId(  )
	local kType , no = self:getKanban();

	if not kType then
		kType = 1;
	end

	if not no then
		no = 13;
	end

	local resourceId = nil;
	if kType == KanbanType.KANBAN_ROLE then
		resourceId = ResTable:getValue('role' , no , 'resource_id');
	elseif kType == KanbanType.KANBAN_AVATAR then
		resourceId = ResTable:getValue('role_avatar' , no , 'resource_id');
	elseif kType == KanbanType.KANBAN_LOVE then
		local roleId = ResTable:getValue('role_love' , no , 'role_id');
		resourceId = ResTable:getValue('role' , roleId , 'resource_id');
	elseif kType == KanbanType.KANBAN_BARE then
		resourceId = ResTable:getValue('bare_collect' , no , 'resource_id');
	end

	return kType,resourceId, no;
end

function UserModel:getCvTypeIndex( roleID,avatarID,cvType,defaultIndex )
	-- print('=====UserModel ==== cvType === ',cvType,'.... defaultIndex == ',defaultIndex);
	local curIndex = 1;
	if self.user == nil then
		return curIndex,cvType;
	end
	if self.user.cvTypeIndex == nil then
		self.user.cvTypeIndex = {}
	end

	if cvType ~= cvTypeEnum.CV_TOUCH1 then
		-- 如果是皮肤，则TOUCH类型特殊处理
		avatarID = nil;
	end

	-- local hour = TimeManager:getCurHour();
	local curTime = os.time(); -- 此处取本地时间
	local hour = tonumber(os.date("%H",curTime));

	curIndex = self.user.cvTypeIndex[cvType] or 0;
	if cvType == cvTypeEnum.CV_TOUCH1  then
		if hour >= 6 and hour < 10  then
			cvType = cvTypeEnum.CV_TOUCH2
		elseif (hour >= 22 and hour < 24) or (hour >= 0 and hour < 2) then 
			cvType = cvTypeEnum.CV_TOUCH3
		else
			-- do nothing
		end
		curIndex = self.user.cvTypeIndex[cvType] or 0
		curIndex = curIndex + 1;

	elseif cvType == cvTypeEnum.CV_HOME_LOGIN1 then
		--进入主界面：首次登录，按当前时间选择播放[8-10,12-14,16-18,22-24]
		if hour >= 6 and hour < 10 then
			cvType = cvTypeEnum.CV_HOME_LOGIN2
		elseif hour >= 12 and hour < 14 then 
			cvType = cvTypeEnum.CV_HOME_LOGIN3
		elseif hour >= 16 and hour < 18 then 
			cvType = cvTypeEnum.CV_HOME_LOGIN4
		elseif (hour >= 22 and hour < 24) or (hour >= 0 and hour < 2) then 
			cvType = cvTypeEnum.CV_HOME_LOGIN5
		else
			-- do nothing
		end
		curIndex = self.user.cvTypeIndex[cvType] or 0
		--[[
		if curIndex ~= 0 and cvType ~= cvTypeEnum.CV_HOME_LOGIN1 then 
			-- 本次游戏，非首次点击
			cvType = cvTypeEnum.CV_HOME_LOGIN1;
			curIndex = self.user.cvTypeIndex[cvType] or 0
		end
		]]
		curIndex = curIndex + 1;

	elseif cvType == cvTypeEnum.CV_GIFT then 
		-- gift
		curIndex = defaultIndex; -- 顺序播放
	elseif cvType == cvTypeEnum.CV_PURIFIED then 
		-- PURIFIED 净化 此处默认音效数量为5
		local curPurifiedIndex = defaultIndex; 
		if curPurifiedIndex == nil or curPurifiedIndex > 12  then
			curIndex = math.random( 1,12)
		else
			curIndex = curPurifiedIndex
		end
	end

	local cvFileID = roleID;
	if avatarID ~= nil then
		cvFileID = avatarID
	end
	local checkHave = false 
	local cvFullName = AudioManager:getCVFileName( cvFileID,cvType,curIndex )
	if not cc.FileUtils:getInstance():isFileExist( cvFullName ) then
		curIndex = 1
	else
		checkHave = true;
	end
	if checkHave == false then
		local defaultCvType = nil 
		if  cvType > cvTypeEnum.CV_HOME_LOGIN1 and cvType <= cvTypeEnum.CV_HOME_LOGIN5 then
			defaultCvType = cvTypeEnum.CV_HOME_LOGIN1
		elseif  cvType > cvTypeEnum.CV_TOUCH1 and cvType <= cvTypeEnum.CV_TOUCH3 then 
			defaultCvType = cvTypeEnum.CV_TOUCH1
		end
		cvFullName = AudioManager:getCVFileName( cvFileID,cvType,curIndex )
		if defaultCvType ~= nil and (not cc.FileUtils:getInstance():isFileExist( cvFullName )) then
			cvType = defaultCvType
			curIndex = self.user.cvTypeIndex[cvType] or 0
			curIndex = curIndex + 1;
			cvFullName = AudioManager:getCVFileName( cvFileID,cvType,curIndex )
			if not cc.FileUtils:getInstance():isFileExist( cvFullName ) then
				curIndex = 1;
			end
		end
	end

	self.user.cvTypeIndex[cvType] = curIndex;
	self.user.curCvType = cvType;
	return cvFileID,curIndex,cvType;
end

function UserModel:getLoveCvIndex(roleId,loveLv,cvType)
	-- local curTime = os.time(); -- 此处取本地时间
	-- local hour = tonumber(os.date("%H",curTime));
	-- if hour >= 8 and hour < 10  then
	-- 	cvType = cvTypeEnum.CV_TOUCH2
	-- elseif hour >= 22 and hour < 24 then 
	-- 	cvType = cvTypeEnum.CV_TOUCH3
	-- else
	-- 	-- do nothing
	-- end
	local curIndex = self.user.cvTypeIndex[cvType] or 0
	curIndex = curIndex + 1;

	local cvFullName = AudioManager:getLoveCvFileName(roleId,loveLv,curIndex )
	if not cc.FileUtils:getInstance():isFileExist( cvFullName ) then
		curIndex = 1
	end
	self.user.cvTypeIndex[cvType] = curIndex;
	self.user.curCvType = cvType;
	
	return curIndex,cvType;
end

function UserModel:getCurrentCvIndex( cvType )

	if self.user == nil or self.user.cvTypeIndex == nil then
		return 1;
	end
	if cvType == nil then
		cvType = self.user.curCvType;
	end

	return self.user.cvTypeIndex[cvType] or 1,cvType

end
function UserModel:getRoleCvInfo( resourceId,cvType )
	local cvIndex,curCvType = self:getCurrentCvIndex(cvType)
	local info = '';

	if curCvType == cvTypeEnum.CV_NULL or 
		curCvType == cvTypeEnum.CV_GIFT or 
		curCvType == cvTypeEnum.CV_PURIFIED or 
		curCvType == cvTypeEnum.CV_BROKEN or 
		curCvType == cvTypeEnum.CV_BROKEN_FIGHT_OVER or 
		curCvType == cvTypeEnum.CV_BROKEN_FIGHT or 
		curCvType == cvTypeEnum.CV_HIT or 
		curCvType == cvTypeEnum.CV_SKILL or 
		curCvType == cvTypeEnum.CV_UNABLE
	then
		return info ,cvTypeEnum.CV_NULL;
	end

	--if avatarId ~= nil then
	--	if  curCvType == cvTypeEnum.CV_TOUCH1 or
	--		curCvType == cvTypeEnum.CV_TOUCH2 or
	--		curCvType == cvTypeEnum.CV_TOUCH3 then 
--
	--		roleId = avatarId;
	--	end
	--end
	local cvData = ResTable:getRowData('role_resource', resourceId);
	if cvData == nil then
		return info;
	end
	local cvId = tostring(cvData['cv_id']);
	-- print('cvId=== ',cvId,'   curCvType == ',curCvType,'   cvIndex === ',cvIndex)
	local cvData = ResTable:getMultiValue('role_cv', cvId,curCvType,cvIndex);
	if cvData == nil then
		print('error =================  not found role_cv with roleId:',roleId)
		return info;
	end
	info = cvData['language'];
	if info == nil then
		print(' error =============== not found key[language] in table [role_cv]');
		info = '';
	end

	return info;
end

function UserModel:getCurrentCVInfo( resourceId ,cvType )
	local kanbanType = nil;
	if resourceId == nil then
		kanbanType,resourceId = self:getKanbanTypeAndResourceId()
	end

	local cvInfo = self:getRoleCvInfo(resourceId,cvType)

	-- local cvData = ResTable:getRowData('role_resource', roleId);
	-- if cvData ~= nil  then
	-- 	local naviShow = cvData['navi_show'];
	-- 	if naviShow == nil or next(naviShow) == nil then
	-- 		naviShow = {0,0}
	-- 	end
	-- 	cvPos = cc.pAdd(cvPos,cc.p(naviShow[1],naviShow[2]))
	-- end

	return cvInfo;

end

function UserModel:InitLoveCvInfo(dialogId)
	if not self.loveDialog then
		self.loveDialog = {};
	end
	if not self.loveDialog[dialogId] then
		local loveDialog = {
			data = {};
			startId = 0;
		};
		local startId = nil;
		local dialogLove = ResTable:getTable('dialog_love');
		for _, data in pairs(dialogLove.contain) do
			if data['task_id'] == dialogId then
				local id = data['no'];
				loveDialog.data[id] = data;
				if not startId then
					startId = id;
				elseif id < startId then
					startId = id;
				end
			end
		end
		loveDialog.startId = startId;
		self.loveDialog[dialogId] = loveDialog;
	end
	self.curDialogId = dialogId;
	return self.loveDialog[dialogId].startId
end
function UserModel:getLoveCvInfo(nextTaskId)
	if self.loveDialog[self.curDialogId] and self.loveDialog[self.curDialogId].data[nextTaskId] then
		local info = self.loveDialog[self.curDialogId].data[nextTaskId].main_plot;
		nextTaskId = self.loveDialog[self.curDialogId].data[nextTaskId].follow and self.loveDialog[self.curDialogId].data[nextTaskId].follow[1] or nil;
		return info,nextTaskId;
	end
	return '',nil;
end

function UserModel:SelfSettingConfig()
	return device.writablePath.. '/'..self.user.uid ..'_setting_config.json';
end

function UserModel:initBg(msg)
	local curMainBg = 1;
	local mBgUsed = {};
	local settingConfig = {}
	local fileUtils = cc.FileUtils:getInstance();
	local fileName = self:SelfSettingConfig();
	if fileUtils:isFileExist(fileName) then
		local fileData = fileUtils:getStringFromFile(fileName);
		if fileData ~= '' then
			settingConfig = Cjson.decode(fileData);
			self.user.kanbanType = settingConfig.kanbanType or self.user.kanbanType;
			self.user.kanban = settingConfig.kanban or self.user.kanban;
			--self.user.headIcon = settingConfig.headIcon or self.user.headIcon;
			curMainBg = settingConfig.mainBg or curMainBg;
			mBgUsed = settingConfig.mainBgUsed or {};
		end
	end

	local mainBgUsed = {};
	for _k , bgId in pairs(mBgUsed) do
		mainBgUsed[bgId] = 1;
	end

	local curMainBgUsed = {[DEF_MAIN_BG_ID] = 1};
	local bgActivate = {};
	if msg.BgActivate then
		for _k , bg in pairs(msg.BgActivate) do
			local bgId = bg[1];
			bgActivate[bgId] = bg;
			if curMainBg == bgId then
				self.user.mainBg = bgId;
			end

			if mainBgUsed[bgId] then
				curMainBgUsed[bgId] = 1;--使用过的
			end
		end
	end

	self.user.mainBgList = {};
	local bgData = ResTable:getTable('background').contain;
	for _k , data in pairs(bgData) do

		self.user.mainBgList[data.id] = data;
		if data.id == DEF_MAIN_BG_ID then
			self.user.mainBgList[data.id].activateTime = 0;
			self.user.mainBgList[data.id].newState = 0;
			self.user.mainBgList[data.id].limitTime = 0;
		else
			local bgActivateData = bgActivate[data.id];
			if bgActivateData then
				self.user.mainBgList[data.id].newState = curMainBgUsed[data.id] and 0 or 1;
			else
				self.user.mainBgList[data.id].newState = 0;
			end
			self.user.mainBgList[data.id].activateTime = bgActivateData and bgActivateData[2] or -1;
			self.user.mainBgList[data.id].limitTime = bgActivateData and bgActivateData[3] or 0;
		end
	end

	settingConfig.kanbanType = self.user.kanbanType;
	settingConfig.kanban = self.user.kanban;
	--settingConfig.headIcon = self.user.headIcon;
	settingConfig.mainBg = self.user.mainBg;
	settingConfig.mainBgUsed = table.keys(curMainBgUsed);
	fileUtils:writeStringToFile(Cjson.encode(settingConfig), fileName);
end

function UserModel:setUserStatus(msg)
	if msg.userStatus and msg.userStatus.status then
		self.user.userStatus = msg.userStatus.status
	else
		self.user.userStatus = UserStatus.NORMAL
	end
	if msg.userStatus and msg.userStatus.time then
		self.user.userStatusTime = msg.userStatus.time
	else
		self.user.userStatusTime = 0
	end
	if self.user.userStatus == UserStatus.BANNED then		
		local MessageBoxType = Enum:getTable('MessageBoxType')
		--[[
		local banTime = self.user.userStatusTime
		local t = os.date('*t', banTime)
		local timeStr = string.format(LANG.TIME_FORMAT, t.year, t.month, t.day, t.hour, t.min)
		local str = string.format(LANG.USER_STATUS_BANNED, tostring(timeStr))
		MessageBox:create(str, MessageBoxType.OK, function()
			cc.Director:getInstance():endToLua();
		end, function()
		end, function()
		end);
		--]]
		MessageBox:create(LANG.VERSION_UPDATE_10, MessageBoxType.CANCEL_OK, function()
			local user_app_id = model.AccountModel:getAccountUid();
			local nickname = model.UserModel:getUserName();
			local device_info = model.AccountModel:getDeviceModel()
			local game_ver = model.AccountModel:getGameVersion();
			local os_ver = model.AccountModel:getOSVersion()
			local platform = model.AccountModel:getGameCode();
			local game_server = model.UserModel:getServer();
			local url = GlobalFunctions:getURLbyLanguage()..'&user_app_id={UserID}&nickname={NickName}&device_info={DeviceInfo}&game_ver={Version}&os_ver={OSVersion}&platform={PlaformCode}&game_server={Servername}'
			local url = string.format(GlobalFunctions:getURLbyLanguage()..'&user_app_id=%s&nickname=%s&device_info=%s&game_ver=%s&os_ver=%s&platform=%s&game_server=%s',
			tostring(user_app_id),tostring(nickname),tostring(device_info),tostring(game_ver),tostring(os_ver),tostring(platform),tostring(game_server));
			cc.Application:getInstance():openURL(url)
			cc.Director:getInstance():endToLua();
		end, function()
			cc.Director:getInstance():endToLua();
		end, function()
			cc.Director:getInstance():endToLua();
		end);
		return;
	end
end

function UserModel:getUserStatus()
	return self.user.userStatus, self.user.userStatusTime;
end

function UserModel:getUserTopPower(  )
	return self.user.top_power;
end
function UserModel:setUserTopPower( newPower )
	self.user.top_power = newPower
end

function UserModel:getUserLevel()
	return self.user.level;
end
function UserModel:getUserName()
	return self.user.name
end
function UserModel:getUserExp()
	return self.user.exp;
end

function UserModel:getUserLoginDays()
	return self.user.loginDays
end

function UserModel:getExpPercent()
	local maxExp = ResTable:getValue('user_lv', self.user.level, 'exp');
	local percent = 0;
	if self.user.level == 1 then
		percent = math.floor(self.user.exp / maxExp * 100);
	else
		local prevExp = ResTable:getValue('user_lv', self.user.level-1, 'exp');
		percent = math.floor((self.user.exp-prevExp) / (maxExp-prevExp) * 100);
	end
	return percent;
end

function UserModel:getDeviceToken(  )
	if self.user == nil then
		return 0
	end
	return self.user.deviceToken or 0;
end

function UserModel:getUserUid()
	return self.user.uid;
end

function UserModel:setServer(server)
	self.server = server;
end

function UserModel:getIsPopNotice()
	return self.pop_notice_today;
end

function UserModel:getServer()
	return self.server;
end

function UserModel:getKanban()
	return self.user.kanbanType , self.user.kanban;
end

function UserModel:setKanban(kType , no)
	if self.user.kanbanType ~= kType or self.user.kanban ~= no then
		-- 重置CV播放序列
		for i=cvTypeEnum.CV_TOUCH1,cvTypeEnum.CV_TOUCH3 do
			self.user.cvTypeIndex[i] = 0;
		end
		for i=cvTypeEnum.CV_HOME_LOGIN1,cvTypeEnum.CV_HOME_LOGIN5 do
			self.user.cvTypeIndex[i] = 0;
		end
	end
	self.user.kanbanType = kType;
	self.user.kanban = no;
end

function UserModel:setHeadIcon(no)
	self.user.headIcon = no
end

function UserModel:getHeadIcon()
	return self.user.headIcon
end

function UserModel:setMainBg(bgId)
	
	local fileUtils = cc.FileUtils:getInstance();
	local fileName = self:SelfSettingConfig();
	local settingConfig = {};
	local useMainBg = {};
	if fileUtils:isFileExist(fileName) then
		local fileData = fileUtils:getStringFromFile(fileName);
		if fileData ~= '' then
			settingConfig = Cjson.decode(fileData);
			useMainBg = settingConfig.mainBgUsed;
		end
	end
	
	local isUsed = false;
	for _k , useBgId in pairs(useMainBg) do
		if useBgId == bgId then
			isUsed = true;
			break;
		end
	end
	if not isUsed then
		table.insert(useMainBg , bgId);
	end
	
	settingConfig.mainBg = bgId;
	settingConfig.mainBgUsed = useMainBg;
	fileUtils:writeStringToFile(Cjson.encode(settingConfig), fileName);

	self:updateMainBgActivateNewState(bgId , 0)

	self.user.mainBg = bgId;
end

function UserModel:getMainBgList(bgId)
	if bgId then
		return self.user.mainBgList[bgId];
	end
	return self.user.mainBgList;
end

function UserModel:getMainBg()
	local bgId = self.user.mainBg;
	local bgData = self.user.mainBgList[bgId];
	if bgData.activateTime > 0 then
		local lastTime = bgData.activateTime - TimeManager:getCurTime();
		if lastTime <= 0 then
			self:setBgTimeOut(bgId);
		end
	elseif bgData.activateTime < 0 then
		self:setMainBg(1);
	end
	return self.user.mainBg
end

function UserModel:addMainBg(bgList)
	if bgList then
		for _k , data in pairs(bgList) do
			self:updateMainBgActivateTime(data.bg_id , data.activate_time)
			self:updateMainBgActivateLimitTime(data.bg_id , data.limit_time)
		end
	end
end

function UserModel:setBgTimeOut(bgId)
	self:updateMainBgActivateTime(bgId , -1);
	self:setMainBg(1)
end

function UserModel:updateMainBgActivateTime(bgId , activateTime)
	if self.user.mainBgList[bgId] then
		self.user.mainBgList[bgId].activateTime = activateTime;
	end
end
function UserModel:updateMainBgActivateNewState(bgId , newState)
	if self.user.mainBgList[bgId] then
		self.user.mainBgList[bgId].newState = newState;
	end
end

function UserModel:updateMainBgActivateLimitTime(bgId , limitTime)
	if self.user.mainBgList[bgId] then
		self.user.mainBgList[bgId].limitTime = limitTime;
	end
end

-- 订阅/退订 推送主题事件
-- return true/false
function UserModel:changePushState( newState, callbacklua )

	local pushState = model.UserDefaultModel:getPushEventState()
	if newState == pushState then
		callbacklua(newState)
		return 
	end

	local PushType = Enum:getTable("PushType");
	local platform = cc.Application:getInstance():getTargetPlatform()

	local subsEventAll = PushType.PUSH_EVENT_11
	local subsEventDay = PushType.PUSH_EVENT_12
	-- 后续可能根据包名区分推送主题
	if (cc.PLATFORM_OS_IPHONE == platform) then
		subsEventAll = PushType.PUSH_EVENT_21
		subsEventDay = PushType.PUSH_EVENT_22
	end
	
	local function sdkCallback( strData )
		
		local jsonData = Cjson.decode(strData)
		local bTrue = jsonData.code == 0;
		-- local state2 = jsonData.state == 1;
		--print("===============bTrue === ",bTrue,'   state2 == ',state2);
		if bTrue then
			model.UserDefaultModel:setPushEventState(newState)
		end
		if callbacklua then
			callbacklua(newState)
		end
	end
	

	if newState == 0 then
		-- 取消所有推送
		local args = {
			state = 0,
			name = subsEventAll
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,sdkCallback)

		args = {
			state = 0,
			name = subsEventDay
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,nil)


	elseif newState == 1 then 
		-- 开启 all，关闭 day
		local args = {
			state = 1,
			name = subsEventAll
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,sdkCallback)

		args = {
			state = 0,
			name = subsEventDay
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,nil)
	else -- newState == 2
		-- 关闭all,开启day
		local args = {
			state = 0,
			name = subsEventAll
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,sdkCallback)

		args = {
			state = 1,
			name = subsEventDay
		}
		sdkManager.SDKLuaHelper:changeSubscriptionStatus(args,nil)
	end

end

function UserModel:setUI(no)
	self.user.ui = no;
end

function UserModel:getUI()
	return self.user.ui;
end

function UserModel:getCreateTime()
	return self.user.createTime
end

function UserModel:setUserLevelAndExp(level , exp)
	if level then
		if tonumber(level) ~= self.user.level then
			EventManager:FireEvent(EventType.ModelUserLevelUp)
		end
		self.user.level = tonumber(level);
	end
	if exp then
		self.user.exp = tonumber(exp);
	end
end
function UserModel:updateUserLevelAndExp(data)
	if not data or not data.level or not data.exp then
		return;
	end

	-- if data.level > self.user.level then
	-- 	LevelUpPanel:Show(self.user.level , data.level)
	-- 	EventManager:FireEvent(EventType.UIActivityBtn , 2);
	-- end
	self:setUserLevelAndExp(data.level , data.exp);
	EventManager:FireEvent(EventType.UIHeaderPanelUpdateLevel , self:getExpPercent());
	EventManager:FireEvent(EventType.UIEventUnlockChange)

	model.TeamModel:updateFormationState();
	model.ActivityModel:updateRedDotStateLevelGift();
	model.GuideModel:initCheckGuide();
	model.RechargeModel:updateGrowupRedDot()

end

function UserModel:reqCDKGift( cdkey )
	local msg = {};
    msg.cdkey   =   cdkey;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_user_cdk_gift, msg);
end
function UserModel:checkTopPower( newFp )
	
	local userTopPower = self:getUserTopPower()
	local firstFp = ConvertDigitUnitToStr(tostring(userTopPower));
	local secondFp = ConvertDigitUnitToStr(tostring(newFp));

	if HpCompare(secondFp , '>' , firstFp) then
		self:reqSaveTopPower(secondFp);
		self:setUserTopPower(secondFp);
	end
	model.ActivityModel:updateRedDotStateFpGift();
end

function UserModel:reqSaveTopPower( newFp )
	local msg = {};
    msg.power   =   newFp;
	Network.NetworkManager:Send(Network.NetworkCmdType.req_save_team_top_power, msg);

end

function UserModel:reqLevelUp(  )
	local msg = {};
	Network.NetworkManager:Send(Network.NetworkCmdType.req_user_level_up, msg);

end
--- 等级解锁 -------
function UserModel:getLevelLockList(  )
	if self.levelLockList == nil  then
		--self.levelLockList = GlobalFunctions:sortTableByKey(ResTable:getTable('start').contain);
		self.levelLockList = {};
		for _k , data in pairs(ResTable:getTable('start').contain) do
			table.insert(self.levelLockList , data);
		end

		if #self.levelLockList > 1 then
			table.sort(self.levelLockList , function(first , second)
				if first.lv == second.lv then
					return first.id < second.id;
				else
					return first.lv < second.lv
				end
				return false;
			end);
		end
	end
	return self.levelLockList
end

function UserModel:getLevelLockItemById( id )
	--self:getLevelLockList()

	return ResTable:getRowData('start',id);--self.levelLockList[id];
end
-- ret 对应等级解锁项 或者 最近未解锁项
-- ret id,lv
function UserModel:getUnlockIdByLv( lv )
	self:getLevelLockList()

    local index = 1;
    local item = self.levelLockList[index];
    local itemlv = nil;
    local bTrue = true;
    while bTrue and item do
        itemlv = item.lv;
        if itemlv >= lv then
            bTrue = false;
            return item.id,item.lv;
        else
            index = index + 1;
            item = self.levelLockList[index];
        end
    end
    return -1,lv;
end

function UserModel:getUnlockIdsByLv( lv )
	self:getLevelLockList()
	local ret = {}
	for k,v in pairs(self.levelLockList) do
		if v.lv == lv then
			table.insert(ret, v.id )
		end
	end
	return ret;
end



return UserModel;
