--resTable.lua

--========================================================================
--资源表格
local ResTable = class("ResTable")
-- local Cjson = require "Cjson"

function ResTable:ctor()
	self.table_index = 
	{
		--errors				= 0, -- 错误表
		item				= 1, -- 物品表
		reward				= 2, -- 委托表
		reward_condition	= 3, -- 委托状态表
		--dialog				= 4, -- 任务对话表
		npc					= 5, -- 任务对话用NPC表
		item_bag			= 6, -- 背包页与物品的对应表
		role				= 7, -- 角色
		role_resource		= 8, -- 角色资源
		role_skill			= 9, -- 角色技能
		role_rank			= 10, -- 角色突破
		role_rank_cost		= 11, -- 角色突破消耗
        role_info			= 12, -- 角色档案
        attribute_lv_exp	= 13, -- 角色强化
		attribute_grow		= 14, -- 角色强化成长
        attribute_exp_value = 15, -- 角色强化经验价值
		skill				= 16, -- 技能
        goldmine			= 17, -- 金矿
		oilwell				= 18, -- 油矿
		role_lv_attribute	= 19, -- 角色升级属性
        recharge			= 20, -- 商店:彩钻
        recharge_free		= 21, -- 商店:蓝钻
        shop				= 22, -- 补给
		shop_gift			= 23, -- 商店:礼盒
		role_lv				= 24, -- 角色升级
		equip_lv_cost		= 25, -- 装备升级消耗
		equip_role			= 26, -- 角色换装
		equip				= 27, -- 装备配置
		shop_discount		= 28, -- 军火库折扣表
		shop_renovate		= 29, -- 军火库物品表
		dailyround			= 30, -- 每日副本
		dailyround_dif		= 31, -- 每日副本难度
		round				= 32, -- 副本
		monster				= 33, -- 怪物
		monster_attribute	= 34, -- 怪物属性
		chapter				= 35, -- 章节信息
		skill_lv			= 36, -- 技能等级对应经验
		skill_lv_item		= 37, -- 技能等级对应消耗物品
		contract_bag		= 38, -- 抽卡
		contract_gain		= 39, -- 卡牌兑换
		role_retired		= 40, -- 角色退役
		equip_resolve		= 41, -- 装备拆解
		item_compose		= 42, -- 物品合成
		buff				= 43, -- buff
		role_remodeling		= 44, -- 改造信息
		event_dialog		= 45, -- 活动剧情
		event_mission		= 46, -- 活动关卡
		event_mission_type	= 47, -- 活动
		event_reward		= 48, -- 活动奖励
		guild_mission		= 49, -- 公会副本关卡
		guild_mission_type	= 50, -- 公会副本
		daily_mission		= 51, -- 每日任务
		daily_bonus_mission	= 52, -- 每日任务
		event_daily_mission = 53, -- 活动任务
		role_promotion 		= 54, -- 觉醒
		promotion_attribute_order = 55, -- 觉醒
		promotion_lv_up		= 56, --觉醒
		loginmaster			= 57, --签到
		lv_guide			= 58, --升级
		role_fetter			= 59, --羁绊
		role_fetter_exp		= 60, --羁绊升级
		guild_skill			= 61, --公会技能
		equip_rank			= 62, --装备合成
		user_lv				= 63, --用户等级
		role_break_attribute= 64, --角色突破
		dialog_role			= 65, --角色对话
		equip_effect		= 66, --装备阵营加成
		contract_reward		= 67, --抽卡奖励
		role_pre            = 69,

		achievement			= 70, --成就
		round_bg			= 71, --关卡背景
		unit				= 72, --大位数数字单位

		guild_boss			= 73,	--工会副本boss

		skill_effect		= 74,
		skill_script		= 75,
		round_rewardbuff	= 76, --驻守
		round_boss			= 77, --随机boss
		round_bossinfo		= 78, --随机boss

		start				= 79,

		role_avatar			= 80, -- 角色皮肤
		bare_collect		= 81,
		altar				= 82,
	}

	self.multiKeyTable = {
		role_remodeling		= handler(self, self.handlerRoleRemodel),
		event_drop			= handler(self, self.handlerEventDrop),
		event_buff_effect	= handler(self, self.handlerEventBuffEffect),
		guild_mission		= handler(self, self.handlerGuildMission),
		promotion_attribute_order = handler(self, self.handlerPromotionAttributeOrder),
		promotion_lv_up		= handler(self, self.handlerPromotionLvUp),
		role_promotion		= handler(self, self.handlerPromotionRole),
		loginmaster			= handler(self, self.handlerLoginmaster),
		role_fetter			= handler(self, self.handlerRoleFetter),
		role_fetter_exp		= handler(self, self.handlerRoleFetterExp),
		guild_skill			= handler(self, self.handlerGuildSkill),
		equip_rank			= handler(self, self.handlerEquipRank),
		role_break_attribute= handler(self, self.handlerRoleBreakAttribute),
		base_skill_lv		= handler(self, self.handlerBaseSkillLv),
		equip_lv_cost		= handler(self, self.handlerEquipLvCost),
		event_hard			= handler(self, self.handlerEventHard),
		role_lv				= handler(self, self.handlerRoleLv),
		role_star_attribute = handler(self, self.handlerRoleStarAttribute),
		role_grade_attribute= handler(self, self.handlerRoleGradeAttribute),
		arena_reward        = handler(self, self.handlerArenaReward),
		role_reset        	= handler(self, self.handlerRoleReset),
		bare_collect        = handler(self, self.handlerBareCollectReset),
		role_cv 	        = handler(self, self.handlerRoleCV),
		shop_buy_icon      	= handler(self, self.handlerShopIconType),
	}

	self.table_replace_field = {
		role_avatar				= {'name','gateway'},
		item					= {'name', 'instruction', 'info'},
		skill					= {'name', 'info'},
		formation				= {'name'},
		formation_attribute		= {'info'},
		lifemission				= {'name', 'descript'},
		battlepass_mission		= {'name', 'descript'},
		contract_bag			= {'name', 'des'},
		background				= {'bg_name'},
		trial_round				= {'info'},
		daily_round				= {'info'},
		arena_rank				= {'name'},

		base_skill_lv			= {'name'},
		achievement				= {'name'},
		robot					= {'name'},
		npc						= {'name', 'title'},

		monster					= {'name'},
		recharge				= {'name'},
		shop_gift				= {'giftname'},
		shop					= {'name'},
		guild_rewardshop		= {'name'},

		shop_renovate			= {'name'},
		operate					= {'descript'},
		event_fp				= {'fp_name'},
		maze_buff				= {'name','info'},
		event_thing				= {'name', 'info'},
		event_shop				= {'name'},

		event_gift				= {'giftname'},
		guide_newer				= {'title', 'description'},
		start					= {'name', 'explain'},
		chapter					= {'name', 'description'},
		bare_collect			= {'get_des'},
		buff					= {'name', 'info'},

		round_bossinfo		    = {'info'},

		month_card 				= {'name'},

		recharge_growup 		= {'name'},
		recharge_growup_sp 		= {'name'},
		maze_thing				= {'name', 'info'},
		trial_round				= {'info'},
		trial_type				= {'name'},
		language_change			= {'name'},
		round_bank_mission		= {'mission_info'},
	}

	self.table_local_index = {
		role_sound			= 1,
	}

	self.table_content = {};
	self.table_isLoad = {};
	self.table_language = {};
	self.languageTableName = '';

	self.special_table = {
		buff = handler(self,self.changeNoToKey), -- 将 No 代替id 作为key,

		role_avatar = handler(self, self.changeAvatarIdToKey), --将 avatar_id 代替 id 作为key,

		maze_box = handler(self,self.changeBoxNoToKey), -- 将 No 代替id 作为key,
	}

	self.mul_langTable = {
		role_info = 1,
		dialog_love	= 1,
		role_cv 	= 1,
	}

	self.resetkey_table = {
		role = 'role_id',
		npc = 'no',
		arena_rank = 'point'
	}
end

-- 重置table状态，
-- 有些数据加载到内存中后会额外添加一些网络字段，切换角色后需要清理
function ResTable:setIsLoadFalse( tableName )
	self.table_isLoad[tableName] = false
end

function ResTable:loadTable(tableName)

	if self.table_replace_field[tableName] and self.languageTableName=='' then
		self:loadLanguageTable();
	end

	local fileUtils = cc.FileUtils:getInstance();

	local tmpTableName = tableName;
	if self.mul_langTable[tableName] then --不同语言使用不同的表
		local ret = model.UserDefaultModel:getLanguageSet();
		if ret == '' then
		    tmpTableName = tmpTableName..'_kr';
		else
		    tmpTableName = tmpTableName..'_'..ret;
		end
	end

	local fileName = device.writablePath..'/' .. tmpTableName .. '.json';
	local localFileName = 'table/' .. tmpTableName .. '.json';

	if fileUtils:isFileExist(fileName) then
		local fileData = fileUtils:getStringFromFile(fileName);
		self.table_content[tableName] = Cjson.decode(fileData);
	elseif fileUtils:isFileExist(localFileName) then
		local fileData = fileUtils:getStringFromFile(localFileName);
		self.table_content[tableName] = Cjson.decode(fileData);
	else
		print("There's no table named "..tableName.." in table");
	end

	self:resetKeyTable(tableName);

	if self.multiKeyTable[tableName] then
		self.multiKeyTable[tableName](self.table_content[tableName]);
	end
	if self.special_table[tableName] then
		self.special_table[tableName](self.table_content[tableName]);
	end

	if self.table_replace_field[tableName] then
		self:replaceTable( tableName );
	end
	self.table_isLoad[tableName] = true;
end

function ResTable:resetKeyTable(tableName)
	if not self.resetkey_table[tableName] then
		return;
	end
	
	local key = self.resetkey_table[tableName];

	local tableData = self.table_content[tableName].contain;
	local newTableData = {};
	for _k , data in pairs(tableData) do
		if data[key] then
			newTableData[data[key]] = data;
		end
	end

	self.table_content[tableName].contain = newTableData;
end

function ResTable:getRowData(tableName, rowID, hideDebug)
	local table_content = self:getTable(tableName);
	--local rowID = tostring(rowID)
	if table_content and rowID then
		if table_content.contain[tostring(rowID)] then
			return table_content.contain[tostring(rowID)];
		elseif table_content.contain[tonumber(rowID)] then
			return table_content.contain[tonumber(rowID)];
		else
			if not hideDebug then
				--print(debug.traceback())
				local errorMsg = "There's no row index named " .. rowID  .. " in table " .. tableName
				print(errorMsg)
				--GlobalFunctions:showErrorPromt(errorMsg);

			end
			return nil;
		end
	end
	return nil
end
--==============================--
--desc:根据key值获取tableName对应列，
-- 同一key下可能有多个列，这里只取第一列
--time:2019-07-30 12:05:36
--@tableName:
--@key:
--@keyValue:
--@return 
--==============================--
function ResTable:getRowDataByKey( tableName,key,keyValue )
	local table_content = self:getTable(tableName);
	if table_content and key and keyValue then
		for k,rowData in pairs(table_content.contain) do
			if rowData[key] and rowData[key] == keyValue then
				return rowData;
			end
		end
	end

	return nil ;

end
function ResTable:getValue(tableName, rowID, key)
	local table_row = self:getRowData(tableName, rowID);
	local key = tostring(key)
	if table_row and key then
		if table_row[key] then
			return table_row[key];
		else
			print("There's no key named " .. key  .. " in table " .. tableName .. " row " .. rowID);
			return nil;
		end
	end
	return nil;
end

function ResTable:getTable(tableName)
	if self.table_isLoad[tableName] then
		if self.table_content[tableName] then
			return self.table_content[tableName];
		else
			-- print("There's no table named " .. tableName .. " in content");
			-- 初始化时加载lang_cn，如果json表不存在，将会循环print，导致mac加载失败
			return nil;
		end
	else
		self:loadTable(tableName);
		return self:getTable(tableName);
	end
end

function ResTable:getMultiValue(tableName, ...)
	if not self.multiKeyTable[tableName] then
		return nil;
	end
	if not self.table_isLoad[tableName] then
		self:loadTable(tableName);
	end
	local order = self.table_content[tableName].contain.order;

	local arg = { ... };
	local count = 0;
	for k, v in pairs(arg) do
		count = count + 1;
	end

	local find = true;
	local breakPoint = -1;
	for i=1, count do
		if order~= nil and order[arg[i]] then
			order = order[arg[i]];
		else
			find = false;
			breakPoint = i;
			break;
		end
	end
	if find then
		return order;
	else
		print("There's no key(" .. breakPoint.. ") named " .. arg[breakPoint] .. " in table " .. tableName);
		return nil;
	end
end

function ResTable:handlerRoleRemodel(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.role_no)] then
			order[tonumber(v.role_no)] = {};
		end
		order[tonumber(v.role_no)][tonumber(v.board_no)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerRoleCV(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.cv_id] then
			order[v.cv_id] = {};
		end
		if not order[v.cv_id][tonumber(v.type)] then
			order[v.cv_id][tonumber(v.type)] = {};
		end
		order[v.cv_id][tonumber(v.type)][tonumber(v.order)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerShopIconType(content)
	local order = {}
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.buytype)] then
			order[tonumber(v.buytype)] = {}
		end
		order[tonumber(v.buytype)][tonumber(v.buyvalue)] = v;
	end
	content.contain.order = order;
end

function ResTable:changeNoToKey(content)
	local contain = content.contain;
	local new_contain = {};
	for k, v in pairs(contain) do
		new_contain[v.no] = v;
	end
	content.contain = new_contain;
end

function ResTable:changeBoxNoToKey(content)
	local contain = content.contain;
	local new_contain = {};
	for k, v in pairs(contain) do
		new_contain[v.box_no] = v;
	end
	content.contain = new_contain;
end

function ResTable:changeAvatarIdToKey(content)
	local contain = content.contain;
	local new_contain = {};
	for k, v in pairs(contain) do
		new_contain[v.avatar_id] = v;
	end
	content.contain = new_contain;
end


function ResTable:handlerRoleSound(content)
	local sound_contain = content.contain;
	for k, v in pairs(self:getTable('role').contain) do
		if tonumber(v.base) ~= -1 then
			sound_contain[tostring(k)] = sound_contain[tostring(v.base)];
		end
	end
end

function ResTable:handlerRoundDailyreward(content)
	local contain = content.contain;
	local new_contain = {};
	for k, v in pairs(contain) do
		new_contain[v.round_id] = v;
	end
	content.contain = new_contain;
end

function ResTable:handlerEventDrop(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.round_no)] then
			order[tonumber(v.round_no)] = {};
		end
		order[tonumber(v.round_no)][tonumber(v.tier)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerEventBuffEffect(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.type)] then
			order[tonumber(v.type)] = {};
		end
		if not order[tonumber(v.type)][v.quality] then
			order[tonumber(v.type)][v.quality] = {};
		end
		if not order[tonumber(v.type)][v.quality][v.color] then
			order[tonumber(v.type)][v.quality][v.color] = {};
		end
		order[tonumber(v.type)][v.quality][v.color][v.attribute] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerEventMission(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.table_no)] then
			order[tonumber(v.table_no)] = {};
		end
		if not order[tonumber(v.table_no)][tonumber(v.chapter_no)] then
			order[tonumber(v.table_no)][tonumber(v.chapter_no)] = {};
		end
		order[tonumber(v.table_no)][tonumber(v.chapter_no)][tonumber(v.stage_no)] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerGuildMission(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.table_no)] then
			order[tonumber(v.table_no)] = {};
		end
		if not order[tonumber(v.table_no)][tonumber(v.chapter_no)] then
			order[tonumber(v.table_no)][tonumber(v.chapter_no)] = {};
		end
		order[tonumber(v.table_no)][tonumber(v.chapter_no)][tonumber(v.stage_no)] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerPromotionAttributeOrder(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.role_type)] then
			order[tonumber(v.role_type)] = {};
		end
		order[tonumber(v.role_type)][tonumber(v.order)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerPromotionLvUp(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.role_quality)] then
			order[tonumber(v.role_quality)] = {};
		end
		if not order[tonumber(v.role_quality)][tonumber(v.role_type)] then
			order[tonumber(v.role_quality)][tonumber(v.role_type)] = {};
		end
		order[tonumber(v.role_quality)][tonumber(v.role_type)][tonumber(v.promotion_lv)] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerPromotionRole(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.role_type)] then
			order[tonumber(v.role_type)] = {};
		end
		order[tonumber(v.role_type)][tonumber(v.promotion_lv)] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerLoginmaster(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.month)] then
			order[tonumber(v.month)] = {};
		end
		order[tonumber(v.month)][tonumber(v.login)] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerRoleFetter(content)
	local order = {};
	for _, v in pairs(content.contain) do
		local startPos = 2;
		local fetter = {};
		while(startPos < #v.fetter-1) do
			local findStartPos = string.find(v.fetter , '%[' , startPos);
			local findEndPos = string.find(v.fetter, '%]' , findStartPos + 1);
			local str = string.sub(v.fetter , findStartPos + 1 , findEndPos - 1);
			local data = string.split(tostring(str), ",")
			table.insert(fetter , {tonumber(data[1]) , tonumber(data[2]) , tostring(data[3])})
			startPos = findEndPos + 1;
		end
		v.fetter = fetter;
		--print_ro(v.fetter);
		for _k , fetter in pairs(v.fetter) do
			if not order[fetter[1]] then
				order[fetter[1]] = {};
			end
			order[fetter[1]][v.id] = v;
		end
	end
	content.contain.order = order;
end
function ResTable:handlerRoleFetterExp(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.exp_id] then
			order[v.exp_id] = {};
		end
		order[v.exp_id][v.level] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerGuildSkill(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.type] then
			order[v.type] = {};
		end
		order[v.type][v.lv] = v;
	end
	content.contain.order = order;
end
function ResTable:handlerEquipRank(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.quality] then
			order[v.quality] = {};
		end
		order[v.quality][v.lv] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerRoleBreakAttribute(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.role_type] then
			order[v.role_type] = {};
		end
		order[v.role_type][v.lv] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerBaseSkillLv(content)
	local order = {};
	local maxLevel = {};
	for _, v in pairs(content.contain) do
		if not order[v.skill_id] then
			order[v.skill_id] = {};
			maxLevel[v.skill_id] = 0;
		end
		if maxLevel[v.skill_id] < v.lv then
			maxLevel[v.skill_id] = v.lv
		end
		order[v.skill_id][v.lv] = v;
	end
	model.HomeModel:setBaseSkillMaxLevel(maxLevel)
	content.contain.order = order;
end
function ResTable:handlerEquipLvCost(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[v.type] then
			order[v.type] = {};
		end
		if not order[v.type][v.quality] then
			order[v.type][v.quality] = {};
		end
		if not order[v.type][v.quality][v.level] then
			order[v.type][v.quality][v.level] = {};
		end
		order[v.type][v.quality][v.level] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerEventHard(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.fp_no)] then
			order[tonumber(v.fp_no)] = {};
		end
		order[tonumber(v.fp_no)][tonumber(v.hard)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerRoleLv(content)
	local order = {};
	local tmp = {}
	
	local maxLevel = model.CardModel:getCardMaxLevel();
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.no)] then
			order[tonumber(v.no)] = {};
			tmp[tonumber(v.no)] = {};
		end
		tmp[tonumber(v.no)][tonumber(v.lv)] = v
	end
	for k, v in pairs(order) do
		local curLevel = 1
		if tmp[k][curLevel] then
			v[1] = tmp[k][curLevel];
		else
			v[1] = {
				no = k,
				lv = 1,
				hp = 0,
				atk = 0,
				cost = '10',
				exp = 0,
			}
		end
		for i=2, maxLevel do
			if tmp[k][i] then
				curLevel = i
			end
			v[i] = {
				no = k,
				lv = i,
				hp = v[i-1].hp + tmp[k][curLevel].hp,
				atk = v[i-1].atk + tmp[k][curLevel].atk,
				cost = HpPlus(ConvertDigitUnitToStr(v[i-1].cost), ConvertDigitUnitToStr(tmp[k][curLevel].cost)),
				exp = v[i-1].exp + tmp[k][curLevel].exp,
			}
		end
	end
	content.contain.order = order
end

function ResTable:handlerRoleStarAttribute(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.no)] then
			order[tonumber(v.no)] = {};
		end
		order[tonumber(v.no)][tonumber(v.star)] = v;
		if CARD_STAR_MAX_LEVEL < v.star then
			CARD_STAR_MAX_LEVEL = v.star;
		end
	end
	content.contain.order = order;
end

function ResTable:handlerRoleGradeAttribute(content)
	local order = {};
	for _, v in pairs(content.contain) do
		if not order[tonumber(v.grade)] then
			order[tonumber(v.grade)] = {};
		end
		order[tonumber(v.grade)][tonumber(v.def_star)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerArenaReward(content)
	content.contain.order = GlobalFunctions:sortTableByKey(content.contain); 
end

function ResTable:handlerRoleReset(content)
	local order = {};
	for _, v in pairs(content.contain) do
		order[tonumber(v.star)] = v;
	end
	content.contain.order = order;
end

function ResTable:handlerBareCollectReset(content)
	local contain = content.contain;
	local new_contain = {};
	for k, v in pairs(contain) do
		new_contain[v.bare_id] = v;
	end
	content.contain = new_contain;
end

function ResTable:loadLanguageTable()
	-- body
	local tableName = 'language_';
	local ret = model.UserDefaultModel:getLanguageSet();
	tableName = tableName..ret;

	print('loadLanguageTable---', tableName)

	self.languageTableName = tableName;
	self:loadTable( self.languageTableName );

end

function ResTable:reloadMulLangTable()
	for k,v in pairs( self.mul_langTable ) do
		self:loadTable( k )
	end
end

function ResTable:replaceTable(tableName)
	if self.table_content[tableName] == nil then
		return;
	end
	local contain = self.table_content[tableName].contain;
	local replaceFields = self.table_replace_field[tableName]
	-- local cnt = #replaceFields;

	-- local idx = 1;
	-- print("fubin===pairs====tableName ", tableName);
	-- print("fubin===pairs====start ", #contain);

	-- for i,v in ipairs(contain) do

	local lang_content = self:getTable(self.languageTableName);
	if not lang_content then
		return;
	end

	for k, v in pairs(contain) do

		-- print("fubin===test==== ", k);
		-- idx = idx + 1;

		for _,key in pairs(replaceFields) do
		-- for j=1,cnt do
			local langIdx = v[ key..'_no' ];
			if langIdx then

				local table_row = nil;
				if lang_content.contain[tostring(langIdx)] then
					table_row = lang_content.contain[tostring(langIdx)];
				elseif lang_content.contain[tonumber(langIdx)] then
					table_row = lang_content.contain[tonumber(langIdx)];
				end
				if table_row and table_row['info'] then
					v[ key ] = tostring( table_row['info'] );
				end

				-- local msg = self:getValue( self.languageTableName, langIdx, 'info' ); --这里找不到内容会有提示框，在登录期间，有提示框 导致登录不进去了，跟协程有关
				-- if msg ~= nil then
				-- 	-- print("fubin===msg==== ", msg);
				-- 	v[ key ] = tostring(msg);
				-- end
			end
		end

	end

	-- print("fubin===pairs====end");


end

function ResTable:getLangByNum( key )
	local lang_content = self:getTable(self.languageTableName);
	if not lang_content then
		return 'Lack dialog: '..key;
	end

	local table_row = nil;
	if lang_content.contain[tostring(key)] then
		table_row = lang_content.contain[tostring(key)];
	elseif lang_content.contain[tonumber(key)] then
		table_row = lang_content.contain[key(langIdx)];
	end
	if table_row and table_row['info'] then
		return tostring( table_row['info'] );
	end

	return 'Lack dialog: '..key;

end

function ResTable:resetReplaceFieldTable()
	
	for tableName,v in pairs(self.table_replace_field) do
		self:replaceTable( tableName );
	end

end

cc.exports.ResTable = ResTable:create();
