local CardStarLevelUpTemplate = class("CardStarLevelUpTemplate")
local fonts = Enum:getTable("SystemFont")

function CardStarLevelUpTemplate:ctor(father)
	self.root = cc.Node:create();
	self.root:setCascadeOpacityEnabled(true)
	father:addChild(self.root);
	
	self.iconNode = cc.Node:create();
	self.iconNode:addTo(self.root);

	self.qualityBg = cc.Sprite:create('image/commonImg/item_quality_2.png');
	self.qualityBg:addTo(self.iconNode);

	self.iconImg = cc.Sprite:create();
	self.iconImg:addTo(self.iconNode);
	self.iconImg:setScale(0.85);
	self.iconImg:setPosition(0 , 2);
	--local stencil = cc.Sprite:create('image/commonImg/item_quality_2.png');
	--local clippingNode = cc.ClippingNode:create();
	--clippingNode:setStencil(stencil);
	--clippingNode:addChild(self.iconImg);
	--clippingNode:setInverted(false);
	--clippingNode:setAlphaThreshold(0);
	--clippingNode:addTo(self.iconNode);
	--clippingNode:setScale(0.95);

	self.infoNode = cc.Node:create();
	self.infoNode:addTo(self.root);

	self.defTypeBg = cc.Sprite:create('image/card/card_deftype_bg.png')
		:addTo(self.infoNode)
		:setPosition(-29 , 29)
		:setScale(0.55)

	self.defType = cc.Sprite:create()
		:addTo(self.infoNode)
		:setPosition(-29 , 29)
		:setScale(0.41)

	self.starList = {};
	for i = 1 , 5 do
		local starBg = cc.Sprite:create('image/card/card_star_bg.png')
			:addTo(self.infoNode)
			:setScale(0.55)
			:setPosition(-27 + 13 * (i - 1) , -33)

		self.starList[i] = cc.Sprite:create()
			:addTo(self.infoNode)
			:setScale(0.55)
			:setPosition(-27 + 13 * (i - 1) , -33)
	end

	self.bg = ccui.Button:create('image/commonImg/icon_item_add.png' , 'image/commonImg/icon_item_add.png');
	self.bg:addTo(self.root);

	self.levelLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
		:enableOutline(cc.c3b(50, 50, 50), 2)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setAnchorPoint(cc.p(1, 0.5))
		:setPosition(40, 29)
		:addTo(self.root)

	--self.sameLabel = cc.Label:createWithTTF(LANG.UI_CARD_59 , fonts.FONT_SIYUAN , 18)
	--	:addTo(self.root)
	--	:setPosition(cc.p(0,-32))
	--	:setTextColor(cc.c3b(255 , 255 , 255))
	--	:enableOutline(cc.c4b(155 , 155 , 155 , 255) , 2)
end

function CardStarLevelUpTemplate:Init(defType , starNum , no)

	if not defType then
		self.root:setVisible(false);
	else 
		self.root:setVisible(true);
		self.levelLabel:setVisible(false);
		self.qualityBg:setTexture('image/commonImg/item_quality_2.png');
		if no then
			--self.sameLabel:setVisible(true);
			--self.infoNode:setVisible(false);
			self.iconNode:setVisible(true);

			local imagePath = ResTable:getValue('role_resource', no , 'icon');
			self.iconImg:setTexture('image/icon/'..imagePath..'.png');

			GlobalFunctions:setSpriteGrey(self.iconImg)
		else
			--self.sameLabel:setVisible(false);
			--self.infoNode:setVisible(true);
			self.iconNode:setVisible(false);
		end

		self.bg:setLocalZOrder(10);

		if defType == 0 then
			self.defTypeBg:setVisible(false);
			self.defType:setVisible(false);
		else
			self.defTypeBg:setVisible(true);
			self.defType:setVisible(true);
			self.defType:setTexture('image/commonImg/def_type_'..defType..'.png');
		end

		self:updateStar(starNum);
	end
end

function CardStarLevelUpTemplate:setData(pid)
	local card = model.CardModel:GetCard(pid);
	if card then
		self.root:setVisible(true);
		--self.sameLabel:setVisible(false);
		self.infoNode:setVisible(true);
		self.iconNode:setVisible(true);

		self.bg:setLocalZOrder(-1);

		self:updateStar(card.star_lv);

		self.qualityBg:setTexture('image/commonImg/item_quality_'..(card.quality-1)..'.png');

		local rolePre = ResTable:getRowData('role_pre', card.no);
		self.defType:setTexture('image/commonImg/def_type_'..rolePre.def_type..'.png');

		local imagePath = ResTable:getValue('role_resource', card.no , 'icon');
		self.iconImg:setTexture('image/icon/'..imagePath..'.png');

		self.levelLabel:setVisible(true);
		self.levelLabel:setString(tostring(card.level));

		GlobalFunctions:removeSpriteGrey(self.iconImg)
	end
end

function CardStarLevelUpTemplate:updateStar(starNum)
	local color = math.ceil(starNum / 5);
	local num = starNum % 5;
	num = num == 0 and 5 or num;
	for i = 1 , 5 do
		if i <= num then
			self.starList[i]:setTexture('image/card/star_level_'..color..'.png');
		end
		self.starList[i]:setVisible(i <= num);
	end
end

function CardStarLevelUpTemplate:setTag(tag)
	self.bg:setTag(tag);
end

function CardStarLevelUpTemplate:addClickEventListener(callback)
	self.bg:addClickEventListener(callback);
end

function CardStarLevelUpTemplate:setPosition(x, y)
	self.root:setPosition(x, y);
end

function CardStarLevelUpTemplate:setScale(scale)
	self.root:setScale(scale);
end

function CardStarLevelUpTemplate:setVisible(visible)
	self.root:setVisible(visible);
end

return CardStarLevelUpTemplate;
