local DailyRoundTemplate = class("DailyRoundTemplate")

local imgPath = 'image/dailyRound/'
local fonts = Enum:getTable("SystemFont")

function DailyRoundTemplate:ctor(father)
	self.father = father;
	self.root = cc.Node:create();
	self.root:addTo(self.father);

	self.bg = cc.Sprite:create(imgPath .. 'bg.png')
		:addTo(self.root)

	self.titleSp = cc.Sprite:create(imgPath .. 'title_2.png')
		:setPosition(0, 55)
		:addTo(self.root)

	self.name = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 25)
		:setTextColor(cc.c3b(60, 60, 60))
		:enableOutline(cc.c3b(255, 255, 255), 2)
		:setPosition(-160, 0)
		:addTo(self.root)

	self.itemBg = ccui.Scale9Sprite:create(imgPath .. 'item_bg.png')
	self.itemBg:setCapInsets(CCRectMake(20, 20, 266, 83))
	self.itemBg:setAnchorPoint(cc.p(1, 0.5))
	self.itemBg:setContentSize(cc.size(260, 123))
	self.itemBg:setPosition(cc.p(278, -5))
	self.itemBg:addTo(self.root)

	self.item1 = template.IconTemplate:create(self.root)
	self.item1:setPosition(85, -5)

	self.item2 = template.IconTemplate:create(self.root)
	self.item2:setPosition(210, -5)

	self.maskNode = cc.Node:create()
		:addTo(self.root)

	self.lockMask = cc.Sprite:create(imgPath .. 'mask.png')
		:addTo(self.maskNode)
	
	self.lock = cc.Sprite:create('image/lock/item_lock2.png')
		:setPosition(-130, 0)
		:addTo(self.maskNode)

	self.lockWord = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setAnchorPoint(cc.p(0, 0.5))
		:setPosition(-90, 0)
		:addTo(self.maskNode)

	self.mark1 = cc.Sprite:create(imgPath .. 'mark1.png')
		:setPosition(255, 60)
		:addTo(self.root)

	self.mark2 = cc.Sprite:create(imgPath .. 'mark2.png')
		:setPosition(255, 60)
		:addTo(self.root)

	self.canFight = false
	self.canPass = false
end

function DailyRoundTemplate:Init(data)

	self.titleSp:setTexture(imgPath .. 'title_' .. data.quality .. '.png')

	self.name:setString(data.name)

	self.lockWord:setString(string.format(LANG.UI_DAILY_ROUND_2, tostring(data.fp)))
	self.fpLimit = ConvertDigitUnitToStr(data.fp);

	self.item1:setVisible(false)
	self.item2:setVisible(false)
	if data.reward[1] and data.reward[2] then
		self.item1:setVisible(true)
		self.item1:Init(data.reward[1][1], data.reward[1][2])
		self.item2:setVisible(true)
		self.item2:Init(data.reward[2][1], data.reward[2][2])
		self.itemBg:setContentSize(cc.size(260, 123))
	end

	if data.reward[1] and not data.reward[2] then
		self.item2:setVisible(true)
		self.item2:Init(data.reward[1][1], data.reward[1][2])
		self.itemBg:setContentSize(cc.size(130, 123))
	end
end

function DailyRoundTemplate:setFp(topfp)
	if HpCompare(topfp , '<' , self.fpLimit) then
		self.maskNode:setVisible(true)
		self.canFight = false
	else
		self.maskNode:setVisible(false)
		self.canFight = true
	end
end

function DailyRoundTemplate:setCanPass(flag)
	self.mark1:setVisible(false)
	self.mark2:setVisible(false)
	if self.canFight then
		if flag then
			self.canPass = true
			self.mark2:setVisible(true)
		else
			self.canPass = false
			self.mark1:setVisible(true)
		end
	end
end

function DailyRoundTemplate:getCanFight()
	return self.canFight
end

function DailyRoundTemplate:getCanPass()
	return self.canPass
end

function DailyRoundTemplate:setPosition(x, y)
	self.root:setPosition(x, y);
end

return DailyRoundTemplate;
