local EventHardLogTemplate = class("EventHardLogTemplate")

local fonts = Enum:getTable("SystemFont");
local outlineColor = {
	[1] = cc.c4b(100 , 100 , 100),
	[2] = cc.c4b(30 , 170 , 70),
	[3] = cc.c4b(30 , 115 , 200),
	[4] = cc.c4b(170 , 20 , 190),
	[5] = cc.c4b(200 , 140 , 30),
}

function EventHardLogTemplate:ctor(father)
	local root  = cc.Node:create();
	self.root   = root;
	self.father = father;

	--大背景
	self.bg = cc.Sprite:create();
	self.bg:addTo(self.root);

	self.icon = cc.Sprite:create();
	self.icon:addTo(self.root);

	self.fg = cc.Sprite:create();
	self.fg:addTo(self.root);

	self.name = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
	self.name:addTo(self.root);
	self.name:setPosition(23 , -185);

	self.color = cc.Sprite:create();
	self.color:addTo(self.root);
	self.color:setPosition(-65 , -179);
	self.color:setScale(0.75);

	local bg = cc.Sprite:create('image/event/event_round_item_bg.png');
	bg:addTo(self.root);
	bg:setPosition(0 , -92);

	self.rewardItem = template.IconTemplate:create(self.root);
	self.rewardItem:setPosition(0 , -92);

	self.state = cc.Sprite:create();
	self.state:addTo(self.root);
	self.state:setPosition(51 , 171);

	self.stateLabel = cc.Label:createWithTTF('' ,  fonts.FONT_IMPACT , 20);
    self.stateLabel:enableOutline(cc.c4b(0, 0, 0 , 255) , 1);
    self.stateLabel:addTo(self.root);
    self.stateLabel:setPosition( 51 , 171 );

	self.father:addChild(self.root);
end

function EventHardLogTemplate:Init(hardLog)

	local curFpNo = model.EventModel:getFpNo();
	local curHard = model.EventModel:getHard();

	local eventFpName = ResTable:getValue('event_fp' , hardLog.fp_no , 'fp_name');
	local hard = math.ceil(hardLog.hard / 3);

	self.name:setString(eventFpName..hardLog.hard);
	self.name:enableOutline(outlineColor[hard] , 2);

	self.bg:setTexture('image/card/card_quality_bg_'..hard..'.png');
	self.fg:setTexture('image/event/event_hard_bg_'..hard..'.png');

	local roleResource = ResTable:getRowData('role_resource', hardLog.show_icon);
	if roleResource then
		self.icon:setTexture('image/iconL/'..roleResource.iconlarge..'.png');
		self.icon:setTextureRect(LRect);
	end
	
	if hardLog.color == 4 then
		self.color:setVisible(false);
	else
		self.color:setVisible(true);
		self.color:setTexture('image/event/color_'..hardLog.color..'.png');
	end
	
	self.rewardItem:Init(EVENT_MONEY,hardLog.event_money);
	if hardLog.event_money == 0 then
		self.rewardItem:setIconNum(hardLog.event_money);
	end

	if hardLog.fp_no == curFpNo and hardLog.hard == curHard then
		self.state:setTexture('image/event/event_hard_state_1.png');
		self.stateLabel:setString( LANG.UI_L_FIGHTING );
	else
		self.state:setTexture('image/event/event_hard_state_2.png');
		self.stateLabel:setString( LANG.UI_L_PASS );
	end
end

function EventHardLogTemplate:setPosition(x , y)
	self.root:setPosition(x , y);
end

return EventHardLogTemplate;
