local FrameLoadScrollViewTemplate = class("FrameLoadScrollViewTemplate")
local scheduler = cc.Director:getInstance():getScheduler();
function FrameLoadScrollViewTemplate:ctor()
    self.scrollView = ccui.ScrollView:create();
	self.scrollView:setTouchEnabled(true);
	self.scrollView:setBounceEnabled(true);
	self.scrollView:setDirection(ccui.ScrollViewDir.vertical);
    self.scrollView:setScrollBarEnabled(false);
end

function FrameLoadScrollViewTemplate:Init(viewSize , cellSize , loadNum , margin,group)
    self.viewSize = viewSize --scrollView Size
    self.cellSize = cellSize --cell Size
    self.loadNum = loadNum or 1; --单次加载个数
    self.margin = margin or 0; --间距
    self.group = group or 1; --多个数据放一组

    self.scrollView:setContentSize(self.viewSize);
end

function FrameLoadScrollViewTemplate:reloadData(data)
    if self.m_scheduleHandler then 
        scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
        self.m_scheduleHandler = nil
    end
    self.scrollView:removeAllChildren();

    self.data = {};

    self.cellList = {};

    if not data then
        return;
    end

    self.data = data;
    self.dataNum = #self.data;
    self.totalNum = math.ceil(self.dataNum/self.group);
   
    self.maxHeight = self.viewSize.height;
    self.maxWidth = self.viewSize.width;

    self.curViewDirection = self.scrollView:getDirection();
    
    if self.curViewDirection == ccui.ScrollViewDir.vertical then
        self.maxHeight = self.totalNum * (self.cellSize.height + self.margin);
        self.maxHeight = self.maxHeight < self.viewSize.height and self.viewSize.height or self.maxHeight;
    else
        self.maxWidth = self.totalNum * (self.cellSize.width + self.margin);
        self.maxWidth = self.maxWidth < self.viewSize.width and self.viewSize.width or self.maxWidth;
    end

	self.scrollView:setInnerContainerSize(cc.size(self.maxWidth, self.maxHeight));

    self.totalLoadedNum = 0;

    self:loadData();

	self.m_scheduleHandler = scheduler:scheduleScriptFunc(handler(self, self.loadData), 0, false);
    
end

function FrameLoadScrollViewTemplate:loadData()
	local startIndex = self.totalLoadedNum + 1;

	self.totalLoadedNum = self.totalLoadedNum + self.loadNum;
	self.totalLoadedNum = self.totalLoadedNum > self.totalNum and self.totalNum or self.totalLoadedNum;

	if startIndex > self.totalLoadedNum then
		if self.m_scheduleHandler then 
			scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
			self.m_scheduleHandler = nil
		end
        if self.loadEndCallback then
            self.loadEndCallback();
        end
		return;
    end
    
    if self.loadCellCallback then
        for i = startIndex , self.totalLoadedNum do
            local cell = ccui.Layout:create();
            cell:addTo(self.scrollView);
            local posX = 0;
            local posY = 0;
            if self.curViewDirection == ccui.ScrollViewDir.vertical then
                posX = self.viewSize.width / 2;
                posY = self.maxHeight - self.cellSize.height/2 - (self.cellSize.height + self.margin) * (i - 1);
            else
                posX = self.maxWidth - self.cellSize.width / 2 - (self.cellSize.width + self.margin) * (i - 1);
                posY = self.viewSize.height / 2
                
            end
            cell:setPosition(posX , posY);

            self.cellList[i] = {};
            self.cellList[i].defPosX = posX;
            self.cellList[i].defPosY = posY;
            self.cellList[i].cell = cell;

            local bret = nil;
            if self.group <= 1 then
                bret = self.loadCellCallback(cell , i , self.data[i]);
            else
                local datas = {};
                local maxNum = i * self.group;
                maxNum = maxNum > self.dataNum and self.dataNum or maxNum;
                for j = ((i-1)*self.group) + 1 ,maxNum do
                    table.insert(datas , self.data[j]);
                end

                bret = self.loadCellCallback(cell , i , datas);
            end
            if bret == nil then
                -- 切换页面，导致强制停止加载
                if self.m_scheduleHandler then 
                    scheduler:unscheduleScriptEntry(self.m_scheduleHandler)
                    self.m_scheduleHandler = nil
                end
            end
        end
    end
end

function FrameLoadScrollViewTemplate:getCell(index)
    if self.cellList[index] then
        return self.cellList[index].cell;
    end
    return nil;
end

function FrameLoadScrollViewTemplate:getCellList()
    return self.cellList;
end

function FrameLoadScrollViewTemplate:setDirection(direction)
    self.scrollView:setDirection(direction);
end

function FrameLoadScrollViewTemplate:setAnchorPoint(x , y)
    self.scrollView:setAnchorPoint(x , y);
end

function FrameLoadScrollViewTemplate:setPosition(x , y)
    self.scrollView:setPosition(x , y);
end

function FrameLoadScrollViewTemplate:getInnerContainerPosition()
    return self.scrollView:getInnerContainerPosition();
end

--显示到最上面
function FrameLoadScrollViewTemplate:jumpToItem(index)

    if not index or index == 0 then
        index = 1;
    end

    if self.curViewDirection == ccui.ScrollViewDir.vertical then
        local height = self.cellSize.height + self.margin;
        local posY = -self.maxHeight + self.viewSize.height + height * (index - 1);
        posY = posY > 0 and 0 or posY;
        self.scrollView:setInnerContainerPosition(cc.p(0 , posY));
    else

    end
end

function FrameLoadScrollViewTemplate:addTo(father)
    self.scrollView:addTo(father);
end

function FrameLoadScrollViewTemplate:addLoadCellHandler(callback)
    if callback then
        self.loadCellCallback = callback;
    end
end

function FrameLoadScrollViewTemplate:addLoadEndHandler(callback)
    if callback then
        self.loadEndCallback = callback;
    end
end

function FrameLoadScrollViewTemplate:addEventListener(callback)
    self.scrollView:addEventListener(callback);
end

function FrameLoadScrollViewTemplate:setTouchEnabled(enabled)
    self.scrollView:setTouchEnabled(enabled);
end

function FrameLoadScrollViewTemplate:stopAllActions()
    for _k , cellData in pairs(self.cellList) do
        cellData.cell:stopAllActions();
        cellData.cell:setPosition(cellData.defPosX , cellData.defPosY);
    end
end

function FrameLoadScrollViewTemplate:setSwallowTouches(swallow)
    self.scrollView:setSwallowTouches(swallow);
end

function FrameLoadScrollViewTemplate:setBounceEnabled(enabled)
    self.scrollView:setBounceEnabled(enabled);
end

function FrameLoadScrollViewTemplate:jumpToTop()
    self.scrollView:jumpToTop();
end

return FrameLoadScrollViewTemplate;
