local FutureCardTemplate = class("FutureCardTemplate" , function() 
	return ccui.Layout:create();
end)
local fonts = Enum:getTable('SystemFont')
local MessageBoxType = Enum:getTable('MessageBoxType')

function FutureCardTemplate:ctor(father)
	self:addTo(father);

	self.emptyNode = cc.Node:create()
	:addTo(self)

	local bg = cc.Sprite:create('image/commonImg/icon_item_add.png')
	:addTo(self.emptyNode)

	self.shadow = cc.Sprite:create("image/card/cardfuture/future_shadow.png");
	self.shadow:addTo(self.emptyNode)
	self.shadow:setVisible(false)

	self.lockSpr = cc.Sprite:create('image/card/cardfuture/future_lock.png')
	:addTo(self.emptyNode)

	self.cdNode = cc.Node:create()
	:addTo(self)
	local cdBg = ccui.Scale9Sprite:create("image/commonImg/bg_1_11_11_34_28.png");
	cdBg:setContentSize(cc.size(100 , 100));
	cdBg:addTo(self.cdNode)

	self.cdMsg = cc.Label:createWithTTF(LANG.UI_L_11709, fonts.FONT_SIYUAN, 18)
	:addTo(self.cdNode)
	:enableGlow(cc.c4b(255,0,0,255))
	self.cdTimeLabel = cc.Label:createWithTTF('22:59:59', fonts.FONT_SIYUAN, 18)
	:addTo(self.cdNode)
	:setPosition(0, -30)


	self.roleNode = template.ItemTemplate:create()
	self.roleNode:addTo(self);
	-- self.roleNode:setScale(0.7)

	local btn = ccui.Button:create('image/commonImg/icon_item_add.png');
	btn:addTo(self);
	-- btn:setPosition(0, -70);
	btn:addClickEventListener(handler(self, self.clickBtn));
	btn:setOpacity(0)

	EventManager:RegisterEvent(EventType.UIFutureCardUpdate, handler(self, self.changeUpdate));

end

function FutureCardTemplate:Init( v )
	self.data = v;
	self.sid = v.no;
	self.unlock = v.unlock;
	self.pid = v.pid;
	self.time = v.time;
	self:updateInfo()
end

function FutureCardTemplate:changeUpdate( sid )
	-- body
	if sid ~= self.sid then
		return
	end
	local sinfo = model.FutureCardModel:getFutureCardById(self.sid)
	self.sid = sinfo.no;
	self.unlock = sinfo.unlock;
	self.pid = sinfo.pid;
	self.time = sinfo.time;
	self:updateInfo()
end

function FutureCardTemplate:updateInfo()
	local curTime = TimeManager:getCurTime();
	self.cdNode:setVisible(false)
	if self.unlock == 1 then
		self.lockSpr:setVisible(false)
		self.shadow:setVisible(false)
		if self.pid > 0 then
			self.roleNode:setVisible(true)
			local card = model.CardModel:GetCard(self.pid);
	        self.roleNode:Init(ROLE_TYPE , card.no,card.star_lv,card.level,self.pid);
		else
			if self.time > curTime then
				self.cdNode:setVisible(true)
			end
			self.roleNode:setVisible(false)
		end
	else
		self.roleNode:setVisible(false)
		self.lockSpr:setVisible(true)
		self.shadow:setVisible(true)
		local sinfo = model.FutureCardModel:getFutureCardById(self.sid-1)
		if sinfo and sinfo.unlock==1 then
			self.shadow:setVisible(false)
		end
	end
	self:updateCDTime(curTime)
end

function FutureCardTemplate:updateCDTime( curTime )
	-- body
	if self.cdNode:isVisible() then
		local time = math.floor(self.time - curTime);
		if time>0 then
			self.cdTimeLabel:setString( Sec2Time(time)  )
		else
			self.cdNode:setVisible(false)
		end
	end
end

function FutureCardTemplate:clickBtn()
	if self.unlock == 1 then
		if self.pid > 0 then
			UIManager:push('FutureCardTakeOffPanel', self.sid, self.pid);
		else
			local curTime = TimeManager:getCurTime();
			if self.time > curTime then
				MessageBox:create(LANG.UI_L_11712 , MessageBoxType.CANCEL_OK,function()
					if model.ItemModel:getItemNum(BLUE_DIA) < 100 then
						Prompt:Show(LANG.UI_GUILD_35);
						return;
					else
						model.FutureCardModel:clearSlot(self.sid)
					end
				end);
			else
				UIManager:push('FutureCardSelPanel', self.sid);
			end
		end
	else
		if self.shadow:isVisible() then
			return;
		end
		local sinfo = model.FutureCardModel:getFutureCardById(self.sid)
		local toolId = sinfo.cost[1]
		local costValue = sinfo.cost[2]
		local icon = ResTable:getValue('item' , toolId , 'icon');
		local itemIcon = "image/icon/I"..icon..".png";
		local itemNum = model.ItemModel:getItemNum(toolId);

		if itemNum >= costValue then
			MessageBox:create(LANG.UI_L_11710 , MessageBoxType.CANCEL_OK,function()
				model.FutureCardModel:unLockSlot(self.sid)
			end);
			MessageBox:setCostInfo(itemIcon, itemNum, costValue)
		else
			MessageBox:create(LANG.UI_L_11711 , MessageBoxType.OK,function()
			end);
			MessageBox:setCostInfo(itemIcon, itemNum, costValue)
		end
	end
	
end

return FutureCardTemplate;