local ItemTemplate = class("ItemTemplate" , function()
	return ccui.Layout:create();
end)

local fonts = Enum:getTable('SystemFont')
local qualityBg = Enum:getTable('ItemQualityBg')

function ItemTemplate:ctor(bgType)

	self.id = -1;
	self.count = -1;
	
	self:iconInit(bgType);
end

function ItemTemplate:iconInit( bgType )
	
	self:setCascadeOpacityEnabled(true);

	self.qualityBg = ccui.ImageView:create('image/commonImg/item_quality_2.png') --cc.p(103,103)
		:addTo(self)

	if bgType == 2 then
		self.lightBg = cc.Sprite:create('image/commonImg/icon_item_light.png')
			:addTo(self)
	end

	self.iconNode = cc.Node:create();
	self.iconNode:addTo(self);
	self.iconNode:setCascadeOpacityEnabled(true);
	--[[
	self.icon = cc.Sprite:create();
	local stencil = cc.Sprite:create('image/commonImg/icon_clipping_bg.png');
	local clippingNode = cc.ClippingNode:create();
	clippingNode:setStencil(stencil);
	clippingNode:addChild(self.icon);
	clippingNode:setInverted(false);
	clippingNode:setAlphaThreshold(0);
	clippingNode:addTo(self);
	clippingNode:setCascadeOpacityEnabled(true);
	]]
	self.roleInfoNode = cc.Node:create()
		:addTo(self)
		:setVisible(false)
		:setCascadeOpacityEnabled(true)

	local defTypeBg = cc.Sprite:create('image/card/card_deftype_bg.png')
		:addTo(self.roleInfoNode)
		:setPosition(-29 , 29)
		:setScale(0.55)

	self.defType = cc.Sprite:create()
		:addTo(self.roleInfoNode)
		:setPosition(-29 , 29)
		:setScale(0.41)

	self.starList = {};
	for i = 1 , 5 do
		local starBg = cc.Sprite:create('image/card/card_star_bg.png')
			:addTo(self.roleInfoNode)
			:setScale(0.55)
			:setPosition(-27 + 13 * (i - 1) , -33)

		self.starList[i] = cc.Sprite:create()
			:addTo(self.roleInfoNode)
			:setScale(0.55)
			:setPosition(-27 + 13 * (i - 1) , -33)
	end

	self.levelLabel = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20)
		:setVisible(false)
		:enableOutline(cc.c3b(50, 50, 50), 2)
		:setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT)
		:setAnchorPoint(cc.p(1, 0.5))
		:setPosition(40, 29)
		:addTo(self.roleInfoNode)

	self.otherNode = cc.Node:create()
		:addTo(self)

	self.itemNum = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20);
	self.itemNum:addTo(self);
	self.itemNum:setAnchorPoint(cc.p(1,0.5))
	self.itemNum:setPosition(39, -29);
	self.itemNum:enableOutline(cc.c4b(0 , 0 , 0 ,255),2);
end

function ItemTemplate:Init(id, count, star, level, pid, quality, avatar_no)
	self.id = id;
	self.count = count;

	count = count or 0;
	local data = {};

	self.roleInfoNode:setVisible(false);
	self.otherNode:removeAllChildren();

	local icon = '';
	if id == MONEY_ID then
		data = ResTable:getRowData('item', id)
		icon = "image/icon/I"..data.icon..".png";
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..qualityBg[data.quality]..'.png');
		if string.len(count) <= 2 then
			self.itemNum:setString(count == 0 and '' or tostring(count));
		else
			self.itemNum:setString(count == 0 and '' or ConvertStrToDigitUnit(tostring(count)));
		end
	elseif id == MONSTER_TYPE then
		id = count;
		data = ResTable:getRowData('monster', id);
		if not data then
			return;
		end

		local iconPath = data['icon'];
		if not iconPath then
			return;
		end
		icon = 'image/icon/'..iconPath..'.png';
		local restraint = cc.Sprite:create('image/commonImg/def_type_'..data['def_type']..'.png');
		restraint:setScale(0.5);
		restraint:setPosition(0, 52);
		restraint:addTo(self.otherNode);
		self.itemNum:setString('');
	elseif id == MONSTER_ROLE_TYPE then
		id = count;
		data = ResTable:getRowData('monster', id);
		if not data then
			return;
		end

		star = star or 1;

		local quality = math.max(star,2);
		quality = math.min(star,5)
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..quality..'.png');

		icon = 'image/icon/'..data.icon..'.png';
		self.itemNum:setString('');

		self.roleInfoNode:setVisible(true);
		self:updateStar(star);
		self.defType:setTexture('image/commonImg/def_type_'..data.def_type..'.png');

		if level then
			self.levelLabel:setVisible(true)
			self.levelLabel:setString(tostring(level))
		end
	elseif id == ROLE_TYPE then
		id = count;
		data = ResTable:getRowData('role' , id);
		if not data then
			return;
		end

		local resItemId = 0;
		local quality = quality or data.quality;
		if avatar_no and avatar_no > 0 then
			resItemId = ResTable:getValue('role_avatar', avatar_no, 'resource_id')
		elseif pid and pid > 0 then
			resItemId = model.CardModel:getCardResIdByPid(pid);
			local cardData = model.CardModel:GetCard(pid);
			quality = cardData.quality;
		end
		if resItemId == 0 then
			resItemId = data.resource_id;
		end
		local imagePath = ResTable:getValue('role_resource', resItemId , 'icon');
		if not imagePath then
			return;
		end
		
		local rolePre = ResTable:getRowData('role_pre', data.role_id);

		icon = 'image/icon/'..imagePath..'.png';
		
		self.itemNum:setString('');

		self.roleInfoNode:setVisible(true);
		self:updateStar(star or rolePre.initial_star);
		self.defType:setTexture('image/commonImg/def_type_'..rolePre.def_type..'.png');

		self.qualityBg:loadTexture('image/commonImg/item_quality_'..(quality-1)..'.png');

		if level and level>0 then
			self.levelLabel:setVisible(true)
			self.levelLabel:setString(tostring(level))
		end
	elseif id == ROLE_AVATAR_TYPE then
		id = count;
		data = ResTable:getRowData('role_avatar' , id);
		if not data then
			return;
		end

		local imagePath = ResTable:getValue('role_resource', data.resource_id , 'icon');
		if not imagePath then
			return;
		end

		local roleData = ResTable:getRowData('role', data.role_id);
		if not roleData then
			return
		end

		local rolePre = ResTable:getRowData('role_pre', data.role_id);
	
		icon = 'image/icon/'..imagePath..'.png';
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..(roleData.quality-1)..'.png');
		self.itemNum:setString('');

		self.roleInfoNode:setVisible(true);
		self:updateStar(star or rolePre.initial_star);
		self.defType:setTexture('image/commonImg/def_type_'..rolePre.def_type..'.png');

		if level and level>0 then
			self.levelLabel:setVisible(true)
			self.levelLabel:setString(tostring(level))
		end
	elseif id == BARE_COLLECT_TYPE then
		id = count;
		data = ResTable:getRowData('role' , id);
		if not data then
			return;
		end

		local imagePath = ResTable:getValue('role_resource', data.resource_id , 'icon');
		if not imagePath then
			return;
		end

		icon = 'image/icon/'..imagePath..'.png';
		
		self.itemNum:setString('');
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..(data.quality-1)..'.png');

	elseif id > 80000 and id < 90000 then
		data = ResTable:getRowData('equip', id)
		if not data then
			return;
		end

		local itemData = ResTable:getRowData('item' , id);

		icon = 'image/icon/E'..itemData.icon..'.png';
		if count and count > 0 then
			self.itemNum:setString(tostring(count));
		else
			self.itemNum:setString('');
		end
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..qualityBg[data.quality]..'.png');

		local starNum = equipStar[itemData.quality];
		if starNum then
			for i = 5, 1, -1 do
				if i <= starNum then
					local star = cc.Sprite:create('image/equip/equip_star.png');
					star:setPosition(-38 + 9 * (i - 1) , -34);
					star:addTo(self.otherNode);
				end
			end
		end
	else
		data = ResTable:getRowData('item', id)
		if not data then
			return;
		end
		icon = "image/icon/I"..data.icon..".png";
		self.itemNum:setString(count == 0 and '' or tostring(count));
		self.qualityBg:loadTexture('image/commonImg/item_quality_'..qualityBg[data.quality]..'.png');
	end

	self:initIcon(icon);
	--self.icon:setTexture(icon);
end

function ItemTemplate:initIcon(icon)
	self.iconNode:removeAllChildren();
	local icon = cc.Sprite:create(icon);
	if not icon then
		return;
	end
	local size = icon:getContentSize();

	local stencil = cc.Sprite:create('image/commonImg/icon_clipping_bg.png');
	local clippingNode = cc.ClippingNode:create();
	clippingNode:setStencil(stencil);
	clippingNode:addChild(icon);
	clippingNode:setInverted(false);
	clippingNode:setAlphaThreshold(0);
	clippingNode:setPosition(size.width/2,size.height/2)

	local renderTexture = cc.RenderTexture:create(size.width,size.height,cc.TEXTURE2_D_PIXEL_FORMAT_RGB_A8888,gl.DEPTH24_STENCIL8_OES);
	renderTexture:beginWithClear(0,0,0,0);
	clippingNode:visit();
	renderTexture:endToLua();
	local spTexture = renderTexture:getSprite():getTexture();
	icon = cc.Sprite:createWithTexture(spTexture)
	icon:addTo(self.iconNode);
	icon:setScaleY(-1);
end

function ItemTemplate:updateStar(star)
	local color = math.ceil(star / 5);
	local num = star % 5;
	num = num == 0 and 5 or num;
	for i = 1 , 5 do
		if i <= num then
			self.starList[i]:setTexture('image/card/card_star_'..color..'.png');
		end
		self.starList[i]:setVisible(i <= num);
	end
end

function ItemTemplate:clearItem()
	self:updateStar(0)
	self.itemNum:setString('');
	self.roleInfoNode:setVisible(false);
	self.defType:setTexture('');
	self.qualityBg:loadTexture('image/commonImg/icon_item_add.png');
	self.levelLabel:setVisible(false)
	self.iconNode:removeAllChildren();
end

function ItemTemplate:setItemNum(no,count)
	self.itemNum:setString(ConvertStrToDigitUnit(tostring(count)));
end

function ItemTemplate:addItemClickEventListener(callback)
	self.qualityBg:addClickEventListener(callback)
	self.qualityBg:setTouchEnabled(true)
	self.qualityBg:setSwallowTouches(false);
end

function ItemTemplate:setItemTag(tag)
	self.qualityBg:setTag(tag);
end

function ItemTemplate:getItemTag()
	return self.qualityBg:getTag();
end

function ItemTemplate:setItemSwallowTouches(swallow)
	self.qualityBg:setSwallowTouches(swallow);
end

function ItemTemplate:setEquipInSlot(visible)
	if visible then
		self.itemNum:setString('');
	end
	if not self.equipInSlot then
		self.equipInSlot = cc.Sprite:create('image/equip/equip_bg2.png');
		self.equipInSlot:addTo(self);
		self.equipInSlot:setPosition(21 , -13);

		local equipInSlotSize = self.equipInSlot:getContentSize();
		local equipInSlotLabel = cc.Label:createWithTTF(LANG.UI_EQUIP_43 , fonts.FONT_SIYUAN , 18);
		equipInSlotLabel:addTo(self.equipInSlot);
		equipInSlotLabel:setPosition(equipInSlotSize.width/2 , equipInSlotSize.height/2);
		equipInSlotLabel:enableOutline(cc.c3b(0 , 100 , 200),2);
	end

	self.equipInSlot:setVisible(visible);
end

function ItemTemplate:setSelect(select)
	if not self.select then
		self.select = cc.Sprite:create('image/commonImg/common_item_select.png');
		self.select:addTo(self);
	end

	self.select:setVisible(select)
end

function ItemTemplate:showRedDot(show)
	tools.RedDotTools:setRedDotToNode(self,show,nil,cc.p(41.5 , 41.5));
end

function ItemTemplate:setGachaProbability(rate, st)
	if not self.gachaProbability then
		local gachaProbabilityNode = cc.Node:create();
		gachaProbabilityNode:addTo(self);
		if st == true then
			gachaProbabilityNode:setPosition(0,-59)
		else
			gachaProbabilityNode:setPosition(0,-39)
		end
		
		local bg = cc.Sprite:create('image/commonImg/item_gacha_probability.png');
		bg:addTo(gachaProbabilityNode);

		self.gachaProbability = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 15);
		self.gachaProbability:addTo(gachaProbabilityNode);
		self.gachaProbability:setTextColor(cc.c3b(255,255,0));
	end

	self.gachaProbability:setString(tostring(rate))
end

function ItemTemplate:setBlackShader(visible)
	if not self.blackShader then
		self.blackShader = cc.Sprite:create('image/mail/mail_bg_2.png');
		self.blackShader:addTo(self);
	end

	self.blackShader:setVisible(visible);
end

function ItemTemplate:setCanShowInfo(showInfo,bSwallowEnabled)
	if showInfo then
		self.qualityBg:addClickEventListener(function()
			GlobalFunctions:showCommonInfo(self.id,self.count);
		end)
		self.qualityBg:setTouchEnabled(true)
		self.qualityBg:setSwallowTouches(bSwallowEnabled == true);
	else
		self.qualityBg:setTouchEnabled(false)
		self.qualityBg:setSwallowTouches(false);
	end
end

function ItemTemplate:updateAltarLevelColor(oriLv)
	if oriLv and oriLv>=1 then
		self.levelLabel:setTextColor(cc.c3b(255,255,0));
	else
		self.levelLabel:setTextColor(cc.c3b(255,255,255));
	end
end
return ItemTemplate;
