local LifeMissionTemplate = class('LifeMissionTemplate')
local fonts = Enum:getTable("SystemFont")
function LifeMissionTemplate:ctor(father)
	self.root = cc.Node:create();
	self.root:addTo(father);
	self.root:setCascadeOpacityEnabled(true);

	local bg = cc.Sprite:create('image/commonImg/bg_4_563_126.png');
	bg:addTo(self.root);

	local bg = cc.Sprite:create('image/fight/roundReward/round_reward_1.png');
	bg:setPosition(-252 , 35);
	bg:addTo(self.root);

	self.title = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 18);
	self.title:addTo(self.root);
	self.title:setPosition(-232 , 35);
	self.title:setAnchorPoint(0 , 0.5);
	self.title:setTextColor(cc.c3b(255 , 255 , 255));
	self.title:enableOutline(cc.c3b(40 , 50 , 60) , 2);

	local itemBg = cc.Sprite:create('image/commonImg/bg_3_194_102.png');
	itemBg:addTo(self.root);
	itemBg:setPosition(58 , 0);

	self.itemView = ccui.ScrollView:create();
	self.itemView:setTouchEnabled(true);
	self.itemView:setBounceEnabled(true);
	self.itemView:setDirection(ccui.ScrollViewDir.horizontal);
	self.itemView:setContentSize(cc.size(185, 87));
	self.itemView:setAnchorPoint(cc.p(0.5, 0.5));
	self.itemView:setPosition(58 , 0);
	self.itemView:setScrollBarEnabled(false);
	self.itemView:addTo(self.root);

	local lifeMissionProgressBg = cc.Sprite:create('image/commonImg/pgs_bg_1_213_12.png');
	lifeMissionProgressBg:addTo(self.root);
	lifeMissionProgressBg:setPosition(-160 , -1);

	self.lifeMissionProgress = cc.Scale9Sprite:create('image/commonImg/pgs_2_14_12.png');
	self.lifeMissionProgress:setCapInsets(CCRectMake(6,0,2,12));
	self.lifeMissionProgress:addTo(self.root);
	self.lifeMissionProgress:setAnchorPoint(0 , 0.5);
	self.lifeMissionProgress:setPosition(-266 , -1);


	self.num = cc.Label:createWithTTF('' , fonts.FONT_SIYUAN , 18);
	self.num:addTo(self.root);
	self.num:setPosition(-160 , -25);
	self.num:enableOutline(cc.c4b(40 , 50 , 60 , 255) , 2);

	self.btnLabel = cc.Label:createWithTTF(LANG.UI_ACTIVITY_13, fonts.FONT_SIYUAN, 18)
	self.btnLabel:setTextColor(cc.c3b(255 , 255 , 255));
	self.btnLabel:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);

	self.btn = ccui.Button:create('image/commonImg/btn_2_103_36.png' ,'image/commonImg/btn_2_103_36.png' , 'image/commonImg/btn_3_103_36.png');
	self.btn:addTo(self.root);
    self.btn:setTitleLabel(self.btnLabel)
	self.btn:addClickEventListener(handler(self , self.btnClick));
	self.btn:setPosition(219 , 0);

	self.effect = cc.ParticleSystemQuad:create('Effect/particle/lingqu.plist');
	self.effect:addTo(self.root);
	self.effect:setPositionType(cc.POSITION_TYPE_RELATIVE);
	self.effect:setPosition(219 , 0)
	self.effect:stopSystem();

	local spine_res = 'shengya_spine';
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.skeleton = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
		self.skeleton:addTo(self.root)
		self.skeleton:registerSpineEventHandler(function (event)
			if event.eventData.name == 'remove' then
				self.skeleton:setVisible(false);
				local action = cc.Sequence:create(cc.FadeOut:create(0.1) , cc.CallFunc:create(handler(self , self.actionEndCallback)));
				self.root:runAction(action)
			end
		end, sp.EventType.ANIMATION_EVENT)
	end
end

function LifeMissionTemplate:initData(data , idx)
	self.itemView:removeAllChildren();

	local num = #data.reward;
	local maxWidth = 80 * num;

	self.itemView:setInnerContainerSize(cc.size(maxWidth , 86));

	for _k , item in pairs(data.reward) do
		local icon = template.ItemTemplate:create(2);
		icon:addTo(self.itemView);
		icon:setScale(0.8);
		icon:setPosition(41.2 + 82.4 * (_k - 1) , 43.5);
		icon:Init(item[1] , item[2]);
	end

	if data.type == Enum:getTable("MissionType").MISSION_GOLD_COST then
		self.num:setString(string.format("%s/%s",tostring(data.num),tostring(data.nextNum)));
	else
		self.num:setString(string.format("%d/%d",data.num,data.nextNum));
	end
	
	self.title:setString(string.format(tostring(data.name) , data.nextNum));

	local percentage = data.percent;
	percentage = percentage > 1 and 1 or percentage;
	local width = 213 * percentage;
	if width > 0 then
		width = width < 14 and 14 or width;
	end
	self.lifeMissionProgress:setContentSize(cc.size(width , 12)); --min:14 max: 213

	self.btn:setTag(data.no);

	if data.state == 2 then
		self.effect:resetSystem();
		self.btn:setBright(true)
		self.btnLabel:setString(LANG.UI_ACTIVITY_13);
        self.btnLabel:setTextColor(cc.c3b(255 , 255 , 255))
		self.btnLabel:enableOutline(cc.c3b(55, 85, 155), 2)
		self.btn:setTag(1);
	else
		self.effect:stopSystem();
		-- self.btn:loadTextures('image/battlePass/battle_pass_bg_19.png' , 'image/battlePass/battle_pass_bg_19.png');
		self.btn:setBright(false)
		--self.btnLabel:setString(LANG.COMMON_GO);
		self.btnLabel:setString(LANG.UI_MISSION_27);
        self.btnLabel:setTextColor(cc.c3b(60 , 60 , 60))
        self.btnLabel:enableOutline(cc.c3b(255, 255, 255), 2)

		self.btn:setTag(2);
	end

	
	--self.notReachBtn:setVisible(data.state ~= 2);
	self.data = data;
	self.index = idx;
end

function LifeMissionTemplate:setOpacity(opacity)
	self.root:setOpacity(opacity);
end

function LifeMissionTemplate:setPosition(x , y)
	self.root:setPosition(x , y);
end

function LifeMissionTemplate:btnClick(sender , event)
	local tag = sender:getTag();
	if self.callback then
		self.callback(tag , self.data);
	end
end

function LifeMissionTemplate:addClickEventListener(callback)
	self.callback = callback;
end

function LifeMissionTemplate:runAction(action)
	self.root:runAction(action);
end

function LifeMissionTemplate:addShowSpineEndEventListener(callback)
	self.showSpineEndCallback = callback
end

function LifeMissionTemplate:showSpine(bext)
	self.bext = bext;
	if self.skeleton then
		self.skeleton:setVisible(true);
		self.skeleton:clearTracks();
		self.skeleton:addAnimation(0, 'shengya_spine' , false);
	end
end

function LifeMissionTemplate:actionEndCallback()
	if self.showSpineEndCallback then
		if self.bext==true then
			self.showSpineEndCallback(self.index);
		end
		self.root:setOpacity(255);
	end
end

return LifeMissionTemplate;