local StorySkillCell = class("StorySkillCell" , function()
	return ccui.Layout:create();
end)

local fonts = Enum:getTable('SystemFont')
local qualityBg = Enum:getTable('ItemQualityBg')

function StorySkillCell:ctor()

	self.id = -1;
	self.itemId = -1;
	
	self:iconInit();
end

function StorySkillCell:iconInit()
	
	self:setCascadeOpacityEnabled(true);

	self.qualityBg = ccui.ImageView:create('image/storyLine/skill_ibg_4.png')
		:addTo(self)
		:setSwallowTouches(false)

	self.select = cc.Sprite:create('image/commonImg/common_item_select.png');
	self.select:addTo(self)
	self.select:setScale(1.05)
	self.select:setVisible(false)

	self.iconNode = cc.Node:create();
	self.iconNode:addTo(self);
	self.iconNode:setCascadeOpacityEnabled(true);
	self.iconSpr = cc.Sprite:create();
	self.iconSpr:addTo(self.iconNode)

	self.otherNode = cc.Node:create()
		:addTo(self)

	self.numBg = cc.Sprite:create('image/storyLine/skill_nbg.png')
	self.numBg:setPosition(0, -42)
	self.numBg:addTo(self)

	self.itemNum = cc.Label:createWithTTF('100%', fonts.FONT_SIYUAN, 20);
	self.itemNum:addTo(self);
	self.itemNum:setAnchorPoint(cc.p(0.5,0.5))
	self.itemNum:setPosition(0, -42);
	self.itemNum:enableOutline(cc.c4b(0 , 0 , 0 ,255),2);

	self.itemName = cc.Label:createWithTTF('', fonts.FONT_SIYUAN, 20);
	self.itemName:addTo(self);
	self.itemName:setAnchorPoint(cc.p(0.5,0.5))
	self.itemName:setPosition(0, 45);
	self.itemName:enableOutline(cc.c4b(0 , 0 , 0 ,255),2);

	EventManager:RegisterEvent('StorySkillCell:clickSelCell', handler(self, self.setSelect));
end

function StorySkillCell:Init(id, itemId)
	self.id = id;
	self.itemId = itemId;

	local data = {};

	local icon = '';
	if self.id == 1 then
		data = ResTable:getRowData('skill', self.itemId)
		if not data then
			return;
		end
		icon = "image/skillicon/"..data.icon..".png";
		-- self.itemNum:setString(count == 0 and '' or tostring(count));
		-- self.qualityBg:loadTexture('image/commonImg/item_quality_'..qualityBg[data.quality]..'.png');

		self.iconSpr:setTexture(icon)
		self.itemName:setString( data.name )
	end

	-- self:initIcon(icon);
end

function StorySkillCell:initIcon(icon)
	self.iconNode:removeAllChildren();
	local icon = cc.Sprite:create(icon);
	if not icon then
		return;
	end
	local size = icon:getContentSize();

	local stencil = cc.Sprite:create('image/commonImg/icon_clipping_bg.png');
	local clippingNode = cc.ClippingNode:create();
	clippingNode:setStencil(stencil);
	clippingNode:addChild(icon);
	clippingNode:setInverted(false);
	clippingNode:setAlphaThreshold(0);
	clippingNode:setPosition(size.width/2,size.height/2)

	local renderTexture = cc.RenderTexture:create(size.width,size.height,cc.TEXTURE2_D_PIXEL_FORMAT_RGB_A8888,gl.DEPTH24_STENCIL8_OES);
	renderTexture:beginWithClear(0,0,0,0);
	clippingNode:visit();
	renderTexture:endToLua();
	local spTexture = renderTexture:getSprite():getTexture();
	icon = cc.Sprite:createWithTexture(spTexture)
	icon:addTo(self.iconNode);
	icon:setScaleY(-1);
end

function StorySkillCell:addItemClickEventListener(callback)
	self.qualityBg:addClickEventListener(callback)
	self.qualityBg:setTouchEnabled(true)
	self.qualityBg:setSwallowTouches(false);
end

function StorySkillCell:setItemTag(tag)
	self.id = tag;
	self.qualityBg:setTag(tag);
end

function StorySkillCell:getItemTag()
	return self.qualityBg:getTag();
end

function StorySkillCell:setItemSwallowTouches(swallow)
	self.qualityBg:setSwallowTouches(swallow);
end

function StorySkillCell:setSelect(v)
	self.select:setVisible( self.id==v )
end

function StorySkillCell:showRedDot(show)
	tools.RedDotTools:setRedDotToNode(self,show,nil,cc.p(41.5 , 41.5));
end

function StorySkillCell:setBlackShader(visible)
	if not self.blackShader then
		self.blackShader = cc.Sprite:create('image/mail/mail_bg_2.png');
		self.blackShader:addTo(self);
	end

	self.blackShader:setVisible(visible);
end

function StorySkillCell:setCanShowInfo(showInfo,bSwallowEnabled)
	if showInfo then
		self.qualityBg:addClickEventListener(function()
			GlobalFunctions:showCommonInfo(self.id,self.count);
		end)
		self.qualityBg:setTouchEnabled(true)
		self.qualityBg:setSwallowTouches(bSwallowEnabled == true);
	else
		self.qualityBg:setTouchEnabled(false)
		self.qualityBg:setSwallowTouches(false);
	end
end

return StorySkillCell;
