local TeamRoleTemplate = class("TeamRoleTemplate")

local imgPath = 'image/team/';
local fonts = Enum:getTable('SystemFont');

local selectPos = {
	[1] = cc.p(-1 , -2),
	[2] = cc.p(0 , -3),
	[3] = cc.p(0 , -4),
	[4] = cc.p(0 , -3),
}

function TeamRoleTemplate:ctor(father)
	self.root = cc.Node:create();
	father:addChild(self.root);

	self.beSelected = false;
	self.pid = -1;

	-- bg
	self.clickLayout = ccui.Layout:create();
	self.clickLayout:addTo(self.root);
	self.clickLayout:setAnchorPoint(cc.p(0.5, 0));
	self.clickLayout:setPosition(0, -50);
	self.clickLayout:setContentSize(cc.size(100, 200));
	self.clickLayout:setTouchEnabled(true);
	self.clickLayout:setVisible(false)
	self.clickLayout:addClickEventListener(handler(self , self.beClicked));

	self.bg = cc.Sprite:create(imgPath .. 'team_role_bg.png');
	self.bg:addTo(self.root);

	---- select

	self.selectSp = cc.Sprite:create(imgPath..'team_select.png')
		:addTo(self.root)
		:setVisible(false)
		:setPosition(0,6)

	-- role
	self.roleNode = cc.Node:create();
	self.roleNode:setPosition(0, 0);
	-- self.root:addChild(self.roleNode);
	
	self.stencil = cc.Sprite:create('image/commonImg/mask_icon.png');
	self.stencil:setContentSize(cc.size(150,180));
	self.stencil:setPosition(cc.p(0,50))
	self.clippingNode = cc.ClippingNode:create();
	self.clippingNode:setStencil(self.stencil);
	self.clippingNode:addChild(self.roleNode);
	self.clippingNode:setInverted(false);
	self.clippingNode:setAlphaThreshold(0);
	self.root:addChild(self.clippingNode);

   	local spine_res = 'team_spine'
	if cc.FileUtils:getInstance():isFileExist('spine/'..spine_res.."/"..spine_res..".json") then
		self.skeletonNode2 = sp.SkeletonAnimation:create('spine/'..spine_res.."/"..spine_res..".json", 'spine/'..spine_res.."/"..spine_res..".atlas", 1)
        self.skeletonNode2:addTo(self.root)
	else
        print('error : 缺少spine 资源  team_spine')
    end 

	self.modelNode = cc.Node:create();
	self.modelNode:setPosition(0, 70);
	self.modelNode:setScale(0.45);
	self.modelNode:addTo(self.roleNode);

	-- hp
	self.bloodVolume = require('app.template.BloodVolume'):create(self.roleNode);
	self.bloodVolume:setPosition(0, 125);

	-- star
	--self.starList = {};
	--for i = 1 , 5 do
	--	self.starList[i] = cc.Sprite:create();
	--	self.starList[i]:addTo(self.roleNode);
	--	self.starList[i]:setScale(0.6);
	--	self.starList[i]:setPosition(-8 + 10 * (i - 1) , 127);
	--end

	self.formationIndex =  cc.Label:createWithTTF(string.format( "GN - 0%d",1 ), fonts.FONT_SIYUAN, 20);
	self.formationIndex:addTo(self.roleNode);
	self.formationIndex:setPosition(cc.p(0,0))
	self.formationIndex:enableOutline(cc.c4b(60 , 60 , 60 , 255) , 2);
end

function TeamRoleTemplate:Init(index)
	self.index = index or 1;
	--if selectPos[index] then
	--	self.selectSp:setPosition(selectPos[index])
	--end
	self.formationIndex:setString(string.format( "GN - 0%d",self.index ))
end

function TeamRoleTemplate:setInfo(pid, hp)
	-- self:removeModel();
	if pid ~= -1 then
		self:removeModel();
		self.roleNode:setVisible(true);
		local card = model.CardModel:GetCard(pid);
		if card then
			local no = card.no;
			local roleData = ResTable:getRowData('role', no);
			local rolePreData = ResTable:getRowData('role_pre', roleData.role_id);
			local resId = model.CardModel:getCardResIdByPid(pid);
			local roleResData = ResTable:getRowData('role_resource', resId);

			-- set model
			local modelPath = roleResData.model;
			if not modelPath then return; end
			local armature, id = DragonBonesManager:GetArmature(modelPath);
			armature:setName('model');
			armature:addTo(self.modelNode);
			self.lastArmature,self.lastArmatureId = armature,id;
			self.stencil:setPosition(cc.p(0,50))
			-- self.removeArmature = function()
			-- 	armature:removeFromParent();
			-- 	DragonBonesManager:RemoveDisplay(id)
			-- 	armature = nil
			-- end

			-- star
			--local rank = card.star_lv;
			--local color = math.ceil(rank / 5);
			--local num = rank % 5;
			--num = num == 0 and 5 or num;

			--for i = 1 , 5 do
			--	if i <= num then
			--		self.starList[i]:setTexture('image/card/card_star_'..color..'.png');
			--	end
			--	self.starList[i]:setVisible(i <= num);
			--end

			-- hp
			if not hp then
				self.bloodVolume:setPercent(math.floor(100));
			else
				self.bloodVolume:setPercent(math.floor(hp/100))
			end

			if card.hp == 0 then
				--[[
				local vertShaderByteArray, flagShaderByteArray = GetVFShader(0); 
				local glProgram = cc.GLProgram:createWithByteArrays(vertShaderByteArray, flagShaderByteArray);
				glProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_POSITION, cc.VERTEX_ATTRIB_POSITION);
				glProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_COLOR, cc.VERTEX_ATTRIB_COLOR);
				glProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_TEX_COORD, cc.VERTEX_ATTRIB_TEX_COORDS);
				glProgram:link();
				glProgram:updateUniforms();
				for k, v in pairs(armature:getArmature():getBones()) do
					v:setGLProgram(glProgram);
				end
				--]]
			else
				self.lastArmature:getAnimation():play('idle');
			end

			--level
			self.bloodVolume:setLevel(card.level);

			-- set restraint
			self.bloodVolume:setDefType(rolePreData.def_type);
		end
		if self.showActionType ~= nil  then
			self:showAction()
		end
	else
		self:showAction()
	end
	self.pid = pid;
end

function TeamRoleTemplate:checkStopShowAction()
	if self.showActionType ~= nil then
		self.showActionType = nil;
		self:showAction()
		self.stencil:stopAllActions()
		self.stencil:setPosition(cc.p(0,50))
	end
end

function TeamRoleTemplate:showAction(   )
	if self.showActionType == nil or self.showActionType == -1 then
		self:removeModel();
		self.roleNode:setVisible(false);
		self.pid = -1;
		return 
	end
	local aniName = 'up'
	local stencilPosBegin = cc.p(0,-155)
	local stencilPosEnd = cc.p(0,50)
	if self.showActionType == 1 then
		-- 上阵
		self.showActionType = nil;
		if self.pid ~= -1 then
			-- 切换角色
			-- print('===============up====change==========22222========')
		end

	elseif self.showActionType == 2 then 
		-- 下阵
		aniName = 'down'
		stencilPosBegin = cc.p(0,50)
		stencilPosEnd = cc.p(0,-155)
	else
		print('=====warning not set other state !!',self.showActionType);
		return 
	end

	self.stencil:stopAllActions()

	self.stencil:setPosition(stencilPosBegin)
	self.stencil:runAction(cc.Sequence:create(
		cc.MoveTo:create(30/30,stencilPosEnd),
		cc.CallFunc:create(function (  )
			if self.showActionType ~= nil then
				self.showActionType = nil;
				self:showAction()
			end
		end )
		))

	if self.skeletonNode2 then
		self.skeletonNode2:setAnimation(0, aniName, false)
	end
end

function TeamRoleTemplate:setOfflineInfo(pid)
	--[[
	self.infoNode:setVisible(false);
	self.levelLabel:setVisible(false);
	self.labelType:setVisible(false);
	if pid ~= self.pid then
		self:removeModel();
	end

	if pid ~= -1 then
		self.roleNode:setVisible(true);
		if pid ~= self.pid then
			local data = model.CardModel:GetCard(pid);
			if data then
				local no = data.no;

				-- set model
				local modelPath = ResTable:getValue('role_resource', no, 'model');
				if not modelPath then return; end
				local armature = DragonBonesManager:GetArmature(modelPath);
				armature:setName('model');
				armature:getAnimation():play('idle');
				armature:getArmature():setFlipX(true);
				armature:addTo(self.modelNode);

				self.removeArmature = function()
					armature:removeFromParent();
				end
			end
		end
		self.expGet = exp;
		self.expLabel:setString('+' .. exp);
	else
		self.roleNode:setVisible(false);
	end
	self.pid = pid;
	--]]
end

function TeamRoleTemplate:setSelected(flag)
	self.beSelected = flag;
	self.selectSp:setVisible(flag);
end

function TeamRoleTemplate:getPid()
	return self.pid;
end

function TeamRoleTemplate:setPosition(x, y)
	self.root:setPosition(x, y);
end

function TeamRoleTemplate:setScale(scale)
	self.root:setScale(scale);
end

function TeamRoleTemplate:setClickEvent(click_hander)
	self.clickLayout:setVisible(true)
	self.callback = click_hander;
end

function TeamRoleTemplate:beClicked(sender, eventType)
	if self.callback then
		self.callback(self.index);
	end
end

function TeamRoleTemplate:move(x, y)
	self.clippingNode:setPosition(x, y)
end

function TeamRoleTemplate:removeFromParent()
	self:removeModel();
	self.root:removeFromParent();
end

function TeamRoleTemplate:removeModel(  )
	if self.lastArmature ~= nil then
		self.lastArmature:removeFromParent();
		DragonBonesManager:RemoveDisplay(self.lastArmatureId)
		self.lastArmature = nil
		self.lastArmatureId = -1;
	end
end

function TeamRoleTemplate:setLocalZOrder(z)
	self.root:setLocalZOrder(z);
end

-- onOnOff 1 上阵 2 下阵 3 切换
function TeamRoleTemplate:setShowActType( onOrOff )
	self.showActionType = onOrOff;
end

return TeamRoleTemplate;
