local TrialRewardTemplate = class("TrialRewardTemplate")
local fonts = Enum:getTable("SystemFont")
local scheduler = cc.Director:getInstance():getScheduler();

function TrialRewardTemplate:ctor(father)
	self.father = father;

	self.root = cc.Node:create();
	self.root:addTo(father);
	self.root:setCascadeOpacityEnabled(true);

	self.bg = cc.Sprite:create('image/fight/roundReward/round_reward_bg_2.png');
	self.bg:addTo(self.root);

	self.round = cc.Label:createWithTTF('' , fonts.FONT_JP , 18);
	self.round:setPosition(-137 , 55);
	self.round:addTo(self.root);
	self.round:setTextColor(cc.c3b(60 , 60 , 60));
	self.round:setCascadeOpacityEnabled(true);

	self.rewardView = ccui.ScrollView:create();
	self.rewardView:setTouchEnabled(true); 
	self.rewardView:setBounceEnabled(true);
	self.rewardView:setDirection(ccui.ScrollViewDir.horizontal);
	self.rewardView:setContentSize(cc.size(303, 81));
	self.rewardView:setAnchorPoint(cc.p(0.5, 0.5));
	self.rewardView:setPosition(-52 , -13);
	self.rewardView:setScrollBarEnabled(false);
	self.rewardView:addTo(self.root);

	self.rewardBtn = ccui.Button:create('image/common/common_button_21.png' , 'image/common/common_button_21.png');
	self.rewardBtn:addTo(self.root);
	self.rewardBtn:setPosition(154 , -12);
	self.rewardBtn:setTitleText(LANG.UI_TRIAL_18);
	self.rewardBtn:setTitleFontSize(20);
	self.rewardBtn:setTitleFontName(fonts.FONT_JP);
	self.rewardBtn:addClickEventListener(handler(self , self.rewardBtnClick));
end

function TrialRewardTemplate:Init(reward)

	local curRound = model.TrialModel:getFinshTrialId();

	if not model.TrialModel:getStatusById(reward.id) and curRound >= reward.round_id then
		self.rewardBtn:setTitleText(LANG.UI_TRIAL_18);
		self.rewardBtn:loadTextures('image/common/common_button_21.png' , 'image/common/common_button_21.png');
		self.rewardBtn:setTag(1);
	else
		self.rewardBtn:loadTextures('image/common/common_button_22.png' , 'image/common/common_button_22.png');
		self.rewardBtn:setTitleText(LANG.UI_TRIAL_19);
		self.rewardBtn:setTag(0);
	end

	self.round:setString(tostring(LANG.UI_TRIAL_20..tostring(reward.round_id))..' ('..tostring(curRound)..'/'..tostring(reward.round_id)..')');

	local rewardNum = #reward.aim_reward;

	local maxWidth = rewardNum * 77.2;
	maxWidth = maxWidth < 303 and 303 or maxWidth;

	self.rewardView:removeAllChildren();
	self.rewardView:setInnerContainerSize(cc.size(maxWidth, 74));

	for i=1, rewardNum do
		local r = reward.aim_reward[i];
		local layout = ccui.Layout:create();
		layout:addTo(self.rewardView);
		local icon = template.ItemTemplate:create();
		icon:addTo(layout)
		icon:Init(r[1] , r[2]);
		icon:setPosition(35.7 + 77.2 * (i - 1) , 37);
		icon:setScale(0.7);
	end

	self.rewardView:setTouchEnabled(rewardNum >= 4);
end

function TrialRewardTemplate:setPosition(x, y)
	self.root:setPosition(x, y);
end

function TrialRewardTemplate:setAnchorPoint(x, y)
	self.root:setAnchorPoint(cc.p(x, y));
end

function TrialRewardTemplate:rewardBtnClick(sender , event)
	local tag = sender:getTag();
	if self.callback then
		self.callback(self.tag , tag);
	end
end

function TrialRewardTemplate:setTag(tag)
	self.tag = tag;
end

function TrialRewardTemplate:addClickEventListener(callback)
	self.callback = callback;
end

function TrialRewardTemplate:setVisible(visible)
	self.root:setVisible(visible);
end

function TrialRewardTemplate:setGetRewardBtnVisible(visible)
	self.rewardBtn:setVisible(visible);
end

return TrialRewardTemplate;
